//=============================================================================
// MOG_EventText.js
//=============================================================================

/*:
 * @plugindesc (v1.2)[v1.1]  地图 - 事件头顶漂浮文字
 * @author Moghunter （Drill_up翻译）
 *
 * @param 平移-位置 X
 * @desc 以事件的头顶为基准，x轴方向平移，单位像素。
 * @default 0
 *
 * @param 平移-位置 Y
 * @desc 以事件的头顶为基准，y轴方向平移，单位像素。
 * @default 0
 *
 * @param 字体大小
 * @desc 漂浮文字的字体大小。
 * @type number
 * @min 1
 * @default 18 
 *
 * @param 文字宽度
 * @desc 漂浮文字的宽度，如果过小，文字会被压缩在一起。
 * @type number
 * @min 1
 * @default 120
 *
 * @help  
 * =============================================================================
 * +++ MOG - Event Text (v1.2) +++
 * By Moghunter 
 * https://atelierrgss.wordpress.com/
 * =============================================================================
 * 在事件头顶显示固定不动的漂浮文字。 
 *
 * -----------------------------------------------------------------------------
 * ----激活条件
 * 事件中，添加注释，在注释中填入以下指令：
 *
 *  事件漂浮文字:这是一串被显示出来的文字
 *
 * 中文冒号英文冒号都可以，但是冒号左右不能有空格。
 *
 * -----------------------------------------------------------------------------
 * ----关于Drill_up优化：
 * [v1.1]
 * mog v1.0版本的注释关键字是 event text: aaa
 * mog v1.2版本的注释关键字变成了 event_text : aaa
 * 由于这里面空格要求还十分严格，格式不对还不显示，所以我索性把这里的直接
 * 换成中文了。
 */

//=============================================================================
// ** PLUGIN PARAMETERS
//=============================================================================
　　var Imported = Imported || {};
　　Imported.MOG_EventText = true;
　　var Moghunter = Moghunter || {}; 

  　Moghunter.parameters = PluginManager.parameters('MOG_EventText');
    Moghunter.charText_x = Number(Moghunter.parameters['平移-位置 X'] || 0);
	Moghunter.charText_y = Number(Moghunter.parameters['平移-位置 Y'] || 0);
    Moghunter.charText_Size = Number(Moghunter.parameters['字体大小'] || 18);
    Moghunter.charText_Length = Number(Moghunter.parameters['文字宽度'] || 120);
	
//=============================================================================
// ** Character Base
//=============================================================================

//==============================
// * Init Members
//==============================
var _alias_mog_eventext_cbase_initMembers = Game_CharacterBase.prototype.initMembers;
Game_CharacterBase.prototype.initMembers = function() {
    _alias_mog_eventext_cbase_initMembers.call(this);
	this._char_text = [false,""];
};

//=============================================================================
// ** Game Event
//=============================================================================

//==============================
// * Setup Page
//==============================
var _alias_mog_eventext_gevent_setupPage = Game_Event.prototype.setupPage;
Game_Event.prototype.setupPage = function() {
	_alias_mog_eventext_gevent_setupPage.call(this);
    this.check_event_text();
};

//==============================
// * Check Event Text
//==============================
Game_Event.prototype.check_event_text = function() {
	this._need_clear_text = true
	if (!this._erased && this.page()) {this.list().forEach(function(l) {
	       if (l.code === 108) {
			   var comment = l.parameters[0].split(/[:：]/);		//冒号正则
			   if (comment[0].toLowerCase() === "事件漂浮文字"){
                  this._char_text = [true,String(comment[1])];
				  this._need_clear_text = false;			  
			   };};
	}, this);};
	if (this._need_clear_text) {this._char_text = [true,""]};
};

//=============================================================================
// ** Sprite Character
//=============================================================================

//==============================
// * Initialize
//==============================
var _alias_mog_eventext_schar_initialize = Sprite_Character.prototype.initialize;
Sprite_Character.prototype.initialize = function(character) {
    _alias_mog_eventext_schar_initialize.call(this,character);
	if (this._character && this._character._eventId) {this._character.check_event_text()};
};

//=============================================================================
// ** Spriteset Map
//=============================================================================

//==============================
// * create Lower Layer
//==============================
var _alias_mog_eventext_srmap_createLowerLayer = Spriteset_Map.prototype.createLowerLayer;
Spriteset_Map.prototype.createLowerLayer = function() {
	_alias_mog_eventext_srmap_createLowerLayer.call(this);
	this.create_event_text_field();
};

//==============================
// * create Event Text Field
//==============================
Spriteset_Map.prototype.create_event_text_field = function() {
	this._etextField = new Sprite();
	this._baseSprite.addChild(this._etextField);
	this._sprite_char_text = [];
	for (var i = 0; i < this._characterSprites.length; i++) {
	     this._sprite_char_text[i] = new Sprite_CharText(this._characterSprites[i]);
		 this._etextField.addChild(this._sprite_char_text[i]);
    };
};

//=============================================================================
// ** Sprite CharText
//=============================================================================
function Sprite_CharText() {
    this.initialize.apply(this, arguments);
};

Sprite_CharText.prototype = Object.create(Sprite.prototype);
Sprite_CharText.prototype.constructor = Sprite_CharText;

//==============================
// * Initialize
//==============================
Sprite_CharText.prototype.initialize = function(target) {
    Sprite.prototype.initialize.call(this);
	this.sprite_char = target;
};

//==============================
// * Character
//==============================
Sprite_CharText.prototype.character = function() {
	 return this.sprite_char._character;
};

//==============================
// * Update Char Text
//==============================
Sprite_CharText.prototype.update = function() {
	Sprite.prototype.update.call(this);
	if (this.character()._char_text[0]) {this.refresh_char_text()};
	if (!this._char_text) {return};
	this._char_text.x = this.textX_axis();
	this._char_text.y = this.textY_axis();
};

//==============================
// * Create Char Text
//==============================
Sprite_CharText.prototype.create_char_text = function() {
	 if (this._char_text) {this.removeChild(this._char_text)};
	 if (this.character()._char_text[1] === "") {return};
     this._char_text = new Sprite(new Bitmap(Moghunter.charText_Length,32));
	 this._char_text.anchor.x = 0.5;
	 this._char_text.y = -(this.sprite_char.patternHeight());
	 this._char_text.bitmap.fontSize = Moghunter.charText_Size;
	 this.addChild(this._char_text);
};

//==============================
// * Refresh Char Text
//==============================
Sprite_CharText.prototype.refresh_char_text = function() {
    this.create_char_text();
	this.character()._char_text[0] = false;
	if (this.character()._char_text[1] === "") {return};
	var text = this.character()._char_text[1];
	this._char_text.bitmap.clear();
	this._char_text.bitmap.drawText(text,0,0,Moghunter.charText_Length,32,"center");
};

//==============================
// * Text X Axis
//==============================
Sprite_CharText.prototype.textX_axis = function() {
	return Moghunter.charText_x + this.sprite_char.x;
};

//==============================
// * Text Y Axis
//==============================
Sprite_CharText.prototype.textY_axis = function() {
	return -(this.sprite_char.patternHeight() + 24) + Moghunter.charText_y + this.sprite_char.y;
};
