﻿//=============================================================================
// MenuCancel.js
//=============================================================================

/*:
 * @plugindesc 触屏菜单自定义关闭按钮
 * @author 江湖梦网 Fanzi
 * 
 * @param 按钮图片
 * @desc 按钮图片名，不加后缀，放在 img/pictures。
 * @default cancel1
 * @require 1
 * @dir img/pictures/
 * @type file
 * 
 * @param 按下图片
 * @desc 按钮图片名，不加后缀，放在 img/pictures。
 * @default cancel2
 * @require 1
 * @dir img/pictures/
 * @type file
 *
 * @param  按钮坐标
 * @desc 希望显示在菜单屏幕的自定义位置，格式如：888,666
 * @default 756,0
 *
 * @param  反应时间
 * @desc 按下按钮后多久反应，填数字单位毫秒。
 * @default 2000
 * 
 * @param 关闭声效
 * @desc 声效文件名，不加后缀，放在 audio/se。
 * @default Cancel2
 * @require 1
 * @dir audio/se/
 * @type file
 *
 * @help
 * 本插件专为发布手机版开发，无需插件指令与脚本指令，
 * 在所有菜单界面（包括商店）建立关闭按钮，图像与坐标均可自定义，
 * 任何游戏均可无偿使用，请保留作者信息。
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.MenuCancel = 1.0;

var MenuCancel = MenuCancel || {};
MenuCancel.Parameters = PluginManager.parameters('MenuCancel');
MenuCancel.Param = MenuCancel.Param || {};

MenuCancel.Param.ButtenName = String(MenuCancel.Parameters['按钮图片']);
MenuCancel.Param.ButtenClick = String(MenuCancel.Parameters['按下图片']);
MenuCancel.Param.CancelSound = String(MenuCancel.Parameters['关闭声效']);
MenuCancel.Param.Position = MenuCancel.Parameters['按钮坐标'];
MenuCancel.Param.DelayTime = MenuCancel.Parameters['反应时间'] || 2000;
MenuCancel.Position = MenuCancel.Param.Position.split(",");

MenuCancel.Scene_MenuBase_create = Scene_MenuBase.prototype.create;
Scene_MenuBase.prototype.create = function() {
    MenuCancel.Scene_MenuBase_create.call(this);
    this.createCancelButten();
};

Scene_MenuBase.prototype.createCancelButten = function() {
    if (MenuCancel.Param.ButtenName) {
        this._buttenSprite = new Sprite(ImageManager.loadPicture(MenuCancel.Param.ButtenName));
        this.addChild(this._buttenSprite);
        this._buttenSprite.move(MenuCancel.Position[0], MenuCancel.Position[1]);
    }
};

Scene_MenuBase.prototype.createClickButten = function() {
    if (MenuCancel.Param.ButtenClick) {
        this._buttenSprite = new Sprite(ImageManager.loadPicture(MenuCancel.Param.ButtenClick));
        this.addChild(this._buttenSprite);
        this._buttenSprite.move(MenuCancel.Position[0], MenuCancel.Position[1]);
    }
};

Scene_MenuBase.prototype.update = function() {
    Scene_Base.prototype.update.call(this);
    var wx = MenuCancel.Position[0];
    var wy = MenuCancel.Position[1];
    var ww = MenuCancel.Position[0] + this._buttenSprite.bitmap.width;
    var wh = MenuCancel.Position[1] + this._buttenSprite.bitmap.height;
    if (TouchInput.isTriggered()) {
        var x = TouchInput.x;
        var y = TouchInput.y;
        if (x >= wx && y >= wy && x <= ww && y <= wh) {
            AudioManager.playSe({"name":MenuCancel.Param.CancelSound,"volume":90,"pitch":100,"pan":0});
            this.removeChild(this._butten);
            this.createClickButten();
            setTimeout(this.popScene(), MenuCancel.Param.DelayTime);
        }
    }
};