/*:
 * @plugindesc DragonBones Integration Battler Change Extension
 * Adding the ability to change battlers mid-battle
 * @author Yorae Rasante
 *
 * @help
 * Usually, the DragonBones extension does not allow for either
 * enemy transformation or change of actors
 * No error message appears, but the new DragonBones won't appear
 * and the old ones stay.
 * This is a small fix for this.
 */

var DB_GE_Transform = Game_Enemy.prototype.transform;
Game_Enemy.prototype.transform = function(enemyId) {
    var tempArmatureName = $dataEnemies[enemyId].meta.dragonbone;
    var tempScaleX = $dataEnemies[enemyId].meta.dragonbone_scalex;
    var tempScaleY = $dataEnemies[enemyId].meta.dragonbone_scaley;
    var tempStartAnimation = $dataEnemies[enemyId].meta.dragonbone_ani_walk;

    if (dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()]) {
      dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()].dispose();
    }
    if (tempArmatureName) {
      dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()] = 
        dragonBonesIntegration.CreateArmature(tempArmatureName, 0, 0);
  
      if (dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()]) {
        $gameTroop.members()[this.index()].hasDragonBone = true;

        $gameTroop.members()[this.index()].dragonboneIndex = i;
        $gameTroop.members()[this.index()].dragonboneAnimation = [];

        for (var key in $dataEnemies[enemyId].meta) {
          if ($dataEnemies[enemyId].meta.hasOwnProperty(key)) {
            var animationIndex = key.split("dragonbone_ani_");
            if (animationIndex[1]) {
              $gameTroop.members()[this.index()].dragonboneAnimation[animationIndex[1]] =
                $dataEnemies[enemyId].meta[key];
            }
          }
        }

        $gameTroop.members()[this.index()].AutoTransitionToIdle = function(event) {
          if (dragonBonesIntegration.consoleDebug) {
            console.log("Enemy AutoTransitionToIdle");
          }
          switch (event.type) {
            case dragonBones.EventObject.COMPLETE:
              if (this._states.contains(1) === false) {
                dragonBonesIntegration.PlayAnimationOnBattler(this, "walk");
              } else {
                dragonBonesIntegration.Game_Enemy_prototype_performCollapse.call(this);
              };
              break;
            default:
          }                                           
        }

        var enemyData = dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()];
        enemyData.addEvent(dragonBones.EventObject.COMPLETE,
          $gameTroop.members()[this.index()].AutoTransitionToIdle, 
          $gameTroop.members()[this.index()]);

        dragonBonesIntegration.PlayAnimationOnBattler($gameTroop.members()[this.index()], "walk");   

        if (tempScaleX) {
          dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()].scale.x = tempScaleX;
        }

        if (tempScaleY) {
          dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()].scale.y = tempScaleY;
        }

        BattleManager.getSprite(this).addChild(dragonBonesIntegration.ArmatureDatabaseEnemy[this.index()]);
      }

    } else {
      $gameTroop.members()[this.index()].dragonboneIndex = null;
    }

    DB_GE_Transform.call(this, enemyId);
};

DB_SB_update = Scene_Battle.prototype.update;
Scene_Battle.prototype.update = function() {
  DB_SB_update.apply(this, arguments);
  if (!this.actorsToRefresh) return;
  for (var j = 0; j < $gameParty.battleMembers().length; j++) {
	  var i = $gameParty.battleMembers().indexOf($gameActors.actor(this.actorsToRefresh[j]));
    var actorId = $gameParty._actors[i];
    if (actorId) {
      var tempArmatureName = $dataActors[actorId].meta.dragonbone;
      var tempScaleX = $dataActors[actorId].meta.dragonbone_scalex;
      var tempScaleY = $dataActors[actorId].meta.dragonbone_scaley;
        
      if (dragonBonesIntegration.ArmatureDatabaseActor[i]) {
        dragonBonesIntegration.ArmatureDatabaseActor[i].dispose();
      }

      if (tempArmatureName) {
        dragonBonesIntegration.ArmatureDatabaseActor[i] = 
          dragonBonesIntegration.CreateArmature(tempArmatureName, 0, 0);
        
        if (dragonBonesIntegration.ArmatureDatabaseActor[i]) {
          $gameActors.actor(actorId).hasDragonBone = true;
          $gameActors.actor(actorId).dragonboneIndex = i;
          $gameActors.actor(actorId).dragonboneAnimation = [];

          $gameActors.actor(actorId).AutoTransitionToIdle = function(event) {
            switch (event.type) {
              case dragonBones.EventObject.COMPLETE:
                if (this._states.contains(1) === false) {
                  this.lastMotionType = null;
                  dragonBonesIntegration.PlayAnimationOnBattler(this, "walk");
                }
                break;
              default:
            }                                           
          }

          var actorData = dragonBonesIntegration.ArmatureDatabaseActor[i];
          actorData.addEvent(dragonBones.EventObject.COMPLETE,
            $gameActors.actor(actorId).AutoTransitionToIdle, 
            $gameActors.actor(actorId));

          for (var key in $dataActors[actorId].meta) {
            if ($dataActors[actorId].meta.hasOwnProperty(key)) {
              var animationIndex = key.split("dragonbone_ani_");
              if (animationIndex[1]) {
                var actor = $gameActors.actor(actorId)
                actor.dragonboneAnimation[animationIndex[1]] = 
                  $dataActors[actorId].meta[key];
              }
            }
          }

          if (tempScaleX) {
            dragonBonesIntegration.ArmatureDatabaseActor[i].scale.x = 
              tempScaleX;
          }
          if (tempScaleY) {
            dragonBonesIntegration.ArmatureDatabaseActor[i].scale.y = 
              tempScaleY;
          }
          var actorData = dragonBonesIntegration.ArmatureDatabaseActor[i];
          BattleManager.getSprite($gameParty.battleMembers()[i]).addChild(actorData);
          $gameParty.battleMembers()[i].requestMotionRefresh();
        }                    
      }
    }
  }
  this.actorsToRefresh = undefined;
};

var DB_GP_addActor = Game_Party.prototype.addActor;
Game_Party.prototype.addActor = function(actorId) {
  DB_GP_addActor.call(this, actorId);
  if(this.inBattle()) {
    SceneManager._scene.actorsToRefresh = SceneManager._scene.actorsToRefresh || [];
    SceneManager._scene.actorsToRefresh.push(actorId);
  }
};

var DB_GP_removeActor = Game_Party.prototype.removeActor;
Game_Party.prototype.removeActor = function(actorId) {
  if (this.inBattle() && dragonBonesIntegration.ArmatureDatabaseActor[this.battleMembers().indexOf(actorId)]) {
    dragonBonesIntegration.ArmatureDatabaseActor[this.battleMembers().indexOf(actorId)].dispose();
  }
  DB_GP_removeActor.call(this, actorId);
};