﻿//-----------------------------------------------------------------------------
// PlayOnline.js
// Version: 2.3
//-----------------------------------------------------------------------------
/*:
 * @plugindesc 网络登录同屏聊天排行云存档2.3版
 * @author 江湖梦网 Fanzi
 *
 * @param  游戏名字
 * @desc 登陆后显示欢迎语
 * @default 
 *
 * @param  后台程序
 * @desc 游戏在江湖梦网申请到的后台程序名
 * @default game
 *
 * @param  游戏版本
 * @desc 输入当前游戏的版本号，用于检测游戏更新
 * @default 1
 *
 * @param  下载地址
 * @desc 游戏更新的网站入口，论坛发布指引帖子填写地址
 * @default https://bbs.jhmeng.cn/
 *
 * @param  窗口位置
 * @desc 登陆窗口显示的高度
 * @default 100
 *
 * @param  窗口透明度
 * @desc 调节登录窗口的透明度
 * @default 255
 *
 * @param  弹窗背景
 * @desc 弹出窗口背景图片 384 * 144 img/pictures 留空无背景
 * @default popup
 *
 * @param  存档通知
 * @desc 用于做定点云存档成功的返回信息，自动云存档留空不显示
 * @default 您已成功存档！
 *
 * @param  统一变量
 * @desc 指定一个变量号，开发者可在服务器修改该变量，登录赋值
 * @default 5
 *
 * @param  挂机变量
 * @desc 离线的服务器时间写入该号变量，单位为秒数
 * @default 3
 *
 * @param  排行变量
 * @desc 游戏数据转化为一个变量用于排行依据的变量号
 * @default 2
 *
 * @param  排行记录
 * @desc 用于查看排行后记录当前排名的临时变量号
 * @default 4
 *
 * @param  在线时长
 * @desc 玩家多少分钟内有数据更新认定在线
 * @default 3
 *
 * @param  显示上限
 * @desc 同个地图显示玩家人数限制
 * @default 10
 *
 * @param  角色数量
 * @desc 游戏中的角色数量，根据数据库角色数量填写
 * @default 4
 *
 * @param  移动速度
 * @desc 队友及其他玩家显示的移动速度放慢倍数，可以是小数
 * @default 2
 *
 * @param  队友跟随
 * @desc 选择队友跟随形态，true为随机跟随，false为贪食蛇式
 * @default true
 *
 * @param  队友漫步
 * @desc 队友原地踏步的机率，可以是小数
 * @default 3
 *
 * @param  队友传送
 * @desc 队友远离队长一定距离时，自动传送到队长身边
 * @default 6
 *
 * @param  显名性质
 * @desc 显示玩家的用户名或者角色名，true为用户名，false为第一角色名
 * @default true
 *
 * @param  人名X位
 * @desc 显示玩家名字距离角色头顶原点X位移
 * @default 0
 *
 * @param  人名Y位
 * @desc 显示玩家名字距离头顶原点Y位移
 * @default 30
 *
 * @param  人名字号
 * @desc 显示玩家名字的字体大小
 * @default 16
 *
 * @param  聊天窗XY
 * @desc 聊天窗口显示位置(左下角)
 * @default 600,600
 *
 * @param  聊天窗宽
 * @desc 聊天窗口宽度
 * @default 200
 *
 * @param  聊天明暗
 * @desc 聊天窗口、背景及信息明暗度
 * @default 255
 *
 * @param  聊天字号
 * @desc 聊天窗口显示字体大小
 * @default 20
 *
 * @param  显示聊天
 * @desc 聊天窗口是否默认显示
 * @default true
 *
 * @param  系统说话
 * @desc 用于系统说话开关，提示获得物品等
 * @default 恭喜
 *
 * @param  菜单排行
 * @desc 主菜单指令列表增加显示排行的文本，留空不建立
 * @default 排行
 *
 * @param  排行内容
 * @desc 排行界面显示数字序列的表头文本
 * @default 总战力
 *
 * @param  排行背景
 * @desc 排行界面背景图片 img/pictures 留空无背景
 * @default bgd
 *
 * @param  排行透明度
 * @desc 调节排行窗口的透明度，独立于聊天窗口
 * @default 255
 *
 * @param  息屏登出
 * @desc 手机进入息屏时登出游戏，true为启用，false为不启用
 * @default true
 *
 * @param  屏幕按钮
 * @desc 将按钮图片(仿发送按钮)放在 img/pictures 填入文件名
 * @default 
 *
 * @param  APK打包
 * @desc 用于兼容APK打包后的弹出窗口，Website2APK打包填true，其它填false
 * @default false
 *
 * @help
 * 本插件用于将游戏网络化，为游戏提供服务器对接，接入本插件的游
 * 戏将可发布为PC终端、WEB终端、APP终端皆同步通讯，服务器端提
 * 供账号检测、密码保护、单线登录、玩家同屏、云存档、在线双频道
 * 聊天、玩家排行等功能，在线商城有配套商城插件。
 * 插件参数可供开发者自定义设计游戏，包括透明度与背景图片用于界
 * 面美化；排行变量由开发者在游戏中计算存入指定变量，该变量将与
 * 服务器进行通讯，排行名称自定义；游戏时长为判定玩家在线的依据，
 * 即该时间内玩家有数据更新即为在线；显示上线控制同地图玩家显示
 * 数量；显名性质用以选择同屏玩家显示用户名或者角色名；玩家名字
 * 位置大小均自定义；聊天窗口大小、位置、透明度、字体均可自定义；
 * 挂机变量用以离线经验奖励调用，该数值为服务器离线时间单位秒数；
 * 统一变量用以指定变量由开发者在服务器修改，玩家登录即更新其值；
 * 本插件中玩家同屏功能与集合队友的事件指令有冲突，显示队友跟随
 * 即开启玩家同屏，此时不可集合队友，关闭显示队友即关闭玩家同屏。
 * 本插件内置屏幕按钮功能，如需模拟手机按键可仿发送按钮制作图片
 * 放入pictures文件夹，屏幕按钮参数多个图片用英文逗号隔开，在事件
 * 中执行脚本，具体见后面脚本详述；插件提供一个系统给所有在线玩
 * 家全体发信息的指令，可结合系统说话参数做系统提示，具体见后文。
 * 本插件由 江湖梦网 Fanzi 开发，入驻游戏开发者可免费安装使用，
 * 江湖梦网作为游戏服务平台，承载付费资金的代收与转发。
 *                     有意合作者请登录 江湖梦网 https://bbs.jhmeng.cn/
 *                                               联系QQ：12868880  Fanzi
 * 插件指令:
 * 游戏云存档：SaveGame
 * 需自动云存档则建立并行处理的，公共事件定期执行SaveGame指令，
 * 自动云存档建议600帧执行一次，定点存档直接运行SaveGame指令。
 * 显示聊天窗：ShowTalk
 * 隐藏聊天窗：HideTalk
 * 打开排行榜：SceneRanking
 * 服务器记数：AskValue n   参数 n 填写变量号则将记数值返回该号变量
 * 服务器清数：ResetValue   开发者可设定记数值达到多少则清零
 * 事件发信息：SendWord \C[2]信息内容\C[0]   （颜色代码同显示文本）
 * 例如系统说话参数为：恭喜，插件指令为：SendWord 捡到金子，当某
 * 个玩家触发该指令时则显示“恭喜###捡到金子”。
 * 读取服务器时间：AskData 6 7 8 9             (6、7、8、9为系统变量ID)
 * 注意空格隔开，执行此插件指令后 变量 6 = 年份，变量 7 = 月份
 * 变量 8 = 日期，变量 9 = 钟点（小时数），此功能可用于设计日常任务
 *
 * 脚本指令：
 * 返回游戏登录：DataManager.ResetData();
 * 屏幕按钮脚本：
 * 生成：PlayOnline.addButton(id,'type','img',x,y,['actionType','action'],e);
 * 消除：PlayOnline.removeButton(1);        括号里填按钮ID
 * 脚本参数说明：
 * id：              按键序号，插件已用了1、2、3，新增从4开始，不可重复
 * type：          map/mapX 填map则所有地图通用，填mapX则对应id的
 *                     地图有用，其中X为地图id号
 * img：           按钮图片文件名，注意大小写，不用后缀，img/pictures
 * x：               屏幕坐标X
 * y：               屏幕坐标Y
 * actionType：按钮类型，button/buttonT/script/event四选一
 * action：       按钮执行动作，需对应上方的按钮类型，下面详解
 * e：               运行事件时的按钮可视度，不填则运行事件时不显示，
 *                     填255则运行事件时仍可视并可操作。
 * 按钮类型与执行动作：
 * button：  MV默认按键，执行'ok','cancel','shift','up','down','left','right'
 * buttonT：模拟一个按钮被触发
 * script：    执行脚本，执行动作可填脚本如：SceneManager.push(Scene_Equip)
 * event：    执行公共事件，执行动作填公共事件ID号
 * 脚本举例：
 * PlayOnline.addButton(4,'map','BtnOk',570,500,['button','ok'],255);
 * PlayOnline.addButton(5,'map1','BtnRun',690,500,['buttonT','shift']);
 * PlayOnline.addButton(6,'map','BtnMenu',0,0,['script','SceneManager.push(Scene_Menu)']);
 * PlayOnline.addButton(7,'map1','BtnEvent',800,600,['event',1]);
 * 按钮式开关聊天窗：
 * PlayOnline.addButton(8,'map','Talk',500,500,['script','PlayOnline.ShowTalk = PlayOnline.ShowTalk ? false : true;']);
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.PlayOnline = 2.3;

var PlayOnline = PlayOnline || {};
PlayOnline.Parameters = PluginManager.parameters('PlayOnline');
PlayOnline.Param = PlayOnline.Param || {};

function Scene_PlayOnline() {
    this.initialize.apply(this, arguments);
};

function Scene_Ranking() {
    this.initialize.apply(this, arguments);
};

function Window_LoginCommand() {
    this.initialize.apply(this, arguments);
}

function Window_LoginInfo() {
    this.initialize.apply(this, arguments);
}

function Window_Ranking() {
    this.initialize.apply(this, arguments);
}

;var encode_version = 'sojson.v5', ximyn = '__0x4d7c8',  __0x4d7c8=['w7HDmQHDiwE=','I25GPyo=','5a2w56Gw6ZS96K+W5ZuV5qyw5Yqc5LmT5q2r5p+o5L6C','w4EFXQ==','w7wTQw==','wpTDpyzCo8OS','wos0AhIo','wqVUwqtHPA==','6L6j5oyd5Lis5Yu15p+U5Yuz5Zm4776W6K6R5qKI5p+457yF57qa6YeG6K2n44Cj','JMOGwozDt27CvEDDq1bCrATCvcK3WA==','wo1UwoVBO8O1wr9EBilhwovCq8K1wo8=','L8Kcwr8GUwnDmiAiwqDCjTDDp8OP','wpBHwoFQF8OuwqtZAg==','wo8wPQQbw7o=','woTDjiXCssOqO8OawowqL8Kq','w4RLXsKMD8O9w7LDhMObYg==','P8KcQsK4TMOAwrrDiSQ=','wp0CNsK6w5TCi0bDs8KVw6AfZsKGGMOPwrc=','K8OUWA==','T8K7w7l/w7U=','DEbCmcOMehtzC1I=','Ci5Iw7fCvS8=','EsOVZMKcwo4=','wqvDuhZBMg==','LMKcSMKtV8ORwoDDlizDjMKpRcKrPHrDqsKnYiE=','wrcuwpHDrF9TwrHCr2E=','w4jCrUgeLDsfdhbCqixYdDY=','M0NWw4Y=','w7/DuwLDugkuVWkkw70G','w5RsRRPDoQ==','Q8Kaw6Z+w7YsEQxR','RV/DtyRTKMOhwphLE8Kbw77DhsKB','OEdTw43Cjn4=','wpfDkyTCtcOxPcOWwp87','woNHwotFDMO/wpBIBCxiwrXCrcKzwpUp','wrfDgDnCoMOz','wo7CmmsHw5ovw4w=','IsOKw6nCoDjDjQnDrsKhSMKm','woxbXsK+E8Kiwq7CuQ==','TsOZw5LDpjfCoRzDszHCuVk=','f0vDtAFf','wp8yNAwb','wrp7fwYZwofCmz9AHMOy','PT3DjcO0C8OsP3Rxw5bDpQ==','UMKiw67CoMKzdBrCqcOteAg=','Eg3DosKDYsKFVcKwLQ==','dMOfwq7DpXk=','w4vCrUNIwrXDpXR2','ZSQaw7nDvsKDSBbDu2jCpMOzB2I9DQZsHA==','w4PCusKWw6p1Cw==','AMKewrcOSxg=','SMKHw5nCi8Kt','UMKow6/Cq8KxQh3Co8OefhEsM8Oz','P1Jbw4nCj35x','w4DDgzjDqMOEXwhEWQ==','JX8+R1fDrwnDs8Kadg==','w7BRwoNHwpkKw7fCm8OqfsOHwppmCQ==','UsKQw7JTw7Ax','fsKWw7Fbw74kGAByE8KOw45PwoU=','DsONw7zCizLDsB3DsMKtTg==','wrcowofDgA==','ESZKdcK7w4svBF8Ww4xjw6rDig==','wrRVworCmcK3DifCqyVp','OC5OUsKZ','N34uWF/DrwnDsMKI','w7JGWMKVC8O9w7DDicO2bsK7wrPCr2Q=','wo5eVMKeHsKn','wpwtKDYMw7ofwpc=','w7fDoMOISA==','w6/DnMK2woNVeA==','wpdRwpnCtA==','w4F5wo7CmzEhUcK2CcO3w7PDu1bDug==','SMOewpjDnV0V','w5nDs8OE','wq8Na3zCkg==','wrnDmcKnCTfDiQ==','wpVRwpDCusK/DiQ=','w6nDh8K8woRCfMOzw4Y=','w5JgwoM=','bcK7w4pww7Y=','GMOAworDtg==','w43DuC/DuzY=','GjlZw57CqCTChMKObHw=','A8KBSsKlTcO7wqg=','w5QYSl0=','w4UUUHFvf8KRPy/CoA==','w4p1fQM=','HXZnEzXDvA==','EHtHECk=','w6xydx3DrMOowqIiw47ClmI=','PkjChsOh','X8Kqw6fCow==','w4F+ZB3DtMOKwrcp','LcOZRsKgwp09Z8O0','w6XCn2TCkA==','w4jDnMKswpBd','Am4kbXc=','dEnDtQ==','VcOGwq/DmQ==','w45RwphDwoIKw63Cmg==','f8K5w6/CssKzVwrCt8Os','w43DmRzDnjoyP8OFw57CqA==','wrZzZwU/wr7ClD4=','wrDCu8OGWQ==','KSpRd8KCw4M1DG0=','w7VMw4/CrcOlHHfCu3BnZw==','PjdKbMK5w544EG0=','LTdAecKiw48CD2USw4Npw6HDqsKcw4gfMhA=','w6USQVdrTsK4NiTCp8OHw4AYwp9UVS7CnUjDpBUhCCjCiRwfD8KIZw==','w4HDjSzDjA==','dsOCwrzDtFseNXNOwrg=','w6QQSlJnf8KS','wrYuwoTDiVtUwrLClXQKLcOdwp3Cig==','wo0mPCodw7YWwpvCvEvDtyVkU1kSOsOYw7E=','w4LDoMOISMKQwpLDuUJGDzF1YcKmRMKkw4dxEUEDwofDoMOtEcOrw5lqI8OTLVfDiMK8UcKZDDA=','DUjCjcOo','OcOEcMONKcOyAcK2PjTDqRnClxcSLHfDvsK8E8O1wq5HTsK5w6tGTFR0VzArw63CiRdVcA==','N34uQETDqArDtsKRZcOrA1bCmC5zPMOYwoM=','w79+woXCpD0uVMKRMcOzw7DDvlfDqQ==','wqBaVMKIKcKqwqzCtg==','wq0TNsK1w4XCsUnDt8KYw74nd8KbCMOAwqfCmB4YDMOgw6vCsMKewo1/KFHCg1Y=','HsOaw63CoifDuzrDs8KlUcKidFQyw4rDq8KsZsKH','wqXDicK6Bg==','H8OXwoLDrVfCuFbDuw==','w4YJbg==','AMOBTMOkEw==','w7rCr1MQKQ==','w5tUYsKxJA==','wolrehAP','w4IUXE1NfsKZPDg=','w5bDiTjDlMOiWQNDXw==','CcOAwpPDtmLCvks=','w5bCkW3CgcONB8OHwow=','IixLfcKew48oB2AL','wrELwq7DsVM=','GsOMUBXChcKbwrI2ZsK5d8Kq','wo0hLAod','w58CclB9eMKXPy8=','D8OOw4PCoj8=','fsKcw7BSw7Y9','w4tAwo1dwqAOw6HCiw==','I8KHQ8Kpa8ORwqrDninDlQ==','w6fCvEgHLi4IYiQ=','GMOKwo3DqXfCr1vDrHXCqhg=','wpwwNxEAw6sIwoLCtw==','bzgWw6zDo8KHZxDDrGA=','w53Cs8KSw7pPAWtnIsKDw5VTw7zDgF0=','wqbCp8OBRW8nw67CnFBnwrDDmWA=','Ikdcw5jCg3lg','HMOLw7zCqiXDvw3DuQ==','A8OMZsOqKcOpLQ==','w417wpfCkxUuWcK9','OgPDlsKBQA==','wr8cOsKYw68=','HMKPW8KpasOawqXDlg==','DcOKTsKxwrYmacOqAw==','DUzCj8OwegA=','w4EYQE1m','w6kYSkl7ZcKiOiTCtsO3w5Q=','wrTDiCHCkcOr','IsORT8K1wqwmR8OzCCxyIw==','ESxLaMKjw54WCWYbw41w','GMKawrkMVhw=','w7N7worCkx8vUsK/P8Oww7nDk1DDvhM=','5oiF5bya6K+w5Z2U','PHIneA==','Wk4Aw5BOw48=','Ki7Dng==','w4XCrVp7wrvDoG17','5Yao5pW+5LiP55Su','wp3DlQXCjcO2','bmXDjQ==','w53DkMK9wrVjcMOzw4Jdw5A=','w4FKVsKcAcO9w7PDiA==','DjjDh8OcK8OxM3Z2w4Y=','5YaP5pSR5Lux55ee','w5gUU15vfMKQ','w73DscOqXsKG','woxbXsK+FMKmwq/CvBBs','wpfDlx3CgMOH','w51tccKzPA==','O31LPzfDtDrCusKDwq4=','McOKw4zCjyo=','O8OOesOPEw==','CDhJw5XCpifCjcKDZ2o=','55iU5byJ5rqb5oqQ','w6vDo8KpwqxW','5oig5b2p6Ky85Z6P','w5HCtsKdw5xjA2trLcKJ','6YKv5YeI5ruU5oil','wqnChMOCQnE=','CXhZGRHDtzHCtA==','X8Kqw7LCp8Kx','w5l7wozCkwolTcKhN8Oxw7M=','wqRYwpvCssK9MC/CsidyPCnCiMK/woZN','5Lus6L+c5pq/5pWx','YcOIwr/DklYfMHs=','w4F5dDfDrcOGwq4tw4PClw==','6YKw5YSo5ruB5oqQ','wq3Cv8O5e3Q=','Eg3DosKXZMKGXMK+NAY=','wrZtwoNoMg==','JMOFUTvCiMKTwq0eZsK5','w4NjdMKUEw==','w5vCm3TCksOJBMOW','w45WwohpwpsGw7TCnsOTcw==','w54JaEt3','w47DgyfDicOP','5oux5b6s6K+65Z+7','wrnDrh5rKT5xZh7DqQ==','IsOPWsOlPw==','w4HCp8KBw7ZtAQ==','DVvChMOlaxc=','DcOCcMOaMsO3K8KYLzXDrg==','SsOeE8K0dE9ww5rDoQ==','w4lzeQDDq8OKwq8lw5fClg==','GBPDkcK1RQ==','Y8OaCMKFfA==','w57Dm8KhwqFlecOqw4s=','wrBUwpxFFQ==','wogyTHDCm2rDqQ==','wolbwodQEcO7wr5AHSI=','H8OAwoLDuXfCtFjDrnXCoA==','w5DDiSbDksOERQc=','w4XCjGzCgcOHHcOKwo/DjA==','w47DkcK/woRpbsO2','IcOTVA/CrsKQwqYQ','w5LDtwrDrgMKVA==','DS5Mw6HCgSXCksKYRWcsPw==','Q8Kzw6tnw4M=','IGFBCR0=','wpUaF8Khw6I=','woNawoBQHcO0wqZa','wrYuwofDkF9UwqLCtA==','CTXDj8OzFsO5PWM=','w5bCkW3CgcONB8OHwozDvsOHw6PCssKR','wrLClHcBw7kkw6vClsKbJMKR','w5PCvcKXw6tpAHJ5','P8KPRMKiV8O7wrPDmCLDiMK8Ug==','w4DCoMKWw6tjGn96Jg==','PMKaTMKiR8OVwrHDnQfDjsKmX8KcAmnDoQ==','w59AwoNewpsfw6DCj8OY','dlXDtix2LMOmwpt0Dg==','w5IDRU5Hf8KTPA==','EcONw67Ctw==','5a6QOmrnoI7vvKw=','SF4Ww41Kw5I=','6Laz5Y2+56Sn5q2O5ael57ma55ud5b2a77yv5aej6KOv55uh5Y+56Ky85YiZ6K2/5Z+y5L+75pW35a+m56Gm44Oh','w4LChUQ=','6K2U5Y+k5Yuc5oKC5Lq/5Yim55id6K2P5b2777yb5pqG5ZOY5oGr5aWz6Kyg5b+g77+Q','BMOdTcKpwrYl','5q2t6LygDQ==','L+i8kOWHpea7vuaLkwM=','A8KPwrwCSg==','csKUw6hTw5orEAs=','w5TCoMKYw6hYC35+','55eE5om15ZGV772/','SFQWw41Kw44QN0fDkMKGw6Be','w7HDtjPDshE=','bMKkw67CssK5TQfCtMOefhs8NA==','5pS86Led5Y+y6K6b5Yqo5rCX5ru+5qGf576m5rKj5YWv77265bm16K+m6K2B57yF5ayd56K25Ly45oi044Ob','HsOHw6bCtzbDsA3Dr8KfVcKnblg=','CsOcw6PCiSQ=','wrHDjsKoGRfDhMKFOQ==','CjNDw6LCrCTClMKRXmcmLmc=','OXxBCD3Dqw==','w5jCm8KSw4le','w4fDjxfDiTI=','wqpTwop+CQ==','w6rDnAfDkRE=','5qCH5rW75YuO5pWh54qe5p2M772P6Kyw5Ymw6K6h5ZyJ5LmR6L6z5pii5pWa5ZCm6L205YW85rqF5ouN44C4','IW4hRUE=','FsOAQx3CrsKQwqYQ','K8KcTMK7d8ORwrvDjQ==','5oG055iV5YmI5bG25aac5aW55b2u5LuU77+76K685b+j5ZGZ5YWr5pah5LmF55W044OZ','SV3Drix3J8OpwpM=','FMKNwrEcag3DjDA=','wrYuwofDkF9UwqLCtEINIsOAwps=','CsOMSsKPwq4=','wqfDqMKuBhA=','IAjDsMKxQsKFV8Kw','McOOeRfChMKfwqwaW8KpYMK7w6dJ','ZsOIwqbDiF8eJWx8wqMzHMKA','woHCocOaZEo=','55aK5oqb5ZGN7766','w4NyfgDDp8OFwrc/w7rCmmpFJw==','EDhvw7XCvA==','5a+8woxc56OB77yB','woZ9fxQTwr7ChiJzGsOhw4F3','GcOaw6nCtAfDuwHDqA==','6K6/6LS/5Y+Y5Zqc5LyE5b6T6KO85bOa77+h6Ky96IOD57OW5peV56uN5qCK6K6v6KWx5bKW44Ce','w5/DkcK3woJpbw==','6K6Y5bGW6YWu5LmJ5Lu+5Yiv5Yu25oiH5p6E776u5L2B5oy05YSe5bmm5riZ5omk54yX5aG/44O/','w4rCrVVrwrnDr3Rtw5ACFcOydg==','GMOKwo3DrmbCs1rDvFbCrA7CrcKw','PEdcw54=','EAbDqMKgbsKFRcKsDQsPw5nDrQ==','dsKvw4LCpcKp','wrzDuBtfEjZkcw==','6L6k5LqS6LSH5Yyy5q665Z265L2v55ay772q6K6/5o2/5Lm+6LWg5Y+86YW/5paR55qi5b6O5ru05oqe44OW','w4HDgy7DlMOEWBtfesKKH8Kdwp0=','wps2My8Y','6LWb5Y2f56SJ5q6h5aWJ57ih55iC5b+J77+E5aa+6KGo55uN5Y+H6K6S5YqT6Ky/5Z6T5L+n5peJ5a+s56Ky44GS','w6luworCvCs=','wqTCiGQfw7s=','QsKaw7BCw7YrAhdyE8KEw55I','AQDDocK8fw==','E8KXwrEFWQ3DoCENwr3CoDvDpMOXEA==','wp87KxEKw7Iywp3CvkXDqQ==','CjRMw7jCri/CtMKHcXoBNWPDjcKQ','w6vDoyHDnSU=','w7HDu8OHW8KrwoDDnns=','acOkGcKSeA==','cRgnw5/DrQ==','RcKHw79Bw4cgDhA=','DUbCj8OwehxgEHFCRBN/','wqRVwpDCpcKzEg==','w4rDmwzDkgMmP8Oh','wpMRKw==','w5TCoMKYw6hEAXRwD8KEw596','EsO/w6TDksKAwr8=','dcOVwqfDiFUEKG9O','KcOARgzCl8Kfwqca','w7EUUGptfsKHNg==','HEzCh8O2egF8','wp4nPhcKw6wZ','QsKZw7tXw6E=','fk7DuT52JsO9woZQE8Kbw78=','CMOiwq3Dr2c=','wqUgwoDDik51wqbCpnYNMsON','wp0fPcKvw4XCgHDDoQ==','w7hzwo3Cmg4lXMKm','w57CvcKLw7JtAkVlL8KCw4M=','w5/Dm8K3woJpc8Oqw5A=','woFWVMKYM8KuwqvCuhZ8','fMKow6/CtMK5','woxVW8KF','GV/Cl8O2eQ==','w7oYSlI=','w6/Ct1pIwqo=','w7vCo1V0wrXDr2c=','wrsuSG7CjQ==','I8Osw6/DmMKdwrQf','wqtVwpDCtsKiCA==','UMO+Mw==','wrbCqUo=','w7LDswLDtw8XWw==','QcKkw7fClMK9TRg=','wqnDjjzCk8O/J8OE','BXRKETrDvCXCksKDwq7DslU=','azcHw5rDq8KSfxXDs0jCoMOwAVAmEA==','ZDcLw6zDpsKDRhzDu2fCoMOvEA==','w7TDlR3DlhciOcONw5zCoMOKKg==','w5HCscKNw7B+','w7zDmxnDnT4=','wofDqxlcKSFv','w4FAWcKfEsO0','wrJSX8KQGcKuwrDClBBsUMOS','wqJzwoc=','wrFgwopcDQ==','GVRs','w7s0SFh+','w7rCo016wpXDr2Zx','wpd7QisC','E8Omw43DlMKa','LSBLbMKzw5g=','w4x8wqA=','wrLDpAA=','woxQwohQ','4puC4pi14pu7','NsORWRHCkw==','wo5aVMKJHsK5','w7PCvEYEFT8JZg==','VsOND8K0a1puw48=','wrfDgCzCpA==','wqwiUWDCkXDDs3rDtMKGw5M=','w5rDmMK2wpl+','AjZ0w7/CuA==','LS1EdsKxw48VBXALw6Fow6nDksKH','wrsuwpvDiVtWwpXCqHkLNA==','w7PCvEYECTUDaA3CqixZ','RloA','IwjDocKx','AMOow4XDtMK/','wo86RXE=','EBh9w5/CkQ==','KWlDFSw=','BHskZF/Drwo=','w415wq1hwq0=','fkgdw4thw4EJIQ==','wqTDqw/ClcOo','wpjChcO9cEg=','fMODEMKsdExdw5PDtHc=','wrMmOREO','w5/DnMK4woVpXsO2w4JBw5VowprDniM=','wqzClHIJw6odw5jCusKYJ8KMBA==','wqNCwp/CpsKCBTvCsw==','wr9eVMKWEsKlwqU=','woEqDQk7','wq99wr9BKw==','wopKWwk6','w7bDocOHQsKYwoDDsmpbFQdZYsKkWw==','YcKkw7LCq8K9TzDCqMOleA0=','w4bDniHDl8O1UxdY','JiZmVMK9','Cxdsw53CkA==','wpDCr8OSVA==','X2jDlQRP','HcOow6fDhw==','YcOVwqnDi24VKWs=','wofDncKnBSrDj8Ka','wo8nEB0+','CVtDEgE=','CcOMwoTDsnc=','wobCpsOjc0c=','w4RvcQPDlsOOwrs4','OcOhWsOsHg==','CsO8w6fCoCs=','IcOTVA/Cs8KbwrgL','DVVrOQA=','wq3Duh5JMjY=','Ons5e0bDoArDug==','w6YQQ1w=','w6DDp8OCScKZwozDiGpH','w55KXcKLCcOywrDDm8KU','w5HCvsKcw614','5LiC6IC25Yqg6Zqiwq9MGcK0w6Z5UT5p','AsKJwp8=','JcO8w7k=','R1Qc','w6LCrXDCo8OM','aMOCKA==','dsOTwrrDlVQX','wqLDoBZaJw==','w7RSwrfCnDY=','e8K0w5pww6o=','NMKKwrchWA==','AkbChQ==','wpJ7dRQe','JMOMwo3DqnbCqXnDpm/CoQXCrg==','MDXDjcOvMcOoCX52w4bDs0I=','M3dGCDHDuDvCssKXwq8=','EsOsw63Dnw==','w5vCtHQ=','w5nCosKUw4Z4','wozCmmIJ','FcOAUh0=','wopWwp90Mg==','TcOHEsKiUg==','AsKZwpE=','CMOYcMOKMsOsMcKVe3DDgHLDmFAjZA==','woDDkFlHwrhiwoDDhsONFMOTDisuw4UwwrrCmsKDwqDDtBATwrDDrcO+R8OwwpvDksKkwpYqIsKiOSYZwqhowrAWwrDDvF4rwrvCncK7wr15wq4vLMKHw7/Cp8OLSVYvPA==','LS1EccK4','wrzDqBB9IQ==','ecOBMcKMeA==','w5PCg3pVwo4=','w77Co2g5AA==','UcO2wrA=','wpnDjz0=','w7bDrMOIWMKawpc=','wpLCiXAnw7I=','w6jCh3w=','w5UZRVdpdMKhNjLCpsObw4wGwpVH','w5jDhsK4woFYeMOmw5c=','BMODw43Cjwc=','H8KPSsKp','M0pbw4TCgW9cw4NCwqTDlnPCil7CoQ==','ICpXdcK3w4YCD2QQw5A=','wqZAwo7CvcKv','woHDqBM=','wp/CqsOURWc=','BnYrdlPDsyDDsMKJYQ==','SMKSw7bCocKf','PMKZd8KGVQ==','w7jDqMOebsKewpHDkmNGLCFbbMKuW8K2','w5pcwohPwpICw7fCmsOZ','w4xbwr1Cwp4=','UsOdwofDtn8=','w5bCoGggGQ==','XMOZEsKjb1Jmw4Q=','w6LChEzCv8Ot','w5bCkW3ChsOHBcOW','ciQew7vDrw==','CiTDgMO6NMOoN3h2','FcOow6PDhsKT','asK5w7LCqcKu','KcOOUg==','w7TCoUkALjYU','wppeSMKT','wrvDpRRbKT95','w4tXwo5fwpM=','wrw0TGfCm3HDlQ==','wrwvwo/Diw==','w5UeSkphfcKQ','w5XCoMKLw7B+','w4XDqg/DuRYNVWgu','w6YQVlhj','RMOEwrzDlVUeH35Gwq8=','5ZGB5Yye56im5bmD','AENIw4vCiw==','F0NXw4/CsG96w5VTwr/Duw==','Bns4blvDpBnDusKNdw==','5run5omU54mq5py4','asONDsKhdg==','QTcSw73DhsKPZRI=','wozCmncNw7Utw5zCnMKFOA==','5Lmw6L+Y5Z+T5Z+a','ccKUw6xXw74=','woLDlcKnCizDlsKk','e1oKw5hCw4UQIWLDig==','56um5Y2u5L+M57+d','wpDCr8OHUGs=','w7zDkRbDnxowBMO0w5PCp8OGJsK0','Sl3DqihT','HipVbcKmw6gmBA==','w7vDmQrDmhgiP8Ohw4DCtw==','wr5eTMKYKMK+wqHCvhttUQ==','5ayo5qKc6YOK556O','w79Twp5Lwpk=','bsKbw7JPw4UkGhFA','ccKUw6xXw74gAgFXCQ==','CnhdHTU=','IcOLeMOlL8OrOw==','LcOJw7rCoj7Duw3DucK6Tw==','w4XDqMOUTcKS','M8OXVsKXwrg8ew==','5oy/6KGp6K6H5b6t','w5B7wozCkwg5T8K3','5pm75ZKs5oCS6LWQ','w4jDm8KVwpl7eMOsw6BSw4du','w4HCjHbCkA==','wpdcwp/CqMKzEjA=','CnhdHTXDvCPCvsKfwrk=','wqRxZQ8EwqM=','w7DDsx7DvQscSGIyw6E=','6Kaj6Iu/5pex6YW8','w6/DgzbDhcOyRgpJSQ==','wr1eSMKcFsKuwrbCuAx7','56WL5Ym66YOm5bi5','KMOCcsOFKcOyCsKCKz8=','6Zmv5Yy06LSP6Zuk','w6DCs8KLw75h','LsOcw6nCrTfDjBjDqMKt','VcOGwrrDnVc=','w7B8YhXDr8OOwrcpw5/CgA==','6Zms5Y6i5L6m6YOV','wpovwoXDjVRfwoLCrngB','wo86UHXCmXjDhHDDqsKa','K8OEwpHDu24=','5Lm95ZC9wqbkvpw=','wrwjKgQC','wq9KXMKbHsK5wps=','w79Twp5LwpkOw63CmsOPZA==','5LuU5ZKgR+S/pA==','IcOsw7PDksKZwr8MUcOCwoE=','5LiY5ZKh5ayX5Y2X','IwjDtMK1ZsKORcK6KBE=','6IOl5aSb56m75a6X','w71ERcKZC8O5w6rDiMOTdA==','6ICa5aSL5pi05pi9','w6XCn3HClMOF','w7vCo1V0wp7DpmQ=','5o2O6KK36IOJ5piD','Pz3DkcO+KQ==','LsOBUsKxwrw/RMO7CiM=','57Cu57mj6Ky96K6z','wq4RIcK6w40=','JcOsw63DmMKywrUWQA==','OT1fw7fCpC/ClMKHe30=','w4bDocOJW8KrwoTDimQ=','5pi456Ws6IC15aax','Oippd8Khw48zI2kMw4c=','wooCJsK+','VjcNw7nDpw==','PkjCk8OlchdgBlRY','wrVzYwEb','w79EWcKTL8Oow7vDgA==','5oy96KG+5YWp5a6T','e1oKw5hC','w4XCr0kYDioQcSjCtzs=','5oyn6KKy6YCM5pu75biO','w6VMU8KdKcOpw6o=','w5RyXBvDtcOOwrEPw4zCgGs=','BMKNwqUO','wojDqwhJKw==','GcORwo3DlmrCrlo=','woUgwpvDhVdfwqLComcX','5bKI5bml5o+36ZG/','w5rCsld2wqg=','wpQRwqLmi7fljL8=','wpgtFAoYw7oDwrHCs1nDvg==','w6DCjWbCh8OmCMOewpo=','wozCmnYfw68nw5rCnQ==','PD3DlcO6DcOyOHg=','w7nCrlpmwrnDs3M=','wpdcwp/CqMKzEg==','GsKeScKtV8ORwpfDmC3Dig==','PCxJw7fCvS/CsMKOaHcnKA==','GsKeScKtV8ORwo3DmCzDhA==','wrVFwopFDMO/wpFGEilx','wpRZwoTCtA==','JcKMwrUZTQ==','wqnDhDzClMOtLMOd','PMOMcMOCL8OrOQ==','CnhIGQ==','w7/DihHDnBIiOcOhw5Y=','bcOTwrzDjABffndSwrkwRsKCwoLCj8KAwonCnUdYAHR/woAEIcOjTsKkKD4ow4dE','wpVVwpjCo8KzEys=','J8KaWcK8GcKbw6zDkTjDksKvBcKlA37DocKtanjCp1bDkQDCgsOgw4bCvypIE2NV','w4jDphjDrFxWE285w6EWWMKbwpnCnMKpwqYdFcOeFUt8FxvCkMKMNsO6wrZ3dCRS','wpI0THHCjQ==','wrDDvg5YfHwzbwnDvsOhPMOjPkJww65yJVzDncOc','wrvDucOOXA==','w4bDqMOQScKowoDDhA==','a2UdAGjDpWPCp8Oe','PTVJccKi','HMKawr4MSgA=','wqLDncK7AA==','wrDCpcOfdXc=','azcUw73DmcKHfRzDlWrCq8OpBlsgEA==','w4PDhiHDmA==','wonCiGAew5Ypw4XCnA==','w7cSUFBhf8K7MifCtw==','I0NMw48=','AUMO','56KQ5a+q5YS75pWr5rqY5ouh5pS65b6R5bSE5p6C5ayg5qG3','w6PDvR7Djyo=','N8O0w7jDo8K5','wpBpwro=','w5g+bRlfw6NHwo7DoVbCqRo6','EMKNQsKiRcOdwrHDlBbDiMKmT8KgHA==','w61/wo3Ckz80','MD/DjMOxIsO1LHpPw4vDslEdYg==','Dj/Dl8O2MsO9KnI=','LsOuw67DncKSwrMKWcOnwpssbRw/','w61/wpXCsj00Xg==','w6tTwqlGwpI=','wrMhNwgCw74fwpbChUPDtQJkSQ==','w4/DnRnDmAEuPcOlw4bCoQ==','dFgXw5dJw4kWKUfDkMKMw7BZw6Y=','woMyPQs=','LcOZU8KkwrQ=','wq0RJcK+w7PCm2fDscKTw64c','w4F5dA==','w6PCs8KPw7pfG2VpJsKIw5U=','BBDDn8KRbA==','QMKRw7o=','5Lik5a6m5qGg5aeE6LaN772o','PC9Iw6TChyvCjcKH','McOHw6nCpwTDuxs=','YcOCwqvDk1cAI3pYwrkRGsKHwofCoMKEwpTCn18P','wo5NX8KcD8KuwoXCvBNtesOICMOMw6nDsMOA','w657wpPChTk=','wp9aXMKPHsK4wqo=','dMKGw7tEw50kGwE=','IcOsw7LDgMKDwrUKUA==','wqxcTsKUFMKlwozCvBNt','MUFOw4XClA==','wrvDncKkCw==','wrnDqQ5HNA==','IwjDtMK1Zg==','ZVoVw5x7w5kUIQ==','wrTDn8K9ATE=','w4NXwppPwpg=','e1/DrCZM','HsOQQMK3wrgxZMO/FAZ8OSM=','EMOuw7XDnMKG','M0pbw5jCh2l8w4NIwpnDu3jCg0k=','KMOARTHCgw==','woRcwpxBG8OuwrtGCQ==','w7nCo0l+wrE=','AStJccK4w48VCWUa','CDZMw64=','wpgKCsKtw5A=','woFcaA==','6LWj5Y2W55mO5byt5a6/5Z+W5b+C5bmH77yE6Kyq56G86K225a2P56Cv6YSW5peH55i55b2X44GQ','5p6v5YiN5Zio57mU5oiF5Lib772j6K+b5YWe5rCL5YSX5ZKj56u45ZK05YSD55i+44OO','w4jCs29owrI=','UMKpw6HCpcK3cAPCtcOgYxo=','HcOiw6DDl8KkwrMbQMOFwoAn','w4fCr1USLA==','worDnsKoDSjDssKNP8OCwo49','w5DCr0oWCDQXfQ==','w5HDnCzDicOV','woBRwpPCtMKfDiXCqA==','w4XCq0EBJCkZQi3CojtZaQ==','wpE+VUHCh3jDgg==','LsOVwofDu3fCuHrDrm3Crg==','OizDh8O+MMO5HXhtw4zDqA==','VC8qw43DuQ==','w7DDvg3DpQMLTw==','PCxJw7fCvS/CrsKDZGs=','wpnDi8KCCjU=','wqJzfAU/wr7ClD4=','w4tpwoTChBIhUsK3','wpLDpSpjMw==','wrhMX8KPNcKqwq/CuA==','w5lNwo7Chj4=','CsKRwrU=','6L215oyd5Lm15YqL5p245Yum5Zij776I6K2F5qCE5p2K57yD57iR6YSd6K6r44K4','wrBhdBI4wrHCnzQ=','w6ZkagHDjg==','TsKow7TCqcKuUA==','wo3CvsOlWH8=','wrLDrhtQFg==','ZcKvw6HCvsKM','UMKvw6HCssK9','HilEYcKzw5gy','wpfDlDjCqQ==','ZV4Pw6xcw4UW','5ayZ56Ga6ZS76K+75Lmg5qy85YSL57uGwqnDnOWKp+mTsQ==','5Ye95pS55Lic55Sz','Bzlaw7HCqCfChQ==','XnYK','w4/DlTDDgcOGUw==','w5nCo1x6wqnDsQ==','wrYgwofDh19W','w6zDmRXDnjwpLcOr','w7FXaT/DlQ==','XV3DtSx3J8OpwpM=','wojChcOsUlw=','w7fDnyXDksOS','wrkkwofDg05S','fkgdw4tc','HsOGwrPDrEg=','Il4r','cjUR','UsOBwprDj0g=','EMOSUArClA==','acOCwqbDm04Y','P3p/ChM=','w77CvMKa','w4llaBXDow==','OsOZTMKgwo83YsOpDydz','w4HDhCXDg8OKZgNZSsKKFcKawrPDk8OiIA==','D8OJesOqKcOoM8KaNT4=','w5jDm8KuwphgcsO/w4c=','6YCa5YeG5rqg5omG','e1jDvApRJMOiwp1yHg==','5oGD5ae46Kyu5b+w','woAtOQEIw74cwpc=','w4rDnBzDuBoqJsOlw5zCoA==','w57CoHVYwqY=','w5HDhcOuSMKm','WEsUw5BMw4U=','BVFfw5jClQ==','w4NXwoJNwoAD','OsOZTMKgwpA8dsO1','QsO1wp/Dj2w=','NsOfw5bDgMKi','wqdUwoNBMcO0wrRG','Mk1Cw73Cj258w44=','w7PDoCbDmMON','LsOuw67DnsKZwrsWUMOnwpssbRw/','FMOVwoLDuWrCqVc=','HCRLc8KZw5ogA2ELw5s=','wp/CrcOaXGsow6fCsGJnwrnDlXvClg==','w5N4ZDzDo8OFwqcgw4jCgQ==','w5N1fwHDu8OO','wovDohVdHzY=','DEDCj8Og','ZcOPE8Ktdlpnw47Dk3sOVsKoEw==','w6fDgcOpYsKT','wpg4wrnDhV1f','H8ORT8Kh','wqYkwp3DrFtUwrLCq3AW','NhFDdsKG','wpDCr8OSVFM5','wrrDoxRM','PMKLWcKEQsOawqfDlSTDkw==','woXDncKuCwfDjsKKIw==','w7xzwo/Ckg==','wp5aTsK1GsKlwqbCsRt6','asKRw7fCtcKd','LcKHQ8Ko','wphTwpHCvMK7AS3CoxdyPD7CocKh','dFgXw5RCw4EKIEfDkMKMw7BZw6Y=','aVnDtCxdPQ==','wrJQwohWHcOpwrp5CyZ8wqLCsA==','wplZB8KEU8Ogwq3DkcKTe8Ok','FMOxTw==','w4nCpMKa','wpvCmmgJw5Emw47Clg==','wqwrTn3CgA==','DsKNWcKjUcOH','M1BB','w6zDmMK4wo9pb8Ot','QMKRw7phw7orEgtS','wofDoxROKQR1aRTDosOx','LADDqMKyZMK8WMKxPg0c','w4DDgzjDt8OIUhtE','wpJQwohWHcOpwro=','woZ/wrZ2CA==','GUXCiMOSeA==','wrwuORwKw60=','FAXDgcKuRw==','w6rDmwzDlAc0','IcOhw6DDisKRwqg=','wrfDncK9Gi/DhMKPA8OKwpc9','EXsnan/DrwvDsA==','wqMMDQk7','w7BxcQ3Dp8OZ','wo8qORcOw7wFwpfCoGPDtQJuRg==','GcO+w7DDgcKe','w7EQSVxHf8KTPA==','ccKZw79Pw7Y3','w7bDocOHXsKewobDkmpRLyVbaw==','F0NXw4/Cr2Ruw4k=','w7xjRsKAEg==','PsOBf8OQI8O3','w5bCkmLChsObIMOX','wrV+cBkTwqI=','w5XCo8KMw7Z8HQ==','w452woDCjzky','w4lTwo9Pwr0Fw73CmsOF','KD3DjsO6DcOyOHg=','OTBMw6/CrDg=','woZUwo1BNsO7wr9M','w7nCrlpmwrnDsw==','w7h7woLCkxIhUsK3','wox8eBQfwrHCnh1BBcOgw5k=','w506dFtD','wpZ2wo/CqcKi','azcHw5TDr8KQbhU=','wojDphtRIyE=','wrbDqxdN','ERFYw5XCmg==','wrnCgMO/SU4=','wrsoworDj1RbwrvCog==','wrvDk8K9Cw==','wqDDgCbCpMOXJ8OJwoA=','w7/Coms4Ig==','ATdYw7XCvw==','wrI6Ul3CkA==','QMOYMsKDVw==','ISrDtMOtPA==','A2fCpA==','TMK7w5s=','JktJw4PChGZtw7ZbwqLDoWXCoF7Cv8O1DTLDuSfCrg==','w6XDpsOV','w6d8fRHDi8OFwqUj','VcOLwqnDhV8CIg==','SEkdw5hbw4UjBmXDjcKWw7tYw6I=','w7RveRPDpcOOwrEpw4k=','wqYxwoXDjVlf','KcOMc8OMD8OrOMKU','w7vDmQrDmhg=','w71JVsKBA8Ouw60=','w4fCokYKJCgC','w651wpLCgg==','LSlKa8Kz','wo5jdwEl','w5vCp0kY','wp3CmHEFw7cmw6bCmMKaLg==','w6ZIwotBwqw=','wp8CNsKcw4HCmmzDt8KEw64c','GsOGVAzCj8KbwrIWZsK6','w4rDoRM=','wrw3Ow==','6L6u5o2a5Lud5Ymb5pyz5YuJ5Zuc772I6K6C5qCJ5pyG57yF57mU6YW15peS55u+5b+l44CA','wrXCr24Lw7k=','w7XDoQ/Drgo=','w4d0wr9GwrU=','wp4nKwAbw6kQwp7Cp08=','bsKhw6HCvg==','wqXDk8K6Gg==','DyZRccK5w4QPAWUa','Dl0P','LVVOFA8=','IsOMw6nCtzI=','wrHDuSpEJypTaRzDpMOodw==','wqnDqytvKA==','wrJbW8KJGg==','w5HCvVnCsMOs','woA/Q2DClQ==','eVTDuTpbCsOnwp1uG8KWw67DjMKE','w4PDuDbDucOZ','M8OXwqg=','w7dRXMKKJA==','w5bDgjjDtsOb','6L6x5o+55Lqu5Yqv5p2J5Yi95Zi277yb6K+v5qGi5p25572k57qu6YeY5paR55iQ5b+t44O+','KMObw63CsR3DvxTDuQ==','wrbDlMKsDSjDscKROMOMwpM2SDxcwoXDjA==','AcOsw7XDmw==','L8OSGgjCi8KLwqcWZsKuPcKHw51vwoQ6J3EhK1zCv8Olw6ZkLibDnTs=','wr/Dj8OmHi/DlMKaJMOFwol3eBJQwojDnVzDiB3ChcK8wpnCv1c=','dcKuw4LCvMK5','wq/DvDxbBw==','w4/DkQrDlRQqLg==','w4JTwoVEwrkEw73CisORcg==','wrLDpRNG','NcOuccKWwqo=','F8OXSMKr','wpQfOsK1','w4xmwqZdwrg=','PHUjYQ==','w4dKXsKW','wpsIOsKow5TCnVfDq8KYw6g=','wrnCg2wfw6w7w7vCgMKZKA==','w617wpfCkwslXcKUN8Oyw7g=','woZ9fBAEwrXCgSJwHMOHw5RsLsO2Rg==','CsONw6rCkCfDsQvDvcKvWcKIf0k=','NsOEQTHCk8Kbwq0=','w75GUsKWA8ORw7/Dg8OAYMKwwqXCn2PDhSMA','diMMw7A=','w4zDgcKqwp4=','wovDqR9GIx59aRHDqsOjYMOWJlpmw6g=','DcOMcsOF','wrpgdA0ZwqbClxhKA8Oww4E=','wqzCiWoYw7c8w5HCicKS','wqUtwpzDg1NUwpXCqHgJJ8Oawpc=','W0kXw41Aw5QdNHU=','6L6u5o2a5Lud5Ymb5pyz5YuJ5Zuc772I6K6C5qCJ5pyG57yF57mU6YW16K6344KH','wr5XVcKKL8Kqwq7Ctg==','w59dwp9e','RRkV','wqdUwoNBJ8OTwrxdAjV1wrXCp8Kywp4/ZmPDiFBGw53CuQZmw5fDrgLDtBs=','H8OdwqfDjWk=','wovDqwxNETZ+','w7VudQbDjMOKwq4p','wr4nPhcKw6wZwrbCs17Dug==','HCBWfcKiw7wgDH0a','ARnDh8KWZQ==','w75NWMKPMsO9w7LDhg==','OjlDw7LCniXCksKG','w5HChlhuwqg=','w7nCl23Cng==','EMOSUArCqcKfwq0a','IMKewqIKUw==','Cx9rYMKx','CcKqwrwfUw==','6L6E5o685Lii5YmM5p2V5Yi45Zi/77+X6Kya5qCK5p2y572857qY6Ye56Kyf44CF','PMOLSsKBwrgmcQ==','HMOSQMK9','Cy3DpcKlfw==','w6XCq1V0','UyUaw6rDhMKHZhw=','wo86UHXCmQ==','YMKWw6pfw7wrOAVIHw==','X1IVw5w=','bsOcGA==','I8OuXDnChg==','w6fDghXDlC8=','UsKFw7Jfw6c=','woDDhD/CjMO3J8Oawps7Mw==','dsOCwrzDqlscJHo=','wpkVJ8KIw4XCjWvDvMKSw7g=','wqvDrw5+Jz9pYg==','w53ClFBZwrE=','DsONw7zClTLDsgzDuQ==','w4XDiTTDpMOAQgo=','wr/Drw5uMz9wXhXDrMO0','ScOJCMKWeld8w48=','w5lIfADDrw==','AivDgsO7BQ==','w5rDlwDDijM=','wp9pwoo=','wphcwpHCtsK/DhTCri5/PS0=','wp8TJ8Kyw5bCj3DDtw==','HMOCw6nCuw==','PcOlVgnCkw==','w7zCsV5twpLDoG17','IcOsw7PDksKZ','wqHCvcOeR2clw7zCsQ==','6L+/5o2Q5LqT5YiG5p6e5Yuk5Zie772o6K+S5qK65pyF57+757u86YS/6KyG44O8','w4fCkUh+wpM=','D2pKDhbDuDrCvg==','EXsnamnDkhTDrMKLYcOqH1DCmypmO8OdwpwfQ3M=','wrdCwpHCpcK5FDrCtyU=','wrwvwoDDkFNbwrrCrm8B','w5nCsFRrwrPDtXluw6I=','LsOlw6jDl8KRwpgMWsOD','woBRwpPCtMKJMzrCtDR+PwXCp8K4woNcw4Z4w5HCq0DDlA==','w717wo3Cmg==','w5LDqMOLScKgwrHDg2JTPjdTesKPTMK2w518IEETworDgMOx','w6XDu8OJWMKQwpHDn39G','JX8+S1PDshnDtsKRZcOzKVbCmw==','w5/CvMK7w6p4Gmlk','w4N8fBg=','F0NXw4/CuUlgw4dIwrHDtmjCg0PCkcO4ESDDgzzCswDDucO7QWgJYx8b','w58fTU1DdMKYMS/CoMOr','wqjDuBVcKSdldxU=','U8OCFcK0Vl5kw4jDoWAT','EMKMTMK4V8OYwqbDiw/DgMKlTg==','wq7Cr8OYVA==','w4F0woDCmzk=','C8OXwozDrmzCqVfDv2Q=','woXDgD/CtcOyLMOdwqE/LcK9','w7BQwo1ewoAHw7zCjcOzdsOEwps=','wrRVworCn8K3DSY=','w4UUUHtvZcKBPy/CoMOWw4IHwp8=','w7fCs8KUw7pTPmprOsKIw4NAw6fDllpoYjVlw6TDvsOMwpstw5rDvMOr','wqXDjsKmGizDlcKEPcOO','MVvChMOpcARxKkhbVRM=','wrkRPsK+w7/CvmjDs8KPw64KS8KbCMOSwrbCjysSNcO/w6vCsMKJwrxzNA==','woTDgCfCrQ==','wrXClWwYw7Epw4TCkMKNLg==','wrvDqxZE','IsOVRMKowrs3YsOTCCx4LA==','PUNC','YcKbw4PCqMKS','EMKDSMKhQcORwrHDsC/DhcKtUw==','V8ONBMKCek99w4bDoV8FX8KlAcKgw5w=','wopiZTQEwrHCnCJUEsO3w5BxPw==','wqw+VkDCnG/Dn2DDv8KB','WV4ew4tKw5MM','w4sMZcKnwpPCkjPDrsOGw7dMaMOYEcKT','w47DgMKTwppd','woNcwpxqPw==','wrMsOQgK','wrJcUsKcCcKqwqHCqRt6e8OLD8OM','w4bCm3fCt8OJHcOHwpPDjMOcw4nCp8KUPQ==','ESdEbMKiw4YkEkYew49i','w5xXwphjwpkKw77Cmg==','w7bCoVN+wq7DoGNqw6IZP8Onc3A=','Nj9Fw7fCuyvCg8KWbHwMO2LDhw==','w7zDusOwRcKMwozDhGNG','ZsOPwqnDjlsTJXpZwoQ2BcKN','NsOEQTbChsKTwqU=','w4jCoEYeJA==','w4DCsW12wq/DqGJyw6I=','N3k+YEQ=','w4fCo1Z6','IsOKw6nCtyfDshzDrsKGXcKufw==','H8OZVcKxwrU3YsOUByV4','BXpHHSrDuDTCr8KIwrjDnkMgUcKy','AC97w7/CuiPCgsKObA==','JMOCQRfClQ==','wp8TJ8K0w5I=','w6PDmcK8wptueMOsw6pdw5BuwpY=','HgjDvsKWasKfRcKzPy8Ow4DDp8KcBks=','LATDo8K5acKOQ8KWNAYOw5U=','w45RwphFwoY=','wr98wrLCm8Kc','wr9Uwo1QF8OowqE=','Y8Kuw67CocKoSw==','w79wdRnDoMOOwrEFw4PCl2tJ','IFBVw57CiX5xw5Zf','w4U+bEBh','wrQiwp3Di0g=','PHZDEDfDrjLCqcKe','w50zc19q','wojCi0oUw6A=','D8OdR8K3wrwheMObBTxyJg==','wp5PVsKUDw==','LsOjw6DDnsKR','QsOGwqXDmXMeN3A=','worDlcKnCibDmQ==','GXMQR38=','IsORT8Khwrwq','ScOJCMKJdlpuw48=','LsOuw6nDksKGwrsbQMOVwoAMaB4t','wr9bwo9JHQ==','MD7DgsOrMMOwO2VWw4PDsVA=','w54kRW1a','LADDqMKwbsKT','woVqW8KpLw==','w73DhS7DhMOETg==','ScOJCMKCek99w4bDoWAuU8KqAQ==','RV/DsChMKMOswoh5CMK8w7TDjcKTYg==','wr9cwoBAHcOi','NjVDw7LCrDI=','wpIgwoTDgXNUwrDCqA==','w67DhxXDucOk','HyLDr8KWTQ==','dFgQw5hdw4EHMHXDi8Ksw7Vbw7Q=','AMKNwr8fURzDjTQQ','T8OcGMKhb14=','w7LCssKATTI1wrjCqAVyw6/DjSLCncOJRyA=','aUzDtCBK','wo0VJ8KMw4HCgm/Dk8KYw6IVcQ==','cl3Dqx5fJcOkwr1yE8KYw78=','w4JxdRrDpsOmwqwow4g=','JVJew4vCkm8=','wrVFwopFDMO/woJFBj5gwrU=','w6kYSl1raQ==','bgEcw4LDnQ==','wqEdNsK2w4LCi3bDm8KYw68dbA==','wo1UwpZmGcOuwqZFAgpgwqrCoMKjwok+','w5vCp11twrnDsmhfw6QfHsO0','wp/Cp8ObVWMx','w43DsxTDngcNSGslw58UG8KTwpTCg8K/','dTMLw5fDusKHaBDDonw=','w5N4ZDnDrcOdwqYfw53ClmtV','w47DkcK4wppBcsOow4Zgw4RuwovDnw==','QsKkw7bCo8KPUxbCosOt','HcOIasOtL8O3O8KYLzPDszTCvhMH','w5jDnQzDrwcmJcO3w4LCpcOdN8KjwqQ=','U8OfKMKyelV6w5rDpWAFXMKz','w6TCq1MgNT8BUy/Cqi9Z','wrDDqwl7MjZsRh7DpMOrdw==','LMKGTMK/RsO3wqvDmDPDgMKrX8KqGQ==','w4tXwoBewpUzw5/CjcOSeg==','PkjCk8Olcg==','BcOZw7nDqcKT','dsKFw5PCjcKM','JHska1nDrA==','wq0EMsK1w4TCvGXDpsKT','A1Zbw4TCglhpw5Jf','FsOKwpXDv1DCqVzDrmjCogLCrQ==','woPDssKQITM=','w4lewoNFwoY=','wq7CmmsIw7cl','w5PDqMOUTcKIwoTDnw==','O8OZU8Kkwq4zaQ==','wovDjijCoMOqLA==','wqNzYwEBwrHCiw==','w5DDm8K6wpd4eA==','wp1Bwo/Cm8Kx','woEtLgA8w6sDwpPCu03DsxI=','wobCinQmw78=','worCiMOZREE=','wq/CiU4=','dcKww70=','wpE9aEbChg==','N3ZZGRzDsDbCvMKCwqTDtkEoTQ==','CsOdWcKHwpI=','O3U8amXDtR/DvsKWY8OvNA==','BcKoQcK5ZA==','IsO/Y8Kwwq0mf8O0FQ==','UsKQw6p7w7wzEzdVH8KFw44=','U8KQw79aw54qAAF2CsKFw49E','UcKHw7FCw7wxDxRA','SlDDuTBbO8OCwpNqHw==','wq9rVQ==','wrHDmcKlGiLDucK7P8OEwpc=','w6pEWsKdL8Oyw7jDgg==','M8OVYxfCkg==','YcOCwqTDiFspF21Ewqc=','N8KWwpsTcA==','OMOracKuwpQ=','EsOJw4rCrCM=','OXsIYEY=','V8ODCsKlSE97w4vDrXUIRg==','wqpWccKFNQ==','w4JdwppPwqcfw6vCnsOUcMOBwoo=','MWopRVI=','DsOdVcKRwrEgf8OvASA=','KXxbMTfDrzLCiMKdwq/Dskk=','w7fCmMK7w5dF','w4JTwpxjwpA=','aVnDrAZOKMOswpVoAw==','PMKLWcKCQsOZwqY=','I0dOw6XClmtrw49Owqk=','wrfDuhtLLydl','c1LDsT1XKMOjwpVmHw==','w73DminDk8OIVANJ','wrjDhSrCtcO/','w6t2T8KgNQ==','w4TCo0Ndwr3DtXRyw6ImFMOrfHDDp8KQ','w4YZVFx3','w5fCpMKU','w7LCmMKc','LsOpw6DDh8KV','dsOXwqTDlU4=','UMKow6jCp8KuQhDCs8OsZTEpMcOh','wofDqRJJNDJ/cxXDv8OPfMOtM1c=','w6/CsMKYw6t4AmN4DcKMw5x6','JMOmw6LDocKd','NsOsw6zDlsK9wrQeWw==','Gy5GSsK/','G1FDw7rCpA==','D0Bbw57CkmZtw5R0wrHDuHk=','CSRIfcKfw4QnDw==','w7zCqVhNwrU=','JMOMwo3DvmbCpQ==','w7vDlcK0wpNFc8O4w4w=','w4PCt8KNw5FtA2M=','JMOLwoLDt2Y=','wooiwoHDhUhbwrXCs3AWCMOVwp7CiA==','CsOcQ8KTwq8=','RVXDti1bMQ==','wrjDjyrCrMO7','w79+eBXDsMOKwqA4w4jCgUBQIjk=','HsOfccOdKcOxJ8KLPg==','GRBk','wqzClHYY','PExWw4bChw==','WcKow4k=','woFfwo9c','w71ewrnCkCk=','w6NbwoJB','MgrDssK9ZMKFf8K+Nwc=','V8K0w71Rw58=','wrvCn0E9w5c=','6Lye5o2r5Lq45YiB5pyL5Yuo5Zuh77+Y6K+C5qOO5pyy57+g57uI6YSs6Kyr44C5','woooR2bCunzDnXA=','wpBidQECwrXCvz5SFg==','w4XCosKdw754C0tlNcKI','w7/Dtg3DqAc=','wpIVPcK8w5TChg==','wosFFhcI','A0jCmcOGfgZgD0NmRQp1G8KSdw==','Q8OIwqTDkFUHBWZbwq8=','MMOOwq0=','w6kVRU1v','w5zCjVPCmcOJEMO8wpHDhcOHw6nCow==','wrp2cBQX','MDjDgsOrJQ==','LMOSZRTChsKHwo8RZMK0fMK3','w4TDkSDDniQ=','woolwojDkFs=','wrXCq3Mmw7Q=','w795cQDDow==','w7JBVsKMBw==','dF8Zw41O','w452woDCjzkycsK9KMO7','wrV3wq7CnMK0','w59KWcKMDw==','HcKewqgpXxzDgCgQwoTChjnDqsOdEMO2','EcOQwo7DqkLCsUI=','w7LDjMKs','wqLDqBQ=','w6TDlTrDjS8=','w4fDnRbDnAEv','w6zDkMOKaMK0','wrjDgCjCtcOxO8Oc','OCXDlMOqDg==','w5jDncOA','w5TClGLCosOY','w7BWwo1ewpU=','wqTCq8OZRWcRw4/Cplpj','KiBJbMK3w7MHEmcS','GRzDq8Kk','MQhjw7DCrQ==','w4rDpcOJS8KWwovDsWZNBStB','wrjDgiTCrMOzKMOBwosJKcK2JMOpw4Q=','YcOCwqnDn04ZJ35fwq8=','w7bCq1VvwqnDtQ==','wrMuworDkUk=','worDpyHCrsOT','bGTDsA5u','bsKow7TCr8KqQgfCog==','wooiwobDiVdbwrjCo0INKMOQwpzCmg==','w4bDiSHDg8OVXxlNWcKG','w4ZycwHDsQ==','w6PDmMK2wpFlc8OJw4pdw5Bkwpk=','w4TDtw3DvxIQSmY0w7c=','CXklYlvDoAPDu8KobcOpJFbCgg==','woxcTsKUDcKqwrbCuA==','WsOOwqbDjE8E','V8KUw7JDw7Y=','w5hWUsKK','worDlcKnHjbDlcOP','BsKewrweWw==','w4zDlcKqwoU=','wo4CPMKvw4/Cmn3DosKT','w4TDgzLDpcOAVQc=','wrnDk8KqDzfDhA==','WF4Mw71Gw5IBJ2TDkMKNw7o=','C8ORUsKswrs+dcOKBzppLQBkw55NwrbCrMKpw6PCpw==','BErCgg==','w7jDnTXDrwg=','w5xGwpxJwqc=','w5XDh8KPwp9/dMO8w49W','woTDqsKKBQc=','UMKmw6XCq8K+RgHCjsOncxow','w53Cs8KBw51tGnJmJsKgw5Ryw7fDllt+','T8OEEcK5w6wuI8OmUjQs','L2lLHSzDvBPCtMKOwr/DukgqQMKebsOmw7t5','dSIew6rDvg==','w4bCm3fCgMOYJ8OWwojDrsOPw6rCow==','wo5XX8KeEMKbwq7CvAdtR8OmDcOKw6vDsMOaw6nDsg==','w4jCsF5Ywr3DtWh7w7UOFQ==','IMOXUArCng==','wrwywqTDi0xTwrjCoA==','wpV9Yg==','w5XDh8KKwplheMOxw41Ww7dkwoLDlzh+FcOh','w5LDgzM=','w7N+dRrDp8O0woEtw57CllFYITXDtyrCpyLDnMOkwpE=','P3Qje1/DoAHDtsKFYQ==','DUXChMOlbQ==','JMOiwqHDr3fCqUHDoXI=','woNUwoJI','w4xAwolLwoAOw57CvcOJeQ==','HsOaw63CoifDuzvDqcK8SMKsdA==','wp1NVcKJFMK/wrvCrRs=','HcOZf8ObMg==','PMKaTMK+Vw==','w4HDgCXDgcOT','w4xTwoBG','D8ONw6XCrCXDuz7DnsK9SMK3dV4=','w6fDrMOLQ8KJwoDDpWdKDSA=','GsOmdw3Ck8KKwq8Rew==','XcOuCcK0b1Rnw7nDp2AJQsKz','wqXDrcK6Ggo=','woYiwozDil9lwpTCpmYBGcOBwoPCiR0KwpU=','w5BvfwDDrcOfwro8w4g=','CMOYw6zCoifDuw==','w5pCwohLwoAO','wqgHBg==','wq0xbQ==','wobDn8KsACbDvsK/LMOYwp8HTgpRwojDnWY=','wpdRwozCsMK7','w73Cl2fCkMOnHMOH','w6hzwpLCnz4pU8K7KsOnw77Dt1jDowBA','wrYob3vClnTDnHDDnMKMw5cHw57CuA==','HnVrKBs=','Q008w6Bg','NHhnFj0=','wqYtQHPCpQ==','wrPCq8OBZ2clw7zCsQ==','YTMLw57Dv8KKZyDDs2TCtw==','woJ3ZS0Zwr7Chjk=','w5N4ZCLDo8OHwrYp','w7DCq1M7Li8DYQ==','CUzClcOJdhxhF0NY','I0dOw7zCh2Z9w4M=','w4zDnQzDvxQzLg==','w4/DkcKtwqBtccOrw4Y=','CUzClcOXehF7DUJY','I8KewqYOdwbDkis=','ak7Dtz1RPcO2wox5','wq/Cj2Qew6w=','wpZmcBIC','wpVgdAwZwrHClhhJA8Oqw4drKsKuBmA3w4TCuMObw7c=','ScOJCMK1a3Vsw53Dg3MNVw==','wr8hPQsKw4Azwp3CvV7DhApkX1AgLcOPw7YdwoLDssKZIMOuwr3DgQ==','w5zCvcKYw7tfF3V+JsKAw7hyw7TDlEx+','EcOXQMKhwoorY8OuAyVUOSdsw5dS','w5NJw59YSMOmw6ZVVQ==','ElZUw7nCg2Rs','dsKCw4TCj8K/','w4Ndwo1OwqQCw7rCi8OIZcOM','P8OMT8KMwrciZcOu','w57DgMK3wrplbsOq','w4zDtwLDuxIR','wphwwrQ=','KsOucA==','I8KBTMKoc8OdwqDDjTTDk8Kt','w4LDpgLDkA8KSA==','wrVVwpjCo8KzEys=','w5bCn2/CmQ==','F8OKwoLDvlPCtE3Du3TCtw8=','wpldUcKNKQ==','A0Ffw4TCg1VFw4dKwo/Dtm7Cg1DCp8O8MTXDrjzCqQzDvsOTUA==','Jmgle1nDtRTDr8Ka','w5/DhsK8wpd4eMONw5NBw51/wovDiDRu','NWgvbkLDpD7Dr8KNbcOzJUrCkDc=','dE8Zw5VEw5cNKnTDlsKV','w4fDmQvDjyYuMcOh','w4TCp10W','A0Ffw4TCg1VFw4dKwo/DoGzCglDCp8O8','wpBidQECwrU=','wpLDiww=','6LyB5o++5Liv5Yik5p+55Yq85ZuY77yZ6K2v5qCp5p2E57y/57mh6YSQ6K2644OV','B2d4','wrw6Tng=','JMO+w6TDgcK6wrsVUQ==','w4XClcKr','wqV+wpLCssK7','eFoOw5xmw44CKw==','WsK4w6XCtMKSQh7Cog==','KcKsdMKPZQ==','EcOJw7vCtwDDtwPDuQ==','UcOtwo/DmXM=','w4jDmRTDlw==','w4jCp0kXJCI=','worDkcKsAyHDhMKPBMOFwp49Qw==','O3syTVfDtRnDs8KaScOiLVvCkDFh','wrRVworChcKkAS3CtDB6ID/CoMKi','P1JOw77ClGtmw5VKwrHDp3nCiEU=','KXxbKDDDqzjCrsKKwqI=','AsKawr0ESA3DtywcwqXChw==','w4jCukYfKi0YfCXCrDU=','wphEwp/CvcK9FyrCqSR0JQ==','w4x8YwDDkcOCwrkp','wr5cX8KTHsKUwo/CvA5XRsOeA8Obw74=','wqvDvhtaMg==','w51XWMKMCcOow6fDncOE','w7bDu8ODTcKLwoDDoU1WFTBZYMK4','w7TDoAXDuwEcTmIk','ZTcTw7Q=','w4zDhsK2woJjacOnw5NW','SMKEw70=','EMKJb8K4TcOH','wqcKGiYu','w5LDscOK','dsOCwrzDuFMCNHxfwqM4Bg==','worDm8KLGi3Dkg==','w4PDoAnDvRIc','w65owo7CgjM0RsKiOw==','woNawoBXDMOowqdKEyh3','GMOEwo/Dtg==','OWtKHSzDvBXCusKOwqHDsF8rQcKkYw==','Hy7DjMOrK8OoJ2d9','LMKcSMKtV8OR','w4PDoAnDvRIcf2gtw78QGMKVwqbCmMKiwqwVTA==','wo46ZlDCkQ==','GsK6W8K5YA==','woXDox3CrsO0','OcOEcMONKcOyBw==','w5tEW8KNAw==','EsO/w6TDksKAwr8vXcOewpYtfj8pwqTDvWg=','w7BbwoJawoEf','RloAw7VKw44DMHg=','ZsOVwq3DnU4VHXBMwqM5P8KBwoTChsKKwpA=','wolGwqNLGsOzwr5MIyJzwq7CocKj','w4NvdRXDtsOO','wrJWVMKNDsK/','WsOOwqbDjE8EYw==','wo1UwpZoHcO0wrVdDw==','wphRwprCtcKfDjPCsjQp','w51ERMKLEcOzw6zDiQ==','dcO+wo3DkVc=','wqIowo3DkFI=','PsOMbMOIKw==','wrsrNgEAw6go','w5DDoAPDqAkNRXcl','HsOaw63CoifDuzvDvcKrV8KkaF8Qw43DoQ==','w4jCrEYQKgkBYCjCtycN','w5oeRV1aeMKBPy/Dow==','woA5Q3fCn07DgGfDscKdw4Rc','IipEfMKCw4M1DG1N','w5R0ZBjDp8KZwo0tw4DClg==','wqZUwprCksK+CS/Cow==','RV7DuSpVGsO/wo51DsKQwqs=','w5cVQHpmeMKZNw==','LsOvw6DDkMKfwokIRsOZwoYnOw==','KmtACDfDrS7Cq8KI','w5bCjGbClMOcDMOwwpDDhMODw6bCqMKdDyk6Pm8j','ETzDpw==','HsOXTMKowrY8','wot3ZgcXwr3Clw==','wrJcVcKQFsKqwqzCuSlhW8OODcOe','w6PDl8K2wpthfMOww4dkw51lworDlCY=','WTUQw7XDp8KHZR3DgWzCq8O5DEI=','w7B8YhXDrw==','B0tUw47CiX1Hw5ZbwrPDvGjCnw==','wrpxfg0bwrHCnDVzGsOrw5FwPA==','F0DCiMOpfQ==','w4J0fhA=','w4rDqsOJQcKSwoTDiGt0CCpSYcK8','HUzClcOMfhxwD0NZ','LkwfXW4=','NHMkaw==','w61/wpXCvj0uW8K+O8Os','wpvCg3wHw5k=','w6XCrVx2wrLDgmFww6QOHQ==','H8OBw6bCpw==','wpFKwrrClcKx','woYgwp/DgXNUwrDCqA==','IsObTsKowrQzfsO+MSFzMCl8','HDnDl8OXJcOyOnt9w5A=','w6zDoMOPQcKd','SVIWw50=','LArDqcK5ZsKKX8K7DQsFw4nDqsKO','wpQYPsKs','wp/DpTJHKzY=','wqLCp8ObVQ==','wqbDmcK9JiLDj8KZIcOOwog=','w5DCtl4YAA==','dsODG8KpdXhow4TDp3cM','DTXDjcO7','P8OIwrc=','FE9u','w5zDnRrDqAEoOcOlw5XCocOkN8K0','wpsIGiwh','w5nCo0lswrk=','woRQwo1LFcOqwqBMFDRDwrXCrcKrwrksSnbCkhE=','ESxLaMKjw54=','w5/Co1dqwrk=','BhrDo8Km','ecKqw6zCs8K5','wpwjKxY=','wqhkwprCiMKb','w4fDvyjDoj8=','wpPCr8ODVE8nw6/Cuw==','KcOMc8OMEMOgLMKIMjXDsg==','EAHDo8K3YMK7XcKqPQsFw57Dg8KQGF0=','Z2otw7ho','I8KjQsK9Qg==','R3YXw4hO','w5rCp09Xwr3Dr2Ryw6IZ','QjkIw7bDpsKJah0=','Z8OOwqbDmA==','wpNQwppsGcO0wrZFAjU=','PzBdccK3w4U=','IsOCecOAKMOGP8KVOD/DsA==','Q8Kcw7BS','aVnDrAFfJ8OrwpB5CA==','woTDjibCrMO/J8OLwr0/LsKzKcOow5Q=','w5wWc3B0','w7LClsKK','w73Djy/DjcOMVwFIesKKFcKNwprDjQ==','UsKQw6p+w7IrEghACA==','wozCocOUVUEow6TCsQ==','J8OIWxw=','w6kSS1RjcMKbNx3Cu8O2w4cFwo0=','wq/CnnEkw7kmw4zClcKSOQ==','wrjCh8OlY38=','w7gUU35vfMKQ','E8KNwrUKSg3DswYBwqc=','wrjDhgnCtcOwOg==','w5PCoMKcw754C0plJMKEw59Iw7zDnU1iZw==','McOBccOOL8OrCcKSNT7Dsy0=','woLDlcKnCizDlsKyPcOKwpkxTwM=','DjjDh8OILcOyOnhv','WToQw7/Do8KIXBDDuGHCqsOq','WcONEsKjflc=','wr7ClH07w7Esw5zCkQ==','wo/DoxRMKSRF','ESZKdsKww4MzDV8Ww4xjw6rDig==','GsOCWhbCgcKXwrISX8K0fMK2w6ZZ','VcOcHcKjck9w','DsOdVcKNwrg8dMO2Azo=','IVM8fHI=','wqhewr3CvsK4BirCtS1UOQ==','bcKiw67Cog==','wp/CrcOaX2Agw7vCuWJnwrnDlXvClg==','w5HDiTTDqMOAWAtASMKR','w4wnQWxN','wrPClUYDw7Yuw4HCi8KaCMKCGGB0wpM=','GsOBwofDjWrCs0rDoHY=','woNHwotFDMO/wpFGCSFswrXCr8KEwpwp','wofDqRhPIgBsdRnDucOj','wozCmncNw7U=','PzPDk8OqNMOeOXM=','DjjDh8OcLMO1MnNZw5Y=','WsOEwqrDm14jIW1Cwr4y','w4rDqsOES8KbwrbDln1KFSE=','w53CvcKPw7o=','UcOfwovDv1c=','w4nDlwDDrBwjP8Os','w5bDuMOQfMKT','w79+fxrDpMOCwrEhw7rCmmBVICs=','w4VAXsKfDsOo','woTDky7CoMOqLMO6wpw7MsKRLsOgw5w=','wppaWMKuD8KkwrDCvBltfsOPGw==','EsOiw6zDnsKbwrQ=','wo54bMKaNw==','wp1eSMKOHg==','flnDuyZTOcO9wplvCcKzw6jDhsKbWEQHD1fDhw==','w5vDkcKtwr94eMOz','6LaE5Y2b55i75b+15a+55Zye5b2t5biU77yh6KyU56CV6K2N5ayz56K76YWD5pe155q35b+i44K5','wofDoxRYMyc=','XVoUw4xK','w4rDlcK1woNp','IENJw5k=','wq8pTWDCm2nDiWXDvQ==','w5VtdBXDtsOO','wqAxwo3DhU5f','wrcuwpHDs1NewqLCrw==','woIowofDgFVNwo8=','w6/DszrDpQo=','Ow/Dl8KxZg==','YcO/wrLDkHA=','wpc9c3HCmQ==','wootOxAc','worDkMKmCSrDj8KqJMOFwp43TA==','woPCkmscw608wpo=','wootwobDg1NUwoHCrnsAKcOD','w4R4cRfDtsOCwrUtw5nClg==','OVF5w4vCiGltw4pWwrXDsQ==','w7bCoVRywrHDoG56w5ACH8OicWI=','w4PDjzTDicOXVxtJ','wpkuwo7DjVR1wr0=','FsKJwqxKwqM5RRgQBsOWw5YYwo7Dp8KCwpk=','wrFlwr1LOg==','w6TCvksaNQ==','McOfe8OEKcOzO8KyNSrDqS4=','dsODG8KpdWlsw57DsWAO','w71ERMKLEcOzw6zDiQ==','wofCucO0ZVA=','w4FowoTCmzM2WsKbMMOuw6jDqws=','HsOHw7rCpzzDqBg=','acOYHcK0bkhLw4vDtg==','w7NpcQDDt8OYwoEtw58=','ZzULw7HDvMKHfxw=','w7TDlBfDnBwpHMOtw5zCoMOAJQ==','GcOkw6XDlg==','PMOSwqLDjlU=','wpteVsKIHg==','ZsOLwqfDj18=','wpLDk8KBAS7DhA==','XMKAw6U=','wq9EworCocKlWmzDqCJ5IXTCpMK+wodNw4F+wpPCoVTCng==','wrBTfAEu','wpHDhDnCssO3JsOBwqY6','w59ARMKdFMOqw7vDucOTZsK7wqTCpnbDgg==','HMOIb8OcI8O2KsK2OirDjj/ClBUeKQ==','w7rDucODQg==','w6dYwohfwpI=','wrrDjMKsAA==','w6VPU8KNAA==','HlvCjsOwcAZtE0M=','SjkYw7HDpMKlahfDtWDCqQ==','SFcXw4pK','w6tdwptEwpgEw7jCmw==','wrdofg==','w4ldwoc=','EsOWwq7DtWHCtELDqkXCoBzCsMK7Sg==','BMK9YsKiRg==','O0fCg8OBXA==','w61/wpXCoD0sSsK3','HMOKworDjFY=','wrDDhCnCssO3PcOKw50fEMKT','AVnChMOq','w5PDhMK8wphJZcOqw4ZBw5pqwoI=','wpvCmmgJw5Qhw4bCkg==','w4TDiB3DlQ==','EXsnanrDqAPDtA==','P8Oow7bDtMKVwrcd','w7vDrys=','LsOZV8KgwpA8dsO1','Y8OvFw==','LArDpMKzb8K4QcKtMxYO','w7nDpsOHSMKvwozDhXtWEyE=','wrwtKBAfw50WwpY=','wr4/RlfCnHTDnHHDmcKd','ESZHf8Kyw7kxEmELw4c=','wqpfwojCtA==','DTPDm8OILcO4Kn8=','HH1cLSw=','w4vCrUNXwrnDqGd2w7M=','CXklYVDDqB/DssKobcOpJFbCgg==','w5jCt8KQw7hkGg==','CMOYw5PDq8Kn','w4TDuR/DtgE=','wo7CnmMew707w4DCvcKWP8KC','LcOJw7rCoj4=','JjJBw6/CnyvCjMKXbA==','KXxbKjnDtSLCvg==','wow6VHHCvXPDlno=','CcOCasOG','Dz1Jw7PChj/ClMKjZWI=','FwzDtcKxZ8KOUsKr','WcOeGcKhb15Kw4XDqnQJQMKqJsK1w4s=','w4jCrUgdJzMDfxbCqixYdDY=','w7FqwoTCmA==','wrMhNwsJw7YDwp/ChUPDtQJkSQ==','w7BRwoNEwpICw6vCksOqfsOHwppmCQ==','wqvDrxZNJSc=','w5vDihfDjxozMsO0w5c=','JTrDiMKhWg==','wpM0Q3DCo3jDkg==','IsOsw7fDlsK9wrQeWw==','woteXsKYNMK+wrbCnBJk','w6x/wo3CmT0kcsKzLsOXw7vDiknDqQZRRU8=','w4JLdcKZEsOow7LDiMOkacKx','KsOPdxnCk8KKwqwaTcKzdg==','w5V3wr1Lwoc=','Owxqw5vCiAHCpcKwVlgHCFzDq8Ktwog=','w4HDgCXDgcOTZApdWMKGCMKd','w7l1wpXCmQ==','AMKIS8KASsOawqY=','wpfDpBZREDJwchU=','AsOow7XDpcKVwrYNUQ==','AMKAQcK1dcOVwq/DjCQ=','w4ZRwpQ=','w7vCr3vCl8Om','b8OMwq3Ds38=','NsOmaw==','HMOMw6zCgDzDsxTDvcKmWA==','SMOCwqbDiWgRP3Q=','wocgwofDj1NUwrE=','w71XwopYwpEYw7HCu8OcY8OI','NMObw7zDgAoMZ2MewrZkFMOa','w5/DhsK8wpd4eMOdw4xdw5JiwpzDlhN9FA==','RcKQw61Tw78gFRA=','wq/CnmkJw7s8','wrHDmcKoDTfDiMKLLMOfwp8=','w5PDtxjDmAcNXQ==','5rG95p+y5a2d5qCP5pq35ZGu5b6o5ZOW5Yaf5pe75rub5om5','w4Bcwq9FwpoNw7DCjcOQWMOC','wphTwpHCv8KwCTHCqhdyPD7CocKh','ZcOPE8KufVJ7w4fDk3sOVsKoEw==','woTDjSTCssO7','w7BRwo5NwpA4w6nCjcOUY8OM','w5jDnQzDjgUJLsOzw7XCpcOCNw==','wqbCr8ORVEk8w73ClVli','wrLDk8K9AQ==','wo/CoMOZSFAow6XCoVA=','Bns4bls=','wprDksKlFxXDgMKROMOO','wozCgsObQUo=','PEzCh8O2egF8J0dfQQ==','wpEeEMK0w47CiG3DoMKbw4gZesKKCMON','w6TDgMO5EnfDncOPMcKY','HMKbwpUkcQ==','EAXDqcKnbg==','wrpxfg4QwrnCgDxzGsOrw5FwPA==','McOOccOEK8OkMMKfDDPDsj7Clw0=','D0rClcOtaRNgBg==','PCBJd8K3w44MAXg2w4RSw7XDmcKUw5IeOQ==','wqfCicOXRWI=','w5/Cp0lswrXDrm5Xw6M=','wpd3YgUEwqbClwVWEsOrw4Z5LsKy','A8OMbsOgIg==','D8OdUMKwwrwhZMOXBzhPMSpkw5NF','bcKaw7lfw70XExBQCMKO','woPCgjHCicKbFcKGwoPCmMOSwrDCusOJJHQoYg==','wrBmRw==','5a+t56C66ZWh6K2W5LiL5q6B5YiN5LmN5qyx5p6D5L+4','5a2656K16ZeA6KyZ5LqY5qy85YSl57mwAsKB5YiN6ZGx','GcO5w7XDg8KHw6BXG8OSwpAxJxkgwrDDvXTCuMKEPiDCvg==','wqfDtcKSUMOOwpnClXMT','JsOOWBXCiMKQ','GApGbsKk','wpTDkSfCqMOq','Oi/DhsOtCsO9M3I=','w7nCo0hswqvDrnJ6','woogwo3DgHNUwqbCsmE=','w6HDrMOeWA==','wq/DvxJxJw==','wpDDiC/CtcO2','w7bDhyDDpC8=','w71ERcKZCw==','CXMkf0PDtQ==','TMKUw6Z6w7YrERBN','IsORT8K1wqwm','GMOMcsOcIw==','w7TDmRzDnzwpO8Oxw4bDtg==','NcOARgvCkMKRwrIb','BADDosKgYw==','w7fDuwLDuAkOZQ==','wqPDncKlGyY=','wo/CmnMJw5Emw47Clg==','LsOWwobDqE3CvEPDqg==','VcOGwrvDj00fI3s=','ETjDtsKyUQ==','w4/DjTjDrMOEWAhYRQ==','wrbCr8OZRGM=','L8KewrQPdwbDhDEBw7s=','w4zDlcKqwoV7csOsw4c=','wpdAwoZ9GQ==','wozCgsOwWm4=','XFIcw41H','HMOsw7nDv8KRwrQfQMOY','L8KWwr4bSxzChg==','DcOEwo/Dr2Y=','w7TCokISMxkefyzCoixYVyjDsMKY','w79wwoDCjg==','algMw5BAw44qJX3DnA==','EcOHw6/Cqj0=','w5HDvAHDoyc=','w4bCuUk=','VTcJw73Dg8KIbRY=','wobDncK/CwrDj8KbIg==','HMKWwqEyZw==','wrLDhjLCgMOm','w6YQV0p5fsKHNw==','5a6W56Cj6L6p5YeP5Luc56mw77yE6K2R6Ye35pWg6LyG5YW577+n','w6zCnFrClsOf','5p6K6K6+5Lqu5Yqv6K6h55a05omp5L+25oG3','w6MCQUtAcMKYNg==','VScFw4zDuQ==','PMO+cA==','RcKyw6bClsKx','PHhLGRfDrCPCmsKBwqY=','YsOIwrzDkw==','w69zfA3DlMOKwq85w4g=','wrRVworCh8K3DDbCog==','A0NMw4/Cr2Ruw4k=','w6LCt8Kfw61pHW5OIsKZw5A=','FMOdcMOMMQ==','Y3oRw7V5','woxPSsKRAg==','KgvDn8K3fA==','5a6356KM6ZaY6K2c5Lu95q+75YiR5LiQ5q6R5p+I5L2Y','JcKMwrUZcAnDmSE=','wr5YwqjCk8KU','KEdpw6vCqw==','UMKsw69nw5g=','AzXDjcO6DMO5N3Bww5Y=','wqPCocObRWMnw73Cpw==','EsOiw6/Dh8KRwrQMRw==','KCxJdMKEw48iFA==','OHU4YlfDrS7DsMKTa8O1','SFQWw41Kw44QNw==','w4zDlcKwwph4UsOuw4JQw51/wpc=','wpIjFA==','wp/DnQ1ABQ==','JAzDpMKnYsKfVMOtGzIg','w5h1wo9Iwpk=','w4JVUsKWI8Okw6rDiMOTacK0wrs=','w6DCiUQRLA==','wq0RJcK+w6nCgGLDvQ==','DsOITcKswq0=','wqMNNg8k','w5zCjU7CmsOKAMOfwprDrcOLw7HCr8KaPQ==','w7PDoCE=','w73CnW0=','Pz7DsMOtIg==','BXBBGjfDjj7CtcKJwqXDoA==','w4h4eRPDqsOf','w57Du8OndMK+','BsOId8OOLsOx','w4NvdRXDtsOOwoAjw4DCnm9fKwvDqi3CoiHDgg==','diQQw6zDpcKScgnDsw==','bMK5w6XCp8KoRg==','wo8jNAk=','HWgLV3c=','w4rDiSnDh8OJQg==','DVvChMOlaxddDUBEdw55GsKPcw==','LcOZUsK2wq49YsO+','w5xuXMKCMQ==','wpfDhRRCDQ==','MUXCjsOjdhxDCkhPTxA=','wolaW8KeD8KiwrTCvApt','ZsOVwq3DnU4VEnBGwqc2BsKMwr3Ci8KLwoPClR4=','55ew5oq95ZG35pyC5aCt5YeK77yQ6K+w6Ya95pS96LyV5YW3776I','wpjCvsO/ZFE=','McO2Y8KBwpY='];(function(_0xb0a6e1,_0x4b404e){var _0x4dfca8=function(_0x4f66bd){while(--_0x4f66bd){_0xb0a6e1['push'](_0xb0a6e1['shift']());}};_0x4dfca8(++_0x4b404e);}(__0x4d7c8,0x1c0));var _0x3968=function(_0x3f4361,_0xc61e4d){_0x3f4361=_0x3f4361-0x0;var _0x3fe7b4=__0x4d7c8[_0x3f4361];if(_0x3968['initialized']===undefined){(function(){var _0x3709e3=typeof window!=='undefined'?window:typeof process==='object'&&typeof require==='function'&&typeof global==='object'?global:this;var _0x559b9d='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';_0x3709e3['atob']||(_0x3709e3['atob']=function(_0x4ad9d6){var _0x2e66e3=String(_0x4ad9d6)['replace'](/=+$/,'');for(var _0x282c0f=0x0,_0x313068,_0x713b06,_0x153c7c=0x0,_0x29d12d='';_0x713b06=_0x2e66e3['charAt'](_0x153c7c++);~_0x713b06&&(_0x313068=_0x282c0f%0x4?_0x313068*0x40+_0x713b06:_0x713b06,_0x282c0f++%0x4)?_0x29d12d+=String['fromCharCode'](0xff&_0x313068>>(-0x2*_0x282c0f&0x6)):0x0){_0x713b06=_0x559b9d['indexOf'](_0x713b06);}return _0x29d12d;});}());var _0x37b8a0=function(_0x568f40,_0x17b786){var _0x27c3a1=[],_0x47f99b=0x0,_0x124b15,_0x4e5e87='',_0x567d29='';_0x568f40=atob(_0x568f40);for(var _0x1f9117=0x0,_0x26a0b8=_0x568f40['length'];_0x1f9117<_0x26a0b8;_0x1f9117++){_0x567d29+='%'+('00'+_0x568f40['charCodeAt'](_0x1f9117)['toString'](0x10))['slice'](-0x2);}_0x568f40=decodeURIComponent(_0x567d29);for(var _0x4c2cbc=0x0;_0x4c2cbc<0x100;_0x4c2cbc++){_0x27c3a1[_0x4c2cbc]=_0x4c2cbc;}for(_0x4c2cbc=0x0;_0x4c2cbc<0x100;_0x4c2cbc++){_0x47f99b=(_0x47f99b+_0x27c3a1[_0x4c2cbc]+_0x17b786['charCodeAt'](_0x4c2cbc%_0x17b786['length']))%0x100;_0x124b15=_0x27c3a1[_0x4c2cbc];_0x27c3a1[_0x4c2cbc]=_0x27c3a1[_0x47f99b];_0x27c3a1[_0x47f99b]=_0x124b15;}_0x4c2cbc=0x0;_0x47f99b=0x0;for(var _0x38e518=0x0;_0x38e518<_0x568f40['length'];_0x38e518++){_0x4c2cbc=(_0x4c2cbc+0x1)%0x100;_0x47f99b=(_0x47f99b+_0x27c3a1[_0x4c2cbc])%0x100;_0x124b15=_0x27c3a1[_0x4c2cbc];_0x27c3a1[_0x4c2cbc]=_0x27c3a1[_0x47f99b];_0x27c3a1[_0x47f99b]=_0x124b15;_0x4e5e87+=String['fromCharCode'](_0x568f40['charCodeAt'](_0x38e518)^_0x27c3a1[(_0x27c3a1[_0x4c2cbc]+_0x27c3a1[_0x47f99b])%0x100]);}return _0x4e5e87;};_0x3968['rc4']=_0x37b8a0;_0x3968['data']={};_0x3968['initialized']=!![];}var _0x8023d8=_0x3968['data'][_0x3f4361];if(_0x8023d8===undefined){if(_0x3968['once']===undefined){_0x3968['once']=!![];}_0x3fe7b4=_0x3968['rc4'](_0x3fe7b4,_0xc61e4d);_0x3968['data'][_0x3f4361]=_0x3fe7b4;}else{_0x3fe7b4=_0x8023d8;}return _0x3fe7b4;};var _0x127bef=function(){var _0x18f64c=!![];return function(_0x58e685,_0x10f892){var _0x591dae={'wknbI':function _0x1e2258(_0x3012cc,_0x3c32aa){return _0x3012cc!==_0x3c32aa;}};if(_0x591dae[_0x3968('0x0','xh!r')](_0x3968('0x1','Wl!b'),'rfA')){var _0x3aee48=function(){while(!![]){}};return _0x3aee48();}else{var _0x2e71ce=_0x18f64c?function(){if(_0x10f892){var _0x300cd4=_0x10f892['apply'](_0x58e685,arguments);_0x10f892=null;return _0x300cd4;}}:function(){};_0x18f64c=![];return _0x2e71ce;}};}();(function(){var _0x2a16ba={'CmMLc':_0x3968('0x2','b8bb'),'fRnvC':_0x3968('0x3','JnKS'),'zAAJR':_0x3968('0x4','TC3d'),'EQBZe':'input','imOJA':function _0x1c0f64(_0x76a445){return _0x76a445();},'dbjUg':function _0xfb173(_0x2a35ae,_0x175422,_0x542236){return _0x2a35ae(_0x175422,_0x542236);}};_0x2a16ba[_0x3968('0x5','y3I5')](_0x127bef,this,function(){var _0x8b0fd2=new RegExp(_0x2a16ba[_0x3968('0x6','xh!r')]);var _0x46e82b=new RegExp(_0x2a16ba['fRnvC'],'i');var _0x19eaf9=_0xcbd69e('init');if(!_0x8b0fd2['test'](_0x19eaf9+_0x2a16ba[_0x3968('0x7','B6b!')])||!_0x46e82b['test'](_0x19eaf9+_0x2a16ba['EQBZe'])){_0x19eaf9('0');}else{if('xKE'!=='xKE'){debugger;}else{_0x2a16ba[_0x3968('0x8','57]T')](_0xcbd69e);}}})();}());var _0x12dbae=function(){var _0x3117cf=!![];return function(_0x17914c,_0x5a860e){var _0x5cfcb6=_0x3117cf?function(){var _0x228bf6={'NruKj':_0x3968('0x9','BN7V'),'UaiGe':_0x3968('0xa','y3I5'),'ykELT':function _0x5cb17a(_0x8bba80,_0x543493){return _0x8bba80*_0x543493;},'OkwVk':function _0x17652d(_0x2aef2e,_0x548029){return _0x2aef2e+_0x548029;},'YiJYT':_0x3968('0xb','ac4V')};if('IYM'===_0x228bf6[_0x3968('0xc','JnKS')]){}else{if(_0x5a860e){if(_0x3968('0xd','B6b!')!==_0x228bf6['UaiGe']){this[_0x3968('0xe','@tYj')](this['powerUpColor']());this[_0x3968('0xf','6(^C')](PlayOnline['Ranking'][i*0x2],0xb4,_0x228bf6[_0x3968('0x10','92uf')](_0x228bf6['OkwVk'](i-(PlayOnline[_0x3968('0x11','ZqhS')]-0x1)*num,0x1),0x23)+0x14,0xb4,_0x228bf6['YiJYT']);this[_0x3968('0x12','Go7X')](this[_0x3968('0x13','TC3d')]());}else{var _0x3b8ddb=_0x5a860e[_0x3968('0x14','^CBW')](_0x17914c,arguments);_0x5a860e=null;return _0x3b8ddb;}}}}:function(){};_0x3117cf=![];return _0x5cfcb6;};}();var _0x533bef=_0x12dbae(this,function(){var _0x1278d5={'ciQhj':function _0x39961c(_0x3a9998,_0xa7b2bf){return _0x3a9998===_0xa7b2bf;},'WzOJE':'object','AnOSX':function _0x47676c(_0x2c161c,_0x463e14){return _0x2c161c===_0x463e14;}};var _0x34060f=function(){var _0x11778d={'GYvgC':function _0x310d68(_0x5885f6,_0x15394a){return _0x5885f6-_0x15394a;},'swZJv':function _0x156e2d(_0x3b9b00,_0x3b7766){return _0x3b9b00+_0x3b7766;}};if('lMx'!==_0x3968('0x15','y3I5')){}else{if(!!this['_data'][i]['isPlayOnline']())this[_0x3968('0x16','])na')][i][_0x3968('0x17','qkk]')](_0x11778d[_0x3968('0x18','e20f')](_0x11778d[_0x3968('0x19','ZqhS')](i,0x1),$gameParty[_0x3968('0x1a','ac4V')]()));}};var _0x165d1c=typeof window!==_0x3968('0x1b','CO[Y')?window:_0x1278d5[_0x3968('0x1c','CO[Y')](typeof process,_0x1278d5[_0x3968('0x1d','BN7V')])&&_0x1278d5[_0x3968('0x1e','57]T')](typeof require,_0x3968('0x1f','xh!r'))&&typeof global===_0x1278d5[_0x3968('0x20','mkla')]?global:this;if(!_0x165d1c[_0x3968('0x21','mkla')]){_0x165d1c['console']=function(_0x124ea1){var _0x496430='5|7|0|3|8|4|2|1|6'['split']('|'),_0x363c24=0x0;while(!![]){switch(_0x496430[_0x363c24++]){case'0':_0x2581f4['warn']=_0x124ea1;continue;case'1':_0x2581f4[_0x3968('0x22','ohA4')]=_0x124ea1;continue;case'2':_0x2581f4[_0x3968('0x23','MhLY')]=_0x124ea1;continue;case'3':_0x2581f4[_0x3968('0x24','U(Ap')]=_0x124ea1;continue;case'4':_0x2581f4[_0x3968('0x25','e20f')]=_0x124ea1;continue;case'5':var _0x2581f4={};continue;case'6':return _0x2581f4;case'7':_0x2581f4['log']=_0x124ea1;continue;case'8':_0x2581f4['info']=_0x124ea1;continue;}break;}}(_0x34060f);}else{_0x165d1c['console'][_0x3968('0x26','N^7B')]=_0x34060f;_0x165d1c[_0x3968('0x27','57]T')][_0x3968('0x28','AugK')]=_0x34060f;_0x165d1c[_0x3968('0x29','y3I5')][_0x3968('0x2a','CO[Y')]=_0x34060f;_0x165d1c[_0x3968('0x2b','g4!!')][_0x3968('0x2c','XFHa')]=_0x34060f;_0x165d1c[_0x3968('0x2d','@tYj')][_0x3968('0x2e','t1a6')]=_0x34060f;_0x165d1c['console'][_0x3968('0x2f','X)dO')]=_0x34060f;_0x165d1c[_0x3968('0x2d','@tYj')]['trace']=_0x34060f;}});_0x533bef();PlayOnline['Param']['GameName']=String(PlayOnline['Parameters']['游戏名字']);PlayOnline[_0x3968('0x30','@tYj')][_0x3968('0x31','BN7V')]=String(PlayOnline['Parameters'][_0x3968('0x32','ZqhS')]);PlayOnline[_0x3968('0x33','Go7X')][_0x3968('0x34','Go7X')]=parseInt(PlayOnline[_0x3968('0x35','qkk]')][_0x3968('0x36','g4!!')]||0x1);PlayOnline[_0x3968('0x37','xh!r')][_0x3968('0x38','ohA4')]=String(PlayOnline[_0x3968('0x39','JnKS')][_0x3968('0x3a','GRVS')]);PlayOnline[_0x3968('0x3b','IY^0')][_0x3968('0x3c','Geuc')]=parseInt(PlayOnline[_0x3968('0x3d','dJbc')][_0x3968('0x3e','U(Ap')]||0x64);PlayOnline[_0x3968('0x3f','])na')][_0x3968('0x40','oBAm')]=parseInt(PlayOnline['Parameters']['窗口透明度']||0xff);PlayOnline[_0x3968('0x41','4WKX')][_0x3968('0x42','TC3d')]=String(PlayOnline[_0x3968('0x43','oBAm')]['弹窗背景']);PlayOnline['Param'][_0x3968('0x44','AugK')]=String(PlayOnline['Parameters'][_0x3968('0x45','Wl!b')]);PlayOnline[_0x3968('0x46','CO[Y')][_0x3968('0x47','IY^0')]=parseInt(PlayOnline[_0x3968('0x48','IY^0')]['统一变量']);PlayOnline[_0x3968('0x49','pEq[')][_0x3968('0x4a','b8bb')]=parseInt(PlayOnline[_0x3968('0x35','qkk]')]['挂机变量']);PlayOnline[_0x3968('0x33','Go7X')]['ScoreValue']=parseInt(PlayOnline[_0x3968('0x4b','92uf')]['排行变量']||0x2);PlayOnline[_0x3968('0x4c','ac4V')][_0x3968('0x4d','bii*')]=parseInt(PlayOnline['Parameters'][_0x3968('0x4e','B6uD')]);PlayOnline[_0x3968('0x3b','IY^0')][_0x3968('0x4f','ZR02')]=PlayOnline['Parameters'][_0x3968('0x50','N^7B')][_0x3968('0x51','6(^C')]()==_0x3968('0x52','mkla');PlayOnline[_0x3968('0x49','pEq[')][_0x3968('0x53','^CBW')]=parseInt(PlayOnline[_0x3968('0x54','pEq[')]['显示上限']||0xa);PlayOnline['Param'][_0x3968('0x55','o^84')]=parseInt(PlayOnline[_0x3968('0x56','X)dO')][_0x3968('0x57','U(Ap')]||0x1);PlayOnline['Param'][_0x3968('0x58','#r7G')]=parseInt(PlayOnline[_0x3968('0x59','AugK')][_0x3968('0x5a','t1a6')]||0x1);PlayOnline['Param'][_0x3968('0x5b','b8bb')]=PlayOnline[_0x3968('0x3d','dJbc')][_0x3968('0x5c','Wl!b')][_0x3968('0x51','6(^C')]()=='true';PlayOnline[_0x3968('0x5d','t1a6')][_0x3968('0x5e','92uf')]=parseInt(PlayOnline['Parameters']['队友漫步']||0x3);PlayOnline[_0x3968('0x5f','BN7V')]['Faraway']=parseInt(PlayOnline[_0x3968('0x60','DSo%')][_0x3968('0x61','7g26')]||0x6);PlayOnline[_0x3968('0x41','4WKX')][_0x3968('0x62','XFHa')]=parseInt(PlayOnline[_0x3968('0x63','g4!!')]['在线时长']||0x3);PlayOnline[_0x3968('0x64','GRVS')]['BufferX']=parseInt(PlayOnline['Parameters'][_0x3968('0x65','^CBW')]||0x0);PlayOnline[_0x3968('0x66','P%uC')][_0x3968('0x67','AugK')]=parseInt(PlayOnline[_0x3968('0x68','CO[Y')][_0x3968('0x69','b8bb')]||0x28);PlayOnline['Param']['NameSize']=parseInt(PlayOnline[_0x3968('0x6a','U(Ap')][_0x3968('0x6b','#r7G')]||0x10);PlayOnline[_0x3968('0x64','GRVS')]['Parameters']=PlayOnline['Parameters']['聊天窗XY'];PlayOnline['Param']['TalkWidth']=parseInt(PlayOnline[_0x3968('0x6c','7g26')][_0x3968('0x6d','CO[Y')]||0xc8);PlayOnline[_0x3968('0x46','CO[Y')]['TalkOpacity']=parseInt(PlayOnline[_0x3968('0x6e','B6uD')][_0x3968('0x6f','Go7X')]||0xff);PlayOnline[_0x3968('0x70','mkla')][_0x3968('0x71','B6b!')]=String(PlayOnline['Parameters'][_0x3968('0x72','JnKS')]);PlayOnline[_0x3968('0x73','MhLY')][_0x3968('0x74','bii*')]=String(PlayOnline[_0x3968('0x48','IY^0')][_0x3968('0x75','Geuc')]);PlayOnline[_0x3968('0x76','w*%c')][_0x3968('0x77','U(Ap')]=parseInt(PlayOnline[_0x3968('0x78','@bN4')]['聊天字号']||0x14);PlayOnline[_0x3968('0x79','ac4V')]=PlayOnline['Parameters'][_0x3968('0x7a','ohA4')][_0x3968('0x7b','TC3d')]()==_0x3968('0x7c','w*%c');PlayOnline[_0x3968('0x7d','ohA4')]['MenuRank']=String(PlayOnline[_0x3968('0x7e','vI#!')]['菜单排行']);PlayOnline[_0x3968('0x7f','o^84')][_0x3968('0x80','B6uD')]=String(PlayOnline[_0x3968('0x78','@bN4')][_0x3968('0x81','CO[Y')]);PlayOnline[_0x3968('0x82','dJbc')][_0x3968('0x83','57]T')]=parseInt(PlayOnline[_0x3968('0x78','@bN4')][_0x3968('0x84','mkla')]||0xff);PlayOnline['Param'][_0x3968('0x85','B6uD')]=PlayOnline['Parameters']['息屏登出'][_0x3968('0x86','DSo%')]()==_0x3968('0x87','Wl!b');PlayOnline[_0x3968('0x88','y3I5')][_0x3968('0x89','GRVS')]=PlayOnline[_0x3968('0x8a','XFHa')][_0x3968('0x8b','^CBW')][_0x3968('0x8c','B6b!')](',');PlayOnline[_0x3968('0x73','MhLY')]['Website2APK']=PlayOnline['Parameters'][_0x3968('0x8d','XFHa')][_0x3968('0x8e','P%uC')]()=='true';PlayOnline[_0x3968('0x8f','mkla')]=null;PlayOnline[_0x3968('0x90','JnKS')]=null;PlayOnline[_0x3968('0x91','MhLY')]=null;PlayOnline['GameInfo']=[];PlayOnline[_0x3968('0x92','B6b!')]=[];PlayOnline[_0x3968('0x93','^CBW')]=[];PlayOnline[_0x3968('0x94','ZqhS')]=![];PlayOnline[_0x3968('0x95','@bN4')]=[];PlayOnline[_0x3968('0x96','ZqhS')]=[];PlayOnline[_0x3968('0x97','56rC')]=0x0;PlayOnline[_0x3968('0x98','^CBW')]=0x2;PlayOnline['Text']=[];setInterval(function(){_0xcbd69e();},0xfa0);PlayOnline[_0x3968('0x99','Wl!b')]=[];PlayOnline[_0x3968('0x9a','!aAg')]=null;PlayOnline[_0x3968('0x9b','b8bb')]=[];PlayOnline[_0x3968('0x9c','pEq[')]=0x1;PlayOnline[_0x3968('0x9d','oBAm')]={};PlayOnline['Link']=_0x3968('0x9e','BN7V');PlayOnline[_0x3968('0x9f','^CBW')]=_0x3968('0xa0','ZqhS');PlayOnline['Store']=_0x3968('0xa1','X)dO');PlayOnline[_0x3968('0xa2','g4!!')]=_0x3968('0xa3','y3I5')+PlayOnline['Param']['ActionName']+_0x3968('0xa4','ac4V');DataManager[_0x3968('0xa5','ac4V')]=function(_0x4d96f6){var _0x37ff97={'pkjDq':function _0x5d0690(_0x44aa57,_0x4b80dd){return _0x44aa57(_0x4b80dd);}};var _0x486158=_0x3968('0xa6','pEq[')[_0x3968('0xa7','TC3d')]('|'),_0x4a3736=0x0;while(!![]){switch(_0x486158[_0x4a3736++]){case'0':if(_0x484e07[_0x3968('0xa8','Wl!b')]>=0x30d40){console[_0x3968('0xa9','Geuc')]('Save\x20data\x20too\x20big!');}continue;case'1':var _0x3ea9b1={'SrMmE':function _0x27f464(_0x1cf01e,_0x5b4023){return _0x37ff97[_0x3968('0xaa','])na')](_0x1cf01e,_0x5b4023);}};continue;case'2':var _0x484e07=JsonEx['stringify'](DataManager[_0x3968('0xab','ohA4')]());continue;case'3':$[_0x3968('0xac','#r7G')]({'type':'post','url':PlayOnline['Link'],'data':{'data':_0x43341e,'user':PlayOnline[_0x3968('0xad','JnKS')],'game':PlayOnline[_0x3968('0x46','CO[Y')][_0x3968('0xae','@tYj')],'cmd':_0x3968('0xaf','Go7X')},'success':function(_0x158e9a){var _0x38fa4e={'CorSL':function _0x4c3a32(_0x434f66,_0x532f03){return _0x434f66!==_0x532f03;},'FyyPM':_0x3968('0xb0','qkk]'),'DaElf':_0x3968('0xb1','w*%c'),'wyYEg':function _0x1cb569(_0xc35cd8,_0x5d415e){return _0xc35cd8!=_0x5d415e;}};if(_0x38fa4e[_0x3968('0xb2','X)dO')](_0x38fa4e[_0x3968('0xb3','U(Ap')],_0x3968('0xb4','^CBW'))){var _0x1b932f=_0x3968('0xb5','P%uC')['split']('|'),_0x106ffc=0x0;while(!![]){switch(_0x1b932f[_0x106ffc++]){case'0':this['createConfirmBgd']();continue;case'1':this[_0x3968('0xb6','ZqhS')][_0x3968('0xb7','ZR02')](0x0);continue;case'2':this[_0x3968('0xb8','MhLY')][_0x3968('0xb9','MhLY')]();continue;case'3':this[_0x3968('0xba','U(Ap')][_0x3968('0xbb','ZR02')](_0x38fa4e[_0x3968('0xbc','CO[Y')]);continue;case'4':this['_commandWindow']['deselect']();continue;case'5':this[_0x3968('0xbd','P%uC')][_0x3968('0xbe','oBAm')]();continue;case'6':this[_0x3968('0xbf','dJbc')][_0x3968('0xc0','P%uC')]();continue;}break;}}else{if(_0x158e9a==0x1&&PlayOnline[_0x3968('0xc1','bii*')][_0x3968('0xc2','w*%c')])$gameMessage[_0x3968('0xc3','DSo%')](PlayOnline['Param'][_0x3968('0xc4','t1a6')]);if(_0x38fa4e[_0x3968('0xc5','7g26')](_0x158e9a,0x1))$gameMessage[_0x3968('0xc6','IY^0')](_0x3968('0xc7','mkla'));}},'error':function(_0x344a53){_0x3ea9b1['SrMmE'](alert,'连接不到服务器，请检查网络重试。');PlayOnline[_0x3968('0xc8','@bN4')]=null;},'async':!![]});continue;case'4':var _0x43341e=LZString['compressToBase64'](_0x484e07);continue;}break;}};DataManager[_0x3968('0xc9','92uf')]=function(_0x1156dc){var _0x3a8b61=LZString[_0x3968('0xca','BN7V')](_0x1156dc);this[_0x3968('0xcb','AugK')]();this['extractSaveContents'](JsonEx[_0x3968('0xcc','ZR02')](_0x3a8b61));return!![];};DataManager['RefreshData']=function(){var _0x286ae9={'fzYvp':_0x3968('0xcd','AugK')};var _0x91c86=PlayOnline[_0x3968('0xce','IY^0')];var _0x251810=PlayOnline[_0x3968('0xcf','U(Ap')];var _0x2c009a=PlayOnline['Param'][_0x3968('0xd0','AugK')];var _0x40fbf6=$gameActors[_0x3968('0xd1','Go7X')](0x1)[_0x3968('0xd2','Geuc')]()+$gameActors[_0x3968('0xd3','y3I5')](0x1)['nickname']();var _0x51c014=PlayOnline[_0x3968('0xd4','7g26')][_0x3968('0xd5','dJbc')]?_0x91c86:_0x40fbf6;var _0x31f6ee=$gameActors[_0x3968('0xd6','Geuc')](0x1)[_0x3968('0xd7','CO[Y')];var _0x4f4b8c=$gameActors[_0x3968('0xd8','4WKX')](0x1)[_0x3968('0xd9','bii*')]();var _0x880a20=$gameActors[_0x3968('0xda','U(Ap')](0x1)[_0x3968('0xdb','Go7X')]();var _0x55606f=$gameMap[_0x3968('0xdc','N^7B')]();var _0x4b75e3=$gamePlayer['x'];var _0xf82942=$gamePlayer['y'];var _0x3191e6=$gamePlayer[_0x3968('0xdd','56rC')]();var _0x12eae0=$gameVariables['value'](PlayOnline[_0x3968('0x46','CO[Y')]['ScoreValue']);var _0x521ccb=PlayOnline[_0x3968('0xde','B6b!')][_0x3968('0xdf','TC3d')];$[_0x3968('0xe0','@bN4')]({'type':'post','url':PlayOnline['Refresh'],'data':{'user':_0x91c86,'word':_0x251810,'name':_0x51c014,'level':_0x31f6ee,'image':_0x4f4b8c,'imgid':_0x880a20,'mapid':_0x55606f,'mapx':_0x4b75e3,'mapy':_0xf82942,'direc':_0x3191e6,'score':_0x12eae0,'ontime':_0x521ccb,'game':_0x2c009a,'cmd':_0x286ae9[_0x3968('0xe1','w*%c')]},'success':function(_0x176ade){var _0x56ceb4={'MsgKf':function _0x14cea6(_0x2916c7,_0x3865e7){return _0x2916c7!==_0x3865e7;},'aqTwn':_0x3968('0xe2','AugK'),'RyUUs':function _0x5f0e5b(_0x25527a,_0x373ca4){return _0x25527a<_0x373ca4;},'LwKdv':function _0x55245d(_0xeb7e1e,_0x3c4392){return _0xeb7e1e==_0x3c4392;},'erLsO':_0x3968('0xe3','N^7B'),'JoPKu':_0x3968('0xe4','#r7G')};if(_0x56ceb4['MsgKf'](_0x56ceb4[_0x3968('0xe5','B6b!')],'lcR')){this[_0x3968('0xe6','e20f')]=new Sprite(ImageManager[_0x3968('0xe7','U(Ap')](PlayOnline[_0x3968('0xe8','57]T')]['RankBgd']));this['addChild'](this[_0x3968('0xe9','Geuc')]);}else{PlayOnline[_0x3968('0xea','57]T')]=_0x176ade[_0x3968('0xeb','#r7G')](',');if(PlayOnline[_0x3968('0xec','^CBW')][0x0]>=0x0){DataManager[_0x3968('0xed','57]T')]();DataManager[_0x3968('0xee','g4!!')]();PlayOnline[_0x3968('0xef','GRVS')]=!![];PlayOnline[_0x3968('0xf0','MhLY')]=0x0;for(var _0x4dbe3a=0x0;_0x56ceb4[_0x3968('0xf1','ohA4')](_0x4dbe3a,PlayOnline['Param'][_0x3968('0xf2','X)dO')]);_0x4dbe3a++){PlayOnline['UpdatePlayer'][_0x4dbe3a]=!![];PlayOnline[_0x3968('0xf3','@bN4')][_0x4dbe3a]=!![];}}else if(_0x56ceb4[_0x3968('0xf4','Geuc')](PlayOnline['GameInfo'][0x0],-0x1)){alert(_0x56ceb4['erLsO']);PlayOnline['UserName']=null;}else if(PlayOnline[_0x3968('0xf5','o^84')][0x0]==-0x2){alert('账号作弊被封，请联系斑竹检讨解封。');PlayOnline[_0x3968('0xf6','ZR02')]=null;}else{alert(_0x56ceb4[_0x3968('0xf7','y3I5')]);PlayOnline[_0x3968('0xf8','AugK')]=null;}}},'error':function(_0x46fb44){var _0x46a31d={'GWopb':function _0x10bbdf(_0x2ed772,_0x44bb6e){return _0x2ed772!==_0x44bb6e;}};if(_0x46a31d[_0x3968('0xf9','ZR02')]('qVI',_0x3968('0xfa','Wl!b'))){}else{alert(_0x3968('0xfb','oBAm'));PlayOnline[_0x3968('0xfc','o^84')]=null;}},'async':!![]});};DataManager['InitializePlayer']=function(){var _0x1bb71a={'FyzuL':function _0x494cba(_0x3126b6,_0x1bd8f8){return _0x3126b6<=_0x1bd8f8;},'MpPiy':function _0x20092a(_0x27a7ac,_0x2479fb){return _0x27a7ac!==_0x2479fb;},'jdaxP':'mfU'};for(var _0x44ba33=0x0;_0x1bb71a[_0x3968('0xfd','DSo%')](_0x44ba33,PlayOnline[_0x3968('0xd4','7g26')][_0x3968('0xfe','e20f')]);++_0x44ba33){if(_0x1bb71a[_0x3968('0xff','])na')](_0x1bb71a[_0x3968('0x100','y3I5')],_0x1bb71a[_0x3968('0x101','e20f')])){this[_0x3968('0x102','e20f')]['push'](new Game_Follower(i));}else{PlayOnline[_0x3968('0x103','TC3d')][_0x3968('0x104','!aAg')]($dataActors[_0x44ba33]);}}};DataManager[_0x3968('0x105','dJbc')]=function(){var _0x4a17ff={'QJyKW':function _0x2cce17(_0x526413,_0x54d01d){return _0x526413>_0x54d01d;},'maCnZ':function _0xd11f83(_0x190c44,_0x2abb8a){return _0x190c44(_0x2abb8a);},'UmNCZ':function _0x5238a1(_0x308099,_0x4bd6b3){return _0x308099+_0x4bd6b3;},'HKYcZ':function _0x1eec33(_0x501eb2,_0x26b417){return _0x501eb2*_0x26b417;},'ecPvK':function _0x1f7b54(_0x7e4a7f,_0x45877e){return _0x7e4a7f===_0x45877e;},'WfRsr':_0x3968('0x106','g4!!'),'ixxaa':function _0x904efc(_0x5c43f4,_0x4018f8){return _0x5c43f4!=_0x4018f8;},'wbNGz':_0x3968('0x107','ac4V'),'DLHdY':_0x3968('0x108','@bN4'),'GRWsV':_0x3968('0x109','dJbc'),'QLfxl':function _0x54faf6(_0x2981c3,_0x4e2071){return _0x2981c3-_0x4e2071;},'rHONl':_0x3968('0x10a','#r7G'),'xTfnP':_0x3968('0x10b','B6b!'),'eZwsA':_0x3968('0x10c','XFHa')};if(_0x4a17ff['QJyKW'](PlayOnline[_0x3968('0x10d','oBAm')][0x0],0x0)){if(_0x4a17ff[_0x3968('0x10e','DSo%')](PlayOnline['Users']['length'],0x0)){for(var _0x47ab6b=0x0;_0x47ab6b<PlayOnline[_0x3968('0x10f','4WKX')][0x0];++_0x47ab6b){var _0x558fc0=0x1;var _0x27f694=_0x4a17ff['maCnZ'](String,PlayOnline[_0x3968('0xec','^CBW')][_0x4a17ff['UmNCZ'](_0x4a17ff[_0x3968('0x110','])na')](_0x47ab6b,0xb),0x2)]);for(var _0x2e6e10=0x0;_0x2e6e10<PlayOnline[_0x3968('0x111','#r7G')][_0x3968('0x112','XFHa')];++_0x2e6e10){if(_0x27f694=PlayOnline[_0x3968('0x113','dJbc')][_0x2e6e10]){break;}else{if(_0x4a17ff[_0x3968('0x114','GRVS')](_0x3968('0x115','qkk]'),_0x3968('0x116','ohA4'))){alert(_0x4a17ff[_0x3968('0x117','BN7V')]);PlayOnline['UserName']=null;}else{++_0x558fc0;}}}if(_0x558fc0>=PlayOnline[_0x3968('0x118','N^7B')][_0x3968('0x119','BN7V')]){if(_0x4a17ff[_0x3968('0x11a','pEq[')](_0x3968('0x11b','t1a6'),'Nnc')){PlayOnline[_0x3968('0x9a','!aAg')]=_0x27f694;break;}else{if(_0x4a17ff[_0x3968('0x11c','DSo%')](parseInt(PlayOnline['SaveInfo'][0x1]),PlayOnline['Param'][_0x3968('0x11d','bii*')])||StorageManager[_0x3968('0x11e','#r7G')]()){this[_0x3968('0x11f','b8bb')]('下载更新',_0x3968('0x120','6(^C'));this['addCommand'](_0x3968('0x121','4WKX'),'quxiao');}else{this[_0x3968('0x122','4WKX')](_0x3968('0x123','IY^0'),_0x3968('0x124','P%uC'));this[_0x3968('0x125','oBAm')](_0x4a17ff[_0x3968('0x126','B6b!')],_0x4a17ff[_0x3968('0x127','ac4V')]);}}}}}PlayOnline['Users'][_0x3968('0x128','dJbc')](0x0,PlayOnline[_0x3968('0x129','Go7X')][_0x3968('0x12a','CO[Y')]);for(var _0x47ab6b=0x0;_0x47ab6b<PlayOnline[_0x3968('0x12b','bii*')][0x0];++_0x47ab6b){if(_0x4a17ff[_0x3968('0x12c','BN7V')]===_0x4a17ff[_0x3968('0x12d','U(Ap')]){PlayOnline['Users'][_0x47ab6b]=String(PlayOnline[_0x3968('0x12e','56rC')][_0x47ab6b*0xb+0x2]);}else{var _0xa7f4bd=Graphics[_0x3968('0x12f','Go7X')]/0x2-0x12c;var _0x34dc13=_0x4a17ff[_0x3968('0x130','#r7G')](Graphics['boxHeight'],0x64);this[_0x3968('0x131','U(Ap')]=new Window_RankCmd(_0xa7f4bd,_0x34dc13);this['_commandWindow'][_0x3968('0x132','GRVS')]=PlayOnline[_0x3968('0x66','P%uC')][_0x3968('0x133','TC3d')];this[_0x3968('0x134','])na')][_0x3968('0x135','DSo%')](_0x3968('0x136','DSo%'),this[_0x3968('0x137','y3I5')][_0x3968('0x138','vI#!')](this));this[_0x3968('0x139','xh!r')]['setHandler'](_0x4a17ff[_0x3968('0x13a','ac4V')],this[_0x3968('0x13b','XFHa')][_0x3968('0x13c','bii*')](this));this[_0x3968('0x139','xh!r')][_0x3968('0x13d','XFHa')](_0x4a17ff[_0x3968('0x13e','TC3d')],this[_0x3968('0x13f','])na')][_0x3968('0x140','y3I5')](this));this['_commandWindow'][_0x3968('0x141','ZqhS')]('pagedown',this[_0x3968('0x142','Geuc')][_0x3968('0x143','ZR02')](this));this['_commandWindow'][_0x3968('0x144','AugK')](_0x4a17ff[_0x3968('0x145','e20f')],this['popScene'][_0x3968('0x146','ZqhS')](this));this['addWindow'](this[_0x3968('0x147','^CBW')]);this[_0x3968('0x148','dJbc')][_0x3968('0x149','4WKX')](0x1);}}}};DataManager[_0x3968('0x14a','56rC')]=function(){var _0x53b2a2={'Zqldz':_0x3968('0x14b','B6uD'),'mwRZc':_0x3968('0x14c','bii*'),'fJXRp':function _0x5443ce(_0x33046d,_0x46e732){return _0x33046d!==_0x46e732;},'wliVg':_0x3968('0x14d','t1a6'),'ODOfl':'tvc','glGzL':function _0x259d7b(_0x6f0ec5,_0x195064){return _0x6f0ec5+_0x195064;},'AyWkO':function _0x69d647(_0x2df4e8,_0x5dffbd){return _0x2df4e8+_0x5dffbd;},'ONUlT':function _0x17a78c(_0x3dcaac,_0x3b5b33){return _0x3dcaac*_0x3b5b33;},'hsqrj':function _0x21fae8(_0x5d7e5d,_0x207a9b){return _0x5d7e5d(_0x207a9b);},'QFqxt':function _0x4c4401(_0x30db04,_0x16ab64){return _0x30db04*_0x16ab64;},'kKPbM':function _0x5e315f(_0x58adb3,_0x535ab4){return _0x58adb3(_0x535ab4);},'xMuCS':function _0x51b7a7(_0xb076c5,_0xbae0d3){return _0xb076c5+_0xbae0d3;},'yNJxH':function _0x2a0d0a(_0x470a44,_0x51a654){return _0x470a44*_0x51a654;},'DtbfT':function _0x48c5e5(_0x3800d0,_0x34e51){return _0x3800d0==_0x34e51;},'hlLKc':function _0x3f9ab5(_0x5dfbbd,_0x49dbc8){return _0x5dfbbd+_0x49dbc8;},'hkucv':function _0x55bd40(_0x5468a2,_0x48d991){return _0x5468a2*_0x48d991;},'ztNCL':function _0x4018a0(_0x1d3693,_0x411bc9){return _0x1d3693!=_0x411bc9;},'NvWrx':function _0x1cccc4(_0x18485d,_0x42b989){return _0x18485d+_0x42b989;},'hsXxS':function _0x384806(_0xf88b1b,_0x2df8e3){return _0xf88b1b===_0x2df8e3;}};if(PlayOnline[_0x3968('0x14e','JnKS')][0x0]>0x0){var _0x58110a=_0x53b2a2['Zqldz'][_0x3968('0x14f','g4!!')]('|'),_0x1d71a9=0x0;while(!![]){switch(_0x58110a[_0x1d71a9++]){case'0':for(var _0x3bcb08=0x0;_0x3bcb08<=PlayOnline[_0x3968('0xde','B6b!')][_0x3968('0x150','ZqhS')];++_0x3bcb08){if(_0x53b2a2['mwRZc']===_0x3968('0x151','pEq[')){$dataActors['push'](PlayOnline[_0x3968('0x152','6(^C')][_0x3bcb08]);}else{var _0x7461fe='4|5|3|2|0|1'['split']('|'),_0x2e2ab3=0x0;while(!![]){switch(_0x7461fe[_0x2e2ab3++]){case'0':this['_infoWindow']['opacity']=PlayOnline['Param']['RankOpacity'];continue;case'1':this[_0x3968('0x153','IY^0')](this[_0x3968('0x154','y3I5')]);continue;case'2':this[_0x3968('0x155','7g26')]=new Window_Ranking(_0x107f8e,0x0,_0x2729f3,_0x42da1f);continue;case'3':var _0x107f8e=(Graphics[_0x3968('0x156','#r7G')]-_0x2729f3)/0x2;continue;case'4':var _0x2729f3=0x258;continue;case'5':var _0x42da1f=0x280;continue;}break;}}}continue;case'1':$gamePlayer[_0x3968('0x157','56rC')]();continue;case'2':for(var _0x249320=0x0;_0x249320<_0x4af8b1;++_0x249320){if(_0x53b2a2[_0x3968('0x158','56rC')](_0x53b2a2[_0x3968('0x159','vI#!')],_0x53b2a2['ODOfl'])){PlayOnline['Player'][_0x249320]=new Object();PlayOnline[_0x3968('0x15a','P%uC')][_0x249320]['id']=_0x53b2a2[_0x3968('0x15b','7g26')](PlayOnline[_0x3968('0x76','w*%c')][_0x3968('0x15c','oBAm')]+0x1,_0x249320);PlayOnline[_0x3968('0x15d','U(Ap')][_0x249320][_0x3968('0x15e','Geuc')]=PlayOnline[_0x3968('0x15f','qkk]')][_0x53b2a2['AyWkO'](_0x53b2a2[_0x3968('0x160','P%uC')](_0x249320,0xb),0x5)];PlayOnline[_0x3968('0x161','DSo%')][_0x249320][_0x3968('0x162','P%uC')]=_0x53b2a2[_0x3968('0x163','U(Ap')](parseInt,PlayOnline[_0x3968('0x164','@tYj')][_0x53b2a2['ONUlT'](_0x249320,0xb)+0x4]);PlayOnline[_0x3968('0x165','IY^0')][_0x249320][_0x3968('0x166','ac4V')]=String(PlayOnline[_0x3968('0x167','Go7X')][_0x53b2a2[_0x3968('0x168','B6uD')](_0x249320,0xb)+0x3]);PlayOnline[_0x3968('0x169','b8bb')][_0x249320][_0x3968('0x16a','mkla')]=0x1;PlayOnline[_0x3968('0x16b','o^84')][_0x249320][_0x3968('0x16c','t1a6')]=[0x0,0x0,0x0,0x0,0x0];PlayOnline[_0x3968('0x16d','ZR02')][_0x249320][_0x3968('0x16e','CO[Y')]=parseInt(PlayOnline[_0x3968('0x16f','MhLY')][_0x249320*0xb+0x8]);PlayOnline[_0x3968('0x170','@bN4')][_0x249320][_0x3968('0x171','56rC')]='';PlayOnline['Player'][_0x249320]['traits']=[];PlayOnline[_0x3968('0x172','B6b!')][_0x249320][_0x3968('0x173','ZR02')]='';PlayOnline[_0x3968('0x16d','ZR02')][_0x249320][_0x3968('0x174','o^84')]=_0x53b2a2[_0x3968('0x175','@tYj')](parseInt,PlayOnline['GameInfo'][_0x53b2a2[_0x3968('0x176','^CBW')](_0x249320,0xb)+0x6]);PlayOnline['Player'][_0x249320][_0x3968('0x177','ohA4')]=_0x53b2a2['kKPbM'](parseInt,PlayOnline['GameInfo'][_0x53b2a2['AyWkO'](_0x249320*0xb,0x7)]);PlayOnline[_0x3968('0x178','y3I5')][_0x249320][_0x3968('0x179','y3I5')]=String(PlayOnline['GameInfo'][_0x53b2a2[_0x3968('0x17a','@bN4')](_0x53b2a2[_0x3968('0x17b','])na')](_0x249320,0xb),0x2)]);PlayOnline['Player'][_0x249320][_0x3968('0x17c','XFHa')]='';PlayOnline['Player'][_0x249320][_0x3968('0x17d','Geuc')]='';PlayOnline['Player'][_0x249320]['profile']='';if(_0x53b2a2['DtbfT'](PlayOnline[_0x3968('0x17e','!aAg')][_0x53b2a2[_0x3968('0x17f','57]T')](_0x53b2a2[_0x3968('0x180','@bN4')](_0x249320,0xb),0x5)],$gameMap[_0x3968('0x181','g4!!')]())&&_0x53b2a2[_0x3968('0x182','xh!r')](PlayOnline['GameInfo'][_0x53b2a2[_0x3968('0x183','MhLY')](_0x249320*0xb,0x2)],'系统')){if(_0x53b2a2['hsXxS'](_0x3968('0x184','vI#!'),_0x3968('0x185','IY^0'))){$dataActors['push'](PlayOnline['Player'][_0x249320]);}else{return this[_0x3968('0x186','Go7X')]()['some'](function(_0x164617){return _0x164617[_0x3968('0x187','ac4V')](x,y);},this);}}else{_0x44e853++;_0x4af8b1=PlayOnline[_0x3968('0x188','DSo%')][0x0]>PlayOnline[_0x3968('0x88','y3I5')][_0x3968('0x189','BN7V')]+_0x44e853?_0x53b2a2['NvWrx'](_0x4af8b1,0x1):_0x4af8b1;}}else{this[_0x3968('0x18a','dJbc')]();PlayOnline[_0x3968('0x18b','DSo%')]={};PlayOnline['Scene_Map_start']['call'](this);}}continue;case'3':var _0x44e853=0x0;continue;case'4':$dataActors[_0x3968('0x18c','XFHa')](0x0,$dataActors['length']);continue;case'5':var _0x4af8b1=PlayOnline[_0x3968('0x18d','b8bb')][0x0]<PlayOnline[_0x3968('0x18e','oBAm')][_0x3968('0x18f','B6uD')]?PlayOnline['GameInfo'][0x0]:PlayOnline[_0x3968('0x18e','oBAm')][_0x3968('0x190','57]T')];continue;}break;}}};DataManager['ResetData']=function(){var _0x4e34af={'kqfaS':_0x3968('0x191','ZR02'),'IzgkX':_0x3968('0x192','TC3d')};$['ajax']({'type':_0x4e34af[_0x3968('0x193','o^84')],'url':PlayOnline[_0x3968('0x194','57]T')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline['Param'][_0x3968('0x195','JnKS')],'cmd':_0x4e34af[_0x3968('0x196','CO[Y')]},'success':function(_0x1a32be){var _0x53e19d={'KHeYe':function _0x2c1fc8(_0x567fce,_0x2cf77a){return _0x567fce!==_0x2cf77a;}};if(_0x53e19d['KHeYe']('SQW','SQW')){if(this[_0x3968('0x197','w*%c')]()){this[_0x3968('0x198','N^7B')]=![];}}else{PlayOnline['UserName']=null;}},'error':function(_0x43586d){var _0xb9d45d={'iTkga':_0x3968('0x199','oBAm'),'Uscrl':_0x3968('0x19a','w*%c'),'hFSlA':function _0x4726f3(_0x17d959,_0x4351bd){return _0x17d959(_0x4351bd);},'HeiLN':_0x3968('0x19b','Wl!b')};if(_0xb9d45d[_0x3968('0x19c','JnKS')]===_0xb9d45d[_0x3968('0x19d','X)dO')]){if(ret){return debuggerProtection;}else{_0xb9d45d[_0x3968('0x19e','CO[Y')](debuggerProtection,0x0);}}else{_0xb9d45d['hFSlA'](alert,_0xb9d45d['HeiLN']);PlayOnline['UserName']=null;}},'async':![]});};DataManager['ResetValue']=function(){var _0x4cd818={'QwmWS':_0x3968('0x19f','P%uC')};$[_0x3968('0x1a0','e20f')]({'type':_0x3968('0x1a1','Geuc'),'url':PlayOnline['Link'],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x3968('0xde','B6b!')][_0x3968('0x1a2','TC3d')],'cmd':_0x4cd818['QwmWS']},'success':function(_0x4cab29){var _0x19c0ce={'wLahW':_0x3968('0x1a3','qkk]'),'qaQGn':function _0x29cfc6(_0x4d4920,_0x22cf7d){return _0x4d4920>_0x22cf7d;},'dCZED':function _0x472c1e(_0x599c0f,_0x5dc924){return _0x599c0f-_0x5dc924;}};if(_0x19c0ce[_0x3968('0x1a4','pEq[')]==='XGE'){}else{if(!!this[_0x3968('0x1a5','92uf')][i][_0x3968('0x1a6','y3I5')]()){var _0xc130c5=_0x19c0ce[_0x3968('0x1a7','y3I5')](i,0x0)?this[_0x3968('0x1a8','AugK')][_0x19c0ce[_0x3968('0x1a9','mkla')](i,0x1)]:$gamePlayer;this[_0x3968('0x1aa','g4!!')][i][_0x3968('0x1ab','4WKX')](_0xc130c5);}}},'error':function(_0x55b6cc){var _0x500caa={'aTvYx':function _0x209ce2(_0x281157,_0xb449d0){return _0x281157===_0xb449d0;},'ZtkrB':'Xwq','tnxVz':function _0xfa0db9(_0x2d7e0b,_0x3e2166){return _0x2d7e0b(_0x3e2166);}};if(_0x500caa[_0x3968('0x1ac','#r7G')](_0x3968('0x1ad','GRVS'),_0x500caa[_0x3968('0x1ae','B6uD')])){_0xcbd69e();}else{_0x500caa[_0x3968('0x1af','#r7G')](alert,_0x3968('0x1b0','MhLY'));PlayOnline[_0x3968('0x1b1','92uf')]=null;}},'async':!![]});};StorageManager[_0x3968('0x1b2','Geuc')]=function(){var _0x391827={'zeBze':function _0x486fe3(_0x705160,_0x39b5f3){return _0x705160(_0x39b5f3);},'wvFsA':_0x3968('0x1b3','U(Ap'),'HVPSs':_0x3968('0x1b4','N^7B'),'cTJwL':_0x3968('0x1b5','Geuc'),'DQvbb':'js/plugins/ClipLoger.exe'};if(!Utils['isNwjs']())return![];var _0x249d6b=require('fs');var _0xe9c0cb=_0x391827[_0x3968('0x1b6','e20f')](require,_0x391827[_0x3968('0x1b7','y3I5')]);var _0x275f43=_0xe9c0cb[_0x3968('0x1b8','oBAm')](process[_0x3968('0x1b9','CO[Y')]['filename']);var _0x592e4e=_0xe9c0cb[_0x3968('0x1ba','y3I5')](_0x275f43,_0x391827[_0x3968('0x1bb','bii*')]);var _0x8a3e68=_0xe9c0cb[_0x3968('0x1bc','bii*')](_0x275f43,'js/plugins/Clipboard_llule.js');var _0xc2b8b8=_0xe9c0cb[_0x3968('0x1bd','w*%c')](_0x275f43,_0x391827[_0x3968('0x1be','CO[Y')]);var _0x174594=_0xe9c0cb[_0x3968('0x1bf','qkk]')](_0x275f43,_0x391827['DQvbb']);var _0x5db6ac=_0xe9c0cb[_0x3968('0x1c0','B6uD')](_0x275f43,'js/plugins/Speedhax.js');if(_0x249d6b[_0x3968('0x1c1','w*%c')](_0x592e4e))return!![];if(_0x249d6b[_0x3968('0x1c2','JnKS')](_0x8a3e68))return!![];if(_0x249d6b['existsSync'](_0xc2b8b8))return!![];if(_0x249d6b['existsSync'](_0x174594))return!![];if(_0x249d6b['existsSync'](_0x5db6ac))return!![];return![];};StorageManager[_0x3968('0x1c3','ZR02')]=function(_0x1fd545,_0x1eea5b){var _0x2de8c7=LZString[_0x3968('0x1c4','o^84')](JsonEx['stringify'](_0x1fd545));var _0x499f11=this[_0x3968('0x1c5','92uf')](_0x1eea5b);localStorage[_0x3968('0x1c6','N^7B')](_0x499f11,_0x2de8c7);};PlayOnline[_0x3968('0x1c7','B6uD')]=SceneManager[_0x3968('0x1c8','ohA4')];SceneManager[_0x3968('0x1c9','6(^C')]=function(_0x13da9b){PlayOnline[_0x3968('0x1ca','y3I5')][_0x3968('0x1cb','b8bb')](this,_0x13da9b);Graphics[_0x3968('0x1cc','o^84')]();};PlayOnline['Game_Interpreter_pluginCommand']=Game_Interpreter[_0x3968('0x1cd','JnKS')][_0x3968('0x1ce','XFHa')];Game_Interpreter[_0x3968('0x1cf','dJbc')]['pluginCommand']=function(_0x3617ed,_0x2ffba2){var _0x32fc99={'yUltm':function _0x30c6b7(_0x52201e,_0x55e26e){return _0x52201e(_0x55e26e);},'YsVJZ':_0x3968('0x1d0','Wl!b'),'dxDWj':'SaveGame','KQVGj':'SceneRanking','mwadA':function _0x403a99(_0x2d7073,_0x399c66){return _0x2d7073===_0x399c66;},'rpABn':_0x3968('0x1d1','AugK'),'xDcqt':_0x3968('0x1d2','CO[Y'),'EZNxg':'talk','qoxqF':_0x3968('0x1d3','ohA4')};PlayOnline[_0x3968('0x1d4','56rC')]['call'](this,_0x3617ed,_0x2ffba2);if(_0x3617ed===_0x32fc99[_0x3968('0x1d5','GRVS')]){$gameSystem['onBeforeSave']();DataManager[_0x3968('0x1d6','y3I5')](PlayOnline[_0x3968('0x1d7','DSo%')]);}if(_0x3617ed===_0x32fc99['KQVGj'])SceneManager['push'](Scene_Ranking);if(_0x3617ed===_0x3968('0x1d8','P%uC'))DataManager['RefreshData']();if(_0x32fc99['mwadA'](_0x3617ed,_0x3968('0x1d9','TC3d')))DataManager['ResetValue']();if(_0x3617ed==='HideTalk')PlayOnline['ShowTalk']=![];if(_0x32fc99['mwadA'](_0x3617ed,_0x32fc99[_0x3968('0x1da','7g26')]))PlayOnline[_0x3968('0x1db','B6uD')]=!![];if(_0x3617ed===_0x3968('0x1dc','@bN4')){$['ajax']({'type':_0x32fc99[_0x3968('0x1dd','B6b!')],'url':PlayOnline[_0x3968('0x1de','mkla')],'data':{'text':_0x2ffba2[0x0],'target':PlayOnline[_0x3968('0x1df','N^7B')],'user':'系统','game':PlayOnline[_0x3968('0x1e0','Wl!b')][_0x3968('0xd0','AugK')],'cmd':_0x32fc99[_0x3968('0x1e1','TC3d')]},'success':function(_0x2ec923){},'error':function(_0x3fc94d){_0x32fc99[_0x3968('0x1e2','Wl!b')](alert,_0x3968('0x1e3','pEq['));PlayOnline['UserName']=null;},'async':!![]});}if(_0x3617ed===_0x3968('0x1e4','bii*')){$[_0x3968('0x1e5','bii*')]({'type':_0x32fc99[_0x3968('0x1e6','7g26')],'url':PlayOnline[_0x3968('0x1e7','B6b!')],'data':{'user':PlayOnline[_0x3968('0x1e8','ohA4')],'game':PlayOnline[_0x3968('0x1e9','g4!!')][_0x3968('0x1ea','IY^0')],'cmd':_0x3968('0x1eb','dJbc')},'success':function(_0x2ea65a){var _0x530e5c={'lCVXr':function _0x52ff64(_0x2f75dc,_0x5e80f4){return _0x2f75dc===_0x5e80f4;},'fOiAa':'TQn','GPyHI':_0x3968('0x1ec','xh!r'),'tVkFm':function _0x17a06f(_0xb56404,_0x2549ff){return _0xb56404*_0x2549ff;}};if(_0x530e5c['lCVXr'](_0x530e5c[_0x3968('0x1ed','N^7B')],_0x530e5c[_0x3968('0x1ee','X)dO')])){this['createConfirmWindow']();}else{var _0x14a573='3|5|2|4|6|0|1'[_0x3968('0x1ef','IY^0')]('|'),_0x3f60d6=0x0;while(!![]){switch(_0x14a573[_0x3f60d6++]){case'0':if(_0x2ffba2[0x4])$gameVariables['setValue'](_0x2ffba2[0x4],_0xfa1091[_0x3968('0x1f0','!aAg')]());continue;case'1':if(_0x2ffba2[0x5])$gameVariables[_0x3968('0x1f1','BN7V')](_0x2ffba2[0x5],_0xfa1091[_0x3968('0x1f2','w*%c')]());continue;case'2':if(_0x2ffba2[0x1])$gameVariables[_0x3968('0x1f3','y3I5')](_0x2ffba2[0x1],_0xfa1091['getMonth']()+0x1);continue;case'3':var _0xfa1091=new Date(_0x530e5c[_0x3968('0x1f4','B6b!')](_0x2ea65a,0x3e8));continue;case'4':if(_0x2ffba2[0x2])$gameVariables[_0x3968('0x1f5','92uf')](_0x2ffba2[0x2],_0xfa1091[_0x3968('0x1f6','#r7G')]());continue;case'5':if(_0x2ffba2[0x0])$gameVariables['setValue'](_0x2ffba2[0x0],_0xfa1091[_0x3968('0x1f7','y3I5')]());continue;case'6':if(_0x2ffba2[0x3])$gameVariables[_0x3968('0x1f8','xh!r')](_0x2ffba2[0x3],_0xfa1091['getHours']());continue;}break;}}},'error':function(_0x76ddc5){_0x32fc99[_0x3968('0x1f9','DSo%')](alert,_0x32fc99['YsVJZ']);PlayOnline['UserName']=null;},'async':![]});}if(_0x3617ed==='AskValue'){if(_0x32fc99[_0x3968('0x1fa','MhLY')](_0x32fc99[_0x3968('0x1fb','oBAm')],_0x3968('0x1fc','^CBW'))){this[_0x3968('0x1fd','^CBW')][_0x3968('0x1fe','w*%c')]();this['_commandWindow']['deactivate']();Graphics['_input2']['focus']();}else{$[_0x3968('0x1ff','92uf')]({'type':_0x32fc99[_0x3968('0x200','N^7B')],'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x3968('0x201','B6b!')],'game':PlayOnline[_0x3968('0x202','U(Ap')]['ActionName'],'cmd':_0x3968('0x203','])na')},'success':function(_0x637eb3){if(_0x2ffba2[0x0])$gameVariables['setValue'](_0x2ffba2[0x0],parseInt(_0x637eb3));},'error':function(_0x4b82ee){var _0x181a01={'nSsaO':_0x3968('0x204','IY^0')};alert(_0x181a01[_0x3968('0x205','B6b!')]);PlayOnline[_0x3968('0x206','pEq[')]=null;},'async':![]});}}};PlayOnline[_0x3968('0x207','qkk]')]=Game_System[_0x3968('0x208','^CBW')][_0x3968('0x209','XFHa')];Game_System[_0x3968('0x20a','B6b!')]['initialize']=function(){this[_0x3968('0x20b','U(Ap')]=![];this['_gBtns']=[];PlayOnline[_0x3968('0x20c','^CBW')][_0x3968('0x20d','ZR02')](this);};PlayOnline[_0x3968('0x20e','ac4V')]=Game_Temp['prototype']['setDestination'];Game_Temp[_0x3968('0x20f','ac4V')][_0x3968('0x210','qkk]')]=function(_0x16fc71,_0x39cfd1){if(PlayOnline[_0x3968('0x211','t1a6')]())return;PlayOnline['Game_Temp_setDestination'][_0x3968('0x212','DSo%')](this,_0x16fc71,_0x39cfd1);};PlayOnline[_0x3968('0x213','Go7X')]=Game_CharacterBase['prototype'][_0x3968('0x214','@tYj')];Game_CharacterBase[_0x3968('0x215','y3I5')][_0x3968('0x216','xh!r')]=function(){PlayOnline['Game_CharacterBase_initMembers']['call'](this);this['_name']='';this[_0x3968('0x217','ZqhS')]=0x0;};Game_CharacterBase['prototype'][_0x3968('0x218','])na')]=function(){return this[_0x3968('0x219','ZR02')];};Game_CharacterBase[_0x3968('0x21a','GRVS')][_0x3968('0x21b','!aAg')]=function(){return this[_0x3968('0x21c','CO[Y')];};Game_CharacterBase['prototype'][_0x3968('0x21d','^CBW')]=function(_0x30b9fa){this['_name']=_0x30b9fa;};Game_CharacterBase['prototype'][_0x3968('0x21e','@tYj')]=function(_0x294f08){this['_battlerName']=_0x294f08;};PlayOnline[_0x3968('0x21f','t1a6')]=Game_Player[_0x3968('0x220','Geuc')]['reserveTransfer'];Game_Player['prototype']['reserveTransfer']=function(_0x5ae0c8,_0xa6db40,_0x2ca58a,_0x5c90f8,_0x103f9d){Graphics[_0x3968('0x221','vI#!')]();PlayOnline[_0x3968('0x222','w*%c')][_0x3968('0x223','!aAg')](this,_0x5ae0c8,_0xa6db40,_0x2ca58a,_0x5c90f8,_0x103f9d);};Game_Follower['prototype'][_0x3968('0x224','JnKS')]=function(_0x5e0e2d){var _0x224d05={'nPCnN':function _0xecaaa3(_0x33c20a,_0x2aba3e){return _0x33c20a-_0x2aba3e;}};Game_Character['prototype']['initialize'][_0x3968('0x225','y3I5')](this);this[_0x3968('0x226','bii*')]=_0x5e0e2d;this['_index']=Math[_0x3968('0x227','Go7X')](_0x224d05[_0x3968('0x228','e20f')](this[_0x3968('0x229','ZqhS')],$gameParty[_0x3968('0x22a','xh!r')]()),0x0);this['setTransparent']($dataSystem[_0x3968('0x22b','o^84')]);this[_0x3968('0x22c','g4!!')](![]);};Game_Follower['prototype'][_0x3968('0x22d','dJbc')]=function(){var _0x2a639b={'rtJlQ':_0x3968('0x22e','w*%c'),'cirNG':function _0x214418(_0x46fb90,_0x204df5){return _0x46fb90==_0x204df5;}};var _0x377167=_0x2a639b[_0x3968('0x22f','6(^C')]['split']('|'),_0x455005=0x0;while(!![]){switch(_0x377167[_0x455005++]){case'0':if(_0x2a639b[_0x3968('0x230','56rC')](this[_0x3968('0x231','P%uC')],'系统'))this[_0x3968('0x232','AugK')]='';continue;case'1':this[_0x3968('0x233','mkla')](this[_0x3968('0x234','TC3d')]);continue;case'2':this[_0x3968('0x235','CO[Y')](this[_0x3968('0x236','B6b!')],this['_characterIndex']);continue;case'3':this[_0x3968('0x237','@bN4')]=this[_0x3968('0x238','ac4V')]()?this['actor']()[_0x3968('0x239','BN7V')]():'';continue;case'4':this[_0x3968('0x23a','N^7B')](this['_name']);continue;case'5':this[_0x3968('0x23b','57]T')]=this[_0x3968('0x23c','B6b!')]()?this[_0x3968('0x23d','qkk]')]()[_0x3968('0x23e','B6b!')]():'';continue;case'6':this[_0x3968('0x23f','92uf')]=this['isVisible']()?this['actor']()[_0x3968('0x240','bii*')]():0x0;continue;case'7':this[_0x3968('0x241','pEq[')]=this[_0x3968('0x242','@bN4')]()?this[_0x3968('0x243','N^7B')]()['characterIndex']():0x0;continue;}break;}};Game_Follower['prototype'][_0x3968('0x244','w*%c')]=function(){var _0x353dad={'xLLJJ':function _0x16166b(_0x2eba9f,_0xa80dca){return _0x2eba9f+_0xa80dca;}};if(this[_0x3968('0x245','6(^C')]<$gameParty[_0x3968('0x246','7g26')]())return $gameParty['battleMembers']()[this[_0x3968('0x247','7g26')]];else return $gameActors[_0x3968('0x248','CO[Y')](_0x353dad[_0x3968('0x249','^CBW')](PlayOnline[_0x3968('0x76','w*%c')][_0x3968('0x15c','oBAm')]-$gameParty[_0x3968('0x24a','56rC')][_0x3968('0x24b','e20f')],this[_0x3968('0x24c','DSo%')])+0x1);};Game_Follower[_0x3968('0x24d','Go7X')]['isPlayOnline']=function(){var _0x2408f3={'sOHyo':function _0x106c18(_0x3d0253,_0x2aa021){return _0x3d0253<_0x2aa021;},'kBWfd':function _0x29b1f2(_0x32d8ac,_0x4ddfdd){return _0x32d8ac+_0x4ddfdd;},'TpOxx':function _0x2c5a74(_0xde713c,_0x3cd548){return _0xde713c*_0x3cd548;}};if(_0x2408f3[_0x3968('0x24e','@tYj')](this['_memberIndex'],$gameParty['maxBattleMembers']()))return this[_0x3968('0x24f','XFHa')]()&&$gamePlayer[_0x3968('0x250','pEq[')]()['isVisible']();else return!!PlayOnline['GameInfo'][_0x2408f3[_0x3968('0x251','@tYj')](_0x2408f3[_0x3968('0x252','JnKS')](this['_index'],0xb),0x2)];};Game_Follower['prototype'][_0x3968('0x253','bii*')]=function(){var _0xbe2fd9={'upoPI':'0|3|6|5|7|2|4|1','OiZHI':function _0x21b9bf(_0x24e028,_0x56c75a){return _0x24e028+_0x56c75a;},'hUaTT':function _0x2b9a92(_0x237bc7,_0x408587){return _0x237bc7*_0x408587;},'LkUYE':function _0x433925(_0x194942,_0x232efa){return _0x194942*_0x232efa;},'lKiBF':function _0x2caace(_0x51f15f,_0x89a2bd){return _0x51f15f==_0x89a2bd;}};var _0x2b802d=_0xbe2fd9['upoPI'][_0x3968('0x254','AugK')]('|'),_0x259433=0x0;while(!![]){switch(_0x2b802d[_0x259433++]){case'0':this[_0x3968('0x255','U(Ap')]=PlayOnline[_0x3968('0x256','BN7V')][this[_0x3968('0x257','Geuc')]*0xb+0x2]?PlayOnline[_0x3968('0x16f','MhLY')][_0xbe2fd9[_0x3968('0x258','qkk]')](this[_0x3968('0x259','bii*')]*0xb,0x2)]:'';continue;case'1':this[_0x3968('0x25a','xh!r')](this[_0x3968('0x25b','U(Ap')],this['_characterIndex']);continue;case'2':this['setName'](this[_0x3968('0x25c','56rC')]);continue;case'3':this[_0x3968('0x25d','MhLY')]=PlayOnline['GameInfo'][_0xbe2fd9[_0x3968('0x25e','@tYj')](this[_0x3968('0x25f','7g26')],0xb)+0x5]?PlayOnline['GameInfo'][_0xbe2fd9[_0x3968('0x260','AugK')](this[_0x3968('0x261','#r7G')],0xb)+0x5]:0x0;continue;case'4':this[_0x3968('0x262','xh!r')](this['_battlerName']);continue;case'5':this[_0x3968('0x263','4WKX')]=PlayOnline[_0x3968('0x188','DSo%')][_0xbe2fd9['hUaTT'](this[_0x3968('0x264','56rC')],0xb)+0x4]?PlayOnline['GameInfo'][_0xbe2fd9['LkUYE'](this[_0x3968('0x265','@bN4')],0xb)+0x4]:0x0;continue;case'6':this['_characterName']=PlayOnline[_0x3968('0x266','XFHa')][_0xbe2fd9['LkUYE'](this['_index'],0xb)+0x3]?PlayOnline[_0x3968('0x10d','oBAm')][_0xbe2fd9[_0x3968('0x267','#r7G')](this['_index'],0xb)+0x3]:'';continue;case'7':if(_0xbe2fd9[_0x3968('0x268','7g26')](this['_name'],'系统'))this[_0x3968('0x269','dJbc')]='';continue;}break;}};Game_Follower[_0x3968('0x26a','Wl!b')][_0x3968('0x26b','xh!r')]=function(){var _0x59bdc5={'hWcZW':function _0x5c9d6f(_0x7511e8,_0x318844){return _0x7511e8>=_0x318844;},'BJoHJ':function _0x35a862(_0x44d04d,_0x277f51){return _0x44d04d/_0x277f51;}};var _0x461863=_0x3968('0x26c','])na')[_0x3968('0x26d','4WKX')]('|'),_0x2a8b42=0x0;while(!![]){switch(_0x461863[_0x2a8b42++]){case'0':this[_0x3968('0x26e','w*%c')]($gamePlayer[_0x3968('0x26f','4WKX')]());continue;case'1':this['setBlendMode']($gamePlayer[_0x3968('0x270','DSo%')]());continue;case'2':Game_Character['prototype'][_0x3968('0x271','Go7X')]['call'](this);continue;case'3':if(PlayOnline[_0x3968('0x272','56rC')][this[_0x3968('0x273','@tYj')]]&&_0x59bdc5[_0x3968('0x274','ohA4')](this[_0x3968('0x275','w*%c')],$gameParty[_0x3968('0x276','56rC')]())){this[_0x3968('0x277','B6b!')]();PlayOnline['UpdatePlayer'][this[_0x3968('0x278','])na')]]=![];}continue;case'4':if(this['_memberIndex']<$gameParty[_0x3968('0x279','X)dO')]())this[_0x3968('0x27a','ohA4')]($gamePlayer['opacity']());continue;case'5':this[_0x3968('0x27b','DSo%')](_0x59bdc5['BJoHJ']($gamePlayer[_0x3968('0x27c','6(^C')](),PlayOnline[_0x3968('0x3f','])na')][_0x3968('0x27d','e20f')]));continue;case'6':this[_0x3968('0x27e','b8bb')]($gamePlayer['isDirectionFixed']());continue;case'7':this[_0x3968('0x27f','oBAm')]($gamePlayer[_0x3968('0x280','xh!r')]());continue;case'8':this[_0x3968('0x281','57]T')]($gamePlayer[_0x3968('0x282','y3I5')]());continue;}break;}};Game_Follower[_0x3968('0x20f','ac4V')][_0x3968('0x283','ZqhS')]=function(_0x359d7c){var _0x295012={'tTxZg':function _0x15dbb9(_0x4c1052,_0x238e74){return _0x4c1052<=_0x238e74;},'yNSKP':function _0x1a195e(_0x1e164a,_0x23e8da){return _0x1e164a>=_0x23e8da;},'VNYOp':function _0x40329e(_0xa1bc5b,_0x3069f4){return _0xa1bc5b*_0x3069f4;},'UEKJM':function _0x5cca82(_0x2cdcf1,_0x2b5932){return _0x2cdcf1+_0x2b5932;},'nqsPm':function _0x1d7c67(_0x3e1827,_0xd9b914){return _0x3e1827>_0xd9b914;},'ZqqJg':function _0x1f3e06(_0x39f7b6,_0x133c22){return _0x39f7b6<_0x133c22;},'JFluG':function _0x3b4244(_0x3baefb,_0xe1d27f){return _0x3baefb>_0xe1d27f;},'NfJRr':function _0x587392(_0x5b76bd,_0x99b999){return _0x5b76bd!==_0x99b999;},'cefjG':function _0x2d2e78(_0x5c871a,_0x23aea9){return _0x5c871a!==_0x23aea9;},'wexBK':'vyb'};var _0xbdbf6a=this[_0x3968('0x284','CO[Y')](_0x359d7c['x']);var _0x186a4f=this['deltaYFrom'](_0x359d7c['y']);if(PlayOnline[_0x3968('0x285','vI#!')]['FollowType']){if(_0xbdbf6a>=-0x2&&_0x295012[_0x3968('0x286','U(Ap')](_0xbdbf6a,0x2)&&_0x295012[_0x3968('0x287','e20f')](_0x186a4f,-0x2)&&_0x186a4f<=0x2){var _0x52215a=Math[_0x3968('0x288','qkk]')]()*PlayOnline['Param'][_0x3968('0x289','w*%c')];if(_0x52215a>PlayOnline[_0x3968('0x1e9','g4!!')][_0x3968('0x28a','Go7X')]-0x1)this[_0x3968('0x28b','GRVS')](_0x295012[_0x3968('0x28c','Geuc')](Math[_0x3968('0x28d','CO[Y')](_0x295012['UEKJM'](Math[_0x3968('0x28e','JnKS')]()*0x4,0x1)),0x2));}else if(_0xbdbf6a<-PlayOnline['Param'][_0x3968('0x28f','ac4V')]||_0x295012['nqsPm'](_0xbdbf6a,PlayOnline[_0x3968('0x33','Go7X')][_0x3968('0x290','bii*')])){this[_0x3968('0x291','!aAg')]($gamePlayer['x'],$gamePlayer['y']);}else if(_0x186a4f<-PlayOnline[_0x3968('0x1e0','Wl!b')]['Faraway']||_0x186a4f>PlayOnline['Param'][_0x3968('0x292','o^84')]){this[_0x3968('0x293','6(^C')]($gamePlayer['x'],$gamePlayer['y']);}else if(_0x295012[_0x3968('0x294','^CBW')](_0xbdbf6a,-0x2)||_0xbdbf6a>0x2){this[_0x3968('0x295','P%uC')](_0x295012['JFluG'](_0xbdbf6a,0x0)?0x4:0x6);}else if(_0x295012[_0x3968('0x296','JnKS')](_0x186a4f,-0x2)||_0x295012[_0x3968('0x297','])na')](_0x186a4f,0x2)){if(_0x3968('0x298','JnKS')!==_0x3968('0x299','IY^0')){this['moveStraight'](_0x186a4f>0x0?0x8:0x2);}else{return 0x1e;}}}else{if(_0x295012[_0x3968('0x29a','g4!!')](_0xbdbf6a,0x0)&&_0x186a4f!==0x0){this[_0x3968('0x29b','pEq[')](_0x295012['JFluG'](_0xbdbf6a,0x0)?0x4:0x6,_0x295012['JFluG'](_0x186a4f,0x0)?0x8:0x2);}else if(_0x295012['cefjG'](_0xbdbf6a,0x0)){if('vyb'===_0x295012[_0x3968('0x29c','bii*')]){this[_0x3968('0x29d','qkk]')](_0x295012[_0x3968('0x29e','ZqhS')](_0xbdbf6a,0x0)?0x4:0x6);}else{this['removeChild'](this[_0x3968('0x29f','bii*')][id]);}}else if(_0x186a4f!==0x0){this['moveStraight'](_0x186a4f>0x0?0x8:0x2);}}this[_0x3968('0x2a0','IY^0')]($gamePlayer[_0x3968('0x2a1','IY^0')]());};Game_Follower[_0x3968('0x2a2','IY^0')][_0x3968('0x2a3','4WKX')]=function(_0x36bfae){var _0x152a89={'vtVou':function _0x23645c(_0x4c2c90,_0x13c265){return _0x4c2c90*_0x13c265;},'GiKxN':function _0x43b425(_0x4bca69,_0x1c1975){return _0x4bca69!==_0x1c1975;},'gpcJd':function _0x18ac48(_0x1f399d,_0x1ffff1){return _0x1f399d>_0x1ffff1;},'ESHkM':function _0x4f8060(_0xa7ef72,_0x30fb69){return _0xa7ef72===_0x30fb69;},'oaBop':_0x3968('0x2a4','o^84'),'GJBHI':function _0x2410f4(_0x3fa744,_0xbcc193){return _0x3fa744!=_0xbcc193;},'YCIue':function _0x11cd60(_0x1fd85e,_0x35e0a9){return _0x1fd85e+_0x35e0a9;},'SFwna':function _0x48a7d9(_0x50427f,_0x29eded){return _0x50427f*_0x29eded;}};var _0xfaf407=this[_0x3968('0x2a5','Geuc')](PlayOnline[_0x3968('0x2a6','B6uD')][_0x152a89[_0x3968('0x2a7','N^7B')](_0x36bfae,0xb)+0x6]);var _0x19e944=this[_0x3968('0x2a8','BN7V')](PlayOnline[_0x3968('0x266','XFHa')][_0x152a89['vtVou'](_0x36bfae,0xb)+0x7]);if(_0xfaf407!==0x0&&_0x152a89['GiKxN'](_0x19e944,0x0)){this['moveDiagonally'](_0x152a89['gpcJd'](_0xfaf407,0x0)?0x4:0x6,_0x19e944>0x0?0x8:0x2);}else if(_0x152a89[_0x3968('0x2a9','Wl!b')](_0xfaf407,0x0)){if(_0x152a89[_0x3968('0x2aa','bii*')](_0x152a89[_0x3968('0x2ab','92uf')],_0x152a89[_0x3968('0x2ac','qkk]')])){this[_0x3968('0x2ad','xh!r')](_0xfaf407>0x0?0x4:0x6);}else{PlayOnline['UserName']=null;}}else if(_0x152a89[_0x3968('0x2ae','AugK')](_0x19e944,0x0)){this[_0x3968('0x2af','CO[Y')](_0x152a89[_0x3968('0x2b0','qkk]')](_0x19e944,0x0)?0x8:0x2);}this[_0x3968('0x2b1','bii*')](!![]);this[_0x3968('0x2b2','pEq[')](0x6);if(_0x152a89[_0x3968('0x2b3','t1a6')](PlayOnline[_0x3968('0x10f','4WKX')][_0x152a89['YCIue'](_0x152a89['SFwna'](_0x36bfae,0xb),0x5)],$gameMap[_0x3968('0x2b4','CO[Y')]())){this[_0x3968('0x2b5','4WKX')](0x0);this[_0x3968('0x2b6','ZqhS')]('');}else{this[_0x3968('0x2b7','Go7X')]($gamePlayer[_0x3968('0x2b8','y3I5')]());}};Game_Followers[_0x3968('0x21a','GRVS')][_0x3968('0x2b9','4WKX')]=function(){var _0x3ab8f7={'FSxXS':function _0x58501a(_0x32212e,_0x4d28a4){return _0x32212e+_0x4d28a4;},'phpey':function _0x2b4f42(_0x1d04c7,_0x3fea8e){return _0x1d04c7!==_0x3fea8e;},'wAWKc':function _0x2e0334(_0xb2c1b0,_0x5d8485){return _0xb2c1b0*_0x5d8485;},'UkcRi':function _0xa5a61c(_0x13ae5c,_0x58a827){return _0x13ae5c+_0x58a827;},'KsyPB':function _0x3eff3c(_0x1218b7,_0x4481b4){return _0x1218b7*_0x4481b4;},'GWiyk':function _0x1e58f5(_0x2a440d,_0x3fdc23){return _0x2a440d+_0x3fdc23;},'GGEZC':function _0x21e7f0(_0x13d89e,_0x801c94){return _0x13d89e*_0x801c94;},'wdbVv':function _0x547237(_0x57e0b5,_0x2f46fb){return _0x57e0b5+_0x2f46fb;}};this[_0x3968('0x2ba','#r7G')]=$dataSystem['optFollowers'];this['_gathering']=![];this[_0x3968('0x2bb','!aAg')]=[];for(var _0x486fe4=0x1;_0x486fe4<_0x3ab8f7[_0x3968('0x2bc','B6uD')]($gameParty[_0x3968('0x2bd','B6b!')](),PlayOnline['Param'][_0x3968('0xf2','X)dO')]);_0x486fe4++){if(_0x3ab8f7[_0x3968('0x2be','@tYj')](_0x3968('0x2bf','t1a6'),_0x3968('0x2c0','t1a6'))){this[_0x3968('0x2c1','U(Ap')][_0x3968('0x1c8','ohA4')](new Game_Follower(_0x486fe4));}else{var _0x2ea228='2|1|6|4|7|5|3|0'[_0x3968('0x2c2','BN7V')]('|'),_0xca5a26=0x0;while(!![]){switch(_0x2ea228[_0xca5a26++]){case'0':this['setImage'](this[_0x3968('0x2c3','e20f')],this[_0x3968('0x2c4','y3I5')]);continue;case'1':this[_0x3968('0x2c5','t1a6')]=PlayOnline[_0x3968('0x256','BN7V')][_0x3ab8f7['wAWKc'](this['_index'],0xb)+0x5]?PlayOnline['GameInfo'][_0x3ab8f7[_0x3968('0x2c6','U(Ap')](_0x3ab8f7['KsyPB'](this['_index'],0xb),0x5)]:0x0;continue;case'2':this['_name']=PlayOnline['GameInfo'][this[_0x3968('0x273','@tYj')]*0xb+0x2]?PlayOnline[_0x3968('0x2c7','U(Ap')][_0x3ab8f7[_0x3968('0x2c8','TC3d')](_0x3ab8f7[_0x3968('0x2c9','Go7X')](this[_0x3968('0x265','@bN4')],0xb),0x2)]:'';continue;case'3':this['setBattlerName'](this[_0x3968('0x2ca','Go7X')]);continue;case'4':this['_characterIndex']=PlayOnline[_0x3968('0x2cb','TC3d')][_0x3ab8f7[_0x3968('0x2cc','B6b!')](this[_0x3968('0x2cd','GRVS')]*0xb,0x4)]?PlayOnline[_0x3968('0x2ce','6(^C')][_0x3ab8f7['GWiyk'](_0x3ab8f7['GGEZC'](this['_index'],0xb),0x4)]:0x0;continue;case'5':this[_0x3968('0x2cf','t1a6')](this[_0x3968('0x2d0','GRVS')]);continue;case'6':this[_0x3968('0x2d1','XFHa')]=PlayOnline['GameInfo'][_0x3ab8f7[_0x3968('0x2d2','bii*')](this[_0x3968('0x2d3','4WKX')]*0xb,0x3)]?PlayOnline[_0x3968('0x2a6','B6uD')][this['_index']*0xb+0x3]:'';continue;case'7':if(this[_0x3968('0x2d4','!aAg')]=='系统')this[_0x3968('0x2d5','DSo%')]='';continue;}break;}}}};Game_Followers[_0x3968('0x2d6','b8bb')]['update']=function(){if(this['areGathering']()){if(this['areGathered']()){this['_gathering']=![];}}this['forEach'](function(_0x1c7b4c){var _0x36d87d={'lnlla':function _0x2a7f5f(_0x53f78e,_0x16964c){return _0x53f78e!==_0x16964c;},'NTLfU':_0x3968('0x2d7','@bN4'),'cDXfu':_0x3968('0x2d8','JnKS')};if(_0x36d87d[_0x3968('0x2d9','Go7X')](_0x36d87d['NTLfU'],_0x3968('0x2da','e20f'))){_0x1c7b4c['update']();}else{$[_0x3968('0x2db','56rC')]({'type':_0x36d87d[_0x3968('0x2dc','ZR02')],'url':PlayOnline[_0x3968('0x2dd','CO[Y')],'data':{'user':PlayOnline[_0x3968('0xfc','o^84')],'game':PlayOnline['Param'][_0x3968('0x2de','7g26')],'cmd':'askvalue'},'success':function(_0x32ab96){var _0x1be9ac={'vAcgL':function _0x4da12f(_0x4e0533,_0x529969){return _0x4e0533(_0x529969);}};if(args[0x0])$gameVariables['setValue'](args[0x0],_0x1be9ac[_0x3968('0x2df','IY^0')](parseInt,_0x32ab96));},'error':function(_0x4b0779){var _0x25fd6c={'gdDQO':function _0x1c5053(_0x34a09a,_0x24956c){return _0x34a09a(_0x24956c);}};_0x25fd6c[_0x3968('0x2e0','JnKS')](alert,_0x3968('0x2e1','])na'));PlayOnline[_0x3968('0x2e2','g4!!')]=null;},'async':![]});}},this);if(!this['areMoving']())this[_0x3968('0x2e3','o^84')]();};Game_Followers['prototype'][_0x3968('0x2e4','t1a6')]=function(){var _0xffd4e5={'vHNfl':function _0x24358f(_0x170355,_0x25035d){return _0x170355<_0x25035d;},'gGNrg':function _0x1347df(_0x55c7b3,_0x1a8d9c){return _0x55c7b3-_0x1a8d9c;},'StlNq':function _0x2ba40e(_0xa172d,_0x192dc2){return _0xa172d===_0x192dc2;},'dCLBB':function _0x31f225(_0x3b9b48,_0x4f2b9c){return _0x3b9b48>_0x4f2b9c;},'iPvJl':function _0x572b83(_0x28d542,_0x2a93e3){return _0x28d542-_0x2a93e3;},'rGPMb':function _0x3ec758(_0x16648f,_0x34dd22){return _0x16648f-_0x34dd22;},'ronti':function _0x1678a3(_0x1e3e12,_0x292f33){return _0x1e3e12+_0x292f33;}};for(var _0x1e82bb=this[_0x3968('0x2e5','X)dO')][_0x3968('0x2e6','w*%c')]-0x1;_0x1e82bb>=0x0;--_0x1e82bb){if(_0xffd4e5['vHNfl'](_0x1e82bb,_0xffd4e5[_0x3968('0x2e7','P%uC')]($gameParty[_0x3968('0x2e8','vI#!')](),0x1))){if(PlayOnline[_0x3968('0x4c','ac4V')][_0x3968('0x2e9','BN7V')]){if(_0xffd4e5['StlNq'](_0x3968('0x2ea','GRVS'),'KkN')){if(!!this[_0x3968('0x2eb','@tYj')][_0x1e82bb][_0x3968('0x2ec','mkla')]())this[_0x3968('0x2ed','o^84')][_0x1e82bb]['chaseCharacter']($gamePlayer);}else{this['refreshActor']();PlayOnline['UpdatePlayer'][this['_index']]=![];}}else{if(!!this[_0x3968('0x2ee','MhLY')][_0x1e82bb][_0x3968('0x2ef','N^7B')]()){var _0x26312b=_0xffd4e5[_0x3968('0x2f0','X)dO')](_0x1e82bb,0x0)?this[_0x3968('0x2f1','XFHa')][_0xffd4e5[_0x3968('0x2f2','JnKS')](_0x1e82bb,0x1)]:$gamePlayer;this[_0x3968('0x2f3','DSo%')][_0x1e82bb][_0x3968('0x283','ZqhS')](_0x26312b);}}}else{if(!!this[_0x3968('0x2f4','B6uD')][_0x1e82bb]['isPlayOnline']())this[_0x3968('0x2f5','dJbc')][_0x1e82bb][_0x3968('0x2f6','ZR02')](_0xffd4e5[_0x3968('0x2f7','^CBW')](_0xffd4e5[_0x3968('0x2f8','B6uD')](_0x1e82bb,0x1),$gameParty[_0x3968('0x2f9','Wl!b')]()));}}};Game_Followers['prototype'][_0x3968('0x2fa','GRVS')]=function(){var _0x30339f={'OmBvZ':'zbn','yYlDK':function _0x4b2562(_0x491c5b,_0x4a7a42){return _0x491c5b<_0x4a7a42;},'uLqTh':function _0x4c8b07(_0x963223,_0x153552){return _0x963223-_0x153552;},'WywuJ':function _0x3d3fcd(_0x596e98,_0x326d00){return _0x596e98!==_0x326d00;},'ajaWp':_0x3968('0x2fb','6(^C'),'XTNfd':function _0xffaf5f(_0x29b412,_0x571546){return _0x29b412>=_0x571546;},'mFjoM':function _0x8b5c72(_0x5ad118,_0x464f0d){return _0x5ad118>=_0x464f0d;},'vXhGP':function _0x5afa14(_0x2fe729,_0x3e5267){return _0x2fe729+_0x3e5267;}};if($gamePlayer['isJumping']()){if(_0x3968('0x2fc','y3I5')===_0x30339f[_0x3968('0x2fd','oBAm')]){for(var _0x5b5869=0x0;_0x5b5869<this[_0x3968('0x2e5','X)dO')][_0x3968('0x2fe','oBAm')];++_0x5b5869){if(_0x30339f[_0x3968('0x2ff','ac4V')](_0x5b5869,_0x30339f['uLqTh']($gameParty[_0x3968('0x300','!aAg')]['length'],0x1))){if(_0x30339f[_0x3968('0x301','MhLY')](_0x3968('0x302','ac4V'),_0x30339f[_0x3968('0x303','mkla')])){var _0x47e817=this[_0x3968('0x304','CO[Y')][_0x5b5869];var _0x3a0125=$gamePlayer[_0x3968('0x305','])na')](_0x47e817['x']);var _0x30cacf=$gamePlayer[_0x3968('0x306','TC3d')](_0x47e817['y']);_0x47e817[_0x3968('0x307','7g26')](_0x3a0125,_0x30cacf);}else{var _0x386500=TouchInput['x'];var _0x1dba5f=TouchInput['y'];if(_0x30339f['XTNfd'](_0x386500,wx)&&_0x30339f[_0x3968('0x308','@bN4')](_0x1dba5f,_0x30339f['uLqTh'](wy,wh))&&_0x386500<wx+ww&&_0x30339f['yYlDK'](_0x1dba5f,wy)){this[_0x3968('0x309','ac4V')]['activate']();this[_0x3968('0x30a','!aAg')][_0x3968('0x30b','BN7V')]();Graphics[_0x3968('0x30c','B6b!')][_0x3968('0x30d','XFHa')]();}else if(_0x30339f[_0x3968('0x30e','!aAg')](_0x386500,wx)&&_0x1dba5f>=wy&&_0x30339f['yYlDK'](_0x386500,wx+ww)&&_0x1dba5f<_0x30339f[_0x3968('0x30f','4WKX')](wy,wh)){this['_loginWindow'][_0x3968('0x310','e20f')]();this[_0x3968('0x311','XFHa')][_0x3968('0x312','#r7G')]();Graphics['_input2'][_0x3968('0x313','DSo%')]();}else{this[_0x3968('0x314','6(^C')][_0x3968('0x315','X)dO')]();this[_0x3968('0x316','qkk]')][_0x3968('0x317','AugK')]();}}}}}else{Graphics[_0x3968('0x318','BN7V')][_0x3968('0x319','IY^0')]=data[_0x3968('0x31a','B6uD')];Graphics[_0x3968('0x31b','Geuc')][_0x3968('0x31c','Wl!b')]=data[_0x3968('0x31d','6(^C')];}}};Game_Followers[_0x3968('0x31e','w*%c')]['synchronize']=function(_0x4e1b8a,_0x3564c7,_0x5d51cc){var _0x18dcfe={'FjGpa':function _0x130621(_0x56e2d5,_0x5b11e6){return _0x56e2d5<_0x5b11e6;}};this[_0x3968('0x31f','#r7G')](function(_0x115ae8){if(_0x18dcfe['FjGpa'](_0x115ae8['_memberIndex'],$gameParty['maxBattleMembers']())){_0x115ae8[_0x3968('0x320','Geuc')](_0x4e1b8a,_0x3564c7);_0x115ae8[_0x3968('0x321','dJbc')](_0x5d51cc);}},this);};Game_Followers['prototype'][_0x3968('0x322','bii*')]=function(){return this[_0x3968('0x2ee','MhLY')]['filter'](function(_0x5e3f64){var _0x44307e={'XOYsn':function _0x2388f0(_0x5f3c6f,_0x3a659e){return _0x5f3c6f!==_0x3a659e;},'stpcS':_0x3968('0x323','vI#!'),'QVCkD':function _0x1b1c6a(_0x15c178,_0x198dea){return _0x15c178<_0x198dea;}};if(_0x44307e[_0x3968('0x324','X)dO')]('AMu',_0x44307e[_0x3968('0x325','CO[Y')])){return _0x5e3f64[_0x3968('0x326','6(^C')]()&&_0x44307e[_0x3968('0x327','Geuc')](_0x5e3f64[_0x3968('0x328','e20f')],$gameParty[_0x3968('0x329','t1a6')]());}else{var _0x344faa=_0x3968('0x32a','bii*')['split']('|'),_0x4aac78=0x0;while(!![]){switch(_0x344faa[_0x4aac78++]){case'0':SoundManager['preloadImportantSounds']();continue;case'1':this[_0x3968('0x32b','pEq[')]();continue;case'2':Scene_Base['prototype'][_0x3968('0x32c','ohA4')][_0x3968('0x225','y3I5')](this);continue;case'3':DataManager[_0x3968('0x32d','mkla')]();continue;case'4':SceneManager['push'](Scene_PlayOnline);continue;case'5':this[_0x3968('0x32e','AugK')]();continue;}break;}}},this);};Game_Followers['prototype'][_0x3968('0x32f','B6b!')]=function(){return this['visiblePartyFollowers']()[_0x3968('0x330','N^7B')](function(_0x4619d8){return!_0x4619d8[_0x3968('0x331','XFHa')]()&&_0x4619d8[_0x3968('0x332','o^84')]($gamePlayer['x'],$gamePlayer['y']);},this);};Game_Followers[_0x3968('0x21a','GRVS')][_0x3968('0x333','6(^C')]=function(_0x3178e,_0x43cbd9){return this['visiblePartyFollowers']()['some'](function(_0x268bee){return _0x268bee[_0x3968('0x334','#r7G')](_0x3178e,_0x43cbd9);},this);};PlayOnline[_0x3968('0x335','DSo%')]=Scene_Base['prototype'][_0x3968('0x336','qkk]')];Scene_Base[_0x3968('0x220','Geuc')]['initialize']=function(){Input[_0x3968('0x337','vI#!')]();this[_0x3968('0x338','GRVS')]=[];PlayOnline['Scene_Base_initialize'][_0x3968('0x339','56rC')](this);this[_0x3968('0x33a','CO[Y')]=PlayOnline[_0x3968('0x33b','92uf')]['bind'](this);};PlayOnline['Scene_Base_start']=Scene_Base[_0x3968('0x33c','AugK')][_0x3968('0x33d','b8bb')];Scene_Base['prototype'][_0x3968('0x33e','ZqhS')]=function(){Input[_0x3968('0x33f','#r7G')]();PlayOnline['Scene_Base_start'][_0x3968('0x340','CO[Y')](this);};Scene_Base['prototype'][_0x3968('0x341','92uf')]=function(_0x5b5907){this[_0x3968('0x342','ac4V')](this[_0x3968('0x343','N^7B')][_0x5b5907]);};Scene_Base[_0x3968('0x31e','w*%c')][_0x3968('0x344','xh!r')]=function(_0x3c7e5e){var _0x2017bf={'pQstI':function _0x50786(_0x450ba7,_0x2d8af){return _0x450ba7(_0x2d8af);}};_0x2017bf[_0x3968('0x345','Geuc')](eval,_0x3c7e5e);};PlayOnline[_0x3968('0x346','XFHa')]=Scene_Base[_0x3968('0x347','DSo%')][_0x3968('0x348','92uf')];Scene_Base[_0x3968('0x21a','GRVS')][_0x3968('0x349','CO[Y')]=function(){var _0x2efe70={'DlDTC':function _0xe5836d(_0x558eb9,_0x347c79){return _0x558eb9===_0x347c79;},'hvDYO':_0x3968('0x34a','w*%c'),'naHje':_0x3968('0x34b','g4!!'),'yvbgQ':'2|0|1|5|3|4|6','LaBBG':function _0x1f3fa2(_0x4c11d4,_0x31fb18){return _0x4c11d4*_0x31fb18;}};PlayOnline[_0x3968('0x34c','Geuc')]['call'](this);if(PlayOnline[_0x3968('0x34d','^CBW')][_0x3968('0x34e','mkla')])document['addEventListener'](_0x3968('0x34f','ZR02'),function(){if(document['hidden']&&Utils[_0x3968('0x350','g4!!')]()){if(_0x2efe70[_0x3968('0x351','pEq[')](_0x2efe70[_0x3968('0x352','dJbc')],_0x2efe70[_0x3968('0x353','pEq[')])){var _0x2ca375=_0x2efe70[_0x3968('0x354','g4!!')]['split']('|'),_0x21c53e=0x0;while(!![]){switch(_0x2ca375[_0x21c53e++]){case'0':if(args[0x0])$gameVariables[_0x3968('0x355','])na')](args[0x0],_0x2cc2e3[_0x3968('0x356','ohA4')]());continue;case'1':if(args[0x1])$gameVariables['setValue'](args[0x1],_0x2cc2e3[_0x3968('0x357','o^84')]()+0x1);continue;case'2':var _0x2cc2e3=new Date(_0x2efe70['LaBBG'](msg,0x3e8));continue;case'3':if(args[0x3])$gameVariables[_0x3968('0x358','DSo%')](args[0x3],_0x2cc2e3[_0x3968('0x359','57]T')]());continue;case'4':if(args[0x4])$gameVariables[_0x3968('0x1f3','y3I5')](args[0x4],_0x2cc2e3[_0x3968('0x35a','vI#!')]());continue;case'5':if(args[0x2])$gameVariables[_0x3968('0x35b','Go7X')](args[0x2],_0x2cc2e3[_0x3968('0x35c','oBAm')]());continue;case'6':if(args[0x5])$gameVariables[_0x3968('0x35d','6(^C')](args[0x5],_0x2cc2e3[_0x3968('0x35e','vI#!')]());continue;}break;}}else{PlayOnline[_0x3968('0x35f','Wl!b')]=null;PlayOnline['UserName']=null;SceneManager['push'](Scene_PlayOnline);}}});};Scene_Boot[_0x3968('0x360','4WKX')][_0x3968('0x361','JnKS')]=function(){Scene_Base['prototype'][_0x3968('0x362','o^84')]['call'](this);SoundManager[_0x3968('0x363','o^84')]();this[_0x3968('0x32e','AugK')]();DataManager[_0x3968('0x364','xh!r')]();SceneManager['push'](Scene_PlayOnline);this['updateDocumentTitle']();};PlayOnline[_0x3968('0x365','P%uC')]=Scene_Boot[_0x3968('0x366','t1a6')];Scene_Boot[_0x3968('0x367','bii*')]=function(){var _0x2a6254={'yIDIc':_0x3968('0x368','56rC'),'mvnjR':_0x3968('0x369','Go7X'),'SskRD':function _0x4234b4(_0x1f5dda,_0x443b0e){return _0x1f5dda<_0x443b0e;},'tbkpR':'BtnSize'};var _0x5ae1b1=_0x2a6254[_0x3968('0x36a','e20f')][_0x3968('0x26d','4WKX')]('|'),_0x78877e=0x0;while(!![]){switch(_0x5ae1b1[_0x78877e++]){case'0':ImageManager['loadPicture'](_0x2a6254['mvnjR']);continue;case'1':ImageManager[_0x3968('0x36b','CO[Y')](_0x3968('0x36c','bii*'));continue;case'2':for(var _0x5f0018=0x0;_0x2a6254['SskRD'](_0x5f0018,PlayOnline['Param'][_0x3968('0x36d','6(^C')][_0x3968('0x36e','X)dO')]);_0x5f0018++){if(_0x3968('0x36f','56rC')!==_0x3968('0x370','bii*')){ImageManager[_0x3968('0x371','ZqhS')](PlayOnline['Param'][_0x3968('0x372','X)dO')][_0x5f0018]);}else{this[_0x3968('0x373','^CBW')]();}}continue;case'3':PlayOnline[_0x3968('0x365','P%uC')][_0x3968('0x374','mkla')](this);continue;case'4':ImageManager[_0x3968('0x375','GRVS')](_0x2a6254[_0x3968('0x376','AugK')]);continue;}break;}};PlayOnline[_0x3968('0x377','Go7X')]=Scene_Map[_0x3968('0x378','qkk]')][_0x3968('0x379','6(^C')];Scene_Map['prototype'][_0x3968('0x37a','qkk]')]=function(){PlayOnline['Scene_Map_createSpriteset']['call'](this);this[_0x3968('0x37b','dJbc')]=new Window_TalkList();this['addChild'](this['_talkwindow']);this[_0x3968('0x37c','oBAm')]=PlayOnline[_0x3968('0x37d','57]T')];};PlayOnline[_0x3968('0x37e','Go7X')]=Scene_Map[_0x3968('0x208','^CBW')]['update'];Scene_Map['prototype'][_0x3968('0x37f','o^84')]=function(){var _0x21c81c={'bNlcm':_0x3968('0x380','y3I5'),'fBYCF':function _0x2843b4(_0x1111a5,_0x3a8934){return _0x1111a5(_0x3a8934);},'uCHbe':_0x3968('0x381','g4!!'),'TJGeI':_0x3968('0x382','Go7X')};PlayOnline['Scene_Map_update'][_0x3968('0x383','g4!!')](this);if(PlayOnline[_0x3968('0x384','U(Ap')]==null){if(_0x3968('0x385','t1a6')!==_0x21c81c[_0x3968('0x386','^CBW')]){PlayOnline[_0x3968('0x387','dJbc')]=null;SceneManager['push'](Scene_PlayOnline);}else{PlayOnline[_0x3968('0x388','e20f')]=null;_0x21c81c[_0x3968('0x389','ZqhS')](alert,_0x21c81c['uCHbe']);}}if(this[_0x3968('0x38a','92uf')]!=PlayOnline['Size']){if(_0x21c81c[_0x3968('0x38b','BN7V')]!==_0x21c81c['TJGeI']){Game_Character[_0x3968('0x360','4WKX')]['initialize'][_0x3968('0x38c','oBAm')](this);this[_0x3968('0x226','bii*')]=memberIndex;this[_0x3968('0x38d','57]T')]=Math['max'](this[_0x3968('0x38e','Geuc')]-$gameParty[_0x3968('0x38f','qkk]')](),0x0);this[_0x3968('0x390','^CBW')]($dataSystem[_0x3968('0x391','Go7X')]);this[_0x3968('0x392','pEq[')](![]);}else{this[_0x3968('0x393','Wl!b')](this['_talkwindow']);this[_0x3968('0x394','57]T')]=new Window_TalkList();this['addChild'](this[_0x3968('0x395','^CBW')]);this[_0x3968('0x396','DSo%')]=PlayOnline['Size'];}}};PlayOnline[_0x3968('0x397','AugK')]=Scene_Map['prototype'][_0x3968('0x398','y3I5')];Scene_Map[_0x3968('0x399','B6uD')]['start']=function(){this[_0x3968('0x39a','ac4V')]();PlayOnline[_0x3968('0x39b','X)dO')]={};PlayOnline['Scene_Map_start'][_0x3968('0x39c','ohA4')](this);};Scene_Map[_0x3968('0x39d','6(^C')]['createGButtons']=function(){var _0x4564e2={'KHBCA':function _0x204f5e(_0x1641e5,_0x5377c4){return _0x1641e5===_0x5377c4;},'tHwty':_0x3968('0x39e','IY^0')};for(var _0x3768ba=0x0;_0x3768ba<$gameSystem[_0x3968('0x39f','ZqhS')]['length'];_0x3768ba++){if(_0x4564e2[_0x3968('0x3a0','P%uC')](_0x3968('0x3a1','ac4V'),_0x4564e2['tHwty'])){follower['locate'](x,y);follower[_0x3968('0x3a2','BN7V')](d);}else{if(PlayOnline['canHasButton'](_0x3768ba)){this['createGBtn']($gameSystem[_0x3968('0x3a3','Geuc')][_0x3768ba]);}}}};Scene_PlayOnline['prototype']=Object[_0x3968('0x3a4','X)dO')](Scene_Base[_0x3968('0x3a5','ZR02')]);Scene_PlayOnline['prototype'][_0x3968('0x3a6','56rC')]=Scene_PlayOnline;Scene_PlayOnline['prototype'][_0x3968('0x2b9','4WKX')]=function(){Scene_Base[_0x3968('0x3a5','ZR02')]['initialize'][_0x3968('0x3a7','GRVS')](this);this[_0x3968('0x3a8','pEq[')]();};Scene_PlayOnline[_0x3968('0x3a9','MhLY')][_0x3968('0x3aa','ZqhS')]=function(){var _0x56a2be={'QaDDe':'text','UTvuC':function _0x49c702(_0x3e6296,_0x1d9976){return _0x3e6296-_0x1d9976;},'bBVoj':function _0x258ada(_0x891b2c,_0x1a600e){return _0x891b2c+_0x1a600e;},'pYEmm':function _0x31b753(_0x21e998,_0x5392ec){return _0x21e998-_0x5392ec;}};var _0xe49b4f='7|3|5|0|1|4|8|10|9|2|6'['split']('|'),_0x1ccc64=0x0;while(!![]){switch(_0xe49b4f[_0x1ccc64++]){case'0':this[_0x3968('0x3ab','X)dO')]();continue;case'1':Graphics['_addInput'](_0x56a2be[_0x3968('0x3ac','g4!!')],_0x56a2be[_0x3968('0x3ad','ZqhS')](Graphics['width']/0x2,0x14),_0x56a2be[_0x3968('0x3ae','!aAg')](PlayOnline[_0x3968('0x5f','BN7V')][_0x3968('0x3af','b8bb')],0x50),0xc8,0x19,0x18);continue;case'2':Graphics['_input2'][_0x3968('0x3b0','B6uD')]='';continue;case'3':this[_0x3968('0x3b1','U(Ap')]();continue;case'4':Graphics[_0x3968('0x3b2','CO[Y')][_0x3968('0x3b3','dJbc')]=0xf;continue;case'5':this[_0x3968('0x3b4','BN7V')]();continue;case'6':if(Utils[_0x3968('0x3b5','56rC')]())this['createUserInfo']();continue;case'7':Scene_Base['prototype'][_0x3968('0x3b6','DSo%')]['call'](this);continue;case'8':Graphics[_0x3968('0x3b7','AugK')]['value']='';continue;case'9':Graphics[_0x3968('0x3b8','BN7V')][_0x3968('0x3b9','56rC')]=0xf;continue;case'10':Graphics[_0x3968('0x3ba','^CBW')](_0x3968('0x3bb','B6uD'),_0x56a2be[_0x3968('0x3bc','BN7V')](Graphics[_0x3968('0x3bd','XFHa')]/0x2,0x14),PlayOnline[_0x3968('0x3be','b8bb')][_0x3968('0x3bf','P%uC')]+0x82,0xc8,0x19,0x18);continue;}break;}};Scene_PlayOnline[_0x3968('0x3c0','X)dO')][_0x3968('0x3c1','92uf')]=function(){this[_0x3968('0x3c2','57]T')]=new Sprite(ImageManager[_0x3968('0x3c3','@tYj')]($dataSystem['title1Name']));this[_0x3968('0x3c4','g4!!')]=new Sprite(ImageManager[_0x3968('0x3c5','TC3d')]($dataSystem[_0x3968('0x3c6','DSo%')]));this[_0x3968('0x3c7','^CBW')](this[_0x3968('0x3c8','4WKX')]);this[_0x3968('0x3c9','@tYj')](this[_0x3968('0x3ca','U(Ap')]);};Scene_PlayOnline[_0x3968('0x3cb','pEq[')][_0x3968('0x3cc','mkla')]=function(){var _0x457caf={'yiimb':'login','xVURX':'jhmw','GxykA':'quxiao','WYAtW':_0x3968('0x3cd','7g26'),'VzDDg':'lnR','VFhPA':function _0x2463c5(_0x32b8ce,_0x48d8c2){return _0x32b8ce(_0x48d8c2);},'dOsTg':function _0x5a6eb5(_0x472baa,_0x4f2bf2){return _0x472baa!==_0x4f2bf2;},'wJBIN':_0x3968('0x3ce','bii*'),'oTdYM':function _0x417a4d(_0x4e568c,_0x172f1c){return _0x4e568c!=_0x172f1c;},'lGPYJ':function _0x510ccc(_0x9059c4,_0x54fee1){return _0x9059c4(_0x54fee1);},'LQUAG':function _0x5d7223(_0x93e66b,_0x3b5ad1){return _0x93e66b===_0x3b5ad1;},'lMoqa':'FnL','jgWIz':'maG','xIPRy':_0x3968('0x3cf','o^84')};this[_0x3968('0x3d0','AugK')]=new Window_LoginCommand(this[_0x3968('0x309','ac4V')]);this[_0x3968('0x3d1','6(^C')]['y']=this['_loginWindow']['y']+this['_loginWindow']['height'];this[_0x3968('0x3d2','ohA4')]['opacity']=PlayOnline[_0x3968('0x3d3','DSo%')][_0x3968('0x3d4','Go7X')];if(!PlayOnline['SaveInfo']){this[_0x3968('0x3d5','o^84')]['setHandler'](_0x457caf[_0x3968('0x3d6','vI#!')],this['LoginOk'][_0x3968('0x3d7','DSo%')](this));this[_0x3968('0x3d8','ac4V')][_0x3968('0x3d9','vI#!')](_0x457caf[_0x3968('0x3da','qkk]')],this['GoHome'][_0x3968('0x3db','qkk]')](this));this['_commandWindow'][_0x3968('0x3dc','ZR02')](_0x457caf[_0x3968('0x3dd','JnKS')],this[_0x3968('0x3de','B6b!')][_0x3968('0x3df','92uf')](this));}else{if(_0x457caf['WYAtW']===_0x457caf[_0x3968('0x3e0','^CBW')]){_0x457caf['VFhPA'](alert,'连接不到服务器，请检查网络重试。');PlayOnline[_0x3968('0x1d7','DSo%')]=null;}else{if(parseInt(PlayOnline[_0x3968('0x3e1','XFHa')][0x0])<=0x5){this[_0x3968('0x3e2','bii*')][_0x3968('0x3e3','MhLY')](_0x457caf[_0x3968('0x3e4','ac4V')],this['LoginOk'][_0x3968('0x3e5','dJbc')](this));this[_0x3968('0x3e6','7g26')][_0x3968('0x144','AugK')](_0x3968('0x3e7','w*%c'),this[_0x3968('0x3e8','y3I5')][_0x3968('0x3e9','])na')](this));this[_0x3968('0x139','xh!r')][_0x3968('0x3ea','Geuc')](_0x457caf[_0x3968('0x3eb','57]T')],this[_0x3968('0x3ec','xh!r')][_0x3968('0x3ed','MhLY')](this));}else{if(_0x457caf['dOsTg'](_0x3968('0x3ee','GRVS'),_0x3968('0x3ef','Go7X'))){var _0x516e85=null;var _0x424b0f=StorageManager[_0x3968('0x3f0','oBAm')](_0x457caf[_0x3968('0x3f1','P%uC')]);try{_0x516e85=JsonEx[_0x3968('0x3f2','B6b!')](LZString[_0x3968('0x3f3','56rC')](localStorage['getItem'](_0x424b0f)));}catch(_0x25da72){}if(_0x516e85){Graphics[_0x3968('0x3f4','TC3d')][_0x3968('0x3f5','B6b!')]=_0x516e85[_0x3968('0x3f6','7g26')];Graphics['_input2'][_0x3968('0x3f7','e20f')]=_0x516e85[_0x3968('0x3f8','P%uC')];}}else{if(_0x457caf[_0x3968('0x3f9','^CBW')](_0x457caf[_0x3968('0x3fa','oBAm')](parseInt,PlayOnline[_0x3968('0x3fb','])na')][0x1]),PlayOnline[_0x3968('0x66','P%uC')][_0x3968('0x3fc','b8bb')])||StorageManager[_0x3968('0x3fd','7g26')]()){if(_0x457caf[_0x3968('0x3fe','dJbc')](_0x457caf[_0x3968('0x3ff','ZqhS')],_0x457caf[_0x3968('0x400','dJbc')])){this['_commandWindow'][_0x3968('0x401','B6b!')]('download',this[_0x3968('0x402','ohA4')][_0x3968('0x403','BN7V')](this));this['_commandWindow'][_0x3968('0x404','56rC')](_0x3968('0x405','TC3d'),this[_0x3968('0x406','b8bb')][_0x3968('0x407','IY^0')](this));}else{PlayOnline['Scene_Menu_createCommandWindow'][_0x3968('0x212','DSo%')](this);this['_commandWindow'][_0x3968('0x408','4WKX')]('Ranking',this[_0x3968('0x409','!aAg')]['bind'](this));}}else{if(_0x457caf['dOsTg'](_0x457caf[_0x3968('0x40a','@tYj')],_0x3968('0x40b','t1a6'))){this[_0x3968('0x40c','#r7G')][_0x3968('0x40d','IY^0')]('loadgame',this[_0x3968('0x40e','])na')][_0x3968('0x40f','N^7B')](this));this[_0x3968('0x410','@tYj')][_0x3968('0x411','JnKS')](_0x457caf[_0x3968('0x412','])na')],this[_0x3968('0x413','@tYj')]['bind'](this));}else{this[_0x3968('0x414','Wl!b')]($gameSystem[_0x3968('0x415','!aAg')][i]);}}}}}}this['addWindow'](this['_commandWindow']);};Scene_PlayOnline[_0x3968('0x3c0','X)dO')][_0x3968('0x416','t1a6')]=function(){this['_loginWindow']=new Window_LoginInfo();this[_0x3968('0x417','b8bb')]['opacity']=PlayOnline['Param'][_0x3968('0x418','Geuc')];this[_0x3968('0x419','MhLY')](this[_0x3968('0x41a','ohA4')]);};Scene_PlayOnline['prototype']['createConfirmWindow']=function(){var _0x2466dd={'iDbhk':function _0x49705d(_0x128c91,_0x1757e8){return _0x128c91-_0x1757e8;},'wIvsD':'confirm','zVeUC':_0x3968('0x41b','xh!r')};var _0x436425=0x190;var _0x141f27=_0x2466dd['iDbhk'](Graphics[_0x3968('0x41c','JnKS')],_0x436425)/0x2;var _0x3237b0=PlayOnline['Param'][_0x3968('0x41d','y3I5')];this[_0x3968('0x41e','TC3d')]=new Window_Action(_0x141f27,_0x3237b0,_0x436425);this[_0x3968('0x41f','N^7B')][_0x3968('0x420','xh!r')]=PlayOnline[_0x3968('0x41','4WKX')]['WindowOpacity'];this['_confirmWindow'][_0x3968('0x421','bii*')](_0x2466dd[_0x3968('0x422','qkk]')],this[_0x3968('0x423','^CBW')][_0x3968('0x424','e20f')](this));this[_0x3968('0x425','])na')][_0x3968('0x426','#r7G')](_0x2466dd[_0x3968('0x427','@tYj')],this[_0x3968('0x428','JnKS')]['bind'](this));this[_0x3968('0x429','GRVS')](this['_confirmWindow']);};Scene_PlayOnline[_0x3968('0x347','DSo%')][_0x3968('0x42a','56rC')]=function(){var _0x3f0582={'TxCCm':function _0x5e4396(_0x41d488,_0x29ca8e){return _0x41d488/_0x29ca8e;},'CqvPl':function _0x4dfab2(_0x2d90fd,_0x5b8cea){return _0x2d90fd-_0x5b8cea;}};if(PlayOnline[_0x3968('0x73','MhLY')]['PopupBgd']){this[_0x3968('0x42b','y3I5')]=new Sprite(ImageManager['loadPicture'](PlayOnline[_0x3968('0x42c','JnKS')][_0x3968('0x42d','MhLY')]));this[_0x3968('0x42e','MhLY')](this[_0x3968('0x42f','BN7V')],0x1);this[_0x3968('0x430','ac4V')][_0x3968('0x431','t1a6')](_0x3f0582[_0x3968('0x432','BN7V')](Graphics[_0x3968('0x433','oBAm')]-0x180,0x2),_0x3f0582[_0x3968('0x434','ac4V')](Graphics['boxHeight'],this[_0x3968('0x435','DSo%')][_0x3968('0x436','B6uD')])/0x2);}};Scene_PlayOnline[_0x3968('0x3a5','ZR02')][_0x3968('0x437','!aAg')]=function(){var _0x29808d={'cGVgL':function _0x59b8ad(_0x556cf7,_0xfc205){return _0x556cf7===_0xfc205;}};var _0x324bc8=null;var _0x32b385=StorageManager[_0x3968('0x438','AugK')](_0x3968('0x439','U(Ap'));try{if(_0x29808d[_0x3968('0x43a','AugK')]('FUr','FUr')){_0x324bc8=JsonEx[_0x3968('0x43b','AugK')](LZString[_0x3968('0x43c','4WKX')](localStorage[_0x3968('0x43d','6(^C')](_0x32b385)));}else{alert(_0x3968('0x43e','#r7G'));PlayOnline['UserName']=null;}}catch(_0x3fd946){}if(_0x324bc8){Graphics[_0x3968('0x43f','y3I5')][_0x3968('0x440','dJbc')]=_0x324bc8['user'];Graphics['_input2'][_0x3968('0x441','6(^C')]=_0x324bc8[_0x3968('0x442','Go7X')];}};Scene_PlayOnline[_0x3968('0x443','g4!!')][_0x3968('0x444','DSo%')]=function(){var _0x519d19={'VxXEm':function _0x1e1332(_0x31bb0d,_0x565d7e){return _0x31bb0d/_0x565d7e;},'OaVyl':function _0x25150c(_0x4cb15a,_0x185191){return _0x4cb15a-_0x185191;},'HfQem':function _0x150351(_0x5b8ae7,_0x4af953){return _0x5b8ae7<_0x4af953;},'dXzlJ':function _0x1af03c(_0x43ffd7,_0x3d4a92){return _0x43ffd7+_0x3d4a92;},'OQnNl':function _0x39ad6c(_0x16d18b,_0x405041){return _0x16d18b>=_0x405041;},'JbZio':function _0x732883(_0x165c0a,_0x837213){return _0x165c0a+_0x837213;}};Scene_Base[_0x3968('0x3a9','MhLY')][_0x3968('0x445','XFHa')][_0x3968('0x1cb','b8bb')](this);var _0x4b76f8=_0x519d19['VxXEm'](Graphics[_0x3968('0x446','XFHa')],0x2)-0x14;var _0x42f8e9=PlayOnline['Param'][_0x3968('0x447','XFHa')]+0x75;var _0x3fad15=0xc8;var _0x56ecc7=0x28;if(TouchInput['isPressed']()){var _0x481d8f=TouchInput['x'];var _0x5766b0=TouchInput['y'];if(_0x481d8f>=_0x4b76f8&&_0x5766b0>=_0x519d19[_0x3968('0x448','X)dO')](_0x42f8e9,_0x56ecc7)&&_0x519d19[_0x3968('0x449','7g26')](_0x481d8f,_0x519d19[_0x3968('0x44a','BN7V')](_0x4b76f8,_0x3fad15))&&_0x519d19[_0x3968('0x44b','g4!!')](_0x5766b0,_0x42f8e9)){this[_0x3968('0x1fd','^CBW')]['activate']();this['_commandWindow']['deactivate']();Graphics['_input'][_0x3968('0x44c','P%uC')]();}else if(_0x481d8f>=_0x4b76f8&&_0x519d19['OQnNl'](_0x5766b0,_0x42f8e9)&&_0x481d8f<_0x4b76f8+_0x3fad15&&_0x5766b0<_0x519d19['JbZio'](_0x42f8e9,_0x56ecc7)){this[_0x3968('0x44d','Geuc')]['activate']();this[_0x3968('0x3d2','ohA4')][_0x3968('0xbe','oBAm')]();Graphics[_0x3968('0x44e','JnKS')]['focus']();}else{this[_0x3968('0x44f','XFHa')][_0x3968('0x450','DSo%')]();this[_0x3968('0x410','@tYj')]['activate']();}}if(TouchInput[_0x3968('0x451','Go7X')]()){this[_0x3968('0x452','B6b!')][_0x3968('0x453','#r7G')]();}};Scene_PlayOnline['prototype'][_0x3968('0x454','XFHa')]=function(){var _0x118921={'QPSoB':_0x3968('0x455','IY^0'),'GwATV':function _0x2939c1(_0x353436,_0x33fe7e){return _0x353436+_0x33fe7e;}};var _0x1cae83=_0x118921[_0x3968('0x456','56rC')][_0x3968('0x457','57]T')]('|'),_0x498200=0x0;while(!![]){switch(_0x1cae83[_0x498200++]){case'0':Graphics[_0x3968('0x458','b8bb')]();continue;case'1':this[_0x3968('0x459','xh!r')]();continue;case'2':PlayOnline[_0x3968('0x45a','B6uD')]=_0x118921[_0x3968('0x45b','])na')]('',Graphics['_input2'][_0x3968('0x440','dJbc')]);continue;case'3':Graphics[_0x3968('0x45c','ZR02')]();continue;case'4':if(window[_0x3968('0x45d','92uf')]&&window[_0x3968('0x45e','xh!r')])window[_0x3968('0x45f','DSo%')]['hide']();continue;case'5':this['_commandWindow'][_0x3968('0x460','ohA4')]();continue;case'6':this[_0x3968('0x461','oBAm')][_0x3968('0x462','U(Ap')]();continue;case'7':PlayOnline[_0x3968('0xf6','ZR02')]=_0x118921[_0x3968('0x463','GRVS')]('',Graphics['_input'][_0x3968('0x464','AugK')]);continue;case'8':this[_0x3968('0x147','^CBW')][_0x3968('0x465','BN7V')]();continue;}break;}};Scene_PlayOnline[_0x3968('0x347','DSo%')][_0x3968('0x466','Geuc')]=function(){var _0x25d4a8={'GSbQf':_0x3968('0x467','e20f'),'UAmaX':function _0x43438b(_0x39fbfc,_0x376fef){return _0x39fbfc!==_0x376fef;},'Hjduf':_0x3968('0x468','^CBW')};if(Utils['isMobileDevice']()){if(_0x25d4a8['GSbQf']==='Cix'){if(_0x25d4a8[_0x3968('0x469','o^84')]($gameSystem['versionId'](),$dataSystem[_0x3968('0x46a','!aAg')])){$gamePlayer[_0x3968('0x46b','B6uD')]($gameMap['mapId'](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x3968('0x46c','b8bb')]();}}else{if(!PlayOnline['Param']['Website2APK'])window[_0x3968('0x46d','ac4V')](_0x25d4a8[_0x3968('0x46e','CO[Y')]);else Website2APK['openExternal']('https://bbs.jhmeng.cn/');}}else window[_0x3968('0x46f','Geuc')](_0x25d4a8[_0x3968('0x470','B6uD')]);};Scene_PlayOnline[_0x3968('0x471','vI#!')][_0x3968('0x472','ohA4')]=function(){window[_0x3968('0x473','dJbc')]();};Scene_PlayOnline[_0x3968('0x33c','AugK')][_0x3968('0x474','CO[Y')]=function(){var _0x1837db={'KSOne':function _0x489b24(_0xacb47,_0x28d346){return _0xacb47===_0x28d346;},'CBLtF':_0x3968('0x475','AugK'),'UnbEC':_0x3968('0x476','CO[Y'),'goiVU':function _0x4116a4(_0xccdc0f,_0x5abd44){return _0xccdc0f(_0x5abd44);}};if(Utils[_0x3968('0x477','GRVS')]()){if(_0x1837db[_0x3968('0x478','ZqhS')](_0x1837db['CBLtF'],_0x1837db[_0x3968('0x479','vI#!')])){if(args[0x0])$gameVariables[_0x3968('0x47a','ZR02')](args[0x0],_0x1837db[_0x3968('0x47b','GRVS')](parseInt,msg));}else{if(!PlayOnline['Param'][_0x3968('0x47c','!aAg')])window[_0x3968('0x47d','vI#!')](PlayOnline['Param']['GameLink']);else Website2APK[_0x3968('0x47e','6(^C')](PlayOnline['Param'][_0x3968('0x47f','JnKS')]);}}else window[_0x3968('0x480','oBAm')](PlayOnline[_0x3968('0x5f','BN7V')][_0x3968('0x481','qkk]')]);};Scene_PlayOnline[_0x3968('0x471','vI#!')][_0x3968('0x482','U(Ap')]=function(){var _0x13735d={'DQFoI':_0x3968('0x483','#r7G'),'FdsQt':function _0x39522d(_0x5c309b,_0x78f29e){return _0x5c309b-_0x78f29e;},'yURXS':'4|3|2|1|0','oAgMt':function _0x327eab(_0x1355ec,_0x38f0f1,_0x2baaf1){return _0x1355ec(_0x38f0f1,_0x2baaf1);}};SoundManager['playOk']();if(PlayOnline[_0x3968('0x484','bii*')][0x3]==0x0){if(_0x13735d['DQFoI']!==_0x3968('0x485','xh!r')){this[_0x3968('0x486','7g26')]=new Sprite(ImageManager[_0x3968('0x487','ac4V')](PlayOnline[_0x3968('0x76','w*%c')][_0x3968('0x488','P%uC')]));this[_0x3968('0x489','g4!!')](this['_cbgdSprite'],0x1);this[_0x3968('0x48a','TC3d')][_0x3968('0x48b','^CBW')]((Graphics[_0x3968('0x48c','MhLY')]-0x180)/0x2,_0x13735d[_0x3968('0x48d','pEq[')](Graphics[_0x3968('0x48e','B6b!')],this[_0x3968('0x48f','qkk]')][_0x3968('0x490','t1a6')])/0x2);}else{var _0x4d9a5e=_0x13735d[_0x3968('0x491','U(Ap')]['split']('|'),_0x53aa30=0x0;while(!![]){switch(_0x4d9a5e[_0x53aa30++]){case'0':_0x13735d[_0x3968('0x492','oBAm')](setInterval,function(){DataManager[_0x3968('0x493','JnKS')]();},0x3e8);continue;case'1':if(PlayOnline[_0x3968('0x494','92uf')][_0x3968('0x495','@bN4')])$gameVariables[_0x3968('0x496','pEq[')](PlayOnline[_0x3968('0x42c','JnKS')]['OnlyValue'],PlayOnline[_0x3968('0x497','g4!!')][0x2]);continue;case'2':SceneManager[_0x3968('0x498','b8bb')](Scene_Map);continue;case'3':this[_0x3968('0x499','@bN4')]();continue;case'4':DataManager['setupNewGame']();continue;}break;}}}else{this['_commandWindow'][_0x3968('0x49a','7g26')]();this[_0x3968('0x410','@tYj')][_0x3968('0x450','DSo%')]();this[_0x3968('0x49b','xh!r')]();this[_0x3968('0x49c','57]T')][_0x3968('0x49d','ZR02')]();this[_0x3968('0x49e','P%uC')][_0x3968('0xbb','ZR02')]('确定全新游戏放弃已有存档');this[_0x3968('0x49f','CO[Y')]['activate']();this[_0x3968('0x425','])na')][_0x3968('0x4a0','y3I5')](0x0);}};Scene_PlayOnline[_0x3968('0x4a1','oBAm')]['LoadGame']=function(){var _0x3ed659={'VSNuQ':function _0x358dd4(_0x3e6d13,_0x23babf){return _0x3e6d13!=_0x23babf;},'zEQas':function _0x3a2160(_0x3589a0,_0x56d7f8){return _0x3589a0(_0x56d7f8);}};SoundManager['playOk']();if(_0x3ed659[_0x3968('0x4a2','7g26')](PlayOnline['SaveInfo'][0x3],0x0)){DataManager[_0x3968('0x4a3','g4!!')](PlayOnline[_0x3968('0x4a4','U(Ap')][0x4]);this[_0x3968('0x4a5','AugK')]();this[_0x3968('0x4a6','ZR02')]();$gameParty[_0x3968('0x4a7','B6uD')]();$gameTroop[_0x3968('0x4a8','N^7B')]();var _0x4f6b82=_0x3ed659[_0x3968('0x4a9','CO[Y')](parseInt,Utils[_0x3968('0x4aa','@bN4')]['substr'](0x2,0x1));if(_0x4f6b82>=0x5)ImageManager[_0x3968('0x4ab','#r7G')]();SceneManager[_0x3968('0x4ac','ZR02')](Scene_Map);if(PlayOnline[_0x3968('0xde','B6b!')][_0x3968('0x4ad','ZqhS')])$gameVariables['setValue'](PlayOnline[_0x3968('0x42c','JnKS')]['OffLine'],parseInt(PlayOnline['SaveInfo'][0x0]));if(PlayOnline[_0x3968('0xc1','bii*')][_0x3968('0x4ae','y3I5')])$gameVariables[_0x3968('0x4af','U(Ap')](PlayOnline['Param'][_0x3968('0x4b0','ZqhS')],PlayOnline[_0x3968('0x484','bii*')][0x2]);setInterval(function(){var _0x2940da={'NQxbN':function _0x1cffb8(_0x3cf2e1,_0x8b97c7){return _0x3cf2e1!==_0x8b97c7;},'jkeOE':_0x3968('0x4b1','ZR02')};if(_0x2940da[_0x3968('0x4b2','mkla')](_0x2940da[_0x3968('0x4b3','BN7V')],_0x3968('0x4b4','b8bb'))){var _0x2d05d1=!![];this[_0x3968('0x4b5','92uf')](PlayOnline['Param'][_0x3968('0x4b6','BN7V')],_0x3968('0x4b7','XFHa'),_0x2d05d1);}else{DataManager[_0x3968('0x4b8','CO[Y')]();}},0x3e8);}else{var _0x21b086=_0x3968('0x4b9','BN7V')['split']('|'),_0x190fe9=0x0;while(!![]){switch(_0x21b086[_0x190fe9++]){case'0':this[_0x3968('0x4ba','6(^C')]();continue;case'1':this[_0x3968('0x131','U(Ap')][_0x3968('0x4bb','IY^0')]();continue;case'2':this['_confirmWindow'][_0x3968('0x4bc','JnKS')](0x0);continue;case'3':this[_0x3968('0x49c','57]T')]['activate']();continue;case'4':this['_commandWindow'][_0x3968('0x4bd','Geuc')]();continue;case'5':this['_confirmWindow'][_0x3968('0x4be','X)dO')](_0x3968('0x4bf','JnKS'));continue;case'6':this['_confirmWindow']['open']();continue;}break;}}};Scene_PlayOnline[_0x3968('0x360','4WKX')][_0x3968('0x4c0','CO[Y')]=function(){var _0x15b662={'LLnpL':function _0x47c9cc(_0x800714,_0x250f20,_0x1deb62){return _0x800714(_0x250f20,_0x1deb62);}};this[_0x3968('0x4c1','^CBW')]['deactivate']();this[_0x3968('0x4c2','xh!r')][_0x3968('0x4c3','!aAg')]();this['removeChild'](this[_0x3968('0x4c4','CO[Y')]);DataManager[_0x3968('0x4c5','oBAm')]();this[_0x3968('0x4c6','])na')]();SceneManager[_0x3968('0x4c7','Geuc')](Scene_Map);if(PlayOnline['Param'][_0x3968('0x4c8','])na')])$gameVariables['setValue'](PlayOnline[_0x3968('0x4c9','qkk]')][_0x3968('0x4ca','Geuc')],PlayOnline['SaveInfo'][0x2]);_0x15b662[_0x3968('0x4cb','])na')](setInterval,function(){DataManager[_0x3968('0x4cc','vI#!')]();},0x3e8);};Scene_PlayOnline[_0x3968('0x347','DSo%')][_0x3968('0x4cd','w*%c')]=function(){var _0x4b5ba8={'ldEOO':_0x3968('0x4ce','Geuc')};var _0x864098=_0x4b5ba8[_0x3968('0x4cf','Wl!b')][_0x3968('0x8c','B6b!')]('|'),_0x1d048a=0x0;while(!![]){switch(_0x864098[_0x1d048a++]){case'0':this[_0x3968('0xb8','MhLY')][_0x3968('0x4d0','7g26')]();continue;case'1':this[_0x3968('0x4d1','o^84')]['deactivate']();continue;case'2':this['_commandWindow']['select']();continue;case'3':this[_0x3968('0x4d2','b8bb')][_0x3968('0x4d3','vI#!')]();continue;case'4':this['removeChild'](this['_cbgdSprite']);continue;}break;}};Scene_PlayOnline['prototype'][_0x3968('0x4d4','TC3d')]=function(){var _0x5060a6={'gGbtd':function _0x4a4617(_0x23078a,_0x1ceb1f){return _0x23078a!==_0x1ceb1f;}};if(_0x5060a6[_0x3968('0x4d5','])na')]($gameSystem['versionId'](),$dataSystem[_0x3968('0x4d6','B6b!')])){$gamePlayer[_0x3968('0x4d7','o^84')]($gameMap[_0x3968('0x4d8','b8bb')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x3968('0x4d9','bii*')]();}};Scene_PlayOnline['prototype'][_0x3968('0x4da','IY^0')]=function(){var _0x420959={'VOcvr':_0x3968('0x4db','mkla'),'wuhYa':function _0x3d9aa0(_0xfb2607,_0xc9af7){return _0xfb2607-_0xc9af7;},'VULxI':function _0xdf852f(_0x2fc4eb,_0x3104fa){return _0x2fc4eb+_0x3104fa;},'LLEkh':function _0x456cce(_0x20bacd,_0x5812a6){return _0x20bacd/_0x5812a6;},'xGUKj':function _0x42a543(_0x16372c,_0x2f3308){return _0x16372c(_0x2f3308);},'bQpfZ':'text','XpJUW':function _0x43db19(_0x25ad6e,_0xb24895){return _0x25ad6e(_0xb24895);},'LNBDO':function _0x3ede8e(_0x2f99cc,_0x23fb2e){return _0x2f99cc==_0x23fb2e;},'rsKZG':_0x3968('0x4dc','o^84'),'Zaypt':_0x3968('0x4dd','dJbc'),'ywiCr':'ckc','MzZuG':_0x3968('0x4de','6(^C'),'yjFoL':_0x3968('0x4df','U(Ap'),'sFgbL':function _0x56153d(_0x27bd68,_0x3d5122){return _0x27bd68(_0x3d5122);},'VneVy':_0x3968('0x4e0','ac4V'),'gvZwG':function _0x1f10a0(_0x3b410f,_0x577312){return _0x3b410f-_0x577312;},'EaEcD':_0x3968('0x4e1','N^7B')};var _0x101a68=_0x420959[_0x3968('0x4e2','TC3d')][_0x3968('0x4e3','!aAg')]('|'),_0x3ad9d1=0x0;while(!![]){switch(_0x101a68[_0x3ad9d1++]){case'0':if(!PlayOnline[_0x3968('0x35f','Wl!b')]){PlayOnline[_0x3968('0x4e4','MhLY')]=null;PlayOnline[_0x3968('0x4e5','B6b!')]=null;Graphics[_0x3968('0x4e6','XFHa')](_0x3968('0x4e7','ac4V'),_0x420959[_0x3968('0x4e8','y3I5')](Graphics[_0x3968('0x4e9','!aAg')]/0x2,0x14),_0x420959[_0x3968('0x4ea','X)dO')](PlayOnline[_0x3968('0x4eb','B6uD')]['WindowY'],0x50),0x96,0x19,0x18);Graphics[_0x3968('0x4ec','qkk]')][_0x3968('0x4ed','IY^0')]=0xf;Graphics[_0x3968('0x4ee','bii*')][_0x3968('0x4ef','b8bb')]='';Graphics[_0x3968('0x4f0','oBAm')](_0x3968('0x4f1','N^7B'),_0x420959['LLEkh'](Graphics[_0x3968('0x4f2','7g26')],0x2)-0x14,PlayOnline['Param'][_0x3968('0x4f3','X)dO')]+0x82,0x96,0x19,0x18);Graphics['_input2']['maxLength']=0xf;Graphics['_input2'][_0x3968('0x4f4','Geuc')]='';}else{if(_0x420959['xGUKj'](parseInt,PlayOnline[_0x3968('0x4f5','JnKS')][0x0])<=0x5){PlayOnline[_0x3968('0x4f6','GRVS')]=null;PlayOnline[_0x3968('0x4f7','BN7V')]=null;Graphics['_addInput'](_0x420959[_0x3968('0x4f8','7g26')],Graphics['width']/0x2-0x14,_0x420959['VULxI'](PlayOnline[_0x3968('0x5d','t1a6')][_0x3968('0x41d','y3I5')],0x50),0x96,0x19,0x18);Graphics['_input'][_0x3968('0x4f9','#r7G')]=0xf;Graphics['_input'][_0x3968('0x4fa','])na')]='';Graphics[_0x3968('0x4fb','Wl!b')](_0x3968('0x4fc','6(^C'),_0x420959[_0x3968('0x4fd','56rC')](_0x420959[_0x3968('0x4fe','])na')](Graphics[_0x3968('0x4ff','dJbc')],0x2),0x14),_0x420959['VULxI'](PlayOnline[_0x3968('0x3b','IY^0')]['WindowY'],0x82),0x96,0x19,0x18);Graphics['_input2'][_0x3968('0x500','U(Ap')]=0xf;Graphics[_0x3968('0x501','Wl!b')][_0x3968('0x502','GRVS')]='';}else{this['createConfirmWindow']();}}continue;case'1':this[_0x3968('0x3d5','o^84')][_0x3968('0x503','57]T')]();continue;case'2':$[_0x3968('0x504','ZR02')]({'type':'post','url':PlayOnline[_0x3968('0x194','57]T')],'data':{'user':PlayOnline['UserName'],'word':PlayOnline['Password'],'game':PlayOnline[_0x3968('0x49','pEq[')][_0x3968('0x505','dJbc')],'cmd':_0x3968('0x506','92uf')},'success':function(_0x29e7a4){if(!PlayOnline[_0x3968('0x206','pEq[')]){if(_0x3f36dd[_0x3968('0x507','oBAm')](_0x3968('0x508','57]T'),'DSm')){if(!PlayOnline[_0x3968('0x509','ohA4')])return 0x3;else{if(_0x3f36dd['QOjiA'](parseInt(PlayOnline[_0x3968('0x50a','Geuc')][0x0]),0x5))return 0x3;else return 0x2;}}else{_0x3f36dd[_0x3968('0x50b','Wl!b')](alert,_0x3f36dd[_0x3968('0x50c','!aAg')]);}}else if(!PlayOnline[_0x3968('0x50d','@tYj')]){_0x3f36dd['LBgEU'](alert,_0x3968('0x50e','Go7X'));PlayOnline[_0x3968('0xad','JnKS')]=null;}else if(_0x3f36dd['QOjiA'](_0x29e7a4,-0x5)){_0x3f36dd[_0x3968('0x50f','mkla')](alert,_0x3968('0x510','MhLY'));PlayOnline[_0x3968('0x511','@tYj')]=null;}else if(_0x3f36dd['wYInl'](_0x29e7a4,-0x1)){if(_0x3f36dd[_0x3968('0x512','ohA4')]!==_0x3968('0x513','b8bb')){_0x3f36dd[_0x3968('0x50f','mkla')](alert,_0x3f36dd[_0x3968('0x514','e20f')]);PlayOnline['UserName']=null;}else{DataManager['setupNewGame']();this[_0x3968('0x515','pEq[')]();SceneManager[_0x3968('0x516','BN7V')](Scene_Map);if(PlayOnline[_0x3968('0x5d','t1a6')][_0x3968('0x517','DSo%')])$gameVariables[_0x3968('0x518','^CBW')](PlayOnline[_0x3968('0x202','U(Ap')][_0x3968('0x4c8','])na')],PlayOnline[_0x3968('0x519','Go7X')][0x2]);setInterval(function(){DataManager[_0x3968('0x51a','t1a6')]();},0x3e8);}}else if(_0x3f36dd['wYInl'](_0x29e7a4,-0x2)){_0x3f36dd['YbYcw'](alert,_0x3f36dd[_0x3968('0x51b','b8bb')]);PlayOnline[_0x3968('0x388','e20f')]=null;}else if(_0x3f36dd[_0x3968('0x51c','dJbc')](_0x29e7a4,-0x3)){if(_0x3f36dd['tmddn']==='Ixt'){var _0x14add6=fn[_0x3968('0x51d','AugK')](context,arguments);fn=null;return _0x14add6;}else{_0x3f36dd[_0x3968('0x51e','7g26')](alert,_0x3968('0x51f','U(Ap'));PlayOnline[_0x3968('0x520','Wl!b')]=null;}}else if(_0x29e7a4==-0x4){alert(_0x3f36dd[_0x3968('0x521','^CBW')]);PlayOnline['UserName']=null;}else if(_0x29e7a4==-0x5){if(_0x3f36dd[_0x3968('0x522','Go7X')](_0x3f36dd[_0x3968('0x523','IY^0')],'ixb')){var _0x551c4b=y+this[_0x3968('0x524','MhLY')]()/0x2-0x1;this[_0x3968('0x525','])na')]['paintOpacity']=0x64;this[_0x3968('0x526','U(Ap')][_0x3968('0x527','TC3d')](0x0,_0x551c4b,this['contentsWidth'](),0x2,this[_0x3968('0x528','qkk]')]());this[_0x3968('0x529','dJbc')][_0x3968('0x52a','6(^C')]=0xff;}else{alert(_0x3f36dd['oHuVC']);PlayOnline[_0x3968('0x8f','mkla')]=null;}}else{if(_0x3968('0x52b','w*%c')===_0x3f36dd[_0x3968('0x52c','y3I5')]){if(!PlayOnline['Param'][_0x3968('0x52d','7g26')])window['open'](_0x3f36dd[_0x3968('0x52e','CO[Y')]);else Website2APK[_0x3968('0x52f','B6uD')](_0x3f36dd[_0x3968('0x530','57]T')]);}else{PlayOnline[_0x3968('0x531','w*%c')]=_0x29e7a4[_0x3968('0x532','bii*')](',');if(_0x3f36dd[_0x3968('0x533','P%uC')](parseInt,PlayOnline['SaveInfo'][0x0])>0x5)DataManager['InitializePlayer']();if(Utils[_0x3968('0x534','mkla')]()){if(_0x3968('0x535','X)dO')===_0x3968('0x536','57]T')){var _0x2540c1=_0x3f36dd[_0x3968('0x537','MhLY')]['split']('|'),_0x2a44fe=0x0;while(!![]){switch(_0x2540c1[_0x2a44fe++]){case'0':this[_0x3968('0x538','pEq[')][_0x3968('0x539','DSo%')]=_0x3f36dd[_0x3968('0x53a','ac4V')](Graphics['boxHeight'],this['_commandWindow'][_0x3968('0x53b','b8bb')]);continue;case'1':this[_0x3968('0x53c','DSo%')]();continue;case'2':Scene_MenuBase[_0x3968('0x53d','ohA4')][_0x3968('0x53e','e20f')][_0x3968('0x53f','P%uC')](this);continue;case'3':this['_commandWindow']['y']=_0x3f36dd[_0x3968('0x540','qkk]')](Graphics['boxHeight'],this['_commandWindow'][_0x3968('0x541','#r7G')]);continue;case'4':this[_0x3968('0x542','vI#!')]();continue;}break;}}else{var _0x402b64={'user':PlayOnline['UserName'],'pass':PlayOnline[_0x3968('0x543','bii*')]};StorageManager['saveWebFile'](_0x402b64,_0x3f36dd[_0x3968('0x544','B6uD')]);}}}}},'error':function(_0x4b407c){PlayOnline[_0x3968('0x1e8','ohA4')]=null;_0x3f36dd[_0x3968('0x545','y3I5')](alert,_0x3f36dd['qbOkJ']);},'async':![]});continue;case'3':this['createLoginWindow']();continue;case'4':this[_0x3968('0x546','vI#!')][_0x3968('0x547','AugK')]();continue;case'5':this[_0x3968('0x548','BN7V')]();continue;case'6':var _0x3f36dd={'zDyXR':function _0x56bb60(_0x1fa488,_0x2a4bd9){return _0x1fa488===_0x2a4bd9;},'QOjiA':function _0x589f85(_0x93e64b,_0x21ed3f){return _0x93e64b<=_0x21ed3f;},'liqYY':function _0x437268(_0x3c8ccd,_0x2005b8){return _0x3c8ccd(_0x2005b8);},'UgyAx':_0x3968('0x549','y3I5'),'LBgEU':function _0x3943f2(_0x321812,_0x35579c){return _0x420959[_0x3968('0x54a','])na')](_0x321812,_0x35579c);},'YbYcw':function _0x44b12f(_0x17e8dd,_0x47a841){return _0x17e8dd(_0x47a841);},'wYInl':function _0x478df6(_0x3d82ea,_0x423bb5){return _0x420959[_0x3968('0x54b','bii*')](_0x3d82ea,_0x423bb5);},'SqzTs':_0x420959['rsKZG'],'JyfPm':'密码错误请确认密码再试','zpnew':_0x420959[_0x3968('0x54c','oBAm')],'HAiLV':function _0x5d0cd2(_0x29841d,_0x48ff3e){return _0x29841d==_0x48ff3e;},'tmddn':_0x420959[_0x3968('0x54d','pEq[')],'yhVBB':_0x3968('0x54e','@tYj'),'xeSAM':function _0x45f6f2(_0x159169,_0x369dcc){return _0x159169===_0x369dcc;},'qYqQK':_0x3968('0x54f','@tYj'),'oHuVC':_0x420959['MzZuG'],'GWwhC':_0x3968('0x550','@tYj'),'wGcbm':_0x420959['yjFoL'],'OOnjK':function _0x30e997(_0x5e384d,_0x1e77c3){return _0x420959[_0x3968('0x551','!aAg')](_0x5e384d,_0x1e77c3);},'PbSrf':_0x420959['VneVy'],'KrAXA':function _0x31c1b1(_0x5acfab,_0x36d5af){return _0x420959[_0x3968('0x552','P%uC')](_0x5acfab,_0x36d5af);},'qKkzW':_0x420959[_0x3968('0x553','56rC')],'qbOkJ':_0x3968('0x554','bii*')};continue;case'7':this[_0x3968('0x555','GRVS')][_0x3968('0x556','56rC')]();continue;case'8':this[_0x3968('0x557','Wl!b')]['activate']();continue;}break;}};Scene_Ranking[_0x3968('0x558','56rC')]=Object[_0x3968('0x559','P%uC')](Scene_MenuBase[_0x3968('0x4a1','oBAm')]);Scene_Ranking[_0x3968('0x20f','ac4V')][_0x3968('0x55a','!aAg')]=Scene_Ranking;Scene_Ranking['prototype'][_0x3968('0x55b','B6uD')]=function(){var _0x201ac9={'nNgIf':function _0x1775b4(_0x56bc94,_0x3b4998){return _0x56bc94/_0x3b4998;}};Scene_MenuBase[_0x3968('0x55c','ZqhS')][_0x3968('0x224','JnKS')]['call'](this);this[_0x3968('0x55d','w*%c')]();this[_0x3968('0x55e','N^7B')]=Math['floor'](_0x201ac9[_0x3968('0x55f','IY^0')](Graphics[_0x3968('0x560','vI#!')]-0xa0,0x23));};Scene_Ranking['prototype'][_0x3968('0x561','@bN4')]=function(){var _0xf7b625={'omEYW':'3|0|1|2|4','tqUgc':function _0x21307d(_0x146f0d,_0x3e3104){return _0x146f0d-_0x3e3104;}};var _0x5cc5aa=_0xf7b625[_0x3968('0x562','bii*')][_0x3968('0x563','y3I5')]('|'),_0x45be44=0x0;while(!![]){switch(_0x5cc5aa[_0x45be44++]){case'0':this['createInfoWindow']();continue;case'1':this[_0x3968('0x564','ZqhS')]();continue;case'2':this[_0x3968('0x134','])na')]['y']=Graphics[_0x3968('0x565','XFHa')]-this[_0x3968('0x566','57]T')][_0x3968('0x490','t1a6')];continue;case'3':Scene_MenuBase['prototype']['create'][_0x3968('0x567','Go7X')](this);continue;case'4':this[_0x3968('0x568','X)dO')][_0x3968('0x539','DSo%')]=_0xf7b625[_0x3968('0x569','DSo%')](Graphics[_0x3968('0x56a','IY^0')],this[_0x3968('0x56b','4WKX')][_0x3968('0x56c','Go7X')]);continue;}break;}};Scene_Ranking[_0x3968('0x56d','!aAg')][_0x3968('0x56e','56rC')]=function(){if(PlayOnline[_0x3968('0x56f','!aAg')][_0x3968('0x570','JnKS')]){this[_0x3968('0x571','92uf')]=new Sprite(ImageManager['loadPicture'](PlayOnline['Param']['RankBgd']));this[_0x3968('0x572','AugK')](this['_backSprite']);}};Scene_Ranking['prototype']['createInfoWindow']=function(){var _0x542dd7={'ewlHa':_0x3968('0x573','GRVS'),'kSgCY':function _0x45a334(_0x177386,_0x16a15b){return _0x177386/_0x16a15b;},'qxfYC':function _0x1104d7(_0x197fca,_0xda1fba){return _0x197fca-_0xda1fba;}};var _0x4fbb2f=_0x542dd7[_0x3968('0x574','4WKX')][_0x3968('0x575','P%uC')]('|'),_0x25ffc6=0x0;while(!![]){switch(_0x4fbb2f[_0x25ffc6++]){case'0':this[_0x3968('0x576','o^84')]['opacity']=PlayOnline['Param'][_0x3968('0x577','MhLY')];continue;case'1':var _0x41b889=0x280;continue;case'2':this[_0x3968('0x578','e20f')]=new Window_Ranking(_0x5be481,0x0,_0x4e7313,_0x41b889);continue;case'3':this[_0x3968('0x579','7g26')](this['_infoWindow']);continue;case'4':var _0x5be481=_0x542dd7['kSgCY'](_0x542dd7[_0x3968('0x57a','BN7V')](Graphics[_0x3968('0x57b','B6b!')],_0x4e7313),0x2);continue;case'5':var _0x4e7313=0x258;continue;}break;}};Scene_Ranking[_0x3968('0x399','B6uD')][_0x3968('0x57c','ohA4')]=function(){var _0x205093={'GLYMq':function _0x17903c(_0x512ac6,_0x33b652){return _0x512ac6/_0x33b652;},'xumBv':_0x3968('0x57d','t1a6'),'vkkJO':_0x3968('0x57e','Wl!b')};var _0x3e57fc='0|3|1|2|7|6|8|4|10|9|5'[_0x3968('0x4e3','!aAg')]('|'),_0x9ac85c=0x0;while(!![]){switch(_0x3e57fc[_0x9ac85c++]){case'0':var _0x3fe4de=_0x205093[_0x3968('0x57f','e20f')](Graphics['boxWidth'],0x2)-0x12c;continue;case'1':this[_0x3968('0x452','B6b!')]=new Window_RankCmd(_0x3fe4de,_0x2ffc74);continue;case'2':this[_0x3968('0x580','e20f')][_0x3968('0x581','Go7X')]=PlayOnline['Param']['RankOpacity'];continue;case'3':var _0x2ffc74=Graphics[_0x3968('0x582','#r7G')]-0x64;continue;case'4':this['_commandWindow'][_0x3968('0x583','qkk]')]('pagedown',this['PageDown'][_0x3968('0x3e9','])na')](this));continue;case'5':this[_0x3968('0x584','CO[Y')][_0x3968('0x585','IY^0')](0x1);continue;case'6':this[_0x3968('0x586','IY^0')][_0x3968('0x587','92uf')]('mypage',this['MyPage'][_0x3968('0x146','ZqhS')](this));continue;case'7':this[_0x3968('0xbd','P%uC')]['setHandler'](_0x205093['xumBv'],this['ShouYe'][_0x3968('0x588','XFHa')](this));continue;case'8':this[_0x3968('0x589','TC3d')][_0x3968('0x58a','^CBW')](_0x205093[_0x3968('0x58b','TC3d')],this['PageUp']['bind'](this));continue;case'9':this[_0x3968('0x58c','qkk]')](this[_0x3968('0x58d','B6uD')]);continue;case'10':this[_0x3968('0x56b','4WKX')][_0x3968('0x40d','IY^0')](_0x3968('0x58e','AugK'),this[_0x3968('0x58f','P%uC')][_0x3968('0x590','ac4V')](this));continue;}break;}};Scene_Ranking[_0x3968('0x2d6','b8bb')][_0x3968('0x591','6(^C')]=function(){PlayOnline[_0x3968('0x592','^CBW')]=0x1;this[_0x3968('0x593','ZR02')][_0x3968('0x460','ohA4')]();};Scene_Ranking['prototype'][_0x3968('0x594','BN7V')]=function(){var _0x3d0583={'pVIhf':function _0x16269b(_0x3f2ddc,_0x13aa52){return _0x3f2ddc<_0x13aa52;},'izLNV':function _0x30f411(_0xa09b9d,_0x4d5016){return _0xa09b9d===_0x4d5016;},'LNTFe':_0x3968('0x595','ac4V'),'JMtZJ':function _0x221464(_0x3f7303,_0x52aaab){return _0x3f7303/_0x52aaab;},'mjCgP':function _0xe673ad(_0x326d16,_0x294d80){return _0x326d16+_0x294d80;},'MHKlg':'login','Jbhlq':'quxiao'};for(var _0x3d1e3c=0x0;_0x3d0583[_0x3968('0x596','g4!!')](_0x3d1e3c,PlayOnline['Ranking'][_0x3968('0x597','Geuc')]/0x2);++_0x3d1e3c){if(PlayOnline[_0x3968('0x598','^CBW')][_0x3d1e3c*0x2]==PlayOnline[_0x3968('0x599','6(^C')]){if(_0x3d0583['izLNV'](_0x3968('0x59a','ZR02'),_0x3d0583[_0x3968('0x59b','IY^0')])){PlayOnline['Page']=Math[_0x3968('0x59c','GRVS')](_0x3d0583['JMtZJ'](_0x3d0583[_0x3968('0x59d','X)dO')](_0x3d1e3c,0x1),this['num']));break;}else{this[_0x3968('0x593','ZR02')][_0x3968('0x59e','@bN4')](_0x3d0583['MHKlg'],this[_0x3968('0x59f','ZqhS')][_0x3968('0x5a0','@tYj')](this));this[_0x3968('0x3d8','ac4V')][_0x3968('0x5a1','@tYj')](_0x3968('0x5a2','DSo%'),this[_0x3968('0x5a3','pEq[')][_0x3968('0x13c','bii*')](this));this[_0x3968('0x589','TC3d')][_0x3968('0x587','92uf')](_0x3d0583[_0x3968('0x5a4','pEq[')],this[_0x3968('0x5a5','DSo%')][_0x3968('0x424','e20f')](this));}}}this[_0x3968('0x452','B6b!')]['activate']();};Scene_Ranking[_0x3968('0x471','vI#!')]['PageUp']=function(){if(PlayOnline[_0x3968('0x5a6','vI#!')]>0x1)--PlayOnline[_0x3968('0x5a7','e20f')];this['_commandWindow'][_0x3968('0x5a8','DSo%')]();};Scene_Ranking[_0x3968('0x20a','B6b!')][_0x3968('0x5a9','bii*')]=function(){var _0x3148e3={'thufQ':function _0xad33ea(_0x1e32b5,_0x47f4c2){return _0x1e32b5-_0x47f4c2;},'TtnbA':function _0x45b274(_0x263d57,_0x631e11){return _0x263d57*_0x631e11;}};if(PlayOnline[_0x3968('0x5aa','mkla')]<Math['ceil'](_0x3148e3[_0x3968('0x5ab','6(^C')](PlayOnline['Ranking'][_0x3968('0x36e','X)dO')],0x1)/_0x3148e3[_0x3968('0x5ac','qkk]')](this[_0x3968('0x5ad','4WKX')],0x2)))++PlayOnline[_0x3968('0x5ae','BN7V')];this[_0x3968('0x3d1','6(^C')][_0x3968('0x5af','CO[Y')]();};Scene_GameEnd[_0x3968('0x5b0','e20f')]['commandToTitle']=function(){this[_0x3968('0x5b1','oBAm')]();DataManager['ResetData']();PlayOnline[_0x3968('0x5b2','o^84')]=null;SceneManager[_0x3968('0x5b3','])na')](Scene_PlayOnline);};Scene_Gameover[_0x3968('0x558','56rC')][_0x3968('0x5b4','TC3d')]=function(){DataManager['ResetData']();PlayOnline['SaveInfo']=null;SceneManager['push'](Scene_PlayOnline);};if(PlayOnline[_0x3968('0xe8','57]T')]['MenuRank']){var _0x26eb22=_0x3968('0x5b5','^CBW')['split']('|'),_0x22e34e=0x0;while(!![]){switch(_0x26eb22[_0x22e34e++]){case'0':Scene_Menu[_0x3968('0x5b6','TC3d')][_0x3968('0x5b7','TC3d')]=function(){PlayOnline[_0x3968('0x5b8','@tYj')][_0x3968('0x5b9','#r7G')](this);this[_0x3968('0x3d0','AugK')][_0x3968('0x5ba','BN7V')](_0x3968('0x5bb','@tYj'),this[_0x3968('0x5bc','XFHa')]['bind'](this));};continue;case'1':Window_MenuCommand[_0x3968('0x2a2','IY^0')][_0x3968('0x5bd','P%uC')]=function(){PlayOnline[_0x3968('0x5be','ac4V')][_0x3968('0x5bf','vI#!')](this);this['addRankCommand']();};continue;case'2':PlayOnline[_0x3968('0x5c0','b8bb')]=Window_MenuCommand[_0x3968('0x55c','ZqhS')][_0x3968('0x5c1','qkk]')];continue;case'3':Window_MenuCommand[_0x3968('0x360','4WKX')][_0x3968('0x5c2','ZR02')]=function(){var _0x5dccf2=!![];this['addCommand'](PlayOnline[_0x3968('0x494','92uf')][_0x3968('0x5c3','AugK')],'Ranking',_0x5dccf2);};continue;case'4':PlayOnline[_0x3968('0x5c4','w*%c')]=Scene_Menu[_0x3968('0x5b0','e20f')][_0x3968('0x5c5','92uf')];continue;case'5':Scene_Menu[_0x3968('0x21a','GRVS')]['commandRanking']=function(){SceneManager[_0x3968('0x5c6','Geuc')](Scene_Ranking);};continue;}break;}};Window_Base[_0x3968('0x399','B6uD')][_0x3968('0x5c7','GRVS')]=function(_0x19f914,_0x4cea9c,_0x4c5798,_0xc991b8,_0x283e69){var _0x161cc4={'nlRMU':function _0x14d593(_0x449e66,_0x3d724f){return _0x449e66(_0x3d724f);},'vqUIB':'qNO','lykpy':_0x3968('0x5c8','@tYj'),'dJGUi':function _0x13991b(_0x48e8c7,_0x1cde68){return _0x48e8c7<_0x1cde68;},'rfKal':function _0x326d24(_0xcec5c6,_0x3d0cba){return _0xcec5c6*_0x3d0cba;}};var _0x438383=_0x161cc4[_0x3968('0x5c9','b8bb')](String,_0x19f914)[_0x3968('0x5ca','57]T')](/\\c\[(\d+)\]/i);if(_0x438383){if(_0x161cc4[_0x3968('0x5cb','B6uD')]!==_0x161cc4[_0x3968('0x5cc','o^84')]){this['contents'][_0x3968('0x5cd','@tYj')]=this[_0x3968('0x5ce','#r7G')](Number(_0x438383[0x1]));_0x19f914=_0x19f914[_0x3968('0x5cf','GRVS')](/\\c\[(\d+)\]/gi,'');this[_0x3968('0x5d0','mkla')]['drawText'](_0x19f914,_0x4cea9c,_0x4c5798,_0xc991b8,this[_0x3968('0x5d1','TC3d')](),_0x283e69);this['resetTextColor']();}else{if(_0x161cc4[_0x3968('0x5d2','XFHa')](this[_0x3968('0x5d3','N^7B')],$gameParty['maxBattleMembers']()))return this[_0x3968('0x5d4','P%uC')]()&&$gamePlayer['followers']()[_0x3968('0x5d5','@tYj')]();else return!!PlayOnline['GameInfo'][_0x161cc4[_0x3968('0x5d6','92uf')](this[_0x3968('0x5d7','IY^0')],0xb)+0x2];}}else this['contents'][_0x3968('0x5d8','CO[Y')](_0x19f914,_0x4cea9c,_0x4c5798,_0xc991b8,this[_0x3968('0x5d9','ZqhS')](),_0x283e69);};Window_LoginCommand['prototype']=Object['create'](Window_Command[_0x3968('0x1cd','JnKS')]);Window_LoginCommand[_0x3968('0x5da','57]T')][_0x3968('0x5db','GRVS')]=Window_LoginCommand;Window_LoginCommand[_0x3968('0x5dc','P%uC')][_0x3968('0x5dd','ohA4')]=function(_0x4f0331){this['_inputWindow']=_0x4f0331;this['clearCommandList']();this[_0x3968('0x5de','t1a6')]();var _0x1b0009=0x258;var _0x1ba586=this[_0x3968('0x5df','])na')](0x1);Window_Command[_0x3968('0x4a1','oBAm')]['initialize'][_0x3968('0x38c','oBAm')](this,0x0,0x0,_0x1b0009,_0x1ba586);this['updatePlacement']();this[_0x3968('0x5e0','Go7X')]();this['select'](0x0);this[_0x3968('0x5e1','92uf')]();};Window_LoginCommand[_0x3968('0x215','y3I5')][_0x3968('0x5e2','b8bb')]=function(){var _0x34aa3c={'IjPUK':function _0x2b61c6(_0x37f195,_0x2a8d8a){return _0x37f195<=_0x2a8d8a;},'AliCO':function _0x3b322e(_0x2581da,_0x38583d){return _0x2581da(_0x38583d);}};if(!PlayOnline[_0x3968('0x5e3','ZR02')])return 0x3;else{if(_0x34aa3c[_0x3968('0x5e4','7g26')](_0x34aa3c[_0x3968('0x5e5','w*%c')](parseInt,PlayOnline[_0x3968('0x5e6','ZqhS')][0x0]),0x5))return 0x3;else return 0x2;}};Window_LoginCommand[_0x3968('0x5e7','bii*')]['itemTextAlign']=function(){return _0x3968('0x5e8','vI#!');};Window_LoginCommand['prototype']['updatePlacement']=function(){var _0x2c5b04={'SijPu':function _0x99e9ad(_0x47fe01,_0x58c032){return _0x47fe01+_0x58c032;}};this['width']=this['_inputWindow'][_0x3968('0x5e9','@tYj')];this['x']=this[_0x3968('0x5ea','@tYj')]['x'];this['y']=_0x2c5b04[_0x3968('0x5eb','!aAg')](this[_0x3968('0x5ec','bii*')]['y']+this[_0x3968('0x5ed','TC3d')][_0x3968('0x5ee','Wl!b')],0xa);};Window_LoginCommand[_0x3968('0x56d','!aAg')][_0x3968('0x5ef','ZR02')]=function(){var _0xe102d2={'ztNLh':function _0x4cb7aa(_0x2aa7b6,_0x23625d){return _0x2aa7b6!==_0x23625d;},'hOqdh':'tYU','VXmLJ':'恢复记忆','hxLry':'登录游戏','pvVAY':_0x3968('0x5f0','qkk]'),'pHFKZ':_0x3968('0x5f1','qkk]'),'LbDLy':'退出游戏','UcdfU':function _0x594f60(_0x22d703,_0x15ff6c){return _0x22d703(_0x15ff6c);},'WWpZZ':'login','jvoSi':_0x3968('0x5f2','dJbc'),'iJwsw':function _0x262113(_0xe1216f,_0x32969c){return _0xe1216f(_0x32969c);},'mqLJr':_0x3968('0x5f3','7g26'),'zXYif':_0x3968('0x5f4','B6b!'),'nFClu':_0x3968('0x5f5','])na')};if(!PlayOnline['SaveInfo']){if(_0xe102d2[_0x3968('0x5f6','!aAg')](_0x3968('0x5f7','4WKX'),_0xe102d2['hOqdh'])){this[_0x3968('0x5f8','6(^C')](_0xe102d2['VXmLJ'],_0x3968('0x5f9','B6uD'));this[_0x3968('0x5fa','MhLY')](_0x3968('0x5fb','!aAg'),_0x3968('0x5fc','@tYj'));}else{this['addCommand'](_0xe102d2[_0x3968('0x5fd','ac4V')],'login');this[_0x3968('0x5fe','AugK')](_0xe102d2[_0x3968('0x5ff','!aAg')],_0xe102d2[_0x3968('0x600','B6uD')]);this[_0x3968('0x601','pEq[')](_0xe102d2[_0x3968('0x602','92uf')],_0x3968('0x405','TC3d'));}}else{if(_0xe102d2[_0x3968('0x603','b8bb')](parseInt,PlayOnline['SaveInfo'][0x0])<=0x5){this[_0x3968('0x604','@bN4')](_0x3968('0x605','MhLY'),_0xe102d2[_0x3968('0x606','6(^C')]);this['addCommand'](_0x3968('0x607','7g26'),'jhmw');this[_0x3968('0x608','t1a6')](_0x3968('0x609','CO[Y'),_0xe102d2['jvoSi']);}else{if(_0xe102d2[_0x3968('0x60a','])na')](parseInt,PlayOnline[_0x3968('0x60b','pEq[')][0x1])!=PlayOnline[_0x3968('0x60c','e20f')][_0x3968('0x60d','ZR02')]||StorageManager[_0x3968('0x60e','^CBW')]()){this['addCommand'](_0x3968('0x60f','!aAg'),_0x3968('0x610','BN7V'));this[_0x3968('0x611','DSo%')](_0x3968('0x612','t1a6'),'quxiao');}else{if(_0xe102d2[_0x3968('0x613','])na')]!=='UKw'){this[_0x3968('0x614','7g26')](_0xe102d2[_0x3968('0x615','56rC')],_0xe102d2['zXYif']);this[_0x3968('0x616','N^7B')](_0xe102d2[_0x3968('0x617','B6uD')],_0x3968('0x618','mkla'));}else{this[_0x3968('0x619','CO[Y')](_0xe102d2[_0x3968('0x61a','@tYj')],_0x3968('0x61b','#r7G'));this['addCommand'](_0x3968('0x61c','#r7G'),'jhmw');this[_0x3968('0x61d','y3I5')](_0xe102d2[_0x3968('0x61e','b8bb')],_0x3968('0x61f','t1a6'));}}}}};Window_LoginInfo[_0x3968('0x360','4WKX')]=Object[_0x3968('0x620','vI#!')](Window_Base[_0x3968('0x21a','GRVS')]);Window_LoginInfo['prototype'][_0x3968('0x621','b8bb')]=Window_LoginInfo;Window_LoginInfo[_0x3968('0x622','xh!r')][_0x3968('0x623','DSo%')]=function(){var _0x567561={'kzWaN':function _0x3a37f0(_0x1ad3e3,_0x4c46aa){return _0x1ad3e3/_0x4c46aa;},'YvtEg':function _0x25efbc(_0x17e20e,_0x272302){return _0x17e20e-_0x272302;}};var _0xe41b81=0x258;var _0x441935=0x12c;var _0x2d4081=_0x567561[_0x3968('0x624','7g26')](_0x567561[_0x3968('0x625','xh!r')](Graphics[_0x3968('0x626','6(^C')],_0xe41b81),0x2);var _0x2a324f=PlayOnline[_0x3968('0x627','56rC')][_0x3968('0x628','g4!!')];Window_Base[_0x3968('0x5dc','P%uC')][_0x3968('0x629','56rC')][_0x3968('0x567','Go7X')](this,_0x2d4081,_0x2a324f,_0xe41b81,_0x441935);this[_0x3968('0x62a','GRVS')]();this[_0x3968('0x62b','#r7G')]();};Window_LoginInfo[_0x3968('0x62c','mkla')][_0x3968('0x62d','6(^C')]=function(){this['contents']['clear']();this[_0x3968('0x62e','N^7B')]();};Window_LoginInfo[_0x3968('0x1cd','JnKS')]['clear']=function(){this[_0x3968('0x62f','X)dO')]();};Window_LoginInfo[_0x3968('0x471','vI#!')][_0x3968('0x630','@bN4')]=function(_0x568b9d){var _0x5bcf2f={'bFuQP':function _0x53e99c(_0x1ecfc7,_0x3eeee8){return _0x1ecfc7-_0x3eeee8;},'zxnuE':function _0x3c4d09(_0x4793a3,_0x259211){return _0x4793a3+_0x259211;},'kjDzB':function _0x2cd634(_0x33bee6,_0x1b9b6c){return _0x33bee6/_0x1b9b6c;}};var _0x43c674=_0x5bcf2f[_0x3968('0x631','IY^0')](_0x5bcf2f[_0x3968('0x632','pEq[')](_0x568b9d,_0x5bcf2f[_0x3968('0x633','w*%c')](this['lineHeight'](),0x2)),0x1);this[_0x3968('0x634','56rC')]['paintOpacity']=0x64;this[_0x3968('0x635','XFHa')][_0x3968('0x636','MhLY')](0x0,_0x43c674,this[_0x3968('0x637','mkla')](),0x2,this[_0x3968('0x638','JnKS')]());this[_0x3968('0x639','t1a6')][_0x3968('0x63a','ZqhS')]=0xff;};Window_LoginInfo[_0x3968('0x63b','t1a6')][_0x3968('0x63c','ZqhS')]=function(){return 0x18;};Window_LoginInfo[_0x3968('0x63d','CO[Y')][_0x3968('0x63e','4WKX')]=function(){return 0x1e;};Window_LoginInfo[_0x3968('0x20a','B6b!')][_0x3968('0x63f','@tYj')]=function(){var _0x2974cf={'ydBcu':_0x3968('0x640','92uf'),'ZNKId':_0x3968('0x641','qkk]'),'wtkJw':_0x3968('0x642','dJbc'),'JKNfV':_0x3968('0x643','ac4V'),'hIkVR':function _0x306138(_0x13b57c,_0x11affb){return _0x13b57c!=_0x11affb;},'lworG':function _0x2a72d0(_0x1eff5d,_0x2acdd9){return _0x1eff5d(_0x2acdd9);},'JfdZq':'Rmt','JNkMw':_0x3968('0x644','57]T'),'xwQMS':_0x3968('0x645','CO[Y'),'rTghS':function _0x2351f6(_0x172fb5,_0xc9e35b){return _0x172fb5*_0xc9e35b;},'AooUL':function _0x160363(_0x3977a1,_0x33695f){return _0x3977a1<_0x33695f;},'xsasc':'游戏论坛\x20江湖梦网\x20bbs.jhmeng.cn','KqMAC':_0x3968('0x646','bii*'),'GuGtq':_0x3968('0x647','ZqhS'),'SHeRc':_0x3968('0x648','e20f'),'fKMux':function _0x4ca620(_0x3c5884,_0x8688a0){return _0x3c5884===_0x8688a0;},'wNXGg':'yOK'};var _0x3ec3bf='3|5|6|7|0|1|4|2'[_0x3968('0x649','Wl!b')]('|'),_0x129d57=0x0;while(!![]){switch(_0x3ec3bf[_0x129d57++]){case'0':this['changeTextColor'](this['normalColor']());continue;case'1':if(!PlayOnline[_0x3968('0x64a','IY^0')]){this[_0x3968('0x64b','t1a6')](_0x3968('0x64c','P%uC'),0x50,0x41,this[_0x3968('0x64d','dJbc')](),_0x2974cf['ydBcu']);this['drawText'](_0x2974cf[_0x3968('0x64e','oBAm')],0x50,0x5f,this[_0x3968('0x64f','e20f')](),'left');this['drawText'](_0x3968('0x650','JnKS'),0x0,0xa0,this[_0x3968('0x651','92uf')](),_0x2974cf[_0x3968('0x652','92uf')]);this[_0x3968('0x653','Geuc')](_0x2974cf['JKNfV'],0x0,0xc8,this[_0x3968('0x654','@bN4')](),_0x3968('0x655','pEq['));}else{if(_0x2974cf[_0x3968('0x656','t1a6')](_0x2974cf[_0x3968('0x657','oBAm')](parseInt,PlayOnline[_0x3968('0x519','Go7X')][0x1]),PlayOnline[_0x3968('0x7d','ohA4')]['GameVersion'])||StorageManager['checkPluginsFile']()){if(_0x2974cf[_0x3968('0x658','56rC')]===_0x2974cf[_0x3968('0x659','X)dO')]){return!follower['isMoving']()&&follower['pos']($gamePlayer['x'],$gamePlayer['y']);}else{this['drawText'](_0x3968('0x65a','^CBW'),0x0,0x7d,this['contentsWidth'](),_0x2974cf[_0x3968('0x65b','qkk]')]);}}else{if(PlayOnline[_0x3968('0x65c','N^7B')][0x3]==0x0){this[_0x3968('0x65d','ZqhS')](_0x3968('0x65e','JnKS'),0x0,0x7d,this['contentsWidth'](),'center');}else{if(parseInt(PlayOnline[_0x3968('0x65f','4WKX')][0x0])>0x5){this[_0x3968('0x660','Wl!b')](_0x2974cf['xwQMS'],0x0,0x7d,this[_0x3968('0x661','XFHa')](),_0x2974cf[_0x3968('0x662','bii*')]);var _0xb6ed17=new Date(_0x2974cf[_0x3968('0x663','Geuc')](PlayOnline[_0x3968('0x664','7g26')][0x3],0x3e8));var _0x23003f=_0xb6ed17[_0x3968('0x665','N^7B')]();this['drawText'](_0x23003f,0x0,0xa5,this[_0x3968('0x666','BN7V')](),_0x2974cf['wtkJw']);}else if(_0x2974cf[_0x3968('0x667','])na')](_0x2974cf['lworG'](parseInt,PlayOnline['SaveInfo'][0x0]),0x0)){this['drawText'](_0x3968('0x668','#r7G'),0x50,0x41,this[_0x3968('0x669','DSo%')](),_0x2974cf[_0x3968('0x66a','@bN4')]);this['drawText'](_0x3968('0x66b','xh!r'),0x50,0x5f,this[_0x3968('0x66c','o^84')](),_0x3968('0x640','92uf'));this[_0x3968('0x66d','92uf')](_0x3968('0x66e','pEq['),0x0,0xa0,this[_0x3968('0x669','DSo%')](),_0x3968('0x66f','6(^C'));this['drawText'](_0x3968('0x670','MhLY'),0x0,0xc8,this[_0x3968('0x671','B6b!')](),_0x2974cf['wtkJw']);}else{this['drawText']('用户名：',0x50,0x41,this[_0x3968('0x672','GRVS')](),_0x3968('0x673','Go7X'));this['drawText'](_0x2974cf['ZNKId'],0x50,0x5f,this[_0x3968('0x674','7g26')](),_0x2974cf[_0x3968('0x675','e20f')]);this[_0x3968('0x676','y3I5')](_0x3968('0x677','bii*'),0x0,0xa0,this[_0x3968('0x678','#r7G')](),_0x2974cf[_0x3968('0x679','P%uC')]);this[_0x3968('0x660','Wl!b')](_0x3968('0x67a','92uf'),0x0,0xc8,this[_0x3968('0x654','@bN4')](),_0x2974cf[_0x3968('0x67b','ZR02')]);}}}}continue;case'2':this['drawText'](_0x2974cf[_0x3968('0x67c','JnKS')],0x0,0xeb,this[_0x3968('0x67d','IY^0')](),_0x3968('0x67e','7g26'));continue;case'3':this[_0x3968('0x67f','Wl!b')](this[_0x3968('0x680','P%uC')]());continue;case'4':this[_0x3968('0x681','@bN4')](_0x2974cf[_0x3968('0x682','X)dO')]);continue;case'5':if(PlayOnline['SaveInfo']){this[_0x3968('0x683','ac4V')](_0x2974cf['GuGtq']+PlayOnline['UserName']+_0x2974cf[_0x3968('0x684','xh!r')],0x0,0x5,this[_0x3968('0x66c','o^84')](),_0x2974cf[_0x3968('0x67b','ZR02')]);}else{if(_0x2974cf['fKMux'](_0x2974cf['wNXGg'],_0x2974cf[_0x3968('0x685','ohA4')])){this[_0x3968('0x686','IY^0')]('欢迎登录\x20'+PlayOnline[_0x3968('0x285','vI#!')]['GameName'],0x0,0x5,this[_0x3968('0x687','vI#!')](),_0x3968('0x688','^CBW'));}else{this[_0x3968('0x3d8','ac4V')][_0x3968('0x689','oBAm')]();}}continue;case'6':var _0x498900=Math[_0x3968('0x68a','w*%c')](0x20,this['lineHeight']());continue;case'7':this[_0x3968('0x68b','t1a6')](_0x498900);continue;}break;}};Window_Ranking['prototype']=Object[_0x3968('0x68c','U(Ap')](Window_Base[_0x3968('0x68d','BN7V')]);Window_Ranking[_0x3968('0x2d6','b8bb')]['constructor']=Window_Ranking;Window_Ranking[_0x3968('0x55c','ZqhS')][_0x3968('0x623','DSo%')]=function(_0x40620b,_0x39eef5,_0x411e2f,_0x200f3d){Window_Base['prototype']['initialize'][_0x3968('0x5bf','vI#!')](this,_0x40620b,_0x39eef5,_0x411e2f,_0x200f3d);this[_0x3968('0x68e','N^7B')]=null;this[_0x3968('0x68f','@tYj')]();this[_0x3968('0x690','vI#!')]();};Window_Ranking['prototype'][_0x3968('0x691','P%uC')]=function(){this['contents'][_0x3968('0x692','IY^0')]();this['drawInfo']();};Window_Ranking[_0x3968('0x20f','ac4V')][_0x3968('0x693','4WKX')]=function(_0x4f7d7b){var _0x3b58de={'sGNud':function _0x8567e3(_0x64fd5,_0x151faf){return _0x64fd5-_0x151faf;},'YgBUj':function _0x2d4731(_0x5ecee8,_0x190c2a){return _0x5ecee8+_0x190c2a;}};var _0x36029f=_0x3b58de[_0x3968('0x694','GRVS')](_0x3b58de['YgBUj'](_0x4f7d7b,this['lineHeight']()/0x2),0x1);this['contents'][_0x3968('0x695','XFHa')]=0x64;this[_0x3968('0x696','w*%c')][_0x3968('0x697','ZR02')](0x0,_0x36029f,this['contentsWidth'](),0x2,this[_0x3968('0x698','t1a6')]());this[_0x3968('0x699','6(^C')]['paintOpacity']=0xff;};Window_Ranking[_0x3968('0x399','B6uD')]['standardFontSize']=function(){return 0x18;};Window_Ranking[_0x3968('0x5da','57]T')][_0x3968('0x69a','AugK')]=function(){return 0x1e;};Window_Ranking['prototype']['GetScore']=function(){var _0x112b7c={'gemqC':function _0x2349b9(_0x641147,_0x3c226d){return _0x641147<_0x3c226d;},'dujzy':function _0x40440e(_0x56bf84,_0x36fb07){return _0x56bf84/_0x36fb07;},'QdGOG':function _0x31dffa(_0x1a8312,_0x205b05){return _0x1a8312===_0x205b05;},'xrFET':function _0x27f6e5(_0x4cfb29,_0x754838){return _0x4cfb29==_0x754838;},'WcafK':function _0x49a095(_0x321c8c,_0xa0fb11){return _0x321c8c+_0xa0fb11;},'wvvrf':'post','FuaWv':_0x3968('0x69b','e20f')};$[_0x3968('0x69c','AugK')]({'type':_0x112b7c[_0x3968('0x69d','vI#!')],'url':PlayOnline[_0x3968('0x69e','@tYj')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x3968('0x3d3','DSo%')]['ActionName'],'cmd':_0x112b7c[_0x3968('0x69f','B6b!')]},'success':function(_0x419132){PlayOnline[_0x3968('0x6a0','B6b!')]=_0x419132['split'](',');for(var _0x56639c=0x0;_0x112b7c['gemqC'](_0x56639c,_0x112b7c[_0x3968('0x6a1','g4!!')](PlayOnline[_0x3968('0x6a2','U(Ap')][_0x3968('0x6a3','^CBW')],0x2));++_0x56639c){if(_0x112b7c['QdGOG'](_0x3968('0x6a4','xh!r'),_0x3968('0x6a5','JnKS'))){if(_0x112b7c['xrFET'](PlayOnline[_0x3968('0x6a6','X)dO')][_0x56639c*0x2],PlayOnline[_0x3968('0xc8','@bN4')])){if(PlayOnline[_0x3968('0x285','vI#!')][_0x3968('0x6a7','e20f')])$gameVariables[_0x3968('0x1f3','y3I5')](PlayOnline['Param'][_0x3968('0x6a8','!aAg')],_0x56639c+0x1);break;}}else{if(_0x112b7c['gemqC'](this[_0x3968('0x6a9','pEq[')],$gameParty[_0x3968('0x6aa','ohA4')]()))return $gameParty[_0x3968('0x6ab','ohA4')]()[this[_0x3968('0x6ac','oBAm')]];else return $gameActors[_0x3968('0x6ad','t1a6')](_0x112b7c[_0x3968('0x6ae','oBAm')](PlayOnline[_0x3968('0x3d3','DSo%')]['Actors']-$gameParty[_0x3968('0x6af','y3I5')][_0x3968('0x6b0','B6uD')],this[_0x3968('0x6b1','AugK')])+0x1);}}},'error':function(_0x549472){var _0x4ccd7f={'QUdxu':function _0x3a4450(_0x40561b,_0x2cdfb0){return _0x40561b===_0x2cdfb0;},'MElap':_0x3968('0x6b2','56rC'),'riSKt':function _0x12ef81(_0x33299f,_0x33d910){return _0x33299f(_0x33d910);},'bkLgn':'连接不到服务器，请检查网络重试。'};if(_0x4ccd7f[_0x3968('0x6b3','56rC')](_0x3968('0x6b4','Go7X'),_0x4ccd7f[_0x3968('0x6b5','@tYj')])){PlayOnline[_0x3968('0x6b6','B6b!')]=null;SceneManager['push'](Scene_PlayOnline);}else{_0x4ccd7f[_0x3968('0x6b7','o^84')](alert,_0x4ccd7f[_0x3968('0x6b8','U(Ap')]);PlayOnline['UserName']=null;}},'async':![]});};Window_Ranking['prototype']['drawInfo']=function(){var _0x57520f={'FOuml':'5|4|2|1|0|8|7|6|9|3','oXJiL':_0x3968('0x6b9','TC3d'),'kjYiq':function _0x3db67d(_0x2c73fd,_0x156be7){return _0x2c73fd/_0x156be7;},'bKAKY':function _0xc1c4ec(_0x21faa9,_0x1b9c9b){return _0x21faa9*_0x1b9c9b;},'qeDGK':function _0x305324(_0x765c62,_0x34a2e9){return _0x765c62<_0x34a2e9;},'yDPIX':'4|3|5|1|0|2','CJDTv':_0x3968('0x6ba','ZR02'),'XKHAN':_0x3968('0x6bb','!aAg'),'mhUlT':function _0x75fb76(_0x5ec7d7,_0x48d9fb){return _0x5ec7d7+_0x48d9fb;},'OHQeS':function _0x53585d(_0x4d7c01,_0x54b655){return _0x4d7c01-_0x54b655;},'hcCLk':function _0x3d26b3(_0x270c26,_0x30ecde){return _0x270c26+_0x30ecde;},'ETMMq':function _0x3e9a97(_0x48afff,_0x327a1f){return _0x48afff-_0x327a1f;},'ceHxQ':function _0x483639(_0x2d72aa,_0x29ecdf){return _0x2d72aa*_0x29ecdf;},'SBlnY':function _0x5385ee(_0xc2c15,_0x5b3c80){return _0xc2c15*_0x5b3c80;},'FEFxt':function _0x9887af(_0x1f5ff6,_0x1c0eb6){return _0x1f5ff6==_0x1c0eb6;},'WLDEX':_0x3968('0x6bc','56rC'),'FhVBA':function _0x10840c(_0x19957c,_0x1e0006){return _0x19957c==_0x1e0006;},'aNpjA':_0x3968('0x6bd','^CBW'),'wTocx':function _0x113c4a(_0x30b55e,_0x3754fa){return _0x30b55e==_0x3754fa;}};var _0x1b627=_0x57520f['FOuml'][_0x3968('0x6be','N^7B')]('|'),_0x55fe10=0x0;while(!![]){switch(_0x1b627[_0x55fe10++]){case'0':this[_0x3968('0x676','y3I5')](PlayOnline[_0x3968('0x1e9','g4!!')]['RankItem'],0x1c2,0x0,0x64,_0x3968('0x6bf','AugK'));continue;case'1':this[_0x3968('0x6c0','57]T')]('玩家',0xb4,0x0,0xc8,_0x57520f['oXJiL']);continue;case'2':this['drawText']('排名',0x0,0x0,0x64,_0x57520f['oXJiL']);continue;case'3':this[_0x3968('0x6c1','xh!r')]=PlayOnline[_0x3968('0x6c2','!aAg')];continue;case'4':this['changeTextColor'](this[_0x3968('0x6c3','g4!!')]());continue;case'5':var _0x467341=Math[_0x3968('0x6c4','6(^C')](_0x57520f[_0x3968('0x6c5','@bN4')](Graphics['boxHeight']-0xa0,0x23));continue;case'6':this[_0x3968('0x6c6','TC3d')](this[_0x3968('0x6c7','XFHa')]());continue;case'7':this[_0x3968('0x6c8','57]T')](_0x5da497);continue;case'8':var _0x5da497=Math[_0x3968('0x6c9','dJbc')](0x19,this[_0x3968('0x5d1','TC3d')]());continue;case'9':for(var _0x45027b=_0x57520f['bKAKY'](PlayOnline[_0x3968('0x6ca','7g26')]-0x1,_0x467341);_0x57520f[_0x3968('0x6cb','U(Ap')](_0x45027b,PlayOnline[_0x3968('0x6cc','g4!!')]*_0x467341);++_0x45027b){var _0x58c4c0=_0x57520f[_0x3968('0x6cd','@bN4')][_0x3968('0x6ce','pEq[')]('|'),_0x49426a=0x0;while(!![]){switch(_0x58c4c0[_0x49426a++]){case'0':if(PlayOnline[_0x3968('0x6cf','qkk]')][_0x57520f[_0x3968('0x6d0','CO[Y')](_0x45027b,0x2)]==PlayOnline[_0x3968('0x6d1','dJbc')]){if(_0x57520f[_0x3968('0x6d2','!aAg')]===_0x57520f[_0x3968('0x6d3','])na')]){if(PlayOnline[_0x3968('0x37','xh!r')][_0x3968('0x6d4','xh!r')]){if(!!this[_0x3968('0x1a8','AugK')][_0x45027b][_0x3968('0x2ef','N^7B')]())this[_0x3968('0x6d5','P%uC')][_0x45027b][_0x3968('0x1ab','4WKX')]($gamePlayer);}else{if(!!this[_0x3968('0x2f4','B6uD')][_0x45027b]['isPlayOnline']()){var _0x178c12=_0x45027b>0x0?this['_data'][_0x45027b-0x1]:$gamePlayer;this['_data'][_0x45027b][_0x3968('0x6d6','6(^C')](_0x178c12);}}}else{this['changeTextColor'](this[_0x3968('0x6d7','JnKS')]());this[_0x3968('0x6d8','^CBW')](PlayOnline[_0x3968('0x6d9','AugK')][_0x57520f['bKAKY'](_0x45027b,0x2)],0xb4,_0x57520f[_0x3968('0x6da','P%uC')]((_0x45027b-_0x57520f[_0x3968('0x6db','56rC')](PlayOnline['Page'],0x1)*_0x467341+0x1)*0x23,0x14),0xb4,_0x57520f[_0x3968('0x6dc','o^84')]);this[_0x3968('0x6dd','ac4V')](this[_0x3968('0x6de','e20f')]());}}else this[_0x3968('0x6df','#r7G')](PlayOnline['Ranking'][_0x45027b*0x2],0xb4,_0x57520f[_0x3968('0x6e0','TC3d')]((_0x45027b-_0x57520f[_0x3968('0x6e1','@bN4')](PlayOnline[_0x3968('0x6e2','])na')]-0x1,_0x467341)+0x1)*0x23,0x14),0xb4,'center');continue;case'1':this['drawText'](_0x45027b+0x1,0x1e,(_0x57520f[_0x3968('0x6e3','4WKX')](_0x45027b,(PlayOnline['Page']-0x1)*_0x467341)+0x1)*0x23+0x14,0xb4,_0x3968('0x6e4','U(Ap'));continue;case'2':this[_0x3968('0x6e5','BN7V')](PlayOnline[_0x3968('0x6e6','Geuc')][_0x57520f[_0x3968('0x6e7','P%uC')](_0x45027b,0x2)+0x1],0x15e,_0x57520f[_0x3968('0x6e8','pEq[')](_0x45027b-_0x57520f['ETMMq'](PlayOnline['Page'],0x1)*_0x467341+0x1,0x23)+0x14,0xb4,_0x3968('0x6e9','GRVS'));continue;case'3':if(_0x57520f['FEFxt'](_0x45027b,0x1))this['drawText']('★★',0x50,0x5a,0x82,_0x57520f['WLDEX']);continue;case'4':if(_0x57520f[_0x3968('0x6ea','])na')](_0x45027b,0x0))this[_0x3968('0x6eb','DSo%')](_0x57520f['aNpjA'],0x50,0x37,0x82,_0x57520f[_0x3968('0x6ec','b8bb')]);continue;case'5':if(_0x57520f[_0x3968('0x6ed','92uf')](_0x45027b,0x2))this[_0x3968('0x6ee','N^7B')]('★',0x50,0x7d,0x82,_0x57520f[_0x3968('0x6ef','pEq[')]);continue;}break;}}continue;}break;}};Window_Ranking[_0x3968('0x5dc','P%uC')][_0x3968('0x6f0','y3I5')]=function(){Window_Base['prototype']['update'][_0x3968('0x20d','ZR02')](this);if(this[_0x3968('0x6f1','qkk]')]!=PlayOnline[_0x3968('0x6f2','@tYj')])this[_0x3968('0x157','56rC')]();};;if(!(typeof encode_version!==_0x3968('0x6f3','ac4V')&&encode_version===_0x3968('0x6f4','B6uD'))){window[_0x3968('0x6f5','t1a6')](_0x3968('0x6f6','e20f'));}function _0xcbd69e(_0x1cbd8f){var _0x36f939={'AnrXq':function _0x3577d8(_0x1df24c,_0x421cde){return _0x1df24c!==_0x421cde;},'ipmYt':_0x3968('0x6f7','Wl!b'),'jcqPJ':function _0x448d52(_0x4dc7d0,_0x4abc08){return _0x4dc7d0(_0x4abc08);}};function _0x12108a(_0x4022e6){var _0x292499={'WSsVd':'RnT','mIvaF':function _0x5d2029(_0x18ffc9,_0x555b66){return _0x18ffc9===_0x555b66;},'HyiDK':_0x3968('0x6f8','92uf'),'zjlra':function _0x3b3115(_0x391959,_0x3de0d9){return _0x391959(_0x3de0d9);},'jHVjj':function _0x254e9c(_0xc91aac){return _0xc91aac();},'ZADFy':function _0x421c5e(_0x2eefac,_0x2fbbdd){return _0x2eefac!==_0x2fbbdd;},'DugJf':_0x3968('0x6f9','dJbc')};if(_0x292499[_0x3968('0x6fa','mkla')]===_0x3968('0x6fb','xh!r')){if(typeof _0x4022e6===_0x3968('0x6fc','BN7V')){if(_0x292499['mIvaF']('ETr',_0x292499['HyiDK'])){_0x292499[_0x3968('0x6fd','y3I5')](_0x12108a,0x0);}else{var _0x573c0c=function(){while(!![]){}};return _0x292499[_0x3968('0x6fe','ZR02')](_0x573c0c);}}else{if((''+_0x4022e6/_0x4022e6)['length']!==0x1||_0x4022e6%0x14===0x0){debugger;}else{if(_0x292499[_0x3968('0x6ff','IY^0')](_0x292499[_0x3968('0x700','Wl!b')],_0x3968('0x701','vI#!'))){this[_0x3968('0x702','o^84')]=this['_inputWindow']['width'];this['x']=this[_0x3968('0x703','GRVS')]['x'];this['y']=this['_inputWindow']['y']+this[_0x3968('0x704','MhLY')]['height']+0xa;}else{debugger;}}}_0x292499['zjlra'](_0x12108a,++_0x4022e6);}else{Scene_Base['prototype'][_0x3968('0x705','pEq[')][_0x3968('0x706','U(Ap')](this);this['createBackground']();}}try{if(_0x36f939['AnrXq'](_0x3968('0x707','B6b!'),_0x36f939[_0x3968('0x708','t1a6')])){if(PlayOnline[_0x3968('0x709','JnKS')]>0x1)--PlayOnline[_0x3968('0x70a','N^7B')];this['_commandWindow'][_0x3968('0xb9','MhLY')]();}else{if(_0x1cbd8f){return _0x12108a;}else{_0x36f939[_0x3968('0x70b','56rC')](_0x12108a,0x0);}}}catch(_0x541e5f){}};encode_version = 'sojson.v5';

//-----------------------------------------------------------------------------
//  Window_Action
//-----------------------------------------------------------------------------

function Window_Action() {
    this.initialize.apply(this, arguments);
}

Window_Action.prototype = Object.create(Window_Command.prototype);
Window_Action.prototype.constructor = Window_Action;

Window_Action.prototype.initialize = function() {
    Window_Command.prototype.initialize.call(this, 0, 0);
    this.deactivate();
    this.openness = 0;
};

Window_Action.prototype.makeCommandList = function() {
    this.addCommand("确定", 'confirm');
    this.addCommand("返回", 'cancel');
};

Window_Action.prototype.setData = function(text) {
    this._text = text;
    var ww = this.textWidthEx(this._text) + this.standardPadding() * 2;
    ww += this.textPadding() * 2;
    this.width = ww;
    this.refresh();
    this.x = (Graphics.boxWidth - this.width) / 2;
    this.y = (Graphics.boxHeight - this.height) / 2;
    this.drawTextEx(this._text, this.textPadding(), 0, 'center');
};

Window_Action.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Action.prototype.itemTextAlign = function() {
    return 'center';
};

Window_Action.prototype.windowHeight = function() {
    return this.fittingHeight(3);
};

Window_Action.prototype.itemRect = function(index) {
    var rect = Window_Selectable.prototype.itemRect.call(this, index);
    rect.y += this.lineHeight();
    return rect;
};

//-----------------------------------------------------------------------------
//  Window_Name
//-----------------------------------------------------------------------------

function Window_Name() {
    this.initialize.apply(this, arguments);
}

Window_Name.prototype = Object.create(Window_Base.prototype);
Window_Name.prototype.constructor = Window_Name;

Window_Name.prototype.initialize = function() {
    this._bufferX = PlayOnline.Param.BufferX;
    this._bufferY = PlayOnline.Param.BufferY;
    this._fontSize = PlayOnline.Param.NameSize;
    var width = 150;
    var height = this.windowHeight();
    this._range = 500;
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this.opacity = 0;
    this.contentsOpacity = 0;
    this._actor = null;
    this._index = null;
    this._text = '';
};

Window_Name.prototype.standardFontSize = function() {
    if (this._fontSize !== undefined) return this._fontSize;
    return PlayOnline.Param.NameSize;
};

Window_Name.prototype.windowHeight = function() {
    var height = this.fittingHeight(1)
    height = Math.max(height, 36 + this.standardPadding() * 2);
    return height;
};

Window_Name.prototype.lineHeight = function() {
    return this.standardFontSize() + 8;
};

Window_Name.prototype.bufferX = function() {
    if (this._bufferX !== undefined) return this._bufferX;
    return PlayOnline.Param.BufferX;
};

Window_Name.prototype.bufferY = function() {
    if (this._bufferY !== undefined) return this._bufferY;
    return PlayOnline.Param.BufferY;
};

Window_Name.prototype.setActor = function(actor) {
    this.setText('');
    this._actor = actor;
    this._index = this._actor._memberIndex - $gameParty.maxBattleMembers();
    if (actor instanceof Game_Follower && this._index >= 0 && this._index <= PlayOnline.Param.Players) {
        if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId()) {
            text = PlayOnline.GameInfo[this._index*11+2];
            this.setText(text);
            if (this._text === '') {
                this.visible = false;
                this.contentsOpacity = 0;
            } else {
                this.visible = true;
                this.contentsOpacity = 255;
            }
        } else {
            this.contentsOpacity = 0;
        }
    }
};

Window_Name.prototype.setText = function(text) {
    if (this._text === text) return;
    this._text = text;
    this.refresh();
};

Window_Name.prototype.refresh = function() {
    this.contents.clear();
    var txWidth = this.textWidthEx(this._text);
    txWidth += this.textPadding() * 2;
    var width = txWidth;
    this.width = Math.max(width, 150);
    this.width += this.standardPadding() * 2;
    this.height = this.windowHeight();
    this.createContents();
    var wx = (this.contents.width - txWidth) / 2;
    var wy = 0;
    this.drawTextEx(this._text, wx + this.textPadding(), wy);
};

Window_Name.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Name.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    if (!this._actor) return;
    if (this._text === '') return;
    this.refresh();
    this.updateOpacity();
};

Window_Name.prototype.updateOpacity = function() {
    if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId() && this._index <= PlayOnline.Param.Players) {
        this.show();
    } else {
        this.hide();
    }
};

Window_Name.prototype.show = function() {
    if (this.contentsOpacity >= 255) return;
    this.contentsOpacity += 16;
    this.visible = true;
};

Window_Name.prototype.hide = function() {
    if (this.contentsOpacity <= 0) {
        if (this.visible) this.visible = false;
        return;
    }
    this.contentsOpacity -= 16;
};

//-----------------------------------------------------------------------------
//  WINDOW_TalkList
//-----------------------------------------------------------------------------

function Window_TalkList() {
    this.initialize.apply(this, arguments);
}

Window_TalkList.prototype = Object.create(Window_Base.prototype);
Window_TalkList.prototype.constructor = Window_TalkList;

Window_TalkList.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = this.windowHeight();
    var position = PlayOnline.Param.Parameters.split(",");
    var x = parseInt(position[0]);
    var y = parseInt(position[1]);
    Window_Base.prototype.initialize.call(this, x, y - height, width, height);
    this.opacity = PlayOnline.Param.TalkOpacity;
    this.refresh();
    this.drawInfo();
    PlayOnline.addButton(1,'map','BtnInput',x+10,y-35,['script','Graphics._input.focus();']);
    PlayOnline.addButton(2,'map','BtnSend',x+width-70,y-55,['script','']);
    PlayOnline.addButton(3,'map','BtnSize',x+width-23,y-height,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
    if (PlayOnline.ShowTalk) {
        Graphics._addInput("text", x + 10, y - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
        Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
        Graphics._input.value = "";
    }
    this.lastShowTalk = PlayOnline.ShowTalk;
    this.lastBusy = $gameMessage.isBusy();
};

Window_TalkList.prototype.standardPadding = function() {
    return 10;
};

Window_TalkList.prototype.windowWidth = function () {
    return PlayOnline.Param.TalkWidth;
};

Window_TalkList.prototype.windowHeight = function () {
    switch (PlayOnline.Size) {
        case 1: return 200;break;
        case 2: return 300;break;
        case 3: return 400;break;
    }
};

Window_TalkList.prototype.refresh = function() {
    if (PlayOnline.ShowTalk) {
        this.contents.paintOpacity = 255;
        this.opacity = PlayOnline.Param.TalkOpacity;
    } else {
        Graphics._removeInput();
        PlayOnline.removeButton(1);
        PlayOnline.removeButton(2);
        PlayOnline.removeButton(3);
        this.contents.paintOpacity = 0;
        this.opacity = 0;
    }
};

Window_TalkList.prototype.drawHorzLine = function(y) {
    if (PlayOnline.ShowTalk) {
        var lineY = y + this.lineHeight() / 2 - 1;
        this.contents.paintOpacity = 100;
        this.contents.fillRect(0, lineY, this.contentsWidth(), 2, this.normalColor());
        this.contents.paintOpacity = 255;
    }
};

Window_TalkList.prototype.standardFontSize = function() {
    return PlayOnline.Param.TalkFont;
};

Window_TalkList.prototype.lineHeight = function() {
    return 30;
};

Window_TalkList.prototype.drawInfo = function() {
    var num = PlayOnline.GameInfo[0] ? parseInt(PlayOnline.GameInfo[0]) + 1 : 1;
    var j = PlayOnline.Text.length >= 15 ? 15 : PlayOnline.Text.length;
    for (var i = 0; i < PlayOnline.GameInfo[0]; ++i) {
        if (PlayOnline.GameInfo[i*11+10]) {
            if (PlayOnline.GameInfo[i*11+2] == '系统') {
                PlayOnline.Text[j] = PlayOnline.Param.SystemTalk+PlayOnline.GameInfo[i*11+11]+PlayOnline.GameInfo[i*11+10];
                ++j;
            } else {
                if (PlayOnline.GameInfo[i*11+11]) {
                    if (PlayOnline.GameInfo[i*11+11] == PlayOnline.UserName) {
                        PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"对你说："+PlayOnline.GameInfo[i*11+10];
                        ++j;
                    }
                } else {
                    PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"说："+PlayOnline.GameInfo[i*11+10];
                    ++j;
                }
            }
        }
    }
    var info = [];
    if (PlayOnline.Text.length > 15) {
        var k = PlayOnline.Text.length-15;
        for (var i = 0; i < 15; ++i) {
            info[i] = PlayOnline.Text[k+i];
        }
    } else {
        for (var i = 0; i < j; ++i) {
            info[i] = PlayOnline.Text[i];
        }
    }
    if (PlayOnline.ShowTalk) {
        var user = PlayOnline.NewUser ? PlayOnline.NewUser : PlayOnline.UserName;
        if (user == '系统') {
            this.changeTextColor(this.powerUpColor());
            this.drawText("注意！ "+user+" 出来发话了！",0,0,this.contentsWidth(),'left');
        } else {
            this.changeTextColor(this.systemColor());
            this.drawText("欢迎 "+user+" 进入游戏！",0,0,this.contentsWidth(),'left');
        }
        var line1 = Math.min(12,this.lineHeight());
        this.drawHorzLine(line1);
        this.changeTextColor(this.normalColor());
        if (j > 0) {
            for (var i = 15 - (this.height-100)/20; i < 15; ++i) {
                this.drawText(info[i], 0, 25+(j-1-i)*20, this.contentsWidth(), 'left');
            }
        }
        var line2 = Math.max(this.height-87,this.lineHeight());
        this.drawHorzLine(line2);
            this.changeTextColor(this.systemColor());
            this.drawText("目前有："+num+"人在线。",0,this.height-74,this.contentsWidth(),'left');
    }
    PlayOnline.Text.splice(0,PlayOnline.Text.length);
    for (var i = 0; i < 15; ++i) {
        PlayOnline.Text[i] = info[i];
    }
};

Window_TalkList.prototype.update = function () {
    Window_Base.prototype.update.call(this);
    this.dianji();
    if (PlayOnline.UpdateTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        PlayOnline.UpdateTalk = false;
    }
    if (this.lastShowSwitch != PlayOnline.ShowTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        if (PlayOnline.ShowTalk) {
            Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
            Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
            Graphics._input.value = "";
            PlayOnline.addButton(1,'map','BtnInput',this.x+10,this.y+this.windowHeight()-35,['script','Graphics._input.focus();']);
            PlayOnline.addButton(2,'map','BtnSend',this.x+this.width-70,this.y+this.windowHeight()-85,['script','']);
            PlayOnline.addButton(3,'map','BtnSize',this.x+this.width-23,this.y,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
        }
        this.lastShowSwitch = PlayOnline.ShowTalk;
    }
    if (this.lastBusy != $gameMessage.isBusy()) {
        if ($gameMessage.isBusy()) Graphics._removeInput();
        else {
            if (PlayOnline.ShowTalk && !$gamePlayer.isTransferring() && !SceneManager.isSceneChanging()) {
                Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
                Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
                Graphics._input.value = "";
            }
        }
        this.lastBusy = $gameMessage.isBusy();
    }
};

Window_TalkList.prototype.dianji = function() {
    if (!$gameMessage.isBusy() && PlayOnline.ShowTalk) {
        if (TouchInput.isRepeated()) {
            var dx = this.canvasToLocalX(TouchInput.x);
            var dy = this.canvasToLocalY(TouchInput.y);
            if(dx >= PlayOnline.Param.TalkWidth-70 && dy >= this.windowHeight() -74 && dx < PlayOnline.Param.TalkWidth && dy < this.windowHeight() - 30){
                if (Graphics._input.value.length > 0 && Graphics._input.value.indexOf(":") <= 0) {
                    PlayOnline.Text[PlayOnline.Text.length] = "你说：" + Graphics._input.value;
                    this.SendText(Graphics._input.value);
                } else if (Graphics._input.value.indexOf(":") > 0) {
                    var Text = Graphics._input.value.split(":");
                    if (Text[0] == '系统') PlayOnline.Text[PlayOnline.Text.length] = "不支持人机对话。";
                    else if (Text[0] == PlayOnline.UserName) PlayOnline.Text[PlayOnline.Text.length] = "屏蔽自言自语。";
                    else if (this.Online(Text[0])) {
                        PlayOnline.Text[PlayOnline.Text.length] = "你对"+Text[0]+"说："+Text[1];
                        this.SendText(Text[1], Text[0]);
                    } else PlayOnline.Text[PlayOnline.Text.length] = Text[0]+"不在线。";
                }
                Graphics._input.value = "";
            }
        }
    }
};

Window_TalkList.prototype.Online = function(user) {
    if (PlayOnline.Users.length > 0) {
        for (var j = 0; j < PlayOnline.Users.length; ++j) {
            if (user == PlayOnline.Users[j]) return true;
        }
        return false;
    } else return false;
};

Window_TalkList.prototype.SendText = function(text, name) {
    if (!name) var name = 'X';
    $.ajax({
        type: 'post',
        url: PlayOnline.Link,
        data: {user:PlayOnline.UserName, game:PlayOnline.Param.ActionName, cmd:'talk', text:text, target:name},
        success: function(msg) {},
        error: function (msg) {
            alert("连接不到服务器，请检查网络重试。");
            PlayOnline.UserName = null;
        },
        async: true,
    });
};

//-----------------------------------------------------------------------------
//  WINDOW_RankCmd
//-----------------------------------------------------------------------------

function Window_RankCmd() {
    this.initialize.apply(this, arguments);
}

Window_RankCmd.prototype = Object.create(Window_HorzCommand.prototype);
Window_RankCmd.prototype.constructor = Window_RankCmd;

Window_RankCmd.prototype.initialize = function(x,y) {
    Window_HorzCommand.prototype.initialize.call(this, x, y);
};

Window_RankCmd.prototype.windowWidth = function() {
    return 600;
};

Window_RankCmd.prototype.maxCols = function() {
    return 4;
};

Window_RankCmd.prototype.makeCommandList = function() {
    this.addCommand("首页",  'shouye');
    this.addCommand("我排在哪",  'mypage');
    this.addCommand("上一页",  'pageup');
    this.addCommand("下一页",  'pagedown');
};

Window_RankCmd.prototype.shouye = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= 0 && y >= 0 && x < this.width/4 && y < this.height){
            this.callHandler("shouye");
        } 
    } 
};

Window_RankCmd.prototype.mypage = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/4 && y >= 0 && x < this.width/2 && y < this.height){
            this.callHandler("mypage");
        } 
    } 
};

Window_RankCmd.prototype.pageup = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/2 && y >= 0 && x < this.width*3/4 && y < this.height){
            this.callHandler("pageup");
        } 
    } 
};

Window_RankCmd.prototype.pagedown = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width*3/4 && y >= 0 && x < this.width && y < this.height){
            this.callHandler("pagedown");
        } 
    } 
};

//-----------------------------------------------------------------------------
// Sprite_Character
//-----------------------------------------------------------------------------

PlayOnline.Sprite_Character_update = Sprite_Character.prototype.update;
Sprite_Character.prototype.update = function() {
    PlayOnline.Sprite_Character_update.call(this);
    this.updateShowName();
    this.updateShowNameZoom();
};

Sprite_Character.prototype.updateShowName = function() {
    this.setupShowName();
    if (!this._showname) return;
    this.positionShowName();
};

Sprite_Character.prototype.setupShowName = function() {
    var num = $gameParty.maxBattleMembers() + PlayOnline.Param.Players - 1;
    if (this._showname) {
        if (PlayOnline.UpdateName[PlayOnline.UpdateCount]) {
            this._showname.setActor(this._character);
            this._showname.update();
            PlayOnline.UpdateName[PlayOnline.UpdateCount] = false;
            PlayOnline.UpdateCount++;
        }
        return;
    }
    if (!SceneManager._scene._spriteset) return;
    if (this._character instanceof Game_Follower && this._character._memberIndex < num) {
        this._showname = new Window_Name();
        this._showname.setActor(this._character);
        SceneManager._scene._spriteset.addChild(this._showname);
    }
};

Sprite_Character.prototype.positionShowName = function() {
    if (this._showname) {
        var win = this._showname;
        var width = win.width * win.scale.x;
        win.x = this.x + width / -2 + win.bufferX();
        var height = win.height * win.scale.y;
        var buffer = win.bufferY() * win.scale.y;
        win.y = this.y + (this.height * -1) - height + buffer;
    }
};

Sprite_Character.prototype.updateShowNameZoom = function() {
    if (!this._showname) return;
    var spriteset = SceneManager._scene._spriteset;
    this._showname.scale.x = 1 / spriteset.scale.x;
    this._showname.scale.y = 1 / spriteset.scale.y;
};

//-----------------------------------------------------------------------------
//  Sprite_PlayButton
//-----------------------------------------------------------------------------

function Sprite_PlayButton() {
    this.initialize.apply(this, arguments);
}

Sprite_PlayButton.prototype = Object.create(Sprite_Button.prototype);
Sprite_PlayButton.prototype.constructor = Sprite_PlayButton;

Sprite_PlayButton.prototype.initialize = function(Button) {
    Sprite_Button.prototype.initialize.call(this);
    this._hidden = false;
    this.setupButton(Button);
};

Sprite_PlayButton.prototype.setupButton = function(Button) {
    this._button = Button;
    this.bitmap = ImageManager.loadPicture(Button.image);
    var h = this.bitmap.height / 2;
    var w = this.bitmap.width;
    this.setColdFrame(0, 0, w, h);
    this.setHotFrame(0, h, w, h);
    this.x = Button.x;
    this.y = Button.y;
};

Sprite_PlayButton.prototype.processTouch = function() {
    if (this.isActive() && this.opacity >= 255) {
        if (TouchInput.isTriggered() && this.isButtonTouched()) {
            this._touching = true;
            this._wasTouching = true;
        }
        if (this._touching) {
            if (TouchInput.isReleased() || !this.isButtonTouched()) {
                if (this._wasTouching) {
                    this.callClickHandler();
                }
	this._touching = false;
	this._wasTouching = false;
            } else {
	this.callPressHandler();
            }
        } else if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
            this._touching = false;
        }
    } else {
        this._touching = false;
        if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
        }
    }
};

Sprite_PlayButton.prototype.setPressHandler = function(method) {
    this._pressHandler = method;
};

Sprite_PlayButton.prototype.callPressHandler = function() {
    if (this._pressHandler) {
        this._pressHandler();
    }
};

Sprite_PlayButton.prototype.update = function() {
    Sprite_Button.prototype.update.call(this);
    this.updateVisibility();
};

Sprite_PlayButton.prototype.updateVisibility = function() {
    if ($gameSystem._hideBtns) {
        this.opacity -= 20;
    } else if ($gameMap.isEventRunning()) {
        this.opacity = Math.max(this.opacity - 20,this._button.eOpacity);
    } else {
        this.opacity += 20;
    }
};

//-----------------------------------------------------------------------------
//  PlayOnline function
//-----------------------------------------------------------------------------

PlayOnline.createButton = function(obj) {
    if (!obj) return;
    var index = obj.id;
    if (!PlayOnline.canHasButton(index)) return;
    this.removeChild(this._GButtons[index]);
    this._GButtons[index] = new Sprite_PlayButton(obj);
    if (obj.action) {
        var type = obj.action[0];
        var data = obj.action[1];
        switch (type) {
            case 'button':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnPress.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'buttonT':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnTrigger.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'script':
                var script = data;
                this._GButtons[index].setClickHandler(this.gButtonScript.bind(this,data));
                break;
            case 'event':
                this._GButtons[index].setClickHandler(PlayOnline.runGCommentEvent.bind(this,data));
                break;	
        }
    }
    this.addChild(this._GButtons[index]);
};

PlayOnline.onButton = function() {
    var x = TouchInput.x;
    var y = TouchInput.y;
    var btns = SceneManager._scene._GButtons;
    if (!btns) return false;
    var result = false;
    for (var i = 0; i < btns.length; i++) {
        if (btns[i] && x > btns[i].x && x < btns[i].x + btns[i].width && y > btns[i].y && y < btns[i].y + btns[i].height) {
            result = true;
            break;
        }
    }
    return result;
};

PlayOnline.canHasButton = function(i) {
    return $gameSystem._gBtns[i] && $gameSystem._gBtns[i].location == 'map' && ($gameSystem._gBtns[i].mapId == 0 || $gameSystem._gBtns[i].mapId == $gameMap.mapId());
};

PlayOnline.runGCommentEvent = function(id) {
    $gameTemp.reserveCommonEvent(id);
};

PlayOnline.btnPress = function(btn) {
    Input._currentState[btn] = true;
};

PlayOnline.btnTrigger = function(btn) {
    if (!PlayOnline.Triggered[btn]) {
        Input._currentState[btn] = true;
        PlayOnline.Triggered[btn] = true;
    } else {
        Input._currentState[btn] = false;
    }
};

PlayOnline.btnRelease = function(btn) {
    Input._currentState[btn] = false;
    PlayOnline.Triggered[btn] = false;
};

PlayOnline.addButton = function(id,location,img,x,y,action,e) {
    var obj = {id:id,image:img,x:x,y:y,action:action,eOpacity:e || 0};
    var location = location.toLowerCase();
    if (location[0] == 'm') {
        obj.location = 'map';
        obj.mapId = Number(location.replace('map',''));
        $gameSystem._gBtns[id] = obj;
        SceneManager._scene.createGBtn($gameSystem._gBtns[id]);	
    }
};

PlayOnline.removeButton = function(id) {
    $gameSystem._gBtns[id] = null;
    SceneManager._scene.removeGButton(id);
};

//-----------------------------------------------------------------------------
//  Graphics._Input
//-----------------------------------------------------------------------------

Graphics._createAllElements = function() {
    this._createErrorPrinter();
    this._createCanvas();
    this._createVideo();
    this._createUpperCanvas();
    this._createRenderer();
    this._createFPSMeter();
    this._createModeBox();
    this._createGameFontLoader();
    this._createInput();
    this._createInput2();
};

Graphics._updateAllElements = function() {
    this._updateRealScale();
    this._updateErrorPrinter();
    this._updateCanvas();
    this._updateVideo();
    this._updateUpperCanvas();
    this._updateRenderer();
    this._updateInput();
    this._updateInput2();
    this._paintUpperCanvas();
};

Graphics._createInput = function() {
    this._input = document.createElement("input");
    this._input.id = 'Input';
    this._input.type ="text";
    this._input._sx ={};
    var sx = this._input._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._createInput2 = function() {
    this._input2 = document.createElement("input");
    this._input2.id = 'Input2';
    this._input2.type ="text";
    this._input2._sx ={};
    var sx = this._input2._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._addInput = function(type,x,y,width,height,fontSize) {                
    this._input.type = type || "text";
    var sx = this._input._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput();
    sx.xs = true;
    document.body.appendChild(this._input);
};

Graphics._addInput2 = function(type,x,y,width,height,fontSize) {                
    this._input2.type = type || "text";
    var sx = this._input2._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput2();
    sx.xs = true;
    document.body.appendChild(this._input2);
};

Graphics._removeInput = function() {
    this._input.remove();
    this._input.value = null;
    this._input._xs = false;
};

Graphics._removeInput2 = function() {
    this._input2.remove();
    this._input2.value = null;
    this._input2._xs = false;
};

Graphics._updateInput =function () {   
    this._input.style.zIndex = 13;
    var sx = this._input._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input.style.position = 'absolute';
    this._input.style.margin = 'auto';
    this._input.style.top = y  + 'px';
    this._input.style.left = x  + 'px';
    this._input.style.width = width + 'px';
    this._input.style.height = height + 'px';
    this._input.style.fontSize = fontSize + 'px';
};

Graphics._updateInput2 =function () {   
    this._input2.style.zIndex = 12;
    var sx = this._input2._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input2.style.position = 'absolute';
    this._input2.style.margin = 'auto';
    this._input2.style.top = y  + 'px';
    this._input2.style.left = x  + 'px';
    this._input2.style.width = width + 'px';
    this._input2.style.height = height + 'px';
    this._input2.style.fontSize = fontSize + 'px';
};

Input._onKeyDown = function(event) {
    if (this._shouldPreventDefault(event.keyCode)) {
        if (Graphics && Graphics._input && Graphics._input._sx && Graphics._input._sx.xs) {
        } else if (Graphics && Graphics._input2 && Graphics._input2._sx && Graphics._input2._sx.xs) {
        } else {
            event.preventDefault();
        }
    }
    if (event.keyCode === 144) {
        this.clear();
    }
    var buttonName = this.keyMapper[event.keyCode];
    var version = parseInt(Utils.RPGMAKER_VERSION.substr(2, 1));
    if (version < 5) {
        if (buttonName) {
            this._currentState[buttonName] = true;
        }
    } else {
        if (ResourceHandler.exists() && buttonName === 'ok') {
            ResourceHandler.retry();
        } else if (buttonName) {
            this._currentState[buttonName] = true;
        }
    }
};