﻿//-----------------------------------------------------------------------------
// PlayOnline.js
// Version: 2.3
//-----------------------------------------------------------------------------
/*:
 * @plugindesc 网络登录同屏聊天排行云存档2.3版
 * @author 江湖梦网 Fanzi
 *
 * @param  游戏名字
 * @desc 登陆后显示欢迎语
 * @default 
 *
 * @param  后台程序
 * @desc 游戏在江湖梦网申请到的后台程序名
 * @default game
 *
 * @param  游戏版本
 * @desc 输入当前游戏的版本号，用于检测游戏更新
 * @default 1
 *
 * @param  下载地址
 * @desc 游戏更新的网站入口，论坛发布指引帖子填写地址
 * @default https://bbs.jhmeng.cn/
 *
 * @param  窗口位置
 * @desc 登陆窗口显示的高度
 * @default 100
 *
 * @param  窗口透明度
 * @desc 调节登录窗口的透明度
 * @default 255
 *
 * @param  弹窗背景
 * @desc 弹出窗口背景图片 384 * 144 img/pictures 留空无背景
 * @default popup
 *
 * @param  存档通知
 * @desc 用于做定点云存档成功的返回信息，自动云存档留空不显示
 * @default 您已成功存档！
 *
 * @param  统一变量
 * @desc 指定一个变量号，开发者可在服务器修改该变量，登录赋值
 * @default 5
 *
 * @param  挂机变量
 * @desc 离线的服务器时间写入该号变量，单位为秒数
 * @default 3
 *
 * @param  排行变量
 * @desc 游戏数据转化为一个变量用于排行依据的变量号
 * @default 2
 *
 * @param  排行记录
 * @desc 用于查看排行后记录当前排名的临时变量号
 * @default 4
 *
 * @param  在线时长
 * @desc 玩家多少分钟内有数据更新认定在线
 * @default 3
 *
 * @param  显示上限
 * @desc 同个地图显示玩家人数限制
 * @default 10
 *
 * @param  角色数量
 * @desc 游戏中的角色数量，根据数据库角色数量填写
 * @default 4
 *
 * @param  移动速度
 * @desc 队友及其他玩家显示的移动速度放慢倍数，可以是小数
 * @default 2
 *
 * @param  队友跟随
 * @desc 选择队友跟随形态，true为随机跟随，false为贪食蛇式
 * @default true
 *
 * @param  队友漫步
 * @desc 队友原地踏步的机率，可以是小数
 * @default 3
 *
 * @param  队友传送
 * @desc 队友远离队长一定距离时，自动传送到队长身边
 * @default 6
 *
 * @param  显名性质
 * @desc 显示玩家的用户名或者角色名，true为用户名，false为第一角色名
 * @default true
 *
 * @param  人名X位
 * @desc 显示玩家名字距离角色头顶原点X位移
 * @default 0
 *
 * @param  人名Y位
 * @desc 显示玩家名字距离头顶原点Y位移
 * @default 30
 *
 * @param  人名字号
 * @desc 显示玩家名字的字体大小
 * @default 16
 *
 * @param  聊天窗XY
 * @desc 聊天窗口显示位置(左下角)
 * @default 600,600
 *
 * @param  聊天窗宽
 * @desc 聊天窗口宽度
 * @default 200
 *
 * @param  聊天明暗
 * @desc 聊天窗口、背景及信息明暗度
 * @default 255
 *
 * @param  聊天字号
 * @desc 聊天窗口显示字体大小
 * @default 20
 *
 * @param  显示聊天
 * @desc 聊天窗口是否默认显示
 * @default true
 *
 * @param  系统说话
 * @desc 用于系统说话开关，提示获得物品等
 * @default 恭喜
 *
 * @param  菜单排行
 * @desc 主菜单指令列表增加显示排行的文本，留空不建立
 * @default 排行
 *
 * @param  排行内容
 * @desc 排行界面显示数字序列的表头文本
 * @default 总战力
 *
 * @param  排行背景
 * @desc 排行界面背景图片 img/pictures 留空无背景
 * @default bgd
 *
 * @param  排行透明度
 * @desc 调节排行窗口的透明度，独立于聊天窗口
 * @default 255
 *
 * @param  息屏登出
 * @desc 手机进入息屏时登出游戏，true为启用，false为不启用
 * @default true
 *
 * @param  屏幕按钮
 * @desc 将按钮图片(仿发送按钮)放在 img/pictures 填入文件名
 * @default 
 *
 * @param  APK打包
 * @desc 用于兼容APK打包后的弹出窗口，Website2APK打包填true，其它填false
 * @default false
 *
 * @help
 * 本插件用于将游戏网络化，为游戏提供服务器对接，接入本插件的游
 * 戏将可发布为PC终端、WEB终端、APP终端皆同步通讯，服务器端提
 * 供账号检测、密码保护、单线登录、玩家同屏、云存档、在线双频道
 * 聊天、玩家排行等功能，在线商城有配套商城插件。
 * 插件参数可供开发者自定义设计游戏，包括透明度与背景图片用于界
 * 面美化；排行变量由开发者在游戏中计算存入指定变量，该变量将与
 * 服务器进行通讯，排行名称自定义；游戏时长为判定玩家在线的依据，
 * 即该时间内玩家有数据更新即为在线；显示上线控制同地图玩家显示
 * 数量；显名性质用以选择同屏玩家显示用户名或者角色名；玩家名字
 * 位置大小均自定义；聊天窗口大小、位置、透明度、字体均可自定义；
 * 挂机变量用以离线经验奖励调用，该数值为服务器离线时间单位秒数；
 * 统一变量用以指定变量由开发者在服务器修改，玩家登录即更新其值；
 * 本插件中玩家同屏功能与集合队友的事件指令有冲突，显示队友跟随
 * 即开启玩家同屏，此时不可集合队友，关闭显示队友即关闭玩家同屏。
 * 本插件内置屏幕按钮功能，如需模拟手机按键可仿发送按钮制作图片
 * 放入pictures文件夹，屏幕按钮参数多个图片用英文逗号隔开，在事件
 * 中执行脚本，具体见后面脚本详述；插件提供一个系统给所有在线玩
 * 家全体发信息的指令，可结合系统说话参数做系统提示，具体见后文。
 * 本插件由 江湖梦网 Fanzi 开发，入驻游戏开发者可免费安装使用，
 * 江湖梦网作为游戏服务平台，承载付费资金的代收与转发。
 *                     有意合作者请登录 江湖梦网 https://bbs.jhmeng.cn/
 *                                               联系QQ：12868880  Fanzi
 * 插件指令:
 * 游戏云存档：SaveGame
 * 需自动云存档则建立并行处理的，公共事件定期执行SaveGame指令，
 * 自动云存档建议600帧执行一次，定点存档直接运行SaveGame指令。
 * 显示聊天窗：ShowTalk
 * 隐藏聊天窗：HideTalk
 * 打开排行榜：SceneRanking
 * 服务器记数：AskValue n   参数 n 填写变量号则将记数值返回该号变量
 * 服务器清数：ResetValue   开发者可设定记数值达到多少则清零
 * 事件发信息：SendWord \C[2]信息内容\C[0]   （颜色代码同显示文本）
 * 例如系统说话参数为：恭喜，插件指令为：SendWord 捡到金子，当某
 * 个玩家触发该指令时则显示“恭喜###捡到金子”。
 * 读取服务器时间：AskData 6 7 8 9             (6、7、8、9为系统变量ID)
 * 注意空格隔开，执行此插件指令后 变量 6 = 年份，变量 7 = 月份
 * 变量 8 = 日期，变量 9 = 钟点（小时数），此功能可用于设计日常任务
 *
 * 脚本指令：
 * 返回游戏登录：DataManager.ResetData();
 * 屏幕按钮脚本：
 * 生成：PlayOnline.addButton(id,'type','img',x,y,['actionType','action'],e);
 * 消除：PlayOnline.removeButton(1);        括号里填按钮ID
 * 脚本参数说明：
 * id：              按键序号，插件已用了1、2、3，新增从4开始，不可重复
 * type：          map/mapX 填map则所有地图通用，填mapX则对应id的
 *                     地图有用，其中X为地图id号
 * img：           按钮图片文件名，注意大小写，不用后缀，img/pictures
 * x：               屏幕坐标X
 * y：               屏幕坐标Y
 * actionType：按钮类型，button/buttonT/script/event四选一
 * action：       按钮执行动作，需对应上方的按钮类型，下面详解
 * e：               运行事件时的按钮可视度，不填则运行事件时不显示，
 *                     填255则运行事件时仍可视并可操作。
 * 按钮类型与执行动作：
 * button：  MV默认按键，执行'ok','cancel','shift','up','down','left','right'
 * buttonT：模拟一个按钮被触发
 * script：    执行脚本，执行动作可填脚本如：SceneManager.push(Scene_Equip)
 * event：    执行公共事件，执行动作填公共事件ID号
 * 脚本举例：
 * PlayOnline.addButton(4,'map','BtnOk',570,500,['button','ok'],255);
 * PlayOnline.addButton(5,'map1','BtnRun',690,500,['buttonT','shift']);
 * PlayOnline.addButton(6,'map','BtnMenu',0,0,['script','SceneManager.push(Scene_Menu)']);
 * PlayOnline.addButton(7,'map1','BtnEvent',800,600,['event',1]);
 * 按钮式开关聊天窗：
 * PlayOnline.addButton(8,'map','Talk',500,500,['script','PlayOnline.ShowTalk = PlayOnline.ShowTalk ? false : true;']);
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.PlayOnline = 2.3;

var PlayOnline = PlayOnline || {};
PlayOnline.Parameters = PluginManager.parameters('PlayOnline');
PlayOnline.Param = PlayOnline.Param || {};

function Scene_PlayOnline() {
    this.initialize.apply(this, arguments);
};

function Scene_Ranking() {
    this.initialize.apply(this, arguments);
};

function Window_LoginCommand() {
    this.initialize.apply(this, arguments);
}

function Window_LoginInfo() {
    this.initialize.apply(this, arguments);
}

function Window_Ranking() {
    this.initialize.apply(this, arguments);
}

;var encode_version = 'sojson.v5', ahozm = '__0x49358',  __0x49358=['wqNLJsOzw4XDs23DrMO8XlIYdmM=','X8KIecObSQ==','PG7Cnmg=','w6VZDhPCvQ==','dQTDicKiwqrCusODwps5','L8OOMlVPDsKSwrjCtwDDv8KeNkk=','w7HDv8O0wqV1','wrI7w7PChRxm','fSwkwqnCnUY=','wpjDlwDClcOOwow=','w7bDgW5E','wq8jw4TCgQ==','w4LCg8KUw5d/LcOQbA==','wqfDo8Oqwpxqew==','wrrDmsOQ','S0ZUwqbCsWzDlA==','PMKCw5fCtGM=','OgNtNWrDssKOWQ==','w7hkEQvCjg==','wrnDvcOf','w5xNwpZV','OSvCpDPDtQ==','wrcfw6ptKSw=','w7TClSvDjsOSw44sAQ==','PxFvInHDow==','dA5hwo0=','AMKQIm8XEMKjQw==','wpwnw4rCiA==','W8OTIcKyJg==','w78+wqUTw5U=','M8KSOG0IH8Kw','A8OGwotfw5XDsQ==','w4/CvmI=','wqRVw6LCvQ==','w40WwrsQw5LDiWBjwr4=','E8OCMFVDAcKYwojCjz3DuMKONVs=','XSU9wrnCrU3DlsKN','wpnDiMOGw6E=','wpHCn1gRwqF7wqwfLQ==','AsOJw5TDiMOvw5JEdUE=','wqwjw5XCgTJtcHQ=','wprDr8OJwpU=','E2LCnnlDRlLDhA==','QALDn8KOwqDCpMO7wr4oSMO/SMO1wqx7JsK5wq8qKsKmwpfDtMOsDmHCoUc9w6YKw4slw6bCoFEww4g=','w53DhcKGGF89woUebnx0w5stw55vwqEFw646','OcOuE8Knw5XCqVZlwo3Cg8OOHsOPY2vDpVDDuWg=','eENewp/CuWzDmCZdUsOJw5HCq10=','wqrDmwvCssO2wofCgAkzVsOienQg','U8K/csO3fAvCrsOcwo7DocOqwqTDj8OxccKcaFIo','w4fDi2VOAcKvQVR7w5RYWcKYZyrDrsKFwoIGw7IvRcORMU/CucOlwonCocON','aMO/CsKX','TsKMNsOHwr84Fhw+DMOYw5oYwqQ=','LWLChERwSVjDg0/Dpg==','CQzCph7Dow==','wpwtw47CiRptckkcw4tRVn7DnA==','w7fCnzHDgw==','wpTDi8K8TA==','Ai7DjMK1w50=','w6cuw6wFTg==','w6HCkyfDk8Onw4A0C8Kb','wo/DpwjCt8OD','B8KZZ8Oqw7JAw7E=','fsODw6p5RsO1w7HDtA==','w6TCtcOkM3RUwrYGwo3CgA==','ZQ7DgsKPwrvCh8OBwos5ZcOlZ8O1wrM=','PMOFw4nDiMOTw7NMZkjCtw==','w7jCrsOlIlNFwqYRwoA=','w5HCuWDCgD1GFsKKw4Y=','FMKfY8Ovw6VCw6DCsw==','w7HClcOHw5grwr8zVXxFCMK/','wobCgkMKwppmwqEDLQ==','w5QKwr0Qw5TDnHV6wqHCuA==','I8OJw4vDiMO4w6I=','wonDn8K3Z8Oswr7Csg==','w4fDhsOwwoh7wrpEAw==','wqfDtMO+w6vCmQ==','ecKcw6MGAxwBw5VC','w75JBwnCj8OE','K3fClG1lQmzDg0vDt8Kfwo17wojDow==','fsKHw6gGBA==','AW7CnnxkU2vDhkTDsMKVwpc=','wp7DlMORw73CgA==','w6INwroUw4jDiU56wrXCuWNi','SALDn8KawrrCp8OzwpojQsOlfA==','woHDmMOcw67CgDA=','w7nDhsOtwohxwrtPAcOnwojDr8OvJ8K5Vg==','55q65b2G5ru25oi0','6YCX5YaR5rqJ5oul','5Li76L6w5puj5pSm','Cx9/KUjDvMKCWA==','w7XDjGRjC8KdYVB7w4U=','w6V9Dj7CpQ==','wpY6w5xi','DyrCoxnDjcKFIFEaOA==','VG95wpnCiQ==','wqdXwqk=','C8OrAcKNw67Crldj','asO6AsK4Kl/CnSjDoMKi','GcKTcMOvw70=','EcOJOXtNAsKRwr3Cjg0=','5ouO5bys6K+T5Zym','wpzCmEEJ','Iy/Di8Kzw5w=','wrvCuG8qwqQ=','LMKhw4bCon9ZesKJ','wp86w5R2BsO3FMKZNQHCpxMlR8KcRg==','w53DhcKGFEI5wo8Wbnk=','Y8Kcw4g9HA==','CGTCgUd/','FMKYc8OFw7xOw7nCt8Kvw78=','WiPDssK+wp4=','wpfClEg9wpp/wrUSJsOw','w497JSrCrw==','w7jDh2FEA8KRYVQ=','cMKLPcOpwr00FRkHAQ==','w48Yw6I2ZMKcWw==','w73DicOvwplbwrVOBcO8woM=','YiggwrbCiA==','LV/ClUtX','a8KBw7QlBQwMw40=','OSfCqT7DjcKfFA==','wpwjw4/CiA==','w5jDhMKDNFk9wpQWdHg=','OFDCsGLDs8OXw78=','E8KWMHQEAsK/','w4DCj8KOw4psIsOQeg==','w43CkMOMw4ks','DMOPwoBZw5M=','VgpgwprDhBQY','wprDqMOVwoliasKNw6PDtw==','w5DDiMKMMmUxwosQaGk=','wrvDngbCjsOjwqbCmys/UsO7Yg==','wp89w59hCMOJDMKf','w5oaEcOuw5/CsTTCt8Oiw7NADA==','w6fDk8OnwoNWwrVQCMOAwonDpcOXHcKjWC8=','LnXCn3h+U0XDn08=','wpA7w59wJcOCEcKLOhw=','CjzCpi3Dq8KGK18=','w4novablh5DmurHmi6dk','bSElwqjCgVE=','5a6pUCjnoYbvvL4=','6LW25Y2b56Sm5q+P5aaB57ip55me5b2U77ys5aWB6KOy55qX5YyX6K2k5Yum6Kyg5Z6i5L6t5pWP5a+Z56Os44Kj','OcOiV8KHcE7DhDXCvsK6w4V9w74=','5rCA5p+C5a2X5qKX5pm95ZCU5b2v5ZOV5YeL5pe35rmb5oup','6KyH5Y6E5YqB5oK95Lim5Yuq55u86K2c5b6U772k5pum5ZGG5oCB5aSj6K2A5b+l776I','WMKeFQ==','wp/CqGM=','55a85oqf5ZCN77y6','6Kye5bK36YW45Lmf5LmS5Yu35Yi45ouj5p6w77yx5L2g5o+c5YWA5bu+5rqo5omQ542l5aKM44KY','aBMt','PMOJw4HDmQ==','w4ZNwpNVw7pQ','wqNrw6XDjsKRw6jDsml6woF2JlkMcQ==','wq/DjQ7Cl8ODwozCkz4=','5q2m6L6Lwr4=','BcOfw4LDn8OVw7dIZA==','bSslwqjCgU3DhMKROsKpw79PAA==','c2VZwqfCiA==','wrHDicK1','VMK/dsOhXAvClcOH','FsKTecOyw7ZNw6DCpcKWw7J1w5wo','w5DCrVXCozQ=','w4/CicKOw5tBKcONbsKVwqA=','w7fDgGFOA8KVWFRtw5VEVcKGbTk=','w6LCuMKYVMObFlLDlsKHwqw+','wrrDnMODw6zCoSrDtsKc','wqVhwq9Mw6vDm1XDjg==','55SY5ou65ZCa776M','w5/DjsKMI0g6wpYEV3R8w6wq','wo7DqMObwopZe8KMw6c=','w4DChcKOw4psPg==','w6nCjsOzPXc=','CMKOw7TCn1s=','w5/DicKHNEYEwo4CZ3R2w6sEw5puwqU=','HhZSEEI=','wpNiw7LDhcK/w6PDgGM=','QhUF','5oCk55i55Yqn5bCj5aa+5aWL5b+X5LqU77+n6Ky05b2M5ZOT5Yaq5pWg5LiU55e644Ok','wrgfw6d4OCfCoMOKAWjDim/Ciw==','wpXDmMKVc8Ol','w5PCjMKBw4dGJw==','TcONw7ZsTg==','w64bw7MdbMKfWw==','EsKWIlAAHcKiQw==','w5vDjmZsDcKeaQ==','w45NHxjCo8OYwrBw','bcO/AsKeCkfChAjDosKq','IiHCpj7DtcKNLw==','DmbCgm18','wrnCnkAHwqNzwrQGLQ==','wrNmw7DDtsKXw6HDk2k=','w5jCvcO4N1E=','wqzDkBvCjw==','w48BwrgLw5zDmVRywqvClGpAwrU5wrPDoE9P','Wgw4OxXDr8OQQMOnw5PChcKywrI=','HMOTwolRw5U=','wpXCgkkfwoF3wpscJsOyworDoi3CnCXCgQ==','wqnCk0MQwpN7wqoeH8O9wo3DtC/CqQ==','HBVkIkfDpw==','w7zCg8KPw5BvJcOWZMKqwr3CqC/Cjn4=','DsKDM2g=','b8KueMO7ZQ/Cg8OXwrTDpcOlwq7DhMOR','OmLCg2l9Ql/Dmw==','w4vDi29OAsKZflxCw4hpXsKFdQ==','OVDColTDt8OQw7Y=','wojDp8KnWyU=','DArCvivDgw==','w49pwol0w6c=','wqjDkAHClMOywofCnzkLUsOrb3I=','wqdVw7PCvcKPeMOIw7Y=','w7HChD7DkMOww4ogEA==','DSHCqS7Dh8KGOUMjNQDDssOV','wrFlw57Dt8KQ','wq7Dl8KsfB8=','w6LCoCzDj8Oj','wqtJHsOow54=','wq3DjMKUSRg=','w7bCmTHDk8OBw4EsF8K+acKwL8Oh','JlDCsGQ=','CwJpMHDDtsKbSA==','AsKcOHIEH8KjVcOTwoIQw6vDuA==','6K+46LaK5Y2z5Zut5L2/5b6R6KCu5bKG77+a6K6t6ICn57K85paQ56iU5qGD6K+h6Keo5bK544O7','RcOhWGxmJMOmQQ==','DDrCsTnDug==','JyrDsMK0w79QVsKRfx0jasOH','E8OIM0xHHQ==','M8KVRcOew7g=','w6fDmGxJEA==','IMOZw5TDhQ==','IMOew4jDmcO0w6JccUU=','UR9iwonDlQI0akTDhMOAL8KZwr9RHgLCgHA=','wqNrw6HDg8Kdw53Dim17wpBHBVoAYsKvwoogBA==','OUnCiUlj','55ee5ouH5ZCh772k','asKBw6IGCQYMw5Zww6Ecw5QT','5a65w6DCkOejhu+8rA==','wqNsw6rDlMKTw6PDkn9VwpxRPV0=','6L2E5LiG6LWP5Y6K5q6J5Z6e5L6p55S377296K625o+s5LiW6LWZ5Y666YaK5pSF55qO5byZ5rmw5ous44Ox','fsOJw6p5RsOp','C8ORwoRPw7XDvMKrw7s=','6LSW5Yy656aW5q+05aSS57uR55qW5b+m77+v5aSO6KCg55id5Y2c6K2R5Yio6K2I5Z2X5L+T5pWm5ayl56CT44G7','NSPDhMKXw7w=','w43ClMOIw4Y5wq4wWWpVJMKnw5TDtsOv','w5dVKh/Cuw==','wpzDr8KEWCHCpE3Dig==','5rmm5oiI6K2K5Z2XMeaxuOa5quagiee9u8K0wpjCgm3DiMO9w692w6jDscKpJwXCiw==','w6fCnzjDj8OQ','U8K/csO3fAs=','JyrDsMKzw65MV8KBXBs1','wrIew6B4NCjCuMOQLGQ=','fjYkwqjCi1fDicKSCA==','w4kaFMOs','wphVw7bCrMK2d8OJw7w=','woZRw6PCqsKjZcOG','wrgfw6d4OCfCoMOK','w5xEwppYw6c=','w44JGcO3w6LCkCLCuQ==','wpLCgk0Jwr19wqoJBMO9wo3DtQ==','w40Uw7s0TcKUVzDCsMKX','aMOxCMKPIFzChDo=','wpnDnMOcw6fCnAvDoMKST1TDicKk','dATDn8KewqrCvcOQwoA=','wpkrw4/CiClmdW8=','wpXCn0IKwpB8wqwAH8O9wofDpCg=','w4/CpH3CmTNeLMKVw48oEQ==','w4ISw7slYMKfSiQ=','NCTDt8Kuw65xUsKDSx0zZw==','w4xcwp5Xw7FGwqJFw43DtUgWNDbDhMO/','XMKkecOzQAvChMOUwovDuA==','TMO7EsKoJl3Cgiw=','wqs4w7l4NA==','ccKuY8O/ZwDCo8OSwo7DqQ==','wpfDncKgVsOm','w67DgMKMPEQ6woU=','VMOJw4dOaQ==','Pzh5AVM=','VhcywrHCqA==','w5vCmSjDtcOFw4Ez','AsOCwpM=','JMOFNA==','w7vCj8OMw5oQwqoJWQ==','w7bClzHDr8OFw5waEcKddMK7NQ==','w4vDj0JUCsKD','dcKdOMOdwps3Hhc=','wqNmw6rDlMKTw78=','CQHDhw==','w5NNwplN','F8KcLA==','DcKcMW8P','wozDtsOVwpJ/','w4gUAMOIw47ClyPCvsO1','OnXCkXtFQkTDmw==','wq4zw59+JMOTHcKB','UMO6w6x1bw==','wq1iw7w=','w77CncOOw40=','w57Dg8OcwoR5','wrd1w70=','5p2a6K+K5Luy5YiJ6K2w55SP5oun5L+A5oOk','NMOew4bDmsOPw7NddQ==','4pih4pmq4piD','w7rCjsOOEXU=','wrcVw694','dcKdOMOdwoY8AAw=','wobDv8Ocwok=','wqRxw6XDl8Kiw6jDnng=','G8KTT8Ovw4E=','OsOVwq5Nw6o=','XiUswrk=','TcO2X28=','A8OPDFFY','w6TDiMOxwohAwoFSL8OpworDpMOR','wrzCnkoEwpY=','LSXCpBbDoA==','wrvCs3oIwpw=','w4kTGcOuw4zCmxDCs8O5w653Gioqwqg=','w58NwroA','wpfDm8K7bMOiwrzCpcOwwq18','S8O7VGw=','woZ8woZUw5LDmw==','w7bDjsOowok=','w68FwroPw5TDk34=','wrvDiMOiw7DCiw==','IxjCryLDrg==','ajYqwqvCsEbDiMKW','AMONw4DDiA==','w48NwrMMw4k=','w4/CgcKTw4p5LcODbA==','wrrDu8Odwpg=','wrRjwqpaw4vDmw==','wooyw4fChQ9m','LiJvE3A=','w7jDhsO1wplCwrVFCQ==','wosRw65p','I8ODw43DnsO0w7gLdxU=','w4DCp2rChiY=','5Lms6ICO5Ym26ZmiEh7CvVXDq8KFWsOpwqU=','woXChF4Xwpt1','w43CncOHw4s7wqc=','bcOcdnZ5','OsKSwrwOWBRKw5kQw7RJw5xNaU8=','wo9YJcO3w5w=','AcOBwqJKw4s=','UsKib8OBYQrCmcOb','w5BGwrxWw7tBwrlTw6bDlU0=','KFzCuHQ=','wp4mw4fCsxJtcnQK','H8ODHldMCcKVwq7CjSrDsMKUOlsZ','Q8O6V38=','J1PCpGNi','wrgqw5DCqA8=','wqTDtCbCscO/','wrUHw6B+OQ==','wq9kwqdJw5s=','wq7Du8OPwpFv','T8OrPw==','JWDCtVzDhw==','wpoxw48=','C3TCkA==','w4UPw7EATQ==','P3fCgGBo','w6LCjw4=','IDfCrBjDug==','KX7CoQ==','woQUw6h4PA==','w6DCssKEXsOOEg==','SArDksKewqDCocOX','TsKLOMOewrM=','BcKWOnIAKcKRVMOrwoY=','BQVlNw==','wrjDqcOEw6PCog==','Fh7CijvCtsKOwr/DkMOwwrLCiMKUw6zDoCdGwq3Dt8KAw4MlOMO+EybDvMOcFcOSGi/Cp8Kdw6DCrMOIbcKkGmTClcOBegJiwoMsw69SHVFEXjZ+UkNrw7vDpCk4','VMOka1Rr','B2PCt2rDsQ==','wr52w5DCrsK0','VSnDnsKZwp4=','w6HCkyzDkw==','wrRmw7fDlA==','HMKSZ8Ozw6c=','6LyV5oya5Lmi5YuQ5p6a5YmI5ZqD772G6K6r5qO75p2q572K57uG6YaJ6K2N44Cn','OcKJwo3CisK0','NcKhwrrCj8KgwqdCYjQ6AMKfHmcow7DDvMOyw5wLwro=','NcOaw4LDn8Oi','wqlww4nDj8KAw6TDiGs=','ecO0w5M=','worDmMObw73CjTY=','TsO4NMKfHA==','woTDiMOcw7zCow==','wrfDn8K5QcOKwrzCp8Oz','MCrDksKvw7lfTsKHewA1d8OBKA==','w4XCuW7CgwZXF8KO','E8OCM0xHAcKIwq/CtwDDtcKOMQ==','CsK0HUMn','w6rCosKEWsOOE37Dlw==','IhbDlQ==','wrxfw70=','wqXDrcOZwo1e','woPDlBc=','RDUAwr/Cjw==','fsOwAsKeI1vCnizDqg==','DsKRPGMCBQ==','ABJiIkfDpw==','w5xHwpFKw7pLwrU=','w7fDh25TC8KcaQ==','GFrCtQ==','T8OJFA==','w4nDocOQU0PCvQLDghQZaMKVw4VlWMKMw5w=','PsKVRcOJw4s=','w4UfPMODw4w=','ewtnwpzDgA==','wqAmw4LCkBo=','YyUzwp7ChVfDhMKOCMKNw75WCkbCvRI=','w4V5wpRQw7o=','wpnDqsOWwpR5','w5nDmcKBMl0gwosYbg==','w5fCksKBw51s','wp1aw6PCtw==','wphbw6I=','woNVw7fCtg==','A2LCsnHDhg==','bsOcw6hkVw==','woHDhsKsQcOzwqbCqMOzwqY=','AsKcOHUOHcKy','woXDksOS','wpPCgl4Rwoc=','wqjDkAHCk8O4woXCjg==','GS/CtTQ=','wqNsw6rDk8KZw6HDgw==','w73DicOgwoI=','wq/Dmg3ClcOw','wojDjcOFw6XCkQ==','wqjDvMKXThg=','d8KsesOzRg/CgMOW','wqwzw4N0AMOCDMKJIBs=','wpBiw7bDgcKb','Tyc/wrXCi03DvsKDAMKl','5ZKv5Y6N56qe5bue','wqxJO8O/w4U=','wodiw6nDhcKgw6jDlH9rwppb','ccOyS3pfJMOqUBcu','5rmX5oqs54qt5pyU','wrnDnMOHw6jChSHDpMKWXk4=','Tk5UwqnCt3XDqg==','wosRw7ttMCzCoMOcJHI=','w41NGxzChw==','w4pFBxnChcOBwplvECLDp0gG','56uP5Y2p6YG45pum5biB','IMOML1lP','WcKBw7wHHCofw4E=','QcKOK8OLwr88DB0bFg==','5b6Y56mc6IOD5pqb','5a6x5qOe6YKv552s','wrAsw4/CnS1iem4Y','FCTDrMKhw7dbVsKHWgc=','57u75Lmv5Y+e6YSn','WcKPw74TAQ==','wrM0w5dZBMOJHQ==','w4fDhMOpwp9XwoJDAMOzwoM=','w6zDgMKQNkAxwpYScm4=','5oya6KKQ5Y2S6YaZ','w6/CpHjCpjNcBA==','W8O/FMKaKFfChCzDvMK1','w60FwqYFw5A=','PsOMMF12FsKMwrk=','w7oaCsOhw4Y=','wq8uw4LCnR5xZQ==','GlTCpHHDu8OBw6PCisK4wp4=','E8KpwrvCh8Kv','6Kat6Iqy5peA6YSI','SUZIwqzCtQ==','w6wSw6M0VsKBWzLCvA==','w5zCtsKYWMOXH2XDnMKZwrA=','56Wa5Ymj6YCQ5bmS','KR9kK0vDpMK3RcKhw4o=','w4TDhsO0woxfwrFWCcO0wpU=','6ZmP5Y2n6LW46Zii','RMKiW8O5fwvCn8OwwoLDv8Ou','XiU5wr3CiQ==','6ZqL5Y2j5ryr5q2F','HsOrBcKJw5DCoUg=','GlTCpHHDuw==','wqXDtMOWwpRje8Kgw7rDvyQ=','5Z+C57il5pWM6ZaC','VR7Dl8KMwqrCocO8','RwrDg8KLwqLCtsOQwpY/VQ==','5LqK5ZOAT+S/mw==','csK4ccOwbRzCtA==','wpBiw7bDgcKbw6jDkmlwwoY=','5Lm65ZCOw53kv60=','QCUmwrnCt0rDisKH','SUZIwqzCtWfDhwBATA==','5LmG5ZCl5aye5Y2p','wrTDn8K9RcOu','PxF6JknDtsKXWcKjw5w=','6IK25aSB56uew4bDsQ==','wpvDnh3CgcO6','wo8Rw6VnCiDCsMONPg==','6IOp5aeJ56m35ayD','6ICS5aaj5pm55pm/','wpFywrxaw5I=','wrjDu8OUwpZPecKQ','5o226KOy6ICD5pmL','wqdNw7bCrMKje8O6w7hZRA==','57CR57mF6K2O6Kyg','OxFkLGLDvMKNSA==','6IOe5aaO5a+R5Yya','C8OiGMKfw7PCoV1n','IMOML1lPCsKIwrnCkho=','KmjCvGNmQk7DrEvDp8Kf','wohaPMO7','wqRVw7fCucKr','wrXDuMKLWifCoFvDlQ==','w7HCqn3ClT9XG8Kfw5E0','6Iyg5Y295o+b6KOS','w4fClzHDjMOtw5s9CQ==','wqbCkV4fwph3wqwWOsOn','ccOyS3pf','w6zDgMKQNkA=','C8Kvw7zCqEFSbsKlw7g0w6Y=','w79YBzHCg8OFwqI=','wq8jw5HChRZmYn4Pw5Y=','5bC/5bu45o2U6ZKW','wrjDjwPCicOj','w7Ecw6cwaA==','w69Jwo1Yw7hCwqREw7nDqQ==','w6DCm0TmiqfljZc=','w4tawopc','JcOeOEpsDsKRwrk=','w6/DgMKUMmQ6woQY','RMKfPcOLwqY8LBkFDg==','wpVzw6DDgcKCw6jDtmBjwoxQOw==','w7QNw7EwccKUcDbCtcKG','w78LHMOhw5/CmwfCucO0w7RA','wpjDlhXChQ==','w49NBxbCg8OYwrE=','M8Kpw57CrA==','woLDrsOOwo03McObw7TDsywkMMKfwpnCpcOww77CkU3DgAwELhJaw4xGwrHCmHfDoHHDoB3CvsKaUUhmaFo=','LMK0w5/CtVM=','w73CgivDl8KewoB3A8KIbcKxdcOjYsKsMml8PzcAwqXDi2rDvsOSwowOa8K0w49ASlkZwph4woB/w6cHw7nCsw==','FcOlGcKNw54=','dcOYw7B9GcK0wqrDoMK3NxZpa8KFw64sZcOfw5fDn8ObwoXDuxnDvnVIw5HDpcO3wp0=','W8O/FMKaKA==','H2TChGV+SXLDjkfDsQ==','cHfCmHw=','6L+65o+K5LiL5YuY5p6s5YuG5ZiY77yJ6K+Q5qGx5p2I57yb57ir6YSG6K+Q44G1','w53CiMObw4EwwqwNWms=','wo1CCMOfw6A=','w5bCqn3Cmg==','LsKrDEwp','wpdbw6jCqMK0c8Odw6phQHALYHpvMw==','aMKEw60K','XsOtA8KJC1PCnSw=','w7PCgcKSw59k','wr4hw5fCjRRtWHoQw4A=','HMOCwpNd','wpVQw6E=','EcOJOQ==','5LuS5a6Q5qKq5aeX6Len77+K','PgPCjSjDrQ==','w7rCkcONGFM=','wqPCg0kMwrtzwrUW','wr8Vw6pjMDnCpsOcJXLDqGnCjDtJw6rCsjpiwoU=','wprDu8OIwo5o','w5rCucOsJFlCwrclwoTCgA8=','VAB1wpw=','w41NGg7CncOZwqR7','EcOPw5PDhMO0w7hrYE3Cpg==','w7rDhsOrwog=','w5wHwqALw48=','wpjCmU8VwptzwrUW','U8ONw6lod8Oiw7XDog==','w7xPHRLCmA==','YiE9wrnCiA==','OcOpA8KHw5U=','w7fDj8Onwp9TwrdWCcO0wqjDqsOOKw==','E8OFPEpDDMKIwrnCkiDDv8KePEY=','w7nDhsO2wqRW','w6jCvsKYXMOZDnjDlsKF','wrZiw6jDlcKT','w45PBg/Cj8OgwrdzBCQ=','FMKWdsO+','EcKdPmot','c8O2X2lXMsO2','worDuMKDXRDCsl0=','W8KYZw==','wplVw4k=','wrBsw7fDlA==','w5xEwpBKw7A=','6Laa5Y2W5L6+5b2d6KKG5bGV77+u6K6A6IGU57Om5paJ56mh5qKC6Kyb6Keh5bOB44Gp','UQXDocKfwpk=','CsKmJg==','UArDnMKPwobCvcOCwpw=','Vx9qwoHDlQ==','wrFmwoFsw4o=','DMKww5zCrkI=','w7QNw7EwccKUajbCtMKI','wosRw7ttMA==','PiLCpiPDh8KaPg==','SMOcw6BsV8O+w4vDpsK7Pw==','ZsOyVH57L8O4Wg==','dgAFwr/Clw==','6LSo5Y6z55iw5b6J5a685ZyL5b6y5bua772h6Ky356O16K6f5a6u56Ci6YSC5peR55mW5b2b44OW','w4HDlMOjwp98wrVPCQ==','wpcPw7HCtR0=','DcKdJ3IP','DhppPw==','w4fCnsOGw54s','H0bCs2LDmMOFw7rCig==','w4nCs8Ofw78z','w4HDm2VSKsKRYVQ=','6L6h5o2l5Lq95Yu35py75YqW5Zm077+q6K2u5qKH5p2m572e57qW6YSi5pWM55ug5b6V44Od','KsKzw5XCtXhWccKD','SMKHw7kWOw==','w53Cr8OvJHJQwrIE','w4A3D8Oow44=','wrjDvMOz','wq1pDw==','5p2y5Ymh5ZuY57iz5oqS5Lia77yQ6KyR5Yeq5rKv5Yev5ZGF56mH5ZOh5YWx55ug44OC','SkRfwqPCvV3DvgRCYMORw4DCoVjCh8O9','asKPw6Ae','wqoxw4bCljVie34=','wpBJOsOqw7vDu3nDrQ==','wpbDm8KiS8O1wrfCgsO0wqFicQ==','wpTDiw7CjMO8wp7CgiQ4VMO4','AxF7M3fDusKZWQ==','RALDi8KP','aMOLb1V/','wqRYw6TCocKjZMOd','e8OrFcKT','wrEnw5TCsQhmZA==','wqfCt10Kwp4=','w4DChTrDlcOX','wrgjw47CgTJtcHQ=','wr/DhcK/eiA=','DcO5EsKaw5Q=','JlDCuHfDosOM','wp7DjArCksOk','w5DDhMKMMFk8','wqFHw6DCqsK1','HT7CqzPDgcKN','w5EBwroDw4nDlQ==','w5PDv8Ocwrhn','wqPDn8KiQcOKwrzCp8Oz','TFRfwr/Cqw==','wpw4w4TCqjU=','PcOGwoNKw4TDqsK7w59mGwbDuC8=','PDfDrg==','diAiwqjClQ==','wrNVw6jCvcKPeMOIw7Y=','wrNzw6jDicKVw6g=','w4/ChcKOw5l9JA==','TcOAw6V0RsOpw7Y=','wr/DvMKISjzCr1PDkQ==','w69Jwo1Yw7g=','wqw+w5BsCMOVCw==','MsKdesOjw5pNw7LCuQ==','P8OCwpdZw4w=','GlnCt2nDs8OW','J8KOwq3ChMK4','Pi/CtTvDjw==','w6rCvcO+IlBUwq0vwoTCmQs=','RMKfNMOhwpE=','wrnDkcOUw7DCjTY=','YMKhdsOvbRw=','DSbCpijDg8KLOVUGEgXDq8OY','NMKDO00i','O8OmFsKbw5TCiVU=','JsK5wrzCj8Kywrg=','w4TDi8OnwpRXwqY=','R8OyWn57L8O6UB0=','w4/CvcO+EVQ=','W8OyB8KCIEA=','LFTCtXXDmMOFw7rCig==','w77CkMOIw5E7wrk=','FsOMPl1sDsKRwrk=','wrTDksKuXcOmwqA=','N2nCmXh4RlDDo0/DosKfwow=','WsONw7BKSw==','BMKpwqTCg8KLwqVBXQ==','w7HCp27CjTdA','TMOyQVdXN8O7WQ==','w7vDgMKPMmQ6woQY','w5jDp8KGNVc=','w7ERw7QoYMKD','PsONw4rDiA==','wp7DjwLCq8OU','w41ACATCj8OE','f8KALcOP','ZxnDnsKMwqbCv8OB','wrHCkUEbwrx8wr4c','OgBlDGc=','KSTDrsKJw74=','w4vCtsKHXMOzFHfDlg==','w7fCmsODw54w','LE/CjA==','ZcObUVdb','w5LCrnvCojNeGsKf','M8KOwqHCosKG','I3fCmUTDoQ==','D8OIw4bDmcO6','OMOmw4rDpcOU','wp9AKMOtw43DkWvDqcOZVl8IfGY=','w4bDjXNFEMK0bUV0','6L6/5o2W5Lmb5Yi25p2s5YuQ5Zq/77yq6K2z5qCr5p6R57+O57mM6YWY5peZ55uX5b+g44C/','w5oUC8O0','RQVnwpA=','wpIEw6bCgTA=','cRxjwprDrwYdYA==','wqNvw6vDk8KT','OsOQwoBKw6/DuMK+w6o=','Hx5gAWI=','wqjDkMOjw4jCsA==','dMOgXGl8IMOzUA==','w57CssKZXMOOLHDDlcKewqY=','woxHOsOq','w4AXw7Qp','wpRgwqtJw7HDn0DDnw==','w6LCscKZVsOT','ATdC','w4DChTrDlcOqw441AQ==','worDlcOQw6rCgxTDvMKGS1TDk8KuasOkw7w6','XjLDsybClcKUfAMIbVbDusKMesOMBgbCqFFlwqPClhHCssOww7s0wpvCiMKUEV/CgCM=','wpzCgwMOwplnwr8aJsOnw4zDhRTCnx3CqMOrw4Zswpc6FhnDr8OmS3PCkcKB','w6gOwo4Iw78=','w53CjMOFw4Eq','OcOfw6nDmsOxw6U=','w5gcwr0Xw4nDjkpqwrXCvg==','wrEfw6Bi','LjbCscKww7ZLRcKLRgdoTcOfKsOFflgjw7QiJcK2','fsO/LMKzCQ==','H8KPOMO2w79Ww7PCv8Kvw6g+w6ssIU8Lb8Kcw4Qdw6UFHGg=','azwiwq/CkFDDo8KbA8Kj','NGjCmWI=','BD3DqCrDjsKdKlkaL0vDhcORI8OAURXDsV83woDDiAHDtsOgwq1mw4DDjw==','bcKHw74cDQUd','YyUiwrLCqUzDlMKXAcKl','wq3DlgPChcO5wojChi8=','eMOUw61+V8Oow5bDvsK4OQ==','MMKpwr/Cg8KVwq5FdDwkEQ==','wrgfw6R8LyzCp8OKAm7DrHrCkDM9wr8=','bsOJw7BEV8O+w6g=','DWTClWJ0al3DgUvDs8KfwpJBwpbDosO0cw==','wownw4J9','wpXCkUAS','w4vDlcOjwoBdwqJHJcOowpbDvsOX','w7hJwpJcw4puwr5Vw67DqFYQAivDm8Oow5PCkMOIwqjDkcOmwoHCuE7DucOYVsKScA==','NCnDq8Knw7NQYcKNRRkmcMOL','HjzCqC7DjcKcNEAR','w6XCmirDgMONw4EbC8KEbcK1NcOt','XjLDtmrDnsOaMQUIaxjCsMOBfsOMCgbCoVFrwqPClw==','w5XCqmPCnw==','wpNiw7LDhcKxw6zDi2k=','wp47w70=','w7DCg8KFw5BsHsOFZ8KWwr3CqCw=','w67DhMKEJUgnwoozYWl5','wrrDlcOaw77CvCXDvMKY','6L2j5o+B5LuZ5YmU5p6w5Yic5Zmx77yf6Kys5qCd5p+p572E57iZ6YaQ6KyH44KW','FyDDsMKkw41RUMKG','w4DCoW7CjA==','IMODw5TDmQ==','R8O3CMKQ','w6sYDMOpw4TCkArCt8Osw78=','w6wXw748Zw==','w5Fxwqd2w7Q=','Wzcuwq7CqkLDncKH','wpc6w4HCiQM=','OsO7GVN4','w5Qxw5s2VQ==','ccKCVMOGWQ==','YT0BwoXCqA==','wq59woxew5nDkV/Dn29MU8KC','w4bClynDgsOzw4o6','w7wXwr8yw5zDkWx2','w55Cwp5B','D8Kvw4PCsw==','w5FFBxY=','w6nCr8OhIF1dwqoE','TldMwoHCmw==','w4k5w5Y=','EcOqw5bDj8O8','fSE/worChU/DhcKH','w7QOw7AjS8KQUzI=','K0fCs1fDt8OQw7/CisK4wojDnA==','w53CnzvDgsOww440Dw==','w7g2w5M5cQ==','EcKGJW4=','wqZRw7bCvcKyQMOPw7VASg==','w7MYw6Y0ccKnXzvCrcKG','YcKWw64fFA==','KMKSJkkv','w5HCpHzCgA==','w60Uw7s6','w4XCly3DhsOJ','woFgw7DDicKZw6PDqG1vwpA=','w44uwodpXcObTMKQYRTDvBxS','LXfCnGVl','esKLw7gkDQQNw4A=','Q8KoY8OAaQLCmMOW','JMKtwr3CtcKnwqhIXDE7','wprDmMOBw5/CiSjDpcKW','w4TChcKUw7ZmOcOWeg==','I8OJw5PDu8O6w7pQZA==','CSvCsx7Dg8KcKA==','wodRw7HCjsKnesObw7w=','wrwVw71BNCfCocONM3I=','wpfDm8K7csOiwr7CtMO5','6L2i5oyE5Luv5Ymn5pyg5Yu15ZqK77276K+35qOd5p+957+J57qe6YW+6K+X44OC','w6tswrBww7I=','wovDqsOKwpF0','w6fCtMOlE2Q=','OMKhw53Coml+csKSw7w1w7N9L8KbwpnDqcKfwq/Dpx7DhjvCtMK3VC9jWn4W','wonDu8OWwpE=','d8KsesOzVz3ClMOAwpfDqcOmwpXDgsOIccKGZVwzwopoHA==','LSvDt8K0w7NfTsKLUhE=','w61eBgnChcOCwq9vFA==','wpU8w5hhBMOGFMKFKA0=','w7UcOsO0w4XCjQ==','w5PDhsOrwohtwodbH8OywoPDpsO8J8KkSz7Dix9Vw4M7wpE=','PWbCnGA=','w7oFwrkBw6LDqXx+wqvCgn9wwrEZwrfDp15CMxZ4VcOsOg==','wo8ww4zCkBR3b2sY','Q8KoY8OSbR3CmcOawo3DrcO/wqPDhMOI','D8Kyw5/Cs1lDZcKWw7w=','wovDkMKNUcO3wqbCrsOy','wrgjw47CgSRXc3YNw7pJWmTDvyMHAMKYwplAJMKmw7U/','w6/CtsKGVQ==','w57CjsOGw5wxwr8dTHc=','w5QKwr0Qw7DDmHRxwr7Cr38=','wqPDn8KiQcOcwpHCqcO9wrpvdhovwozDlcOAw6gnPjzCpMKEwr90KEIuw4FbKQ==','FsKde8Oq','wp9hw6XDlMKCw6HDg35MwpRYLA==','w7zCpcKFTcOVDmjDicKO','HsOMMF0=','w4rCmD7DisOB','OsOrA8Kcw4vCpUNCwoLCj8OH','w77CqW7CgCZeCsKIw60mDsKD','VB1pwpzDjhMJdUI=','bcOew6t5TMOvw7zDt8Kz','LWLChE5wU0jDg0/DpsK0woFzwoM=','w6BKwp5Nw6FLwrVTw4XDu0sH','GWbCnWlOd1DDjlPDscKIwr9swoPDpMOiacO7w7rCmnsHwovCm8KZwqlc','LGLCg2ljUVnDu1jDtcKUwpN4woPDpQ==','w4zDk8KNI0IgwpsHZQ==','w41Nwoxcw6dRwrV1w7nDu0gRATrDjA==','w4rChDrDisOLw5k9LcKHcMKhLw==','Rw5qwoQ=','wqLDkQbClMO+wojChyMmXg==','wrFhwqFPw5DDilTDilk=','w4jCpWbCgDtTA8KTw5ki','w4PCncOR','wpFJMcOcw4nDpnfDpMOOelkRe3FdHg==','w53CmcOdw7wswqoKT2JAFcKtw5bDrQ==','akJOwpnCsHDDnBBVVw==','wpjDv8Ocwo9obcKc','wp5xwq9Pw4vDkkjDiHJMSMKC','wpUhw6d8HsOOGsKANw==','wqrDnBvCj8Ol','worDlcOUw7vCiSfDpMKWXnPDnMKwSQ==','wp59wq9Ww5o=','w6bCkyvDrsOJw44/AQ==','wqfDvsKNTgfCoFbDikIXHsKIwph8','L8OONVlQDsKfwqjChRvDmMKUPVsN','w6HCr8OcP09Ywr0NwoA=','DhN8KFY=','w4DCiMKBw4xoL8OQbMKPwp3CqC/ChHE=','L8ODPFVH','w5fCssOrO1k=','wp1Hw5PCscK1f8OMw7VQ','wqFgw7DDj8KE','c8ONw6lo','w65JHT/Ci8OCwqJzFDPDgF0SKA==','e8OsCcKPKkbCiTnDqw==','fMOPw7BiUQ==','JWLCn2TDmg==','w7zCjcKFw5NrKcOWQMKTwrDCozM=','w4gaDMO0w4fCmwnCs8Osw7hRBzU=','MMOOwoBVw4PDvMKhw4ZkHhrDpQ==','QAPDvMK5woE=','wqvDucOOwpJ/bQ==','IMKhw5PCs1lFbw==','woXDmMObw67CnCw=','w4rCmzrDisOGw4oqLcKHZMKxIw==','ZxnDnsKewqDCp8OdwoMo','HMKPR8Oqw7Jaw5vCuMKtw7J/w40=','asO9EsKUNw==','w4wUFMOsw4TCiSHCpMOy','wpVbH8O3w5vDu2HDpMOO','L8OEM1xHFw==','YcKdNsOewr0tAQgM','ecO7AMKJIEHCmAjDrcKywplz','w5fCv8OiN05QwrwVwoDChiDDuEQQ','w6nCncOEw40XwqUCUw==','woQZw6doODE=','woZywqNew7bDkEvDlQ==','woXDn8OKwqRC','w6TCgcKNw5tAIsOCZg==','wptgwoJUw74=','w7ZFwrxJw4U=','VsKONMOPwps3Hhc=','b8Ouw6dfaA==','M8OJGMK8w4Y=','w5PCvsKEXcOfAg==','UsO2TVlTNcOqWQAvwo5NwpnDsw==','wqNKKMOqw5zDvmbDusOlVlEZ','LWLChEJwSlk=','FyB6MVc=','w7zCg8KIw597LcOHfcKYwqbCiCrCjGw=','QsKaOsOCwrg=','w6XDqsK2Jkc=','BVzChUbDow==','w7zCicKOw5psNA==','w60aFcOlw6LCkCLCuQ==','wqM7w59xCMOf','wrszw5xwJMOJHsKD','KsKVecOiw7Zb','woXClVg3wphzwr8W','wrvDncKnRcOxwrPCosOowq18Ww8nwps=','woQTw6FtLyjCt8ONM3PDp3XChzNz','bsOJw7BATMOtw6DDlMKmPxYj','B8KZdsOqw55Mw6LCs8KSw6t0w40k','WgTDh8KPwpzCo8OBwpYp','M2zCgnrDgQ==','w6fDjXRvFMKRb1hhw5g=','Sx9nwovDiBMJ','wprDmMOBw4vChCHDvsKXYVLDmcK4','wqN/wqtVw5vDs0LDnlk=','wqgVw71bPCXCv8O4OGjDg34=','F8Khw4PCkFdbd8Knw7cuw65q','wo83w4VGGcOCCMKtPAHCpAU=','GMOMLmtWCsKMwp3CjgDDvMKf','w5LCrnvCsDtACsKZw5cuDMKIc8KQwqs=','wprDmMOBw53CmiXDvsKAXFzDj8K4QsO5','KsK7wp3ClMKjwqVUQjQ6EcKILA==','wq5Zw4vDp8Kl','wrYRw7FOPD3CoMOVM0zDi3bCgTN5w7g=','woZRw6PCqsKjZcOGw5hWW10Y','PsKaOGIECQ==','wqsCw6Z4Mj3CrcOJMw==','w54MwrUXw5jDvnFywqnCvG9hwqAv','w4l+wow=','55ec5oiD5ZKI77+C','w54BwroQw5jDjw==','w5Z5wrY=','w4rCmcOFw5w/wpMiTn1M','BcKWOnIAKMKRVMOrwoY=','wqbCkV4fwpg=','wro9w515AsOQLMKVIg0=','woJKwqZ3w7w=','wo1Lw6zDl8KB','wq9pwqZ8w5k=','wo4zw59xAsOK','wrfDisKuSsOnwoDCoMOowq0=','L8OEHMKtw5M=','egTDh8KPwpzCp8OWwpIkQcOifw==','wqbDksOcwrhJ','CcOPwopXw5M=','w7DDqcKEEmk=','SRQ7wpHCvg==','wq8jw5HChRY=','wrnDnMOHw6jChQ==','Z8OyS3pFIMOn','wojDkcKsRcO3wrc=','wrrDkcKTTR0=','McKSJGcM','T8KPw74TGwkB','w7jDiMOlwoxGwrE=','w6cjwqUQw6U=','M2jChmlCU07DjkPDs8KSwpQ=','wo/DusK8','w67DqsKlHH8=','w4IYw7slYMKD','Ah9+InfDp8KRXcK4w4jDmcK6','Z8KjfA==','w4pCAg==','QB1nwp/DtQIIcQ==','EsOOEGJM','BW3CkGfDkQ==','w4fCksKBw4ldKcOcfQ==','IChOMGM=','bcKcw60FOA0Aw5E=','w75DBwnCj8OYwqJsJijDqkgX','KVDCuGTDs8OW','6Ky25bCZ6YSD5Lut5LiF5YiN5YqF5oqq5p6g77yh5L+45o+m5YWt5bur5rmj5omu546Z5aOK44Kj','PMKNXcOiw5A=','wrVZA8O6w6s=','wofDtcOMwphJd8KVw7TDvS8gcsKZwog=','RThqwpHDsA==','KVnCs3HDpA==','b8KKVcOjfBrCgsOdwpA=','Y8KucsO4bTHCr8OSwpDDqcOUwqPDhcOPbMKbbVE2wpl3','HMKhw5zCqw==','w7fDmmVBEMKVS3Nhw48=','bC0lwrg=','w5vCmsOAPEs=','J1rCoHXDhcOQw6XCjsKjworDkMKY','w7LCjUXCniU=','XcKiYcOzWxrCn8OSworDq8Ojwr4=','w7vCucO+G1NHwroywpXCkQvDvQ==','aVVVwrnCt3bDihVX','w7Vvw79HwozDghjDhgxRFw==','LWLChEF+UVnDvFrDscKfwoQ=','w5kBwrgQw5zDpF9hwrTCsA==','d8KsesOzQQDCi8Oc','w4/CvcOnM3VfwrkO','CQdKLnY=','worDv8K3Wxg=','TMOySVJW','akJOwoLCqGPDkAxGRg==','TsOjWHhbNcOn','woXDtcKLUsOp','wqDDjgbCkcO7','AyHCsT/DpsKBLFcbMgXDqsORMw==','W8OCw6VaVQ==','b8O7CsKPJGrCtjvDocKr','w6fDgsOywrlawqZNGcOhwo4=','SB3DmMKZwqbCscOIwpY=','wovDjsK7YsOswr7CrcOzwr9rZx0=','L8OKPExKCsKOwrXCjg4=','woDCmUc5wr4=','w4DChRnDoMOi','MMOHwoRMw4A=','LnLCg2Q=','McOew4LDqsO6w6JNZFLCqjcn','w7xeDDrCi8OCwr56AyTDqg==','SMK7Gg==','W8Omw4I=','w6/Cu8KPWMOI','w7LCqGrCmjdtLcKbw5AiPMKVQcKYwqEG','wq3DkB3CpcO2worCgw==','K3LChw==','RsKqOA==','w608wroLw7M=','w5UxwrcWw5o=','aHVrwrjCkA==','YArDg8KE','w7I4w74nQw==','FMKOcsOLw7xVw73CuMKm','w4nDkcKGNlkxwq8Ydng=','QMK/eMOiZxrClMODwoY=','5q6M6Lyy55uS5b+9fg==','SMKpUg==','w74Zw7QlZA==','wpA3w59yGcOP','w5sewp4tw4w=','wrIyw4fCsys=','wofDu8OCwr9sasKAw7/Dtwwkc8KXwpTCusOm','CjzCpi3DtsKNNUQ=','OMKGw5TCqns=','VMO+w7N3Yg==','w4TDhsO0woxf','w60aFcOlw6XCnynCsw==','ekhUwrnCvWzDhxZlVsOAw4TCrQ==','w7sLwrgIw5LDik1qwqvCuA==','HMKswqjCksKj','M8OEw4bDnsO+w5VNYFLCojo0woXDoQ==','w6HCr8OaOl1IwpAPwonCnQDDvA==','wo/DkMKBaSw=','w5PCpBk=','w6cvw5M=','HMKvw53Ct0RSb8KVw40ow4FuOcKKw4rCrw==','woNRw6fCi8KyecOcw7hSSnkPag==','A8OIKXFWCsKR','VsKKw60GDQ==','ek9bwr7CvUHDmwRAXsOHw4TCoEs=','csOkE8KIHQ==','YUN/','wojDr8KKWxrCtUzDjkI=','RcK9c8O3fAs=','QsOyVXc=','w6zDgMKFMg==','AsOIO0pHHMKU','wpVbGcOyw4nDq0zDpsOHXlIZ','DmvCkXV0VXHDgFzDsQ==','cFRwwrjCtXLDmgtV','MmLCnmtlTw==','D8ONw4TDmcO0w6RW','cw7DncKewq7CisOiwoEiSw==','w4QuCg==','JlzChnrDpA==','wqszw7hwFw==','w5ELwrcFw4nDmA==','w6fDgsOywqlbwqZHD8Oywo/DpMON','w7nDiXRDDA==','AsKcOHIEH8KjVQ==','eiEzwqjCp0zDnMKNHw==','wogQw4HCtT8=','E8KWJmoAEsKy','TcO6V356JMO3Ug0p','wqNsw6rDlMKTw6PDkn8=','w7rCvsKZUMOYFnTDqcKKwrE4woXChMK/w4LCicKJw6kFwrrDjQ==','cQLDncKewqrCoQ==','B8KGw5k=','HMO4J8Khw4s=','TRxQwoHDkg4SaUI=','NcOrD8Kqw4bCtEVgwobCr8OHMMOCa3TDtw==','F8K3w6XCkW8=','wp0gw5RSDMOTEMKJIA3CrQ==','wr3DlhzCicO1woXCjho9ScO7Ylwrw7RJwrR7Jmw/','w48NHcOyw5I=','I0bCm3/DoMONw7nCiA==','w7zCuMKZ','YsOtNcKUKFfCnyfDq8KFwpltwqTDgcOxS0w=','w5cUw6Y4Z8KdWwfCucKRwqQrw6DChjI+w5k1Xhsd','w4zDjsKR','MsKQM2gELsKVR8O3wo4rw7bDvsO8HcOFw5RRbMK8wo4=','woRGw6rCrMKpYsOXw6lQ','UcOhVm9dNcOnRQA=','worDkcOQw6jCmg==','SCzDs8KfwrvCp8OLwp0+','DS/CqzY=','wqJhwqtaw4vDm2rDuEhD','HMKyw5XCpkJSXsKTw60zw6xh','PBNtKUHDjMKhXcKiw4rDrsK9w7Q1wovDtA==','woxaJsOqw4fDpnrDuMOO','w6fDk8Onwp9G','A8OZPEpW','AsKfM2cT','wqwhw4bCih5cVHoOw4BlTGTDmjQA','wowgw55hAsOTAcKcNw==','QsKoesO5fgvCrsObworDoMOv','wpgAw5bCkA9seEgew5dTT2Q=','w45PDBPCj8OpwpR+AiTDkUkPKSbDgcKQ','woPCgEgfwoF3','M8K6wqbCksKtwr9eQjA=','fsOuAsKaMVc=','w7Iew7A/YMKufDbCq8KGwo8nw5bCjT8mw5M=','JyTDssKs','QMO3XV5EJMOwQSk0wrNYwpHDuMKMKg==','woLDs8OewplocA==','wpjDnhnChcOewofCjSU=','Oz3CoijDrMKJIFU=','wrvDjQDClMO4wp3Ckjo5','w5PCksKPw4pmOMOdecKY','wp8zw515','EcKBM2oOEMKzb8Opwpsbw63DpMO0B8OYw6ZScMKowo/CvQ==','OVDComXDpsOqw7LCmMKNwozDlcKJ','HMOCPFxxFsKPwqjChQTDmMKXOFkQew==','w40uwoVpX8ObS8KQYg==','CEHCuEPDs8OKw7M=','w6HClMKOw7dnPMORfQ==','QilQwpvDmw==','w65cBRTCng==','wqzCqmQQwpI=','w4PCv2HCuDtBGw==','w7jDgsOowopGwrw=','w4TDiXJBCQ==','w45PDBPCj8OpwpRwHjXDkVAQLCPDpsKMPzzChsKeMSrDhsOnw71h','O8OrG8KE','w6DCuMKLXcOqE3LDjcKewrEp','NMOlFsKMw7fCqVJ4wpbCkMOH','LQRmFE3DqcKG','w68/AcOZw5M=','w7kYHcOuw47CoQnCt8Oxw4VXByMkwq5ewpvCpcKdZsOuwo/Cmjon','w54WwrEFw4nDmEpjwqnCtHhwwrY4wqY=','wqJhwqtaw4vDm37Dik5EUcKCcsO9Lw==','wq8xw5R7CMO4NcKNIjfCqhIGT8KERsOEwpZUwppVwrDCggES','w5PCo8KLVcORDXjDl8KPwqw7','DyrCoxnDisKBIVQ=','csO6Q34=','WMO9A8KVIG3CvSjDvsKZwoNxwqzDicOhSw==','w6TDmm9UC8KEdUFw','wpjDnArCjsOywrbCpissZMO6a34lw6xA','w5/DgMKOOw==','wqlbLMOsw6bDs27DrQ==','w6UFw7wZTw==','VcKXMMOiwpg=','AMOYLlA=','w6TCvcO5Im9YwqUE','FUHCt3zDvcOTw77CgcKuwoLDjw==','w4JYCBHCgcOBwr9xFS7DuQ==','w4DCr2vCtzpbA8Ke','wrXDrsObwpFmacKdw73Dti42','dUZJwrnCi2vDiQA=','w7IUw680','wo8mw5BnGQ==','alNbwr/CrA==','w4DCksKFw599KcOjS8KIwqDCsiTCj3o=','TVVTwqrCv2fDgQBW','w4bClTrDicOBw7AVBcKZX8KnL8OoeMK1','wpwww4bChQ9mUVkIw5FOUH7DiA==','QsOLw4Z5TcOo','ekhUwr7CrHDDhgZGUMOW','wpTDjMKgUMOswqbCuMOswq0=','Bh5hM03DssKPVcKrw4o=','wr/DmhfClA==','wqTDhiPClsOYwpskVsKVNcKoYsO1MsK9Z3sobWISwr0=','MMK4wqXCj8K2','B8OrE8KMw67CrkF5wpfDkA==','wrvDnhzCk8OgwobCmS4=','w7bDh0NLEw==','R8Kkc8OiYA==','cwZowozDjhAp','w6/CpcKPWMOOH0bDkMKFwqcjwovCjsKxw5fCgMKU','fsO6V2tHNcKs','w7nDiXhsAcKea0V9','E8OfOFlWCsKwwrPChwDDv8KtMFARZ8K1','wqMzw5VxJMOJCMKZJg==','wpYEw51UMw==','KcKnIXUA','AsKVc8Oyw7s=','w5vCvsKEXcOVDUg=','Z8KONcOfwrc=','w7RfJBLCiMOfwrp6NSTDuFUcKA==','fcO/CsKOIA==','w6INwroUw4jDiQ==','ZMKPw7Q+CQYfw5FP','M8Oew4LDjMOvw7M=','w7bClzPDiw==','fcKAOMOOwoYwDBQMVA==','w7FDCBnCvsOfwqJzFHM=','w5rClcOdw4Q7w7kqXX9E','w6nCuMOuFVRYwrMF','w55Mwpt6w71OwrxF','VsKMw60RBzsIw5dOw7wdwpI=','w6/CpcKPWMOOH1LDlsKGwq4twpLCpsKHw4fCi8KCw7EX','c09Xwro=','wp9vw6vDh8Kfw6PDsWVswpFaPg==','KsKfeMOrw75Cw7rCssKWw7J/w4wvPw==','w4JABhrCg8OYwoF2HyXDoUs=','woQTw6ZhMCjCusOdAWjDgH/CjCE=','MXfCkW94U0U=','HVzCuHTDucOTw5jCn8Krwo7DkcKYwr0=','w5HDq0p0FQ==','FkrClQ==','JR9j','woXCgEAXwoE=','w60IwrUdw5jDjw==','w5rDgMKBMmQ6woYSeA==','w5bDh8KlIVk=','wpF+IMOfw6w=','dANnwpHDhBU=','w6HDjsOFwrxn','w5LClzLDgsOtw4E+Cw==','EsKWw5nChnI=','CMOmFsKRw4LCsg==','R8OyWn58IMOzUA==','w5LDgMKPMg==','ey0Iwo3CsQ==','wq8uw4LCnR5x','w7bDhsOywplewrFQIsOnwovDrg==','wpBvw6XDmcKTw78=','w75ACA7CmcO/wrI=','wpJBKsO1w4bDs27DrQ==','w7oXGcO5w47CjA==','AcOMwpFd','DmvCkXV0VQ==','w4wcw60dYMKHWzs=','w5QUw5YAUA==','w7DChyrDjsOUw5w=','wpvDkw7CmcOywps=','Hy3ChD3DuA==','worDnBvCj8Olwpo=','wr0Rw6ppEyjCucOc','w77CmMOAw7A2','G8OHJMK6w6Q=','wonDn8K/bcOn','w61ZGhU=','YsKHeMOfRA==','wrrDu8OIwpxg','e8OsCcKdLF7ClQ==','wq7DnMOYw6zCoSrDtsKc','Z8KGw58CFg==','wqjDscKEVhDCsw==','U8KldsOkaQ3CmcOWwpHDhcOlwq7DjsOe','KBFlIm3DvcKFUw==','wqHDkMO7w4rCmw==','RkRVwqDCtWPDnQFlVsOKw5TCqk4=','NyDDqsKIw7tQRsKOTQY=','L8Knwq7Cj8Ks','SAjDnsKHwqLCssOKwpcaT8Okb8O1wrY=','wrNmw7DDqMKXw6PDgmBnwoc=','acO3CMKf','w7HCn8OGw4UzwqoKWEVICcKsw5fDrg==','wqgVw71EPCfCsMOVM3M=','wpZdw6vCvA==','YMK+McO8wpY=','w49qMxzCgA==','w5LCrnvCvDNcC8KWw4Y1','w6fDicOvwqlB','wp9gw6vDjcKbw6zDiGhVwpxbLVoU','wrJ2wrpzw57DkEnDlllf','w6PCq8Oiw7wU','N8OCFVdPCg==','VsKNw6MfAQkWw4Fww6EWw4QUYg==','wrBHLsO3w4bDkWLDpsOIUlA=','wqnDlgHChA==','c8KIIMOQwqI=','wpnDuTXCgcO9','QsKOL8OPwps3Hhc=','PW/ClW96d1DDmk3DvcKUwpNYwo/Du8Oi','wrFgEA==','wrHDjsKrRcO3wrfCkcOwwql3cBw=','w7TCu2vClSZXIcKbw44i','wpTDnADCjcO6wojChS4LUsOhf3Uz','fSE/wpTChU3DlMKOCMKy','w4XCpHjCmj5dDsKe','f8OFw6pp','wpnDv8OOwrVscMKQw7/DtzM=','EmjCl2V/ZF3DgUnDscKW','w57DiMKMMw==','wqnCk0MTwphzwrYXH8O9wo3DtC/CqQ==','HSvCsxLDg8KGKVwRLg==','DcKcN2IGEMK6Qw==','bcO8WH91IMOzUA==','w4HCicKOw5o=','w4/CmMONw783wqUAU2U=','w4LCuWrClSZXI8KVw4QuDcKxXMKXwrcdWA==','L8OBMl9LAcKrwrXCjg3DvsKN','wp5/wqFcw5bDkHrDk1JJSsKQ','w7vDl8Onwo5bwqBb','QALDn8KOwqDCpMOrwoMsRcOjf8Oj','HsKkw5TCkF9ZeMKJw64=','w49awpBNw7pTwqlRw64=','B8Kww5bCjnc=','w4zCk8ORw783wq8QVA==','LMK4wqjChcKrwr9e','wowZw6doMj7Cm8OJN2LDh2/Cmg==','USckwrLCgkrDgsKPOsKpw7VfB1Q=','w6fDjXRoBcKeaF1ww5M=','woPDlwfCiMOg','w51BwpFd','w74ew7o/Y8KYTDrCj8KKwr42w4nCng==','wpPDvMK7fcOi','fsKBGsOFwrw/EQoEJsOXw5AUwrbDhA==','wqAhw4zCih1qZHYqw4xUW3/DjA==','JzfDu8Khw65bYcKNRhIubMOCDcOHfg==','w7PCj8KQw4t5DsODbQ==','w74ew7c2YcKiTiXCscKXwrU=','L8OOP19GPMKMwq7CiR3DtA==','J1rCoHU=','w5bCo2bCuTA=','w7bDiMO+wqVXwr1FBMOy','D8OPw4jDg8O9w79XbHfCqjckwo/DpA==','wpRNIMO5w4DDpg==','AsKBM2cVFMKCVcOhwpk9w7HDtsO6','OcOFFg==','KWLCkl9lSE7Djk3DscKxwoVn','DB9lKkvDvQ==','JsO4O053','wrhwBcOYw48=','bMO7EsKyMVfCnQ==','w4rCnzHDl8ORw5s=','YhjDlMKY','woxJOsOt','w5oJF8O0w4TCij3CpsOk','w6DChjvDhsOQw4o=','w4DCjB4Cw4Buw6wPecOow5DDrHA=','KzjCjQ==','ZBvDncKDwrs=','wpUhw7J0A8OEHcKAPg3CrQ==','GybDscKtw7dfTMKGfx0pesOAOA==','w7xPHRTCnMOXwqJ6','wpXDhcKEXA0=','PGjCiFt4Q0jDhw==','BgNYNUHDoMKQWcK1','KcOWw5LDhcOw','W8Kvf8ORbQ==','UcKnQsO6Qg==','I8KSZ8O2w7s=','wp0xw4V8G8OGDMKJ','LlDCt3PDosONw6HCjsK+wog=','wpTDlgHCkMOiwp0=','FsOCPk1R','w7jCksOZw5g2','XcOwFsKLLQ==','wqZAJcO5w4k=','w7UXF8Onw4LCkBPCv8Ovw75bAg==','cMKMLcODwqQ4DB0=','w44eGcOjw5/ClzLCt8O1w78=','PsKaOHYUBcOl','aCsowqnClw==','QMOwTXJEIMOqUA==','wqUqw4/Cgxo=','w7YUw7s1asKGZw==','aMOcw6BsV8O+','dArDncKG','w6TDlcOpwpldwqBbHMOj','EmjCl2V/aFc=','w6DCk8OYw4Eu','w4/DkcKOPlk=','RwB0wozDjhER','A8OYw4bDmcOuw6VnYFI=','fwLDlcKP','PsKQOWsMEMK5QsOTwoIaw7vDv8Oi','w7fDhG9TAQ==','QisswrXCinHDlcKWGMKyw7U=','ccOySmhFLsOsUQ==','I8KEZ8Okw4I=','SALDn8KawrrCp8KW','SMOfw6F/bcO6w6jDog==','wqM7w59lGMOT','w5ccw7kkYA==','QsOPw6tgTsO6w6vDo8KBMx0jbsKa','w5wHwqANw4vDnG12','wpTDkwDCh8O+wofCvCMyX8OgbA==','w4bClcONw40=','wrgtw6vCixZm','KUbCvg==','w4fCj8Okw4c8wqIIWVZEEcKhw5vDvA==','w54Xwrw=','w5HDgUFVLw==','wqNMKMOqw4k=','wqd6wqJPw5rDjA==','w6XCpMK8UMOJE3PDlcKO','w4vDisOjwoBQwrFQJcOowoLDrsOb','w4zCqnfCtjNGG8KWw4YKBsKLV8KcwqEB','dA50wonDjA==','RsKKO8OZwrstHUooNcO9','w4HCjMOMw4Y=','F8K0w4TCt0UNM8OJw7slw7AhIMKHwpHDvsKuwrjCpQjDj30=','w5BYwppXw5BfwqREw7nDtEcO','BjrCsyrDkcOSYh8WPhfCqMOXIsOdVhTDtwMwwrHCiw==','w47Cu2rCmg==','HwJnM0vDp8KaTMK0','KVnCuWPDsw==','KiHCsDTDjsKHLFQ=','wo3DjcKCS8OhwrvCrcO5woxrYwcpwps=','AMONw5XDjMO2','J8OIP0tLG8KZw67CoTnDmg==','P8Ocw4LDgw==','w5PDiW1FKMKZYlo=','wodiw6nDhcK6w6TDiGc=','YTQuwrI=','OMKhw53ConpecsKN','QCE8wpvChU7DlQ==','w49Ewp5Aw5pM','wo3DtcOOwpI=','DMKmwqXCn8KUwqpLRzA=','YsKKLcO8wrM1DR0=','FCTDrMKhw7c=','w5rCmDPDnsOyw440EcKM','RCnDlcKmwqM=','wq43w5dnCMOUEMKoMxzCqA==','fUJJwqjCtGfDkBE=','wp5wwqFWw5LDn0PDnmtES8KDbsOv','w4rCmcOIw4sqwqISXWZE','AsKBM2cVFMKUScOqwo0dw63DvcOXDsOI','w7vCucO+El1Fwr4=','56Cb5a2m5YW/5pS25rqr5ois5paq5b+V5bSz5p6S5a2J5qOL','MMOAwopWw4fDsMKhw6JdExHDuTLCmw==','L8OOMlZEBsKOwrHCtwDDv8KeNkk=','wrNmw6jDhcKVw7k=','w5nCmT7Dg8Ojw441AQ==','bMOMCw==','eEnDp2zCpsOYwqPCk8O5wpHCjcKQw7I=','wrF/wq9Cw7DDlQ==','w6wtwpwTw5I=','wrJjwqJSw4s=','w41NwpNWw7RDwp1Aw7vDk0A3FzvDn8Ouw6nChA==','wppJLcO7w6fDp3fDicOHWw==','wrrCn00awqJ3wro=','w7BOwpl1w7xJwrU=','wqzDlsKsWC0=','w6xJwolcw5xJwrZO','wo59wqJCw6nDn0HDj1k=','A8OIKW5DA8KJwrk=','F8OkG8KRw7HCoV15woY=','w7TDrsKDJng=','wqjDjQrCgcOjwozCrCsxXsOAeXAhw7tRwqg=','IT3DqsKyw7tdVsKxSQIiXcOAIcOUf142w78=','wonDqMOfwpx5e8K3w7zDvCcobMKYwrPCr8Ox','HMKrwqbCiMKkwqJVXwIhGsKCN38=','Sx9jwoY=','wrvDncKgScOuwrPCr8O4wp9newolwok=','VsKNw6McCgEKw4hww6EWw4QUYg==','eEROwqTCrmPDhwA=','w4rClTDDicOCw4YqCcK+acK6P8OmfQ==','Q8KoY8OSaRrCjA==','5rGF5p635a6X5qGH5pqs5ZO05b+B5ZKz5Yag5pa+5rit5omh','wpnCnm8Rwpt0wrEBJcObwog=','fsOwVnVUKMOsWDI0wq5IwpvDoQ==','w6zCucOrNUhYwqkAwpHCkQ==','woTDkQPCmcOBwojChz85','wo83w4VDDMOLDcKJ','YMKsZcO3ZQ==','WAXDncKTwpnCssOIwoYo','w7Icw6M0TMKfWDg=','woHCsX0Owro=','b8OHw7U=','wqkRw4dEJw==','w5XDm8KuGnw=','OF7Cpw==','woo7w4J8D8OLHcK8MxrCvRklQcKcT8O4wpFDwoFS','w7vCs8OnMw==','w57Ck8Oa','w4bDgsOgwp9XwqdKKMOnwpLDqg==','YSoIwrPCikXDmcKQAMKDw7pVC0bCow==','wphNKMO9w5zDu3XDqcOfUg==','w5fCv8OlOFpYwq0MwrLCnQDDvUYC','csKDNsOZwrc=','OFDCu3/DoMOBw5TCh8KjwoHDnA==','fsOwW3xWEsOuRwwpwqU=','MS3CqDfDj8KJI1QjNQrDosOSPQ==','HcOGwolXw4DDvcKew656MxnDiC3CiHVlOcOo','wrpGw48=','4puv4pqf4pq/','w7FJDwk=','HMKlw57Cs1NF','wpLDm8K9V8Oqwr3Cr8OVwqw=','HsKtw7LCg3M=','w79kMzrCmQ==','w6tqwrBXw4E=','ecOew6V6d8O+w73Dsw==','Z8O7AMKP','w44JGcO3w7/CmzzCog==','w6zDlsKPGmY=','wqNaw67Cq8KC','XjMmwpHCrw==','S8O2w6dKbA==','aC8ZwovCog==','w6TCucOsIg==','dg5owoPDiAkX','XMK9WcO/bw==','P8OCwoJd','w7HCgcKOw5VgIsOD','w6jCpcKLTsOuH2nDjQ==','XSsEwobCtA==','w5LDg8KEA3U=','wqsHw6l6BA==','w7PCgcKHw5s=','HMOdw6rDqsOo','w4DCiMKBw5BuKcOwbMKFwqDChSTCjWbDog==','woTDtcOIwpBscsK3w7zDvi4z','wphaKMOpw7zDt3vDvA==','Q8KON8OBwrs3Hw==','LAVhBFU=','w6vDtMK6OEQ=','wpcjw43Cox8=','w7oaH8Ol','O8OvGcKcw4LCsg==','LsKpwrnCr8Km','w5MYw6QkYMKCShrCucKTwoI3w4rChj82','NDfDscK0w7VKW8KSTQ==','w5jDiMOhwoRcwoZHGMOzwpTDpQ==','5ay556GD6Za66KyL6K6M56Gt6K6y5a+d56G85YSo6K+v','YMKPTQ==','DMOMwohVw47Dtw==','w5PCj8KTw4o=','e8O1w7Y=','Pzh8wqDDll/CgMKeXsK8wq9HXV/DuQ==','ADbCgA==','w77DkGc=','acOJw7x5','f8KOb8OfXw==','E8KpwrrClcK1wqRVVg==','NMKfY8Ovw7xNw5rCt8Ksw74=','Ax9vLko=','55aC5omM5ZG15p6q5aOA5YWn772I6Kyh6YWM5pSq6L6n5YSQ772H','OA/CtTnDkQ==','wpjDscOdwohm','w4zDkcK6B18=','5p246K+O5Lmk5YmN6KyP55ae5ouh5L++5oCe','XMKdw6kAIgkVw4A=','KwXCkw==','NsO6N2tV','woQdw6xhPyzCpsOwOGXDi2M=','ZMKPw7QwDRwMw4lCw4Udw40ZcAh4','A8OMwoZZw5XDvA==','LMOfP8KEw7Q=','BcKMT8OWw6E=','5a2o56O96Zaw6K2H5Lm65q+q5YiN5Li15qyz5pyb5L29','ZcK+csOkRg/CgMOW','w4TCqsOOw7IP','SsOYQA==','w4EwAQ==','wrvDjzfCsMOl','5a2556Cp6Zem6K+W5Lqc5qyG5Yq55LiF5q6q5p6g5Ly8','wownw5fCqwtidXIJw5w=','woTDlyjCrcOu','GlvCj1nDgA==','YjhswrvDlg==','w6TDl8Oewr1A','5a+P56Ov6ZaV6K6d5Lu45q2J5YeD57m2FsK95Ym+6ZC/','TFRfwr/ClmPDngA=','w7DCgcKWw5tAIsOCZg==','K8O6G8KBw5M=','ecKew5QiHg==','w5/CtsKcXMOzFHfDlg==','RyoiwqjCjULDnMKLF8Klw4tXCVrCqhM=','wqhgwoNUw53Dl0HDn3hIU8KOYsO9','QBtQwpjDtw==','wpdGw6DCucKyc8Oiw7ZSRlw9enE9aMKt','KUfCs3HDosOBw5TCgMKnwoDDmcKCwqDCiHRzwqjCtcKR','DBxtJlbDkMKMUcK8w47Dn8Kqw4w9worDtA==','EsKhw5vConVYccKLw7gpw6dDI8Kcwog=','w4XCucKS','R8K+w4YRKA==','wqzDn8KdS8OR','YyUzwpDCgU3Dl8KWBQ==','wq3DrsKAXTvCoFjDmw==','VsKPw6gWJQYIw5BT','w7zCucOyIg==','KW7ClHh5','w5RoAjPChw==','wpzDlgHChMO4wp7Csg==','SALDn8KawrrCpw==','w4rClzvDg8Otw4EoEcKdMg==','LsOCwrZMw5Q=','GSfCoy7Dig==','A3HCvV7Duw==','w4caAMOMw47CkCPCosOp','a8ONw6h4Rg==','HMOQHsKCw5A=','F8Klw5nCoF5D','w67DmHJKEg==','wqnDkBfCqMOywoDCjCIo','wqtXw6rCtcKrd8OAw71iRlwOfGg=','wql2wqdcw5fDig==','IsKwWcOMw78=','wpTCn1Q2wpB7wr8bPA==','fw7DmMKNwqfCpw==','w4DCksKFw599KcOnZsKQwrnCpyXChV7DucOqwrFmbw==','w7fDmmVBEMKV','E8OfOFlWCsK1wrLChgbDhsKTN1oafw==','wqDDlDBcU8KMPk0gw502RsOZfn0=','EsKDOm8V','wrnDnMOGw7rCnyvDosKX','GnfCuVzDuQ==','XAUBwo7CvA==','w5bComvCgDo=','w7ZswpR3w7g=','MSfCqSrDl8Kc','w4vDjsOowp1HwqAQ','wqAjw4fCgDJtZm4J','YALDlcKewqc=','RwAgwpLCiQ==','wqtBJ8O6w4fDpVo=','wpXCgkkfwoF3wpscJsOyworDoi3CiSvCi8Oqw5po','C8OGwoRbw5XDsMKlw65+Hw==','wp4hw5fCjQ1iYn4=','H8ORwopMw47DrcKqw79v','KUfCs3HDosOB','woPDtMOTwolkf8KYw7rDqCQ=','fgXDmMKewqbCssOIwpo3Qw==','w5xJwpNV','w7NZBA==','OcOLDcKPw6o=','a8KIw4EdJg==','OsOlD8Kgw4LCqVZkwpc=','wrBxw6vDlMKZw7nDn3xn','wpdGw6DCucKycw==','EcKBOXIOBcKuVsOh','wpdVw6nCtA==','wonDqMOfwpx5e8K9w73DtC4Wd8KbwpXCp8Oi','O8O4EsKJw5PCpXJjwo7Cj8ODM8OEWW/DqlrDsmw=','w6PDgsKNOkA1wowTV3R2w7wtw4Q=','w57CmhPDn8Ov','w73CkzbDgMOMw5s=','w4txwqk=','w43CrmHCkyZa','WAPDlsKYwq0=','wpTDs8KMVwM=','wpfClEg9wp17wrQX','F1XCpGZ+','BHvCgUTDhA==','wqVWwolZw68=','BcKOeMOyw7xXw63CpsKk','wrgCw6xtKSzCncOXMG7DuXLCjTJkw7w=','wpPCnMOSw4I4MMKQdcOOwqjDsw==','HT7CqzPDlg==','wojDk8Oowp9l','wpTCn1Qpwpx2wqwb','w4JFBxvChcOhwr9xFS7DuQ==','w67DgMKMPGIkwoMUaWlh','DhRsEE3DvcKHU8Km','wqNBJ8O4w4fDhWrDpsOPWEs=','w4LCuWrClSZXLMKVw44qAsKIUcKuwrocSxpn','w5oaH8Olw4/CkTPCuA==','PWbCnm90Sw==','TMOqSXpVJA==','IMKTYcOWw6AlTQRYVcOKwocLw6fDlMKfEMKgB8OZHVU=','w5PCtMKFVMOXG3/DncK8wqoiwpjCrcKn','A8OIKXBDAcKYwrDChRs=','JcKaw6HCs14=','wosRw65pGSbCo8OX','H8O4DsKgw5A=','DR9wEE3Dt8KXVA==','wrbDnsOaw6TChSXDvsKXe1TDk8K5Q8O6','w6bCkzPDgsOHw5s=','w7/CssKeccObFHXDlcKOwrE=','w6TDiWdFEcKA','wq8jw4TCgS5z','wrXDucOVwpBgf8Kaw7fDhSgvesKawoY=','w61JwpFSw5pXwrFCw6LDrl8=','wqB3wqpsw5bDkEnDlUs='];(function(_0x36d82c,_0x1bd2b3){var _0x161433=function(_0x1d86a1){while(--_0x1d86a1){_0x36d82c['push'](_0x36d82c['shift']());}};_0x161433(++_0x1bd2b3);}(__0x49358,0x164));var _0x24d5=function(_0x4d47dc,_0x41c5ec){_0x4d47dc=_0x4d47dc-0x0;var _0x52fcc7=__0x49358[_0x4d47dc];if(_0x24d5['initialized']===undefined){(function(){var _0x112cbd=typeof window!=='undefined'?window:typeof process==='object'&&typeof require==='function'&&typeof global==='object'?global:this;var _0x36027e='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';_0x112cbd['atob']||(_0x112cbd['atob']=function(_0x3f8be5){var _0x447471=String(_0x3f8be5)['replace'](/=+$/,'');for(var _0x29aa55=0x0,_0x524796,_0x3381a4,_0x368f91=0x0,_0x3e45c5='';_0x3381a4=_0x447471['charAt'](_0x368f91++);~_0x3381a4&&(_0x524796=_0x29aa55%0x4?_0x524796*0x40+_0x3381a4:_0x3381a4,_0x29aa55++%0x4)?_0x3e45c5+=String['fromCharCode'](0xff&_0x524796>>(-0x2*_0x29aa55&0x6)):0x0){_0x3381a4=_0x36027e['indexOf'](_0x3381a4);}return _0x3e45c5;});}());var _0x5d1188=function(_0x5a349b,_0x590c8c){var _0x10fd7f=[],_0xf80af1=0x0,_0x1a9eab,_0xf20bc0='',_0x589399='';_0x5a349b=atob(_0x5a349b);for(var _0x5b7fc2=0x0,_0x426cad=_0x5a349b['length'];_0x5b7fc2<_0x426cad;_0x5b7fc2++){_0x589399+='%'+('00'+_0x5a349b['charCodeAt'](_0x5b7fc2)['toString'](0x10))['slice'](-0x2);}_0x5a349b=decodeURIComponent(_0x589399);for(var _0x47948d=0x0;_0x47948d<0x100;_0x47948d++){_0x10fd7f[_0x47948d]=_0x47948d;}for(_0x47948d=0x0;_0x47948d<0x100;_0x47948d++){_0xf80af1=(_0xf80af1+_0x10fd7f[_0x47948d]+_0x590c8c['charCodeAt'](_0x47948d%_0x590c8c['length']))%0x100;_0x1a9eab=_0x10fd7f[_0x47948d];_0x10fd7f[_0x47948d]=_0x10fd7f[_0xf80af1];_0x10fd7f[_0xf80af1]=_0x1a9eab;}_0x47948d=0x0;_0xf80af1=0x0;for(var _0x387ece=0x0;_0x387ece<_0x5a349b['length'];_0x387ece++){_0x47948d=(_0x47948d+0x1)%0x100;_0xf80af1=(_0xf80af1+_0x10fd7f[_0x47948d])%0x100;_0x1a9eab=_0x10fd7f[_0x47948d];_0x10fd7f[_0x47948d]=_0x10fd7f[_0xf80af1];_0x10fd7f[_0xf80af1]=_0x1a9eab;_0xf20bc0+=String['fromCharCode'](_0x5a349b['charCodeAt'](_0x387ece)^_0x10fd7f[(_0x10fd7f[_0x47948d]+_0x10fd7f[_0xf80af1])%0x100]);}return _0xf20bc0;};_0x24d5['rc4']=_0x5d1188;_0x24d5['data']={};_0x24d5['initialized']=!![];}var _0x414d81=_0x24d5['data'][_0x4d47dc];if(_0x414d81===undefined){if(_0x24d5['once']===undefined){_0x24d5['once']=!![];}_0x52fcc7=_0x24d5['rc4'](_0x52fcc7,_0x41c5ec);_0x24d5['data'][_0x4d47dc]=_0x52fcc7;}else{_0x52fcc7=_0x414d81;}return _0x52fcc7;};var _0x287dd5=function(){var _0x3af273=!![];return function(_0xd99fa8,_0x453bff){var _0x312bd8={'oUcLQ':function _0x4e7688(_0x192460,_0x34320e){return _0x192460!==_0x34320e;},'drdQH':function _0x4ec23f(_0x2508cb,_0xcba15f){return _0x2508cb(_0xcba15f);}};if(_0x312bd8[_0x24d5('0x0','&$JW')](_0x24d5('0x1','0^S3'),_0x24d5('0x2','&$JW'))){if(ret){return debuggerProtection;}else{_0x312bd8[_0x24d5('0x3','kS8t')](debuggerProtection,0x0);}}else{var _0x57085b=_0x3af273?function(){if(_0x453bff){var _0x59ea21=_0x453bff[_0x24d5('0x4','YbX8')](_0xd99fa8,arguments);_0x453bff=null;return _0x59ea21;}}:function(){var _0x3abef8={'NykBX':_0x24d5('0x5','SuRN'),'RgpbM':function _0x2bc431(_0x4069ad,_0x10ceb2){return _0x4069ad<_0x10ceb2;},'DTLcc':function _0x5340fa(_0x123890,_0x1a29c2){return _0x123890<_0x1a29c2;}};if(_0x3abef8[_0x24d5('0x6','4$cG')]===_0x24d5('0x7','YbX8')){}else{if($gamePlayer['isJumping']()){for(var _0x7e4f30=0x0;_0x3abef8['RgpbM'](_0x7e4f30,this[_0x24d5('0x8','0^S3')][_0x24d5('0x9','1eUd')]);++_0x7e4f30){if(_0x3abef8['DTLcc'](_0x7e4f30,$gameParty[_0x24d5('0xa','tBWv')]['length']-0x1)){var _0x494a47=this[_0x24d5('0xb','PNVJ')][_0x7e4f30];var _0x5f118e=$gamePlayer[_0x24d5('0xc','$RCy')](_0x494a47['x']);var _0x15c887=$gamePlayer['deltaYFrom'](_0x494a47['y']);_0x494a47[_0x24d5('0xd','eOf4')](_0x5f118e,_0x15c887);}}}}};_0x3af273=![];return _0x57085b;}};}();(function(){var _0x2a7011={'QTqjJ':function _0x3ce42c(_0x1307ea,_0x1c4989,_0x170a29){return _0x1307ea(_0x1c4989,_0x170a29);}};_0x2a7011[_0x24d5('0xe','AB(m')](_0x287dd5,this,function(){var _0x352f22={'uwROY':function _0x4ae29c(_0x188e51,_0x2b39e8){return _0x188e51===_0x2b39e8;},'MVazg':'Gev','JBUvr':'function\x20*\x5c(\x20*\x5c)','BBosQ':_0x24d5('0xf','&$JW'),'Onjdn':'chain','pGiBf':function _0x5a502d(_0x3ae959,_0x352986){return _0x3ae959(_0x352986);},'zADlv':function _0x126c46(_0xfcb95d){return _0xfcb95d();}};if(_0x352f22[_0x24d5('0x10','wcna')](_0x352f22[_0x24d5('0x11','&$JW')],_0x352f22['MVazg'])){var _0x321267=new RegExp(_0x352f22[_0x24d5('0x12','btPv')]);var _0x52d410=new RegExp(_0x352f22[_0x24d5('0x13','tBWv')],'i');var _0x4aeba0=_0x14a401('init');if(!_0x321267[_0x24d5('0x14','SuRN')](_0x4aeba0+_0x352f22['Onjdn'])||!_0x52d410[_0x24d5('0x15','SGV7')](_0x4aeba0+_0x24d5('0x16','qfib'))){if('rJI'!=='wfW'){_0x352f22['pGiBf'](_0x4aeba0,'0');}else{alert(_0x24d5('0x17','$rYi'));PlayOnline['UserName']=null;}}else{_0x352f22[_0x24d5('0x18','1SDz')](_0x14a401);}}else{return this[_0x24d5('0x19','1SDz')]()[_0x24d5('0x1a','Cf54')](function(_0x36a56c){return!_0x36a56c[_0x24d5('0x1b','SGV7')]()&&_0x36a56c['pos']($gamePlayer['x'],$gamePlayer['y']);},this);}})();}());var _0x540cde=function(){var _0x644818=!![];return function(_0x9aa334,_0x173e55){var _0x510882={'EfRdY':function _0x3736fb(_0x47932b,_0x1719a7){return _0x47932b!==_0x1719a7;},'muiuK':_0x24d5('0x1c','kBJi'),'kGKEF':_0x24d5('0x1d','AB(m')};if(_0x510882[_0x24d5('0x1e','Y*3L')](_0x510882['muiuK'],_0x510882[_0x24d5('0x1f','AB(m')])){this['drawText']('读取到您之前的记忆，是否恢复记忆？',0x0,0x7d,this['contentsWidth'](),_0x510882['kGKEF']);var _0x58594d=new Date(PlayOnline[_0x24d5('0x20','VCQB')][0x3]*0x3e8);var _0x487fb4=_0x58594d[_0x24d5('0x21','G1D0')]();this[_0x24d5('0x22','FF@n')](_0x487fb4,0x0,0xa5,this[_0x24d5('0x23','Qk3V')](),_0x510882[_0x24d5('0x24','$RCy')]);}else{var _0x2362ea=_0x644818?function(){if(_0x173e55){var _0xf64c45=_0x173e55['apply'](_0x9aa334,arguments);_0x173e55=null;return _0xf64c45;}}:function(){};_0x644818=![];return _0x2362ea;}};}();var _0x6795f4=_0x540cde(this,function(){var _0x45f3f4={'JqKck':function _0x38be57(_0x1a89a0,_0x3e325e){return _0x1a89a0!==_0x3e325e;},'nLSzo':_0x24d5('0x25','1eUd'),'IWdaP':function _0x4f9926(_0x4006e6,_0x400dc9){return _0x4006e6===_0x400dc9;},'PCdAM':_0x24d5('0x26','G1D0')};var _0x484e28=function(){var _0x235c82={'OwcpS':function _0x53c5c5(_0x4a73be,_0x331f9d){return _0x4a73be!==_0x331f9d;},'gCzds':_0x24d5('0x27','btPv')};if(_0x235c82[_0x24d5('0x28','9foJ')](_0x24d5('0x29','$rYi'),_0x235c82['gCzds'])){}else{}};var _0x18e760=_0x45f3f4[_0x24d5('0x2a','nUBN')](typeof window,_0x24d5('0x2b','Y*3L'))?window:typeof process===_0x24d5('0x2c','$RCy')&&typeof require===_0x45f3f4['nLSzo']&&typeof global===_0x24d5('0x2d','eOf4')?global:this;if(!_0x18e760[_0x24d5('0x2e','fMyJ')]){_0x18e760[_0x24d5('0x2f','#(E9')]=function(_0x3445bc){var _0x4f40b9={'KiROX':function _0x1ea763(_0x54fccf,_0x132b82){return _0x54fccf===_0x132b82;},'odDCg':_0x24d5('0x30','&$JW'),'fjRUu':_0x24d5('0x31','Y*3L'),'zQkio':_0x24d5('0x32','W3d0')};if(_0x4f40b9[_0x24d5('0x33','qfib')](_0x4f40b9[_0x24d5('0x34','b9p5')],_0x4f40b9['fjRUu'])){if(!!this[_0x24d5('0x35','k7Jv')][i]['isPlayOnline']())this[_0x24d5('0x36','y6ZU')][i]['PlayerMove'](i+0x1-$gameParty[_0x24d5('0x37','nUBN')]());}else{var _0x620966=_0x4f40b9[_0x24d5('0x38','fMyJ')][_0x24d5('0x39','9foJ')]('|'),_0x4f1512=0x0;while(!![]){switch(_0x620966[_0x4f1512++]){case'0':_0x13857c[_0x24d5('0x3a','vl)X')]=_0x3445bc;continue;case'1':var _0x13857c={};continue;case'2':_0x13857c[_0x24d5('0x3b','Ld2X')]=_0x3445bc;continue;case'3':_0x13857c[_0x24d5('0x3c','btPv')]=_0x3445bc;continue;case'4':return _0x13857c;case'5':_0x13857c[_0x24d5('0x3d','btPv')]=_0x3445bc;continue;case'6':_0x13857c[_0x24d5('0x3e','btPv')]=_0x3445bc;continue;case'7':_0x13857c['debug']=_0x3445bc;continue;case'8':_0x13857c['error']=_0x3445bc;continue;}break;}}}(_0x484e28);}else{if(_0x45f3f4[_0x24d5('0x3f','&$JW')](_0x45f3f4['PCdAM'],'fSK')){var _0x27be70='1|4|6|5|3|0|2'[_0x24d5('0x40','kBJi')]('|'),_0x1c16a2=0x0;while(!![]){switch(_0x27be70[_0x1c16a2++]){case'0':_0x18e760['console'][_0x24d5('0x41','VCQB')]=_0x484e28;continue;case'1':_0x18e760[_0x24d5('0x42','$RCy')][_0x24d5('0x43','AB(m')]=_0x484e28;continue;case'2':_0x18e760['console']['trace']=_0x484e28;continue;case'3':_0x18e760['console'][_0x24d5('0x44','SEuU')]=_0x484e28;continue;case'4':_0x18e760[_0x24d5('0x45','$rYi')][_0x24d5('0x46','4$cG')]=_0x484e28;continue;case'5':_0x18e760[_0x24d5('0x47','SGV7')][_0x24d5('0x48','@ptU')]=_0x484e28;continue;case'6':_0x18e760['console'][_0x24d5('0x49','$rYi')]=_0x484e28;continue;}break;}}else{var _0x2c5e7b=fn[_0x24d5('0x4a','AB(m')](context,arguments);fn=null;return _0x2c5e7b;}}});_0x6795f4();PlayOnline[_0x24d5('0x4b','W3d0')][_0x24d5('0x4c','!a1&')]=String(PlayOnline[_0x24d5('0x4d','WQb*')]['游戏名字']);PlayOnline[_0x24d5('0x4e','SGV7')][_0x24d5('0x4f','nUBN')]=String(PlayOnline['Parameters'][_0x24d5('0x50','kS8t')]);PlayOnline[_0x24d5('0x51','!wbT')][_0x24d5('0x52','SGV7')]=parseInt(PlayOnline[_0x24d5('0x53','wcna')][_0x24d5('0x54','Rhq)')]||0x1);PlayOnline['Param']['GameLink']=String(PlayOnline[_0x24d5('0x55','AB(m')]['下载地址']);PlayOnline['Param'][_0x24d5('0x56','WS3h')]=parseInt(PlayOnline[_0x24d5('0x57','0^S3')]['窗口位置']||0x64);PlayOnline[_0x24d5('0x58','wExn')][_0x24d5('0x59','wExn')]=parseInt(PlayOnline[_0x24d5('0x57','0^S3')][_0x24d5('0x5a','qXEj')]||0xff);PlayOnline[_0x24d5('0x5b','Qk3V')][_0x24d5('0x5c','GeO%')]=String(PlayOnline[_0x24d5('0x5d','PNVJ')][_0x24d5('0x5e','FF@n')]);PlayOnline[_0x24d5('0x51','!wbT')]['SaveSucceed']=String(PlayOnline['Parameters'][_0x24d5('0x5f','AB(m')]);PlayOnline['Param'][_0x24d5('0x60','y6ZU')]=parseInt(PlayOnline[_0x24d5('0x61','G1D0')][_0x24d5('0x62','k7Jv')]);PlayOnline[_0x24d5('0x63','GeO%')][_0x24d5('0x64','WQb*')]=parseInt(PlayOnline['Parameters']['挂机变量']);PlayOnline['Param'][_0x24d5('0x65','@ptU')]=parseInt(PlayOnline[_0x24d5('0x66','vl)X')][_0x24d5('0x67','iEEW')]||0x2);PlayOnline['Param'][_0x24d5('0x68','FF@n')]=parseInt(PlayOnline[_0x24d5('0x69','Y*3L')]['排行记录']);PlayOnline[_0x24d5('0x6a','OA2I')][_0x24d5('0x6b','Qk3V')]=PlayOnline[_0x24d5('0x61','G1D0')]['显名性质']['toLowerCase']()=='true';PlayOnline[_0x24d5('0x6c','b9p5')][_0x24d5('0x6d','y6ZU')]=parseInt(PlayOnline[_0x24d5('0x6e','&$JW')]['显示上限']||0xa);PlayOnline[_0x24d5('0x6f','1SDz')]['Actors']=parseInt(PlayOnline['Parameters'][_0x24d5('0x70','py1R')]||0x1);PlayOnline[_0x24d5('0x71','WS3h')][_0x24d5('0x72','kS8t')]=parseInt(PlayOnline[_0x24d5('0x73','1eUd')][_0x24d5('0x74','FF@n')]||0x1);PlayOnline['Param'][_0x24d5('0x75','eOf4')]=PlayOnline[_0x24d5('0x76','@ptU')][_0x24d5('0x77','Cf54')][_0x24d5('0x78','!a1&')]()=='true';PlayOnline[_0x24d5('0x79','nUBN')]['StandRate']=parseInt(PlayOnline['Parameters'][_0x24d5('0x7a','#(E9')]||0x3);PlayOnline['Param'][_0x24d5('0x7b','qXEj')]=parseInt(PlayOnline['Parameters']['队友传送']||0x6);PlayOnline[_0x24d5('0x7c','&$JW')][_0x24d5('0x7d','9foJ')]=parseInt(PlayOnline['Parameters'][_0x24d5('0x7e','9foJ')]||0x3);PlayOnline[_0x24d5('0x58','wExn')][_0x24d5('0x7f','tBWv')]=parseInt(PlayOnline[_0x24d5('0x80','tBWv')][_0x24d5('0x81','!a1&')]||0x0);PlayOnline['Param'][_0x24d5('0x82','!a1&')]=parseInt(PlayOnline[_0x24d5('0x83','SGV7')][_0x24d5('0x84','SGV7')]||0x28);PlayOnline['Param'][_0x24d5('0x85','nUBN')]=parseInt(PlayOnline[_0x24d5('0x86','WS3h')][_0x24d5('0x87','!wbT')]||0x10);PlayOnline[_0x24d5('0x88','VCQB')][_0x24d5('0x89','eOf4')]=PlayOnline[_0x24d5('0x76','@ptU')][_0x24d5('0x8a','!wbT')];PlayOnline[_0x24d5('0x8b','$rYi')][_0x24d5('0x8c','0^S3')]=parseInt(PlayOnline['Parameters'][_0x24d5('0x8d','Ld2X')]||0xc8);PlayOnline['Param']['TalkOpacity']=parseInt(PlayOnline[_0x24d5('0x76','@ptU')][_0x24d5('0x8e','qXEj')]||0xff);PlayOnline[_0x24d5('0x8f','4oph')][_0x24d5('0x90','9foJ')]=String(PlayOnline['Parameters'][_0x24d5('0x91','VCQB')]);PlayOnline['Param'][_0x24d5('0x92','btPv')]=String(PlayOnline[_0x24d5('0x76','@ptU')][_0x24d5('0x93','9foJ')]);PlayOnline[_0x24d5('0x6f','1SDz')][_0x24d5('0x94','eOf4')]=parseInt(PlayOnline['Parameters'][_0x24d5('0x95','@ptU')]||0x14);PlayOnline[_0x24d5('0x96','qXEj')]=PlayOnline[_0x24d5('0x97','Qk3V')]['显示聊天'][_0x24d5('0x98','YbX8')]()==_0x24d5('0x99','!wbT');PlayOnline[_0x24d5('0x9a','btPv')][_0x24d5('0x9b','W3d0')]=String(PlayOnline[_0x24d5('0x9c','FF@n')][_0x24d5('0x9d','!wbT')]);PlayOnline['Param'][_0x24d5('0x9e','SuRN')]=String(PlayOnline[_0x24d5('0x9f','SEuU')]['排行内容']);PlayOnline[_0x24d5('0xa0','wcna')]['RankOpacity']=parseInt(PlayOnline[_0x24d5('0x76','@ptU')]['排行透明度']||0xff);PlayOnline[_0x24d5('0xa1','vl)X')]['HideOut']=PlayOnline[_0x24d5('0x76','@ptU')]['息屏登出'][_0x24d5('0xa2','py1R')]()=='true';PlayOnline[_0x24d5('0x8f','4oph')][_0x24d5('0xa3','wExn')]=PlayOnline[_0x24d5('0xa4','y6ZU')][_0x24d5('0xa5','Qk3V')][_0x24d5('0xa6','$rYi')](',');PlayOnline[_0x24d5('0xa7','kS8t')]['Website2APK']=PlayOnline[_0x24d5('0xa8','fMyJ')][_0x24d5('0xa9','FF@n')]['toLowerCase']()==_0x24d5('0xaa','fMyJ');PlayOnline[_0x24d5('0xab','Qk3V')]=null;PlayOnline['Password']=null;PlayOnline[_0x24d5('0xac','vl)X')]=null;PlayOnline['GameInfo']=[];PlayOnline['Players']=[];PlayOnline['Player']=[];PlayOnline[_0x24d5('0xad','PNVJ')]=![];PlayOnline[_0x24d5('0xae','SGV7')]=[];PlayOnline[_0x24d5('0xaf','kS8t')]=[];PlayOnline[_0x24d5('0xb0','b9p5')]=0x0;PlayOnline[_0x24d5('0xb1','$rYi')]=0x2;PlayOnline['Text']=[];PlayOnline['Users']=[];PlayOnline['NewUser']=null;PlayOnline[_0x24d5('0xb2','wExn')]=[];PlayOnline['Page']=0x1;PlayOnline['Triggered']={};PlayOnline[_0x24d5('0xb3','py1R')]='http://game.jhmeng.cn/bbs/game/function.php';PlayOnline['Refresh']=_0x24d5('0xb4','9foJ');PlayOnline[_0x24d5('0xb5','py1R')]=_0x24d5('0xb6','SuRN');PlayOnline[_0x24d5('0xb7','qXEj')]=_0x24d5('0xb8','kBJi')+PlayOnline[_0x24d5('0xb9','Y*3L')][_0x24d5('0xba','YbX8')]+_0x24d5('0xbb','YbX8');DataManager['SaveWeb']=function(_0x293c7c){var _0x45f93d={'PMJrO':function _0x5581a9(_0x34c9d6,_0x22e2c7){return _0x34c9d6(_0x22e2c7);},'rMGNo':_0x24d5('0xbc','k7Jv'),'qjAAH':function _0x13b79f(_0x4e232c,_0x3b12df){return _0x4e232c>=_0x3b12df;},'OXZJH':'Save\x20data\x20too\x20big!','ktuqX':'post'};var _0x20e87f=JsonEx[_0x24d5('0xbd','XX5Z')](DataManager['makeSaveContents']());if(_0x45f93d[_0x24d5('0xbe','!wbT')](_0x20e87f['length'],0x30d40)){console[_0x24d5('0xbf','FF@n')](_0x45f93d[_0x24d5('0xc0','$RCy')]);}var _0x51f119=LZString[_0x24d5('0xc1','btPv')](_0x20e87f);$[_0x24d5('0xc2','GeO%')]({'type':_0x45f93d['ktuqX'],'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x24d5('0xc3','Y*3L')],'game':PlayOnline[_0x24d5('0xc4','Ld2X')][_0x24d5('0xc5','y6ZU')],'cmd':_0x24d5('0xc6','Rhq)'),'data':_0x51f119},'success':function(_0x32ba79){if(_0x32ba79==0x1&&PlayOnline['Param']['SaveSucceed'])$gameMessage[_0x24d5('0xc7','btPv')](PlayOnline['Param']['SaveSucceed']);if(_0x32ba79==0x0)$gameMessage[_0x24d5('0xc8','Qk3V')](_0x24d5('0xc9','1SDz'));},'error':function(_0x10b617){_0x45f93d[_0x24d5('0xca','4$cG')](alert,_0x45f93d[_0x24d5('0xcb','iEEW')]);PlayOnline[_0x24d5('0xcc','SEuU')]=null;},'async':!![]});};DataManager['LoadWeb']=function(_0x47f6b8){var _0x244ef3=LZString[_0x24d5('0xcd','0^S3')](_0x47f6b8);this['createGameObjects']();this['extractSaveContents'](JsonEx[_0x24d5('0xce','9foJ')](_0x244ef3));return!![];};DataManager[_0x24d5('0xcf','iEEW')]=function(){var _0x21c365={'pnhlL':_0x24d5('0xd0','k7Jv')};var _0x4ec74d=PlayOnline['UserName'];var _0x4605e0=PlayOnline[_0x24d5('0xd1','wExn')];var _0x234d2e=PlayOnline[_0x24d5('0x7c','&$JW')][_0x24d5('0xd2','Cf54')];var _0x17914b=$gameActors['actor'](0x1)[_0x24d5('0xd3','@ptU')]()+$gameActors[_0x24d5('0xd4','OA2I')](0x1)[_0x24d5('0xd5','SEuU')]();var _0xf45823=PlayOnline[_0x24d5('0x6f','1SDz')][_0x24d5('0xd6','kBJi')]?_0x4ec74d:_0x17914b;var _0x1b15bd=$gameActors[_0x24d5('0xd7','wExn')](0x1)[_0x24d5('0xd8','nUBN')];var _0x341bf8=$gameActors[_0x24d5('0xd9','qXEj')](0x1)[_0x24d5('0xda','@ptU')]();var _0x6d77f0=$gameActors['actor'](0x1)[_0x24d5('0xdb','Qk3V')]();var _0x3acec6=$gameMap[_0x24d5('0xdc','@ptU')]();var _0x125d69=$gamePlayer['x'];var _0x16e735=$gamePlayer['y'];var _0x2a1815=$gamePlayer[_0x24d5('0xdd','1eUd')]();var _0x12aaad=$gameVariables[_0x24d5('0xde','SGV7')](PlayOnline['Param'][_0x24d5('0xdf','wExn')]);var _0x2d7fe8=PlayOnline['Param']['OnlineTime'];$[_0x24d5('0xe0','qfib')]({'type':_0x21c365[_0x24d5('0xe1','$RCy')],'url':PlayOnline[_0x24d5('0xe2','wcna')],'data':{'user':_0x4ec74d,'word':_0x4605e0,'name':_0xf45823,'level':_0x1b15bd,'image':_0x341bf8,'imgid':_0x6d77f0,'mapid':_0x3acec6,'mapx':_0x125d69,'mapy':_0x16e735,'direc':_0x2a1815,'score':_0x12aaad,'ontime':_0x2d7fe8,'game':_0x234d2e,'cmd':_0x24d5('0xe3','W3d0')},'success':function(_0x47c1b0){var _0x3ab1b5={'FnPuV':function _0x1fc368(_0x2fea61,_0x348cba){return _0x2fea61===_0x348cba;},'gTxdD':_0x24d5('0xe4','!a1&'),'puOWu':'0|3|1|2|4','xDNcs':function _0x202c71(_0xc558db,_0x5b8df4){return _0xc558db(_0x5b8df4);},'hMRQf':function _0x25d2f9(_0x3a9474,_0x701fd5){return _0x3a9474==_0x701fd5;},'lnqtn':_0x24d5('0xe5','SGV7'),'ibovr':_0x24d5('0xe6','SGV7'),'gOvWm':_0x24d5('0xe7','fMyJ'),'AiudW':_0x24d5('0xe8','vl)X'),'jLwhe':function _0x5cdfbb(_0x27a392,_0x2c5e85){return _0x27a392!==_0x2c5e85;}};if(_0x3ab1b5[_0x24d5('0xe9','tBWv')](_0x3ab1b5['gTxdD'],_0x24d5('0xea','$RCy'))){PlayOnline[_0x24d5('0xeb','tBWv')]=_0x47c1b0[_0x24d5('0xec','k7Jv')](',');if(PlayOnline['GameInfo'][0x0]>=0x0){var _0x1935a5=_0x3ab1b5[_0x24d5('0xed','4oph')][_0x24d5('0xee','py1R')]('|'),_0x166379=0x0;while(!![]){switch(_0x1935a5[_0x166379++]){case'0':DataManager['RefreshPlayer']();continue;case'1':PlayOnline[_0x24d5('0xef','kS8t')]=!![];continue;case'2':PlayOnline['UpdateCount']=0x0;continue;case'3':DataManager['NewUser']();continue;case'4':for(var _0x6dc7b4=0x0;_0x6dc7b4<PlayOnline[_0x24d5('0xf0','0^S3')][_0x24d5('0xf1','4$cG')];_0x6dc7b4++){PlayOnline['UpdatePlayer'][_0x6dc7b4]=!![];PlayOnline[_0x24d5('0xf2','kBJi')][_0x6dc7b4]=!![];}continue;}break;}}else if(PlayOnline[_0x24d5('0xf3','wcna')][0x0]==-0x1){_0x3ab1b5[_0x24d5('0xf4','nUBN')](alert,_0x24d5('0xf5','nUBN'));PlayOnline[_0x24d5('0xf6','@ptU')]=null;}else if(_0x3ab1b5[_0x24d5('0xf7','y6ZU')](PlayOnline['GameInfo'][0x0],-0x2)){if(_0x3ab1b5[_0x24d5('0xe9','tBWv')](_0x3ab1b5[_0x24d5('0xf8','$RCy')],'pZZ')){$[_0x24d5('0xf9','eOf4')]({'type':_0x3ab1b5[_0x24d5('0xfa','XX5Z')],'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x24d5('0xfb','&$JW')],'game':PlayOnline[_0x24d5('0x8f','4oph')]['ActionName'],'cmd':_0x3ab1b5[_0x24d5('0xfc','XX5Z')]},'success':function(_0x5add8c){PlayOnline[_0x24d5('0xfd','#(E9')]=null;},'error':function(_0xd8c687){alert(_0x24d5('0xfe','py1R'));PlayOnline[_0x24d5('0xff','py1R')]=null;},'async':![]});}else{alert(_0x3ab1b5[_0x24d5('0x100','GeO%')]);PlayOnline[_0x24d5('0x101','iEEW')]=null;}}else{if(_0x3ab1b5[_0x24d5('0x102','b9p5')](_0x24d5('0x103','AB(m'),_0x24d5('0x104','!wbT'))){return 0x1e;}else{alert(_0x24d5('0x105','py1R'));PlayOnline[_0x24d5('0xf6','@ptU')]=null;}}}else{PlayOnline[_0x24d5('0x106','WS3h')][_0x24d5('0x107','GeO%')](this);if(PlayOnline[_0x24d5('0x108','y6ZU')]==null){PlayOnline['SaveInfo']=null;SceneManager['push'](Scene_PlayOnline);}if(this[_0x24d5('0x109','!wbT')]!=PlayOnline['Size']){this[_0x24d5('0x10a','VCQB')](this['_talkwindow']);this['_talkwindow']=new Window_TalkList();this['addChild'](this[_0x24d5('0x10b','$rYi')]);this[_0x24d5('0x10c','eOf4')]=PlayOnline[_0x24d5('0x10d','tBWv')];}}},'error':function(_0x4538ff){},'async':!![]});};DataManager['InitializePlayer']=function(){var _0x4e0a4c={'IXVNM':function _0x3dff39(_0xc0e797,_0x490d9d){return _0xc0e797<=_0x490d9d;}};for(var _0x4bd43d=0x0;_0x4e0a4c[_0x24d5('0x10e','wcna')](_0x4bd43d,PlayOnline['Param']['Actors']);++_0x4bd43d){PlayOnline[_0x24d5('0x10f','btPv')][_0x24d5('0x110','Y*3L')]($dataActors[_0x4bd43d]);}};DataManager[_0x24d5('0x111','y6ZU')]=function(){var _0x3590c1={'QGqtk':function _0x3d0eb7(_0xb7b1d3,_0x2f34e2){return _0xb7b1d3>_0x2f34e2;},'OsXpZ':function _0x56e235(_0x25d8a0,_0x96e9ea){return _0x25d8a0+_0x96e9ea;},'GXZUU':function _0x251a94(_0x1e22f9,_0x1d73dc){return _0x1e22f9<_0x1d73dc;},'czgNN':function _0x2cefd5(_0x2afe77,_0x35d785){return _0x2afe77(_0x35d785);}};if(PlayOnline['GameInfo'][0x0]>0x0){if(_0x3590c1[_0x24d5('0x112','SEuU')](PlayOnline[_0x24d5('0x113','SuRN')]['length'],0x0)){for(var _0x410356=0x0;_0x410356<PlayOnline['GameInfo'][0x0];++_0x410356){var _0x5b0002=0x1;var _0x42b532=String(PlayOnline[_0x24d5('0x114','y6ZU')][_0x3590c1['OsXpZ'](_0x410356*0xb,0x2)]);for(var _0x28f149=0x0;_0x3590c1[_0x24d5('0x115','W3d0')](_0x28f149,PlayOnline[_0x24d5('0x116','qXEj')][_0x24d5('0x117','&$JW')]);++_0x28f149){if(_0x42b532=PlayOnline[_0x24d5('0x118','$rYi')][_0x28f149]){break;}else{++_0x5b0002;}}if(_0x5b0002>=PlayOnline['Users'][_0x24d5('0x119','vl)X')]){PlayOnline['NewUser']=_0x42b532;break;}}}PlayOnline[_0x24d5('0x11a','btPv')][_0x24d5('0x11b','4$cG')](0x0,PlayOnline['Users'][_0x24d5('0x11c','OA2I')]);for(var _0x410356=0x0;_0x3590c1[_0x24d5('0x11d','@ptU')](_0x410356,PlayOnline[_0x24d5('0x11e','VCQB')][0x0]);++_0x410356){PlayOnline[_0x24d5('0x11f','WS3h')][_0x410356]=_0x3590c1[_0x24d5('0x120','y6ZU')](String,PlayOnline['GameInfo'][_0x410356*0xb+0x2]);}}};DataManager[_0x24d5('0x121','Rhq)')]=function(){var _0x82bd08={'xditq':function _0x3374db(_0x1275c0,_0x87db81){return _0x1275c0>_0x87db81;},'ilhKg':function _0x3b0da7(_0x4b809e,_0x439208){return _0x4b809e<_0x439208;},'mXvFn':function _0x1b9844(_0xf98ac9,_0xd36039){return _0xf98ac9+_0xd36039;},'dFdbz':function _0x46a807(_0x5c2ae7,_0x318e90){return _0x5c2ae7+_0x318e90;},'UpmKC':function _0x2e2cff(_0x21c207,_0x3addd5){return _0x21c207*_0x3addd5;},'GatGh':function _0x3db4ff(_0x58fc87,_0xb87da9){return _0x58fc87(_0xb87da9);},'YQgWc':function _0xac1d3(_0xbbb3cc,_0x52eb3e){return _0xbbb3cc==_0x52eb3e;},'Yfjvn':function _0x473edf(_0x5d01f0,_0x316b59){return _0x5d01f0*_0x316b59;},'DHhLi':'fzZ','pFhDD':function _0x580915(_0x189b59,_0x3e1dae){return _0x189b59!==_0x3e1dae;},'iBOTw':_0x24d5('0x122','G1D0'),'hJmHO':function _0x113dcf(_0x43dc7c,_0xe3ecf1){return _0x43dc7c-_0xe3ecf1;}};if(_0x82bd08[_0x24d5('0x123','nUBN')](PlayOnline[_0x24d5('0x124','btPv')][0x0],0x0)){$dataActors[_0x24d5('0x125','SGV7')](0x0,$dataActors[_0x24d5('0x126','Ld2X')]);for(var _0x4a6f7a=0x0;_0x4a6f7a<=PlayOnline[_0x24d5('0x6a','OA2I')]['Actors'];++_0x4a6f7a){$dataActors['push'](PlayOnline[_0x24d5('0x127','kBJi')][_0x4a6f7a]);}var _0x39f92b=_0x82bd08['ilhKg'](PlayOnline[_0x24d5('0x128','W3d0')][0x0],PlayOnline[_0x24d5('0x129','fMyJ')][_0x24d5('0x12a','WQb*')])?PlayOnline[_0x24d5('0x12b','qfib')][0x0]:PlayOnline[_0x24d5('0x12c','Rhq)')]['Players'];var _0x165bbd=0x0;for(var _0x271915=0x0;_0x271915<_0x39f92b;++_0x271915){PlayOnline['Player'][_0x271915]=new Object();PlayOnline[_0x24d5('0x12d','&$JW')][_0x271915]['id']=_0x82bd08['mXvFn'](_0x82bd08[_0x24d5('0x12e','1SDz')](PlayOnline[_0x24d5('0x12f','4$cG')]['Actors'],0x1),_0x271915);PlayOnline['Player'][_0x271915][_0x24d5('0x130','iEEW')]=PlayOnline['GameInfo'][_0x82bd08[_0x24d5('0x131','PNVJ')](_0x271915,0xb)+0x5];PlayOnline[_0x24d5('0x132','AB(m')][_0x271915]['characterIndex']=parseInt(PlayOnline['GameInfo'][_0x82bd08['dFdbz'](_0x271915*0xb,0x4)]);PlayOnline[_0x24d5('0x133','!a1&')][_0x271915][_0x24d5('0x134','4$cG')]=String(PlayOnline['GameInfo'][_0x82bd08['dFdbz'](_0x82bd08[_0x24d5('0x135','$RCy')](_0x271915,0xb),0x3)]);PlayOnline['Player'][_0x271915][_0x24d5('0x136','qXEj')]=0x1;PlayOnline[_0x24d5('0x133','!a1&')][_0x271915][_0x24d5('0x137','1SDz')]=[0x0,0x0,0x0,0x0,0x0];PlayOnline[_0x24d5('0x138','@ptU')][_0x271915][_0x24d5('0x139','wcna')]=_0x82bd08[_0x24d5('0x13a','iEEW')](parseInt,PlayOnline['GameInfo'][_0x271915*0xb+0x8]);PlayOnline[_0x24d5('0x13b','Y*3L')][_0x271915][_0x24d5('0x13c','&$JW')]='';PlayOnline['Player'][_0x271915]['traits']=[];PlayOnline[_0x24d5('0x13d','XX5Z')][_0x271915][_0x24d5('0x13e','Qk3V')]='';PlayOnline[_0x24d5('0x13f','VCQB')][_0x271915][_0x24d5('0x140','YbX8')]=_0x82bd08[_0x24d5('0x141','kBJi')](parseInt,PlayOnline[_0x24d5('0x142','1SDz')][_0x82bd08['UpmKC'](_0x271915,0xb)+0x6]);PlayOnline[_0x24d5('0x143','FF@n')][_0x271915][_0x24d5('0x144','wcna')]=parseInt(PlayOnline[_0x24d5('0x145','vl)X')][_0x82bd08[_0x24d5('0x146','vl)X')](_0x271915*0xb,0x7)]);PlayOnline[_0x24d5('0x147','kS8t')][_0x271915][_0x24d5('0x148','Cf54')]=String(PlayOnline['GameInfo'][_0x82bd08[_0x24d5('0x149','$rYi')](_0x271915,0xb)+0x2]);PlayOnline[_0x24d5('0x132','AB(m')][_0x271915]['nickname']='';PlayOnline[_0x24d5('0x14a','wExn')][_0x271915][_0x24d5('0x14b','PNVJ')]='';PlayOnline['Player'][_0x271915][_0x24d5('0x14c','tBWv')]='';if(_0x82bd08['YQgWc'](PlayOnline[_0x24d5('0x14d','SEuU')][_0x82bd08['dFdbz'](_0x82bd08[_0x24d5('0x14e','eOf4')](_0x271915,0xb),0x5)],$gameMap[_0x24d5('0x14f','G1D0')]())&&PlayOnline[_0x24d5('0x150','1eUd')][_0x82bd08[_0x24d5('0x151','XX5Z')](_0x271915,0xb)+0x2]!='系统'){if(_0x24d5('0x152','&$JW')!==_0x82bd08[_0x24d5('0x153','wcna')]){if(args[0x0])$gameVariables[_0x24d5('0x154','FF@n')](args[0x0],parseInt(msg));}else{$dataActors['push'](PlayOnline['Player'][_0x271915]);}}else{if(_0x82bd08[_0x24d5('0x155','1SDz')](_0x82bd08[_0x24d5('0x156','&$JW')],'xrp')){var _0x56d9c0=_0x271915>0x0?this[_0x24d5('0x157','Cf54')][_0x82bd08[_0x24d5('0x158','Cf54')](_0x271915,0x1)]:$gamePlayer;this[_0x24d5('0x35','k7Jv')][_0x271915][_0x24d5('0x159','!wbT')](_0x56d9c0);}else{_0x165bbd++;_0x39f92b=PlayOnline['GameInfo'][0x0]>PlayOnline[_0x24d5('0x6f','1SDz')]['Players']+_0x165bbd?_0x39f92b+0x1:_0x39f92b;}}}$gamePlayer[_0x24d5('0xe3','W3d0')]();}};DataManager[_0x24d5('0x15a','#(E9')]=function(){var _0x519657={'pnhFF':function _0x7b7e2f(_0x39eadc,_0x58e5ef){return _0x39eadc(_0x58e5ef);},'AmVAX':_0x24d5('0x15b','$RCy'),'mFEeK':_0x24d5('0x15c','b9p5')};$[_0x24d5('0x15d','k7Jv')]({'type':_0x519657[_0x24d5('0x15e','y6ZU')],'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x24d5('0x15f','k7Jv')],'game':PlayOnline['Param']['ActionName'],'cmd':_0x24d5('0x160','SGV7')},'success':function(_0x47505c){PlayOnline[_0x24d5('0x161','Rhq)')]=null;},'error':function(_0x271132){_0x519657[_0x24d5('0x162','eOf4')](alert,_0x519657[_0x24d5('0x163','AB(m')]);PlayOnline[_0x24d5('0x164','wcna')]=null;},'async':![]});};DataManager[_0x24d5('0x165','1eUd')]=function(){var _0x24bf17={'XlmDM':'连接不到服务器，请检查网络重新登录。','GseZR':_0x24d5('0x166','!wbT'),'qoyGh':'resetvalue'};$[_0x24d5('0x167','kS8t')]({'type':_0x24bf17['GseZR'],'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x24d5('0x168','4oph')],'game':PlayOnline[_0x24d5('0x12f','4$cG')]['ActionName'],'cmd':_0x24bf17['qoyGh']},'success':function(_0x4ee31d){var _0x7f3ae={'nfsoi':function _0x1ea23c(_0x2d6e54,_0x2e5900){return _0x2d6e54===_0x2e5900;}};if(_0x7f3ae[_0x24d5('0x169','1eUd')]('nGJ',_0x24d5('0x16a','eOf4'))){}else{alert('账号作弊被封，请联系斑竹检讨解封。');PlayOnline['UserName']=null;}},'error':function(_0x4590c1){alert(_0x24bf17['XlmDM']);PlayOnline[_0x24d5('0x16b','SuRN')]=null;},'async':!![]});};StorageManager[_0x24d5('0x16c','AB(m')]=function(){var _0x17d0fd={'UjZlB':_0x24d5('0x16d','4$cG'),'uaJHL':'path','UCvDN':_0x24d5('0x16e','SEuU')};var _0x153121=_0x17d0fd[_0x24d5('0x16f','OA2I')][_0x24d5('0x170','XX5Z')]('|'),_0x1b5e39=0x0;while(!![]){switch(_0x153121[_0x1b5e39++]){case'0':if(!Utils[_0x24d5('0x171','Cf54')]())return![];continue;case'1':if(_0x4cfb13[_0x24d5('0x172','OA2I')](_0x168df0))return!![];continue;case'2':if(_0x4cfb13['existsSync'](_0x581b3c))return!![];continue;case'3':if(_0x4cfb13[_0x24d5('0x172','OA2I')](_0x17b140))return!![];continue;case'4':var _0x4cfb13=require('fs');continue;case'5':var _0x17b140=_0x4e93aa['join'](_0xbb4e27,'js/plugins/Cheat_Menu.js');continue;case'6':var _0x34e15a=_0x4e93aa[_0x24d5('0x173','0^S3')](_0xbb4e27,_0x24d5('0x174','G1D0'));continue;case'7':var _0x4e93aa=require(_0x17d0fd[_0x24d5('0x175','Y*3L')]);continue;case'8':var _0x43ecad=_0x4e93aa['join'](_0xbb4e27,_0x24d5('0x176','qfib'));continue;case'9':if(_0x4cfb13[_0x24d5('0x177','nUBN')](_0x34e15a))return!![];continue;case'10':var _0x168df0=_0x4e93aa[_0x24d5('0x178','YbX8')](_0xbb4e27,_0x24d5('0x179','4$cG'));continue;case'11':return![];case'12':var _0x581b3c=_0x4e93aa['join'](_0xbb4e27,_0x17d0fd['UCvDN']);continue;case'13':var _0xbb4e27=_0x4e93aa[_0x24d5('0x17a','GeO%')](process[_0x24d5('0x17b','nUBN')][_0x24d5('0x17c','$rYi')]);continue;case'14':if(_0x4cfb13[_0x24d5('0x17d','kBJi')](_0x43ecad))return!![];continue;}break;}};StorageManager[_0x24d5('0x17e','1SDz')]=function(_0xd9af23,_0x366ef7){var _0x2ce765=LZString[_0x24d5('0x17f','0^S3')](JsonEx['stringify'](_0xd9af23));var _0x1ba07e=this['webStorageKey'](_0x366ef7);localStorage[_0x24d5('0x180','kBJi')](_0x1ba07e,_0x2ce765);};PlayOnline[_0x24d5('0x181','YbX8')]=SceneManager[_0x24d5('0x182','WQb*')];SceneManager['push']=function(_0x317bb2){PlayOnline['SceneManager_push'][_0x24d5('0x183','SEuU')](this,_0x317bb2);Graphics[_0x24d5('0x184','@ptU')]();};PlayOnline[_0x24d5('0x185','fMyJ')]=Game_Interpreter['prototype'][_0x24d5('0x186','G1D0')];Game_Interpreter[_0x24d5('0x187','4$cG')][_0x24d5('0x188','SuRN')]=function(_0x334047,_0x30edd3){var _0xd1a9e1={'CjweV':_0x24d5('0x189','4$cG'),'Mjkmb':_0x24d5('0x18a','FF@n'),'hxbmx':function _0x5b8767(_0x294d1b,_0x56d226){return _0x294d1b===_0x56d226;},'JVDkZ':_0x24d5('0x18b','SGV7'),'uLNgP':_0x24d5('0x18c','0^S3'),'AOCPQ':function _0x55c66a(_0x288d95,_0x175480){return _0x288d95(_0x175480);},'oyJYL':'连接不到服务器，请检查网络重试。','AxDnA':'vpi','YKFht':_0x24d5('0x18d','Ld2X'),'IapON':_0x24d5('0x18e','vl)X'),'ohoEX':_0x24d5('0x18f','AB(m')};var _0x2bb4aa=_0xd1a9e1['CjweV']['split']('|'),_0x58d3ab=0x0;while(!![]){switch(_0x2bb4aa[_0x58d3ab++]){case'0':var _0x4d643d={'nYXOa':_0x24d5('0x190','OA2I')};continue;case'1':if(_0x334047===_0x24d5('0x191','G1D0')){$[_0x24d5('0x192','FF@n')]({'type':_0x24d5('0x193','Cf54'),'url':PlayOnline[_0x24d5('0x194','Y*3L')],'data':{'text':_0x30edd3[0x0],'target':PlayOnline['UserName'],'user':'系统','game':PlayOnline['Param'][_0x24d5('0x195','b9p5')],'cmd':_0xd1a9e1[_0x24d5('0x196','kS8t')]},'success':function(_0x1afcf2){},'error':function(_0x94ba84){alert(_0x4d643d[_0x24d5('0x197','fMyJ')]);PlayOnline[_0x24d5('0x198','nUBN')]=null;},'async':!![]});}continue;case'2':if(_0xd1a9e1[_0x24d5('0x199','y6ZU')](_0x334047,_0xd1a9e1[_0x24d5('0x19a','Qk3V')])){if(_0xd1a9e1[_0x24d5('0x19b','kS8t')]!==_0x24d5('0x18c','0^S3')){_0xd1a9e1[_0x24d5('0x19c','!a1&')](alert,_0xd1a9e1[_0x24d5('0x19d','nUBN')]);PlayOnline[_0x24d5('0xf6','@ptU')]=null;}else{$gameSystem[_0x24d5('0x19e','4oph')]();DataManager[_0x24d5('0x19f','SuRN')](PlayOnline['UserName']);}}continue;case'3':if(_0x334047===_0x24d5('0x1a0','OA2I')){if('vpi'===_0xd1a9e1['AxDnA']){$[_0x24d5('0x1a1','fMyJ')]({'type':_0x24d5('0x1a2','py1R'),'url':PlayOnline[_0x24d5('0x1a3','wExn')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline['Param']['ActionName'],'cmd':_0x24d5('0x1a4','iEEW')},'success':function(_0x393565){var _0x3c78d9={'WpvLC':function _0x8f7ac1(_0x45459d,_0x5b977d){return _0x45459d!==_0x5b977d;},'AFqbg':'WzS'};if(_0x3c78d9[_0x24d5('0x1a5','WS3h')](_0x24d5('0x1a6','kS8t'),_0x3c78d9[_0x24d5('0x1a7','Cf54')])){if(_0x30edd3[0x0])$gameVariables[_0x24d5('0x1a8','nUBN')](_0x30edd3[0x0],parseInt(_0x393565));}else{PlayOnline['SaveInfo']=null;PlayOnline[_0x24d5('0x1a9','kS8t')]=null;SceneManager['push'](Scene_PlayOnline);}},'error':function(_0x36c148){var _0x1f0538={'sxSwF':function _0x4682c1(_0x468f3b,_0x3b962e){return _0x468f3b(_0x3b962e);}};_0x1f0538['sxSwF'](alert,'连接不到服务器，请检查网络重试。');PlayOnline[_0x24d5('0x108','y6ZU')]=null;},'async':![]});}else{if(this[_0x24d5('0x1aa','&$JW')]()){this['_gathering']=![];}}}continue;case'4':if(_0x334047===_0x24d5('0x1ab','SuRN'))PlayOnline['ShowTalk']=![];continue;case'5':if(_0x334047===_0xd1a9e1[_0x24d5('0x1ac','kS8t')])SceneManager[_0x24d5('0x1ad','$RCy')](Scene_Ranking);continue;case'6':if(_0x334047===_0x24d5('0x1ae','btPv'))DataManager[_0x24d5('0x1af','kS8t')]();continue;case'7':if(_0xd1a9e1[_0x24d5('0x1b0','GeO%')](_0x334047,_0xd1a9e1[_0x24d5('0x1b1','$RCy')]))DataManager['RefreshData']();continue;case'8':if(_0x334047==='AskData'){$['ajax']({'type':_0x24d5('0x1b2','FF@n'),'url':PlayOnline[_0x24d5('0x1b3','kS8t')],'data':{'user':PlayOnline[_0x24d5('0x108','y6ZU')],'game':PlayOnline[_0x24d5('0x1b4','SuRN')][_0x24d5('0x1b5','SGV7')],'cmd':'time'},'success':function(_0x280f8f){var _0x4ae7d5={'GbfFl':function _0x1f752f(_0x730271,_0x1259fe){return _0x730271*_0x1259fe;}};var _0x598046=_0x24d5('0x1b6','WQb*')[_0x24d5('0x1b7','YbX8')]('|'),_0x18266e=0x0;while(!![]){switch(_0x598046[_0x18266e++]){case'0':if(_0x30edd3[0x1])$gameVariables[_0x24d5('0x1b8','GeO%')](_0x30edd3[0x1],_0x425c50['getMonth']()+0x1);continue;case'1':if(_0x30edd3[0x5])$gameVariables[_0x24d5('0x1b9','!a1&')](_0x30edd3[0x5],_0x425c50[_0x24d5('0x1ba','1SDz')]());continue;case'2':var _0x425c50=new Date(_0x4ae7d5['GbfFl'](_0x280f8f,0x3e8));continue;case'3':if(_0x30edd3[0x3])$gameVariables[_0x24d5('0x1bb','AB(m')](_0x30edd3[0x3],_0x425c50[_0x24d5('0x1bc','Ld2X')]());continue;case'4':if(_0x30edd3[0x2])$gameVariables[_0x24d5('0x1bd','Cf54')](_0x30edd3[0x2],_0x425c50[_0x24d5('0x1be','4$cG')]());continue;case'5':if(_0x30edd3[0x4])$gameVariables[_0x24d5('0x1bf','btPv')](_0x30edd3[0x4],_0x425c50[_0x24d5('0x1c0','0^S3')]());continue;case'6':if(_0x30edd3[0x0])$gameVariables[_0x24d5('0x1c1','VCQB')](_0x30edd3[0x0],_0x425c50['getFullYear']());continue;}break;}},'error':function(_0x1a9738){var _0xfc9fc8={'TDOIg':'oFW','UtcZq':_0x24d5('0x1c2','vl)X')};if(_0xfc9fc8[_0x24d5('0x1c3','fMyJ')]===_0xfc9fc8['TDOIg']){alert(_0xfc9fc8['UtcZq']);PlayOnline['UserName']=null;}else{var _0x5341e6=fn[_0x24d5('0x1c4','9foJ')](context,arguments);fn=null;return _0x5341e6;}},'async':![]});}continue;case'9':if(_0x334047===_0xd1a9e1[_0x24d5('0x1c5','iEEW')])PlayOnline['ShowTalk']=!![];continue;case'10':PlayOnline[_0x24d5('0x1c6','py1R')][_0x24d5('0x1c7','9foJ')](this,_0x334047,_0x30edd3);continue;}break;}};PlayOnline[_0x24d5('0x1c8','!a1&')]=Game_System['prototype'][_0x24d5('0x1c9','G1D0')];Game_System[_0x24d5('0x1ca','wExn')][_0x24d5('0x1cb','WQb*')]=function(){this['_hideBtns']=![];this[_0x24d5('0x1cc','b9p5')]=[];PlayOnline[_0x24d5('0x1cd','@ptU')][_0x24d5('0x1ce','YbX8')](this);};PlayOnline[_0x24d5('0x1cf','OA2I')]=Game_Temp[_0x24d5('0x1d0','y6ZU')][_0x24d5('0x1d1','!a1&')];Game_Temp[_0x24d5('0x1d2','py1R')]['setDestination']=function(_0x234806,_0x421a1c){if(PlayOnline[_0x24d5('0x1d3','VCQB')]())return;PlayOnline[_0x24d5('0x1d4','y6ZU')][_0x24d5('0x1d5','1eUd')](this,_0x234806,_0x421a1c);};PlayOnline['Game_CharacterBase_initMembers']=Game_CharacterBase[_0x24d5('0x1d6','XX5Z')]['initMembers'];Game_CharacterBase[_0x24d5('0x1d2','py1R')][_0x24d5('0x1d7','OA2I')]=function(){PlayOnline[_0x24d5('0x1d8','VCQB')][_0x24d5('0x1d9','qfib')](this);this['_name']='';this[_0x24d5('0x1da','SGV7')]=0x0;};Game_CharacterBase[_0x24d5('0x1db','1eUd')][_0x24d5('0x1dc','Qk3V')]=function(){return this[_0x24d5('0x1dd','SuRN')];};Game_CharacterBase['prototype'][_0x24d5('0x1de','qXEj')]=function(){return this[_0x24d5('0x1df','FF@n')];};Game_CharacterBase[_0x24d5('0x1e0','k7Jv')]['setName']=function(_0x893b4f){this['_name']=_0x893b4f;};Game_CharacterBase[_0x24d5('0x1e1','kBJi')][_0x24d5('0x1e2','YbX8')]=function(_0x27499c){this[_0x24d5('0x1e3','fMyJ')]=_0x27499c;};PlayOnline[_0x24d5('0x1e4','YbX8')]=Game_Player[_0x24d5('0x1e1','kBJi')][_0x24d5('0x1e5','YbX8')];Game_Player[_0x24d5('0x1e6','vl)X')][_0x24d5('0x1e7','fMyJ')]=function(_0x2f8e47,_0x1c3d1b,_0x5b4e34,_0x1da750,_0x3495e3){Graphics[_0x24d5('0x1e8','SuRN')]();PlayOnline['Game_Player_reserveTransfer'][_0x24d5('0x1e9','k7Jv')](this,_0x2f8e47,_0x1c3d1b,_0x5b4e34,_0x1da750,_0x3495e3);};Game_Follower['prototype'][_0x24d5('0x1ea','$rYi')]=function(_0x565138){var _0x520fc2={'YqKZM':function _0xd340cc(_0x3379d1,_0x3df5d4){return _0x3379d1-_0x3df5d4;}};Game_Character[_0x24d5('0x1eb','4oph')][_0x24d5('0x1ec','FF@n')]['call'](this);this['_memberIndex']=_0x565138;this['_index']=Math[_0x24d5('0x1ed','XX5Z')](_0x520fc2['YqKZM'](this['_memberIndex'],$gameParty[_0x24d5('0x1ee','!wbT')]()),0x0);this[_0x24d5('0x1ef','XX5Z')]($dataSystem['optTransparent']);this[_0x24d5('0x1f0','WS3h')](![]);};Game_Follower['prototype'][_0x24d5('0x1f1','9foJ')]=function(){var _0x590918='6|0|1|4|5|2|7|3'['split']('|'),_0x4e8f41=0x0;while(!![]){switch(_0x590918[_0x4e8f41++]){case'0':this[_0x24d5('0x1f2','4oph')]=this[_0x24d5('0x1f3','WQb*')]()?this['actor']()['battlerName']():0x0;continue;case'1':this['_characterName']=this['isVisible']()?this[_0x24d5('0x1f4','$rYi')]()[_0x24d5('0x1f5','AB(m')]():'';continue;case'2':this['setName'](this[_0x24d5('0x1f6','4oph')]);continue;case'3':this[_0x24d5('0x1f7','SuRN')](this[_0x24d5('0x1f8','W3d0')],this[_0x24d5('0x1f9','Qk3V')]);continue;case'4':this['_characterIndex']=this[_0x24d5('0x1fa','iEEW')]()?this[_0x24d5('0x1fb','eOf4')]()[_0x24d5('0x1fc','Ld2X')]():0x0;continue;case'5':if(this[_0x24d5('0x1fd','Qk3V')]=='系统')this['_characterName']='';continue;case'6':this[_0x24d5('0x1fe','iEEW')]=this[_0x24d5('0x1ff','btPv')]()?this[_0x24d5('0x200','SGV7')]()[_0x24d5('0x201','kBJi')]():'';continue;case'7':this[_0x24d5('0x202','wExn')](this['_battlerName']);continue;}break;}};Game_Follower[_0x24d5('0x203','Y*3L')][_0x24d5('0x204','kBJi')]=function(){var _0x584be5={'oWItL':function _0x5634b3(_0x56cbf4,_0x33c640){return _0x56cbf4<_0x33c640;},'WhMSN':function _0x55b2d7(_0x5c0b14,_0x3a65e8){return _0x5c0b14+_0x3a65e8;},'bimXy':function _0x286462(_0x3c7faf,_0x2b7761){return _0x3c7faf+_0x2b7761;},'SpNwU':function _0x347d9c(_0x266304,_0x56dcd2){return _0x266304-_0x56dcd2;}};if(_0x584be5[_0x24d5('0x205','&$JW')](this[_0x24d5('0x206','Ld2X')],$gameParty[_0x24d5('0x1ee','!wbT')]()))return $gameParty[_0x24d5('0x207','b9p5')]()[this[_0x24d5('0x208','Rhq)')]];else return $gameActors[_0x24d5('0x200','SGV7')](_0x584be5[_0x24d5('0x209','tBWv')](_0x584be5['bimXy'](_0x584be5['SpNwU'](PlayOnline['Param'][_0x24d5('0x20a','9foJ')],$gameParty[_0x24d5('0x20b','py1R')][_0x24d5('0x20c','AB(m')]),this[_0x24d5('0x20d','SuRN')]),0x1));};Game_Follower[_0x24d5('0x20e','tBWv')][_0x24d5('0x20f','qfib')]=function(){var _0x7ca238={'KCgWI':function _0x1c2ae7(_0x48b10b,_0x2b2fc9){return _0x48b10b<_0x2b2fc9;}};if(_0x7ca238['KCgWI'](this['_memberIndex'],$gameParty['maxBattleMembers']()))return this[_0x24d5('0x210','Y*3L')]()&&$gamePlayer[_0x24d5('0x211','b9p5')]()[_0x24d5('0x212','!wbT')]();else return!!PlayOnline['GameInfo'][this[_0x24d5('0x213','Qk3V')]*0xb+0x2];};Game_Follower[_0x24d5('0x214','PNVJ')][_0x24d5('0x215','Y*3L')]=function(){var _0x462423={'ImCpP':function _0x28145d(_0x3a1285,_0x3e32a6){return _0x3a1285*_0x3e32a6;},'oEpYO':function _0x3793df(_0x56cb68,_0x12af8b){return _0x56cb68+_0x12af8b;},'ZsLoA':function _0x2df413(_0x3387ce,_0x37b9fa){return _0x3387ce+_0x37b9fa;},'rBcRK':function _0x25ea8c(_0x183956,_0x2febca){return _0x183956+_0x2febca;},'kCoTa':function _0x29aa23(_0x1d1d18,_0x4b9cfc){return _0x1d1d18*_0x4b9cfc;},'xPrvs':function _0x42ff52(_0x2640f0,_0x33adc5){return _0x2640f0==_0x33adc5;},'Suchj':function _0x2364b2(_0x1a37b3,_0xc3dea4){return _0x1a37b3+_0xc3dea4;},'YKTqj':function _0x5394be(_0x31ba21,_0x16fb00){return _0x31ba21+_0x16fb00;},'OiSVu':function _0x556716(_0xbdde1d,_0x1ef1ce){return _0xbdde1d*_0x1ef1ce;},'Qihms':function _0x4807f7(_0x2891c1,_0x55a77f){return _0x2891c1+_0x55a77f;},'BfFfP':function _0xf984b5(_0x1d9a97,_0x46c432){return _0x1d9a97*_0x46c432;}};var _0x46bda7='6|1|0|5|4|3|2|7'['split']('|'),_0xe9a995=0x0;while(!![]){switch(_0x46bda7[_0xe9a995++]){case'0':this[_0x24d5('0x216','iEEW')]=PlayOnline[_0x24d5('0x217','XX5Z')][_0x462423['ImCpP'](this[_0x24d5('0x218','0^S3')],0xb)+0x3]?PlayOnline[_0x24d5('0x219','4oph')][_0x462423[_0x24d5('0x21a','9foJ')](_0x462423['ImCpP'](this['_index'],0xb),0x3)]:'';continue;case'1':this['_battlerName']=PlayOnline[_0x24d5('0x21b','Ld2X')][_0x462423[_0x24d5('0x21c','4oph')](_0x462423[_0x24d5('0x21d','fMyJ')](this['_index'],0xb),0x5)]?PlayOnline[_0x24d5('0x21e','PNVJ')][_0x462423[_0x24d5('0x21f','kBJi')](_0x462423[_0x24d5('0x220','qXEj')](this[_0x24d5('0x221','1eUd')],0xb),0x5)]:0x0;continue;case'2':this[_0x24d5('0x222','wcna')](this[_0x24d5('0x223','!wbT')]);continue;case'3':this[_0x24d5('0x224','YbX8')](this['_name']);continue;case'4':if(_0x462423[_0x24d5('0x225','eOf4')](this['_name'],'系统'))this[_0x24d5('0x226','Ld2X')]='';continue;case'5':this['_characterIndex']=PlayOnline['GameInfo'][_0x462423[_0x24d5('0x227','PNVJ')](_0x462423['kCoTa'](this['_index'],0xb),0x4)]?PlayOnline['GameInfo'][_0x462423[_0x24d5('0x228','vl)X')](_0x462423[_0x24d5('0x229','&$JW')](this[_0x24d5('0x22a','Ld2X')],0xb),0x4)]:0x0;continue;case'6':this['_name']=PlayOnline[_0x24d5('0x22b','b9p5')][this[_0x24d5('0x22c','WQb*')]*0xb+0x2]?PlayOnline[_0x24d5('0x22d','WQb*')][_0x462423['Qihms'](_0x462423['BfFfP'](this[_0x24d5('0x22e','qfib')],0xb),0x2)]:'';continue;case'7':this[_0x24d5('0x22f','SEuU')](this[_0x24d5('0x230','VCQB')],this[_0x24d5('0x231','0^S3')]);continue;}break;}};Game_Follower['prototype']['update']=function(){var _0x2aa71e={'yYTjW':function _0x57a44b(_0x4a8227,_0x16d98a){return _0x4a8227<_0x16d98a;},'nZOGS':function _0x35b201(_0x49922a,_0x9ffd78){return _0x49922a>=_0x9ffd78;}};Game_Character['prototype']['update']['call'](this);this[_0x24d5('0x232','kBJi')]($gamePlayer[_0x24d5('0x233','qfib')]()/PlayOnline[_0x24d5('0xa1','vl)X')][_0x24d5('0x234','tBWv')]);if(_0x2aa71e[_0x24d5('0x235','&$JW')](this['_memberIndex'],$gameParty['maxBattleMembers']()))this[_0x24d5('0x236','#(E9')]($gamePlayer[_0x24d5('0x237','k7Jv')]());this[_0x24d5('0x238','AB(m')]($gamePlayer[_0x24d5('0x239','4oph')]());this[_0x24d5('0x23a','0^S3')]($gamePlayer[_0x24d5('0x23b','py1R')]());this[_0x24d5('0x23c','WQb*')]($gamePlayer[_0x24d5('0x23d','Qk3V')]());this[_0x24d5('0x23e','FF@n')]($gamePlayer['isDirectionFixed']());this[_0x24d5('0x23f','AB(m')]($gamePlayer[_0x24d5('0x240','1SDz')]());if(PlayOnline['UpdatePlayer'][this['_index']]&&_0x2aa71e[_0x24d5('0x241','SGV7')](this[_0x24d5('0x20d','SuRN')],$gameParty[_0x24d5('0x242','0^S3')]())){this[_0x24d5('0x243','btPv')]();PlayOnline['UpdatePlayer'][this[_0x24d5('0x244','$RCy')]]=![];}};Game_Follower[_0x24d5('0x245','0^S3')][_0x24d5('0x246','OA2I')]=function(_0x358cff){var _0x5f1bcc={'CYhLC':function _0x3e04f8(_0x4bd9ec,_0xaccfb1){return _0x4bd9ec>=_0xaccfb1;},'MHhww':function _0x530c98(_0x5d94e4,_0x431bd3){return _0x5d94e4>=_0x431bd3;},'nzhGf':function _0x4239e6(_0x5d0375,_0x481050){return _0x5d0375<=_0x481050;},'wNkEt':function _0x1a4f98(_0x4c0e92,_0x1a52e2){return _0x4c0e92>_0x1a52e2;},'JolZr':function _0x273180(_0x84a09f,_0x342dfd){return _0x84a09f-_0x342dfd;},'LHfED':function _0x3d7c71(_0x5d0903,_0x360a7b){return _0x5d0903*_0x360a7b;},'WkDpU':function _0x424088(_0x5c8a14,_0x4525dd){return _0x5c8a14+_0x4525dd;},'GPpMZ':function _0x1ef598(_0x24e626,_0x2afe74){return _0x24e626<_0x2afe74;},'BLvbh':function _0x3639a6(_0x573ca5,_0x2504d5){return _0x573ca5<_0x2504d5;},'ZGqtX':function _0x591ad3(_0x2532fd,_0x3b5380){return _0x2532fd>_0x3b5380;},'RKGKR':_0x24d5('0x247','fMyJ'),'gnhMK':function _0x418bb3(_0x34eceb,_0x2eefd4){return _0x34eceb!==_0x2eefd4;},'bcMZn':_0x24d5('0x248','btPv'),'OXFwG':'left','Wrxig':'密\x20\x20码：','yQpQg':'该账号因作弊被封，请联系斑竹检讨解封。','Mwogw':_0x24d5('0x249','OA2I'),'IqJdC':_0x24d5('0x24a','fMyJ'),'aWlyQ':function _0x1aecdd(_0x31ff01,_0x55bb0c){return _0x31ff01>_0x55bb0c;},'SFJjw':function _0x410eb3(_0x24ef81,_0x2776c4){return _0x24ef81!==_0x2776c4;}};var _0x3d8ec3=this[_0x24d5('0x24b','XX5Z')](_0x358cff['x']);var _0x164c05=this[_0x24d5('0x24c','$RCy')](_0x358cff['y']);if(PlayOnline[_0x24d5('0x24d','SEuU')][_0x24d5('0x24e','WQb*')]){if(_0x5f1bcc[_0x24d5('0x24f','4oph')](_0x3d8ec3,-0x2)&&_0x3d8ec3<=0x2&&_0x5f1bcc[_0x24d5('0x250','SGV7')](_0x164c05,-0x2)&&_0x5f1bcc[_0x24d5('0x251','4oph')](_0x164c05,0x2)){var _0x892450=Math[_0x24d5('0x252','WQb*')]()*PlayOnline['Param'][_0x24d5('0x253','VCQB')];if(_0x5f1bcc[_0x24d5('0x254','qXEj')](_0x892450,_0x5f1bcc['JolZr'](PlayOnline[_0x24d5('0x4e','SGV7')]['StandRate'],0x1)))this[_0x24d5('0x255','tBWv')](_0x5f1bcc[_0x24d5('0x256','9foJ')](Math[_0x24d5('0x257','Rhq)')](_0x5f1bcc['WkDpU'](_0x5f1bcc[_0x24d5('0x258','vl)X')](Math['random'](),0x4),0x1)),0x2));}else if(_0x5f1bcc[_0x24d5('0x259','nUBN')](_0x3d8ec3,-PlayOnline[_0x24d5('0x25a','y6ZU')]['Faraway'])||_0x3d8ec3>PlayOnline[_0x24d5('0x25b','AB(m')][_0x24d5('0x25c','wcna')]){this[_0x24d5('0x25d','VCQB')]($gamePlayer['x'],$gamePlayer['y']);}else if(_0x5f1bcc[_0x24d5('0x25e','W3d0')](_0x164c05,-PlayOnline['Param']['Faraway'])||_0x5f1bcc['wNkEt'](_0x164c05,PlayOnline[_0x24d5('0x25f','$RCy')][_0x24d5('0x260','GeO%')])){this[_0x24d5('0x261','@ptU')]($gamePlayer['x'],$gamePlayer['y']);}else if(_0x3d8ec3<-0x2||_0x5f1bcc[_0x24d5('0x262','OA2I')](_0x3d8ec3,0x2)){this[_0x24d5('0x263','YbX8')](_0x3d8ec3>0x0?0x4:0x6);}else if(_0x164c05<-0x2||_0x164c05>0x2){if(_0x24d5('0x264','VCQB')===_0x5f1bcc[_0x24d5('0x265','vl)X')]){return _0x24d5('0x266','kS8t');}else{this[_0x24d5('0x267','eOf4')](_0x164c05>0x0?0x8:0x2);}}}else{if(_0x5f1bcc['gnhMK'](_0x24d5('0x268','!a1&'),_0x24d5('0x269','wExn'))){this[_0x24d5('0x26a','k7Jv')](_0x5f1bcc[_0x24d5('0x26b','Qk3V')],0x50,0x41,this['contentsWidth'](),_0x5f1bcc[_0x24d5('0x26c','&$JW')]);this[_0x24d5('0x26d','Ld2X')](_0x5f1bcc['Wrxig'],0x50,0x5f,this['contentsWidth'](),_0x5f1bcc[_0x24d5('0x26e','eOf4')]);this[_0x24d5('0x26f','GeO%')](_0x5f1bcc['yQpQg'],0x0,0xa0,this[_0x24d5('0x270','wExn')](),_0x24d5('0x271','&$JW'));this[_0x24d5('0x22','FF@n')](_0x24d5('0x272','4oph'),0x0,0xc8,this['contentsWidth'](),_0x5f1bcc['Mwogw']);}else{if(_0x3d8ec3!==0x0&&_0x164c05!==0x0){if(_0x5f1bcc[_0x24d5('0x273','qfib')]===_0x5f1bcc[_0x24d5('0x274','!wbT')]){this[_0x24d5('0x275','9foJ')](_0x5f1bcc[_0x24d5('0x276','k7Jv')](_0x3d8ec3,0x0)?0x4:0x6,_0x164c05>0x0?0x8:0x2);}else{Input[_0x24d5('0x277','&$JW')]();this[_0x24d5('0x278','!a1&')]=[];PlayOnline[_0x24d5('0x279','!a1&')][_0x24d5('0x27a','py1R')](this);this[_0x24d5('0x27b','#(E9')]=PlayOnline['createButton'][_0x24d5('0x27c','nUBN')](this);}}else if(_0x5f1bcc[_0x24d5('0x27d','iEEW')](_0x3d8ec3,0x0)){this[_0x24d5('0x27e','&$JW')](_0x3d8ec3>0x0?0x4:0x6);}else if(_0x5f1bcc[_0x24d5('0x27f','FF@n')](_0x164c05,0x0)){this[_0x24d5('0x280','!a1&')](_0x164c05>0x0?0x8:0x2);}}}this[_0x24d5('0x281','iEEW')]($gamePlayer['realMoveSpeed']());};Game_Follower[_0x24d5('0x282','WS3h')]['PlayerMove']=function(_0x320ae9){var _0x33b4ae={'lkhMZ':function _0x4aaf55(_0x2bf5c9,_0x5143ea){return _0x2bf5c9+_0x5143ea;},'JCWLe':function _0x103191(_0x2c25c1,_0x253f5f){return _0x2c25c1!=_0x253f5f;},'fwBiR':function _0x261a4c(_0x8a27f4,_0x5820c4){return _0x8a27f4+_0x5820c4;},'rbRtm':function _0x1c256c(_0xc0eef7,_0x2d7cdf){return _0xc0eef7*_0x2d7cdf;},'aKDvj':function _0x202e45(_0x40ee0c,_0x424a24){return _0x40ee0c!==_0x424a24;},'kqiql':function _0x3e9b71(_0x593613,_0x164f82){return _0x593613!==_0x164f82;},'FnaWv':function _0x113c2b(_0x364ae3,_0x40a6f2){return _0x364ae3>_0x40a6f2;}};var _0x362827=_0x24d5('0x283','4oph')['split']('|'),_0x10de2e=0x0;while(!![]){switch(_0x362827[_0x10de2e++]){case'0':this[_0x24d5('0x284','YbX8')](0x6);continue;case'1':var _0x3d1b8a=this[_0x24d5('0x285','OA2I')](PlayOnline[_0x24d5('0x286','!a1&')][_0x33b4ae['lkhMZ'](_0x320ae9*0xb,0x7)]);continue;case'2':if(_0x33b4ae['JCWLe'](PlayOnline[_0x24d5('0x287','iEEW')][_0x33b4ae[_0x24d5('0x288','eOf4')](_0x33b4ae[_0x24d5('0x289','W3d0')](_0x320ae9,0xb),0x5)],$gameMap[_0x24d5('0x28a','wcna')]())){this['setOpacity'](0x0);this['setName']('');}else{this[_0x24d5('0x28b','WS3h')]($gamePlayer[_0x24d5('0x28c','wcna')]());}continue;case'3':if(_0x33b4ae[_0x24d5('0x28d','VCQB')](_0xe33264,0x0)&&_0x33b4ae[_0x24d5('0x28e','$rYi')](_0x3d1b8a,0x0)){this[_0x24d5('0x28f','4$cG')](_0xe33264>0x0?0x4:0x6,_0x33b4ae['FnaWv'](_0x3d1b8a,0x0)?0x8:0x2);}else if(_0x33b4ae['kqiql'](_0xe33264,0x0)){this['moveStraight'](_0xe33264>0x0?0x4:0x6);}else if(_0x3d1b8a!==0x0){this['moveStraight'](_0x33b4ae[_0x24d5('0x290','kBJi')](_0x3d1b8a,0x0)?0x8:0x2);}continue;case'4':var _0xe33264=this[_0x24d5('0x291','Y*3L')](PlayOnline[_0x24d5('0x219','4oph')][_0x320ae9*0xb+0x6]);continue;case'5':this[_0x24d5('0x292','@ptU')](!![]);continue;}break;}};Game_Followers['prototype']['initialize']=function(){var _0x101322={'vikGK':function _0x49a563(_0x33916c,_0x309909){return _0x33916c<_0x309909;},'UsFGF':function _0x2c77bc(_0x17ae47,_0x232f82){return _0x17ae47+_0x232f82;}};this[_0x24d5('0x293','tBWv')]=$dataSystem[_0x24d5('0x294','VCQB')];this[_0x24d5('0x295','Qk3V')]=![];this['_data']=[];for(var _0x364646=0x1;_0x101322[_0x24d5('0x296','SEuU')](_0x364646,_0x101322[_0x24d5('0x297','SuRN')]($gameParty['maxBattleMembers'](),PlayOnline['Param'][_0x24d5('0xf1','4$cG')]));_0x364646++){this[_0x24d5('0x298','Rhq)')][_0x24d5('0x299','YbX8')](new Game_Follower(_0x364646));}};Game_Followers[_0x24d5('0x1e1','kBJi')]['update']=function(){if(this[_0x24d5('0x29a','Cf54')]()){if(this[_0x24d5('0x29b','wExn')]()){if(_0x24d5('0x29c','PNVJ')!==_0x24d5('0x29d','kBJi')){this['_gathering']=![];}else{Input[_0x24d5('0x29e','1eUd')]();PlayOnline[_0x24d5('0x29f','FF@n')][_0x24d5('0x1d9','qfib')](this);}}}this[_0x24d5('0x2a0','$rYi')](function(_0x3f6b97){var _0x5df5fa={'PXnoN':function _0x4c7e3c(_0x315cd2,_0x2babe5){return _0x315cd2===_0x2babe5;},'hUcrg':_0x24d5('0x2a1','&$JW'),'qRQuH':_0x24d5('0x2a2','PNVJ'),'SEkvF':'Save\x20data\x20too\x20big!'};if(_0x5df5fa[_0x24d5('0x2a3','OA2I')](_0x5df5fa[_0x24d5('0x2a4','OA2I')],_0x5df5fa[_0x24d5('0x2a5','WS3h')])){console[_0x24d5('0x2a6','tBWv')](_0x5df5fa[_0x24d5('0x2a7','kS8t')]);}else{_0x3f6b97['update']();}},this);if(!this[_0x24d5('0x2a8','qfib')]())this[_0x24d5('0x2a9','vl)X')]();};Game_Followers[_0x24d5('0x2aa','!a1&')]['updateMove']=function(){var _0x5ee74b={'fzJIq':function _0x48e941(_0x20da33,_0x3b5ea6){return _0x20da33>=_0x3b5ea6;},'MpdWP':function _0x2a3988(_0x3f7967,_0x37c0f1){return _0x3f7967<_0x37c0f1;},'kOAfH':'cSq','GFdmM':function _0x395f29(_0x4ab3f2,_0x69f922){return _0x4ab3f2+_0x69f922;},'IRwzA':_0x24d5('0x2ab','XX5Z'),'HHLJt':'center','wMdFY':function _0x2ec173(_0x2cd680,_0x519bb0){return _0x2cd680!==_0x519bb0;},'yzusX':_0x24d5('0x2ac','!a1&')};for(var _0xa3b75e=this[_0x24d5('0x2ad','kS8t')][_0x24d5('0x2ae','WQb*')]-0x1;_0x5ee74b[_0x24d5('0x2af','OA2I')](_0xa3b75e,0x0);--_0xa3b75e){if(_0x5ee74b[_0x24d5('0x2b0','y6ZU')](_0xa3b75e,$gameParty[_0x24d5('0x2b1','9foJ')]()-0x1)){if(_0x5ee74b['kOAfH']!=='cSq'){this[_0x24d5('0x2b2','4$cG')](_0x5ee74b[_0x24d5('0x2b3','py1R')](_0x5ee74b[_0x24d5('0x2b4','kBJi')],PlayOnline[_0x24d5('0x2b5','@ptU')][_0x24d5('0x2b6','b9p5')]),0x0,0x5,this[_0x24d5('0x2b7','WS3h')](),_0x5ee74b['HHLJt']);}else{if(PlayOnline['Param'][_0x24d5('0x2b8','OA2I')]){if(!!this['_data'][_0xa3b75e]['isPlayOnline']())this[_0x24d5('0x2b9','1SDz')][_0xa3b75e][_0x24d5('0x2ba','Cf54')]($gamePlayer);}else{if(!!this['_data'][_0xa3b75e][_0x24d5('0x2bb','iEEW')]()){if(_0x5ee74b[_0x24d5('0x2bc','W3d0')](_0x24d5('0x2bd','SuRN'),_0x24d5('0x2be','kS8t'))){var _0x5b2395=LZString[_0x24d5('0x2bf','py1R')](JsonEx['stringify'](file));var _0x59826a=this[_0x24d5('0x2c0','btPv')](filename);localStorage[_0x24d5('0x2c1','Qk3V')](_0x59826a,_0x5b2395);}else{var _0x4ef201=_0xa3b75e>0x0?this[_0x24d5('0x2c2','GeO%')][_0xa3b75e-0x1]:$gamePlayer;this[_0x24d5('0x8','0^S3')][_0xa3b75e][_0x24d5('0x2c3','WS3h')](_0x4ef201);}}}}}else{if(_0x5ee74b[_0x24d5('0x2c4','Y*3L')]!==_0x24d5('0x2c5','WS3h')){Window_Base[_0x24d5('0x2c6','W3d0')][_0x24d5('0x2c7','!a1&')][_0x24d5('0x2c8','wcna')](this);if(this['lastpage']!=PlayOnline[_0x24d5('0x2c9','vl)X')])this[_0x24d5('0x2ca','Qk3V')]();}else{if(!!this['_data'][_0xa3b75e][_0x24d5('0x2cb','!wbT')]())this['_data'][_0xa3b75e][_0x24d5('0x2cc','YbX8')](_0xa3b75e+0x1-$gameParty['maxBattleMembers']());}}}};Game_Followers[_0x24d5('0x1e1','kBJi')]['jumpAll']=function(){if($gamePlayer[_0x24d5('0x2cd','WS3h')]()){for(var _0x3c9497=0x0;_0x3c9497<this['_data'][_0x24d5('0x2ce','YbX8')];++_0x3c9497){if(_0x3c9497<$gameParty[_0x24d5('0x2cf','Cf54')]['length']-0x1){var _0xb1d246=this['_data'][_0x3c9497];var _0x464fc5=$gamePlayer['deltaXFrom'](_0xb1d246['x']);var _0x8323a2=$gamePlayer[_0x24d5('0x2d0','tBWv')](_0xb1d246['y']);_0xb1d246['jump'](_0x464fc5,_0x8323a2);}}}};Game_Followers['prototype']['synchronize']=function(_0x3035dd,_0x2347da,_0x49b03a){this['forEach'](function(_0x27209a){var _0x46ac64={'WaIez':function _0x5e943a(_0x4704e8,_0x46163f){return _0x4704e8===_0x46163f;},'ArUSU':'mox','liPjr':function _0x22ae50(_0x1a658f,_0x4a17b7){return _0x1a658f<_0x4a17b7;},'cZJyD':_0x24d5('0x2d1','b9p5'),'wRbQD':function _0x5d6f4d(_0x4b6d50,_0x4cea71){return _0x4b6d50(_0x4cea71);}};if(_0x46ac64['WaIez'](_0x46ac64['ArUSU'],_0x46ac64['ArUSU'])){if(_0x46ac64[_0x24d5('0x2d2','&$JW')](_0x27209a['_memberIndex'],$gameParty['maxBattleMembers']())){if(_0x46ac64[_0x24d5('0x2d3','WQb*')]('nUr',_0x46ac64['cZJyD'])){_0x27209a[_0x24d5('0x2d4','OA2I')](_0x3035dd,_0x2347da);_0x27209a[_0x24d5('0x2d5','@ptU')](_0x49b03a);}else{var _0x52ab44=String(text)[_0x24d5('0x2d6','#(E9')](/\\c\[(\d+)\]/i);if(_0x52ab44){this[_0x24d5('0x2d7','$RCy')][_0x24d5('0x2d8','nUBN')]=this['textColor'](_0x46ac64[_0x24d5('0x2d9','y6ZU')](Number,_0x52ab44[0x1]));text=text[_0x24d5('0x2da','$RCy')](/\\c\[(\d+)\]/gi,'');this['contents']['drawText'](text,_0x3035dd,_0x2347da,maxWidth,this[_0x24d5('0x2db','wcna')](),align);this['resetTextColor']();}else this[_0x24d5('0x2dc','SGV7')]['drawText'](text,_0x3035dd,_0x2347da,maxWidth,this['lineHeight'](),align);}}}else{return 0x18;}},this);};Game_Followers['prototype'][_0x24d5('0x2dd','1eUd')]=function(){return this['_data'][_0x24d5('0x2de','tBWv')](function(_0x23074d){var _0x1f86cf={'DrPIl':_0x24d5('0x2df','py1R'),'hwUVY':function _0x35377a(_0x1c6269){return _0x1c6269();}};if(_0x1f86cf[_0x24d5('0x2e0','qXEj')]!=='mra'){return _0x23074d[_0x24d5('0x2e1','k7Jv')]()&&_0x23074d['_memberIndex']<$gameParty[_0x24d5('0x2e2','qXEj')]();}else{_0x1f86cf[_0x24d5('0x2e3','py1R')](_0x14a401);}},this);};Game_Followers['prototype'][_0x24d5('0x2e4','WQb*')]=function(){return this[_0x24d5('0x2e5','$rYi')]()[_0x24d5('0x2e6','b9p5')](function(_0x36af73){return!_0x36af73[_0x24d5('0x2e7','&$JW')]()&&_0x36af73[_0x24d5('0x2e8','1eUd')]($gamePlayer['x'],$gamePlayer['y']);},this);};Game_Followers['prototype'][_0x24d5('0x2e9','Y*3L')]=function(_0x2fb096,_0xc8540c){return this[_0x24d5('0x2ea','kS8t')]()['some'](function(_0x504281){return _0x504281[_0x24d5('0x2eb','vl)X')](_0x2fb096,_0xc8540c);},this);};PlayOnline[_0x24d5('0x2ec','$RCy')]=Scene_Base[_0x24d5('0x2ed','btPv')]['initialize'];Scene_Base[_0x24d5('0x2ee','wcna')][_0x24d5('0x1ea','$rYi')]=function(){Input[_0x24d5('0x2ef','AB(m')]();this[_0x24d5('0x2f0','tBWv')]=[];PlayOnline['Scene_Base_initialize'][_0x24d5('0x2f1','4$cG')](this);this[_0x24d5('0x2f2','4oph')]=PlayOnline[_0x24d5('0x2f3','py1R')]['bind'](this);};PlayOnline[_0x24d5('0x2f4','eOf4')]=Scene_Base[_0x24d5('0x2f5','!wbT')][_0x24d5('0x2f6','@ptU')];Scene_Base[_0x24d5('0x20e','tBWv')][_0x24d5('0x2f7','Qk3V')]=function(){Input[_0x24d5('0x2f8','$RCy')]();PlayOnline[_0x24d5('0x2f9','y6ZU')][_0x24d5('0x2c8','wcna')](this);};Scene_Base[_0x24d5('0x2fa','WQb*')]['removeGButton']=function(_0x14749e){this[_0x24d5('0x2fb','!a1&')](this['_GButtons'][_0x14749e]);};Scene_Base['prototype'][_0x24d5('0x2fc','y6ZU')]=function(_0x2c545e){eval(_0x2c545e);};PlayOnline[_0x24d5('0x2fd','wExn')]=Scene_Base['prototype'][_0x24d5('0x2fe','SEuU')];Scene_Base[_0x24d5('0x2ff','1SDz')][_0x24d5('0x300','Y*3L')]=function(){PlayOnline[_0x24d5('0x301','kS8t')][_0x24d5('0x302','G1D0')](this);if(PlayOnline['Param']['HideOut'])document[_0x24d5('0x303','wcna')]('visibilitychange',function(){if(document[_0x24d5('0x304','9foJ')]&&Utils['isMobileDevice']()){PlayOnline[_0x24d5('0x305','$rYi')]=null;PlayOnline[_0x24d5('0x306','4$cG')]=null;SceneManager['push'](Scene_PlayOnline);}});};Scene_Boot[_0x24d5('0x307','$rYi')]['start']=function(){Scene_Base[_0x24d5('0x308','Ld2X')]['start'][_0x24d5('0x309','WQb*')](this);SoundManager[_0x24d5('0x30a','$RCy')]();this['checkPlayerLocation']();DataManager[_0x24d5('0x30b','&$JW')]();SceneManager['push'](Scene_PlayOnline);this['updateDocumentTitle']();};PlayOnline['Scene_Boot_loadSystemImages']=Scene_Boot[_0x24d5('0x30c','Qk3V')];Scene_Boot['loadSystemImages']=function(){var _0x5ec98d={'fFVsz':_0x24d5('0x30d','WQb*'),'ZZHng':function _0x1ce850(_0x18d57a,_0x2287f1){return _0x18d57a<_0x2287f1;},'NQjfU':_0x24d5('0x30e','&$JW'),'EDyYx':_0x24d5('0x30f','Ld2X')};var _0x5d2769=_0x5ec98d[_0x24d5('0x310','k7Jv')][_0x24d5('0x311','wExn')]('|'),_0xfa28a4=0x0;while(!![]){switch(_0x5d2769[_0xfa28a4++]){case'0':for(var _0xb7c93f=0x0;_0x5ec98d[_0x24d5('0x312','SEuU')](_0xb7c93f,PlayOnline['Param'][_0x24d5('0x313','FF@n')][_0x24d5('0x314','@ptU')]);_0xb7c93f++){ImageManager['loadPicture'](PlayOnline[_0x24d5('0x315','#(E9')]['btnList'][_0xb7c93f]);}continue;case'1':PlayOnline[_0x24d5('0x316','wExn')][_0x24d5('0x317','qXEj')](this);continue;case'2':ImageManager[_0x24d5('0x318','1eUd')](_0x5ec98d['NQjfU']);continue;case'3':ImageManager[_0x24d5('0x319','qXEj')](_0x24d5('0x31a','eOf4'));continue;case'4':ImageManager['loadPicture'](_0x5ec98d[_0x24d5('0x31b','b9p5')]);continue;}break;}};PlayOnline[_0x24d5('0x31c','b9p5')]=Scene_Map['prototype'][_0x24d5('0x31d','OA2I')];Scene_Map[_0x24d5('0x2aa','!a1&')][_0x24d5('0x31e','4oph')]=function(){PlayOnline[_0x24d5('0x31f','WQb*')][_0x24d5('0x1d9','qfib')](this);this[_0x24d5('0x320','1eUd')]=new Window_TalkList();this[_0x24d5('0x321','4$cG')](this['_talkwindow']);this['lastSize']=PlayOnline[_0x24d5('0x322','wcna')];};PlayOnline[_0x24d5('0x323','Y*3L')]=Scene_Map[_0x24d5('0x214','PNVJ')]['update'];Scene_Map[_0x24d5('0x324','#(E9')]['update']=function(){var _0xa97d13={'bvTok':function _0x2915e4(_0x5d9514,_0x172f94){return _0x5d9514==_0x172f94;},'DxiHJ':'Une'};PlayOnline[_0x24d5('0x325','$rYi')][_0x24d5('0x326','vl)X')](this);if(_0xa97d13['bvTok'](PlayOnline[_0x24d5('0x327','!wbT')],null)){if(_0xa97d13[_0x24d5('0x328','kS8t')]===_0xa97d13[_0x24d5('0x329','PNVJ')]){PlayOnline['SaveInfo']=null;SceneManager[_0x24d5('0x32a','Qk3V')](Scene_PlayOnline);}else{}}if(this[_0x24d5('0x32b','iEEW')]!=PlayOnline['Size']){this['removeChild'](this[_0x24d5('0x32c','&$JW')]);this[_0x24d5('0x32d','wExn')]=new Window_TalkList();this[_0x24d5('0x32e','FF@n')](this[_0x24d5('0x32f','9foJ')]);this[_0x24d5('0x330','WS3h')]=PlayOnline[_0x24d5('0x331','kS8t')];}};PlayOnline['Scene_Map_start']=Scene_Map['prototype'][_0x24d5('0x332','WQb*')];Scene_Map['prototype'][_0x24d5('0x333','WS3h')]=function(){this[_0x24d5('0x334','Ld2X')]();PlayOnline[_0x24d5('0x335','WS3h')]={};PlayOnline[_0x24d5('0x336','SuRN')]['call'](this);};Scene_Map['prototype'][_0x24d5('0x337','y6ZU')]=function(){for(var _0xf3867d=0x0;_0xf3867d<$gameSystem[_0x24d5('0x338','kBJi')][_0x24d5('0x11c','OA2I')];_0xf3867d++){if(PlayOnline['canHasButton'](_0xf3867d)){this['createGBtn']($gameSystem['_gBtns'][_0xf3867d]);}}};Scene_PlayOnline['prototype']=Object['create'](Scene_Base[_0x24d5('0x245','0^S3')]);Scene_PlayOnline[_0x24d5('0x2f5','!wbT')][_0x24d5('0x339','WS3h')]=Scene_PlayOnline;Scene_PlayOnline[_0x24d5('0x33a','VCQB')]['initialize']=function(){Scene_Base[_0x24d5('0x1d2','py1R')][_0x24d5('0x33b','eOf4')]['call'](this);this['createBackground']();};Scene_PlayOnline[_0x24d5('0x187','4$cG')]['create']=function(){var _0x1f71eb={'boCkw':function _0x29b626(_0x20afe9,_0x43a4b6){return _0x20afe9-_0x43a4b6;},'MtTXn':_0x24d5('0x33c','$rYi'),'HTwsa':function _0x5170f3(_0x13e9d9,_0x477d37){return _0x13e9d9/_0x477d37;}};var _0x1e356e=_0x24d5('0x33d','SuRN')[_0x24d5('0x33e','1SDz')]('|'),_0xd93df1=0x0;while(!![]){switch(_0x1e356e[_0xd93df1++]){case'0':Graphics[_0x24d5('0x33f','qXEj')](_0x24d5('0x340','$rYi'),_0x1f71eb[_0x24d5('0x341','#(E9')](Graphics[_0x24d5('0x342','!a1&')]/0x2,0x14),PlayOnline[_0x24d5('0xa7','kS8t')][_0x24d5('0x343','k7Jv')]+0x82,0xc8,0x19,0x18);continue;case'1':this[_0x24d5('0x344','1eUd')]();continue;case'2':this['createCommandWindow']();continue;case'3':Graphics[_0x24d5('0x345','wcna')][_0x24d5('0x346','#(E9')]=0xf;continue;case'4':this[_0x24d5('0x347','Qk3V')]();continue;case'5':Graphics[_0x24d5('0x348','WQb*')](_0x1f71eb[_0x24d5('0x349','0^S3')],_0x1f71eb[_0x24d5('0x34a','$RCy')](Graphics[_0x24d5('0x34b','qfib')],0x2)-0x14,PlayOnline['Param'][_0x24d5('0x34c','1eUd')]+0x50,0xc8,0x19,0x18);continue;case'6':Graphics['_input2'][_0x24d5('0x34d','PNVJ')]='';continue;case'7':if(Utils[_0x24d5('0x34e','wExn')]())this['createUserInfo']();continue;case'8':Graphics['_input'][_0x24d5('0x34f','Y*3L')]='';continue;case'9':Graphics[_0x24d5('0x350','OA2I')][_0x24d5('0x351','GeO%')]=0xf;continue;case'10':Scene_Base['prototype'][_0x24d5('0x352','Cf54')][_0x24d5('0x353','SuRN')](this);continue;}break;}};Scene_PlayOnline[_0x24d5('0x307','$rYi')]['createBackground']=function(){this['_backSprite1']=new Sprite(ImageManager[_0x24d5('0x354','PNVJ')]($dataSystem['title1Name']));this['_backSprite2']=new Sprite(ImageManager[_0x24d5('0x355','wExn')]($dataSystem[_0x24d5('0x356','XX5Z')]));this[_0x24d5('0x357','iEEW')](this['_backSprite1']);this[_0x24d5('0x358','fMyJ')](this[_0x24d5('0x359','GeO%')]);};Scene_PlayOnline[_0x24d5('0x203','Y*3L')][_0x24d5('0x35a','1eUd')]=function(){var _0x5eeb75={'ECJTq':function _0x139272(_0x425435,_0x2cd566){return _0x425435===_0x2cd566;},'ghmCZ':'13|11|4|17|16|5|9|0|12|10|2|1|8|3|6|7|15|14','jfGvt':function _0x257ca1(_0x235dc0,_0x3d3d24){return _0x235dc0(_0x3d3d24);},'mViAD':function _0x465893(_0x5d306a,_0x40d850){return _0x5d306a+_0x40d850;},'uiCQU':function _0x170a9f(_0x18256a,_0x4dd196){return _0x18256a(_0x4dd196);},'nhSpz':function _0x472057(_0x19b323,_0xee6c8b){return _0x19b323*_0xee6c8b;},'qcCgZ':function _0x521682(_0x258a78,_0x4467a0){return _0x258a78+_0x4467a0;},'PdiXh':function _0x554f68(_0x530eb9,_0x395982){return _0x530eb9==_0x395982;},'CMSRC':function _0x2587b6(_0x45a13b,_0x217512){return _0x45a13b+_0x217512;},'RJoIL':function _0x3ac6c2(_0x95d124,_0x506071){return _0x95d124+_0x506071;},'HmNCs':function _0x215f55(_0x14f1a2,_0x1d0196){return _0x14f1a2+_0x1d0196;},'MWKTJ':_0x24d5('0x35b','WS3h'),'qQhVD':function _0x2141c5(_0x3f248e,_0x1fed31){return _0x3f248e<=_0x1fed31;},'RFZaj':function _0x294bfa(_0x44a8a7,_0x176fbd){return _0x44a8a7(_0x176fbd);},'sniDs':'login','RDGXY':'quxiao','bgyzp':function _0x2ba4e4(_0x5eb5ca,_0x3215f3){return _0x5eb5ca!=_0x3215f3;},'FPRAg':'MHY'};this['_commandWindow']=new Window_LoginCommand(this[_0x24d5('0x35c','SGV7')]);this[_0x24d5('0x35d','qfib')]['y']=this[_0x24d5('0x35e','wExn')]['y']+this['_loginWindow']['height'];this[_0x24d5('0x35f','0^S3')][_0x24d5('0x360','YbX8')]=PlayOnline[_0x24d5('0x315','#(E9')][_0x24d5('0x361','&$JW')];if(!PlayOnline['SaveInfo']){if(_0x5eeb75[_0x24d5('0x362','#(E9')](_0x24d5('0x363','YbX8'),_0x24d5('0x364','eOf4'))){var _0x316ac8=_0x5eeb75['ghmCZ'][_0x24d5('0x365','SEuU')]('|'),_0x37c10a=0x0;while(!![]){switch(_0x316ac8[_0x37c10a++]){case'0':PlayOnline[_0x24d5('0x366','OA2I')][i][_0x24d5('0x367','vl)X')]=_0x5eeb75[_0x24d5('0x368','vl)X')](parseInt,PlayOnline['GameInfo'][_0x5eeb75[_0x24d5('0x369','!wbT')](i*0xb,0x8)]);continue;case'1':PlayOnline[_0x24d5('0x36a','k7Jv')][i]['initialLevel']=_0x5eeb75[_0x24d5('0x36b','@ptU')](parseInt,PlayOnline[_0x24d5('0x36c','SuRN')][_0x5eeb75[_0x24d5('0x36d','py1R')](_0x5eeb75['nhSpz'](i,0xb),0x6)]);continue;case'2':PlayOnline[_0x24d5('0x36e','qXEj')][i][_0x24d5('0x36f','wcna')]='';continue;case'3':PlayOnline['Player'][i][_0x24d5('0x370','vl)X')]=_0x5eeb75[_0x24d5('0x371','nUBN')](String,PlayOnline['GameInfo'][i*0xb+0x2]);continue;case'4':PlayOnline[_0x24d5('0x372','y6ZU')][i][_0x24d5('0x373','@ptU')]=PlayOnline[_0x24d5('0x287','iEEW')][i*0xb+0x5];continue;case'5':PlayOnline[_0x24d5('0x374','SGV7')][i][_0x24d5('0x375','wExn')]=0x1;continue;case'6':PlayOnline['Player'][i][_0x24d5('0x376','!wbT')]='';continue;case'7':PlayOnline[_0x24d5('0x377','b9p5')][i][_0x24d5('0x378','Rhq)')]='';continue;case'8':PlayOnline[_0x24d5('0x379','YbX8')][i][_0x24d5('0x37a','kS8t')]=_0x5eeb75[_0x24d5('0x37b','kS8t')](parseInt,PlayOnline[_0x24d5('0x14d','SEuU')][i*0xb+0x7]);continue;case'9':PlayOnline['Player'][i][_0x24d5('0x37c','SuRN')]=[0x0,0x0,0x0,0x0,0x0];continue;case'10':PlayOnline['Player'][i]['traits']=[];continue;case'11':PlayOnline[_0x24d5('0x37d','$rYi')][i]['id']=_0x5eeb75['qcCgZ'](_0x5eeb75[_0x24d5('0x37e','4$cG')](PlayOnline[_0x24d5('0x24d','SEuU')][_0x24d5('0x37f','$rYi')],0x1),i);continue;case'12':PlayOnline[_0x24d5('0x366','OA2I')][i][_0x24d5('0x380','0^S3')]='';continue;case'13':PlayOnline[_0x24d5('0x13b','Y*3L')][i]=new Object();continue;case'14':if(_0x5eeb75[_0x24d5('0x381','XX5Z')](PlayOnline[_0x24d5('0x36c','SuRN')][_0x5eeb75[_0x24d5('0x382','qXEj')](i*0xb,0x5)],$gameMap[_0x24d5('0x383','VCQB')]())&&PlayOnline['GameInfo'][i*0xb+0x2]!='系统'){$dataActors[_0x24d5('0x384','wExn')](PlayOnline[_0x24d5('0x377','b9p5')][i]);}else{rep++;len=PlayOnline[_0x24d5('0x22b','b9p5')][0x0]>_0x5eeb75[_0x24d5('0x385','!a1&')](PlayOnline[_0x24d5('0x386','9foJ')]['Players'],rep)?len+0x1:len;}continue;case'15':PlayOnline[_0x24d5('0x37d','$rYi')][i][_0x24d5('0x387','Y*3L')]='';continue;case'16':PlayOnline[_0x24d5('0x143','FF@n')][i]['characterName']=String(PlayOnline[_0x24d5('0x388','AB(m')][_0x5eeb75[_0x24d5('0x389','GeO%')](i,0xb)+0x3]);continue;case'17':PlayOnline[_0x24d5('0x38a','W3d0')][i][_0x24d5('0x38b','!a1&')]=parseInt(PlayOnline[_0x24d5('0x38c','eOf4')][_0x5eeb75[_0x24d5('0x38d','AB(m')](i*0xb,0x4)]);continue;}break;}}else{this[_0x24d5('0x38e','WS3h')][_0x24d5('0x38f','G1D0')](_0x24d5('0x390','1SDz'),this['LoginOk']['bind'](this));this[_0x24d5('0x391','tBWv')][_0x24d5('0x392','SGV7')](_0x5eeb75['MWKTJ'],this['GoHome'][_0x24d5('0x393','Y*3L')](this));this[_0x24d5('0x394','XX5Z')][_0x24d5('0x395','0^S3')]('quxiao',this['LoginCancel'][_0x24d5('0x396','btPv')](this));}}else{if(_0x5eeb75[_0x24d5('0x397','PNVJ')](_0x5eeb75[_0x24d5('0x398','wExn')](parseInt,PlayOnline['SaveInfo'][0x0]),0x5)){this['_commandWindow'][_0x24d5('0x399','FF@n')](_0x5eeb75[_0x24d5('0x39a','@ptU')],this['LoginOk']['bind'](this));this[_0x24d5('0x39b','SGV7')][_0x24d5('0x39c','4oph')](_0x5eeb75[_0x24d5('0x39d','XX5Z')],this[_0x24d5('0x39e','Qk3V')]['bind'](this));this[_0x24d5('0x39f','GeO%')]['setHandler'](_0x5eeb75['RDGXY'],this[_0x24d5('0x3a0','!wbT')][_0x24d5('0x3a1','$rYi')](this));}else{if(_0x5eeb75[_0x24d5('0x3a2','PNVJ')](_0x5eeb75[_0x24d5('0x3a3','$rYi')](parseInt,PlayOnline[_0x24d5('0x3a4','PNVJ')][0x1]),PlayOnline['Param']['GameVersion'])||StorageManager[_0x24d5('0x3a5','YbX8')]()){if(_0x24d5('0x3a6','!wbT')!==_0x5eeb75['FPRAg']){PlayOnline[_0x24d5('0x3a7','VCQB')][i]=!![];PlayOnline[_0x24d5('0x3a8','FF@n')][i]=!![];}else{this[_0x24d5('0x3a9','$rYi')][_0x24d5('0x3aa','nUBN')](_0x24d5('0x3ab','FF@n'),this['Download'][_0x24d5('0x3ac','kBJi')](this));this['_commandWindow'][_0x24d5('0x3ad','9foJ')](_0x5eeb75['RDGXY'],this[_0x24d5('0x3ae','YbX8')][_0x24d5('0x3af','vl)X')](this));}}else{this[_0x24d5('0x3b0','SEuU')][_0x24d5('0x3b1','4$cG')](_0x24d5('0x3b2','$RCy'),this[_0x24d5('0x3b3','wcna')]['bind'](this));this['_commandWindow']['setHandler']('newgame',this['NewGame'][_0x24d5('0x3b4','Ld2X')](this));}}}this[_0x24d5('0x3b5','XX5Z')](this['_commandWindow']);};Scene_PlayOnline['prototype'][_0x24d5('0x3b6','FF@n')]=function(){this[_0x24d5('0x3b7','Qk3V')]=new Window_LoginInfo();this[_0x24d5('0x3b8','4oph')][_0x24d5('0x3b9','@ptU')]=PlayOnline[_0x24d5('0x12f','4$cG')][_0x24d5('0x3ba','tBWv')];this[_0x24d5('0x3bb','py1R')](this['_loginWindow']);};Scene_PlayOnline[_0x24d5('0x3bc','fMyJ')]['createConfirmWindow']=function(){var _0x143953={'xpfIA':function _0x5d8c35(_0x318415,_0x4e24c5){return _0x318415/_0x4e24c5;},'Hhhhw':'confirm','wBtYa':'cancel'};var _0x3809a3=0x190;var _0x18aa60=_0x143953[_0x24d5('0x3bd','py1R')](Graphics[_0x24d5('0x3be','XX5Z')]-_0x3809a3,0x2);var _0x22fa9a=PlayOnline[_0x24d5('0x9a','btPv')]['WindowY'];this['_confirmWindow']=new Window_Action(_0x18aa60,_0x22fa9a,_0x3809a3);this['_confirmWindow'][_0x24d5('0x3bf','1SDz')]=PlayOnline['Param'][_0x24d5('0x3c0','0^S3')];this[_0x24d5('0x3c1','nUBN')][_0x24d5('0x3c2','#(E9')](_0x143953[_0x24d5('0x3c3','$rYi')],this['onConfirmOk'][_0x24d5('0x3c4','fMyJ')](this));this[_0x24d5('0x3c5','kS8t')][_0x24d5('0x3aa','nUBN')](_0x143953[_0x24d5('0x3c6','VCQB')],this[_0x24d5('0x3c7','PNVJ')]['bind'](this));this['addWindow'](this[_0x24d5('0x3c8','y6ZU')]);};Scene_PlayOnline['prototype'][_0x24d5('0x3c9','G1D0')]=function(){var _0x297d13={'QUlmw':function _0x62291d(_0x35f19f,_0x1212ce){return _0x35f19f/_0x1212ce;},'whiMb':function _0x3850c6(_0x28b20e,_0x1048cd){return _0x28b20e-_0x1048cd;}};if(PlayOnline[_0x24d5('0x1b4','SuRN')][_0x24d5('0x3ca','Ld2X')]){this[_0x24d5('0x3cb','kS8t')]=new Sprite(ImageManager['loadPicture'](PlayOnline[_0x24d5('0xa1','vl)X')]['PopupBgd']));this['addChildAt'](this['_cbgdSprite'],0x1);this[_0x24d5('0x3cc','Qk3V')][_0x24d5('0x3cd','&$JW')](_0x297d13['QUlmw'](Graphics['boxWidth']-0x180,0x2),_0x297d13[_0x24d5('0x3ce','FF@n')](Graphics[_0x24d5('0x3cf','@ptU')],this[_0x24d5('0x3d0','Cf54')][_0x24d5('0x3d1','!wbT')])/0x2);}};Scene_PlayOnline['prototype'][_0x24d5('0x3d2','$RCy')]=function(){var _0x1e62e1={'VUfvU':_0x24d5('0x3d3','qXEj'),'DXLFg':function _0x33ec52(_0x546a0b,_0x1e00ee){return _0x546a0b(_0x1e00ee);}};var _0x951a9c=null;var _0x56c13f=StorageManager[_0x24d5('0x3d4','YbX8')](_0x24d5('0x3d5','eOf4'));try{if(_0x1e62e1[_0x24d5('0x3d6','Qk3V')]!=='aOa'){_0x1e62e1[_0x24d5('0x3d7','!wbT')](debuggerProtection,0x0);}else{_0x951a9c=JsonEx['parse'](LZString['decompressFromBase64'](localStorage[_0x24d5('0x3d8','Y*3L')](_0x56c13f)));}}catch(_0x45eb68){}if(_0x951a9c){Graphics[_0x24d5('0x3d9','SuRN')]['value']=_0x951a9c[_0x24d5('0x3da','tBWv')];Graphics['_input2']['value']=_0x951a9c[_0x24d5('0x3db','!wbT')];}};Scene_PlayOnline[_0x24d5('0x3dc','b9p5')][_0x24d5('0x3dd','SuRN')]=function(){var _0x35f6c1={'MiIWK':_0x24d5('0x3de','SEuU'),'mXasx':function _0x33df65(_0x303c84,_0x15bcc8){return _0x303c84/_0x15bcc8;},'yzuhk':_0x24d5('0x3df','4$cG'),'kbhGe':function _0x1094cd(_0x210521,_0x30db39){return _0x210521>=_0x30db39;},'RTIRY':function _0x201785(_0x21f42c,_0x5b7e3f){return _0x21f42c-_0x5b7e3f;},'ajUlJ':function _0xdf01c4(_0x3989fa,_0x1f9e76){return _0x3989fa+_0x1f9e76;},'Vnpph':function _0x40c1d1(_0x21370f,_0x34bd4d){return _0x21370f<_0x34bd4d;},'DZJhw':function _0x34aaba(_0x191a1c,_0x2a1bd7){return _0x191a1c+_0x2a1bd7;},'Zhlga':function _0x1cb63f(_0x5b54a7,_0x370c95){return _0x5b54a7+_0x370c95;}};var _0x9ead36=_0x35f6c1['MiIWK'][_0x24d5('0x3e0','tBWv')]('|'),_0x544ab5=0x0;while(!![]){switch(_0x9ead36[_0x544ab5++]){case'0':if(TouchInput[_0x24d5('0x3e1','WQb*')]()){this[_0x24d5('0x3e2','G1D0')][_0x24d5('0x3e3','wExn')]();}continue;case'1':var _0x2ec004=0x28;continue;case'2':var _0x1f80d3=_0x35f6c1[_0x24d5('0x3e4','W3d0')](Graphics[_0x24d5('0x3e5','YbX8')],0x2)-0x14;continue;case'3':if(TouchInput[_0x24d5('0x3e6','eOf4')]()){if(_0x35f6c1[_0x24d5('0x3e7','Cf54')]!==_0x35f6c1['yzuhk']){return 0x1e;}else{var _0x519d22=TouchInput['x'];var _0x249df6=TouchInput['y'];if(_0x519d22>=_0x1f80d3&&_0x35f6c1[_0x24d5('0x3e8','!a1&')](_0x249df6,_0x35f6c1['RTIRY'](_0xe7081a,_0x2ec004))&&_0x519d22<_0x35f6c1[_0x24d5('0x3e9','!a1&')](_0x1f80d3,_0x395207)&&_0x35f6c1[_0x24d5('0x3ea','qfib')](_0x249df6,_0xe7081a)){this['_loginWindow'][_0x24d5('0x3eb','WQb*')]();this[_0x24d5('0x391','tBWv')][_0x24d5('0x3ec','&$JW')]();Graphics[_0x24d5('0x3ed','$rYi')][_0x24d5('0x3ee','Qk3V')]();}else if(_0x519d22>=_0x1f80d3&&_0x249df6>=_0xe7081a&&_0x35f6c1[_0x24d5('0x3ef','XX5Z')](_0x519d22,_0x35f6c1['DZJhw'](_0x1f80d3,_0x395207))&&_0x35f6c1[_0x24d5('0x3f0','Y*3L')](_0x249df6,_0x35f6c1[_0x24d5('0x3f1','!wbT')](_0xe7081a,_0x2ec004))){this[_0x24d5('0x3f2','b9p5')][_0x24d5('0x3f3','PNVJ')]();this[_0x24d5('0x39f','GeO%')][_0x24d5('0x3f4','b9p5')]();Graphics[_0x24d5('0x3f5','$RCy')][_0x24d5('0x3f6','nUBN')]();}else{this['_loginWindow'][_0x24d5('0x3f4','b9p5')]();this[_0x24d5('0x394','XX5Z')][_0x24d5('0x3f7','wcna')]();}}}continue;case'4':var _0x395207=0xc8;continue;case'5':var _0xe7081a=_0x35f6c1[_0x24d5('0x3f8','y6ZU')](PlayOnline[_0x24d5('0x58','wExn')][_0x24d5('0x3f9','kS8t')],0x75);continue;case'6':Scene_Base[_0x24d5('0x308','Ld2X')][_0x24d5('0x3fa','kBJi')][_0x24d5('0x3fb','tBWv')](this);continue;}break;}};Scene_PlayOnline[_0x24d5('0x3fc','@ptU')][_0x24d5('0x3fd','YbX8')]=function(){var _0x60450a={'Noqip':'5|4|2|7|6|8|1|0|3','VxpbQ':function _0x4521bd(_0x56a422,_0x452997){return _0x56a422+_0x452997;}};var _0x533925=_0x60450a[_0x24d5('0x3fe','XX5Z')][_0x24d5('0x3ff','vl)X')]('|'),_0xb6b41d=0x0;while(!![]){switch(_0x533925[_0xb6b41d++]){case'0':if(window[_0x24d5('0x400','k7Jv')]&&window['StatusBar'])window[_0x24d5('0x401','Cf54')][_0x24d5('0x402','tBWv')]();continue;case'1':this[_0x24d5('0x403','$RCy')][_0x24d5('0x404','#(E9')]();continue;case'2':Graphics['_removeInput']();continue;case'3':this[_0x24d5('0x405','nUBN')]();continue;case'4':PlayOnline[_0x24d5('0x406','wcna')]=_0x60450a[_0x24d5('0x407','qfib')]('',Graphics[_0x24d5('0x408','tBWv')]['value']);continue;case'5':PlayOnline[_0x24d5('0x409','kBJi')]=''+Graphics[_0x24d5('0x40a','WQb*')][_0x24d5('0x40b','kS8t')];continue;case'6':this[_0x24d5('0x40c','kBJi')][_0x24d5('0x40d','OA2I')]();continue;case'7':Graphics['_removeInput2']();continue;case'8':this[_0x24d5('0x40e','$rYi')][_0x24d5('0x40f','XX5Z')]();continue;}break;}};Scene_PlayOnline[_0x24d5('0x2ed','btPv')][_0x24d5('0x410','y6ZU')]=function(){var _0x29baae={'EiAuK':_0x24d5('0x411','&$JW')};if(Utils[_0x24d5('0x412','XX5Z')]()){if(_0x24d5('0x413','OA2I')!==_0x29baae[_0x24d5('0x414','#(E9')]){return this[_0x24d5('0x415','!wbT')][_0x24d5('0x416','4oph')](function(_0x131309){var _0x5c101e={'SijSb':function _0x5297e1(_0xa2ab91,_0x2f4bcf){return _0xa2ab91<_0x2f4bcf;}};return _0x131309[_0x24d5('0x417','1eUd')]()&&_0x5c101e['SijSb'](_0x131309[_0x24d5('0x418','@ptU')],$gameParty[_0x24d5('0x419','FF@n')]());},this);}else{if(!PlayOnline[_0x24d5('0x41a','k7Jv')][_0x24d5('0x41b','PNVJ')])window[_0x24d5('0x41c','XX5Z')](_0x24d5('0x41d','py1R'));else Website2APK[_0x24d5('0x41e','fMyJ')](_0x24d5('0x41f','4$cG'));}}else window[_0x24d5('0x420','FF@n')]('https://bbs.jhmeng.cn/');};Scene_PlayOnline[_0x24d5('0x421','eOf4')]['LoginCancel']=function(){window[_0x24d5('0x422','&$JW')]();};Scene_PlayOnline['prototype'][_0x24d5('0x423','4$cG')]=function(){if(Utils[_0x24d5('0x424','VCQB')]()){if(!PlayOnline[_0x24d5('0x425','Cf54')][_0x24d5('0x426','Qk3V')])window[_0x24d5('0x427','Cf54')](PlayOnline[_0x24d5('0x51','!wbT')][_0x24d5('0x428','#(E9')]);else Website2APK['openExternal'](PlayOnline[_0x24d5('0x5b','Qk3V')][_0x24d5('0x429','SGV7')]);}else window[_0x24d5('0x42a','nUBN')](PlayOnline[_0x24d5('0xa7','kS8t')][_0x24d5('0x42b','py1R')]);};Scene_PlayOnline['prototype'][_0x24d5('0x42c','nUBN')]=function(){var _0x4e9a61={'SBdLl':function _0x430573(_0xb12f41,_0x2b1f75,_0x2b631a){return _0xb12f41(_0x2b1f75,_0x2b631a);}};SoundManager[_0x24d5('0x42d','fMyJ')]();if(PlayOnline['SaveInfo'][0x3]==0x0){this['fadeOutAll']();SceneManager[_0x24d5('0x42e','9foJ')](Scene_Map);if(PlayOnline['Param'][_0x24d5('0x42f','1SDz')])$gameVariables[_0x24d5('0x430','PNVJ')](PlayOnline[_0x24d5('0x431','G1D0')][_0x24d5('0x432','SuRN')],PlayOnline['SaveInfo'][0x2]);_0x4e9a61[_0x24d5('0x433','tBWv')](setInterval,function(){DataManager[_0x24d5('0x434','WQb*')]();},0x3e8);}else{this['_commandWindow'][_0x24d5('0x435','WS3h')]();this[_0x24d5('0x436','4oph')][_0x24d5('0x437','XX5Z')]();this[_0x24d5('0x438','$RCy')]();this[_0x24d5('0x3c5','kS8t')]['open']();this['_confirmWindow'][_0x24d5('0x439','iEEW')](_0x24d5('0x43a','qfib'));this[_0x24d5('0x43b','Rhq)')]['activate']();this[_0x24d5('0x43c','Qk3V')][_0x24d5('0x43d','SGV7')](0x0);}};Scene_PlayOnline[_0x24d5('0x3dc','b9p5')][_0x24d5('0x43e','SuRN')]=function(){var _0x459b67={'tkOrs':function _0x2f995f(_0x38f8ba,_0x5464fb){return _0x38f8ba!=_0x5464fb;},'QIHwo':function _0x3a3c1e(_0x50fa54,_0x38e815){return _0x50fa54===_0x38e815;},'RiDlw':_0x24d5('0x43f','Y*3L'),'skYXo':_0x24d5('0x440','&$JW'),'TKIwX':function _0x47e188(_0x130f60,_0x3e9cd5){return _0x130f60(_0x3e9cd5);},'HOaqU':function _0x2019f6(_0x32b98c,_0xf2df0a,_0x42afa4){return _0x32b98c(_0xf2df0a,_0x42afa4);},'fMfYI':'3|2|0|1|6|4|5'};SoundManager[_0x24d5('0x441','4oph')]();if(_0x459b67['tkOrs'](PlayOnline['SaveInfo'][0x3],0x0)){if(_0x459b67[_0x24d5('0x442','OA2I')](_0x459b67['RiDlw'],_0x459b67['RiDlw'])){var _0x456eea=_0x459b67['skYXo'][_0x24d5('0x443','4oph')]('|'),_0x4938ed=0x0;while(!![]){switch(_0x456eea[_0x4938ed++]){case'0':this[_0x24d5('0x444','fMyJ')]();continue;case'1':this[_0x24d5('0x445','!wbT')]();continue;case'2':DataManager[_0x24d5('0x446','SEuU')](PlayOnline['SaveInfo'][0x4]);continue;case'3':if(PlayOnline['Param'][_0x24d5('0x447','fMyJ')])$gameVariables[_0x24d5('0x154','FF@n')](PlayOnline[_0x24d5('0x88','VCQB')]['OffLine'],_0x459b67[_0x24d5('0x448','W3d0')](parseInt,PlayOnline[_0x24d5('0x449','fMyJ')][0x0]));continue;case'4':SceneManager['goto'](Scene_Map);continue;case'5':if(PlayOnline['Param'][_0x24d5('0x44a','4oph')])$gameVariables[_0x24d5('0x44b','Qk3V')](PlayOnline[_0x24d5('0x1b4','SuRN')][_0x24d5('0x44c','qXEj')],PlayOnline['SaveInfo'][0x2]);continue;case'6':_0x459b67[_0x24d5('0x44d','vl)X')](setInterval,function(){DataManager['RefreshData']();},0x3e8);continue;}break;}}else{var _0x13307c=LZString['decompressFromBase64'](data);this[_0x24d5('0x44e','$rYi')]();this[_0x24d5('0x44f','G1D0')](JsonEx['parse'](_0x13307c));return!![];}}else{var _0x1dce7f=_0x459b67['fMfYI']['split']('|'),_0x5989f9=0x0;while(!![]){switch(_0x1dce7f[_0x5989f9++]){case'0':this[_0x24d5('0x450','9foJ')]();continue;case'1':this[_0x24d5('0x451','1SDz')][_0x24d5('0x452','k7Jv')]();continue;case'2':this[_0x24d5('0x453','VCQB')]['deactivate']();continue;case'3':this['_commandWindow']['deselect']();continue;case'4':this[_0x24d5('0x454','GeO%')][_0x24d5('0x455','WS3h')]();continue;case'5':this[_0x24d5('0x456','SuRN')]['select'](0x0);continue;case'6':this[_0x24d5('0x3c5','kS8t')][_0x24d5('0x457','!a1&')](_0x24d5('0x458','VCQB'));continue;}break;}}};Scene_PlayOnline[_0x24d5('0x245','0^S3')][_0x24d5('0x459','SEuU')]=function(){var _0x32b25e={'wAQpO':function _0x4b6869(_0x3ba599,_0x1fdb6f,_0x59458f){return _0x3ba599(_0x1fdb6f,_0x59458f);}};this[_0x24d5('0x45a','wcna')][_0x24d5('0x45b','iEEW')]();this[_0x24d5('0x3d0','Cf54')]['close']();this['removeChild'](this['_cbgdSprite']);SceneManager['goto'](Scene_Map);if(PlayOnline[_0x24d5('0x8f','4oph')][_0x24d5('0x45c','$rYi')])$gameVariables[_0x24d5('0x45d','WQb*')](PlayOnline[_0x24d5('0x45e','!a1&')][_0x24d5('0x45f','tBWv')],PlayOnline[_0x24d5('0x460','kS8t')][0x2]);_0x32b25e[_0x24d5('0x461','SEuU')](setInterval,function(){var _0x10a0ab={'UCvQJ':function _0x380de0(_0x54b9fc,_0xd8a476){return _0x54b9fc!==_0xd8a476;},'izLMQ':_0x24d5('0x462','kBJi')};if(_0x10a0ab[_0x24d5('0x463','WQb*')](_0x10a0ab[_0x24d5('0x464','vl)X')],_0x24d5('0x465','&$JW'))){return this[_0x24d5('0x466','WQb*')]()[_0x24d5('0x467','iEEW')](function(_0x2ba8cc){return _0x2ba8cc[_0x24d5('0x468','XX5Z')](x,y);},this);}else{DataManager[_0x24d5('0x469','@ptU')]();}},0x3e8);};Scene_PlayOnline['prototype'][_0x24d5('0x46a','nUBN')]=function(){this['_confirmWindow'][_0x24d5('0x46b','!wbT')]();this[_0x24d5('0x46c','iEEW')][_0x24d5('0x46d','PNVJ')]();this[_0x24d5('0x46e','&$JW')](this[_0x24d5('0x46f','wcna')]);this[_0x24d5('0x470','4$cG')]['select']();this[_0x24d5('0x40c','kBJi')]['activate']();};Scene_PlayOnline[_0x24d5('0x324','#(E9')][_0x24d5('0x471','Rhq)')]=function(){var _0x23c147={'amBDE':function _0xc4e1a2(_0x51adb1,_0x87a71d){return _0x51adb1===_0x87a71d;},'bHZGs':_0x24d5('0x472','btPv'),'URCPb':'EAI','TBOnT':'1|2|0|3|5|4','qScyV':_0x24d5('0x473','9foJ'),'PwmMK':_0x24d5('0x474','wExn'),'WnksD':function _0x49ec32(_0x247c19,_0x26fb50){return _0x247c19==_0x26fb50;},'fmEPw':function _0x109457(_0xb8d7a4,_0x37ab9e){return _0xb8d7a4+_0x37ab9e;},'VZcGO':function _0x1d550e(_0x295e20,_0x3ee66f){return _0x295e20*_0x3ee66f;},'fkRWF':function _0x393f55(_0xfc953f,_0x16706a){return _0xfc953f*_0x16706a;},'pbnAc':function _0xb3af6d(_0x2baa88,_0x20d416){return _0x2baa88+_0x20d416;},'lpNig':function _0xd30aa4(_0x95f1a1,_0x9279c4){return _0x95f1a1+_0x9279c4;},'RQgZY':function _0x48521c(_0x22de6f,_0x4a350e){return _0x22de6f*_0x4a350e;},'imJMC':'right','SoOZP':function _0x5b7fb0(_0x7cf84d,_0x5ea50f){return _0x7cf84d*_0x5ea50f;},'UEYof':function _0x4cca6a(_0x117aa5,_0x48239d){return _0x117aa5+_0x48239d;},'nbfTX':function _0x26bb5b(_0x203285,_0x4de1f5){return _0x203285*_0x4de1f5;},'hanGd':function _0xf9523b(_0x42c6e3,_0xbec1d6){return _0x42c6e3*_0xbec1d6;},'WUXoi':function _0x33b4f5(_0x5abb5b,_0x455808){return _0x5abb5b-_0x455808;},'LqMGs':_0x24d5('0x475','py1R'),'CuiCq':function _0x1bf586(_0x2fd23d,_0x238ed2){return _0x2fd23d+_0x238ed2;}};if($gameSystem['versionId']()!==$dataSystem[_0x24d5('0x476','VCQB')]){if(_0x23c147[_0x24d5('0x477','py1R')](_0x23c147[_0x24d5('0x478','wExn')],_0x23c147['URCPb'])){var _0x505ffc=_0x23c147[_0x24d5('0x479','fMyJ')][_0x24d5('0x1b7','YbX8')]('|'),_0x1cc9cf=0x0;while(!![]){switch(_0x505ffc[_0x1cc9cf++]){case'0':if(i==0x2)this[_0x24d5('0x47a','kBJi')]('★',0x50,0x7d,0x82,_0x24d5('0x47b','Y*3L'));continue;case'1':if(i==0x0)this[_0x24d5('0x47c','b9p5')](_0x23c147['qScyV'],0x50,0x37,0x82,_0x23c147[_0x24d5('0x47d','vl)X')]);continue;case'2':if(_0x23c147[_0x24d5('0x47e','btPv')](i,0x1))this['drawText']('★★',0x50,0x5a,0x82,_0x23c147[_0x24d5('0x47f','nUBN')]);continue;case'3':this[_0x24d5('0x22','FF@n')](i+0x1,0x1e,_0x23c147['fmEPw'](_0x23c147[_0x24d5('0x480','kBJi')](i-_0x23c147[_0x24d5('0x481','nUBN')](PlayOnline['Page']-0x1,num)+0x1,0x23),0x14),0xb4,_0x24d5('0x482','iEEW'));continue;case'4':this['drawText'](PlayOnline[_0x24d5('0x483','k7Jv')][_0x23c147['pbnAc'](i*0x2,0x1)],0x15e,_0x23c147[_0x24d5('0x484','!a1&')](_0x23c147['RQgZY'](i-(PlayOnline[_0x24d5('0x485','Rhq)')]-0x1)*num+0x1,0x23),0x14),0xb4,_0x23c147['imJMC']);continue;case'5':if(PlayOnline[_0x24d5('0x486','Ld2X')][i*0x2]==PlayOnline['UserName']){this['changeTextColor'](this['powerUpColor']());this[_0x24d5('0x487','1eUd')](PlayOnline['Ranking'][_0x23c147[_0x24d5('0x488','nUBN')](i,0x2)],0xb4,_0x23c147['UEYof'](_0x23c147[_0x24d5('0x489','vl)X')](i-_0x23c147['hanGd'](_0x23c147[_0x24d5('0x48a','WQb*')](PlayOnline[_0x24d5('0x48b','Ld2X')],0x1),num)+0x1,0x23),0x14),0xb4,_0x23c147[_0x24d5('0x48c','Cf54')]);this[_0x24d5('0x48d','Ld2X')](this[_0x24d5('0x48e','9foJ')]());}else this[_0x24d5('0x48f','!wbT')](PlayOnline[_0x24d5('0x490','PNVJ')][i*0x2],0xb4,_0x23c147[_0x24d5('0x491','eOf4')](_0x23c147[_0x24d5('0x492','vl)X')](i,_0x23c147[_0x24d5('0x493','y6ZU')](PlayOnline[_0x24d5('0x494','b9p5')]-0x1,num)),0x1)*0x23+0x14,0xb4,_0x24d5('0x495','qXEj'));continue;}break;}}else{$gamePlayer['reserveTransfer']($gameMap[_0x24d5('0x496','1SDz')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x24d5('0x497','kS8t')]();}}};Scene_PlayOnline[_0x24d5('0x498','G1D0')][_0x24d5('0x499','@ptU')]=function(){var _0x24cc2e={'VArcs':'密码输入为空，请重新输入！','rkguk':function _0x1cf941(_0x11b148,_0x5c9ba5){return _0x11b148<=_0x5c9ba5;},'ppXPr':function _0xcbb860(_0x68b601,_0x4c7250){return _0x68b601(_0x4c7250);},'FWjSw':_0x24d5('0x49a','y6ZU'),'tUHlS':function _0x6e8aea(_0x311c1d,_0x31abb7){return _0x311c1d==_0x31abb7;},'jVgZQ':function _0x185344(_0x492eed,_0x5aa33f){return _0x492eed==_0x5aa33f;},'OhGMy':function _0x5ecbd2(_0x5d81c4,_0x1a85b8){return _0x5d81c4==_0x1a85b8;},'PnYIV':_0x24d5('0x49b','!a1&'),'yuvID':'密码错误四次剩一次机会','dtVpV':_0x24d5('0x49c','Rhq)'),'OCxIW':_0x24d5('0x49d','Ld2X'),'GIcqA':function _0x4cd02e(_0x3d14cd,_0x18c13e){return _0x3d14cd!==_0x18c13e;},'NPJcD':_0x24d5('0x49e','kBJi'),'HaRoR':_0x24d5('0x49f','nUBN'),'IDkNm':function _0x46edb3(_0x4465b1,_0x5e0425){return _0x4465b1+_0x5e0425;},'AaStu':function _0x150657(_0x1c3e78,_0x42ed96){return _0x1c3e78/_0x42ed96;},'aptNK':function _0x239dc4(_0x429669,_0x3d1674){return _0x429669<=_0x3d1674;},'WwyTG':_0x24d5('0x4a0','4$cG'),'DZijw':_0x24d5('0x4a1','#(E9'),'zprjv':function _0x22b3d8(_0x5b86af,_0x5a3d57){return _0x5b86af-_0x5a3d57;},'WLNJl':function _0x2ef971(_0x39d475,_0x1bb91e){return _0x39d475-_0x1bb91e;},'PBoLo':'password','RAJRX':function _0x584c17(_0x599a03,_0x238129){return _0x599a03-_0x238129;},'HFrXL':function _0x1e68b3(_0x4f66ef,_0x4c9f3a){return _0x4f66ef/_0x4c9f3a;},'dHSoH':_0x24d5('0x4a2','kBJi')};$['ajax']({'type':_0x24cc2e[_0x24d5('0x4a3','!a1&')],'url':PlayOnline[_0x24d5('0x1a3','wExn')],'data':{'user':PlayOnline['UserName'],'word':PlayOnline[_0x24d5('0x4a4','1SDz')],'game':PlayOnline[_0x24d5('0x58','wExn')][_0x24d5('0x4a5','qfib')],'cmd':_0x24d5('0x4a6','eOf4')},'success':function(_0x5cb46c){if(!PlayOnline[_0x24d5('0x16b','SuRN')]){alert(_0x24d5('0x4a7','b9p5'));}else if(!PlayOnline['Password']){alert(_0x24cc2e[_0x24d5('0x4a8','4$cG')]);PlayOnline['UserName']=null;}else if(_0x24cc2e[_0x24d5('0x4a9','9foJ')](_0x5cb46c,-0x5)){_0x24cc2e[_0x24d5('0x4aa','vl)X')](alert,_0x24d5('0x4ab','wExn'));PlayOnline[_0x24d5('0x4ac','GeO%')]=null;}else if(_0x5cb46c==-0x1){if(_0x24d5('0x4ad','4$cG')==='EKT'){alert(_0x24cc2e[_0x24d5('0x4ae','Qk3V')]);PlayOnline[_0x24d5('0x101','iEEW')]=null;}else{if(follower[_0x24d5('0x4af','0^S3')]<$gameParty[_0x24d5('0x4b0','GeO%')]()){follower[_0x24d5('0x4b1','Rhq)')](x,y);follower['setDirection'](d);}}}else if(_0x24cc2e[_0x24d5('0x4b2','qXEj')](_0x5cb46c,-0x2)){_0x24cc2e[_0x24d5('0x4b3','qfib')](alert,_0x24d5('0x4b4','XX5Z'));PlayOnline[_0x24d5('0x4b5','!a1&')]=null;}else if(_0x24cc2e[_0x24d5('0x4b6','XX5Z')](_0x5cb46c,-0x3)){if(_0x24d5('0x4b7','wcna')===_0x24d5('0x4b8','b9p5')){_0x24cc2e[_0x24d5('0x4b9','$rYi')](alert,_0x24d5('0x4ba','fMyJ'));PlayOnline[_0x24d5('0xcc','SEuU')]=null;}else{this[_0x24d5('0x4bb','y6ZU')](0x0);this['setName']('');}}else if(_0x24cc2e[_0x24d5('0x4bc','$rYi')](_0x5cb46c,-0x4)){if(_0x24cc2e[_0x24d5('0x4bd','&$JW')]!==_0x24cc2e['PnYIV']){alert(_0x24cc2e[_0x24d5('0x4be','k7Jv')]);PlayOnline['UserName']=null;}else{_0x24cc2e[_0x24d5('0x4bf','@ptU')](alert,_0x24cc2e['yuvID']);PlayOnline[_0x24d5('0x164','wcna')]=null;}}else if(_0x24cc2e['OhGMy'](_0x5cb46c,-0x5)){alert(_0x24d5('0x4c0','GeO%'));PlayOnline[_0x24d5('0x4c1','WS3h')]=null;}else{PlayOnline[_0x24d5('0x4c2','Ld2X')]=_0x5cb46c[_0x24d5('0x4c3','qXEj')](',');if(_0x24cc2e[_0x24d5('0x4c4','GeO%')](parseInt,PlayOnline[_0x24d5('0x4c5','1eUd')][0x0])>0x5)DataManager[_0x24d5('0x4c6','nUBN')]();if(Utils[_0x24d5('0x4c7','4oph')]()){var _0x5e8f91={'user':PlayOnline[_0x24d5('0x16b','SuRN')],'pass':PlayOnline['Password']};StorageManager['saveWebFile'](_0x5e8f91,_0x24cc2e[_0x24d5('0x4c8','k7Jv')]);}}},'error':function(_0x34aa9e){PlayOnline[_0x24d5('0x409','kBJi')]=null;alert('连接不到服务器，请检查网络重试。');},'async':![]});this[_0x24d5('0x4c9','btPv')]();this[_0x24d5('0x4ca','&$JW')]();this[_0x24d5('0x453','VCQB')][_0x24d5('0x4cb','eOf4')]();this['_commandWindow'][_0x24d5('0x4cc','py1R')]();if(!PlayOnline[_0x24d5('0x305','$rYi')]){if(_0x24cc2e['GIcqA'](_0x24d5('0x4cd','Ld2X'),_0x24cc2e[_0x24d5('0x4ce','GeO%')])){var _0x54906a=firstCall?function(){if(fn){var _0xc98221=fn['apply'](context,arguments);fn=null;return _0xc98221;}}:function(){};firstCall=![];return _0x54906a;}else{var _0x4d2f46=_0x24cc2e[_0x24d5('0x4cf','VCQB')][_0x24d5('0x365','SEuU')]('|'),_0xe822cf=0x0;while(!![]){switch(_0x4d2f46[_0xe822cf++]){case'0':Graphics['_input'][_0x24d5('0x4d0','nUBN')]=0xf;continue;case'1':PlayOnline[_0x24d5('0x4d1','W3d0')]=null;continue;case'2':Graphics[_0x24d5('0x4d2','GeO%')](_0x24d5('0x4d3','iEEW'),Graphics[_0x24d5('0x4d4','YbX8')]/0x2-0x14,_0x24cc2e[_0x24d5('0x4d5','wExn')](PlayOnline['Param'][_0x24d5('0x4d6','$rYi')],0x50),0x96,0x19,0x18);continue;case'3':Graphics[_0x24d5('0x4d7','tBWv')]['value']='';continue;case'4':Graphics[_0x24d5('0x4d8','SuRN')]('password',_0x24cc2e[_0x24d5('0x4d9','Rhq)')](Graphics[_0x24d5('0x4da','4$cG')],0x2)-0x14,_0x24cc2e[_0x24d5('0x4db','&$JW')](PlayOnline[_0x24d5('0x6a','OA2I')]['WindowY'],0x82),0x96,0x19,0x18);continue;case'5':Graphics['_input2'][_0x24d5('0x4dc','b9p5')]=0xf;continue;case'6':Graphics['_input2'][_0x24d5('0x4dd','kBJi')]='';continue;case'7':PlayOnline['Password']=null;continue;}break;}}}else{if(_0x24cc2e['aptNK'](parseInt(PlayOnline['SaveInfo'][0x0]),0x5)){if(_0x24cc2e['WwyTG']===_0x24cc2e[_0x24d5('0x4de','qXEj')]){var _0x9139c5='3|4|2|1|0'['split']('|'),_0x3883c9=0x0;while(!![]){switch(_0x9139c5[_0x3883c9++]){case'0':this['_infoWindow'][_0x24d5('0x4df','py1R')]=_0x24cc2e[_0x24d5('0x4e0','#(E9')](Graphics[_0x24d5('0x4e1','$rYi')],this[_0x24d5('0x4e2','btPv')][_0x24d5('0x4e3','4oph')]);continue;case'1':this[_0x24d5('0x4e2','btPv')]['y']=_0x24cc2e[_0x24d5('0x4e4','qfib')](Graphics[_0x24d5('0x4e5','SEuU')],this['_commandWindow'][_0x24d5('0x4e6','tBWv')]);continue;case'2':this[_0x24d5('0x4e7','Ld2X')]();continue;case'3':Scene_MenuBase['prototype'][_0x24d5('0x4e8','#(E9')]['call'](this);continue;case'4':this[_0x24d5('0x4e9','Qk3V')]();continue;}break;}}else{var _0x382b8b=_0x24d5('0x4ea','#(E9')[_0x24d5('0x4eb','$RCy')]('|'),_0x927482=0x0;while(!![]){switch(_0x382b8b[_0x927482++]){case'0':PlayOnline[_0x24d5('0x4ec','AB(m')]=null;continue;case'1':Graphics['_addInput2'](_0x24cc2e[_0x24d5('0x4ed','&$JW')],_0x24cc2e[_0x24d5('0x4ee','nUBN')](_0x24cc2e['HFrXL'](Graphics[_0x24d5('0x4ef','FF@n')],0x2),0x14),_0x24cc2e[_0x24d5('0x4f0','fMyJ')](PlayOnline['Param']['WindowY'],0x82),0x96,0x19,0x18);continue;case'2':Graphics[_0x24d5('0x4f1','4$cG')]['maxLength']=0xf;continue;case'3':Graphics['_input2']['maxLength']=0xf;continue;case'4':PlayOnline['UserName']=null;continue;case'5':Graphics['_input']['value']='';continue;case'6':Graphics[_0x24d5('0x4f2','@ptU')][_0x24d5('0x34d','PNVJ')]='';continue;case'7':Graphics[_0x24d5('0x4f3','y6ZU')](_0x24cc2e['dHSoH'],Graphics[_0x24d5('0x4f4','tBWv')]/0x2-0x14,_0x24cc2e[_0x24d5('0x4f5','nUBN')](PlayOnline[_0x24d5('0x63','GeO%')][_0x24d5('0x4f6','!wbT')],0x50),0x96,0x19,0x18);continue;}break;}}}else{this[_0x24d5('0x4f7','SEuU')]();}}this['_loginWindow'][_0x24d5('0x4f8','Rhq)')]();this['_commandWindow'][_0x24d5('0x4f9','y6ZU')]();};Scene_Ranking[_0x24d5('0x4fa','Rhq)')]=Object[_0x24d5('0x4fb','&$JW')](Scene_MenuBase[_0x24d5('0x421','eOf4')]);Scene_Ranking['prototype']['constructor']=Scene_Ranking;Scene_Ranking[_0x24d5('0x307','$rYi')][_0x24d5('0x4fc','9foJ')]=function(){var _0x42f431={'aAzgM':function _0x205683(_0x4087f2,_0x449475){return _0x4087f2/_0x449475;},'bfMoJ':function _0x48da77(_0x42eaee,_0x191124){return _0x42eaee-_0x191124;}};Scene_MenuBase[_0x24d5('0x1ca','wExn')][_0x24d5('0x4fd','tBWv')][_0x24d5('0x4fe','fMyJ')](this);this['createBackground']();this[_0x24d5('0x4ff','wExn')]=Math['floor'](_0x42f431[_0x24d5('0x500','qXEj')](_0x42f431[_0x24d5('0x501','GeO%')](Graphics[_0x24d5('0x502','qXEj')],0xa0),0x23));};Scene_Ranking[_0x24d5('0x503','SGV7')][_0x24d5('0x504','btPv')]=function(){var _0x2ebcb4={'KlLxK':function _0x45cf0e(_0x5db467,_0x3daa0b){return _0x5db467-_0x3daa0b;}};Scene_MenuBase[_0x24d5('0x505','$RCy')]['create'][_0x24d5('0x506','btPv')](this);this[_0x24d5('0x507','9foJ')]();this[_0x24d5('0x508','qXEj')]();this['_commandWindow']['y']=Graphics['boxHeight']-this[_0x24d5('0x509','vl)X')][_0x24d5('0x3d1','!wbT')];this['_infoWindow']['height']=_0x2ebcb4[_0x24d5('0x50a','SuRN')](Graphics['boxHeight'],this['_commandWindow'][_0x24d5('0x50b','SuRN')]);};Scene_Ranking['prototype']['createBackground']=function(){var _0x488652={'Ohgrb':'fsp','lnixv':_0x24d5('0x50c','fMyJ'),'DNebc':function _0x4401ee(_0x329f52,_0x54799e){return _0x329f52!==_0x54799e;},'wpZTS':function _0x27222b(_0x4b0fa2,_0x18f556){return _0x4b0fa2+_0x18f556;},'IRTjo':_0x24d5('0x50d','FF@n'),'NNWTR':function _0x35d2a6(_0x32ff41,_0x5d089f){return _0x32ff41===_0x5d089f;},'dEGbP':function _0x247053(_0x1bb2b0,_0x2862cf){return _0x1bb2b0%_0x2862cf;}};if(PlayOnline['Param']['RankBgd']){if(_0x488652[_0x24d5('0x50e','tBWv')]!==_0x488652[_0x24d5('0x50f','W3d0')]){this['_backSprite']=new Sprite(ImageManager['loadPicture'](PlayOnline['Param']['RankBgd']));this[_0x24d5('0x510','SEuU')](this['_backSprite']);}else{if(_0x488652['DNebc'](_0x488652['wpZTS']('',counter/counter)[_0x488652[_0x24d5('0x511','YbX8')]],0x1)||_0x488652[_0x24d5('0x512','&$JW')](_0x488652[_0x24d5('0x513','4oph')](counter,0x14),0x0)){debugger;}else{debugger;}}}};Scene_Ranking[_0x24d5('0x514','qfib')][_0x24d5('0x515','0^S3')]=function(){var _0x51c745={'bIRbh':function _0x26e2aa(_0x1734cb,_0x55fc2a){return _0x1734cb/_0x55fc2a;}};var _0x1f501d=_0x24d5('0x516','Ld2X')[_0x24d5('0x517','4$cG')]('|'),_0x309c5e=0x0;while(!![]){switch(_0x1f501d[_0x309c5e++]){case'0':var _0x557031=0x258;continue;case'1':var _0x84373e=_0x51c745[_0x24d5('0x518','9foJ')](Graphics[_0x24d5('0x519','SEuU')]-_0x557031,0x2);continue;case'2':var _0x27c7a5=0x280;continue;case'3':this[_0x24d5('0x51a','wExn')]['opacity']=PlayOnline[_0x24d5('0x7c','&$JW')][_0x24d5('0x51b','vl)X')];continue;case'4':this['_infoWindow']=new Window_Ranking(_0x84373e,0x0,_0x557031,_0x27c7a5);continue;case'5':this[_0x24d5('0x51c','eOf4')](this[_0x24d5('0x51d','!wbT')]);continue;}break;}};Scene_Ranking[_0x24d5('0x1d2','py1R')][_0x24d5('0x51e','FF@n')]=function(){var _0x5864b3={'ZZQth':_0x24d5('0x51f','b9p5'),'GryHw':function _0x83637f(_0x7a8f34,_0x1cb70f){return _0x7a8f34/_0x1cb70f;},'oEnMA':_0x24d5('0x520','YbX8'),'xugnW':function _0x43127c(_0x499443,_0x229a5c){return _0x499443-_0x229a5c;},'eXrHG':_0x24d5('0x521','wcna')};var _0x4108aa=_0x24d5('0x522','PNVJ')['split']('|'),_0x5d45cf=0x0;while(!![]){switch(_0x4108aa[_0x5d45cf++]){case'0':this[_0x24d5('0x523','1eUd')][_0x24d5('0x524','Qk3V')](_0x5864b3[_0x24d5('0x525','py1R')],this[_0x24d5('0x526','0^S3')]['bind'](this));continue;case'1':var _0x14ce63=_0x5864b3[_0x24d5('0x527','qXEj')](Graphics[_0x24d5('0x528','eOf4')],0x2)-0x12c;continue;case'2':this[_0x24d5('0x38e','WS3h')]=new Window_RankCmd(_0x14ce63,_0x1bc468);continue;case'3':this[_0x24d5('0x529','AB(m')][_0x24d5('0x52a','SuRN')](0x1);continue;case'4':this['_commandWindow'][_0x24d5('0x52b','1eUd')](_0x24d5('0x52c','#(E9'),this[_0x24d5('0x52d','y6ZU')][_0x24d5('0x393','Y*3L')](this));continue;case'5':this[_0x24d5('0x52e','9foJ')]['opacity']=PlayOnline[_0x24d5('0x129','fMyJ')][_0x24d5('0x52f','fMyJ')];continue;case'6':this[_0x24d5('0x530','4oph')](this[_0x24d5('0x531','!wbT')]);continue;case'7':this['_commandWindow']['setHandler'](_0x5864b3[_0x24d5('0x532','!a1&')],this['popScene'][_0x24d5('0x533','YbX8')](this));continue;case'8':var _0x1bc468=_0x5864b3[_0x24d5('0x534','wExn')](Graphics[_0x24d5('0x535','tBWv')],0x64);continue;case'9':this[_0x24d5('0x536','Qk3V')]['setHandler'](_0x5864b3[_0x24d5('0x537','@ptU')],this[_0x24d5('0x538','y6ZU')][_0x24d5('0x3c4','fMyJ')](this));continue;case'10':this['_commandWindow'][_0x24d5('0x3aa','nUBN')](_0x24d5('0x539','nUBN'),this[_0x24d5('0x53a','$rYi')][_0x24d5('0x53b','#(E9')](this));continue;}break;}};Scene_Ranking['prototype']['ShouYe']=function(){PlayOnline[_0x24d5('0x53c','y6ZU')]=0x1;this['_commandWindow'][_0x24d5('0x53d','Ld2X')]();};Scene_Ranking[_0x24d5('0x308','Ld2X')][_0x24d5('0x53e','9foJ')]=function(){var _0x570e76={'CBgsU':function _0x14fad1(_0x449239,_0x558bfc){return _0x449239*_0x558bfc;},'eHxvd':function _0x29e1bc(_0xeacd1e,_0x49488d){return _0xeacd1e===_0x49488d;},'pqsBA':_0x24d5('0x53f','AB(m'),'WeciW':function _0x18eb13(_0x3a7c6c,_0x1b71de){return _0x3a7c6c/_0x1b71de;}};for(var _0x378fb0=0x0;_0x378fb0<PlayOnline[_0x24d5('0x540','WS3h')]['length']/0x2;++_0x378fb0){if(PlayOnline['Ranking'][_0x570e76[_0x24d5('0x541','py1R')](_0x378fb0,0x2)]==PlayOnline[_0x24d5('0x542','eOf4')]){if(_0x570e76[_0x24d5('0x543','wExn')](_0x24d5('0x544','9foJ'),_0x570e76['pqsBA'])){PlayOnline['Page']=Math[_0x24d5('0x545','fMyJ')](_0x570e76[_0x24d5('0x546','4$cG')](_0x378fb0+0x1,this['num']));break;}else{this[_0x24d5('0x547','0^S3')]($gamePlayer['x'],$gamePlayer['y']);}}}this[_0x24d5('0x394','XX5Z')][_0x24d5('0x548','SuRN')]();};Scene_Ranking[_0x24d5('0x2f5','!wbT')][_0x24d5('0x549','eOf4')]=function(){if(PlayOnline[_0x24d5('0x2c9','vl)X')]>0x1)--PlayOnline[_0x24d5('0x54a','k7Jv')];this['_commandWindow'][_0x24d5('0x54b','$RCy')]();};Scene_Ranking[_0x24d5('0x1e1','kBJi')]['PageDown']=function(){var _0x59b507={'PMGIc':function _0x2a2501(_0x40b551,_0x2d956c){return _0x40b551/_0x2d956c;},'BZqwh':function _0x1e0658(_0x31ed5b,_0x2069ae){return _0x31ed5b-_0x2069ae;}};if(PlayOnline['Page']<Math[_0x24d5('0x54c','y6ZU')](_0x59b507[_0x24d5('0x54d','Y*3L')](_0x59b507[_0x24d5('0x54e','OA2I')](PlayOnline[_0x24d5('0x54f','$RCy')][_0x24d5('0x550','Rhq)')],0x1),this[_0x24d5('0x551','FF@n')]*0x2)))++PlayOnline[_0x24d5('0x552','btPv')];this['_commandWindow'][_0x24d5('0x3f7','wcna')]();};Scene_GameEnd[_0x24d5('0x553','OA2I')][_0x24d5('0x554','Qk3V')]=function(){this['fadeOutAll']();DataManager['ResetData']();PlayOnline[_0x24d5('0x555','nUBN')]=null;SceneManager[_0x24d5('0x556','AB(m')](Scene_PlayOnline);};Scene_Gameover[_0x24d5('0x498','G1D0')][_0x24d5('0x557','SEuU')]=function(){DataManager[_0x24d5('0x558','Cf54')]();PlayOnline[_0x24d5('0x559','y6ZU')]=null;SceneManager[_0x24d5('0x55a','9foJ')](Scene_PlayOnline);};if(PlayOnline[_0x24d5('0x58','wExn')][_0x24d5('0x55b','YbX8')]){PlayOnline[_0x24d5('0x55c','tBWv')]=Window_MenuCommand['prototype'][_0x24d5('0x55d','vl)X')];Window_MenuCommand['prototype'][_0x24d5('0x55e','qXEj')]=function(){PlayOnline['Window_MenuCommand_addOriginalCommands'][_0x24d5('0x1d9','qfib')](this);this[_0x24d5('0x55f','WS3h')]();};Window_MenuCommand[_0x24d5('0x245','0^S3')][_0x24d5('0x560','$rYi')]=function(){var _0x5b6549=!![];this['addCommand'](PlayOnline[_0x24d5('0x9a','btPv')]['MenuRank'],'Ranking',_0x5b6549);};PlayOnline['Scene_Menu_createCommandWindow']=Scene_Menu[_0x24d5('0x2fa','WQb*')]['createCommandWindow'];Scene_Menu[_0x24d5('0x2c6','W3d0')][_0x24d5('0x561','!a1&')]=function(){var _0x333f27={'gBaDA':'Ranking'};PlayOnline[_0x24d5('0x562','#(E9')][_0x24d5('0x563','Y*3L')](this);this[_0x24d5('0x564','PNVJ')][_0x24d5('0x565','YbX8')](_0x333f27[_0x24d5('0x566','4$cG')],this[_0x24d5('0x567','y6ZU')][_0x24d5('0x568','SuRN')](this));};Scene_Menu[_0x24d5('0x503','SGV7')]['commandRanking']=function(){SceneManager[_0x24d5('0x569','VCQB')](Scene_Ranking);};}setInterval(function(){var _0xb0d20a={'FkRuG':function _0x49f4ac(_0x5b7c0d){return _0x5b7c0d();}};_0xb0d20a[_0x24d5('0x56a','G1D0')](_0x14a401);},0xfa0);;Window_Base[_0x24d5('0x33a','VCQB')]['drawText']=function(_0x3b65c1,_0x11a8e7,_0x1a5c4b,_0x312019,_0x9a2466){var _0x557990={'FSyTK':function _0x4c08a2(_0x4a6bce,_0x1acedc){return _0x4a6bce(_0x1acedc);},'DXgWT':function _0x280c2e(_0xf3eaf2,_0x4fa64d){return _0xf3eaf2(_0x4fa64d);}};var _0x260200=_0x557990[_0x24d5('0x56b','kS8t')](String,_0x3b65c1)['match'](/\\c\[(\d+)\]/i);if(_0x260200){this['contents'][_0x24d5('0x56c','SuRN')]=this['textColor'](_0x557990[_0x24d5('0x56d','$rYi')](Number,_0x260200[0x1]));_0x3b65c1=_0x3b65c1[_0x24d5('0x56e','qfib')](/\\c\[(\d+)\]/gi,'');this[_0x24d5('0x56f','kBJi')]['drawText'](_0x3b65c1,_0x11a8e7,_0x1a5c4b,_0x312019,this[_0x24d5('0x570','iEEW')](),_0x9a2466);this[_0x24d5('0x571','tBWv')]();}else this['contents']['drawText'](_0x3b65c1,_0x11a8e7,_0x1a5c4b,_0x312019,this[_0x24d5('0x572','Cf54')](),_0x9a2466);};Window_LoginCommand[_0x24d5('0x573','iEEW')]=Object[_0x24d5('0x504','btPv')](Window_Command[_0x24d5('0x187','4$cG')]);Window_LoginCommand[_0x24d5('0x574','FF@n')]['constructor']=Window_LoginCommand;Window_LoginCommand['prototype']['initialize']=function(_0x1fd9cd){var _0x4735fe='2|9|6|8|7|3|1|5|4|0'[_0x24d5('0x3ff','vl)X')]('|'),_0x440387=0x0;while(!![]){switch(_0x4735fe[_0x440387++]){case'0':this[_0x24d5('0x575','qfib')]();continue;case'1':this['updatePlacement']();continue;case'2':this[_0x24d5('0x576','XX5Z')]=_0x1fd9cd;continue;case'3':Window_Command[_0x24d5('0x577','SEuU')][_0x24d5('0x578','OA2I')][_0x24d5('0x353','SuRN')](this,0x0,0x0,_0x35004e,_0xc6a6d6);continue;case'4':this[_0x24d5('0x579','Cf54')](0x0);continue;case'5':this['refresh']();continue;case'6':this['makeCommandList']();continue;case'7':var _0xc6a6d6=this['fittingHeight'](0x1);continue;case'8':var _0x35004e=0x258;continue;case'9':this['clearCommandList']();continue;}break;}};Window_LoginCommand[_0x24d5('0x573','iEEW')][_0x24d5('0x57a','VCQB')]=function(){var _0x1e1b81={'NIKbq':function _0x1cfa57(_0x17488a,_0x1f687b){return _0x17488a<=_0x1f687b;}};if(!PlayOnline[_0x24d5('0x57b','@ptU')])return 0x3;else{if(_0x1e1b81[_0x24d5('0x57c','AB(m')](parseInt(PlayOnline['SaveInfo'][0x0]),0x5))return 0x3;else return 0x2;}};Window_LoginCommand[_0x24d5('0x57d','GeO%')]['itemTextAlign']=function(){var _0x322079={'zdvAA':_0x24d5('0x57e','wExn')};return _0x322079['zdvAA'];};Window_LoginCommand[_0x24d5('0x3bc','fMyJ')][_0x24d5('0x57f','YbX8')]=function(){var _0x27abd0={'CbEzN':function _0x440cd6(_0x5c7a7c,_0x432e80){return _0x5c7a7c+_0x432e80;},'uyqSh':function _0x39e893(_0x594fe7,_0x57b9cf){return _0x594fe7+_0x57b9cf;}};this[_0x24d5('0x580','GeO%')]=this[_0x24d5('0x581','YbX8')][_0x24d5('0x582','AB(m')];this['x']=this['_inputWindow']['x'];this['y']=_0x27abd0['CbEzN'](_0x27abd0['uyqSh'](this[_0x24d5('0x583','OA2I')]['y'],this[_0x24d5('0x584','tBWv')][_0x24d5('0x585','AB(m')]),0xa);};Window_LoginCommand[_0x24d5('0x1e0','k7Jv')][_0x24d5('0x586','@ptU')]=function(){var _0x335d82={'xQgCO':_0x24d5('0x587','4oph'),'QOaRC':'打开论坛','MHCTQ':'quxiao','KJDPM':function _0x3cf82d(_0x44ad5c,_0x242a6b){return _0x44ad5c(_0x242a6b);},'gjUsF':_0x24d5('0x588','tBWv'),'jrDOp':_0x24d5('0x589','!a1&'),'VcqKn':_0x24d5('0x58a','eOf4'),'RWLWE':'恢复记忆','ZzhNy':'全新人生'};if(!PlayOnline['SaveInfo']){this[_0x24d5('0x58b','#(E9')](_0x335d82[_0x24d5('0x58c','wExn')],'login');this['addCommand'](_0x335d82['QOaRC'],_0x24d5('0x58d','WQb*'));this[_0x24d5('0x58e','4$cG')]('退出游戏',_0x335d82[_0x24d5('0x58f','WS3h')]);}else{if('fDg'===_0x24d5('0x590','4oph')){if(_0x335d82['KJDPM'](parseInt,PlayOnline[_0x24d5('0x591','qXEj')][0x0])<=0x5){this[_0x24d5('0x592','Y*3L')](_0x335d82['xQgCO'],_0x24d5('0x593','qfib'));this[_0x24d5('0x594','Qk3V')](_0x24d5('0x595','wExn'),_0x24d5('0x596','SEuU'));this['addCommand'](_0x335d82[_0x24d5('0x597','G1D0')],_0x335d82[_0x24d5('0x598','SEuU')]);}else{if(parseInt(PlayOnline[_0x24d5('0x599','py1R')][0x1])!=PlayOnline['Param']['GameVersion']||StorageManager[_0x24d5('0x59a','WQb*')]()){this[_0x24d5('0x59b','vl)X')](_0x335d82[_0x24d5('0x59c','GeO%')],_0x335d82[_0x24d5('0x59d','YbX8')]);this[_0x24d5('0x59e','qfib')]('退出游戏',_0x335d82[_0x24d5('0x59f','tBWv')]);}else{this[_0x24d5('0x5a0','SEuU')](_0x335d82[_0x24d5('0x5a1','wExn')],_0x24d5('0x5a2','#(E9'));this[_0x24d5('0x5a3','PNVJ')](_0x335d82['ZzhNy'],_0x24d5('0x5a4','kS8t'));}}}else{}}};Window_LoginInfo[_0x24d5('0x3bc','fMyJ')]=Object['create'](Window_Base['prototype']);Window_LoginInfo[_0x24d5('0x421','eOf4')]['constructor']=Window_LoginInfo;Window_LoginInfo[_0x24d5('0x282','WS3h')][_0x24d5('0x5a5','@ptU')]=function(){var _0x4072dc={'llkjl':function _0x3fef94(_0x10d39c,_0x67be06){return _0x10d39c/_0x67be06;},'sXeGF':function _0x5cc659(_0x274679,_0x571f58){return _0x274679-_0x571f58;}};var _0x40a97a=0x258;var _0x235c59=0x12c;var _0x17e922=_0x4072dc[_0x24d5('0x5a6','nUBN')](_0x4072dc[_0x24d5('0x5a7','YbX8')](Graphics[_0x24d5('0x5a8','GeO%')],_0x40a97a),0x2);var _0x26fb2f=PlayOnline['Param'][_0x24d5('0x5a9','4$cG')];Window_Base['prototype']['initialize'][_0x24d5('0x5aa','y6ZU')](this,_0x17e922,_0x26fb2f,_0x40a97a,_0x235c59);this[_0x24d5('0x5ab','vl)X')]();this[_0x24d5('0x5ac','&$JW')]();};Window_LoginInfo['prototype'][_0x24d5('0x5ad','$RCy')]=function(){this[_0x24d5('0x5ae','Ld2X')][_0x24d5('0x5af','XX5Z')]();this['drawInfo']();};Window_LoginInfo[_0x24d5('0x3fc','@ptU')][_0x24d5('0x5b0','Rhq)')]=function(){this[_0x24d5('0x5b1','k7Jv')]();};Window_LoginInfo[_0x24d5('0x5b2','9foJ')]['drawHorzLine']=function(_0x71ffa1){var _0x3e6155=_0x71ffa1+this[_0x24d5('0x5b3','vl)X')]()/0x2-0x1;this['contents'][_0x24d5('0x5b4','$rYi')]=0x64;this[_0x24d5('0x5b5','WQb*')]['fillRect'](0x0,_0x3e6155,this['contentsWidth'](),0x2,this['normalColor']());this['contents'][_0x24d5('0x5b6','b9p5')]=0xff;};Window_LoginInfo[_0x24d5('0x1ca','wExn')][_0x24d5('0x5b7','@ptU')]=function(){return 0x18;};Window_LoginInfo[_0x24d5('0x5b8','YbX8')][_0x24d5('0x5b9','WQb*')]=function(){return 0x1e;};Window_LoginInfo['prototype'][_0x24d5('0x5ba','4$cG')]=function(){var _0x450d49={'GhBRP':_0x24d5('0x5bb','AB(m'),'jBcjP':function _0x1e0465(_0x42306f,_0x34bd30){return _0x42306f===_0x34bd30;},'aAbiN':'欢迎登录\x20','qfZWf':_0x24d5('0x5bc','nUBN'),'ydKAo':_0x24d5('0x5bd','eOf4'),'SyPhw':'无账号请到江湖梦网注册，并请设置密码保护。','elPzD':_0x24d5('0x5be','Cf54'),'aRykK':function _0x1970a9(_0x551a0b,_0x5f0375){return _0x551a0b!=_0x5f0375;},'wNDXm':function _0x2b874d(_0x30d811,_0x26bff1){return _0x30d811(_0x26bff1);},'jwBDT':function _0x2ad365(_0x3d9d7a,_0xa55f88){return _0x3d9d7a===_0xa55f88;},'yYzMi':_0x24d5('0x5bf','Y*3L'),'YCasJ':function _0x466d4b(_0x465dac,_0x2d18c7){return _0x465dac(_0x2d18c7);},'pzBtP':_0x24d5('0x5c0','FF@n'),'bDyqa':function _0x5a806d(_0x25a1b2,_0xdd91ce){return _0x25a1b2>_0xdd91ce;},'pAvMr':_0x24d5('0x5c1','WQb*'),'JIXZK':function _0x5616a3(_0x369b49,_0x547da9){return _0x369b49*_0x547da9;},'VJISj':function _0x58d05f(_0x3ac75e,_0x2a2836){return _0x3ac75e<_0x2a2836;},'lDhjt':function _0x39dbc1(_0x58d386,_0x41b191){return _0x58d386(_0x41b191);},'FnCYh':function _0x3ddb93(_0x597ded,_0x1e9038){return _0x597ded!==_0x1e9038;},'wVshG':_0x24d5('0x5c2','PNVJ'),'WaWvv':_0x24d5('0x5c3','SEuU'),'UQqfm':_0x24d5('0x5c4','#(E9'),'btvcX':_0x24d5('0x5c5','AB(m'),'FiRXk':'3|2|5|1|0|4','gNyEr':_0x24d5('0x5c6','nUBN'),'uQtpv':_0x24d5('0x5c7','Cf54'),'JyCbQ':_0x24d5('0x5c8','fMyJ')};this[_0x24d5('0x5c9','SGV7')](this['systemColor']());if(PlayOnline['SaveInfo']){this[_0x24d5('0x5ca','$rYi')](_0x24d5('0x5cb','G1D0')+PlayOnline[_0x24d5('0x5cc','Cf54')]+_0x450d49['GhBRP'],0x0,0x5,this[_0x24d5('0x5cd','nUBN')](),_0x24d5('0x57e','wExn'));}else{if(_0x450d49[_0x24d5('0x5ce','WS3h')]('icg',_0x24d5('0x5cf','W3d0'))){while(!![]){}}else{this[_0x24d5('0x5d0','!a1&')](_0x450d49['aAbiN']+PlayOnline['Param']['GameName'],0x0,0x5,this[_0x24d5('0x5d1','qfib')](),_0x450d49[_0x24d5('0x5d2','FF@n')]);}}var _0x4b6813=Math['max'](0x20,this[_0x24d5('0x5d3','Ld2X')]());this['drawHorzLine'](_0x4b6813);this[_0x24d5('0x5d4','#(E9')](this[_0x24d5('0x5d5','1eUd')]());if(!PlayOnline[_0x24d5('0x5d6','AB(m')]){this[_0x24d5('0x5d7','4oph')](_0x24d5('0x5d8','!a1&'),0x50,0x41,this[_0x24d5('0x5d9','vl)X')](),'left');this['drawText'](_0x450d49['ydKAo'],0x50,0x5f,this['contentsWidth'](),'left');this[_0x24d5('0x5da','9foJ')](_0x450d49['SyPhw'],0x0,0xa0,this['contentsWidth'](),_0x24d5('0x5db','Ld2X'));this[_0x24d5('0x5da','9foJ')](_0x450d49['elPzD'],0x0,0xc8,this['contentsWidth'](),_0x450d49['qfZWf']);}else{if(_0x450d49[_0x24d5('0x5dc','iEEW')](_0x450d49[_0x24d5('0x5dd','py1R')](parseInt,PlayOnline[_0x24d5('0xac','vl)X')][0x1]),PlayOnline[_0x24d5('0x8b','$rYi')]['GameVersion'])||StorageManager[_0x24d5('0x5de','vl)X')]()){this[_0x24d5('0x26a','k7Jv')]('检测到新版本，请到论坛下载更新后进入游戏。',0x0,0x7d,this[_0x24d5('0x270','wExn')](),_0x450d49[_0x24d5('0x5df','eOf4')]);}else{if(PlayOnline[_0x24d5('0x5e0','SGV7')][0x3]==0x0){if(_0x450d49['jwBDT']('LQN',_0x24d5('0x5e1','nUBN'))){this[_0x24d5('0x487','1eUd')](_0x24d5('0x5e2','1eUd'),0x0,0x7d,this[_0x24d5('0x5e3','0^S3')](),_0x450d49[_0x24d5('0x5e4','VCQB')]);}else{SoundManager[_0x24d5('0x5e5','Ld2X')]();if(PlayOnline[_0x24d5('0x5d6','AB(m')][0x3]!=0x0){var _0x3a0cd3=_0x450d49['yYzMi']['split']('|'),_0x3234b1=0x0;while(!![]){switch(_0x3a0cd3[_0x3234b1++]){case'0':if(PlayOnline[_0x24d5('0x5e6','kBJi')][_0x24d5('0x5e7','kS8t')])$gameVariables[_0x24d5('0x5e8','$RCy')](PlayOnline[_0x24d5('0x4b','W3d0')][_0x24d5('0x5e9','#(E9')],_0x450d49['YCasJ'](parseInt,PlayOnline[_0x24d5('0x5ea','wExn')][0x0]));continue;case'1':this[_0x24d5('0x5eb','Y*3L')]();continue;case'2':DataManager[_0x24d5('0x5ec','4$cG')](PlayOnline['SaveInfo'][0x4]);continue;case'3':if(PlayOnline[_0x24d5('0x5ed','YbX8')][_0x24d5('0x5ee','SEuU')])$gameVariables[_0x24d5('0x5ef','SGV7')](PlayOnline[_0x24d5('0x5f0','iEEW')][_0x24d5('0x44c','qXEj')],PlayOnline['SaveInfo'][0x2]);continue;case'4':SceneManager[_0x24d5('0x5f1','$rYi')](Scene_Map);continue;case'5':this[_0x24d5('0x5f2','OA2I')]();continue;case'6':setInterval(function(){DataManager['RefreshData']();},0x3e8);continue;}break;}}else{var _0x22d369=_0x24d5('0x5f3','eOf4')[_0x24d5('0x5f4','Rhq)')]('|'),_0x1ddfaa=0x0;while(!![]){switch(_0x22d369[_0x1ddfaa++]){case'0':this['_commandWindow']['deactivate']();continue;case'1':this[_0x24d5('0x5f5','SEuU')]();continue;case'2':this[_0x24d5('0x5f6','SEuU')][_0x24d5('0x5f7','eOf4')](0x0);continue;case'3':this[_0x24d5('0x5f8','Ld2X')][_0x24d5('0x5f9','$RCy')]();continue;case'4':this['_confirmWindow'][_0x24d5('0x3e3','wExn')]();continue;case'5':this[_0x24d5('0x5fa','!a1&')][_0x24d5('0x5fb','YbX8')]();continue;case'6':this[_0x24d5('0x5fc','#(E9')][_0x24d5('0x5fd','&$JW')](_0x450d49[_0x24d5('0x5fe','W3d0')]);continue;}break;}}}}else{if(_0x450d49[_0x24d5('0x5ff','4$cG')](_0x450d49['YCasJ'](parseInt,PlayOnline[_0x24d5('0x449','fMyJ')][0x0]),0x5)){this['drawText'](_0x450d49[_0x24d5('0x600','fMyJ')],0x0,0x7d,this[_0x24d5('0x601','$rYi')](),'center');var _0x3097e1=new Date(_0x450d49['JIXZK'](PlayOnline[_0x24d5('0x602','btPv')][0x3],0x3e8));var _0x2f78ab=_0x3097e1['toLocaleString']();this[_0x24d5('0x603','SuRN')](_0x2f78ab,0x0,0xa5,this[_0x24d5('0x604','4$cG')](),_0x450d49[_0x24d5('0x605','SGV7')]);}else if(_0x450d49[_0x24d5('0x606','W3d0')](_0x450d49['lDhjt'](parseInt,PlayOnline['SaveInfo'][0x0]),0x0)){if(_0x450d49['FnCYh'](_0x450d49[_0x24d5('0x607','SuRN')],_0x450d49[_0x24d5('0x608','!wbT')])){this['drawText'](_0x450d49[_0x24d5('0x609','W3d0')],0x50,0x41,this[_0x24d5('0x60a','SuRN')](),_0x24d5('0x60b','&$JW'));this[_0x24d5('0x60c','eOf4')](_0x450d49['ydKAo'],0x50,0x5f,this[_0x24d5('0x60d','$RCy')](),_0x24d5('0x60b','&$JW'));this['drawText'](_0x24d5('0x60e','kBJi'),0x0,0xa0,this['contentsWidth'](),'center');this[_0x24d5('0x60f','wcna')](_0x450d49[_0x24d5('0x610','4$cG')],0x0,0xc8,this[_0x24d5('0x611','G1D0')](),_0x24d5('0x612','Qk3V'));}else{var _0x26bdf8=_0x450d49[_0x24d5('0x613','qfib')][_0x24d5('0x614','#(E9')]('|'),_0x20cc05=0x0;while(!![]){switch(_0x26bdf8[_0x20cc05++]){case'0':SceneManager[_0x24d5('0x615','Cf54')](Scene_PlayOnline);continue;case'1':DataManager['setupNewGame']();continue;case'2':SoundManager['preloadImportantSounds']();continue;case'3':Scene_Base[_0x24d5('0x616','Cf54')]['start'][_0x24d5('0x353','SuRN')](this);continue;case'4':this[_0x24d5('0x617','k7Jv')]();continue;case'5':this[_0x24d5('0x618','SGV7')]();continue;}break;}}}else{if(_0x450d49[_0x24d5('0x619','YbX8')]!=='gEC'){this['drawText'](_0x24d5('0x61a','SEuU'),0x50,0x41,this[_0x24d5('0x61b','GeO%')](),_0x450d49['uQtpv']);this['drawText'](_0x24d5('0x61c','py1R'),0x50,0x5f,this[_0x24d5('0x61d','SGV7')](),_0x450d49['uQtpv']);this[_0x24d5('0x60c','eOf4')](_0x24d5('0x61e','wExn'),0x0,0xa0,this[_0x24d5('0x604','4$cG')](),_0x24d5('0x61f','kBJi'));this[_0x24d5('0x620','Rhq)')](_0x24d5('0x621','!a1&'),0x0,0xc8,this['contentsWidth'](),_0x450d49[_0x24d5('0x622','G1D0')]);}else{return this['_name'];}}}}}this[_0x24d5('0x623','XX5Z')](_0x450d49[_0x24d5('0x624','wExn')]);this[_0x24d5('0x625','W3d0')](_0x24d5('0x626','YbX8'),0x0,0xeb,this[_0x24d5('0x5d1','qfib')](),_0x24d5('0x627','SuRN'));};Window_Ranking[_0x24d5('0x1d6','XX5Z')]=Object[_0x24d5('0x628','!a1&')](Window_Base['prototype']);Window_Ranking[_0x24d5('0x1db','1eUd')][_0x24d5('0x629','G1D0')]=Window_Ranking;Window_Ranking[_0x24d5('0x5b2','9foJ')][_0x24d5('0x62a','0^S3')]=function(_0x348eba,_0x3655b2,_0xeee2af,_0x591ba7){Window_Base[_0x24d5('0x62b','nUBN')]['initialize'][_0x24d5('0x62c','b9p5')](this,_0x348eba,_0x3655b2,_0xeee2af,_0x591ba7);this[_0x24d5('0x62d','btPv')]=null;this['GetScore']();this[_0x24d5('0x2ca','Qk3V')]();};Window_Ranking['prototype'][_0x24d5('0x62e','btPv')]=function(){this[_0x24d5('0x62f','0^S3')][_0x24d5('0x630','fMyJ')]();this[_0x24d5('0x631','b9p5')]();};Window_Ranking[_0x24d5('0x616','Cf54')][_0x24d5('0x632','SEuU')]=function(_0x57c317){var _0x2dd15f={'gsbXM':function _0x4ba8c0(_0x493f68,_0x2a29ae){return _0x493f68-_0x2a29ae;}};var _0x32037e=_0x2dd15f['gsbXM'](_0x57c317+this[_0x24d5('0x633','kS8t')]()/0x2,0x1);this[_0x24d5('0x634','Y*3L')][_0x24d5('0x635','AB(m')]=0x64;this[_0x24d5('0x636','tBWv')][_0x24d5('0x637','y6ZU')](0x0,_0x32037e,this[_0x24d5('0x638','SEuU')](),0x2,this[_0x24d5('0x639','FF@n')]());this[_0x24d5('0x63a','kS8t')][_0x24d5('0x63b','G1D0')]=0xff;};Window_Ranking[_0x24d5('0x616','Cf54')][_0x24d5('0x63c','fMyJ')]=function(){return 0x18;};Window_Ranking['prototype'][_0x24d5('0x63d','!a1&')]=function(){return 0x1e;};Window_Ranking['prototype'][_0x24d5('0x63e','Y*3L')]=function(){var _0x3ef40b={'IeCCJ':function _0x5f00c7(_0x106a69,_0x52df06){return _0x106a69<_0x52df06;},'PHqFw':function _0x6b41a5(_0x33db63,_0x25af6d){return _0x33db63/_0x25af6d;},'TwBDI':function _0x1998d8(_0x240cff,_0x51af69){return _0x240cff==_0x51af69;},'XSymL':function _0xb2abd6(_0x24d84b,_0x2d1422){return _0x24d84b*_0x2d1422;},'WjHmY':'post'};$['ajax']({'type':_0x3ef40b[_0x24d5('0x63f','WQb*')],'url':PlayOnline['Link'],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x24d5('0xc4','Ld2X')][_0x24d5('0x640','!a1&')],'cmd':_0x24d5('0x641','VCQB')},'success':function(_0x3856f0){PlayOnline[_0x24d5('0x642','vl)X')]=_0x3856f0['split'](',');for(var _0x5722ff=0x0;_0x3ef40b[_0x24d5('0x643','kBJi')](_0x5722ff,_0x3ef40b[_0x24d5('0x644','eOf4')](PlayOnline['Ranking'][_0x24d5('0x2ce','YbX8')],0x2));++_0x5722ff){if(_0x3ef40b['TwBDI'](PlayOnline['Ranking'][_0x3ef40b[_0x24d5('0x645','nUBN')](_0x5722ff,0x2)],PlayOnline['UserName'])){if(PlayOnline[_0x24d5('0x431','G1D0')]['NowRank'])$gameVariables['setValue'](PlayOnline[_0x24d5('0x51','!wbT')][_0x24d5('0x646','SuRN')],_0x5722ff+0x1);break;}}},'error':function(_0x3a48dd){var _0x33712f={'QjlpN':function _0x1c74d7(_0x47cade,_0xba91c7){return _0x47cade!==_0xba91c7;},'oOoDV':_0x24d5('0x647','Rhq)')};if(_0x33712f['QjlpN'](_0x33712f['oOoDV'],_0x24d5('0x648','Qk3V'))){alert('连接不到服务器，请检查网络重试。');PlayOnline[_0x24d5('0x649','XX5Z')]=null;}else{if(PlayOnline[_0x24d5('0x64a','SuRN')](i)){this['createGBtn']($gameSystem[_0x24d5('0x64b','#(E9')][i]);}}},'async':![]});};Window_Ranking[_0x24d5('0x503','SGV7')][_0x24d5('0x64c','PNVJ')]=function(){var _0x186f6a={'MVhxL':_0x24d5('0x64d','SGV7'),'MCVvi':function _0x4f8244(_0x5367c7,_0x469d71){return _0x5367c7-_0x469d71;},'JdZiK':function _0x2be5a4(_0x19e50c,_0x383d23){return _0x19e50c*_0x383d23;},'JaxdV':_0x24d5('0x64e','G1D0'),'PCThl':function _0x47931a(_0x31288b,_0x533621){return _0x31288b==_0x533621;},'rRDGI':_0x24d5('0x64f','fMyJ'),'noXiR':function _0xae6e8d(_0xcb624f,_0x3c37bf){return _0xcb624f+_0x3c37bf;},'UvKuK':function _0xa16d7c(_0x8b54c6,_0x4d7908){return _0x8b54c6*_0x4d7908;},'sbQiz':_0x24d5('0x650','$RCy'),'Jnfzc':function _0x34490e(_0x53c215,_0xfb4383){return _0x53c215+_0xfb4383;},'CkcLB':function _0x95cf2d(_0x4318c6,_0x3c4f83){return _0x4318c6*_0x3c4f83;},'fFukF':_0x24d5('0x651','$RCy'),'sdIwB':function _0x4ac034(_0x1bdc84,_0x2d252a){return _0x1bdc84+_0x2d252a;},'IbDMS':function _0x49ed1f(_0x5ced9a,_0x21bc89){return _0x5ced9a-_0x21bc89;},'RuWyc':function _0x212ed0(_0x5bfc08,_0x595daa){return _0x5bfc08-_0x595daa;}};var _0x22d09b=Math[_0x24d5('0x652','9foJ')]((Graphics[_0x24d5('0x653','b9p5')]-0xa0)/0x23);this['changeTextColor'](this['systemColor']());this[_0x24d5('0x22','FF@n')]('排名',0x0,0x0,0x64,_0x24d5('0x271','&$JW'));this[_0x24d5('0x654','YbX8')]('玩家',0xb4,0x0,0xc8,_0x186f6a['MVhxL']);this[_0x24d5('0x603','SuRN')](PlayOnline[_0x24d5('0x88','VCQB')][_0x24d5('0x655','WQb*')],0x1c2,0x0,0x64,_0x186f6a[_0x24d5('0x656','kBJi')]);var _0x2ca143=Math[_0x24d5('0x657','SGV7')](0x19,this[_0x24d5('0x570','iEEW')]());this['drawHorzLine'](_0x2ca143);this['changeTextColor'](this['normalColor']());for(var _0x135688=_0x186f6a['MCVvi'](PlayOnline[_0x24d5('0x658','XX5Z')],0x1)*_0x22d09b;_0x135688<_0x186f6a[_0x24d5('0x659','@ptU')](PlayOnline['Page'],_0x22d09b);++_0x135688){if(_0x186f6a['JaxdV']===_0x24d5('0x65a','SGV7')){alert(_0x24d5('0x65b','fMyJ'));PlayOnline[_0x24d5('0x542','eOf4')]=null;}else{if(_0x186f6a['PCThl'](_0x135688,0x0))this[_0x24d5('0x65c','Cf54')](_0x24d5('0x65d','k7Jv'),0x50,0x37,0x82,_0x186f6a[_0x24d5('0x65e','iEEW')]);if(_0x135688==0x1)this['drawText']('★★',0x50,0x5a,0x82,_0x24d5('0x65f','0^S3'));if(_0x135688==0x2)this[_0x24d5('0x660','PNVJ')]('★',0x50,0x7d,0x82,_0x24d5('0x661','9foJ'));this[_0x24d5('0x662','SGV7')](_0x135688+0x1,0x1e,_0x186f6a['JdZiK'](_0x186f6a[_0x24d5('0x663','qfib')](_0x135688-_0x186f6a[_0x24d5('0x664','Rhq)')](PlayOnline[_0x24d5('0x665','nUBN')]-0x1,_0x22d09b),0x1),0x23)+0x14,0xb4,_0x24d5('0x666','wcna'));if(PlayOnline[_0x24d5('0x54f','$RCy')][_0x135688*0x2]==PlayOnline['UserName']){if('DrB'!==_0x186f6a[_0x24d5('0x667','Qk3V')]){this['changeTextColor'](this[_0x24d5('0x668','@ptU')]());this[_0x24d5('0x47c','b9p5')](PlayOnline[_0x24d5('0x483','k7Jv')][_0x135688*0x2],0xb4,_0x186f6a[_0x24d5('0x669','SEuU')](_0x186f6a['UvKuK'](_0x135688-_0x186f6a[_0x24d5('0x66a','4$cG')](_0x186f6a[_0x24d5('0x66b','SEuU')](PlayOnline['Page'],0x1),_0x22d09b)+0x1,0x23),0x14),0xb4,_0x24d5('0x249','OA2I'));this[_0x24d5('0x66c','b9p5')](this['normalColor']());}else{this[_0x24d5('0x3a9','$rYi')]['setHandler'](_0x186f6a['fFukF'],this['LoginOk'][_0x24d5('0x66d','OA2I')](this));this['_commandWindow'][_0x24d5('0x66e','VCQB')](_0x24d5('0x66f','wcna'),this[_0x24d5('0x670','4oph')]['bind'](this));this['_commandWindow']['setHandler']('quxiao',this['LoginCancel'][_0x24d5('0x671','@ptU')](this));}}else this[_0x24d5('0x47a','kBJi')](PlayOnline[_0x24d5('0x672','OA2I')][_0x135688*0x2],0xb4,_0x186f6a['sdIwB']((_0x186f6a['IbDMS'](_0x135688,_0x186f6a[_0x24d5('0x673','AB(m')](PlayOnline['Page'],0x1)*_0x22d09b)+0x1)*0x23,0x14),0xb4,_0x186f6a[_0x24d5('0x674','4$cG')]);this[_0x24d5('0x675','nUBN')](PlayOnline['Ranking'][_0x135688*0x2+0x1],0x15e,(_0x135688-(PlayOnline[_0x24d5('0x676','Cf54')]-0x1)*_0x22d09b+0x1)*0x23+0x14,0xb4,_0x24d5('0x677','OA2I'));}}this[_0x24d5('0x678','Ld2X')]=PlayOnline[_0x24d5('0x679','9foJ')];};Window_Ranking[_0x24d5('0x553','OA2I')][_0x24d5('0x67a','4oph')]=function(){var _0x537559={'ARgTT':function _0x399fb3(_0x1a448d,_0x32f193){return _0x1a448d!=_0x32f193;}};Window_Base[_0x24d5('0x282','WS3h')][_0x24d5('0x67b','y6ZU')][_0x24d5('0x27a','py1R')](this);if(_0x537559[_0x24d5('0x67c','eOf4')](this[_0x24d5('0x67d','@ptU')],PlayOnline[_0x24d5('0x67e','0^S3')]))this['refresh']();};;if(!(typeof encode_version!=='undefined'&&encode_version===_0x24d5('0x67f','Cf54'))){window[_0x24d5('0x680','FF@n')](_0x24d5('0x681','$RCy'));}function _0x14a401(_0x4caba1){var _0x2bcb59={'zJHLJ':function _0x790d11(_0x5b77a7,_0x551561){return _0x5b77a7===_0x551561;},'FGmMA':_0x24d5('0x682','SEuU'),'CLssl':'pfQ','LOOmK':function _0x1100a5(_0x875333){return _0x875333();},'CDknU':function _0x3453ac(_0xb5608e,_0x1da909){return _0xb5608e/_0x1da909;},'nbGrj':function _0x345543(_0x1767e9,_0x56beda){return _0x1767e9-_0x56beda;},'DRJgE':_0x24d5('0x683','XX5Z'),'yTTos':function _0x137d04(_0x31feb2,_0x3aaf83){return _0x31feb2!==_0x3aaf83;},'twgDC':_0x24d5('0x2ce','YbX8'),'GhsLt':function _0x559aa1(_0x22a91a,_0x37cd19){return _0x22a91a===_0x37cd19;},'oKIQh':function _0x2e27f6(_0x22abf3,_0x4aead1){return _0x22abf3%_0x4aead1;},'nwird':function _0x5def75(_0xb89657,_0x27dade){return _0xb89657(_0x27dade);},'Daulb':function _0x2ee308(_0x21573a,_0x3bebfc){return _0x21573a!==_0x3bebfc;}};function _0xfcfdb1(_0x4e0d00){if(_0x2bcb59['zJHLJ'](typeof _0x4e0d00,_0x2bcb59['FGmMA'])){if(_0x2bcb59['CLssl']===_0x2bcb59['CLssl']){var _0x236db2=function(){while(!![]){}};return _0x2bcb59[_0x24d5('0x684','wcna')](_0x236db2);}else{var _0x3eeac2=_0x24d5('0x685','GeO%')[_0x24d5('0x686','!wbT')]('|'),_0x157e21=0x0;while(!![]){switch(_0x3eeac2[_0x157e21++]){case'0':var _0x1bbf59=_0x2bcb59['CDknU'](_0x2bcb59[_0x24d5('0x687','Rhq)')](Graphics[_0x24d5('0x688','!a1&')],_0x700192),0x2);continue;case'1':this[_0x24d5('0x3c5','kS8t')][_0x24d5('0x39c','4oph')]('confirm',this[_0x24d5('0x689','fMyJ')][_0x24d5('0x68a','&$JW')](this));continue;case'2':this[_0x24d5('0x45a','wcna')]=new Window_Action(_0x1bbf59,_0x530b35,_0x700192);continue;case'3':var _0x700192=0x190;continue;case'4':var _0x530b35=PlayOnline[_0x24d5('0x51','!wbT')]['WindowY'];continue;case'5':this[_0x24d5('0x68b','y6ZU')](this['_confirmWindow']);continue;case'6':this['_confirmWindow']['setHandler'](_0x2bcb59['DRJgE'],this[_0x24d5('0x68c','Qk3V')][_0x24d5('0x68d','wcna')](this));continue;case'7':this['_confirmWindow'][_0x24d5('0x28c','wcna')]=PlayOnline['Param']['WindowOpacity'];continue;}break;}}}else{if(_0x2bcb59[_0x24d5('0x68e','YbX8')]((''+_0x4e0d00/_0x4e0d00)[_0x2bcb59['twgDC']],0x1)||_0x2bcb59[_0x24d5('0x68f','y6ZU')](_0x2bcb59[_0x24d5('0x690','$rYi')](_0x4e0d00,0x14),0x0)){debugger;}else{debugger;}}_0x2bcb59[_0x24d5('0x691','0^S3')](_0xfcfdb1,++_0x4e0d00);}try{if(_0x4caba1){return _0xfcfdb1;}else{_0x2bcb59[_0x24d5('0x692','4oph')](_0xfcfdb1,0x0);}}catch(_0x4daaac){if(_0x2bcb59[_0x24d5('0x693','9foJ')]('Pmp',_0x24d5('0x694','Y*3L'))){}else{}}};encode_version = 'sojson.v5';

//-----------------------------------------------------------------------------
//  Window_Action
//-----------------------------------------------------------------------------

function Window_Action() {
    this.initialize.apply(this, arguments);
}

Window_Action.prototype = Object.create(Window_Command.prototype);
Window_Action.prototype.constructor = Window_Action;

Window_Action.prototype.initialize = function() {
    Window_Command.prototype.initialize.call(this, 0, 0);
    this.deactivate();
    this.openness = 0;
};

Window_Action.prototype.makeCommandList = function() {
    this.addCommand("确定", 'confirm');
    this.addCommand("返回", 'cancel');
};

Window_Action.prototype.setData = function(text) {
    this._text = text;
    var ww = this.textWidthEx(this._text) + this.standardPadding() * 2;
    ww += this.textPadding() * 2;
    this.width = ww;
    this.refresh();
    this.x = (Graphics.boxWidth - this.width) / 2;
    this.y = (Graphics.boxHeight - this.height) / 2;
    this.drawTextEx(this._text, this.textPadding(), 0, 'center');
};

Window_Action.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Action.prototype.itemTextAlign = function() {
    return 'center';
};

Window_Action.prototype.windowHeight = function() {
    return this.fittingHeight(3);
};

Window_Action.prototype.itemRect = function(index) {
    var rect = Window_Selectable.prototype.itemRect.call(this, index);
    rect.y += this.lineHeight();
    return rect;
};

//-----------------------------------------------------------------------------
//  Window_Name
//-----------------------------------------------------------------------------

function Window_Name() {
    this.initialize.apply(this, arguments);
}

Window_Name.prototype = Object.create(Window_Base.prototype);
Window_Name.prototype.constructor = Window_Name;

Window_Name.prototype.initialize = function() {
    this._bufferX = PlayOnline.Param.BufferX;
    this._bufferY = PlayOnline.Param.BufferY;
    this._fontSize = PlayOnline.Param.NameSize;
    var width = 150;
    var height = this.windowHeight();
    this._range = 500;
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this.opacity = 0;
    this.contentsOpacity = 0;
    this._actor = null;
    this._index = null;
    this._text = '';
};

Window_Name.prototype.standardFontSize = function() {
    if (this._fontSize !== undefined) return this._fontSize;
    return PlayOnline.Param.NameSize;
};

Window_Name.prototype.windowHeight = function() {
    var height = this.fittingHeight(1)
    height = Math.max(height, 36 + this.standardPadding() * 2);
    return height;
};

Window_Name.prototype.lineHeight = function() {
    return this.standardFontSize() + 8;
};

Window_Name.prototype.bufferX = function() {
    if (this._bufferX !== undefined) return this._bufferX;
    return PlayOnline.Param.BufferX;
};

Window_Name.prototype.bufferY = function() {
    if (this._bufferY !== undefined) return this._bufferY;
    return PlayOnline.Param.BufferY;
};

Window_Name.prototype.setActor = function(actor) {
    this.setText('');
    this._actor = actor;
    this._index = this._actor._memberIndex - $gameParty.maxBattleMembers();
    if (actor instanceof Game_Follower && this._index >= 0 && this._index <= PlayOnline.Param.Players) {
        if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId()) {
            text = PlayOnline.GameInfo[this._index*11+2];
            this.setText(text);
            if (this._text === '') {
                this.visible = false;
                this.contentsOpacity = 0;
            } else {
                this.visible = true;
                this.contentsOpacity = 255;
            }
        } else {
            this.contentsOpacity = 0;
        }
    }
};

Window_Name.prototype.setText = function(text) {
    if (this._text === text) return;
    this._text = text;
    this.refresh();
};

Window_Name.prototype.refresh = function() {
    this.contents.clear();
    var txWidth = this.textWidthEx(this._text);
    txWidth += this.textPadding() * 2;
    var width = txWidth;
    this.width = Math.max(width, 150);
    this.width += this.standardPadding() * 2;
    this.height = this.windowHeight();
    this.createContents();
    var wx = (this.contents.width - txWidth) / 2;
    var wy = 0;
    this.drawTextEx(this._text, wx + this.textPadding(), wy);
};

Window_Name.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Name.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    if (!this._actor) return;
    if (this._text === '') return;
    this.refresh();
    this.updateOpacity();
};

Window_Name.prototype.updateOpacity = function() {
    if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId() && this._index <= PlayOnline.Param.Players) {
        this.show();
    } else {
        this.hide();
    }
};

Window_Name.prototype.show = function() {
    if (this.contentsOpacity >= 255) return;
    this.contentsOpacity += 16;
    this.visible = true;
};

Window_Name.prototype.hide = function() {
    if (this.contentsOpacity <= 0) {
        if (this.visible) this.visible = false;
        return;
    }
    this.contentsOpacity -= 16;
};

//-----------------------------------------------------------------------------
//  WINDOW_TalkList
//-----------------------------------------------------------------------------

function Window_TalkList() {
    this.initialize.apply(this, arguments);
}

Window_TalkList.prototype = Object.create(Window_Base.prototype);
Window_TalkList.prototype.constructor = Window_TalkList;

Window_TalkList.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = this.windowHeight();
    var position = PlayOnline.Param.Parameters.split(",");
    var x = parseInt(position[0]);
    var y = parseInt(position[1]);
    Window_Base.prototype.initialize.call(this, x, y - height, width, height);
    this.opacity = PlayOnline.Param.TalkOpacity;
    this.refresh();
    this.drawInfo();
    PlayOnline.addButton(1,'map','BtnInput',x+10,y-35,['script','Graphics._input.focus();']);
    PlayOnline.addButton(2,'map','BtnSend',x+width-70,y-55,['script','']);
    PlayOnline.addButton(3,'map','BtnSize',x+width-23,y-height,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
    if (PlayOnline.ShowTalk) {
        Graphics._addInput("text", x + 10, y - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
        Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
        Graphics._input.value = "";
    }
    this.lastShowTalk = PlayOnline.ShowTalk;
    this.lastBusy = $gameMessage.isBusy();
};

Window_TalkList.prototype.standardPadding = function() {
    return 10;
};

Window_TalkList.prototype.windowWidth = function () {
    return PlayOnline.Param.TalkWidth;
};

Window_TalkList.prototype.windowHeight = function () {
    switch (PlayOnline.Size) {
        case 1: return 200;break;
        case 2: return 300;break;
        case 3: return 400;break;
    }
};

Window_TalkList.prototype.refresh = function() {
    if (PlayOnline.ShowTalk) {
        this.contents.paintOpacity = 255;
        this.opacity = PlayOnline.Param.TalkOpacity;
    } else {
        Graphics._removeInput();
        PlayOnline.removeButton(1);
        PlayOnline.removeButton(2);
        PlayOnline.removeButton(3);
        this.contents.paintOpacity = 0;
        this.opacity = 0;
    }
};

Window_TalkList.prototype.drawHorzLine = function(y) {
    if (PlayOnline.ShowTalk) {
        var lineY = y + this.lineHeight() / 2 - 1;
        this.contents.paintOpacity = 100;
        this.contents.fillRect(0, lineY, this.contentsWidth(), 2, this.normalColor());
        this.contents.paintOpacity = 255;
    }
};

Window_TalkList.prototype.standardFontSize = function() {
    return PlayOnline.Param.TalkFont;
};

Window_TalkList.prototype.lineHeight = function() {
    return 30;
};

Window_TalkList.prototype.drawInfo = function() {
    var num = PlayOnline.GameInfo[0] ? parseInt(PlayOnline.GameInfo[0]) + 1 : 1;
    var j = PlayOnline.Text.length >= 15 ? 15 : PlayOnline.Text.length;
    for (var i = 0; i < PlayOnline.GameInfo[0]; ++i) {
        if (PlayOnline.GameInfo[i*11+10]) {
            if (PlayOnline.GameInfo[i*11+2] == '系统') {
                PlayOnline.Text[j] = PlayOnline.Param.SystemTalk+PlayOnline.GameInfo[i*11+11]+PlayOnline.GameInfo[i*11+10];
                ++j;
            } else {
                if (PlayOnline.GameInfo[i*11+11]) {
                    if (PlayOnline.GameInfo[i*11+11] == PlayOnline.UserName) {
                        PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"对你说："+PlayOnline.GameInfo[i*11+10];
                        ++j;
                    }
                } else {
                    PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"说："+PlayOnline.GameInfo[i*11+10];
                    ++j;
                }
            }
        }
    }
    var info = [];
    if (PlayOnline.Text.length > 15) {
        var k = PlayOnline.Text.length-15;
        for (var i = 0; i < 15; ++i) {
            info[i] = PlayOnline.Text[k+i];
        }
    } else {
        for (var i = 0; i < j; ++i) {
            info[i] = PlayOnline.Text[i];
        }
    }
    if (PlayOnline.ShowTalk) {
        var user = PlayOnline.NewUser ? PlayOnline.NewUser : PlayOnline.UserName;
        if (user == '系统') {
            this.changeTextColor(this.powerUpColor());
            this.drawText("注意！ "+user+" 出来发话了！",0,0,this.contentsWidth(),'left');
        } else {
            this.changeTextColor(this.systemColor());
            this.drawText("欢迎 "+user+" 进入游戏！",0,0,this.contentsWidth(),'left');
        }
        var line1 = Math.min(12,this.lineHeight());
        this.drawHorzLine(line1);
        this.changeTextColor(this.normalColor());
        if (j > 0) {
            for (var i = 15 - (this.height-100)/20; i < 15; ++i) {
                this.drawText(info[i], 0, 25+(j-1-i)*20, this.contentsWidth(), 'left');
            }
        }
        var line2 = Math.max(this.height-87,this.lineHeight());
        this.drawHorzLine(line2);
            this.changeTextColor(this.systemColor());
            this.drawText("目前有："+num+"人在线。",0,this.height-74,this.contentsWidth(),'left');
    }
    PlayOnline.Text.splice(0,PlayOnline.Text.length);
    for (var i = 0; i < 15; ++i) {
        PlayOnline.Text[i] = info[i];
    }
};

Window_TalkList.prototype.update = function () {
    Window_Base.prototype.update.call(this);
    this.dianji();
    if (PlayOnline.UpdateTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        PlayOnline.UpdateTalk = false;
    }
    if (this.lastShowSwitch != PlayOnline.ShowTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        if (PlayOnline.ShowTalk) {
            Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
            Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
            Graphics._input.value = "";
            PlayOnline.addButton(1,'map','BtnInput',this.x+10,this.y+this.windowHeight()-35,['script','Graphics._input.focus();']);
            PlayOnline.addButton(2,'map','BtnSend',this.x+this.width-70,this.y+this.windowHeight()-85,['script','']);
            PlayOnline.addButton(3,'map','BtnSize',this.x+this.width-23,this.y,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
        }
        this.lastShowSwitch = PlayOnline.ShowTalk;
    }
    if (this.lastBusy != $gameMessage.isBusy()) {
        if ($gameMessage.isBusy()) Graphics._removeInput();
        else {
            if (PlayOnline.ShowTalk && !$gamePlayer.isTransferring() && !SceneManager.isSceneChanging()) {
                Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
                Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
                Graphics._input.value = "";
            }
        }
        this.lastBusy = $gameMessage.isBusy();
    }
};

Window_TalkList.prototype.dianji = function() {
    if (!$gameMessage.isBusy() && PlayOnline.ShowTalk) {
        if (TouchInput.isRepeated()) {
            var dx = this.canvasToLocalX(TouchInput.x);
            var dy = this.canvasToLocalY(TouchInput.y);
            if(dx >= PlayOnline.Param.TalkWidth-70 && dy >= this.windowHeight() -74 && dx < PlayOnline.Param.TalkWidth && dy < this.windowHeight() - 30){
                if (Graphics._input.value.length > 0 && Graphics._input.value.indexOf(":") <= 0) {
                    PlayOnline.Text[PlayOnline.Text.length] = "你说：" + Graphics._input.value;
                    this.SendText(Graphics._input.value);
                } else if (Graphics._input.value.indexOf(":") > 0) {
                    var Text = Graphics._input.value.split(":");
                    if (Text[0] == '系统') PlayOnline.Text[PlayOnline.Text.length] = "不支持人机对话。";
                    else if (Text[0] == PlayOnline.UserName) PlayOnline.Text[PlayOnline.Text.length] = "屏蔽自言自语。";
                    else if (this.Online(Text[0])) {
                        PlayOnline.Text[PlayOnline.Text.length] = "你对"+Text[0]+"说："+Text[1];
                        this.SendText(Text[1], Text[0]);
                    } else PlayOnline.Text[PlayOnline.Text.length] = Text[0]+"不在线。";
                }
                Graphics._input.value = "";
            }
        }
    }
};

Window_TalkList.prototype.Online = function(user) {
    if (PlayOnline.Users.length > 0) {
        for (var j = 0; j < PlayOnline.Users.length; ++j) {
            if (user == PlayOnline.Users[j]) return true;
        }
        return false;
    } else return false;
};

Window_TalkList.prototype.SendText = function(text, name) {
    if (!name) var name = 'X';
    $.ajax({
        type: 'post',
        url: PlayOnline.Link,
        data: {user:PlayOnline.UserName, game:PlayOnline.Param.ActionName, cmd:'talk', text:text, target:name},
        success: function(msg) {},
        error: function (msg) {
            alert("连接不到服务器，请检查网络重试。");
            PlayOnline.UserName = null;
        },
        async: true,
    });
};

//-----------------------------------------------------------------------------
//  WINDOW_RankCmd
//-----------------------------------------------------------------------------

function Window_RankCmd() {
    this.initialize.apply(this, arguments);
}

Window_RankCmd.prototype = Object.create(Window_HorzCommand.prototype);
Window_RankCmd.prototype.constructor = Window_RankCmd;

Window_RankCmd.prototype.initialize = function(x,y) {
    Window_HorzCommand.prototype.initialize.call(this, x, y);
};

Window_RankCmd.prototype.windowWidth = function() {
    return 600;
};

Window_RankCmd.prototype.maxCols = function() {
    return 4;
};

Window_RankCmd.prototype.makeCommandList = function() {
    this.addCommand("首页",  'shouye');
    this.addCommand("我排在哪",  'mypage');
    this.addCommand("上一页",  'pageup');
    this.addCommand("下一页",  'pagedown');
};

Window_RankCmd.prototype.shouye = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= 0 && y >= 0 && x < this.width/4 && y < this.height){
            this.callHandler("shouye");
        } 
    } 
};

Window_RankCmd.prototype.mypage = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/4 && y >= 0 && x < this.width/2 && y < this.height){
            this.callHandler("mypage");
        } 
    } 
};

Window_RankCmd.prototype.pageup = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/2 && y >= 0 && x < this.width*3/4 && y < this.height){
            this.callHandler("pageup");
        } 
    } 
};

Window_RankCmd.prototype.pagedown = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width*3/4 && y >= 0 && x < this.width && y < this.height){
            this.callHandler("pagedown");
        } 
    } 
};

//-----------------------------------------------------------------------------
// Sprite_Character
//-----------------------------------------------------------------------------

PlayOnline.Sprite_Character_update = Sprite_Character.prototype.update;
Sprite_Character.prototype.update = function() {
    PlayOnline.Sprite_Character_update.call(this);
    this.updateShowName();
    this.updateShowNameZoom();
};

Sprite_Character.prototype.updateShowName = function() {
    this.setupShowName();
    if (!this._showname) return;
    this.positionShowName();
};

Sprite_Character.prototype.setupShowName = function() {
    var num = $gameParty.maxBattleMembers() + PlayOnline.Param.Players - 1;
    if (this._showname) {
        if (PlayOnline.UpdateName[PlayOnline.UpdateCount]) {
            this._showname.setActor(this._character);
            this._showname.update();
            PlayOnline.UpdateName[PlayOnline.UpdateCount] = false;
            PlayOnline.UpdateCount++;
        }
        return;
    }
    if (!SceneManager._scene._spriteset) return;
    if (this._character instanceof Game_Follower && this._character._memberIndex < num) {
        this._showname = new Window_Name();
        this._showname.setActor(this._character);
        SceneManager._scene._spriteset.addChild(this._showname);
    }
};

Sprite_Character.prototype.positionShowName = function() {
    if (this._showname) {
        var win = this._showname;
        var width = win.width * win.scale.x;
        win.x = this.x + width / -2 + win.bufferX();
        var height = win.height * win.scale.y;
        var buffer = win.bufferY() * win.scale.y;
        win.y = this.y + (this.height * -1) - height + buffer;
    }
};

Sprite_Character.prototype.updateShowNameZoom = function() {
    if (!this._showname) return;
    var spriteset = SceneManager._scene._spriteset;
    this._showname.scale.x = 1 / spriteset.scale.x;
    this._showname.scale.y = 1 / spriteset.scale.y;
};

//-----------------------------------------------------------------------------
//  Sprite_PlayButton
//-----------------------------------------------------------------------------

function Sprite_PlayButton() {
    this.initialize.apply(this, arguments);
}

Sprite_PlayButton.prototype = Object.create(Sprite_Button.prototype);
Sprite_PlayButton.prototype.constructor = Sprite_PlayButton;

Sprite_PlayButton.prototype.initialize = function(Button) {
    Sprite_Button.prototype.initialize.call(this);
    this._hidden = false;
    this.setupButton(Button);
};

Sprite_PlayButton.prototype.setupButton = function(Button) {
    this._button = Button;
    this.bitmap = ImageManager.loadPicture(Button.image);
    var h = this.bitmap.height / 2;
    var w = this.bitmap.width;
    this.setColdFrame(0, 0, w, h);
    this.setHotFrame(0, h, w, h);
    this.x = Button.x;
    this.y = Button.y;
};

Sprite_PlayButton.prototype.processTouch = function() {
    if (this.isActive() && this.opacity >= 255) {
        if (TouchInput.isTriggered() && this.isButtonTouched()) {
            this._touching = true;
            this._wasTouching = true;
        }
        if (this._touching) {
            if (TouchInput.isReleased() || !this.isButtonTouched()) {
                if (this._wasTouching) {
                    this.callClickHandler();
                }
	this._touching = false;
	this._wasTouching = false;
            } else {
	this.callPressHandler();
            }
        } else if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
            this._touching = false;
        }
    } else {
        this._touching = false;
        if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
        }
    }
};

Sprite_PlayButton.prototype.setPressHandler = function(method) {
    this._pressHandler = method;
};

Sprite_PlayButton.prototype.callPressHandler = function() {
    if (this._pressHandler) {
        this._pressHandler();
    }
};

Sprite_PlayButton.prototype.update = function() {
    Sprite_Button.prototype.update.call(this);
    this.updateVisibility();
};

Sprite_PlayButton.prototype.updateVisibility = function() {
    if ($gameSystem._hideBtns) {
        this.opacity -= 20;
    } else if ($gameMap.isEventRunning()) {
        this.opacity = Math.max(this.opacity - 20,this._button.eOpacity);
    } else {
        this.opacity += 20;
    }
};

//-----------------------------------------------------------------------------
//  PlayOnline function
//-----------------------------------------------------------------------------

PlayOnline.createButton = function(obj) {
    if (!obj) return;
    var index = obj.id;
    if (!PlayOnline.canHasButton(index)) return;
    this.removeChild(this._GButtons[index]);
    this._GButtons[index] = new Sprite_PlayButton(obj);
    if (obj.action) {
        var type = obj.action[0];
        var data = obj.action[1];
        switch (type) {
            case 'button':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnPress.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'buttonT':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnTrigger.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'script':
                var script = data;
                this._GButtons[index].setClickHandler(this.gButtonScript.bind(this,data));
                break;
            case 'event':
                this._GButtons[index].setClickHandler(PlayOnline.runGCommentEvent.bind(this,data));
                break;	
        }
    }
    this.addChild(this._GButtons[index]);
};

PlayOnline.onButton = function() {
    var x = TouchInput.x;
    var y = TouchInput.y;
    var btns = SceneManager._scene._GButtons;
    if (!btns) return false;
    var result = false;
    for (var i = 0; i < btns.length; i++) {
        if (btns[i] && x > btns[i].x && x < btns[i].x + btns[i].width && y > btns[i].y && y < btns[i].y + btns[i].height) {
            result = true;
            break;
        }
    }
    return result;
};

PlayOnline.canHasButton = function(i) {
    return $gameSystem._gBtns[i] && $gameSystem._gBtns[i].location == 'map' && ($gameSystem._gBtns[i].mapId == 0 || $gameSystem._gBtns[i].mapId == $gameMap.mapId());
};

PlayOnline.runGCommentEvent = function(id) {
    $gameTemp.reserveCommonEvent(id);
};

PlayOnline.btnPress = function(btn) {
    Input._currentState[btn] = true;
};

PlayOnline.btnTrigger = function(btn) {
    if (!PlayOnline.Triggered[btn]) {
        Input._currentState[btn] = true;
        PlayOnline.Triggered[btn] = true;
    } else {
        Input._currentState[btn] = false;
    }
};

PlayOnline.btnRelease = function(btn) {
    Input._currentState[btn] = false;
    PlayOnline.Triggered[btn] = false;
};

PlayOnline.addButton = function(id,location,img,x,y,action,e) {
    var obj = {id:id,image:img,x:x,y:y,action:action,eOpacity:e || 0};
    var location = location.toLowerCase();
    if (location[0] == 'm') {
        obj.location = 'map';
        obj.mapId = Number(location.replace('map',''));
        $gameSystem._gBtns[id] = obj;
        SceneManager._scene.createGBtn($gameSystem._gBtns[id]);	
    }
};

PlayOnline.removeButton = function(id) {
    $gameSystem._gBtns[id] = null;
    SceneManager._scene.removeGButton(id);
};

//-----------------------------------------------------------------------------
//  Graphics._Input
//-----------------------------------------------------------------------------

Graphics._createAllElements = function() {
    this._createErrorPrinter();
    this._createCanvas();
    this._createVideo();
    this._createUpperCanvas();
    this._createRenderer();
    this._createFPSMeter();
    this._createModeBox();
    this._createGameFontLoader();
    this._createInput();
    this._createInput2();
};

Graphics._updateAllElements = function() {
    this._updateRealScale();
    this._updateErrorPrinter();
    this._updateCanvas();
    this._updateVideo();
    this._updateUpperCanvas();
    this._updateRenderer();
    this._updateInput();
    this._updateInput2();
    this._paintUpperCanvas();
};

Graphics._createInput = function() {
    this._input = document.createElement("input");
    this._input.id = 'Input';
    this._input.type ="text";
    this._input._sx ={};
    var sx = this._input._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._createInput2 = function() {
    this._input2 = document.createElement("input");
    this._input2.id = 'Input2';
    this._input2.type ="text";
    this._input2._sx ={};
    var sx = this._input2._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._addInput = function(type,x,y,width,height,fontSize) {                
    this._input.type = type || "text";
    var sx = this._input._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput();
    sx.xs = true;
    document.body.appendChild(this._input);
};

Graphics._addInput2 = function(type,x,y,width,height,fontSize) {                
    this._input2.type = type || "text";
    var sx = this._input2._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput2();
    sx.xs = true;
    document.body.appendChild(this._input2);
};

Graphics._removeInput = function() {
    this._input.remove();
    this._input.value = null;
    this._input._xs = false;
};

Graphics._removeInput2 = function() {
    this._input2.remove();
    this._input2.value = null;
    this._input2._xs = false;
};

Graphics._updateInput =function () {   
    this._input.style.zIndex = 13;
    var sx = this._input._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input.style.position = 'absolute';
    this._input.style.margin = 'auto';
    this._input.style.top = y  + 'px';
    this._input.style.left = x  + 'px';
    this._input.style.width = width + 'px';
    this._input.style.height = height + 'px';
    this._input.style.fontSize = fontSize + 'px';
};

Graphics._updateInput2 =function () {   
    this._input2.style.zIndex = 12;
    var sx = this._input2._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input2.style.position = 'absolute';
    this._input2.style.margin = 'auto';
    this._input2.style.top = y  + 'px';
    this._input2.style.left = x  + 'px';
    this._input2.style.width = width + 'px';
    this._input2.style.height = height + 'px';
    this._input2.style.fontSize = fontSize + 'px';
};

Input._onKeyDown = function(event) {
    if (this._shouldPreventDefault(event.keyCode)) {
        if (Graphics && Graphics._input && Graphics._input._sx && Graphics._input._sx.xs) {
        } else if (Graphics && Graphics._input2 && Graphics._input2._sx && Graphics._input2._sx.xs) {
        } else {
            event.preventDefault();
        }
    }
    if (event.keyCode === 144) {
        this.clear();
    }
    var buttonName = this.keyMapper[event.keyCode];
    var version = parseInt(Utils.RPGMAKER_VERSION.substr(2, 1));
    if (version < 5) {
        if (buttonName) {
            this._currentState[buttonName] = true;
        }
    } else {
        if (ResourceHandler.exists() && buttonName === 'ok') {
            ResourceHandler.retry();
        } else if (buttonName) {
            this._currentState[buttonName] = true;
        }
    }
};