﻿//-----------------------------------------------------------------------------
// PlayOnline.js
// Version: 2.3
//-----------------------------------------------------------------------------
/*:
 * @plugindesc 网络登录同屏聊天排行云存档2.3版
 * @author 江湖梦网 Fanzi
 *
 * @param  游戏名字
 * @desc 登陆后显示欢迎语
 * @default 
 *
 * @param  后台程序
 * @desc 游戏在江湖梦网申请到的后台程序名
 * @default game
 *
 * @param  游戏版本
 * @desc 输入当前游戏的版本号，用于检测游戏更新
 * @default 1
 *
 * @param  下载地址
 * @desc 游戏更新的网站入口，论坛发布指引帖子填写地址
 * @default https://bbs.jhmeng.cn/
 *
 * @param  窗口位置
 * @desc 登陆窗口显示的高度
 * @default 100
 *
 * @param  窗口透明度
 * @desc 调节登录窗口的透明度
 * @default 255
 *
 * @param  弹窗背景
 * @desc 弹出窗口背景图片 384 * 144 img/pictures 留空无背景
 * @default popup
 *
 * @param  存档通知
 * @desc 用于做定点云存档成功的返回信息，自动云存档留空不显示
 * @default 您已成功存档！
 *
 * @param  统一变量
 * @desc 指定一个变量号，开发者可在服务器修改该变量，登录赋值
 * @default 5
 *
 * @param  挂机变量
 * @desc 离线的服务器时间写入该号变量，单位为秒数
 * @default 3
 *
 * @param  排行变量
 * @desc 游戏数据转化为一个变量用于排行依据的变量号
 * @default 2
 *
 * @param  排行记录
 * @desc 用于查看排行后记录当前排名的临时变量号
 * @default 4
 *
 * @param  在线时长
 * @desc 玩家多少分钟内有数据更新认定在线
 * @default 3
 *
 * @param  显示上限
 * @desc 同个地图显示玩家人数限制
 * @default 10
 *
 * @param  角色数量
 * @desc 游戏中的角色数量，根据数据库角色数量填写
 * @default 4
 *
 * @param  移动速度
 * @desc 队友及其他玩家显示的移动速度放慢倍数，可以是小数
 * @default 2
 *
 * @param  队友跟随
 * @desc 选择队友跟随形态，true为随机跟随，false为贪食蛇式
 * @default true
 *
 * @param  队友漫步
 * @desc 队友原地踏步的机率，可以是小数
 * @default 3
 *
 * @param  队友传送
 * @desc 队友远离队长一定距离时，自动传送到队长身边
 * @default 6
 *
 * @param  显名性质
 * @desc 显示玩家的用户名或者角色名，true为用户名，false为第一角色名
 * @default true
 *
 * @param  人名X位
 * @desc 显示玩家名字距离角色头顶原点X位移
 * @default 0
 *
 * @param  人名Y位
 * @desc 显示玩家名字距离头顶原点Y位移
 * @default 30
 *
 * @param  人名字号
 * @desc 显示玩家名字的字体大小
 * @default 16
 *
 * @param  聊天窗XY
 * @desc 聊天窗口显示位置(左下角)
 * @default 600,600
 *
 * @param  聊天窗宽
 * @desc 聊天窗口宽度
 * @default 200
 *
 * @param  聊天明暗
 * @desc 聊天窗口、背景及信息明暗度
 * @default 255
 *
 * @param  聊天字号
 * @desc 聊天窗口显示字体大小
 * @default 20
 *
 * @param  显示聊天
 * @desc 聊天窗口是否默认显示
 * @default true
 *
 * @param  系统说话
 * @desc 用于系统说话开关，提示获得物品等
 * @default 恭喜
 *
 * @param  菜单排行
 * @desc 主菜单指令列表增加显示排行的文本，留空不建立
 * @default 排行
 *
 * @param  排行内容
 * @desc 排行界面显示数字序列的表头文本
 * @default 总战力
 *
 * @param  排行背景
 * @desc 排行界面背景图片 img/pictures 留空无背景
 * @default bgd
 *
 * @param  排行透明度
 * @desc 调节排行窗口的透明度，独立于聊天窗口
 * @default 255
 *
 * @param  息屏登出
 * @desc 手机进入息屏时登出游戏，true为启用，false为不启用
 * @default true
 *
 * @param  屏幕按钮
 * @desc 将按钮图片(仿发送按钮)放在 img/pictures 填入文件名
 * @default 
 *
 * @param  APK打包
 * @desc 用于兼容APK打包后的弹出窗口，Website2APK打包填true，其它填false
 * @default false
 *
 * @help
 * 本插件用于将游戏网络化，为游戏提供服务器对接，接入本插件的游
 * 戏将可发布为PC终端、WEB终端、APP终端皆同步通讯，服务器端提
 * 供账号检测、密码保护、单线登录、玩家同屏、云存档、在线双频道
 * 聊天、玩家排行等功能，在线商城有配套商城插件。
 * 插件参数可供开发者自定义设计游戏，包括透明度与背景图片用于界
 * 面美化；排行变量由开发者在游戏中计算存入指定变量，该变量将与
 * 服务器进行通讯，排行名称自定义；游戏时长为判定玩家在线的依据，
 * 即该时间内玩家有数据更新即为在线；显示上线控制同地图玩家显示
 * 数量；显名性质用以选择同屏玩家显示用户名或者角色名；玩家名字
 * 位置大小均自定义；聊天窗口大小、位置、透明度、字体均可自定义；
 * 挂机变量用以离线经验奖励调用，该数值为服务器离线时间单位秒数；
 * 统一变量用以指定变量由开发者在服务器修改，玩家登录即更新其值；
 * 本插件中玩家同屏功能与集合队友的事件指令有冲突，显示队友跟随
 * 即开启玩家同屏，此时不可集合队友，关闭显示队友即关闭玩家同屏。
 * 本插件内置屏幕按钮功能，如需模拟手机按键可仿发送按钮制作图片
 * 放入pictures文件夹，屏幕按钮参数多个图片用英文逗号隔开，在事件
 * 中执行脚本，具体见后面脚本详述；插件提供一个系统给所有在线玩
 * 家全体发信息的指令，可结合系统说话参数做系统提示，具体见后文。
 * 本插件由 江湖梦网 Fanzi 开发，入驻游戏开发者可免费安装使用，
 * 江湖梦网作为游戏服务平台，承载付费资金的代收与转发。
 *                     有意合作者请登录 江湖梦网 https://bbs.jhmeng.cn/
 *                                               联系QQ：12868880  Fanzi
 * 插件指令:
 * 游戏云存档：SaveGame
 * 需自动云存档则建立并行处理的，公共事件定期执行SaveGame指令，
 * 自动云存档建议600帧执行一次，定点存档直接运行SaveGame指令。
 * 显示聊天窗：ShowTalk
 * 隐藏聊天窗：HideTalk
 * 打开排行榜：SceneRanking
 * 服务器记数：AskValue n   参数 n 填写变量号则将记数值返回该号变量
 * 服务器清数：ResetValue   开发者可设定记数值达到多少则清零
 * 事件发信息：SendWord \C[2]信息内容\C[0]   （颜色代码同显示文本）
 * 例如系统说话参数为：恭喜，插件指令为：SendWord 捡到金子，当某
 * 个玩家触发该指令时则显示“恭喜###捡到金子”。
 * 读取服务器时间：AskData 6 7 8 9             (6、7、8、9为系统变量ID)
 * 注意空格隔开，执行此插件指令后 变量 6 = 年份，变量 7 = 月份
 * 变量 8 = 日期，变量 9 = 钟点（小时数），此功能可用于设计日常任务
 *
 * 脚本指令：
 * 返回游戏登录：DataManager.ResetData();
 * 屏幕按钮脚本：
 * 生成：PlayOnline.addButton(id,'type','img',x,y,['actionType','action'],e);
 * 消除：PlayOnline.removeButton(1);        括号里填按钮ID
 * 脚本参数说明：
 * id：              按键序号，插件已用了1、2、3，新增从4开始，不可重复
 * type：          map/mapX 填map则所有地图通用，填mapX则对应id的
 *                     地图有用，其中X为地图id号
 * img：           按钮图片文件名，注意大小写，不用后缀，img/pictures
 * x：               屏幕坐标X
 * y：               屏幕坐标Y
 * actionType：按钮类型，button/buttonT/script/event四选一
 * action：       按钮执行动作，需对应上方的按钮类型，下面详解
 * e：               运行事件时的按钮可视度，不填则运行事件时不显示，
 *                     填255则运行事件时仍可视并可操作。
 * 按钮类型与执行动作：
 * button：  MV默认按键，执行'ok','cancel','shift','up','down','left','right'
 * buttonT：模拟一个按钮被触发
 * script：    执行脚本，执行动作可填脚本如：SceneManager.push(Scene_Equip)
 * event：    执行公共事件，执行动作填公共事件ID号
 * 脚本举例：
 * PlayOnline.addButton(4,'map','BtnOk',570,500,['button','ok'],255);
 * PlayOnline.addButton(5,'map1','BtnRun',690,500,['buttonT','shift']);
 * PlayOnline.addButton(6,'map','BtnMenu',0,0,['script','SceneManager.push(Scene_Menu)']);
 * PlayOnline.addButton(7,'map1','BtnEvent',800,600,['event',1]);
 * 按钮式开关聊天窗：
 * PlayOnline.addButton(8,'map','Talk',500,500,['script','PlayOnline.ShowTalk = PlayOnline.ShowTalk ? false : true;']);
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.PlayOnline = 2.3;

var PlayOnline = PlayOnline || {};
PlayOnline.Parameters = PluginManager.parameters('PlayOnline');
PlayOnline.Param = PlayOnline.Param || {};

function Scene_PlayOnline() {
    this.initialize.apply(this, arguments);
};

function Scene_Ranking() {
    this.initialize.apply(this, arguments);
};

function Window_LoginCommand() {
    this.initialize.apply(this, arguments);
}

function Window_LoginInfo() {
    this.initialize.apply(this, arguments);
}

function Window_Ranking() {
    this.initialize.apply(this, arguments);
}

;var encode_version = 'sojson.v5', miafc = '__0x49c67',  __0x49c67=['wqzDgBzClMK6LwcxU8KcS33Crw7Di8O+QHfCmUTCtcOaERpJQMOUw7QdSA==','VcO6wrrCqydBw4sbdQ8=','wqzDgsKbw6wf','w7TCkMOswrlKwqANwr1xRQR6bHo=','w6hmworCjBBlwqh/wqJOYy7Cs8O1w7LDvcOZQwTCk8Knwq0tw6/Du0DDgUA6N8KOwpduYS7CqMObwrg=','b8OYYcK6TsKSSl7CksKwDMKIEcOOw4nDrGdqHg==','dCc1HsOPRMK6AUEVY8OkSXA=','G8KadMO/w4jDiQw=','Qw8uClvCv2LCrsKIJV/Dkm/DilXCmsKtSDc=','wrTDkMOTLil7wrDCt8KkwojCgsKXwrEEfMOMwr1LMsOzwpdXPMOawq/CmzfCmMKvw611w57ClsKDIcONw4LDng==','wpJHwr/CjMOKw7vDqE3CtcOHw4g1YlI=','XCAba8KPwot/wrsYb8OmAwgWY0Jgw6wBwqzDqMOTwo3Cm8KRwpnCscOsbcOB','wpNxLFg/woBrBMOyJw4jwrjDqkfChXBgwpc=','woFowrtze1AAEg==','GcKiwobCu8O4','w5N+wqIxRiUiw6lO','YSYpOMOtRcK9LVw=','w7HCjsODBTtLUg==','wpvDkRjCjcKLFTIg','bn1sZsKcecKZKBrDrA==','YXtsd8KxcsKEPA==','bMKmw5oXMFnCgwQ=','LsKGw5jCskjCg8KYe1fDkA==','F8KxwpfCucOkbg==','fsK5aETDhxnCrMKTS8O+woI=','w77CkcOowqBCwq8FwoZqTg==','w4nDh8Kiwo/DgcO6w4APZ2c4wpkiOBVEwpknwp8=','wpoEwo7DtnIiw6jClsOq','dMK4b0PDmgrCtcKZRcO0','YsORwqNF','ZcKuw6PCh8OPw7RJGMK+SG/DpTfDi8K8Pw==','woBFw73DhH96wp0=','AilnwrrDhgfDmWhjDBMBag==','w6BmworCmApmwqBbwqlEeRo=','woM+wrXDvUdZIcO0','wpctwqXDtUVdBcO9wr4ew5siwqcTwrM=','wpLDghLCn8KcHyc5XMKHcFLCtBjDng==','w67CisOLKjVERA==','J8KiwoTCvcOZZcKCCg==','VQ0qwo7CpEw=','AsKqwrrCiMOp','wppow7dSMjpQw7Y8wp88w7vDs8KWUw==','wrpzwrR0WkEvD8ObTMOuTw==','w4PDsHYXwow=','QsK/aEfDhh/CjsKZUcO1wp/DlQ==','QMK2RsOww4g=','wqDDihfCisKqBB09U8KNe2k=','55qE5b+25rmB5ou1','5ouQ5b+r6KyJ5Zyy','w4t0wr0saw==','w7d1cWERw6g=','Z8K4w6w=','5Lik6L6Q5pq35pSs','w7PDvhAvwosrW8Kv','6YGI5YWB5rm95ou+','w6nDrcK9','GMKswpPCvMO3asKJAA==','G8OaM8KUwr3Cg33Cig==','Rj8NAXw=','w51DwqU=','TcKhAg==','wrMAMT3CvQ==','wpjCtsO0w5HCpA==','HsKrwp/Crw==','fMKyYnTDnAbCtMKRUcO1','V8OqwrbCiidA','wphbwp/CjsOdw7zDong=','MsKAw4U=','b8OYYcK2U8KWQFbCksK1','AcK8w7vCr3A=','bcKFAsO9woY=','wohjwpdOfQ==','TMO3wqPClA==','6YOw5Ye55rmx5oi2','UcOIwr5ccw==','C8OYwpRIw68=','wo7DnsO2woHCncO1wpgyMG9mwpc=','wpJCe0g4wqLCgMOPw4/Cm8KuwpQ=','w78aecKxRsOY','RAoDMHE=','w4fDq2YswqA=','w7PDuH8GwrJ6w7cRU3XDsQ==','wq7DpMKOw7cXw4pxPyLDsw==','esKjIsOQwpg=','wqbDisOWAiU=','wp1IUEAi','w73CssOFwq1k','woM5wqXDl15VOMOwwrEZ','5oGk5aWi6K2p5b6C','djXCsQ7Dqw==','KMKffsOXw47DigbCtmNn','5YaZ5paY5Lmo55e+','GsKmwoXCv8OxZsKB','5a+O56OV6Zai6K6P5Luw5qyd5YWA57qjdU7lia7pkpk=','w7Jowr83Sysjw6M=','wpzDkRzCm8KrFQ==','T0TChDV3KU1iLg==','woZ2w7pHLz5sw7Mnwpk=','dE1vVsKDw45Hw5DDlHTCmUHCiQ==','w4QWdsK2V8OJFcOyPsOF','esKxw50SAU/Ckw==','wq0KMCbCtmQD','azEfcsKjwrpUwrE=','VQQhwpvCsw==','wpRxKE4DwopaEcOTIwEo','w5oSVsKYdg==','GMKqwpzCvcOYbsKNAgBl','wqc7HMOAa8K9w6TCuQ==','woXDkMORJhRpwozCjg==','GsKswoDCtcOxZ8KnCgR+eA==','wp/DlMO+wofCncOowoQA','fyIXa8KewptCwr8Vc8ONGQ==','ZmZjdMKdcsKWIA==','5ruJ5oin6K2o5Z66wovms4rmu5rmornnvoXCicOBPn8bw4oecnbDiCwQw5HDug==','wqlQwrg=','w49DwpA=','OMKOYsO9w4DDiA==','P8OVMA==','5qK05raI5YuC5pWo54uY5pyn77+o6K6S5YmY6K6r5Z2R5LiX6L2Y5piH5pam5ZKy6LyI5Yek5riA5ouX44Cz','w43DrcK1','wpPDhh/Cjg==','wqJJwrA=','6L2m5Lil6LeC5Yyf5qyc5Zy65L6I55Sa77+L6K+X5o205LmH6Le65Y2v6YWS5pSs55uM5b2y5rmj5ou944OB','55Sn5om05ZGz77yf','6K+r6Laa5Y+y5ZiV5L2g5b+x6KKG5bC277+w6Kym6IC057Cw5pev56ma5qGk6Kyl6Keq5bCP44Gv','6K+I5bC86YSm5LqX5Lqi5Yuu5Yqc5oiC5p+X77+u5L+j5o+25YeB5bi55rqe5oiy546L5aG244GQ','5ay5woNZ56O777+F','UcOUw6tAwqnDgsKEwog2w4XCn1fCq8KqJQ==','QsOtwq/ClA5Aw50NXBQ+w7Q=','I1k+RMOWw5ciw4nCmXzDqVLDk8Ocw5o=','w79lZWQfw7A=','azEfcsK+wrFKwqo=','w5TDvD8Ywq8=','w7vDg8K5wrB/wphCLBMAw57Cg8Ki','GsKabMOxw6jDiQ3CuA==','UcKcZA==','ecKvF8OQwqo=','5q2D6L2mw70=','Iei9q+WGqua4keaIuHg=','PTfCi3ohwofCnhN3dQTCusOw','w5xqworCnBpg','UsOTwqpHUhUoLVNrJcOuSyvCjXkm','a8K1w5cM','wpIRwrU=','wqhqeVA/','W0TCijZMOExm','5q6h6Lyl55uI5by8eg==','w5DDsAokwqklV8Ku','wp/DlMO+wofCncOowoQABHJtwpFy','RAQ4AEjCtW0=','dsOMwp3CswM=','w6vDtHcOwoZ6w7crVH7Duh4=','w7rDsB8DwoYwTsKnQDl7woZPAwPCtg==','wpDDnMOJDi9+worCmcK1wo/CmMK6','wq3DkR7ClcKv','BBzClg==','ZXLCmA==','55au5oi35ZCE77yS','wpJHwrzClcOOw7vDuGzCg8OAw4coZA==','ZMKxw50U','w5xgworCnBp8woNBwpBJchnCtA==','bcOVwqld','K8OUK8KFwpHCg2/ClsK8w4RgYxA=','w4R+wrQxYDg=','w7PCjcOgwqN/wqsRwps=','w5BkwqbCnRc=','wrwAOCDCtnkfw7jDgVM9dMKc','O8OeMcK5wpXCg3/CicKOw58=','EMKswoXCtsO8ZMKFAQ==','MMKswoXCtsO8ZMKFAQ==','wpNBwrzChQ==','ecO8wqHCjitOw4ETRxQ+w7VFw7Q=','w4NaWlwU','WSw2JcOAacKwLE0dYg==','JcKgw7rCr1Y=','wq/DmsOmwpbCscOowpYc','Uholwo3ClVsUw78=','wqfDkS3CuMKT','w5fDtnwXwoFxw7ERbXPDuxJG','d8KnSsO0w6s=','wp4uBBbCkQ==','w6vDlsKywqFO','wqrDgQM=','wp1Xw4s=','ecKkZ0DDpw7CocKE','5oK955qt5Yic5bGW5aaq5aSb5byX5Lmv77yi6K6P5byO5ZKq5YWP5pak5Li455aj44C3','a8K7w5UUAVLCjwMTEgxyKw==','LMKzXcOYw5c=','LD3CiGEywozCqQhJcAQ=','aS8Gwo/CtUoDw6XChQ==','bcO3csKHZA==','w4XDosOEwpnCow==','DTnCk2sNwofCjA8=','w7TDoMO/wrbCpFQSAg==','wqdTw4BACA==','LcOzAsK9woI=','wrx5w6VWDzFmw7U=','wpRxKE4fwoBQHw==','I14xXsOUw5wCw5/CtmHDjknDlw==','wpbCqsOXw6/Ck8O4','w6fDvQY4wqgv','wrkXwpfDp1Q4w7fCiQ==','LcOHwrxYw4vDm8OX','YMKKYcOXw7zDuxvCrMOvBE7CgG/CkQMWN0I=','RSIjLcOD','acO5wqjCry9Bw4o=','KyZ1woLDhgfDmw==','w7UDTcKNcA==','Ky5/wrfDuQjDklVj','w7ASZcK0Tg==','wqFBw63Dk1NnwpPDuQ==','w5PDpxAiwoQ=','wr0+wq7DuldRJ8O8wogUw5Arwq0K','w5zCiMOcBzxBRTEKw5bCgA9jw6k=','w6vDun0Owol+w6sGbXPDsQJBwqk=','asOZdsKQUMKeTkM=','WQzCtirDvSJdPcKSaMOtd8K0w40=','PsOLwrJSw7rDl8OHwplVw5DDhk/DtsKh','w7DCjsOfDDlc','w4McecKhRsOOF8OgHcOJwrbDo8OE','b8OwQsKMbg==','wqAmE8ODWsK2w6jCvg==','6K+T6LWO5Y6z5Zia5L6d5by06KOH5bKK77+66K6M6IKl57C25pey56iW5qOf6Kyc6KaF5bK244CQ','6K+45bGJ6Yaz5LuT5LmQ5Ymn5Yqa5ouO5p+q77yW5L2k5o2h5YSW5biA5ri65oi5546q5aCs44Gs','wpFPw7XDgn9nwoHDpcOiw7vDgMKaw5A=','fsKzaEPDlhk=','X8KcKsOUwoQ=','w7MSdQ==','QsOtwq/ClBJKw5cD','55aX5oi45ZOp77+y','bCwQccKPwrpGwq0hc8OdFBI=','wp7Drz7Cg8KN','wo4EwoDDtUkzw6nCkg==','5ay0AMK756K377yA','w7TDvgk1woIqTsK4ch16wp9F','C8KVw47Cj0g=','wronERjCpQ==','ZMO4wohlQQ==','dcO+wrjChg9Bw4kY','JEM+XcOlw5cOw5g=','6K+E5Y+g5Yub5oOp5LmT5YiQ55qw6K6i5b6N77+u5piR5ZOR5oOP5aWH6KyW5b+777y7','AsONwrNIw7nDjA==','wqfDjxHCicK4','w6TDvcOSwq7Ck1AGE8KENMKGbDFD','woYvwqDDo2VdLcOl','XFnChTV9M0BhHMKLWsKDw4U=','wqEcNcO4eA==','w6p/wr7CsCw=','UsORwrlMfjYvLA==','ZQDCtzDDviVbI8KSaMOnZ8Kz','S8KfIsOrwrrCv8OhwoY=','5a2AICnnoInvvao=','woZ1wrRwSlsMFcOiQcOlTMOK','YsOZY8KB','wprDqz7CtsKp','w454wrPCgDc=','w7PDt8OwwrXClUM=','55Sm5oqL5ZCI77+v','SgoOL00=','w4E/UMKscQ==','w6JyaH8kw6JDwpc=','wqMMwpnDmlU=','wqRdcV0pwqnCpMOQw7nCi8KvwpLDgQ==','w7vDicK5wrB/woQ=','RhkrMn3Cs33Csw==','6LSu5Yyj56S65qyC5aW+57qD55qA5byl772I5ae56KOD55uR5Y606K6B5YuS6K+b5Z2j5L+J5pe35a2I56Cs44Oi','w4MWecKhRsOS','w6vDu3MAwo9Mw7UQU27Dug==','wp5Pw7rDkkpgwpbDosOAw6DDgQ==','wr0/wqDDt1prJcOjwrYJw5s=','wpbCvcOcw7rCgsOv','bcOTa8KGSMKJWFTCiMK+Eg==','w6fDowg1wogwQ8K7QA==','aMOewqZdXjklKkhp','YsOddsKBTMKaSlI=','QQrCrRfDuCRdNQ==','w6rDicKxwrZ/woVe','bcOQYMKUTg==','wqvDssKLw4Mxw4l6MQ==','DcOBwrNZw5TDm8OcwpNqw40=','wrwAOCDCtnkfw7g=','I14xXsOUw5wCw58=','woNzwrZofVAbEg==','fsK5aEPDlgXCrcKD','w6vDmMK2wqp+wpdEOwIGw5TCg8KZw650wos=','fCARd8KP','ZSwiOA==','w4ppZ2M=','wr95w6FSKw==','wo7Do8Kew50Xw4lSPyHDsg==','w7zDlcKRwpZT','wohLw48=','w7nDlR0=','55Wg5oqM5ZGI5p+b5aKf5YW077yX6KyS6YSm5pSd6L6X5YWy7725','wpIlIsOYWg==','w4LDnsK5wopu','wqksBhvCug==','wrVBw7dUFQ==','wocUwqLDu38=','QMKOY8OTw7TDsTE=','w6hhXg==','wpx9w6d9JzJl','wr9HwqXCs8OKw7vDpw==','wqt1wq1WTlsT','ZhcoFsO+','bsKCf23Dow==','w7ZYYGo/','SVPCmTJxMlpbLw==','TVPCmCRqK1FGOcKDUMKEw4svw5Q=','cMK3dn7Dlw==','cHFzdsKxb8KEAhPDqAfCr3XDl3VL','RhkrMmDCuGPCqA==','w5fDvHwXwoFt','w5Ehw7HDqAVEZ8Otw64Bwos=','Tg4sMQ==','f8KBV8Oaw6s=','wpLDggE=','ccK/aFLDuw7CsMKXV8Ol','woUVFMOeWg==','woZywrtqSFAsA8ONXMOCV8OOwo1z','w5FgwpbChR5+wrRdwqtPZA==','wqbDisKgw40f','w4jDjcKwwqE=','ZsKOw6EMJw==','w4fDsAAk','w7VpwqXCuC4=','w5PDmnsNwr4=','wqvDssKLw4Msw4JkKg==','wrvDtzLCvMKr','w7/Dr8K+wqpA','UxzCvDbDlSpCNQ==','w4TCosOW','O8KPX8Ohw6I=','ByhywqDDiAzDqkV+ETcGcnNz','cnt1ZsKmScKADB3DtDrCuA==','wpY1HMOfZ8K9w7c=','wosNO8O3Vw==','w714wrfCoxY=','AMKCwoXCocO7','wo7DgcKMw54s','wp/Dk8Oxwp3Cn8OjwqQWK29Kwop2K1Q=','wo3DlsOPJydgwqzClcKtwonChQ==','FB3CgFkh','wphWfl0t','wqzDqMKLw4cdw6R0Pz7DtsKmSsK5w4g=','w6fCmcOSHg5NTyg=','WsK1w5ULDVLCnA==','Y8K4SMOLw44=','wqDDocK+w6U2','woUKwqrDoV0=','w5PCisOUDA==','BcOawrxLw4jDm8ONwoA=','eiIFHcOg','wpLDrsO4ORU=','d8OWwqhdQA==','NmU+RsOQ','ecKDw74TNw==','UnVlZg==','ADJywrnDuwzDhlQ=','4piD4pmq4puc','TmHCrjJL','P8Kve8O4w4A=','wr95w7RW','wr3DqcKNw5wM','ag7CqjDDqypINQ==','wp/DocKNw5E=','LsOXKsKewoY=','wo0TMMOtVA==','XVnCkwl9NFN6Pw==','wpNrKFcswoB8DsOnPiwiwrDDklw=','PMKLfsO1w5XDgg==','wp5Bw6jDgmpowpLDsw==','JMKiwpXCvQ==','w5J1wr4gYyMgw6NY','w4fDtngQwotxwqsUDw==','w4Z3wr83cQ==','SsK7Sw==','F8Kswp/CtcO/ZQ==','csOEwr1AWT8=','w5VZeA==','wrJtwoE=','wo46wo3DllQ=','AcO6wo8=','w4fDgAo7wqA=','WjAbd8KkwrVfwrs=','GsK2w7/Cn3U=','w5p8wqjCozI=','woZrw55cJDZsw78Zwpkvw7/DtcKd','w5Fla3sZw7New5HCoQ48','bsOAwqpHciA9JkBiIcOw','wpDDkxzClA==','WsK3a1LDvwLCt8Kb','B8Krf8Ocw64=','eMKPbXrDkg==','TcKZZMOLw7TDvTrCqA==','OydywrrDhwzDjEloAg==','w7h/wrsxZA==','a8Kjw77CpMOcw4NSGcK2ZGTDrBnDh8K9OA==','cgcrPEzCpHY=','w4PDlXQSwow=','cAbCqi3DuSdKAMKkc8O3asKdw5XDm8O2w6rClVrDicOJ','w5d0wqk=','wpvDv8Ob','w5NwbWkEw6J1woLCjTs=','w4PCkcOEw5o=','w6HDhwE1woM=','w7V6wrQubCQp','w5jDvHwEwpB3','ZwzCrS3DrSpbNQ==','w6bDlnA=','wol8wpA=','YzElCcOh','w5l6worCiwt7wphcw6cKSkXDvMKyw4PCtQ==','HBoDAcKRwphewpPDm1fCmkXCl8KMwpIWwqfDqFbCnsOmwoJQw6FYbsOUwrDDpiVgwq1yNsKTbMOJwotNwoFzwqfCl8KCS8OEw5VJwrkCwrAGwo3DgWbCtcKjfcKEwqzCiMKp','NzbCjHo=','SwU6MF0=','wpsqFA==','wrt0w6dXBQ==','UCclJcOq','wotqWkM8','woVNwqHClQ==','wpbCp8OYw7LCmA==','dnFxdw==','TMOudMK6RQ==','wqZuwq4=','w73DpQQ5wrQ=','w4LCjMOkwqZlwq8Ewoo=','wppqw5o=','wqRoPg==','woZlw7o=','6L+Q5oyZ5LiI5YuF5pyx5Yma5ZmF77y76KyL5qCR5p6F576a57qi6YWu6K2x44KP','wqZaw4J8Pg==','WygJ','w7LDj0ACwpI=','woVLwpE=','B8KFwpvCssO0','w6HDqncRwqp+w6gH','XkbCmy1h','wpbCoMOUw6vChMOvw6A2wofCkTNswolNPxk=','wrzDtMKYw50Ww4B1ODU=','YcOeb8KQX8KP','w4R/ekV7wpkaF8KuNloxw6o=','Yi8PRsKn','wonDlcO0wpbCnsOvwp4WNw==','w7gUVsKZUg==','esOlU8K6ZQ==','w5PDiSotwoI=','fsK5aETDnAfCvA==','wpxsLg==','N1AtRA==','JsKKw5TComc=','SwUsKg==','wpVxO1Y5','bcKsw5gFFEjCkh8q','ch3CuCfDvg==','acKnDQ==','wqlSw50=','w4BPwq8RVg==','wpkGwo3Dq2k=','wqzDr8KEw4cXw4t5','wqEmAMObfA==','wpzDjBfCicKwHC8=','wpDDlMO3','w7vDg8K5wrd1wppT','wqEsEcORfsKnw7nCpcKp','wrwAOCfCvHsO','EDJywq3Dig==','wojDocKHw5Exw4l6MQ==','w5Zhe2kd','XyIMZMKHwrFGwrsEaQ==','R8Khw7LCj8OSw5loFMK+TA==','5ZOx5Y2T56my5bm1','w4fCnsOzwrVG','wqTDmMOQLxBpwp3CicKowonCmQ==','wroXwpPDo3Azw6XCg8O9w7M=','McOJwq9dw7E=','eFfChiRUNFp5','w4jDjcKlwqV3wpNCOjYa','CTHCi2orwp7Csw==','b1fCmSB1OEB3OcKR','56iP5Y2P5L6a576q','QsKOf8OZw7A=','wpM9HMOQYcKkw5/CusKmYk3DugE=','56ue5YyY6YCV5pqa5biH','XyIMZMKH','wqJPw6vDg2pLwpLDsg==','w4fDsBUgwoohTsKuVwc=','5b+c56qN6ICW5pmr','wpdTbUgh','w4fCnsOzwrVGwqsdwopiWA==','w4DDs8OswqDCnQ==','w69uwpbCiRJ3woNXwrVT','57uZ5Lmv5YyB6YaL','w7BpwoLCpBZ8wpI=','WMK1w4kBCVnCjxU2CA==','fMKOLMOuwovCjMO4wp7Dvy8=','5o6P6KKa5Y+e6Ye4','JMKiwoDCucO9bsKQABpi','5o6H6KCP6K+h5byZ','w6TDuGACwok=','woo1H8ORWsKqw6DCrw==','wqzDmsOiwpLClQ==','w69jwoXCkRpgwoQ=','TcK3dFbDng7CrcKVTcOi','5pij56es5LiM6Zmn','EFAtS8Ocw5cCw4nCk3s=','6KSu6IuJ5peg6YS8','w4jDjcKlwqV3','wr9Pw63Dk0l5wpDDs8OR','EsKOw4TCtm3Cg8KFeU3Dlw==','Bl4zRsOew4Uiw5XCkW0=','6ZiR5Y236Lea6Zm6','esOTScKaS8KeX3TCncKiBQ==','DjnCl28pwozCngVSbw==','wrPCrsOLw7rCgcOrw6o=','Vg7CqyXDti5bNcK3cg==','6Ziw5Yym5L2j6YKd','TXpuasK6ecKkJh/DvQ==','wpQ1AMOVY8K2w6TCr8K1cg==','5Z+357uQ5pag6ZSr','EsKOw4TCtm0=','w5PCisOBCDdNQzkvw4w=','5LmP5ZOCw6Hkv5Y=','wp/DocKYw5UV','wrI8wrPDtVw=','5LmQ5ZG75a625Y21','wqBiO1gm','CcOPwrvDu2/Cuygwd8KQ','6IKv5aSz56mNXHY=','6IKv5aSz56mN5a65','wpQ1AMOVYw==','wq3DghfCkcKdFy4=','wroXwpPDo3A=','w4TChsOywqBOwqM9wo58QA==','WMK1w4kBCQ==','HMOaKcKawrLCgnXCkQ==','6IKN5aSb5a2I5Y+e','wqJAwr3ClsO/w7TDoHQ=','XsOdd8KUUcKeWVLCjsKi','wqsAGjvCpHIZw4jDt0k8','AMKxwofCvQ==','w6TDuGACwol6w7EHSGk=','6I2E5Y+55o2F6KKI','w4XDsAkqwq4wX8Km','5o6Q6KGY5YaH5a66','5o+Q6KKj6YK55puZ5bqm','wrPDmMOPKys=','wq/DggvCm8KyFT4xT8Ka','5oG65bCM55iq5Ya2','wpt3w59cMTpyw5k8wo88','wogCwo/DjnQlw6U=','UcORwr1IWj09JkB/','Zgk2wpvCrA==','McOJwq9dw7HDm8OBwpFww4o=','wpYywo3Du0ZdJ8OSwr4Ow5s=','wovDkQzCnw==','NMObwrhOw5LDn8OYwpE=','w4fDsBQywpArSMKv','w6xuwpLCjTZ8wpFd','QcKjw6vCg8O0w5lAGg==','wqFEwrPCmMOOw6fDvw==','NCxywrfDihs=','w7UDc8K0V8OFN8OyJsOL','wrctwqXDtUVdFsO+wqoTw4o=','w4PCmsO5wqA=','dxgvN1o=','T8K3aFzDmgXCvg==','wqXCrsOew74=','UsKww6/CgcOaw5JUEMK3','wpMGOD8=','McOawr3DqjjDsXM9fMKQHTdsGcKPcmZqMsKEXsOzwqjCkldNWyzCu2jCuw4=','Rjc+PsOL','YMKgw48QXhPDlBg9CA8oKR7CjwRWQMOgw61Dw48dIcOhwrTCjB5KwqZtNMKBw7k=','wq8ywq/DsUg=','w5zDrWYTw54wwqoKQ2nDuEhEwrbCjcOVw4sQwrVtw4LDsw==','wr3CgMOLw48=','w6fDuGQGwrN6w6c=','wozDkxXCk8Kr','wpBCwrPCmQ==','w6PCn8OQw4s=','wrcuwqTDpn9ZOMO0','NCFhwq/Dgg==','dwswwpPCrlAiw6rCmx4=','w4xuwpLCjQ==','JMKrY8O7w7Y=','wpRTaUwfwrLCs8OAw4vCh8Kv','wpNEw78=','XcOdc8KQb8KOTlTCmcK0BA==','w5vCm8OJ','wo/DvMOw','woQMwpHDolk=','XXZjd8KgcMKVPTzDuTjCrw==','EsOcwq9Vw7LDmcOcwpJ7','wqtXcU44wq8=','cQ7Cqyo=','IcKAw5vCp3LCg8KCb2vDi8OsNTsWw5PDuQ==','ZcOVwqxGWig7JkF/BsOuezbCums9QMKKUQ==','djE0LcOaT8KWI0MdQcOnTXHDocOIRg==','w6fCnsOzwqdO','w4XCmsOnwqZOwr0BwqtxXw4=','ZsOzwq4=','w6fCnsOywqdcwqEbwos=','XMK2w7g=','6LSZ5Y+B5L635b2L6KKz5bGc77y46K+l6ICf57CZ5pav56iO5qGt6K+i6KWF5bC844Cm','wrzDsMKGw50M','wrIOJh3Ctw==','w7R4wrU3YBwvw6pJRA==','wp7DgA3ClcKt','w7bDjcK6wqE=','wq7Do8Kew5sK','WAEnwpHCr18Bw64=','wqU+E8OM','eV7CsyxU','bVPCjTN9Llw=','UA4sN0zCpW0=','wqU8wqzDsXhWM8O+','w6cSesKwasOOBcO8','GsOsBsKEwro=','XMKdL8O1wpo=','w5bCksOPwpNB','w4fCk8Ogwq1Owrwa','wqrDkx3Cm8KrFRo4XMKQcWw=','w6p/woDCiQt3wrlTwqpF','aQEqworCtEo=','w5JuwpzCpBp8wpBGwq8=','AsKiwp7CrcO1','woZiJUwu','wqJBw6jDhW1mwofDsg==','wrbDjcOywqXCvA==','wr41OcOXdA==','UcORwr1IWg==','wr0fwo/DpnIhw4g=','PsKSfsOgw4k=','GMOaN8KQwpk=','FcKGw5jCs2/CkcKo','wpjDqMKsw5s5','6LW/5Y2Z55qy5b+P5a2a5Z+25b2e5bmt77yJ6KyU56CU6K695a+A56Gw6YSv5pan55mz5b2Y44Ce','V2dnccKafcKdKg==','FsO8wpxrw6o=','GTnCiGsNwofCjA8=','wqdCwp1vQw==','w71XwqPCgxM=','K3kqfsO1','wqrDkBzCiMKREScx','QsKMM8OVwoo=','TVPCmjR9LkBfKsKSbMKSw4Elw4dZ','wq3DjsK4w5Me','XlXCny5q','XcKnw54SKl3ClhU=','w6Jpe20Tw7NSwozCjg==','wqFJwqDCgMOG','wrHDghTCn8KLCTox','wo8OJSfCpHgZw68=','YMOTwrtGRQ==','w5jDvGQGwog=','w5htwoPCrjY=','wqEnI8O7WQ==','w5FswrHCqTU=','bBILFsOg','AMKSasOzw5Q=','w6HDnMOQwq/Ckg==','w4PDnAsCwq8=','DHURZsOd','XcKeJMOGwqY=','XMK1cljDgRg=','GcKXe8Otw4TDlRg=','wp9tw6Bb','wrvDmsO9wpbCscOowpYc','ZDArwqrCtg==','w6plZ28Ew68=','wqJ7wrdhZlseCQ==','MMKJV8OBw5c=','w6Fuwr4tSw==','HcOIIMKDwoc=','agrCtyPDryM=','wrprw7ZBNQ==','wpInLA==','w78feMKySsOONMO6JMOEwr3DoA==','w6djfWEGw6ZPwoY=','UCARaMKHwrVcwrohc8OXBBUE','S8KIIsO/wprCs8OvwpPDvi8=','w4F0wrkwdg==','Aw9ywoPDqw==','U8Kxw6PClMOO','wpEKIQHCoHIZ','B8Kzwp7CscOzbg==','HMKIf8Omw5I=','wo/DnMOTLTJk','w4LDogIzwpQ=','wo9zwo5NYg==','w5fDs8OzwqTCuV8MGQ==','XsOYdMKRSA==','wqBFw73DhH96wp3DhsOZw7PDncKLw4o=','a8KILsOFwqg=','YcO+wqPChg9Bw4kY','wpU/wo7Dvlg=','Z8ONbg==','T8OuwqU=','woZ2wrV3Sg==','TxsewpPCqw==','bsKmal7Dhw==','wpd/wrx2SkYQ','wqXCrsOLw7rCmw==','U8KMecOXw6/DrA==','LcK6Y8Ofw4I=','wqB8w4BUEQ==','w4d9woY=','w7BywrQhaj0Rw4tZTx3CmMK9wpV2w7dtwqEFwpjCkgIyw4/CqsKmw4rCiGjCusKxw63DocOwworChcOWw7k=','w6VhZWQ=','FDVgwqY=','VsKuw6fCn8OYw4VV','wrV2wrt9Skc=','JsOaKMKU','w5DCnsOswrFiwqAPwoA=','wroawoDDu3gk','dsOzwq/CmiNd','T8OxwqfCly9Ow4M7dQs1w70=','woM1H8ORR8K9w7bCpQ==','XsOQZMKMWcKJ','wpIvwq7DslhUMA==','woTDqMONOjQ=','ScOdaMKQdcKVS1g=','eMKhw4gI','wqJMw7rDj397','woBTckwFwqnCtsOM','XsOQZMKMWcKJXg==','w4kQbcKmQQ==','w5IZdcKURg==','wos+wrvDp1M=','VgPCuD3Dvjk=','wrEAIjE=','wqBvKEAuwpc=','wojDicOxwprCjMO1','eMKnc17Dgxg=','w7d3wrs8YDg=','w4ISY8KhT8OFEcOdK8ONwrc=','w4TCisOeDBNGUTM=','wq5swotzRQ==','GcO1N8K3woA=','w69jwoXCkRpg','wokewoDDsHw1w6XCg8O9w47DhcOnZQ==','SsOlwqBfQw==','w6B6wrcgTCQow6k=','RS8wNcOLWA==','ZcK1w4MsAUrCnhw=','eFfChiRRM1J9','Q8Kiw6oXDg==','dxoPdsKm','b1rCijh9Lw==','wp/Dl8OxwoDCi8OPwpQ=','wrFgPVY5wpY=','GMOXJMKIwpHCnw==','wprDmsOzwpbCtsOnwp0W','DjTChHchwps=','VcKOYMOdw5TDsTDCog==','ZcKNw40NNQ==','w5ZsaHEVw7U=','w5FmwofCgxFzwppX','Kwl/wrjDoQ==','wrVBw7bDk1NnwpPDuQ==','GcKaaMO1w4w=','JMKvwpPCocO1ecKX','w4fDsBUgwoo=','EsOYwrFVw7/Dmw==','bnFsZMKgdA==','w7PDvsOxwrLClQ==','YsKAfsOM','w6waecK+','wqLDmsOJIyliwqHCm8KswoM=','EMKOw5jCvGnCiMKW','6L6c5o2K5Lq75Yun5pyN5YmH5ZqZ77yQ6K+I5qGk5p2L57yF57qU6Ya+5pWV55q25byo44KP','w5TDv8OHwqPCsg==','w6fDuncNwoFAw4gHVG/DgAVcwrvCgcOEw4A0w7Rjw4HCvcOSwpDDqFJ4w4Fbw5U=','wpzDghXClg==','wr0+wq7DuVxZO8O1wogUw5Arwq0K','fcOZccK9XcKVSVvCmcKj','w57CpMO5w6h8','bcOTaMKYXcKVSWXCncK/C8KiEMOE','w4VywrQh','DV0lecOD','wrNHwqXCisOR','U8Kxw6PClMOzw5ZLEA==','w5IWZMKwV8OWAsO/P8OF','wo5yw7JL','VMODwqpbeTkkJg==','VsKjw7TCh8OQ','w6EQY8K8TMOOLcOyJ8OF','cnV2aw==','C8Obw7JMw7DDi8OSwp1sw4rCh2jDtcK/ZsOgaMKROMO8T8O/wpRZwpfDqiUDdA==','wp/CvMKWw6vCmsO/w7Qswr3CjV5ewopNbEkJDMOew7TCkjA=','w45owpQybzk=','wp5UbmMa','RgI4K0jCu2A=','TwojK2TCuWHCssKKIQ==','EsKqwp7CvcO+asKJAA==','w7LDg8K+wqo=','EMO/L8Khwpg=','wpbDlMO5wp0=','woV3w7pd','cCAdLkU=','JUk2WcOFw4Elw5XCj2s=','Wk7CgjJsLmdrJcKB','w7LDqQ4ywpM3acKySxc=','w73DlMK+wrduwoVlJioK','woDDlsOQOjRpwpzCicKVwonCtcK1wq0MJ8KZ','w4x7wpbCgRF1wp5Uwr4=','UcO6wqzCsDJAw50Wdxgbw7RT','FyVnwofDmwzDkw==','wrE+wqTDulR1NMO/wr4aw5s9wp0NwrJ9XQ==','woB2OlE=','w6fDpBQp','fsK3als=','FDJ8wrrDgB3Dh1Bj','bcK6c1DDmgXCmsKfUsO8wpHDjD0=','wpV2wq9jRls7CcOYRcOgVsOG','w6fDuncNwoFNw6QMUXPDsQE=','CFg7T8Olw5Maw4c=','wrTCvMOSw5/Cl8O+w7I=','FC9gwro=','wrA9H8OR','csOYwqBcTj0=','ZcKtw4sBA1k=','ByF9wq3DigU=','wpgOOzHCjF4Fw7/Ds0gpcsKRDDvDkiVnbx/CpcOTBQnChcOpAhBmXA==','RiInKcOpS8K8Jw==','DTnCk2sTwozCiA==','w4XDocO7wrPCvlAHEw==','wqPDuMKgw6Qs','w6fCisOywrw=','wqfCqsOfw6nCk8O5w7sBwrLCihA=','XSYYd8KPwqdawpoXbsOY','w4LDt8OtwqTChGcLGsKiJQ==','w4XDtBQkwpMSW8KnUBE=','wr9mdmAA','eMOdwr5NWA==','XcOUasKCaMKaQVw=','Nyh8wrnDuwjDkks=','bF7ChDZMPFh5','wpoJwqjDnX0=','wrx9w71XETByw74=','w4LCssOl','HwHCsw==','FsOJwq9S','w4PDs8OowqTDkFULAsK2YMKAajAEw7/DiWk8','w6dqaHA=','wqNxw71Y','F8Kcw5PCpU7Ch8KceQ==','JSNnwqfDgAfDsEFrAA==','LMOCwpRfw7A=','w7tXwoY=','MFAsWcOGw50Ew4g=','wojDicKyw44/','ecKEL8OIwpQ=','w7MSYcKwasOOBcO8','JMKiwoHCq8OnZMKWAQ==','SiI1KMOnRMKhN1o=','PMOePcKF','woFgd0w1','VcKMC8OtwqU=','asK/YkPDmw==','UQbCtyDDtDx2','wqDDihfCisKqBA==','TcKGY8OIw6jDqw==','w4luwojCnRo=','fQouIWDCuHXCssKSdg==','QcKyw50vNg==','wqUOHiXCmA==','dX1md8K8','RDckPMOd','w4DDuAklwogzYw==','F8OSK8KBwoHCmSk=','a8Kjw77CqsOYw5lBAcK7','w5MHQ8KDUA==','6L+M5o2K5LiA5YqI5p6Q5Yi+5Zi+77+B6K2o5qKN5p+N576E57uD6YS46K6344Gg','w4EZdsKt','asO2wqDCiA==','w5ZYwqDChxo=','w4FcwqrDiip1w4PDqsKHw67CkcKSwow=','KsOewqXDs3Y=','wpLCqsONw5bCmcOkw6ct','wpx9w6dlJzN1w78=','wpjDhg3CssKwBTgn','QcO6wrrCsCNMw4AZdA4=','EsONwqlqw73DksOAwpE=','wrRXa38twqvCpcOG','YQrCrQDDuj9K','c8OswqvCkQhOw4IS','VAw9wrLCgw==','OcKUacOg','wrzCusOI','wrZdVQ==','R8OKwp3CjAk=','HsOPwqTDv0vCsDo6','ZnxQQsKF','6LyZ5o6X5LiS5YiZ5p2B5Ymm5Zq4776v6K2Z5qCi5pyu576357m16YSo6K+f44Gw','DMOdwqzDqEzCvzEw','IMKAw47Cn2XCj8KWdEs=','FsKYdcO5w4zDhgXCs1pqEMObSMOL','wrzDmsOSJyttwoHCnsKWwo/CmcKwwrEe','aR/CuCfDsj9W','EsONwql0w73DkMORwphnw4s=','IMKZwqrClMOo','w4t5WWkXw6I=','woHDkMOTLg==','wqDDgBbCl8KyESQwasKAenrCshw=','w6PDt8OqwonCkV8OGsKyMg==','WXvCpyNI','w4fCnsOmwrF+wr4=','woA0wq/DsA==','w4fDvGYrwoVxw6EOX2g=','wqJBw7zDk15mwoLDuA==','wp3DihfCng==','wrcxBsO8b8K9w7TCpsKicw==','U8OqwrbCth4=','w51mworCjA==','wq5Lwr3CjMOGw7TDonvCg8OAw404Y0I=','w4fDj8K4wql3wpdYOxMAw5TCk8Klw7A=','TFPChyR7KQ==','SCITYMK1wodLwq0Cf8OUPxMda0Jsw44CwqjDv8OX','wox0wrNwRlQUD8OPTQ==','w53Dt3sXwo1+w6kLQH8=','ATDCjGohwqvCng5T','VcKOYMOdw4LDjC/CvsOrKEXCinbCmwsWO0ceF2Nd','w5DDsAokwrgQX8KmVSttwo5ZIhTCtn7DhMOkPMOVVsOjHw==','woNmPX0uwpZcAsOxKxskwrPDkw==','w6PCgsOMw4tGbRDDr8Kn','wpE4wrXDkFRLIcO4wrEcw4omwq0T','wozDl8O/PzJ4woDClA==','YcO+wqPChhl7w4oaYCIjw7Rew4fDjlYiw5zDlcKHLmwMwok=','cQkpwp/Cnn0Ew6rChBrCksK5BsKdXcOVFcOSTS7DtQrDtMOeX8O/w6XCuQjDhw==','wrdAcF0jwrPCqcOTw4s=','bwHCsDDDli5CMsKgc8Ow','Rhorwo7CrkoVw7vCkw==','wq06G8OAQ8K2w73CqMKic1c=','QsK0Z0PDhwfCvMKCccOwwp3Dhw==','eMKmw5QUC0jCggAh','ecOxwq/CjiM=','UhklMUbConzCt8KD','wqY1BsOAYsK2w6LChMKmbEE=','EcOawrJIw7PDisOMwoRn','dcKnw7LCqMOcw5pD','dh3CtjDDtD9WIMKg','w6BtwoXCnAt+wpJAwolBewg=','wpVowrVwQEEBFsOQ','cHFxZsKmasKVGwDDuTvCuX/DnWY=','w4jDowIswogyX8KCSwRrwp8=','ccKOYcOU','OcKJdcOgw47DkxLCp2g=','fC04OMOHS8K9K1Qd','wo/DsMOsHT8=','O8OLKcKYwoA=','TcKGY8Ocw7jDpw==','wpjCrsOB','L8OdwoHDvVE=','wqDDjhzCl8K9FTgdU8KNcWY=','CSFrwozDjh3DikxjKBEEfHlzw7c=','wrcxBsOgZsKhw7/Cv8KgaQ==','acKyw7LCssOPw5ZIBsKjSHPDpBXDlg==','wpltIE0iwoREAsOlLw==','fQYvKEvCs3fCjsKIIFbDqQ==','w5zChcOSBD8=','ZjAobMKZwr1QwrIT','aA7CtCE=','wqPDmcOxwofCjMOqwpUBHXpkwoA=','w77CjMOXwr1YwqcLwoN1','wpfCrsONw6/CmsOvw6ELwrLCkxQ=','wqDDgBHCm8KtESkgWMKbWn/CsA4=','wq0nJMOdfcK6w7LCpsKi','wrwHNybCsnQfw67DpHQ4bcKR','KUIJQ8OCw5sUw4DChA==','KsKTe8Omw4DDhB/Csn9KEMObQsOE','wq5GwrPCjMOO','wq5LwrrCgMOZw7TDr2vCscObw609YVA=','w5R+wq4LZCcr','XMKIN8Oewo/CrsOtwp7DrzjCiV0WUw==','wozDhg3Cs8KyES0x','MEMwXsOew4YPw5zChA==','woR5wq5rXQ==','w40Sb8KXQsOUF8O/L8OtwrfDusOOw7nCqcKN','YHV2d8K4ecK9Kh/DujDCuGo=','wqrCosOcw7bClMOvw6EMwr3CmhR1','VwswwpXCsw==','w55gwozCrjc=','b1fCmSB1','w6Z4wq4qdzk=','w5zCisOQHTVaRA==','CCV9wqnDmwE=','F8OWIMKcwpbCiGnCrMKFw4lhbw==','fsOOasKBU8KPVEfCmQ==','wrYcBjjCsm4kw6XDulM3ZQ==','w5zChsOWBDhNRRUzw5vCixM=','aw7CoQbDuj9bPMKgTMOmfsK5w5/DhcOp','w5Z8wrLCgQx7wpVewqI=','T8K1w5YFLVLCnR8=','w4jClsOvwrBOwrY=','OMOJKsKFwpvCmWLClcKO','wrYxFMOGa8Kgw7jCi8KkdUvDvA==','TcKBbMOVw7g=','w5PClsO3wphG','acKAa2fDkg==','wpgOOzHCmnkNw6Q=','BsOMwqjDrnbCsjknS8KCF3w=','w5TCkcOOw5pgdw/DsA==','wqPDksO+wpfCncO+','YFXCgyBqPFdmLsKQcMKWw4Av','M8Kiwp/CvcOZZcKCCg==','F8OSK8KVwpHClQ==','aMKMLsO5wqfCtMO/wp0=','wpsGIBjCvg==','MsKbw57CuGg=','wphbcU0pwr8=','BsONwqHDu3DCvz8hYMKRM3diFMKa','wrvDig/CtsKy','HcKGw5jCs2XCng==','HsOjwoHDnE8=','YFjCiix9','wq9gIVg5woRLH8O6OCEswrHDmA==','TFPCnw95MFE=','w5luaGUV','w6PDt8OqwoPCkUUeGsKyMsK6ZDJB','wps2E8OAesK/w7XCuMKJYEnDqw==','cXF2SsK5fcKXKg==','w4zCk8OLw55beArDq8Knck/Cn0tE','woFawr3ClcOEw6HDtW/CsQ==','wqofMjXCp3I=','G8K8SA==','woPDpSMfw5ZjwrEeAmbCrBoYwqLDlcOMwpU=','UcOVa8KRWcKD','a8Kzw67Cq8Of','wq9uLFQpwoBaIsOxLgo1','TwoyB0jConHCq8KDCVbDvGLDgkrCiA==','w69sXlkW','wp3Di8Ogwp/CgQ==','wq1Jw7XDkn9x','VcO6wrrCrilZw4okYBg1w7U=','wq/DggvCm8Ky','GcKiworCmsOxf8KQCQ1cb3FHwpZkw48=','wo/DnsOkwqDCjMOjwoAyPXJkwoA=','YMK1w4gzEFnCizEqEgVj','w4xqwpDCqhN3wplWwopPcgg=','wqY4F8OaasKew7/CrsKi','csOVwrt9RTknMEJtMsO5ei8=','w69zXXoRw6lIwpPCgSwSDAA=','dQrCrQDDsjlKM8KxaMOsfcKdw5PDjw==','wptTw5/Dn2hswpbDosOcw73DisKow5HCnMKBwqY=','wp9qw7xHKSt5w6o4','w5xuwojChA==','wozDhg3CrcK+HCEVU8KAeXs=','woXCvcOWw6/CmcO+w6o1wrY=','XF7CijJ9HlxzOcKDXcKDw4g4','wot9w79HJwZGw6gywpE=','CcOPwrvDu28=','GDfCiWIrwp7CvhlQeQ==','wr1lw7Y=','PBpf','KC9ywqrDuAzDnA==','wpRTaUwFwqnCtsOM','woTDlsOJJQ==','XsOdd8KUUQ==','w4PDs8OowqTCuV8MGQ==','TXpuesKCfcKcOhc=','w6TDtBMXwoYoT8Ku','acOxwqLCmhBOw4MCdQ==','fMKMNcO5wqfCtMO/wp0=','w6LDv1APwqc=','w4HClcOFw41MagHDm8KjdGA=','ScO0wpfCoQc=','S01uRMK2','TVfChSV3MA==','wo8OJDXCvg==','DSzChGAgwrvCixRF','wrXDmMOtJxw=','w6fDrXMNwoBNw6QWXw==','w73DvcOowqTCo0UYF8K+J8KccQ==','wowawo7DrW8=','wq3DgsO9wrvCtw==','dA7CtyDDtCY=','DjnCl28p','JMKiwoDCucO9','wqwXwpPDo2o3w6g=','UQIDwoDCrA==','RAs4','wprDmsO0wpbCt8OzwoQyP3c=','YcKtw7LCiQ==','w4PCkcORw55E','wpABOi3ChXYHw77Dsw==','eQYowoPCl18Aw77Ckw==','wpFUXUUP','T8KzYEXDlhjCscK0XsOlwpE=','wqbDqAzCt8KG','RHVwYsKjfcKJ','b8ORwp4=','wqlTTg==','wo/DlsOeKzJp','MTN2wrzDoQjDk0U=','wqomOsOHfg==','GcKswoTCvcODf8KWBAF2Ymg=','wqpZw4/DhnM=','eCwnKcO9XsKjI0cfZsOx','woJRb1wH','wrBDXFED','w40cYcKwcMOUEcOyI8OHwrrDow==','agDCvg==','wpTDmMOPJA==','ACVxwrvDiA==','w7HDgsKxwqs=','wrodJDvCoQ==','woRxKFou','QcO1wonCmSs=','woBFw7rDmldmwoPDs8Omw6LDgcKLw5w=','wrQmHcOAYcKnw6nCusKi','wr90w7JKIy1Nw7Urwpk=','BSFr','wofDnMORPidUwqnCiMKuwos=','SCITYMKjwrpUwrE=','w7PDtAs1woYdfMK5Shk=','JMKUbMOxw6XDjgrCsGJtH8OTS8OF','JVs6R8Oo','awDCryHDiD9dMcKsZsOrZw==','fMKXC8O3wqM=','6L6/5oyN5LuQ5YiM5p6R5Yif5Zud77+46K+15qG55p2N572657mF6YSb6K+D44KA','wpE4wrXDgFlKOsOkwrgV','O8OeMcK8wpvCm37CtsKbw4hhcw==','wo1fwp1hRQ==','JsOJwrBZw5XDkMOTwps=','fcOZccK7XcKWSA==','w4xqwpDCpw9zwpRbwrNZ','w7PCmcOcHTVcTiw4','T8OxwqfCly9Ow4Meahg=','wozDicOJDClgwoPClcK2woPChcKn','ecO4wq/Cly5Kw50efho=','fQ8rMUg=','w5zCj8OSHTs=','wr/DtcKZw5w=','w6TDtA4=','woN3w7RaKA==','wqDChsOUw5/CsQ==','b8OWwo/CrTI=','wq9gJlQmwoRGD8OIIwEpwrPDig==','w6DClcOXw7dIdw3Ds8Kncg==','wqFEb0g7','w7XDuAkl','KsOLwr3DkmPCsDg5YMKR','bMKqw6vCkQ==','woBdV0YhwqI=','wq/DlsOaIyhPwo7ClMKiwoPCmw==','QAIkIQ==','HcKIw5fCo2jCg8KDdVHDgw==','wqI7AMOxb8Kww7g=','ZBAGbsKP','wqwcwoM=','YXZsTMKT','NUE7S8OFw5c=','Ywg+KlvCpQ==','w7d3wrs8YDg9','w5duwqkt','acKmw54tC0rCkh4j','c8Kyw6LCh8OJw5JrGsKlTA==','woJSw7TDgnV9wozDpsOQ','wpctwqXDtUVdGMO+wqkY','w4nDkMOm','wofChcOVw5nCsg==','wq1Ew7rDgns=','VH1AZcK5','wqrDt8KQ','USoXwo/CiQ==','OsKaw7DCvU4=','CMKRUQ==','eVnChy13KmBrO8KH','M8KoW8O/w5Y=','wp7DjzY=','WCQY','wq9nKE0q','IMKTPMOEwqnDo2XCscKqMR7CqS8=','wpDDicORIzI=','QQQkNkbCumA=','WMKMMcOy','wqg7FQ==','NzbCg2E=','ZcOVwq1cUA==','ccKAY8OLw7LDszM=','YXtscMK7cMKV','ZMOIwqxMRywgLFw=','wp1uD0wD','w4jDtQY1woY=','U1LCoQ==','w5vDmGoTwpA=','HcKLw5fCo2E=','w6VoaHsVw4RTwoLCkj8UFhE0','wo8ywrfDsXVRNMO2wrATw58jwq4E','fCopVcK/','wqUWEcOBVA==','WBIEQMKf','woJ3w6VWFStyw7s0wpsxw6I=','OsKSTcOEw7Q=','MjfChm8wwow=','TQUIMF3ComrCqQ==','w4EgdA==','XFrCjiBq','asOOZMKCdcKVS1g=','wpswE8OAbw==','w53DqkIPwoVmw4oMVnPDsQM=','wqBvKEAuwpdlBMOpLw==','Wwk8wrjCoEoYw6fCkzbClMKgAcKKbcOH','acKCL8OwwoHCrcONwovDui8=','wr05wqDDoFA=','cMKJIsOowo8=','wptTw4vDmntwwrrDuMOZw7vDisKL','wpzDixjCicK6MyI1T8KId2rCuBk=','VsOtwqHClylbw5YHdQ==','worDisO3Pyt8wobClMKm','McOZwrrDr20=','AMKqwoTCjsO4','5pyH6K6/5LuM5Yqk6K+U55SQ5omi5L2w5oKw','wprDs8KPw4Y2w4ZxOw==','MzbChF8i','wqPDn8OxwofCmQ==','Tg4kIl3Cvg==','wpMNwpnDk30=','w6BuwofCnBBgwoQ=','fsKKY8Ofw6nDtw==','YFLCijV5','wqAxHsOAb8KKw5bCuMKobA==','w7XCn8ORw7pIegE=','wphfekQuwqLCosOqw4DChsKuwp4=','fcOZccKxVcKJSFTCiMK4D8Kl','Z8OZwqNdUio=','w77DojEowpQtWMKnQA==','ZsKhCMONwos=','HcKCw5PCumLCg8KDVVHDgMOLLA==','cMK3fnXDkh/CrcKcWsOcwpXDjztew7gu','c8KdaMO/w7zDqz7CqMOtKEw=','wpXDiMOdwpzCjsOvwp4U','woBxJk0kwpFRG8O6','wpoZwpI=','wowMMzrCtkgpw6rDpV8GacKaESrDiRt7ahDCpw==','YcK6w5IUDV3Clxk+Hg==','HcKow7TConTCksKeckw=','VcKhw6PCiMOYw6hkFMKgTF7DqBXDi8K7IlLCnRPDmlw=','ZcKjw6rCig==','F8KxwpfCucOkbsKmEBxlZXI=','ZMKrw6jCgg==','w5ABeMKhTMOUGsOjLw==','w4TDq30Xwotrw7wSXw==','wpZuwrt2Ww==','KsKXf8O1w5M=','wrZ5wr9qSmo6B8OGTcOeS8OWwoNzcQ==','wpFBw7fDmg==','bcKkaUPDnB/CoMKAWg==','wr3DpcKHw5sOw4JbHDnDo8KxUcKy','H3YdX8OFw4YZw4LCkg==','w7DDkxI1wpMrVMKYRgZ3wptZ','wq/DmMO1wp3CncOZwrISIH5WwpBqIEdMw4o=','w5Jrwr4kcS8=','worDkx3Cm8KrFQ==','w4LDsGEKwoZ2w6kLTmPDvA5PwrDCh8OV','wqrDjcKsw6MU','w7nDocOTwq7CklgGE8KTJcKCbDxB','MsOJwqtZw5XDkMOTwps=','wqnDiMO1woHCtsOnwp0W','w7PDusO7wqLCm2EGF8KuJcKGSTBHw7zDlGdyCw==','csOVwrtcRxYsNHVtLcO5','w5UDc8K0V8OFJ8O8KcOVwr/DssOCw6jCj8KXacKAAQ==','w4vDj8Kywqp/wql0MCsdw6XCm8Klw6Zqwr0lw4UmLMKDBHfCriBJDA==','fsKAbMOcw47DpiXCucO6IGHCuH7CkgcR','X8KiaGTDlgXCvQ==','ZMOrwqDCsC9Vw4o=','wrNcwrzCqMOFw6XDuWs=','woXCpMOcw7TCnQ==','wqUrJizChA==','wqYgHMO4Z8Kgw6Q=','InMU','wqPDr8KLw5Aow45/KjnDpcKg','TcK3dFbDng==','w7XCi8OvwphCwr0d','YMKKYMOXw6vDuhXCpcO2IUw=','aRwlwpbCqkkFw6XCkhTChg==','LFAsXsOiw5sMw4k=','TsK/fFI=','W8K3w54OAWPCthE0JAt0JhfClgRrV8K8w6dZwoUDMcOh','w6VybGkEw6JowpPCkjcDBwcjwqE=','w6fDuncNwoFAw4gDSkXDvBRLwr/ClMOVw7YHw6lnw5jCucOPwpHDiw==','Y3BmQMK8dcKcKw==','eSIiOMO9Q8KrJw==','VcKrw7zCgw==','w4TCnMOkwrpOwpEkwo5gdBpjZnzDvcKW','w7bCm8OXCC5N','X8KfLMOowoHCrsOgwoLDrw==','N8Kfw5LCtnTCgw==','wpzDo8KPw5odw7hRPzzDiMKwTsK4w5vCvmg=','diI9IA==','RiInKcOnRMK3LQ==','woXCusOKw7M=','ag7CqjDDiCJVNQ==','wqNqM1w=','fSYTasKcwrFxwrYfdsOd','wq1Uw7rDmnF+wpzDuMORw73Dkw==','YyINccK5wr1Iwrs=','VQbCoyE=','TsK1Y1nDljTClMKRT8OOwoPDljhJw74=','cmZtd8K7aMKJPxc=','wpxsw7JBMg==','wpPDi8OSPil4wpbCisKk','B8K3wpPCqsOk','I0M6S8OFw5cxw67ClHzDnlLDkcOA','VcOCwqZOUD07JlY=','w4R6wrYp','BMKxwp3CrMO/f8KdFQ0=','wrrDrcO4','UcObR8KBUsKI','w7TDicK5wqNuwp4=','w4HDl8O1','w5nDisOWwqfCnQ==','T8OMwp3CgQ8=','eMKsb8OXw6c=','wrxfwpNyaw==','AT/Cp3oqwpo=','H1IwRMOXw5sEw4HCtmHDhFnDkMOE','wqAxE8OXesK6w6bCq8KzZA==','wq9gJlctwoxaBsOIIwEpwrPDig==','w5IWesK6VcOFIMO7I8OMwrY=','wrB7w7FUIgxww6g0wog8','woJ1wq5r','dsO+wrzCgis=','w5zCnsOPw4Z/eAXDqsKn','wpE4wrXDglBUIMO0','cFjChzhOPFhnLg==','XcOdc8KQdcKVS1g=','wq7DnsO2woHCncO1wpg3Mm9o','ZR3CvCXDry4=','wq8dOSDCvGMSw7vDsw==','ccOCwqBdWCwwM1c=','CMOGwrRIw7XDn8OZwp14w5w=','MsKdw5nCo2/CksKIbFo=','w7rCnsOKw4tAeAXDtsK4ZQ==','bcOdacKZ','ZTE+OMOBXsKoMks=','woZowr9lW1A=','woEvwqTDtUVdAsO4wrEZw5E4wo4cwr5rRw==','woEvwqTDtUVdFsO+wrIQw58hwqYqwq5gUUfChA==','LcOLwrHDrg==','w5rDql0CwrQ=','dsOZwqtdXw==','Qio/KMOBXcKI','WQbCtzTDrj8=','w5nDuGovwoFxw6IWUg==','w7BhZX0V','w5lhbWw5w6lLwpbClGw=','wpV7wql3WFoKAg==','EsKOwrbCjsO7','WMKEJ8OowoY=','eMKELcO4woHCrcOA','w40Sb8KZRsOOBMOnIg==','w4fDhcK5wrRvwoIE','wphbwp/CjsOJw7zDoHrCkMOMw5U1b1A=','wrwdMzXCp3I+w7jDs0gQbsKSFw==','bcOOYMKUSMKeb1bCn8K6B8K5EcOWw4rDqQ==','SsOwwq/ChxJGw5sbdUw=','cgbCrSjDvnphMcKoZA==','w4zCksOCw5xCShnDrcKrdGTDjA==','SsOwwq/ChxJGw5sbdU8=','SiEwL8OFecKhMEcMa8K0','woLDncOZCS5lwoPCng==','QRkvJF3Cs0bCqMKLKVLDv2TDsFHClcKnQzM=','w47Dn0rChsOtDH4oDQ==','wq4aLj3Csng=','XwvCtA==','w6bCpcO2','EcO+Ew==','XmLChQ==','ZSsTcg==','w5DDtmUNwohww6QG','eyYmK8OPR8K0','wpUfwofDgWc=','w6vDnMK7wq1u','w7nDiMKzwpNzwphSMDM=','wq9vJl4iwot/AsOxLgA6','wpDDo8KFw5kVw4ZyOhvDvsKrWsKzw40=','VsOZwqFNWC8GM1NvKcOobQ==','fQglKETCt2vCo8KxLV3DtW/DkA==','w7/DtA4mwo8w','WQzCtinDtipBNMKSaMOtd8K0w40=','eSw2JcOA','OMKswpXCscO+RMKP','wo1xw71X','w6vDicKjwox7wphSMyEb','wrbCi8Oww7/Chw==','BMOUIsKYwprCrnrCi8KIw4ho','O8OHwqfDvg==','woZKAl8+','ZQkywp/CiFAKw6Q=','w5BXY0E/','TcKMYsOVw7DDvjjCqcOIJEbCsXDCgg==','TFPCnwl5M1B+LsKQ','YsOTYsKcUg==','w6wccMK8TcOvCA==','w5MWY8KdQsOOB8O/L8OS','HQt1wrrDnQ==','SCw2asKHwrE=','wq1Dw7TDm3dowpvDssOiw7vDisKKw5fCkw==','A3UWTsOA','OMKswpXCscO+SMKFCwt0Zg==','Y8OZwqFN','VQkqwrLCoE0uw77Cgg/CnsKj','w5lnS3wew7Q=','aMKMLsO5wrjCv8OrwoHDoyXCqQ==','a8K8w54DD2zClwUjEgZ1BR/CjgQ=','wqtHwqbChMOj','wqt3w6RdKjBhw74=','wphRcEQhwqbCvsOHw7nCi8KlwoLDhsKS','LT3CkUYlwofCjgxFbg==','c2F6asK1cw==','TntlasK6X8KRIRHDvTk=','asK9w5UE','EsKcw4/Co1c=','w4XCqcOmwpVe','wqJVLng+','XsOTwqBEWjknJ2VlLsO4eyw=','agDCuCDDvCpCNQ==','QsOTZMKRe8KaQFI=','bSoQYQ==','WcKhw6nCi8OQw5ZIEcKEQG/DpRTDlQ==','csOVwrthVjYtL1d+','VcKuSlDDkg==','w4zCl8Ohw4tHag==','w7DCgsOGw55dfC7DncK2bg==','w4fDi8KVwrB0woU=','w6jDnsK4wrB1woJPLyE=','K8Kvwp3Cv8O5ZcKzDAZ1ZWs=','w7/DosO/wqLCmUUT','dQIkIUbCoUrCt8KHJ1rDpXk=','w4zCnMOMw5hAdz7DtsKsZG7CiQ==','w5x9woHCiQt3wrRdwqlGfx/CscOPw7bDssOTSCw=','wrLDsS7Cs8KS','WcOVa8KRU8KMdA==','UcOfasKbWsKSX1rCq8K4DsKvEcOU','bsOAwq5KXiww','woAMOTrCtX4Zw6bDgVM3ZMKbDw==','w7kYb8KMUg==','w48dVMK6TcOGCsOhJ8Ovwrk=','OyN8wqDDiQDDjE1RDBoNcWs=','TQUJKkfCsGzCtcKLB1LDv2PDglQ=','XXdtbcKydcKCIiXDsTvCrnbDjw==','w7PDoMO7wqDChFQpGcK5JsKddzJmw7rDhA==','ATvCh2kgwrrCmhJJaAU=','JcKUe8Oww7HDjgjCo3hxGw==','wqzDlMOgwobCiMOEwpcX','w78QdcKyR8OzE8OhI8OUwrc=','w73DvcOowqQ=','bMOTwqLCpQw=','wqDDgBbClMK5GTg5asKAenrCshw=','w6fCjcOuwqBEwroQwp91','w47DlHM=','cD83','QQ0mwqnCtVEew6rCkR7CusKoGg==','I14yR8Oew5w=','MsKOw4TCpGU=','LMOeJsKewpnCnWnCgMKYw55CZRfCnsKDw4XCk1Bww4E=','QcOuwqrCkgs=','woJrwr51Yg==','H1YdXsOfw4E=','wqU8wqzDsW5rLMOiwqsYw5MQwqsTwq56XEnCn1Emfw==','wpJJwr7CjQ==','RcOmX8K9Zg==','woNiPGof','ATHCi34xwp0=','wonDghXCj8K6','PcOIIMKD','F8OJwrFJw7k=','EcOJwq5P','AcKHw6U=','w7IoQcOIO8KvwqTCtsO1fRXDskg=','wpZqwrZtWw==','w4kAVMK0TcODBsO/JsOFwrY=','G8KZw4bCrno=','QMOIYsKzRQ==','w6vDt3MOwoE=','w7IxdMK9SQ==','woF5woLCuMOF','OcKqSsONw48=','fQclIkDCuFLCrsKIIFzDpg==','fMK1cl7DhQrCrcKV','Ug0lwpnCtVcaw6rCgh4=','cMKELcOswpvCrg==','TcKDYsOfw7TDsQHCpMOxKUfCog==','wqNXfko4wq7CpsOCw5rChw==','Jl48X8OC','YMKTw5kYKQ==','w4VoWg==','R8O8wrrCijBOw5sS','wrJ/wrh3RkEdVMO0eMOK','amB2c8KnJsOfYBDDuibDpHPDkHlKworCmsO3wr3Csk0=','wp9zLFcOwp1cDsOtJA4h','IMOPMcKBwofDlzTDisKJw493ORLCm8Ksw4HCjlJowpbCr8Kg','S8KGw6wkBg==','GMOKMMKewoY=','wpbDicOZKzJp','PTnCiWI=','LirCinorwp3CkxBF','wql1wr1tQXoT','wo1zw5fClE5uw4FOw7dcLhHDqMOkwqjDoMKC','XlXCnyhuPEB3','wq5awrfCjMOEw6PDqVbCusOZw5Yo','wqRbwrfCk8Olw7TDoXo=','VAomMEw=','dMOWwq19RA==','K8Kgwp3CtcO9asKKAT94ZHhKwoQ=','VQQrwonCpA==','wqrCvcOcw7bCmcO8w7YMwr3CjgR5w4g=','wpbCoMOLw7/CmcO8w7I=','QcKbbMOMw6jDrBTCrMOt','w4TCi8OgwqBewr0rwo5i','NjHCgWs=','wovDkMOZLw==','WCYcdsKDwqBXw6w3SsOy','J8OLIMKf','JsKLf8O6w6TDnx/Csn9tH8OT','TsOrwrrCkzUVwoBYch8jwr9Aw6vDhkA4w5LClcKFNCo=','wqzDrMKFw4cd','cgczwpTCrVENw68=','bcO6Sg==','w77DoiouwoUtVsKuYRFowoJOAw==','XMKtw7HCpcOs','w5bDhMKUwoV+','UnHChDtK','w4p6wq4mbQ==','wox3w71HIzF0w6k=','w4tqwpzCnDx9wptdwrU=','PcKeYsOgw6LDiAfCuH8=','wqzDr8KEw4Adw4loLQ==','w5t9woXCnyt3wo9G','CCl9wqvDpwzDl0duEQ==','O8KeacOxw5XDsw7Cr3lAEcOTSMOO','diw/OMOLRMKlMQ==','w4QBdsKid8OFG8On','wpBXfVolwrPCtcKRw6/CssKA','UiI8KcOiQ8K/KQ==','wq0XwozDp1E/w7/CjQ==','wrAfMzo=','BcKOw5vCskzCj8Kfdw==','wr/CgMOY','wozDl8OxworCt8Ot','RQwf','I1AzRg==','w6HClcOCw5Nkdh/DusKRcGTCm0I=','XMKKO8OzwoE=','DMOJwqV+w73DisOBwphnw7TDjUbDu8KzZMOx','J8OLJMKSwp3CmWI=','wpZ/wq5GQ1AWAsO4R8OlXQ==','IMKDw5PCuWTCq8KeeFo=','w6DClcOXw6xdfBnDnsKsaWzCmw==','wovDmMOOGTJpwp/Cu8Kvwo/CmsKx','McKKw4LCk2nClMKUf0vDjcOBOg4awp0=','w7rCg8Onw5ZbfArDq8Krb2/CuE9Zw47CoA==','wrRXa30+wqbCvsOQw57Cg8K5woPDh8KR','W8OMYcKUSMKefVvCncKoBcK5','b3hyTcKC','WcKvw6PCi8Ofw5JUPMK9TWTDuQ==','YiIGR8KLwqBGwrITV8OcDRgWcEU=','w6XCmsOnwqZOwr0Bwq5zXwBh','wqDCv8Odw7rCgsOvw4MpwrLChxR/','w4F6wr4gSj86w4dQTQ==','wrV7wqhlQg==','wr1Ow7fDj0xowpnDo8OQ','dcKnw7LCsMOcw5tTEA==','bcO7wqXCpCk=','wqltw6E=','wr19w7VBIyxow548wog4','5oKS5aSO6K+55b+/','wosSwoXDgXI7w7zCh8Ohw6Q=','5YSJ5pSY5Lmn55Sj','fMKKesOfw7zDsjM=','JsKKw4XCsmzCg8KSaA==','JFQ+ScOFw5sAw43ClW0=','wpbCvcOcw7rCgsOvw5Aqwr3CmBh/wpdqbkk=','wp5YwrfCjw==','TcKMYsOWw7vDtiTCoMOIJEbCsXDCgg==','KsOLwr3DnmPCqj0=','56KE5a+s5YaJ5pSy5ril5omZ5pav5b+l5bW95p6J5a+85qOp','wrzDmsOSJCBlwp3Cl8KWwo/CmcKwwrEe','TsKON8O1wpjCu8Otwpc=','woNmJVwowpE=','w5/Cn8OCw5tueATDug==','w4V/eUV5wpkeF8KsNl4xw6g=','w7DCm8OfAC4=','wq8EwpfDm14=','wrMKODPCp38=','w78SdMKhTMOSEA==','eSY/K8OaQg==','w4/DtsO/wrXCkQ==','dsKKYcOMw7zDhhDCv8OwIA==','KkQyWg==','wqNNwrTCk8OOw6bDpFvCtcOdw4I=','wofCqsOVw7TCl8Ouw54kwqPCtxdYwopMaFkECQ==','QcOaY8K5VcKVSA==','FyVnwpjDjgXDi0U=','w4DCoMOyw7RR','w5vCkMOgwrB8wqsL','w5/DvMOywrjCplAGA8Ky','QcOSacKMasKaQULCmQ==','JMKOw5LCsk/Ck8KFXVPDiA==','Ay9nwqE=','wq8lPALChw==','w5NScEsH','W8O0TsKNVA==','w7VwZWEE','wp7DgA3Ck8KpET4x','TMKfJsO9wprCv8Oawp3DpCzCrk4WdMKIaw==','aQsrwpTCp1cew6bCoRLCn8KpDMKY','wozDhg3CvsK+BCs=','w7nDjXgBwqg=','w7h4wrUrYyM8w6trSAbCv8K9wo8=','e8Kxw5cFB0g=','XsOTwqBHUTE7LmVlLsO4eyw=','w6tywrQu','wrXDgcO2wp/CjA==','fcOZccKjXcKXWFI=','w7/DicKjwoJvwppaBiEIw4g=','PcKSwoPCrMOn','woJNwqbCt8OKw7nDuXo=','wqjDpcKew7AZw5N5','w7fDt8OqwonCn0QYBQ==','wpvDnsOkwr7CkcOowoUHNmg=','LT3CkVglwoXCnwU=','6Lya5o+x5Lm/5YqE5pyD5Ymy5Zu477+G6Kyw5qOB5p+B57+f57qk6YSA6K+J44CK','G8KZw77Cv3c=','GsOeI8KDwpHCnnPCocKKw5ll','ayYfZsKewr1Ewr8Cfw==','wphRcEcqwq7CosOOw7nCi8KlwoLDhsKS','w7DCnMOMw4xM','YcKKecOuw7zDsyPCqA==','w5jDvws4wrElVsK+QA==','wrYxH8ObeMK2w5PCosKubUA=','KcOcwqbDrm3CqiUlYA==','Fhd4ORjCqjXCu8OV','wrzDpcKGw5Ebw5M=','UCAcYsKOwodCwqwfbsOc','WcKhw6nCiMObw55UGMKEQG/DpRTDlQ==','wpzDjxbCicK6','w4zCk8OMw5JEeAfDu8KVaW/CmklW','ZcOVwq5KQzE/IkZp','w7RlZWcRw6N2woLCkBcRNwQiwrR0bRs=','w4MSecK2RsOM','wolPw5BLIg==','UMO6wrzCkC9Aw4E+dA==','L8OLwrvDqWvCsTIcYQ==','wrvDg8O+','wq0KJTHCoWEOw5/DpFs3c8KSHSw=','woJ5w6N6Ig==','MMKKw4fComXClcKFUV7DlMO8MSQcwoTCqQ==','w4LCvcOzwoBR','wps3HcOaaMK6w6LCp8KQaErDqhfCug==','woFFw6/DvntnwpHDusOQw6A=','wr0GODA=','wpJsMW4iwoFcAw==','acKww583DVLCnx8z','w6BswovChhl7woVfwpBJeAnCs8Ov','F8OYKsKfwpLChGnCiMK8w4RqcxfChA==','wrTDkMOTLil7wrY=','QsK1aVnDlQLCq8KdaMO4wp7DhjZM','XMKIN8OUwo/CtMO9wp7Drzg=','bXpBbMK6esKZPR/Dlz4=','K8Kgwp3CtsO2YsKWCD94ZHhKwoQ=','w69uwpbCiRI=','w499wovCnBBmwo5CwqI=','OMKswpXCscO+WcKBER1jZA==','55au5oi35ZCE5pyi5aCb5YSe77y36KyU6YSt5peu6L+k5YSH7721','wo8DHg==','eHBy','5a+J56GC6ZWn6K+q5LuO5q+15Ymb5LuX5q2X5pyg5L6j','w7rDusOzwrY=','eMK1w4gTE1PCiRQ=','H8KRccOgw5/CpsKqwo7CsjbDs0BOSsOYc8O3','TEbChyhs','worDsArCs8Kd','wqVzXE47','w59Uwo4/dg==','WsOPQcKwaQ==','w6HCu8O2','wqhrw7FADQ==','w6x/wpXCgzk=','5ayB56Cz6ZSG6K+G6K6756Kp6K205a2l56Kv5YWv6Kye','w7DDuQ4NwqA=','dsKxw6vCnsOz','Tl5MZcKY','RcO5wqtteg==','wpskLj3Cog==','esOZfcKB','w6fDsBQywpArSMKv','w45YeA==','WgcjwpPCrw==','SMOMScKWdA==','6Ly05o+T5Lus5Yqy5pyQ5Yu35Zu577+q6K245qGA5p2B57+b57uc6YSS6Kye44Cs','b8OWZMKN','wrDDksO+wpg=','QDA0PsOgS8K8Jw==','Ywg+LEbCuEvCpsKLIQ==','M8K8w6PCmWU=','ecKOLMO/woI=','SsKzw7AOMw==','w5jDkBMSwoE=','bMOywqVIdg==','5pyi6K+Q5LiS5YiZ6K6p55ev5oun5L2C5oOB','w6p8woHCmjFzwppX','bMOfwrlMZCw7IltrKMOo','wrVbworCgMO5','ZhgSJHs=','w5TDkB0Pwq0=','w4vCnMOFw4Z9','wqrDmMKDw4AM','wrbDtDbCisKl','wr4MIj3CpXYfw64=','wrp5wrVpQlQWAsOiQcOvXMONwpU=','wrzDkMOTOjN4','YMKtw6XCk8OO','QcKDw7QQHg==','wr16W18o','UcOVa8KFScKPHw==','H10wTcOYw5whw4XCj2zDhUo=','aQsrwpfCrF8Cw6/CoRLCn8KpDMKY','KcOYMcKYwoLCjG/CgA==','OcOEwrtFw4g=','CyvCgHwKwojChwU=','w7oiYsKYRA==','RQYi','wrtiw4rDsEA=','5a6E56Ou6Zav6Ky45Zub5q+H5YqY5Lic5qye5p6e5L60','woJYwr7CiMOf','BsK1w7HCklE=','VsKafsOvw6U=','ZcKGacOMw7U=','w5fDq3cCwpB6','woZ7wrZo','YsOCwqpIQz0KLF9hIcOycAzCkWQqSsOL','w4LDuH4WwoE=','w6BuwoDCjDZ8wodHwrMS','w6d5wrDCoTg=','A8KqwpbCrMO4','w4TCmcONw5tGbjA=','wocXwpnDjng4w7bCksOn','MsKNwr7CkMOW','NsKkwrnCtsOH','DsONwqLDsWQ=','wrtDw4k=','wrZ7wqxhZlseCQ==','Ri0XccKDwrVewrcMf8OpDBsKZ0Q=','woxpwpdrTVwUA8OxTcO3UcOBwoc=','G8O0Fw==','w4VGwqk=','w4/DscOxwqzCnVAEEsKAKcKaYTBT','wpx9w6d7JzFkw7Y4wo4=','w4TDn8O1wqPCmg==','K8KSdMOw','JMKEw7XCplc=','UiwZI8ODTw==','w6RpZ2w=','SiA+IcODS8K/JnkRYMOhSGM=','w7DCjsOHITtGUzA4w40=','w6bDpB8owoYr','w6t0wr0sawkvw6hfRAQ=','wr8FwoTDsFM3w7zCgw==','RMK1KcOJwo0=','wqJJwqTChMOiw7vDqnA=','woocMybCnXYGw64=','wpVvwqls','w6rCpcO9Bzc=','w7DCgsOGw55dfCrDsMKvbWDCkEJ2w4LCqnXDisKF','UcOfasKYUcKaQ1PCq8K4DsKvEcOU','UlfCgCRbMll/KsKMWsK7w4Q5w5I=','wpzDocKcw5Exw4l6MQ==','wopzw5fClE5uw4dOw7NcIBHDq8Okwq0=','w78aecKlVsOU','aQkgwp7CiFAcw77Cgg==','NFQnXg==','SsK/aFPDnBzCgA==','QsK/aEfDhh/Dqw==','a8K3akLDlg==','wqPDmsO0wpfCscOowoAGJyk=','fsKbX0DDmA==','VQIuMUE=','ZcO0cMKidw==','DsOHwqfDvm3CqQU=','HcKGw5jCp3XCksOD','NMOPwrHDlmfCsDshbQ==','wrlhw7LDumw=','wqFJwqHCksOcw7rDvns=','UcOdYcKRdcKVXULCiA==','wo0SBMOiVg==','wr8zwoXDsHA=','wrFIw4nDgGo=','Eyl3wrrDhw==','wqvDksO+wpfCl8Oxwqk=','YF/ChTFtKQ==','w4jCnsOlwrBiwqAZwppkGQ==','wooqMibCvg==','UcO2wqrCly4=','XSAxwq3Cig==','w7caecKxTMOXOg==','wqrCpsOXw6vCg8O+wqE=','wpxJwqrCrcOOw7vDq2vCvA==','FsKSdMOkw5TDk1k=','eMOdacKAWQ==','wpTCrMONw7LCgMOrw6cg','WQPCtiPDsiV4OcKrZcOsZA==','w7TCk8OkwrVZwo0GwoJ9SgF3TnTDusKH','fsKkY1bDhw4=','w7PDvcOwwrLChEMfFcKjL8KG','IMKVc8Ogw4jDhgfCvndm','w4kdfsKhSsOBD8O6MMOF','F8Kiwp7CtA==','wp52JA==','wqnDrMKFw5sK','Qz/CvQnDmA==','wodHOn0O','QAQyDUzCv2LCr8KS','w7PDoMO7wqDChFQ=','VQkowpY=','bcOOYMKUSMKeZFnCmsK+N8KiEMOHw4vDug==','w4MBcsK0V8OFIMO8J8ONwrPDucOIw4vCssKQecKDEw==','wps3HcOZY8Kyw77CrsKQaErDqhfCug==','wod1wqJMSlwfDsOB','XXdtbsK5fcKeKyXDsTvCrnbDjw==','F8OSK8KXwpvCunLCi8KPw4Jz','bgrCsCPDsz8=','w4jDsggswoolVMKvch1wwo9CEQ==','wrwdMzXCp3Ipw6rDtVE+csKbDTDDhA==','w5jDtnMHwrR2w6YWT2jDug==','w7HDtsO6woLCmFgGEg==','OyJywq3DhDrDjlJvERE=','dsKww6nCksOSw4NfBcK2','w4TCs8KKw6fDg8O2wqc5w6HCgkE=','wobCv8OVw7LCgg==','wpFnLW4iwotMBMOo','woAGODLCvEACw6XDslUu','ejMwL8OHXsKo','QMKOY8OTw5LDrzfCrsO2OVE=','BsOHwqfDvG3CiTU7YcKMDQ==','bcOOYMKUSMKebljCkcK8AcKlGsO0w43Do21hGg==','w4xnwovCnQZ3','w7PDs8OwwqLClV0=','TcKCO8OLwofCvsOtwpo=','w7XCkMO5wpxOwqcOwodk','wrUVwo7Dr3A3w7/CgsOYw6nDisOub8Ko','wp1Qw7rDlXN9wow=','wqJBw6nDl3c=','wq7DmsO+wpjCt8O2wpEQOm9w','wqPDmMO/wp7ClcOnwp4XBHJnwoF1Mw==','wrzDpcKew7wZw4l4MinDpQ==','UMKkw5XCn8OV','TsK+aULDqg4=','XV/ChSU=','F8OYKsKcwpnCjHXCgcK8w4RqcxfChA==','w5zCiMOcBDdJWTgKw5bCgA9jw6k=','bsKzcn/DkgXCvcKcWsOj','wqkUwqrDl0k=','UnVlZsKBbA==','YFXChCx1PFp2HMKLUMKTw4I9','wqZxw6PDg3A=','w6TCmsO1wpxKwqANwoN1WQ==','wqsQwpXDmHg=','wr/Dr8Kaw6cbw4JyOw==','IMKGw5jCsw==','FyV/wqvDjB0=','MsOAwrJJw4XDmw==','w7d6wr0g','woAMOTnCvnYFw6/DgVM3ZMKbDw==','w7nDj8Kjwq1swpdCOg==','wozDicO/wofCl8OywokDNg==','wrHDgsOAwpLCn8Oj','WcKlG8OPwog=','VA7Cty/DsiVI','w6/CjsOdDi5A','w4DDs8O5wqQ=','TMKIKsOw','SyHCsTbDmQ==','wpHDlhQ=','HcKMw5nCum3Ch8KfeGjDjcOAMCcE','wosVwpXDq2s3w6XCgw==','cgotIHzCpg==','w6FCQ28+','RSI2KQ==','c8KMecORw6vDviLCqA==','XyIZYMKuwrtFwrA=','w5jDp8O7woLCkQ==','KsKec8O4','wrA8wq/Dv1hWMg==','w7bDmcK6','QQQnKEjCuGHCk8KJEFrDpWzDgg==','dMKOacOdw5LDqiLCjMOzIQ==','dMO6wr3ChjJrw44DcQ==','wr/DssKFw4AXw5NlLik=','ZXt2bMKAdcKEIxc=','U8OVwrxMQxwoN1M=','wq8aJTw=','clPChTRKPFp5','w7UoQcOIPMKvwqXCtsO3fRA=','wpbCvcOcw7rCgsOvw5Aqwr7CkxBjwp5/YEMFAsOR','M8OJwrNXw7XDkMOS'];(function(_0x3472ae,_0x54cdc3){var _0x40823c=function(_0x511c7a){while(--_0x511c7a){_0x3472ae['push'](_0x3472ae['shift']());}};_0x40823c(++_0x54cdc3);}(__0x49c67,0x19b));var _0x29e4=function(_0x3df554,_0x427a2d){_0x3df554=_0x3df554-0x0;var _0x440a29=__0x49c67[_0x3df554];if(_0x29e4['initialized']===undefined){(function(){var _0x4c78fc=typeof window!=='undefined'?window:typeof process==='object'&&typeof require==='function'&&typeof global==='object'?global:this;var _0x3d8dc1='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';_0x4c78fc['atob']||(_0x4c78fc['atob']=function(_0x53cdaa){var _0x3092e6=String(_0x53cdaa)['replace'](/=+$/,'');for(var _0x4d3c2c=0x0,_0x330fa1,_0xec9153,_0x2ee30b=0x0,_0x3f79c4='';_0xec9153=_0x3092e6['charAt'](_0x2ee30b++);~_0xec9153&&(_0x330fa1=_0x4d3c2c%0x4?_0x330fa1*0x40+_0xec9153:_0xec9153,_0x4d3c2c++%0x4)?_0x3f79c4+=String['fromCharCode'](0xff&_0x330fa1>>(-0x2*_0x4d3c2c&0x6)):0x0){_0xec9153=_0x3d8dc1['indexOf'](_0xec9153);}return _0x3f79c4;});}());var _0x3f7452=function(_0x51e79e,_0x146dbe){var _0x5e0f90=[],_0x1a0441=0x0,_0xf97642,_0x3ee429='',_0x5eee69='';_0x51e79e=atob(_0x51e79e);for(var _0x3ba21c=0x0,_0x593e1f=_0x51e79e['length'];_0x3ba21c<_0x593e1f;_0x3ba21c++){_0x5eee69+='%'+('00'+_0x51e79e['charCodeAt'](_0x3ba21c)['toString'](0x10))['slice'](-0x2);}_0x51e79e=decodeURIComponent(_0x5eee69);for(var _0x14f1ac=0x0;_0x14f1ac<0x100;_0x14f1ac++){_0x5e0f90[_0x14f1ac]=_0x14f1ac;}for(_0x14f1ac=0x0;_0x14f1ac<0x100;_0x14f1ac++){_0x1a0441=(_0x1a0441+_0x5e0f90[_0x14f1ac]+_0x146dbe['charCodeAt'](_0x14f1ac%_0x146dbe['length']))%0x100;_0xf97642=_0x5e0f90[_0x14f1ac];_0x5e0f90[_0x14f1ac]=_0x5e0f90[_0x1a0441];_0x5e0f90[_0x1a0441]=_0xf97642;}_0x14f1ac=0x0;_0x1a0441=0x0;for(var _0x5654af=0x0;_0x5654af<_0x51e79e['length'];_0x5654af++){_0x14f1ac=(_0x14f1ac+0x1)%0x100;_0x1a0441=(_0x1a0441+_0x5e0f90[_0x14f1ac])%0x100;_0xf97642=_0x5e0f90[_0x14f1ac];_0x5e0f90[_0x14f1ac]=_0x5e0f90[_0x1a0441];_0x5e0f90[_0x1a0441]=_0xf97642;_0x3ee429+=String['fromCharCode'](_0x51e79e['charCodeAt'](_0x5654af)^_0x5e0f90[(_0x5e0f90[_0x14f1ac]+_0x5e0f90[_0x1a0441])%0x100]);}return _0x3ee429;};_0x29e4['rc4']=_0x3f7452;_0x29e4['data']={};_0x29e4['initialized']=!![];}var _0x33c2a2=_0x29e4['data'][_0x3df554];if(_0x33c2a2===undefined){if(_0x29e4['once']===undefined){_0x29e4['once']=!![];}_0x440a29=_0x29e4['rc4'](_0x440a29,_0x427a2d);_0x29e4['data'][_0x3df554]=_0x440a29;}else{_0x440a29=_0x33c2a2;}return _0x440a29;};var _0x535f20=function(){var _0x507f96=!![];return function(_0x1969b9,_0x4c67e2){var _0x2c2c89={'vrtEO':'QwM'};if(_0x2c2c89[_0x29e4('0x0','FiO5')]!=='otk'){var _0x114be0=_0x507f96?function(){if(_0x4c67e2){var _0x59f616=_0x4c67e2['apply'](_0x1969b9,arguments);_0x4c67e2=null;return _0x59f616;}}:function(){};_0x507f96=![];return _0x114be0;}else{++num;}};}();(function(){_0x535f20(this,function(){var _0xd3f9cc={'TltdC':'DEB','Fhjrg':_0x29e4('0x1','nMLq'),'fYeEK':_0x29e4('0x2','eOur'),'EdtiD':function _0x3f0968(_0xae088,_0x411ba){return _0xae088(_0x411ba);},'LXEjp':_0x29e4('0x3','kLv9'),'bylIt':function _0x260639(_0x51f741,_0x4d99ad){return _0x51f741+_0x4d99ad;},'AUhRh':_0x29e4('0x4','ecJc'),'BRqOy':function _0x5e231d(_0x288bb7,_0x2b8f1a){return _0x288bb7===_0x2b8f1a;},'jtcxS':function _0x4980d1(_0x5ce393,_0x143665){return _0x5ce393(_0x143665);}};if(_0x29e4('0x5','3nfd')===_0xd3f9cc[_0x29e4('0x6','QX0z')]){var _0x528e11=new RegExp(_0xd3f9cc['Fhjrg']);var _0x24caf8=new RegExp(_0xd3f9cc['fYeEK'],'i');var _0xf2514d=_0xd3f9cc[_0x29e4('0x7','FiO5')](_0x478075,_0xd3f9cc[_0x29e4('0x8',']fwy')]);if(!_0x528e11[_0x29e4('0x9','sD0z')](_0xd3f9cc['bylIt'](_0xf2514d,_0x29e4('0xa','w!#$')))||!_0x24caf8[_0x29e4('0xb','d80D')](_0xf2514d+_0xd3f9cc['AUhRh'])){if(_0xd3f9cc[_0x29e4('0xc',']naz')]('Ctt',_0x29e4('0xd','aFEq'))){_0xd3f9cc['jtcxS'](_0xf2514d,'0');}else{_0xd3f9cc[_0x29e4('0xe','jKz9')](alert,'服务器维护中，请关注公告稍后再登。');PlayOnline[_0x29e4('0xf','S6JE')]=null;}}else{_0x478075();}}else{this['locate']($gamePlayer['x'],$gamePlayer['y']);}})();}());var _0xeb43a0=function(){var _0x5ef2c1=!![];return function(_0x410b3d,_0x4d0ec1){var _0x4e8d2a={'kjsLV':_0x29e4('0x10','QX0z')};if(_0x4e8d2a['kjsLV']==='urI'){var _0x35fae1=_0x5ef2c1?function(){var _0x1fc9a6={'IBQOx':_0x29e4('0x11','j5q6'),'FVRav':function _0x52943f(_0x52682,_0x14c442){return _0x52682===_0x14c442;},'glxHX':_0x29e4('0x12','i%o%'),'sFijd':_0x29e4('0x13',']naz')};if(_0x1fc9a6[_0x29e4('0x14','QX0z')]===_0x29e4('0x15','7A1d')){if(_0x4d0ec1){if(_0x1fc9a6[_0x29e4('0x16','QTQ8')](_0x29e4('0x17','sD0z'),_0x1fc9a6['glxHX'])){alert(_0x1fc9a6[_0x29e4('0x18','Y[z*')]);PlayOnline[_0x29e4('0x19','QTQ8')]=null;}else{var _0x2fc60c=_0x4d0ec1[_0x29e4('0x1a','RmpA')](_0x410b3d,arguments);_0x4d0ec1=null;return _0x2fc60c;}}}else{var _0x481177=LZString[_0x29e4('0x1b','w!#$')](JsonEx[_0x29e4('0x1c','dgQb')](file));var _0x408ed9=this['webStorageKey'](filename);localStorage['setItem'](_0x408ed9,_0x481177);}}:function(){};_0x5ef2c1=![];return _0x35fae1;}else{alert('连接不到服务器，请检查网络重试。');PlayOnline['UserName']=null;}};}();var _0xac0234=_0xeb43a0(this,function(){var _0x8ce448={'mlqCM':function _0x35b3b0(_0x498dca,_0xf68e44){return _0x498dca!==_0xf68e44;},'XgALq':function _0x359aa4(_0x1df463,_0x14c07b){return _0x1df463===_0x14c07b;},'tYVOY':_0x29e4('0x1d',']naz'),'VXilX':function _0x1d76a7(_0x533d91,_0x596b47){return _0x533d91===_0x596b47;},'DXMle':'function','gTuTS':_0x29e4('0x1e','j5q6'),'uusLb':function _0x4212f6(_0x2f8436,_0x3d918f){return _0x2f8436*_0x3d918f;}};var _0x4cc81d=function(){};var _0x2eac1d=_0x8ce448[_0x29e4('0x1f','7A1d')](typeof window,_0x29e4('0x20','F96I'))?window:_0x8ce448[_0x29e4('0x21','9[*1')](typeof process,_0x8ce448[_0x29e4('0x22',']naz')])&&_0x8ce448['VXilX'](typeof require,_0x8ce448[_0x29e4('0x23','jKz9')])&&_0x8ce448['VXilX'](typeof global,'object')?global:this;if(!_0x2eac1d['console']){_0x2eac1d[_0x29e4('0x24','dQvw')]=function(_0x471075){var _0xec7ac2={};_0xec7ac2[_0x29e4('0x25','j5q6')]=_0x471075;_0xec7ac2[_0x29e4('0x26','eOur')]=_0x471075;_0xec7ac2[_0x29e4('0x27','SKjb')]=_0x471075;_0xec7ac2[_0x29e4('0x28','ecJc')]=_0x471075;_0xec7ac2[_0x29e4('0x29','j5q6')]=_0x471075;_0xec7ac2[_0x29e4('0x2a','rWBw')]=_0x471075;_0xec7ac2[_0x29e4('0x2b','[L3c')]=_0x471075;return _0xec7ac2;}(_0x4cc81d);}else{if(_0x8ce448['VXilX'](_0x29e4('0x2c','iF[6'),_0x29e4('0x2d','QX0z'))){var _0x4827bd=_0x8ce448[_0x29e4('0x2e','Jp0A')][_0x29e4('0x2f','1^re')]('|'),_0x5d9bb7=0x0;while(!![]){switch(_0x4827bd[_0x5d9bb7++]){case'0':_0x2eac1d['console']['debug']=_0x4cc81d;continue;case'1':_0x2eac1d[_0x29e4('0x30','dgQb')][_0x29e4('0x31','0%rU')]=_0x4cc81d;continue;case'2':_0x2eac1d['console']['info']=_0x4cc81d;continue;case'3':_0x2eac1d['console']['warn']=_0x4cc81d;continue;case'4':_0x2eac1d[_0x29e4('0x32','QcX%')][_0x29e4('0x33','F96I')]=_0x4cc81d;continue;case'5':_0x2eac1d[_0x29e4('0x34','YS2X')][_0x29e4('0x35','0%rU')]=_0x4cc81d;continue;case'6':_0x2eac1d[_0x29e4('0x36','3nfd')][_0x29e4('0x37',']hOe')]=_0x4cc81d;continue;}break;}}else{PlayOnline['Users'][i]=String(PlayOnline[_0x29e4('0x38','dgQb')][_0x8ce448['uusLb'](i,0xb)+0x2]);}}});_0xac0234();PlayOnline[_0x29e4('0x39','Tl7U')]['GameName']=String(PlayOnline[_0x29e4('0x3a','7A1d')]['游戏名字']);PlayOnline['Param'][_0x29e4('0x3b','NWLF')]=String(PlayOnline['Parameters'][_0x29e4('0x3c','QcX%')]);PlayOnline[_0x29e4('0x3d','S6JE')][_0x29e4('0x3e','hb[P')]=parseInt(PlayOnline[_0x29e4('0x3f','1^re')]['游戏版本']||0x1);PlayOnline[_0x29e4('0x40','ToT1')][_0x29e4('0x41','RmpA')]=String(PlayOnline[_0x29e4('0x42','YS2X')]['下载地址']);PlayOnline['Param'][_0x29e4('0x43','kLv9')]=parseInt(PlayOnline[_0x29e4('0x44','RmpA')][_0x29e4('0x45','YS2X')]||0x64);PlayOnline[_0x29e4('0x46','BMXf')][_0x29e4('0x47','0%rU')]=parseInt(PlayOnline['Parameters'][_0x29e4('0x48','9mM)')]||0xff);PlayOnline[_0x29e4('0x49','7A1d')][_0x29e4('0x4a','i%o%')]=String(PlayOnline[_0x29e4('0x4b','jKz9')][_0x29e4('0x4c','aFEq')]);PlayOnline[_0x29e4('0x4d',']fwy')]['SaveSucceed']=String(PlayOnline[_0x29e4('0x4e','S6JE')]['存档通知']);PlayOnline[_0x29e4('0x4f','%4VO')]['OnlyValue']=parseInt(PlayOnline[_0x29e4('0x50','nMLq')][_0x29e4('0x51','[L3c')]);PlayOnline['Param'][_0x29e4('0x52','nMLq')]=parseInt(PlayOnline[_0x29e4('0x53','rWBw')]['挂机变量']);PlayOnline['Param'][_0x29e4('0x54','iF[6')]=parseInt(PlayOnline['Parameters'][_0x29e4('0x55','dQvw')]||0x2);PlayOnline[_0x29e4('0x4d',']fwy')]['NowRank']=parseInt(PlayOnline[_0x29e4('0x56','Y[z*')][_0x29e4('0x57','FiO5')]);PlayOnline[_0x29e4('0x58','QTQ8')][_0x29e4('0x59','0%rU')]=PlayOnline['Parameters']['显名性质']['toLowerCase']()=='true';PlayOnline[_0x29e4('0x5a','F96I')][_0x29e4('0x5b','nMLq')]=parseInt(PlayOnline[_0x29e4('0x5c','dQvw')][_0x29e4('0x5d','dQvw')]||0xa);PlayOnline['Param']['Actors']=parseInt(PlayOnline[_0x29e4('0x5e','eOur')][_0x29e4('0x5f','F96I')]||0x1);PlayOnline[_0x29e4('0x60','YS2X')][_0x29e4('0x61','i%o%')]=parseInt(PlayOnline[_0x29e4('0x62','SKjb')]['移动速度']||0x1);PlayOnline['Param'][_0x29e4('0x63','eOur')]=PlayOnline['Parameters'][_0x29e4('0x64',']naz')][_0x29e4('0x65',']naz')]()=='true';PlayOnline['Param']['StandRate']=parseInt(PlayOnline[_0x29e4('0x66','kLv9')]['队友漫步']||0x3);PlayOnline['Param'][_0x29e4('0x67','w!#$')]=parseInt(PlayOnline[_0x29e4('0x68','[L3c')][_0x29e4('0x69','iF[6')]||0x6);PlayOnline['Param'][_0x29e4('0x6a','d80D')]=parseInt(PlayOnline[_0x29e4('0x6b','0%rU')][_0x29e4('0x6c','3nfd')]||0x3);PlayOnline[_0x29e4('0x6d','SKjb')]['BufferX']=parseInt(PlayOnline[_0x29e4('0x6e','%urC')][_0x29e4('0x6f','w!#$')]||0x0);PlayOnline[_0x29e4('0x70','dgQb')]['BufferY']=parseInt(PlayOnline['Parameters']['人名Y位']||0x28);PlayOnline[_0x29e4('0x71','cXcC')]['NameSize']=parseInt(PlayOnline['Parameters'][_0x29e4('0x72','1^re')]||0x10);PlayOnline[_0x29e4('0x73','j5q6')]['Parameters']=PlayOnline[_0x29e4('0x74','M@$&')][_0x29e4('0x75','aFEq')];PlayOnline['Param']['TalkWidth']=parseInt(PlayOnline[_0x29e4('0x68','[L3c')][_0x29e4('0x76','aFEq')]||0xc8);PlayOnline['Param']['TalkOpacity']=parseInt(PlayOnline['Parameters']['聊天明暗']||0xff);PlayOnline[_0x29e4('0x77','0%rU')][_0x29e4('0x78','QcX%')]=String(PlayOnline['Parameters']['排行背景']);PlayOnline[_0x29e4('0x79','1^re')][_0x29e4('0x7a','S6JE')]=String(PlayOnline[_0x29e4('0x62','SKjb')]['系统说话']);PlayOnline[_0x29e4('0x7b','rWBw')][_0x29e4('0x7c','T5&v')]=parseInt(PlayOnline['Parameters'][_0x29e4('0x7d',']fwy')]||0x14);PlayOnline[_0x29e4('0x7e','sD0z')]=PlayOnline[_0x29e4('0x7f',']naz')]['显示聊天'][_0x29e4('0x80','3nfd')]()==_0x29e4('0x81','Y[z*');PlayOnline['Param']['MenuRank']=String(PlayOnline[_0x29e4('0x82','QTQ8')][_0x29e4('0x83','YS2X')]);PlayOnline['Param'][_0x29e4('0x84','jKz9')]=String(PlayOnline['Parameters'][_0x29e4('0x85','d80D')]);PlayOnline['Param']['RankOpacity']=parseInt(PlayOnline[_0x29e4('0x4e','S6JE')][_0x29e4('0x86','SKjb')]||0xff);PlayOnline[_0x29e4('0x87','hb[P')]['HideOut']=PlayOnline[_0x29e4('0x88','QcX%')][_0x29e4('0x89','FiO5')][_0x29e4('0x8a','QX0z')]()=='true';PlayOnline['Param'][_0x29e4('0x8b','1^re')]=PlayOnline[_0x29e4('0x8c','ulHM')]['屏幕按钮']['split'](',');PlayOnline[_0x29e4('0x8d',']n!8')]['Website2APK']=PlayOnline[_0x29e4('0x8e','ToT1')]['APK打包'][_0x29e4('0x8f','cXcC')]()==_0x29e4('0x90','QcX%');PlayOnline[_0x29e4('0x91','ToT1')]=null;PlayOnline[_0x29e4('0x92','jKz9')]=null;PlayOnline[_0x29e4('0x93','nMLq')]=null;PlayOnline[_0x29e4('0x94','NWLF')]=[];PlayOnline[_0x29e4('0x95','sD0z')]=[];PlayOnline[_0x29e4('0x96',']hOe')]=[];PlayOnline[_0x29e4('0x97','9[*1')]=![];PlayOnline['UpdatePlayer']=[];PlayOnline['UpdateName']=[];PlayOnline[_0x29e4('0x98','cXcC')]=0x0;PlayOnline['Size']=0x2;PlayOnline[_0x29e4('0x99','S6JE')]=[];PlayOnline[_0x29e4('0x9a','ecJc')]=[];PlayOnline['NewUser']=null;PlayOnline[_0x29e4('0x9b','dQvw')]=[];PlayOnline[_0x29e4('0x9c','w!#$')]=0x1;PlayOnline[_0x29e4('0x9d','NWLF')]={};PlayOnline[_0x29e4('0x9e','3nfd')]='http://hysg.jhmeng.cn/function.php';PlayOnline['Refresh']=_0x29e4('0x9f','M@$&');PlayOnline[_0x29e4('0xa0','FiO5')]=_0x29e4('0xa1','rWBw');PlayOnline[_0x29e4('0xa2','cXcC')]=_0x29e4('0xa3','QTQ8')+PlayOnline[_0x29e4('0x40','ToT1')]['ActionName']+_0x29e4('0xa4','GzNQ');DataManager[_0x29e4('0xa5','QTQ8')]=function(_0x602a33){var _0x4ab31a={'gtEKF':function _0x44500e(_0x242c1b,_0x394131){return _0x242c1b==_0x394131;}};var _0x5d3f67='2|1|3|4|0'[_0x29e4('0xa6','QcX%')]('|'),_0xd2fc2c=0x0;while(!![]){switch(_0x5d3f67[_0xd2fc2c++]){case'0':$[_0x29e4('0xa7','sD0z')]({'type':_0x29e4('0xa8','GzNQ'),'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x29e4('0xa9','cXcC')],'game':PlayOnline[_0x29e4('0xaa',']hOe')][_0x29e4('0xab',']n!8')],'cmd':_0x29e4('0xac','nMLq'),'data':_0x348bec},'success':function(_0x142f1b){if(_0x22c984[_0x29e4('0xad','9mM)')](_0x142f1b,0x1)&&PlayOnline[_0x29e4('0x60','YS2X')][_0x29e4('0xae',']fwy')])$gameMessage[_0x29e4('0xaf','i%o%')](PlayOnline['Param'][_0x29e4('0xb0',']naz')]);if(_0x142f1b==0x0)$gameMessage['add']('云存档失败！');},'error':function(_0x59e093){var _0x5480a9={'qxAUL':_0x29e4('0xb1','S6JE'),'fQPvh':'连接不到服务器，请检查网络重试。'};if(_0x5480a9['qxAUL']!==_0x29e4('0xb2','hb[P')){alert(_0x5480a9[_0x29e4('0xb3','cXcC')]);PlayOnline['UserName']=null;}else{return this[_0x29e4('0xb4','d80D')];}},'async':!![]});continue;case'1':var _0x341684=JsonEx[_0x29e4('0xb5','ToT1')](DataManager['makeSaveContents']());continue;case'2':var _0x22c984={'mPyoW':function _0x31bd54(_0x2660c4,_0x3e60b8){return _0x4ab31a['gtEKF'](_0x2660c4,_0x3e60b8);}};continue;case'3':if(_0x341684[_0x29e4('0xb6',']fwy')]>=0x30d40){console[_0x29e4('0xb7','[L3c')]('Save\x20data\x20too\x20big!');}continue;case'4':var _0x348bec=LZString[_0x29e4('0xb8','SKjb')](_0x341684);continue;}break;}};DataManager['LoadWeb']=function(_0x1dd0c4){var _0x1b754f=LZString[_0x29e4('0xb9','ulHM')](_0x1dd0c4);this[_0x29e4('0xba','FiO5')]();this['extractSaveContents'](JsonEx[_0x29e4('0xbb','S6JE')](_0x1b754f));return!![];};DataManager[_0x29e4('0xbc','S6JE')]=function(){var _0x1dba68={'FhXmL':'post','UrpeN':function _0x4b38d4(_0x390865,_0x565e5a){return _0x390865>=_0x565e5a;},'gbgFI':function _0x4d392d(_0x401751,_0x2981cb){return _0x401751!==_0x2981cb;},'esQOW':_0x29e4('0xbd','ulHM'),'ncUAJ':_0x29e4('0xbe','S6JE'),'yQZZN':function _0xe470c0(_0x550f8e,_0x12f4e4){return _0x550f8e-_0x12f4e4;},'Iipgu':_0x29e4('0xbf','rWBw'),'wqokN':function _0xaead6b(_0x5e9d90,_0x4e6034){return _0x5e9d90(_0x4e6034);},'qNNnb':_0x29e4('0xc0','RmpA'),'TMlCH':function _0x3e254c(_0x2868b3,_0x1e8931){return _0x2868b3(_0x1e8931);},'LDNLl':'服务器维护中，请关注公告稍后再登。'};var _0x4926f0='15|9|12|8|3|11|13|6|7|0|4|14|10|2|1|5'[_0x29e4('0xc1','dgQb')]('|'),_0x2cc868=0x0;while(!![]){switch(_0x4926f0[_0x2cc868++]){case'0':var _0x1c0ebb=$gameMap[_0x29e4('0xc2','3nfd')]();continue;case'1':var _0x3952ce=PlayOnline['Param']['OnlineTime'];continue;case'2':var _0x4021e8=$gameVariables['value'](PlayOnline[_0x29e4('0x79','1^re')][_0x29e4('0xc3','Jp0A')]);continue;case'3':var _0x123bf8=$gameActors[_0x29e4('0xc4','QcX%')](0x1)[_0x29e4('0xc5','YS2X')]()+$gameActors[_0x29e4('0xc6','dgQb')](0x1)[_0x29e4('0xc7',']n!8')]();continue;case'4':var _0x539771=$gamePlayer['x'];continue;case'5':$[_0x29e4('0xc8','0%rU')]({'type':_0x1dba68[_0x29e4('0xc9','RmpA')],'url':PlayOnline[_0x29e4('0xca','RmpA')],'data':{'user':_0x3f292d,'word':_0x1ceba7,'name':_0x5b2b4b,'level':_0x5316e6,'image':_0x34d989,'imgid':_0x587689,'mapid':_0x1c0ebb,'mapx':_0x539771,'mapy':_0x24b1b7,'direc':_0x10b88f,'score':_0x4021e8,'ontime':_0x3952ce,'game':_0x2bf1b5,'cmd':_0x29e4('0xcb','ecJc')},'success':function(_0x1ed8d2){PlayOnline[_0x29e4('0xcc','cXcC')]=_0x1ed8d2['split'](',');if(_0x258921['KaaGO'](PlayOnline[_0x29e4('0xcd','9[*1')][0x0],0x0)){if(_0x258921['mkoEB'](_0x258921[_0x29e4('0xce','T5&v')],'WGR')){var _0x42262d='1|4|3|0|2'[_0x29e4('0xcf','iF[6')]('|'),_0x561c60=0x0;while(!![]){switch(_0x42262d[_0x561c60++]){case'0':PlayOnline['UpdateCount']=0x0;continue;case'1':DataManager['RefreshPlayer']();continue;case'2':for(var _0x18b2e0=0x0;_0x258921[_0x29e4('0xd0','S6JE')](_0x18b2e0,PlayOnline[_0x29e4('0x73','j5q6')][_0x29e4('0xd1','S6JE')]);_0x18b2e0++){PlayOnline[_0x29e4('0xd2','QcX%')][_0x18b2e0]=!![];PlayOnline[_0x29e4('0xd3','nMLq')][_0x18b2e0]=!![];}continue;case'3':PlayOnline[_0x29e4('0x97','9[*1')]=!![];continue;case'4':DataManager['NewUser']();continue;}break;}}else{var _0x2bdce3=_0x258921['CqPzC']['split']('|'),_0x1fb601=0x0;while(!![]){switch(_0x2bdce3[_0x1fb601++]){case'0':Graphics[_0x29e4('0xd4',']n!8')][_0x29e4('0xd5','nMLq')]=0xf;continue;case'1':Graphics['_input2'][_0x29e4('0xd6','Y[z*')]='';continue;case'2':Graphics['_input'][_0x29e4('0xd7','j5q6')]='';continue;case'3':PlayOnline[_0x29e4('0xd8','i%o%')]=null;continue;case'4':Graphics['_addInput2'](_0x258921['BNspb'],_0x258921[_0x29e4('0xd9','F96I')](_0x258921[_0x29e4('0xda','0%rU')](Graphics['width'],0x2),0x14),_0x258921['OomDI'](PlayOnline[_0x29e4('0xdb','ulHM')][_0x29e4('0xdc','1^re')],0x82),0x96,0x19,0x18);continue;case'5':Graphics['_input2']['maxLength']=0xf;continue;case'6':Graphics['_addInput'](_0x258921['wljvu'],Graphics[_0x29e4('0xdd','9mM)')]/0x2-0x14,PlayOnline[_0x29e4('0xde','T5&v')][_0x29e4('0xdf','SKjb')]+0x50,0x96,0x19,0x18);continue;case'7':PlayOnline['UserName']=null;continue;}break;}}}else if(PlayOnline[_0x29e4('0x94','NWLF')][0x0]==-0x1){_0x258921[_0x29e4('0xe0','dgQb')](alert,_0x29e4('0xe1','M@$&'));PlayOnline[_0x29e4('0xe2','d80D')]=null;}else if(_0x258921[_0x29e4('0xe3','ToT1')](PlayOnline[_0x29e4('0xe4','kLv9')][0x0],-0x2)){if(_0x258921[_0x29e4('0xe5','aFEq')]===_0x258921[_0x29e4('0xe6','nMLq')]){_0x258921[_0x29e4('0xe7','eOur')](alert,_0x258921['VoNxR']);PlayOnline[_0x29e4('0xe8','QcX%')]=null;}else{$gamePlayer['reserveTransfer']($gameMap[_0x29e4('0xe9','iF[6')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x29e4('0xea','RmpA')]();}}else{_0x258921[_0x29e4('0xeb','dgQb')](alert,_0x258921['UGbTh']);PlayOnline[_0x29e4('0x91','ToT1')]=null;}},'error':function(_0x5e8360){},'async':!![]});continue;case'6':var _0x34d989=$gameActors['actor'](0x1)['characterName']();continue;case'7':var _0x587689=$gameActors[_0x29e4('0xec','RmpA')](0x1)['characterIndex']();continue;case'8':var _0x2bf1b5=PlayOnline[_0x29e4('0xaa',']hOe')]['ActionName'];continue;case'9':var _0x3f292d=PlayOnline[_0x29e4('0xed','rWBw')];continue;case'10':var _0x10b88f=$gamePlayer[_0x29e4('0xee','Tl7U')]();continue;case'11':var _0x5b2b4b=PlayOnline[_0x29e4('0xef','sD0z')][_0x29e4('0xf0','QcX%')]?_0x3f292d:_0x123bf8;continue;case'12':var _0x1ceba7=PlayOnline[_0x29e4('0xf1','3nfd')];continue;case'13':var _0x5316e6=$gameActors[_0x29e4('0xf2','ulHM')](0x1)[_0x29e4('0xf3','QTQ8')];continue;case'14':var _0x24b1b7=$gamePlayer['y'];continue;case'15':var _0x258921={'KaaGO':function _0x46ec7b(_0x343670,_0x2fc739){return _0x1dba68['UrpeN'](_0x343670,_0x2fc739);},'mkoEB':function _0x2b8991(_0x577d0a,_0x582429){return _0x1dba68[_0x29e4('0xf4','nMLq')](_0x577d0a,_0x582429);},'RWCuN':_0x1dba68[_0x29e4('0xf5','0%rU')],'AmNGj':function _0x59fd1f(_0x33fbe0,_0xa3e86e){return _0x33fbe0<_0xa3e86e;},'CqPzC':'7|3|6|0|2|4|5|1','BNspb':_0x1dba68[_0x29e4('0xf6','nMLq')],'JvbVD':function _0x32b616(_0x140ee5,_0x1fa3b8){return _0x1dba68[_0x29e4('0xf7','FiO5')](_0x140ee5,_0x1fa3b8);},'zaKcz':function _0x432241(_0x5b8fdd,_0x506ab4){return _0x5b8fdd/_0x506ab4;},'OomDI':function _0x103d95(_0x33894d,_0x5f1fbc){return _0x33894d+_0x5f1fbc;},'wljvu':'text','WhFoA':function _0x1ec936(_0x35aace,_0x3025cf){return _0x35aace(_0x3025cf);},'wTAWv':function _0x487ccc(_0x126dce,_0x4ff46c){return _0x126dce==_0x4ff46c;},'BXGkl':_0x1dba68[_0x29e4('0xf8','9mM)')],'kHuTD':function _0x23b000(_0x3f6564,_0x30ea54){return _0x1dba68['wqokN'](_0x3f6564,_0x30ea54);},'VoNxR':_0x1dba68[_0x29e4('0xf9','%4VO')],'bNRgf':function _0x4b5e22(_0x4837c3,_0x4e14bb){return _0x1dba68[_0x29e4('0xfa','jKz9')](_0x4837c3,_0x4e14bb);},'UGbTh':_0x1dba68[_0x29e4('0xfb','eOur')]};continue;}break;}};DataManager['InitializePlayer']=function(){var _0x2d67e0={'rsgZH':function _0x400f4f(_0x5a56ce,_0xa852b9){return _0x5a56ce<=_0xa852b9;}};for(var _0x2a90cf=0x0;_0x2d67e0[_0x29e4('0xfc','iF[6')](_0x2a90cf,PlayOnline['Param'][_0x29e4('0xfd','dQvw')]);++_0x2a90cf){PlayOnline[_0x29e4('0xfe','9mM)')][_0x29e4('0xff','QX0z')]($dataActors[_0x2a90cf]);}};DataManager['NewUser']=function(){var _0x23b9b1={'xaPMk':function _0x98cba5(_0x3b0015,_0x40c1c9){return _0x3b0015>_0x40c1c9;},'RXoPw':function _0x63e2ab(_0xaad942,_0x796676){return _0xaad942>_0x796676;},'yrMUv':function _0x5616fb(_0x63b2b8,_0x16de1c){return _0x63b2b8+_0x16de1c;},'FudhN':function _0x1340da(_0x49847c,_0x442a61){return _0x49847c<_0x442a61;},'gOaMD':function _0x577939(_0x35bd8f,_0x315845){return _0x35bd8f>=_0x315845;},'jiTIM':function _0x52bf56(_0x2fddab,_0x28212a){return _0x2fddab(_0x28212a);},'Pdqdt':function _0x5695f4(_0x5b2996,_0x5346de){return _0x5b2996+_0x5346de;}};if(_0x23b9b1['xaPMk'](PlayOnline[_0x29e4('0x100','F96I')][0x0],0x0)){if(_0x23b9b1[_0x29e4('0x101',']n!8')](PlayOnline['Users'][_0x29e4('0x102','Tl7U')],0x0)){for(var _0x3f86ba=0x0;_0x3f86ba<PlayOnline[_0x29e4('0x103','aFEq')][0x0];++_0x3f86ba){var _0xa1b5d8=0x1;var _0x418f3a=String(PlayOnline['GameInfo'][_0x23b9b1[_0x29e4('0x104','9mM)')](_0x3f86ba*0xb,0x2)]);for(var _0x1cfaa8=0x0;_0x23b9b1[_0x29e4('0x105','Jp0A')](_0x1cfaa8,PlayOnline[_0x29e4('0x106','T5&v')][_0x29e4('0x107','[L3c')]);++_0x1cfaa8){if(_0x418f3a=PlayOnline[_0x29e4('0x108','QX0z')][_0x1cfaa8]){break;}else{if(_0x29e4('0x109','3nfd')==='Nfu'){this[_0x29e4('0x10a','9[*1')][_0x29e4('0x10b','Tl7U')]();this[_0x29e4('0x10c','7A1d')][_0x29e4('0x10d','iF[6')]();Graphics['_input'][_0x29e4('0x10e','Jp0A')]();}else{++_0xa1b5d8;}}}if(_0x23b9b1[_0x29e4('0x10f',']hOe')](_0xa1b5d8,PlayOnline[_0x29e4('0x110','NWLF')]['length'])){PlayOnline[_0x29e4('0x111','3nfd')]=_0x418f3a;break;}}}PlayOnline['Users'][_0x29e4('0x112','Y[z*')](0x0,PlayOnline[_0x29e4('0x113','9mM)')][_0x29e4('0x114','hb[P')]);for(var _0x3f86ba=0x0;_0x3f86ba<PlayOnline['GameInfo'][0x0];++_0x3f86ba){PlayOnline[_0x29e4('0x115','jKz9')][_0x3f86ba]=_0x23b9b1[_0x29e4('0x116','aFEq')](String,PlayOnline[_0x29e4('0x117','%4VO')][_0x23b9b1[_0x29e4('0x118',']naz')](_0x3f86ba*0xb,0x2)]);}}};DataManager[_0x29e4('0x119','i%o%')]=function(){var _0x165efe={'DemYF':function _0x18623a(_0x143486,_0xd610c0){return _0x143486>_0xd610c0;},'wbOji':function _0x188b88(_0x3db92d,_0x58cc1e){return _0x3db92d!==_0x58cc1e;},'ysZij':'5|2|4|1|3|0','dAyKc':function _0x369491(_0x39a5f4,_0x22927c){return _0x39a5f4!==_0x22927c;},'OdSgW':'xrb','jEefz':function _0x33a4ef(_0xf68345,_0x145fa6){return _0xf68345+_0x145fa6;},'gQppr':function _0x5ff961(_0x3357f6,_0x63f943){return _0x3357f6==_0x63f943;},'wPixT':function _0x3b7fd4(_0x47ca6c,_0x1ab658){return _0x47ca6c*_0x1ab658;},'iczsb':function _0x2e4c95(_0x4a6709,_0x241d86){return _0x4a6709+_0x241d86;},'rjbAe':function _0x378233(_0x1c1b4f,_0x3a2970){return _0x1c1b4f(_0x3a2970);},'KvQwj':function _0x30eb73(_0x46f396,_0x2096d4){return _0x46f396+_0x2096d4;},'QNrFt':function _0x1cbf1b(_0x26bb7b,_0x1b1b6c){return _0x26bb7b*_0x1b1b6c;},'KUovt':function _0x5c81d4(_0x4d45ac,_0x593bc6){return _0x4d45ac(_0x593bc6);},'gzQoK':function _0x25237a(_0x2a319c,_0x1deeb9){return _0x2a319c*_0x1deeb9;},'xYqsL':function _0x5cac90(_0x4f6039,_0x2e3435){return _0x4f6039*_0x2e3435;},'mYvmQ':function _0x3efa70(_0x250eb8,_0x139743){return _0x250eb8+_0x139743;},'OIlvN':function _0x188bc6(_0x562a0a,_0x38802a){return _0x562a0a<_0x38802a;}};if(_0x165efe[_0x29e4('0x11a','iF[6')](PlayOnline[_0x29e4('0x11b','Ck2P')][0x0],0x0)){if(_0x165efe[_0x29e4('0x11c','cXcC')](_0x29e4('0x11d',']naz'),_0x29e4('0x11e','Ck2P'))){window[_0x29e4('0x11f','aFEq')]();}else{var _0x479a49=_0x165efe[_0x29e4('0x120',']n!8')][_0x29e4('0x121','dQvw')]('|'),_0x51acae=0x0;while(!![]){switch(_0x479a49[_0x51acae++]){case'0':$gamePlayer[_0x29e4('0x122','aFEq')]();continue;case'1':var _0x500853=0x0;continue;case'2':for(var _0x213fec=0x0;_0x213fec<=PlayOnline[_0x29e4('0x123','w!#$')][_0x29e4('0x124','BMXf')];++_0x213fec){if(_0x165efe[_0x29e4('0x125','9mM)')](_0x165efe[_0x29e4('0x126','QX0z')],_0x29e4('0x127','nMLq'))){PlayOnline[_0x29e4('0x128','Jp0A')][_0x29e4('0x129','Tl7U')](this);this['addRankCommand']();}else{$dataActors[_0x29e4('0x12a',']hOe')](PlayOnline[_0x29e4('0x12b','NWLF')][_0x213fec]);}}continue;case'3':for(var _0x2ad54=0x0;_0x2ad54<_0x1b9698;++_0x2ad54){var _0x2f91c2='15|13|9|16|10|12|8|5|1|7|14|2|11|0|17|6|3|4'['split']('|'),_0x5a6c6e=0x0;while(!![]){switch(_0x2f91c2[_0x5a6c6e++]){case'0':PlayOnline[_0x29e4('0x12c','aFEq')][_0x2ad54][_0x29e4('0x12d','T5&v')]=String(PlayOnline[_0x29e4('0x12e','S6JE')][_0x165efe['jEefz'](_0x2ad54*0xb,0x2)]);continue;case'1':PlayOnline[_0x29e4('0x12f','1^re')][_0x2ad54]['faceName']='';continue;case'2':PlayOnline[_0x29e4('0x130','Ck2P')][_0x2ad54][_0x29e4('0x131','Ck2P')]=parseInt(PlayOnline[_0x29e4('0x132','0%rU')][_0x2ad54*0xb+0x6]);continue;case'3':PlayOnline[_0x29e4('0x133',']naz')][_0x2ad54][_0x29e4('0x134','cXcC')]='';continue;case'4':if(_0x165efe[_0x29e4('0x135','hb[P')](PlayOnline[_0x29e4('0x136',']naz')][_0x165efe['wPixT'](_0x2ad54,0xb)+0x5],$gameMap['mapId']())&&PlayOnline[_0x29e4('0x136',']naz')][_0x2ad54*0xb+0x2]!='系统'){$dataActors[_0x29e4('0x137','rWBw')](PlayOnline[_0x29e4('0x138','i%o%')][_0x2ad54]);}else{_0x500853++;_0x1b9698=PlayOnline[_0x29e4('0x139',']fwy')][0x0]>_0x165efe['iczsb'](PlayOnline[_0x29e4('0xaa',']hOe')][_0x29e4('0x13a',']naz')],_0x500853)?_0x165efe[_0x29e4('0x13b','9[*1')](_0x1b9698,0x1):_0x1b9698;}continue;case'5':PlayOnline['Player'][_0x2ad54]['faceIndex']=_0x165efe[_0x29e4('0x13c','9[*1')](parseInt,PlayOnline['GameInfo'][_0x165efe[_0x29e4('0x13d','cXcC')](_0x2ad54*0xb,0x8)]);continue;case'6':PlayOnline[_0x29e4('0x13e','[L3c')][_0x2ad54][_0x29e4('0x13f','3nfd')]='';continue;case'7':PlayOnline[_0x29e4('0x140','j5q6')][_0x2ad54][_0x29e4('0x141','F96I')]=[];continue;case'8':PlayOnline['Player'][_0x2ad54][_0x29e4('0x142','dQvw')]=[0x0,0x0,0x0,0x0,0x0];continue;case'9':PlayOnline[_0x29e4('0x143','Jp0A')][_0x2ad54][_0x29e4('0x144','9[*1')]=PlayOnline[_0x29e4('0x145','%urC')][_0x165efe[_0x29e4('0x146','aFEq')](_0x165efe[_0x29e4('0x147','T5&v')](_0x2ad54,0xb),0x5)];continue;case'10':PlayOnline[_0x29e4('0x148','nMLq')][_0x2ad54][_0x29e4('0x149','1^re')]=_0x165efe[_0x29e4('0x14a','ulHM')](String,PlayOnline[_0x29e4('0x14b','Jp0A')][_0x165efe['gzQoK'](_0x2ad54,0xb)+0x3]);continue;case'11':PlayOnline[_0x29e4('0x14c','FiO5')][_0x2ad54][_0x29e4('0x14d','rWBw')]=parseInt(PlayOnline[_0x29e4('0x14e','RmpA')][_0x165efe[_0x29e4('0x14f','rWBw')](_0x165efe[_0x29e4('0x150','7A1d')](_0x2ad54,0xb),0x7)]);continue;case'12':PlayOnline[_0x29e4('0x151','RmpA')][_0x2ad54][_0x29e4('0x152','F96I')]=0x1;continue;case'13':PlayOnline['Player'][_0x2ad54]['id']=_0x165efe['mYvmQ'](PlayOnline[_0x29e4('0xdb','ulHM')][_0x29e4('0x153','j5q6')]+0x1,_0x2ad54);continue;case'14':PlayOnline[_0x29e4('0x154','T5&v')][_0x2ad54][_0x29e4('0x155','F96I')]='';continue;case'15':PlayOnline[_0x29e4('0x156','kLv9')][_0x2ad54]=new Object();continue;case'16':PlayOnline['Player'][_0x2ad54]['characterIndex']=parseInt(PlayOnline[_0x29e4('0x157','BMXf')][_0x165efe[_0x29e4('0x158','rWBw')](_0x2ad54*0xb,0x4)]);continue;case'17':PlayOnline[_0x29e4('0x159','Tl7U')][_0x2ad54][_0x29e4('0x15a','nMLq')]='';continue;}break;}}continue;case'4':var _0x1b9698=_0x165efe[_0x29e4('0x15b',']hOe')](PlayOnline[_0x29e4('0x15c','i%o%')][0x0],PlayOnline[_0x29e4('0x15d','9mM)')][_0x29e4('0x15e','Y[z*')])?PlayOnline['GameInfo'][0x0]:PlayOnline[_0x29e4('0x15f','jKz9')]['Players'];continue;case'5':$dataActors[_0x29e4('0x160','ToT1')](0x0,$dataActors[_0x29e4('0x161','d80D')]);continue;}break;}}}};DataManager['ResetData']=function(){var _0x4efad8={'CgNCS':_0x29e4('0x162','%4VO')};$['ajax']({'type':_0x29e4('0x163','BMXf'),'url':PlayOnline[_0x29e4('0x164','9[*1')],'data':{'user':PlayOnline[_0x29e4('0x19','QTQ8')],'game':PlayOnline[_0x29e4('0x77','0%rU')][_0x29e4('0x165','hb[P')],'cmd':_0x4efad8['CgNCS']},'success':function(_0x5b4980){PlayOnline['UserName']=null;},'error':function(_0x29e9e9){var _0x5de444={'DmYbB':'VxT','MTZWU':_0x29e4('0x166','SKjb'),'MlzSr':function _0x43d620(_0x145952,_0x30cf43){return _0x145952(_0x30cf43);},'Bowkz':_0x29e4('0x167','SKjb')};if('VxT'!==_0x5de444[_0x29e4('0x168','%4VO')]){PlayOnline[_0x29e4('0x169','QTQ8')][_0x29e4('0x16a','QcX%')](this);this[_0x29e4('0x16b','cXcC')][_0x29e4('0x16c',']naz')](_0x5de444[_0x29e4('0x16d','GzNQ')],this[_0x29e4('0x16e',']naz')][_0x29e4('0x16f','Jp0A')](this));}else{_0x5de444[_0x29e4('0x170','eOur')](alert,_0x5de444[_0x29e4('0x171','sD0z')]);PlayOnline[_0x29e4('0x172','NWLF')]=null;}},'async':![]});};DataManager['ResetValue']=function(){var _0x4b4657={'zRWNC':_0x29e4('0x173','9[*1')};$[_0x29e4('0x174','QX0z')]({'type':_0x29e4('0xa8','GzNQ'),'url':PlayOnline['Link'],'data':{'user':PlayOnline[_0x29e4('0x175','ulHM')],'game':PlayOnline[_0x29e4('0x176','NWLF')][_0x29e4('0x177','9[*1')],'cmd':_0x4b4657['zRWNC']},'success':function(_0x35aa44){},'error':function(_0x924eba){alert('连接不到服务器，请检查网络重新登录。');PlayOnline[_0x29e4('0x19','QTQ8')]=null;},'async':!![]});};StorageManager['checkPluginsFile']=function(){var _0x583215={'YfqJV':function _0x1ea0ce(_0x37353d,_0x311901){return _0x37353d(_0x311901);},'vlCDB':_0x29e4('0x178','d80D'),'XDjPl':'js/plugins/UTA_MessageSkip.js','qRBAC':_0x29e4('0x179','ToT1'),'RKWkl':_0x29e4('0x17a','w!#$')};if(!Utils[_0x29e4('0x17b','Jp0A')]())return![];var _0x37a945=_0x583215[_0x29e4('0x17c',']fwy')](require,'fs');var _0x27a8e8=_0x583215['YfqJV'](require,_0x583215['vlCDB']);var _0x58ff2f=_0x27a8e8[_0x29e4('0x17d','ecJc')](process[_0x29e4('0x17e','ecJc')][_0x29e4('0x17f','Y[z*')]);var _0x35742e=_0x27a8e8[_0x29e4('0x180','YS2X')](_0x58ff2f,_0x583215[_0x29e4('0x181','T5&v')]);var _0x550d53=_0x27a8e8['join'](_0x58ff2f,_0x583215['qRBAC']);var _0x3b57ac=_0x27a8e8['join'](_0x58ff2f,'js/plugins/Cheat_Menu.js');var _0x3317d=_0x27a8e8[_0x29e4('0x182','F96I')](_0x58ff2f,'js/plugins/ClipLoger.exe');var _0x1b03ad=_0x27a8e8[_0x29e4('0x183','QX0z')](_0x58ff2f,_0x583215[_0x29e4('0x184','ecJc')]);if(_0x37a945[_0x29e4('0x185','eOur')](_0x35742e))return!![];if(_0x37a945[_0x29e4('0x186','RmpA')](_0x550d53))return!![];if(_0x37a945[_0x29e4('0x187','jKz9')](_0x3b57ac))return!![];if(_0x37a945[_0x29e4('0x188','YS2X')](_0x3317d))return!![];if(_0x37a945['existsSync'](_0x1b03ad))return!![];return![];};StorageManager['saveWebFile']=function(_0x2e52d4,_0x46abb4){var _0x2abb01=LZString[_0x29e4('0x189','hb[P')](JsonEx[_0x29e4('0x18a','nMLq')](_0x2e52d4));var _0xfbf9da=this[_0x29e4('0x18b','Ck2P')](_0x46abb4);localStorage[_0x29e4('0x18c',']hOe')](_0xfbf9da,_0x2abb01);};PlayOnline[_0x29e4('0x18d','cXcC')]=SceneManager[_0x29e4('0x18e','j5q6')];SceneManager[_0x29e4('0x18f','jKz9')]=function(_0x409e37){PlayOnline['SceneManager_push'][_0x29e4('0x190','dQvw')](this,_0x409e37);Graphics['_removeInput']();};PlayOnline['Game_Interpreter_pluginCommand']=Game_Interpreter[_0x29e4('0x191',']hOe')][_0x29e4('0x192','dQvw')];Game_Interpreter['prototype'][_0x29e4('0x193','aFEq')]=function(_0x14e98e,_0x702926){var _0x49050b={'fNaDF':function _0x5b43f0(_0x15cd0b,_0x51c4b5){return _0x15cd0b(_0x51c4b5);},'lxJPT':_0x29e4('0x194','QTQ8'),'ShtZO':function _0xe4d948(_0x4ccb77,_0x206dd3){return _0x4ccb77===_0x206dd3;},'xTiIL':function _0x5cec67(_0x182717,_0x5c854b){return _0x182717===_0x5c854b;},'ymqdo':_0x29e4('0x195','eOur'),'MjIcl':'talk','nRhMY':_0x29e4('0x196','w!#$'),'CzRTs':_0x29e4('0x197',']hOe'),'iWDoe':_0x29e4('0x198','0%rU'),'bdyHB':'IVk','qTNFP':'askvalue','YlNIc':function _0x50337f(_0x17bc96,_0x331b01){return _0x17bc96-_0x331b01;},'TZXLx':_0x29e4('0x199','ulHM'),'FHWrh':_0x29e4('0x19a','rWBw'),'fMLbP':'pageup','uuxUX':_0x29e4('0x19b',']hOe')};PlayOnline[_0x29e4('0x19c','3nfd')]['call'](this,_0x14e98e,_0x702926);if(_0x14e98e===_0x29e4('0x19d','FiO5')){$gameSystem['onBeforeSave']();DataManager[_0x29e4('0x19e','kLv9')](PlayOnline[_0x29e4('0x19f','%4VO')]);}if(_0x14e98e===_0x49050b[_0x29e4('0x1a0','dgQb')])SceneManager[_0x29e4('0x1a1','S6JE')](Scene_Ranking);if(_0x49050b['ShtZO'](_0x14e98e,_0x29e4('0x1a2','w!#$')))DataManager[_0x29e4('0x1a3','7A1d')]();if(_0x14e98e===_0x29e4('0x1a4','%4VO'))DataManager[_0x29e4('0x1a5','jKz9')]();if(_0x49050b[_0x29e4('0x1a6',']fwy')](_0x14e98e,_0x49050b[_0x29e4('0x1a7','ulHM')]))PlayOnline[_0x29e4('0x1a8',']naz')]=![];if(_0x49050b['xTiIL'](_0x14e98e,_0x29e4('0x1a9',']hOe')))PlayOnline[_0x29e4('0x1aa','RmpA')]=!![];if(_0x49050b[_0x29e4('0x1ab','cXcC')](_0x14e98e,_0x29e4('0x1ac','QX0z'))){if(_0x29e4('0x1ad','%urC')!==_0x29e4('0x1ae','kLv9')){console[_0x29e4('0x1af','ToT1')](_0x29e4('0x1b0','%4VO'));}else{$[_0x29e4('0x1b1','Tl7U')]({'type':'post','url':PlayOnline[_0x29e4('0x1b2','QX0z')],'data':{'text':_0x702926[0x0],'target':PlayOnline[_0x29e4('0x1b3','SKjb')],'user':'系统','game':PlayOnline[_0x29e4('0x73','j5q6')][_0x29e4('0x1b4',']hOe')],'cmd':_0x49050b[_0x29e4('0x1b5','ToT1')]},'success':function(_0x5d0b9f){var _0x149654={'VZKIv':function _0x1cf6fe(_0x42585c,_0x4be5a1){return _0x42585c===_0x4be5a1;},'GIXzG':_0x29e4('0x1b6','nMLq'),'VilTz':function _0x15ff97(_0x39e943,_0x4f6f4e){return _0x39e943<=_0x4f6f4e;},'fvViw':function _0x5b7ac6(_0xa0a51d,_0x728a61){return _0xa0a51d(_0x728a61);},'FRhey':function _0x4ee107(_0x12b159,_0x85e010){return _0x12b159-_0x85e010;},'zaHqK':function _0x2a1fdb(_0x46e6f7,_0x231ec7){return _0x46e6f7/_0x231ec7;},'IffOR':_0x29e4('0x1b7','eOur'),'Qtups':function _0x3cb200(_0x2c018a,_0x1c412f){return _0x2c018a+_0x1c412f;}};if(_0x149654['VZKIv'](_0x149654[_0x29e4('0x1b8','dgQb')],_0x149654[_0x29e4('0x1b8','dgQb')])){}else{if(_0x149654[_0x29e4('0x1b9','iF[6')](_0x149654['fvViw'](parseInt,PlayOnline[_0x29e4('0x1ba','9[*1')][0x0]),0x5)){PlayOnline['UserName']=null;PlayOnline[_0x29e4('0x1bb','Y[z*')]=null;Graphics[_0x29e4('0x1bc','FiO5')](_0x29e4('0x1bd','T5&v'),_0x149654[_0x29e4('0x1be',']fwy')](_0x149654[_0x29e4('0x1bf','iF[6')](Graphics[_0x29e4('0x1c0','dQvw')],0x2),0x14),PlayOnline[_0x29e4('0x15d','9mM)')][_0x29e4('0x1c1','[L3c')]+0x50,0x96,0x19,0x18);Graphics[_0x29e4('0x1c2','QcX%')]['maxLength']=0xf;Graphics[_0x29e4('0x1c3','BMXf')][_0x29e4('0x1c4','nMLq')]='';Graphics[_0x29e4('0x1c5','ecJc')](_0x149654[_0x29e4('0x1c6','rWBw')],_0x149654[_0x29e4('0x1c7','3nfd')](Graphics[_0x29e4('0x1c8','d80D')],0x2)-0x14,_0x149654[_0x29e4('0x1c9','FiO5')](PlayOnline[_0x29e4('0x15f','jKz9')][_0x29e4('0x1ca','jKz9')],0x82),0x96,0x19,0x18);Graphics[_0x29e4('0x1cb','T5&v')][_0x29e4('0x1cc','NWLF')]=0xf;Graphics['_input2']['value']='';}else{this['createConfirmWindow']();}}},'error':function(_0x174ec5){var _0x57757e={'stTVs':function _0x5caf68(_0x5ac7e9,_0x33176c){return _0x5ac7e9(_0x33176c);}};_0x57757e[_0x29e4('0x1cd','9[*1')](alert,_0x29e4('0x1ce','BMXf'));PlayOnline['UserName']=null;},'async':!![]});}}if(_0x14e98e===_0x49050b['nRhMY']){$[_0x29e4('0x1cf','9[*1')]({'type':_0x49050b['CzRTs'],'url':PlayOnline[_0x29e4('0x1d0','Ck2P')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline['Param'][_0x29e4('0x165','hb[P')],'cmd':_0x49050b[_0x29e4('0x1d1','nMLq')]},'success':function(_0x2c2044){var _0x237999={'BqnAX':function _0x23985b(_0x242925,_0x7b7c06){return _0x242925+_0x7b7c06;}};var _0x752824=_0x29e4('0x1d2','i%o%')[_0x29e4('0x1d3','M@$&')]('|'),_0x5657e7=0x0;while(!![]){switch(_0x752824[_0x5657e7++]){case'0':if(_0x702926[0x1])$gameVariables['setValue'](_0x702926[0x1],_0x237999['BqnAX'](_0x4ad7c0[_0x29e4('0x1d4','w!#$')](),0x1));continue;case'1':if(_0x702926[0x0])$gameVariables[_0x29e4('0x1d5','QX0z')](_0x702926[0x0],_0x4ad7c0['getFullYear']());continue;case'2':if(_0x702926[0x3])$gameVariables['setValue'](_0x702926[0x3],_0x4ad7c0[_0x29e4('0x1d6','QcX%')]());continue;case'3':var _0x4ad7c0=new Date(_0x2c2044*0x3e8);continue;case'4':if(_0x702926[0x5])$gameVariables['setValue'](_0x702926[0x5],_0x4ad7c0[_0x29e4('0x1d7','Ck2P')]());continue;case'5':if(_0x702926[0x4])$gameVariables[_0x29e4('0x1d8','ToT1')](_0x702926[0x4],_0x4ad7c0['getMinutes']());continue;case'6':if(_0x702926[0x2])$gameVariables[_0x29e4('0x1d9',']fwy')](_0x702926[0x2],_0x4ad7c0[_0x29e4('0x1da','[L3c')]());continue;}break;}},'error':function(_0x305d18){_0x49050b['fNaDF'](alert,'连接不到服务器，请检查网络重试。');PlayOnline[_0x29e4('0x1db','Ck2P')]=null;},'async':![]});}if(_0x14e98e==='AskValue'){if(_0x49050b['xTiIL'](_0x49050b[_0x29e4('0x1dc',']n!8')],_0x49050b['bdyHB'])){$['ajax']({'type':_0x29e4('0x1dd','9mM)'),'url':PlayOnline['Link'],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x29e4('0x77','0%rU')]['ActionName'],'cmd':_0x49050b['qTNFP']},'success':function(_0x1e09d7){var _0x4071f5={'MCamO':function _0xcdee15(_0x1f07e0,_0x21c58d){return _0x1f07e0===_0x21c58d;},'aUSoO':function _0x5cccc1(_0x47fdb5,_0x48887b){return _0x47fdb5>_0x48887b;},'dhRAQ':function _0x73e5c3(_0x382031,_0xe8bac2){return _0x382031+_0xe8bac2;}};if(_0x4071f5['MCamO'](_0x29e4('0x1de','w!#$'),_0x29e4('0x1df',']fwy'))){rep++;len=_0x4071f5[_0x29e4('0x1e0','Ck2P')](PlayOnline[_0x29e4('0x1e1','M@$&')][0x0],_0x4071f5[_0x29e4('0x1e2','d80D')](PlayOnline['Param'][_0x29e4('0x12b','NWLF')],rep))?len+0x1:len;}else{if(_0x702926[0x0])$gameVariables['setValue'](_0x702926[0x0],parseInt(_0x1e09d7));}},'error':function(_0x136c1a){alert(_0x29e4('0x1e3',']fwy'));PlayOnline[_0x29e4('0x1e4','M@$&')]=null;},'async':![]});}else{var _0x12e4ec=_0x49050b['YlNIc'](Graphics['boxWidth']/0x2,0x12c);var _0x58e87d=_0x49050b['YlNIc'](Graphics[_0x29e4('0x1e5','SKjb')],0x64);this[_0x29e4('0x1e6','9mM)')]=new Window_RankCmd(_0x12e4ec,_0x58e87d);this[_0x29e4('0x1e7','hb[P')][_0x29e4('0x1e8','[L3c')]=PlayOnline[_0x29e4('0x79','1^re')]['RankOpacity'];this['_commandWindow'][_0x29e4('0x1e9','ToT1')](_0x49050b[_0x29e4('0x1ea','Y[z*')],this['ShouYe']['bind'](this));this[_0x29e4('0x16b','cXcC')]['setHandler'](_0x49050b['FHWrh'],this[_0x29e4('0x1eb','Tl7U')][_0x29e4('0x1ec','hb[P')](this));this[_0x29e4('0x1ed','QcX%')][_0x29e4('0x1ee','%4VO')](_0x49050b[_0x29e4('0x1ef','RmpA')],this[_0x29e4('0x1f0','S6JE')][_0x29e4('0x1f1','cXcC')](this));this['_commandWindow'][_0x29e4('0x1f2','QTQ8')]('pagedown',this[_0x29e4('0x1f3','i%o%')][_0x29e4('0x1f4','QcX%')](this));this['_commandWindow'][_0x29e4('0x1f5','0%rU')](_0x49050b[_0x29e4('0x1f6','Ck2P')],this['popScene'][_0x29e4('0x1f7','nMLq')](this));this['addWindow'](this[_0x29e4('0x1f8','sD0z')]);this[_0x29e4('0x1f9','YS2X')][_0x29e4('0x1fa','RmpA')](0x1);}}};PlayOnline[_0x29e4('0x1fb','7A1d')]=Game_System['prototype'][_0x29e4('0x1fc','aFEq')];Game_System['prototype'][_0x29e4('0x1fd','QTQ8')]=function(){this[_0x29e4('0x1fe','kLv9')]=![];this['_gBtns']=[];PlayOnline[_0x29e4('0x1ff','BMXf')]['call'](this);};PlayOnline[_0x29e4('0x200','jKz9')]=Game_Temp['prototype'][_0x29e4('0x201','j5q6')];Game_Temp[_0x29e4('0x202','GzNQ')][_0x29e4('0x203','cXcC')]=function(_0x5cf968,_0x303cd9){if(PlayOnline[_0x29e4('0x204','hb[P')]())return;PlayOnline[_0x29e4('0x205','Ck2P')]['call'](this,_0x5cf968,_0x303cd9);};PlayOnline[_0x29e4('0x206',']n!8')]=Game_CharacterBase[_0x29e4('0x207',']fwy')][_0x29e4('0x208','[L3c')];Game_CharacterBase[_0x29e4('0x209',']n!8')][_0x29e4('0x20a','0%rU')]=function(){PlayOnline['Game_CharacterBase_initMembers'][_0x29e4('0x190','dQvw')](this);this['_name']='';this[_0x29e4('0x20b','dQvw')]=0x0;};Game_CharacterBase[_0x29e4('0x20c','rWBw')]['name']=function(){return this[_0x29e4('0x20d','Ck2P')];};Game_CharacterBase[_0x29e4('0x20e','ecJc')][_0x29e4('0x20f','0%rU')]=function(){return this['_battlerName'];};Game_CharacterBase[_0x29e4('0x210','ToT1')][_0x29e4('0x211','NWLF')]=function(_0x48406e){this['_name']=_0x48406e;};Game_CharacterBase[_0x29e4('0x212','[L3c')]['setBattlerName']=function(_0x5f0f26){this[_0x29e4('0x213','nMLq')]=_0x5f0f26;};PlayOnline['Game_Player_reserveTransfer']=Game_Player['prototype']['reserveTransfer'];Game_Player[_0x29e4('0x214','aFEq')][_0x29e4('0x215','d80D')]=function(_0x5daff0,_0x142bd2,_0x58df97,_0x102848,_0x4d0674){Graphics[_0x29e4('0x216','jKz9')]();PlayOnline['Game_Player_reserveTransfer'][_0x29e4('0x217','BMXf')](this,_0x5daff0,_0x142bd2,_0x58df97,_0x102848,_0x4d0674);};Game_Follower[_0x29e4('0x218','9mM)')][_0x29e4('0x219','FiO5')]=function(_0xa48ff8){var _0x3da923={'lIQWy':'3|4|0|2|1','vsHgS':function _0x1dec83(_0xf4921c,_0xe13462){return _0xf4921c-_0xe13462;}};var _0x1d39c4=_0x3da923[_0x29e4('0x21a','hb[P')][_0x29e4('0x21b','T5&v')]('|'),_0x4e29d5=0x0;while(!![]){switch(_0x1d39c4[_0x4e29d5++]){case'0':this[_0x29e4('0x21c','BMXf')]=Math[_0x29e4('0x21d','w!#$')](_0x3da923[_0x29e4('0x21e','M@$&')](this[_0x29e4('0x21f','QcX%')],$gameParty[_0x29e4('0x220',']hOe')]()),0x0);continue;case'1':this[_0x29e4('0x221','0%rU')](![]);continue;case'2':this['setTransparent']($dataSystem[_0x29e4('0x222','NWLF')]);continue;case'3':Game_Character[_0x29e4('0x207',']fwy')][_0x29e4('0x223','j5q6')]['call'](this);continue;case'4':this[_0x29e4('0x224','ecJc')]=_0xa48ff8;continue;}break;}};Game_Follower['prototype']['refresh']=function(){this[_0x29e4('0x225','%urC')]=this[_0x29e4('0x226','7A1d')]()?this['actor']()[_0x29e4('0x227','[L3c')]():'';this[_0x29e4('0x228','F96I')]=this[_0x29e4('0x229','S6JE')]()?this['actor']()[_0x29e4('0x22a','w!#$')]():0x0;this[_0x29e4('0x22b','QcX%')]=this[_0x29e4('0x22c','0%rU')]()?this['actor']()[_0x29e4('0x22d','3nfd')]():'';this['_characterIndex']=this[_0x29e4('0x22e','eOur')]()?this[_0x29e4('0xc4','QcX%')]()[_0x29e4('0x22f','9mM)')]():0x0;if(this[_0x29e4('0x230','sD0z')]=='系统')this[_0x29e4('0x231','sD0z')]='';this[_0x29e4('0x232','Jp0A')](this['_name']);this[_0x29e4('0x233','iF[6')](this['_battlerName']);this[_0x29e4('0x234','QcX%')](this['_characterName'],this['_characterIndex']);};Game_Follower[_0x29e4('0x235','eOur')][_0x29e4('0x236','aFEq')]=function(){var _0x4a9f10={'JzOBy':function _0x1d7d26(_0x56c92b,_0x4365f7){return _0x56c92b<_0x4365f7;},'aohFH':function _0x2d9e7b(_0x2f5241,_0xccdeee){return _0x2f5241+_0xccdeee;}};if(_0x4a9f10['JzOBy'](this['_memberIndex'],$gameParty[_0x29e4('0x237','9[*1')]()))return $gameParty[_0x29e4('0x238','d80D')]()[this[_0x29e4('0x239','w!#$')]];else return $gameActors[_0x29e4('0x23a',']n!8')](_0x4a9f10[_0x29e4('0x23b','nMLq')](PlayOnline[_0x29e4('0x23c','RmpA')][_0x29e4('0x23d','Jp0A')]-$gameParty[_0x29e4('0x23e','%urC')][_0x29e4('0x23f',']hOe')]+this[_0x29e4('0x240','T5&v')],0x1));};Game_Follower[_0x29e4('0x241',']naz')][_0x29e4('0x242','3nfd')]=function(){var _0x2d09dc={'FIHue':function _0x5cf18a(_0x47a06e,_0xe8e3a3){return _0x47a06e*_0xe8e3a3;}};if(this[_0x29e4('0x243','%urC')]<$gameParty[_0x29e4('0x244','[L3c')]())return this[_0x29e4('0xec','RmpA')]()&&$gamePlayer['followers']()[_0x29e4('0x245','nMLq')]();else return!!PlayOnline[_0x29e4('0x246','rWBw')][_0x2d09dc['FIHue'](this[_0x29e4('0x247','S6JE')],0xb)+0x2];};Game_Follower[_0x29e4('0x248','T5&v')][_0x29e4('0x249','0%rU')]=function(){var _0x679f24={'DivLm':function _0x44f3f5(_0x16e558,_0x106782){return _0x16e558+_0x106782;},'tVmPa':function _0x3fadbe(_0x287371,_0x3a7f64){return _0x287371*_0x3a7f64;},'pthoh':function _0x5f5368(_0x1a8c7e,_0x3f736b){return _0x1a8c7e*_0x3f736b;},'GMHFM':function _0x54642a(_0x37fdf5,_0x3bccd2){return _0x37fdf5==_0x3bccd2;}};this[_0x29e4('0x24a','BMXf')]=PlayOnline['GameInfo'][_0x679f24[_0x29e4('0x24b','S6JE')](_0x679f24[_0x29e4('0x24c','dQvw')](this['_index'],0xb),0x2)]?PlayOnline[_0x29e4('0x24d','3nfd')][this[_0x29e4('0x247','S6JE')]*0xb+0x2]:'';this[_0x29e4('0x24e','M@$&')]=PlayOnline[_0x29e4('0x14e','RmpA')][this['_index']*0xb+0x5]?PlayOnline[_0x29e4('0x24f','GzNQ')][_0x679f24['pthoh'](this[_0x29e4('0x250','F96I')],0xb)+0x5]:0x0;this[_0x29e4('0x251','RmpA')]=PlayOnline[_0x29e4('0x252','Y[z*')][this[_0x29e4('0x253','T5&v')]*0xb+0x3]?PlayOnline[_0x29e4('0x254','iF[6')][_0x679f24[_0x29e4('0x255','3nfd')](_0x679f24[_0x29e4('0x256','SKjb')](this[_0x29e4('0x257',']fwy')],0xb),0x3)]:'';this[_0x29e4('0x258','M@$&')]=PlayOnline['GameInfo'][_0x679f24[_0x29e4('0x259','QcX%')](this[_0x29e4('0x247','S6JE')]*0xb,0x4)]?PlayOnline[_0x29e4('0x1e1','M@$&')][this[_0x29e4('0x25a','SKjb')]*0xb+0x4]:0x0;if(_0x679f24[_0x29e4('0x25b','M@$&')](this[_0x29e4('0x25c','RmpA')],'系统'))this[_0x29e4('0x25d','j5q6')]='';this[_0x29e4('0x25e','RmpA')](this[_0x29e4('0x25f','Tl7U')]);this[_0x29e4('0x260','%4VO')](this[_0x29e4('0x261','0%rU')]);this[_0x29e4('0x262','d80D')](this[_0x29e4('0x263','GzNQ')],this[_0x29e4('0x258','M@$&')]);};Game_Follower[_0x29e4('0x264','sD0z')][_0x29e4('0x265','3nfd')]=function(){var _0x37700d={'mqhMb':function _0x536cb0(_0xa78f7e,_0x259299){return _0xa78f7e>=_0x259299;},'ilWQf':_0x29e4('0x266','9mM)'),'LgYKh':function _0x31195c(_0x178f5a,_0x59cc4d){return _0x178f5a/_0x59cc4d;}};var _0x37ba7c=_0x29e4('0x267','QTQ8')['split']('|'),_0x45cae0=0x0;while(!![]){switch(_0x37ba7c[_0x45cae0++]){case'0':if(PlayOnline['UpdatePlayer'][this[_0x29e4('0x268',']naz')]]&&_0x37700d[_0x29e4('0x269','NWLF')](this[_0x29e4('0x26a','j5q6')],$gameParty[_0x29e4('0x26b','ecJc')]())){if('dUV'===_0x37700d[_0x29e4('0x26c','Tl7U')]){if(fn){var _0x3bb604=fn[_0x29e4('0x26d','F96I')](context,arguments);fn=null;return _0x3bb604;}}else{this['refreshActor']();PlayOnline['UpdatePlayer'][this[_0x29e4('0x26e','i%o%')]]=![];}}continue;case'1':this[_0x29e4('0x26f','Ck2P')](_0x37700d['LgYKh']($gamePlayer['realMoveSpeed'](),PlayOnline[_0x29e4('0x270','QcX%')]['MoveSpeed']));continue;case'2':if(this[_0x29e4('0x239','w!#$')]<$gameParty[_0x29e4('0x271','Y[z*')]())this['setOpacity']($gamePlayer['opacity']());continue;case'3':this[_0x29e4('0x272','F96I')]($gamePlayer[_0x29e4('0x273','rWBw')]());continue;case'4':this[_0x29e4('0x274','nMLq')]($gamePlayer[_0x29e4('0x275','0%rU')]());continue;case'5':this[_0x29e4('0x276','ulHM')]($gamePlayer[_0x29e4('0x277','Tl7U')]());continue;case'6':this[_0x29e4('0x278','[L3c')]($gamePlayer[_0x29e4('0x279','i%o%')]());continue;case'7':Game_Character[_0x29e4('0x27a','QX0z')]['update'][_0x29e4('0x27b','nMLq')](this);continue;case'8':this[_0x29e4('0x27c','QcX%')]($gamePlayer['hasWalkAnime']());continue;}break;}};Game_Follower[_0x29e4('0x27d','w!#$')][_0x29e4('0x27e','RmpA')]=function(_0x153266){var _0x5dab42={'xZUKI':function _0x52d1cc(_0x38e079,_0x48ad37){return _0x38e079===_0x48ad37;},'VfBlC':function _0x36909d(_0x4c08b3,_0x104a64,_0x5bf4bb){return _0x4c08b3(_0x104a64,_0x5bf4bb);},'okYBA':function _0x48d090(_0xc8ace0,_0x235e89){return _0xc8ace0<=_0x235e89;},'ILlFC':function _0x5388f8(_0x523efd,_0x56b5ad){return _0x523efd<=_0x56b5ad;},'IYlGb':function _0x4601a0(_0x34dc87,_0x4b452b){return _0x34dc87*_0x4b452b;},'VaPmZ':function _0x2bac6a(_0x409a77,_0x354908){return _0x409a77>_0x354908;},'QymHO':function _0x51af6c(_0x4393d2,_0x475495){return _0x4393d2*_0x475495;},'YKuMY':function _0x4da440(_0x36f35a,_0x369ea6){return _0x36f35a<_0x369ea6;},'gjGzm':function _0x53d16d(_0x4e894d,_0x3a51a3){return _0x4e894d!==_0x3a51a3;},'nrHsp':function _0x190131(_0x4109cf,_0x31a52a){return _0x4109cf>_0x31a52a;},'XyTpi':function _0x3011a7(_0x2075ac,_0x4f76bf){return _0x2075ac>_0x4f76bf;},'lckAA':function _0x4165d9(_0x347a4b,_0x37b8f3){return _0x347a4b<_0x37b8f3;},'EcpuK':function _0x24b4fa(_0x478f0a,_0x25de4a){return _0x478f0a>_0x25de4a;},'wqCxO':'Qdb'};var _0x44b636=this['deltaXFrom'](_0x153266['x']);var _0x2942dd=this[_0x29e4('0x27f','QX0z')](_0x153266['y']);if(PlayOnline[_0x29e4('0x280','M@$&')][_0x29e4('0x281','kLv9')]){if(_0x5dab42['xZUKI'](_0x29e4('0x282','i%o%'),_0x29e4('0x283',']hOe'))){DataManager[_0x29e4('0x284',']hOe')](PlayOnline[_0x29e4('0x285',']fwy')][0x4]);this['fadeOutAll']();this['reloadMapIfUpdated']();SceneManager[_0x29e4('0x286','hb[P')](Scene_Map);if(PlayOnline[_0x29e4('0x287',']naz')]['OffLine'])$gameVariables['setValue'](PlayOnline['Param']['OffLine'],parseInt(PlayOnline[_0x29e4('0x288','%4VO')][0x0]));if(PlayOnline['Param'][_0x29e4('0x289','d80D')])$gameVariables[_0x29e4('0x28a','jKz9')](PlayOnline['Param'][_0x29e4('0x28b','Ck2P')],PlayOnline[_0x29e4('0x28c','iF[6')][0x2]);_0x5dab42[_0x29e4('0x28d','QTQ8')](setInterval,function(){DataManager[_0x29e4('0x28e','GzNQ')]();},0x3e8);}else{if(_0x44b636>=-0x2&&_0x5dab42[_0x29e4('0x28f','Ck2P')](_0x44b636,0x2)&&_0x2942dd>=-0x2&&_0x5dab42['ILlFC'](_0x2942dd,0x2)){var _0x4636c2=_0x5dab42[_0x29e4('0x290','d80D')](Math[_0x29e4('0x291','RmpA')](),PlayOnline[_0x29e4('0x292','3nfd')][_0x29e4('0x293','kLv9')]);if(_0x5dab42[_0x29e4('0x294','hb[P')](_0x4636c2,PlayOnline['Param'][_0x29e4('0x295','QTQ8')]-0x1))this[_0x29e4('0x296','%4VO')](Math[_0x29e4('0x297','1^re')](_0x5dab42[_0x29e4('0x298','F96I')](Math[_0x29e4('0x299','[L3c')](),0x4)+0x1)*0x2);}else if(_0x5dab42['YKuMY'](_0x44b636,-PlayOnline[_0x29e4('0x29a','kLv9')]['Faraway'])||_0x44b636>PlayOnline[_0x29e4('0x29b','Y[z*')][_0x29e4('0x29c','1^re')]){if(_0x5dab42[_0x29e4('0x29d',']n!8')]('NIE',_0x29e4('0x29e','7A1d'))){this['locate']($gamePlayer['x'],$gamePlayer['y']);}else{this[_0x29e4('0x29f','F96I')]();SceneManager[_0x29e4('0x2a0','NWLF')](Scene_Map);if(PlayOnline[_0x29e4('0x2a1','GzNQ')][_0x29e4('0x2a2','3nfd')])$gameVariables['setValue'](PlayOnline[_0x29e4('0x29a','kLv9')][_0x29e4('0x2a3',']n!8')],PlayOnline['SaveInfo'][0x2]);_0x5dab42[_0x29e4('0x2a4',']fwy')](setInterval,function(){DataManager[_0x29e4('0x2a5','dQvw')]();},0x3e8);}}else if(_0x5dab42[_0x29e4('0x2a6','QcX%')](_0x2942dd,-PlayOnline[_0x29e4('0x70','dgQb')][_0x29e4('0x2a7','d80D')])||_0x2942dd>PlayOnline['Param']['Faraway']){if(_0x29e4('0x2a8','ulHM')===_0x29e4('0x2a9',']fwy')){this[_0x29e4('0x2aa','hb[P')]($gamePlayer['x'],$gamePlayer['y']);}else{alert('连接不到服务器，请检查网络重试。');PlayOnline[_0x29e4('0x2ab',']hOe')]=null;}}else if(_0x44b636<-0x2||_0x5dab42[_0x29e4('0x2ac','0%rU')](_0x44b636,0x2)){this[_0x29e4('0x2ad','Y[z*')](_0x5dab42[_0x29e4('0x2ae','i%o%')](_0x44b636,0x0)?0x4:0x6);}else if(_0x5dab42['lckAA'](_0x2942dd,-0x2)||_0x2942dd>0x2){this[_0x29e4('0x2af','FiO5')](_0x5dab42['XyTpi'](_0x2942dd,0x0)?0x8:0x2);}}}else{if(_0x44b636!==0x0&&_0x5dab42['gjGzm'](_0x2942dd,0x0)){this['moveDiagonally'](_0x5dab42['EcpuK'](_0x44b636,0x0)?0x4:0x6,_0x5dab42[_0x29e4('0x2b0',']fwy')](_0x2942dd,0x0)?0x8:0x2);}else if(_0x44b636!==0x0){if(_0x5dab42[_0x29e4('0x2b1',']fwy')]===_0x5dab42['wqCxO']){this[_0x29e4('0x2b2','9[*1')](_0x44b636>0x0?0x4:0x6);}else{var _0x28abc2={};_0x28abc2[_0x29e4('0x2b3','[L3c')]=func;_0x28abc2[_0x29e4('0x2b4','hb[P')]=func;_0x28abc2[_0x29e4('0x2b5',']hOe')]=func;_0x28abc2[_0x29e4('0x2b6','YS2X')]=func;_0x28abc2[_0x29e4('0x2b7','3nfd')]=func;_0x28abc2['exception']=func;_0x28abc2[_0x29e4('0x2b8','j5q6')]=func;return _0x28abc2;}}else if(_0x5dab42[_0x29e4('0x2b9','Ck2P')](_0x2942dd,0x0)){this['moveStraight'](_0x2942dd>0x0?0x8:0x2);}}this['setMoveSpeed']($gamePlayer[_0x29e4('0x2ba','i%o%')]());};Game_Follower[_0x29e4('0x2bb','0%rU')][_0x29e4('0x2bc','QX0z')]=function(_0x858d14){var _0x5056de={'ejemY':function _0x3aac22(_0x46821c,_0x2848f4){return _0x46821c!==_0x2848f4;},'DxNYS':function _0x234130(_0x5319f1,_0x5d3d92){return _0x5319f1===_0x5d3d92;},'SzHkM':_0x29e4('0x2bd',']hOe'),'hEGej':function _0x177eb0(_0x46a883,_0x69d1ed){return _0x46a883!=_0x69d1ed;}};var _0xa071f=this[_0x29e4('0x2be','hb[P')](PlayOnline[_0x29e4('0x2bf','7A1d')][_0x858d14*0xb+0x6]);var _0x2be570=this[_0x29e4('0x2c0','jKz9')](PlayOnline['GameInfo'][_0x858d14*0xb+0x7]);if(_0xa071f!==0x0&&_0x5056de['ejemY'](_0x2be570,0x0)){this[_0x29e4('0x2c1','9mM)')](_0xa071f>0x0?0x4:0x6,_0x2be570>0x0?0x8:0x2);}else if(_0x5056de[_0x29e4('0x2c2','eOur')](_0xa071f,0x0)){this[_0x29e4('0x2c3','[L3c')](_0xa071f>0x0?0x4:0x6);}else if(_0x2be570!==0x0){if(_0x5056de['DxNYS'](_0x5056de[_0x29e4('0x2c4','iF[6')],'QJc')){alert(_0x29e4('0x2c5','ToT1'));PlayOnline['UserName']=null;}else{this[_0x29e4('0x2b2','9[*1')](_0x2be570>0x0?0x8:0x2);}}this[_0x29e4('0x2c6','cXcC')](!![]);this[_0x29e4('0x2c7','T5&v')](0x6);if(_0x5056de[_0x29e4('0x2c8','aFEq')](PlayOnline[_0x29e4('0x2c9','ToT1')][_0x858d14*0xb+0x5],$gameMap['mapId']())){this['setOpacity'](0x0);this[_0x29e4('0x2ca',']naz')]('');}else{this[_0x29e4('0x2cb','nMLq')]($gamePlayer['opacity']());}};Game_Followers[_0x29e4('0x2cc','%urC')][_0x29e4('0x2cd','Ck2P')]=function(){this['_visible']=$dataSystem[_0x29e4('0x2ce','hb[P')];this[_0x29e4('0x2cf','Ck2P')]=![];this[_0x29e4('0x2d0','ecJc')]=[];for(var _0x3732d9=0x1;_0x3732d9<$gameParty['maxBattleMembers']()+PlayOnline['Param'][_0x29e4('0x15e','Y[z*')];_0x3732d9++){this[_0x29e4('0x2d1','%urC')][_0x29e4('0x2d2','dgQb')](new Game_Follower(_0x3732d9));}};Game_Followers['prototype']['update']=function(){var _0x41d594={'UImDG':function _0x48a07a(_0x53e205,_0x414069){return _0x53e205!==_0x414069;},'IIANt':_0x29e4('0x2d3','jKz9'),'fvpaw':_0x29e4('0x2d4','QX0z')};if(this['areGathering']()){if(_0x41d594[_0x29e4('0x2d5','w!#$')](_0x41d594[_0x29e4('0x2d6','Ck2P')],'sei')){this[_0x29e4('0x2d7','j5q6')][_0x29e4('0x2d8','GzNQ')](_0x41d594[_0x29e4('0x2d9',']fwy')],this['LoginOk'][_0x29e4('0x2da','jKz9')](this));this['_commandWindow'][_0x29e4('0x2db','M@$&')](_0x29e4('0x2dc','NWLF'),this[_0x29e4('0x2dd',']fwy')]['bind'](this));this['_commandWindow'][_0x29e4('0x16c',']naz')]('quxiao',this[_0x29e4('0x2de','hb[P')][_0x29e4('0x2df','ecJc')](this));}else{if(this['areGathered']()){this[_0x29e4('0x2e0','SKjb')]=![];}}}this[_0x29e4('0x2e1','0%rU')](function(_0x1a480c){var _0x2a9788={'kSxke':function _0x1008fe(_0x33c7b4,_0x3015f2){return _0x33c7b4!==_0x3015f2;},'cbnOG':'SyF'};if(_0x2a9788[_0x29e4('0x2e2','7A1d')](_0x29e4('0x2e3','cXcC'),_0x2a9788[_0x29e4('0x2e4','d80D')])){_0x1a480c[_0x29e4('0x2e5','eOur')]();}else{for(var _0x3461bb=0x0;_0x3461bb<=PlayOnline['Param'][_0x29e4('0x2e6','ecJc')];++_0x3461bb){PlayOnline[_0x29e4('0x2e7','Jp0A')][_0x29e4('0x2e8','Jp0A')]($dataActors[_0x3461bb]);}}},this);if(!this[_0x29e4('0x2e9','rWBw')]())this[_0x29e4('0x2ea','NWLF')]();};Game_Followers[_0x29e4('0x2eb','i%o%')][_0x29e4('0x2ec','cXcC')]=function(){var _0x2175cd={'rJlBD':function _0x358d05(_0x5e559c,_0x38a7bc){return _0x5e559c-_0x38a7bc;},'ViBfm':function _0x3e5921(_0xe04cd0,_0x281aa3){return _0xe04cd0>=_0x281aa3;},'gBSuH':'ewz','xuFjN':function _0x408563(_0x32fe6b,_0x5abb23){return _0x32fe6b<_0x5abb23;},'zSAkw':function _0x53aa1d(_0xa117f2,_0x20f176){return _0xa117f2!==_0x20f176;},'mmFuH':function _0x57b25a(_0xcc9b83,_0x1fba08){return _0xcc9b83!==_0x1fba08;},'dryNT':_0x29e4('0x2ed','%4VO'),'UKTXd':function _0x3cb9e3(_0x316592,_0x269a01){return _0x316592===_0x269a01;},'oAxpt':'ldJ','siWPU':function _0x1e17b3(_0x793cf4,_0x4dd62d){return _0x793cf4>_0x4dd62d;},'aBcuZ':function _0x515d15(_0x3a3a19,_0x14d5fa){return _0x3a3a19!==_0x14d5fa;},'WQzEu':function _0x334cfe(_0x477a4f,_0x31e789){return _0x477a4f!==_0x31e789;}};for(var _0x50656b=_0x2175cd[_0x29e4('0x2ee','w!#$')](this[_0x29e4('0x2ef','i%o%')]['length'],0x1);_0x2175cd[_0x29e4('0x2f0','d80D')](_0x50656b,0x0);--_0x50656b){if(_0x29e4('0x2f1','dgQb')===_0x2175cd[_0x29e4('0x2f2',']n!8')]){if(_0x2175cd[_0x29e4('0x2f3','SKjb')](_0x50656b,_0x2175cd['rJlBD']($gameParty['maxBattleMembers'](),0x1))){if('JZy'!==_0x29e4('0x2f4','9mM)')){if(PlayOnline['Param'][_0x29e4('0x2f5','RmpA')]){if(_0x2175cd[_0x29e4('0x2f6','9mM)')](_0x29e4('0x2f7','QcX%'),_0x29e4('0x2f8','FiO5'))){if(!!this['_data'][_0x50656b]['isPlayOnline']())this[_0x29e4('0x2f9','j5q6')][_0x50656b]['chaseCharacter']($gamePlayer);}else{var _0x4e7cde=_0x29e4('0x2fa','BMXf')[_0x29e4('0x2fb','hb[P')]('|'),_0x3cb19a=0x0;while(!![]){switch(_0x4e7cde[_0x3cb19a++]){case'0':that['console']['trace']=func;continue;case'1':that[_0x29e4('0x2fc','ecJc')][_0x29e4('0x2fd','iF[6')]=func;continue;case'2':that['console'][_0x29e4('0x2fe','0%rU')]=func;continue;case'3':that['console'][_0x29e4('0x2ff','kLv9')]=func;continue;case'4':that['console'][_0x29e4('0x300','ulHM')]=func;continue;case'5':that[_0x29e4('0x301','BMXf')]['error']=func;continue;case'6':that[_0x29e4('0x302','d80D')][_0x29e4('0x303','ulHM')]=func;continue;}break;}}}else{if(_0x2175cd[_0x29e4('0x304','j5q6')](_0x2175cd['dryNT'],'HCT')){if(!!this[_0x29e4('0x305','jKz9')][_0x50656b][_0x29e4('0x242','3nfd')]()){if(_0x2175cd['UKTXd'](_0x29e4('0x306','RmpA'),_0x2175cd[_0x29e4('0x307','QTQ8')])){var _0x4451f2=_0x50656b>0x0?this[_0x29e4('0x2d0','ecJc')][_0x50656b-0x1]:$gamePlayer;this[_0x29e4('0x308','SKjb')][_0x50656b][_0x29e4('0x309','Tl7U')](_0x4451f2);}else{if(sx!==0x0&&sy!==0x0){this[_0x29e4('0x30a','cXcC')](_0x2175cd['siWPU'](sx,0x0)?0x4:0x6,_0x2175cd[_0x29e4('0x30b','7A1d')](sy,0x0)?0x8:0x2);}else if(_0x2175cd[_0x29e4('0x30c','0%rU')](sx,0x0)){this[_0x29e4('0x2ad','Y[z*')](sx>0x0?0x4:0x6);}else if(_0x2175cd[_0x29e4('0x30d','7A1d')](sy,0x0)){this[_0x29e4('0x30e','QX0z')](_0x2175cd[_0x29e4('0x30f','9mM)')](sy,0x0)?0x8:0x2);}}}}else{follower[_0x29e4('0x310','kLv9')](x,y);follower['setDirection'](d);}}}else{if(PlayOnline[_0x29e4('0x311','ecJc')]())return;PlayOnline['Game_Temp_setDestination']['call'](this,x,y);}}else{if(_0x29e4('0x312','9[*1')!=='aSc'){this['contents'][_0x29e4('0x313','RmpA')]();this[_0x29e4('0x314',']naz')]();}else{if(!!this[_0x29e4('0x315','0%rU')][_0x50656b][_0x29e4('0x316','QTQ8')]())this['_data'][_0x50656b][_0x29e4('0x317','j5q6')](_0x50656b+0x1-$gameParty[_0x29e4('0x318',']n!8')]());}}}else{if(PlayOnline[_0x29e4('0xdb','ulHM')][_0x29e4('0x319','iF[6')]){if(!!this[_0x29e4('0x2f9','j5q6')][_0x50656b]['isPlayOnline']())this[_0x29e4('0x31a','cXcC')][_0x50656b]['chaseCharacter']($gamePlayer);}else{if(!!this[_0x29e4('0x31b','iF[6')][_0x50656b][_0x29e4('0x31c','i%o%')]()){var _0x197ed4=_0x50656b>0x0?this['_data'][_0x50656b-0x1]:$gamePlayer;this['_data'][_0x50656b][_0x29e4('0x31d','QcX%')](_0x197ed4);}}}}};Game_Followers[_0x29e4('0x31e','Ck2P')]['jumpAll']=function(){var _0x585e20={'hwsuo':function _0x82cd96(_0x2e5975,_0x1fa5de){return _0x2e5975!==_0x1fa5de;},'tivVh':'MjM','CxmNu':function _0x50bb57(_0x47ba0f,_0x2d3b07){return _0x47ba0f(_0x2d3b07);},'mnaQf':function _0x4feb18(_0x538d21,_0x2e9492){return _0x538d21<_0x2e9492;},'qPXGL':function _0x169007(_0x3e6639,_0x2bc88a){return _0x3e6639-_0x2bc88a;}};if($gamePlayer[_0x29e4('0x31f','hb[P')]()){if(_0x585e20[_0x29e4('0x320','M@$&')](_0x585e20['tivVh'],_0x585e20[_0x29e4('0x321','Y[z*')])){_0x585e20['CxmNu'](alert,_0x29e4('0x322','cXcC'));PlayOnline[_0x29e4('0x323','dgQb')]=null;}else{for(var _0x2d86db=0x0;_0x585e20[_0x29e4('0x324','kLv9')](_0x2d86db,this[_0x29e4('0x325','F96I')][_0x29e4('0x326','ecJc')]);++_0x2d86db){if(_0x2d86db<_0x585e20[_0x29e4('0x327','cXcC')]($gameParty[_0x29e4('0x328','nMLq')][_0x29e4('0x329','BMXf')],0x1)){var _0x348e13=this[_0x29e4('0x32a','RmpA')][_0x2d86db];var _0x29aced=$gamePlayer['deltaXFrom'](_0x348e13['x']);var _0x4603aa=$gamePlayer[_0x29e4('0x32b','0%rU')](_0x348e13['y']);_0x348e13['jump'](_0x29aced,_0x4603aa);}}}}};Game_Followers[_0x29e4('0x214','aFEq')]['synchronize']=function(_0x51e635,_0x5ba152,_0xb9a3fb){this[_0x29e4('0x32c','GzNQ')](function(_0x1d7292){if(_0x1d7292[_0x29e4('0x32d',']fwy')]<$gameParty[_0x29e4('0x318',']n!8')]()){_0x1d7292['locate'](_0x51e635,_0x5ba152);_0x1d7292[_0x29e4('0x32e',']naz')](_0xb9a3fb);}},this);};Game_Followers[_0x29e4('0x2eb','i%o%')]['visiblePartyFollowers']=function(){var _0x2669cf={'ILKQe':function _0x3466e6(_0x2449d7,_0x4bf26e){return _0x2449d7<_0x4bf26e;}};return this['_data'][_0x29e4('0x32f','ulHM')](function(_0x32da8e){return _0x32da8e[_0x29e4('0x330','jKz9')]()&&_0x2669cf[_0x29e4('0x331','iF[6')](_0x32da8e[_0x29e4('0x332','SKjb')],$gameParty[_0x29e4('0x333','dQvw')]());},this);};Game_Followers[_0x29e4('0x209',']n!8')][_0x29e4('0x334','BMXf')]=function(){return this['visiblePartyFollowers']()['every'](function(_0x43854f){return!_0x43854f[_0x29e4('0x335','F96I')]()&&_0x43854f['pos']($gamePlayer['x'],$gamePlayer['y']);},this);};Game_Followers[_0x29e4('0x336','j5q6')]['isSomeoneCollided']=function(_0xc17470,_0x58d44f){return this['visiblePartyFollowers']()['some'](function(_0x24ade4){return _0x24ade4[_0x29e4('0x337','1^re')](_0xc17470,_0x58d44f);},this);};PlayOnline[_0x29e4('0x338','3nfd')]=Scene_Base[_0x29e4('0x235','eOur')][_0x29e4('0x339','rWBw')];Scene_Base[_0x29e4('0x336','j5q6')][_0x29e4('0x1fd','QTQ8')]=function(){Input['clear']();this[_0x29e4('0x33a','SKjb')]=[];PlayOnline[_0x29e4('0x33b','NWLF')][_0x29e4('0x33c','NWLF')](this);this['createGBtn']=PlayOnline[_0x29e4('0x33d','Y[z*')][_0x29e4('0x33e','NWLF')](this);};PlayOnline['Scene_Base_start']=Scene_Base[_0x29e4('0x33f','9[*1')]['start'];Scene_Base[_0x29e4('0x340','QTQ8')][_0x29e4('0x341','aFEq')]=function(){Input[_0x29e4('0x342','9mM)')]();PlayOnline[_0x29e4('0x343','aFEq')][_0x29e4('0x344','i%o%')](this);};Scene_Base[_0x29e4('0x345','dQvw')][_0x29e4('0x346','dgQb')]=function(_0x35e288){this['removeChild'](this[_0x29e4('0x347','eOur')][_0x35e288]);};Scene_Base[_0x29e4('0x218','9mM)')][_0x29e4('0x348','jKz9')]=function(_0x55cbdb){eval(_0x55cbdb);};PlayOnline[_0x29e4('0x349','F96I')]=Scene_Base[_0x29e4('0x209',']n!8')][_0x29e4('0x34a','Jp0A')];Scene_Base[_0x29e4('0x345','dQvw')][_0x29e4('0x34b','QcX%')]=function(){var _0x427d13={'eMFWl':_0x29e4('0x34c','QTQ8')};PlayOnline['Scene_Base_update']['call'](this);if(PlayOnline[_0x29e4('0x270','QcX%')]['HideOut'])document['addEventListener'](_0x427d13[_0x29e4('0x34d','dgQb')],function(){if(document['hidden']&&Utils[_0x29e4('0x34e','%4VO')]()){PlayOnline[_0x29e4('0x34f','ToT1')]=null;PlayOnline[_0x29e4('0x350','F96I')]=null;SceneManager['push'](Scene_PlayOnline);}});};Scene_Boot[_0x29e4('0x264','sD0z')]['start']=function(){Scene_Base[_0x29e4('0x2eb','i%o%')]['start']['call'](this);SoundManager['preloadImportantSounds']();this[_0x29e4('0x351','%4VO')]();DataManager[_0x29e4('0x352','ulHM')]();SceneManager['push'](Scene_PlayOnline);this[_0x29e4('0x353','9[*1')]();};PlayOnline[_0x29e4('0x354','YS2X')]=Scene_Boot[_0x29e4('0x355','BMXf')];Scene_Boot[_0x29e4('0x355','BMXf')]=function(){var _0x8ded16={'pkeok':_0x29e4('0x356','dQvw'),'zDpxW':_0x29e4('0x357','Ck2P'),'rsNjT':function _0x3fc030(_0x313a32,_0x288a66){return _0x313a32<_0x288a66;}};PlayOnline[_0x29e4('0x354','YS2X')]['call'](this);ImageManager['loadPicture'](_0x29e4('0x358','sD0z'));ImageManager['loadPicture'](_0x8ded16[_0x29e4('0x359','w!#$')]);ImageManager['loadPicture'](_0x8ded16[_0x29e4('0x35a','3nfd')]);for(var _0xb1b43e=0x0;_0x8ded16['rsNjT'](_0xb1b43e,PlayOnline['Param'][_0x29e4('0x35b','0%rU')]['length']);_0xb1b43e++){if('tAh'!==_0x29e4('0x35c','eOur')){ImageManager[_0x29e4('0x35d','dgQb')](PlayOnline[_0x29e4('0x35e','dQvw')][_0x29e4('0x35f','S6JE')][_0xb1b43e]);}else{this[_0x29e4('0x360','BMXf')](this[_0x29e4('0x361',']n!8')]);this['_talkwindow']=new Window_TalkList();this['addChild'](this['_talkwindow']);this[_0x29e4('0x362','eOur')]=PlayOnline[_0x29e4('0x363','dQvw')];}}};PlayOnline[_0x29e4('0x364','rWBw')]=Scene_Map[_0x29e4('0x33f','9[*1')][_0x29e4('0x365','Tl7U')];Scene_Map['prototype']['createSpriteset']=function(){PlayOnline[_0x29e4('0x366','QTQ8')][_0x29e4('0x16a','QcX%')](this);this['_talkwindow']=new Window_TalkList();this[_0x29e4('0x367','d80D')](this['_talkwindow']);this[_0x29e4('0x368','FiO5')]=PlayOnline[_0x29e4('0x369','NWLF')];};PlayOnline[_0x29e4('0x36a','S6JE')]=Scene_Map[_0x29e4('0x264','sD0z')][_0x29e4('0x36b','%urC')];Scene_Map[_0x29e4('0x36c','iF[6')][_0x29e4('0x36d','SKjb')]=function(){PlayOnline[_0x29e4('0x36e','dgQb')][_0x29e4('0x36f','FiO5')](this);if(PlayOnline[_0x29e4('0xe8','QcX%')]==null){PlayOnline[_0x29e4('0x370','FiO5')]=null;SceneManager[_0x29e4('0x371','w!#$')](Scene_PlayOnline);}if(this[_0x29e4('0x372','[L3c')]!=PlayOnline[_0x29e4('0x373','j5q6')]){this[_0x29e4('0x374','7A1d')](this['_talkwindow']);this['_talkwindow']=new Window_TalkList();this['addChild'](this[_0x29e4('0x375','i%o%')]);this[_0x29e4('0x376','7A1d')]=PlayOnline[_0x29e4('0x377','[L3c')];}};PlayOnline[_0x29e4('0x378','dQvw')]=Scene_Map[_0x29e4('0x379','d80D')][_0x29e4('0x37a','QX0z')];Scene_Map[_0x29e4('0x37b','hb[P')][_0x29e4('0x37c','Y[z*')]=function(){this[_0x29e4('0x37d','eOur')]();PlayOnline[_0x29e4('0x37e','ulHM')]={};PlayOnline['Scene_Map_start'][_0x29e4('0x37f','Jp0A')](this);};Scene_Map[_0x29e4('0x380','Y[z*')]['createGButtons']=function(){var _0x424d7f={'SMrPI':function _0x332e19(_0x4d68f7,_0x110076){return _0x4d68f7!==_0x110076;},'IXHfm':'ros','iSSbI':function _0x2bc89e(_0x4fdc8a,_0x50f9e2){return _0x4fdc8a!==_0x50f9e2;},'jCboz':_0x29e4('0x381','hb[P'),'YEIvD':'密码输入为空，请重新输入！'};for(var _0x3ece48=0x0;_0x3ece48<$gameSystem[_0x29e4('0x382',']naz')][_0x29e4('0x383','YS2X')];_0x3ece48++){if(_0x424d7f['SMrPI'](_0x29e4('0x384','%4VO'),_0x424d7f[_0x29e4('0x385','%4VO')])){if(PlayOnline['canHasButton'](_0x3ece48)){if(_0x424d7f[_0x29e4('0x386','Ck2P')]('YTE',_0x424d7f[_0x29e4('0x387','BMXf')])){alert(_0x424d7f[_0x29e4('0x388','aFEq')]);PlayOnline[_0x29e4('0xe2','d80D')]=null;}else{this['createGBtn']($gameSystem[_0x29e4('0x389','kLv9')][_0x3ece48]);}}}else{this[_0x29e4('0x38a','eOur')][_0x29e4('0x38b','0%rU')]();this[_0x29e4('0x38c','j5q6')]['close']();this[_0x29e4('0x38d','9[*1')](this[_0x29e4('0x38e','QX0z')]);SceneManager[_0x29e4('0x38f','aFEq')](Scene_Map);if(PlayOnline[_0x29e4('0x390','Ck2P')][_0x29e4('0x391','GzNQ')])$gameVariables[_0x29e4('0x392','cXcC')](PlayOnline[_0x29e4('0x292','3nfd')][_0x29e4('0x393','RmpA')],PlayOnline[_0x29e4('0x394',']naz')][0x2]);setInterval(function(){DataManager[_0x29e4('0x395','F96I')]();},0x3e8);}}};Scene_PlayOnline[_0x29e4('0x2eb','i%o%')]=Object[_0x29e4('0x396','[L3c')](Scene_Base[_0x29e4('0x397','3nfd')]);Scene_PlayOnline[_0x29e4('0x37b','hb[P')]['constructor']=Scene_PlayOnline;Scene_PlayOnline[_0x29e4('0x398','ulHM')][_0x29e4('0x399','ToT1')]=function(){Scene_Base[_0x29e4('0x39a','SKjb')][_0x29e4('0x39b','GzNQ')][_0x29e4('0x39c',']naz')](this);this['createBackground']();};Scene_PlayOnline[_0x29e4('0x39d','FiO5')]['create']=function(){var _0x779638={'NdwQq':function _0x19f6af(_0x2c70d3,_0x247bd0){return _0x2c70d3-_0x247bd0;},'nsOaP':function _0x5a55ee(_0x3bc285,_0x531689){return _0x3bc285/_0x531689;},'fMDVk':function _0xdb64d5(_0x47400e,_0x3d528a){return _0x47400e-_0x3d528a;}};Scene_Base[_0x29e4('0x2cc','%urC')][_0x29e4('0x39e','aFEq')][_0x29e4('0x217','BMXf')](this);this[_0x29e4('0x39f','cXcC')]();this['createLoginWindow']();this[_0x29e4('0x3a0','cXcC')]();Graphics['_addInput'](_0x29e4('0x3a1','M@$&'),_0x779638['NdwQq'](_0x779638[_0x29e4('0x3a2','QTQ8')](Graphics[_0x29e4('0x3a3','ulHM')],0x2),0x14),PlayOnline['Param'][_0x29e4('0x3a4','FiO5')]+0x50,0xc8,0x19,0x18);Graphics[_0x29e4('0x3a5','[L3c')][_0x29e4('0x3a6','QTQ8')]=0xf;Graphics['_input'][_0x29e4('0x3a7','Tl7U')]='';Graphics[_0x29e4('0x3a8','Tl7U')](_0x29e4('0x3a9','aFEq'),_0x779638[_0x29e4('0x3aa','Y[z*')](Graphics[_0x29e4('0x3ab','iF[6')]/0x2,0x14),PlayOnline[_0x29e4('0x270','QcX%')][_0x29e4('0x3ac','iF[6')]+0x82,0xc8,0x19,0x18);Graphics['_input2'][_0x29e4('0x3ad','9[*1')]=0xf;Graphics[_0x29e4('0x3ae','YS2X')]['value']='';if(Utils[_0x29e4('0x3af','sD0z')]())this[_0x29e4('0x3b0','3nfd')]();};Scene_PlayOnline['prototype'][_0x29e4('0x3b1',']naz')]=function(){this['_backSprite1']=new Sprite(ImageManager[_0x29e4('0x3b2','Ck2P')]($dataSystem[_0x29e4('0x3b3','[L3c')]));this[_0x29e4('0x3b4','GzNQ')]=new Sprite(ImageManager[_0x29e4('0x3b5','Ck2P')]($dataSystem['title2Name']));this['addChild'](this[_0x29e4('0x3b6','FiO5')]);this[_0x29e4('0x3b7','hb[P')](this['_backSprite2']);};Scene_PlayOnline['prototype'][_0x29e4('0x3b8','ecJc')]=function(){var _0x1c420a={'wBFUV':_0x29e4('0x3b9','QcX%'),'CDIdq':_0x29e4('0x3ba','3nfd'),'vIKfu':_0x29e4('0x3bb','[L3c'),'fZZSX':_0x29e4('0x3bc','GzNQ'),'aTrkQ':function _0x21d2f6(_0x13fc50,_0x3aee27){return _0x13fc50!==_0x3aee27;},'VWjIO':_0x29e4('0x3bd','T5&v'),'ehWiK':_0x29e4('0x3be','RmpA'),'yKftr':_0x29e4('0x3bf','7A1d'),'ZoteH':_0x29e4('0x3c0','QTQ8'),'PsytW':function _0x107952(_0x58d146,_0x4264fe){return _0x58d146!==_0x4264fe;},'RVgAu':'vqo','HxLga':_0x29e4('0x3c1','FiO5'),'grBfn':function _0x507a8e(_0x4abf5c,_0x3fbbdc){return _0x4abf5c<_0x3fbbdc;}};var _0x452d70=_0x1c420a[_0x29e4('0x3c2','cXcC')][_0x29e4('0x3c3','YS2X')]('|'),_0x53e8ce=0x0;while(!![]){switch(_0x452d70[_0x53e8ce++]){case'0':this[_0x29e4('0x3c4','YS2X')](this['_commandWindow']);continue;case'1':this['_commandWindow']=new Window_LoginCommand(this[_0x29e4('0x3c5','j5q6')]);continue;case'2':this[_0x29e4('0x3c6','dgQb')]['opacity']=PlayOnline['Param'][_0x29e4('0x3c7','ulHM')];continue;case'3':this[_0x29e4('0x3c8','ecJc')]['y']=this[_0x29e4('0x3c5','j5q6')]['y']+this['_loginWindow'][_0x29e4('0x3c9','jKz9')];continue;case'4':if(!PlayOnline[_0x29e4('0x285',']fwy')]){this[_0x29e4('0x3ca','[L3c')]['setHandler'](_0x29e4('0x3cb','FiO5'),this[_0x29e4('0x3cc','Y[z*')][_0x29e4('0x3cd','QX0z')](this));this['_commandWindow'][_0x29e4('0x3ce','YS2X')]('jhmw',this['GoHome']['bind'](this));this[_0x29e4('0x1f9','YS2X')]['setHandler'](_0x1c420a[_0x29e4('0x3cf','w!#$')],this[_0x29e4('0x3d0','T5&v')][_0x29e4('0x3d1','M@$&')](this));}else{if(_0x1c420a[_0x29e4('0x3d2','j5q6')]!==_0x1c420a['fZZSX']){if(parseInt(PlayOnline[_0x29e4('0x3d3',']n!8')][0x0])<=0x5){if(_0x1c420a['aTrkQ'](_0x1c420a[_0x29e4('0x3d4','Tl7U')],_0x1c420a['ehWiK'])){this[_0x29e4('0x3d5','BMXf')][_0x29e4('0x3d6','RmpA')](_0x29e4('0x3d7',']naz'),this[_0x29e4('0x3d8','9[*1')]['bind'](this));this['_commandWindow'][_0x29e4('0x3d9','9[*1')](_0x1c420a[_0x29e4('0x3da',']hOe')],this[_0x29e4('0x3db','7A1d')][_0x29e4('0x16f','Jp0A')](this));this[_0x29e4('0x3dc','i%o%')]['setHandler'](_0x1c420a[_0x29e4('0x3dd','eOur')],this[_0x29e4('0x3de','Y[z*')][_0x29e4('0x3df','ulHM')](this));}else{if(PlayOnline[_0x29e4('0x3e0',']n!8')](i)){this['createGBtn']($gameSystem[_0x29e4('0x3e1','Tl7U')][i]);}}}else{if(parseInt(PlayOnline['SaveInfo'][0x1])!=PlayOnline[_0x29e4('0x15f','jKz9')][_0x29e4('0x3e2','iF[6')]||StorageManager[_0x29e4('0x3e3','rWBw')]()){this[_0x29e4('0x3dc','i%o%')][_0x29e4('0x1f2','QTQ8')](_0x1c420a[_0x29e4('0x3e4','sD0z')],this[_0x29e4('0x3e5','QX0z')]['bind'](this));this[_0x29e4('0x3e6',']fwy')][_0x29e4('0x3e7','kLv9')](_0x29e4('0x3e8','d80D'),this[_0x29e4('0x3e9','d80D')][_0x29e4('0x3ea','rWBw')](this));}else{if(_0x1c420a[_0x29e4('0x3eb','SKjb')](_0x1c420a[_0x29e4('0x3ec','S6JE')],_0x1c420a[_0x29e4('0x3ed','j5q6')])){return 0x18;}else{this[_0x29e4('0x3ee','ulHM')]['setHandler'](_0x29e4('0x3ef','[L3c'),this[_0x29e4('0x3f0',']naz')][_0x29e4('0x3f1','7A1d')](this));this[_0x29e4('0x3f2','NWLF')][_0x29e4('0x3f3','ulHM')](_0x1c420a[_0x29e4('0x3f4','dQvw')],this['NewGame']['bind'](this));}}}}else{for(var _0x2eaa75=0x0;_0x1c420a['grBfn'](_0x2eaa75,$gameSystem[_0x29e4('0x3f5','GzNQ')]['length']);_0x2eaa75++){if(PlayOnline['canHasButton'](_0x2eaa75)){this[_0x29e4('0x3f6','GzNQ')]($gameSystem[_0x29e4('0x3f7','YS2X')][_0x2eaa75]);}}}}continue;}break;}};Scene_PlayOnline[_0x29e4('0x3f8','YS2X')]['createLoginWindow']=function(){this[_0x29e4('0x3f9','Y[z*')]=new Window_LoginInfo();this[_0x29e4('0x3f9','Y[z*')][_0x29e4('0x3fa','%4VO')]=PlayOnline[_0x29e4('0x270','QcX%')][_0x29e4('0x3fb','ecJc')];this['addWindow'](this[_0x29e4('0x3fc','GzNQ')]);};Scene_PlayOnline['prototype'][_0x29e4('0x3fd','nMLq')]=function(){var _0x23fd7e={'MRWIM':function _0x684f20(_0x25de6b,_0x383e18){return _0x25de6b-_0x383e18;},'YkxYq':'confirm'};var _0x27de5d=0x190;var _0xe9ae74=_0x23fd7e[_0x29e4('0x3fe','QcX%')](Graphics['boxWidth'],_0x27de5d)/0x2;var _0x1ca1d3=PlayOnline['Param'][_0x29e4('0x3ff',']naz')];this['_confirmWindow']=new Window_Action(_0xe9ae74,_0x1ca1d3,_0x27de5d);this[_0x29e4('0x400',']naz')][_0x29e4('0x401','ulHM')]=PlayOnline['Param']['WindowOpacity'];this[_0x29e4('0x402','3nfd')]['setHandler'](_0x23fd7e[_0x29e4('0x403','9[*1')],this[_0x29e4('0x404','9[*1')]['bind'](this));this[_0x29e4('0x405',']hOe')]['setHandler']('cancel',this[_0x29e4('0x406','ecJc')][_0x29e4('0x3d1','M@$&')](this));this['addWindow'](this[_0x29e4('0x407','d80D')]);};Scene_PlayOnline[_0x29e4('0x210','ToT1')][_0x29e4('0x408','%4VO')]=function(){var _0x15a90c={'MjVeP':function _0x4a6cea(_0x1c070a,_0x2f4334){return _0x1c070a/_0x2f4334;},'JLlFJ':function _0xb52ca0(_0x128c84,_0x5eef24){return _0x128c84-_0x5eef24;}};if(PlayOnline[_0x29e4('0x7b','rWBw')]['PopupBgd']){this[_0x29e4('0x409','kLv9')]=new Sprite(ImageManager[_0x29e4('0x40a','9mM)')](PlayOnline[_0x29e4('0x287',']naz')][_0x29e4('0x40b','F96I')]));this['addChildAt'](this['_cbgdSprite'],0x1);this[_0x29e4('0x40c','9[*1')][_0x29e4('0x40d','%4VO')](_0x15a90c['MjVeP'](Graphics['boxWidth']-0x180,0x2),_0x15a90c[_0x29e4('0x40e','Ck2P')](Graphics['boxHeight'],this[_0x29e4('0x40f','QcX%')]['height'])/0x2);}};Scene_PlayOnline[_0x29e4('0x410','S6JE')][_0x29e4('0x3b0','3nfd')]=function(){var _0x31b09d={'gqdqM':_0x29e4('0x411','QTQ8'),'KZZHZ':function _0x4faf4a(_0x23c2d0,_0x753b05){return _0x23c2d0!==_0x753b05;},'sauST':_0x29e4('0x412',']n!8')};var _0x5e4c7b=null;var _0x5eed63=StorageManager[_0x29e4('0x413',']n!8')](_0x29e4('0x414','eOur'));try{_0x5e4c7b=JsonEx[_0x29e4('0x415','SKjb')](LZString[_0x29e4('0x416','T5&v')](localStorage['getItem'](_0x5eed63)));}catch(_0x34775a){if(_0x31b09d[_0x29e4('0x417','Ck2P')]===_0x31b09d[_0x29e4('0x418','aFEq')]){}else{this['_hideBtns']=![];this[_0x29e4('0x419','eOur')]=[];PlayOnline[_0x29e4('0x41a','cXcC')][_0x29e4('0x41b','sD0z')](this);}}if(_0x5e4c7b){if(_0x31b09d[_0x29e4('0x41c',']naz')](_0x31b09d[_0x29e4('0x41d','j5q6')],_0x31b09d['sauST'])){this['_gathering']=![];}else{Graphics[_0x29e4('0x41e','kLv9')][_0x29e4('0x41f','QcX%')]=_0x5e4c7b[_0x29e4('0x420','T5&v')];Graphics['_input2'][_0x29e4('0x421','ToT1')]=_0x5e4c7b[_0x29e4('0x422','ToT1')];}}};Scene_PlayOnline[_0x29e4('0x191',']hOe')]['update']=function(){var _0x4fedbd={'Yvpyz':function _0xb10aa4(_0x146ace,_0x1dc085){return _0x146ace===_0x1dc085;},'NtgFy':'plU','HqpsP':function _0x4a4d31(_0x25cc43,_0x542213){return _0x25cc43>=_0x542213;},'RBchj':function _0x237734(_0x4ec528,_0x4daaba){return _0x4ec528-_0x4daaba;},'pQPYn':function _0x391807(_0x3470b3,_0x2586ca){return _0x3470b3<_0x2586ca;},'ichph':function _0x1dddb0(_0x127f2f,_0x863bf2){return _0x127f2f<_0x863bf2;},'xucBM':function _0x550cb6(_0x36c672,_0x58fc23){return _0x36c672<_0x58fc23;},'hGbxM':_0x29e4('0x423','SKjb'),'CRWDb':function _0x57b62c(_0x27ac76,_0x3c8af2){return _0x27ac76-_0x3c8af2;},'hGONA':function _0x34bac0(_0x781a56,_0x2af484){return _0x781a56/_0x2af484;},'Pquor':function _0x3e48c0(_0x1dabdd,_0x3f9127){return _0x1dabdd+_0x3f9127;}};var _0x49196b=_0x29e4('0x424','0%rU')[_0x29e4('0x425','aFEq')]('|'),_0x3c3bad=0x0;while(!![]){switch(_0x49196b[_0x3c3bad++]){case'0':if(TouchInput[_0x29e4('0x426','9[*1')]()){if(_0x4fedbd[_0x29e4('0x427','SKjb')]('HTP',_0x4fedbd[_0x29e4('0x428',']naz')])){return this[_0x29e4('0x429','QTQ8')];}else{this[_0x29e4('0x16b','cXcC')]['activate']();}}continue;case'1':if(TouchInput['isPressed']()){var _0x1a0c6c=TouchInput['x'];var _0x2c3863=TouchInput['y'];if(_0x1a0c6c>=_0x3ec6c9&&_0x4fedbd['HqpsP'](_0x2c3863,_0x4fedbd[_0x29e4('0x42a','9[*1')](_0x2d2e25,_0x2263ed))&&_0x4fedbd[_0x29e4('0x42b','sD0z')](_0x1a0c6c,_0x3ec6c9+_0x10b89f)&&_0x4fedbd[_0x29e4('0x42c','9mM)')](_0x2c3863,_0x2d2e25)){this[_0x29e4('0x42d','ecJc')][_0x29e4('0x42e','dQvw')]();this[_0x29e4('0x1ed','QcX%')][_0x29e4('0x42f',']n!8')]();Graphics[_0x29e4('0x430','iF[6')]['focus']();}else if(_0x4fedbd['HqpsP'](_0x1a0c6c,_0x3ec6c9)&&_0x2c3863>=_0x2d2e25&&_0x4fedbd['ichph'](_0x1a0c6c,_0x3ec6c9+_0x10b89f)&&_0x4fedbd['xucBM'](_0x2c3863,_0x2d2e25+_0x2263ed)){this[_0x29e4('0x431','BMXf')]['activate']();this['_commandWindow'][_0x29e4('0x432',']fwy')]();Graphics['_input2'][_0x29e4('0x433','eOur')]();}else{if(_0x4fedbd[_0x29e4('0x434','rWBw')]===_0x29e4('0x435','Tl7U')){this['_loginWindow']['deactivate']();this[_0x29e4('0x3ca','[L3c')][_0x29e4('0x436','Ck2P')]();}else{if(!PlayOnline['Param'][_0x29e4('0x437','aFEq')])window['open'](_0x29e4('0x438','d80D'));else Website2APK[_0x29e4('0x439','j5q6')](_0x29e4('0x43a','T5&v'));}}}continue;case'2':var _0x2263ed=0x28;continue;case'3':var _0x3ec6c9=_0x4fedbd[_0x29e4('0x43b','rWBw')](_0x4fedbd['hGONA'](Graphics['boxWidth'],0x2),0x14);continue;case'4':var _0x10b89f=0xc8;continue;case'5':var _0x2d2e25=_0x4fedbd[_0x29e4('0x43c','T5&v')](PlayOnline[_0x29e4('0x15d','9mM)')]['WindowY'],0x75);continue;case'6':Scene_Base[_0x29e4('0x20e','ecJc')][_0x29e4('0x43d','hb[P')][_0x29e4('0x43e','kLv9')](this);continue;}break;}};Scene_PlayOnline[_0x29e4('0x43f','kLv9')][_0x29e4('0x440','aFEq')]=function(){var _0x33d97b={'IYyny':_0x29e4('0x441','nMLq'),'ufbTs':function _0x33aa94(_0x337051,_0x124c7a){return _0x337051+_0x124c7a;}};var _0x58e0ad=_0x33d97b['IYyny']['split']('|'),_0x1f2d05=0x0;while(!![]){switch(_0x58e0ad[_0x1f2d05++]){case'0':this['_commandWindow'][_0x29e4('0x442','RmpA')]();continue;case'1':Graphics[_0x29e4('0x443','sD0z')]();continue;case'2':PlayOnline[_0x29e4('0x444','sD0z')]=''+Graphics['_input'][_0x29e4('0x445','ecJc')];continue;case'3':PlayOnline['Password']=_0x33d97b[_0x29e4('0x446','ulHM')]('',Graphics['_input2'][_0x29e4('0x1c4','nMLq')]);continue;case'4':this[_0x29e4('0x447','Y[z*')][_0x29e4('0x448',']n!8')]();continue;case'5':this['LoginReturn']();continue;case'6':Graphics[_0x29e4('0x449','w!#$')]();continue;case'7':if(window[_0x29e4('0x44a','w!#$')]&&window[_0x29e4('0x44b','BMXf')])window[_0x29e4('0x44c','S6JE')][_0x29e4('0x44d','kLv9')]();continue;case'8':this['_loginWindow'][_0x29e4('0x44e','hb[P')]();continue;}break;}};Scene_PlayOnline[_0x29e4('0x191',']hOe')]['GoHome']=function(){if(Utils[_0x29e4('0x34e','%4VO')]()){if(!PlayOnline['Param'][_0x29e4('0x44f','7A1d')])window[_0x29e4('0x450','T5&v')]('https://bbs.jhmeng.cn/');else Website2APK[_0x29e4('0x451','9mM)')](_0x29e4('0x452','Ck2P'));}else window['open']('https://bbs.jhmeng.cn/');};Scene_PlayOnline[_0x29e4('0x36c','iF[6')]['LoginCancel']=function(){window[_0x29e4('0x453','dgQb')]();};Scene_PlayOnline[_0x29e4('0x340','QTQ8')][_0x29e4('0x454',']n!8')]=function(){var _0x17fe96={'ZowCQ':'iFS','NhCAd':_0x29e4('0x455',']naz'),'mGozR':function _0x1c5b4c(_0x55a4c0,_0x4f7c52){return _0x55a4c0(_0x4f7c52);}};if(Utils[_0x29e4('0x456','jKz9')]()){if(_0x17fe96[_0x29e4('0x457','NWLF')]===_0x17fe96[_0x29e4('0x458','YS2X')]){var _0x51b3d2=_0x17fe96[_0x29e4('0x459','RmpA')](String,text)[_0x29e4('0x45a','Jp0A')](/\\c\[(\d+)\]/i);if(_0x51b3d2){this[_0x29e4('0x45b','QX0z')][_0x29e4('0x45c','nMLq')]=this[_0x29e4('0x45d','9mM)')](Number(_0x51b3d2[0x1]));text=text['replace'](/\\c\[(\d+)\]/gi,'');this[_0x29e4('0x45e','dgQb')][_0x29e4('0x45f','nMLq')](text,x,y,maxWidth,this[_0x29e4('0x460',']hOe')](),align);this[_0x29e4('0x461','9mM)')]();}else this[_0x29e4('0x462','FiO5')][_0x29e4('0x463','9[*1')](text,x,y,maxWidth,this['lineHeight'](),align);}else{if(!PlayOnline['Param'][_0x29e4('0x464',']fwy')])window['open'](PlayOnline[_0x29e4('0x4d',']fwy')][_0x29e4('0x465','FiO5')]);else Website2APK['openExternal'](PlayOnline[_0x29e4('0x60','YS2X')][_0x29e4('0x466','1^re')]);}}else window[_0x29e4('0x467','3nfd')](PlayOnline[_0x29e4('0xef','sD0z')][_0x29e4('0x468','SKjb')]);};Scene_PlayOnline['prototype']['NewGame']=function(){var _0x25d099={'pjsVW':_0x29e4('0x469','w!#$'),'sgxoo':function _0x1522cf(_0x46b8d6,_0x145cd0){return _0x46b8d6<_0x145cd0;},'mlpNV':function _0x57747f(_0x45d92a,_0x1c2e7c){return _0x45d92a>=_0x1c2e7c;},'KdkGo':function _0xf2c2ed(_0x31b06e,_0xc849c4,_0x4b6245){return _0x31b06e(_0xc849c4,_0x4b6245);}};SoundManager[_0x29e4('0x46a','F96I')]();if(PlayOnline['SaveInfo'][0x3]==0x0){if(_0x25d099['pjsVW']!==_0x29e4('0x46b','7A1d')){Game_Character[_0x29e4('0x235','eOur')][_0x29e4('0x265','3nfd')][_0x29e4('0x46c','eOur')](this);this['setMoveSpeed']($gamePlayer[_0x29e4('0x46d','GzNQ')]()/PlayOnline['Param']['MoveSpeed']);if(_0x25d099[_0x29e4('0x46e','iF[6')](this[_0x29e4('0x332','SKjb')],$gameParty[_0x29e4('0x46f','ToT1')]()))this['setOpacity']($gamePlayer[_0x29e4('0x470','T5&v')]());this[_0x29e4('0x471','aFEq')]($gamePlayer[_0x29e4('0x472','SKjb')]());this['setWalkAnime']($gamePlayer['hasWalkAnime']());this[_0x29e4('0x473','GzNQ')]($gamePlayer[_0x29e4('0x474','hb[P')]());this[_0x29e4('0x475','SKjb')]($gamePlayer[_0x29e4('0x476','GzNQ')]());this[_0x29e4('0x477',']fwy')]($gamePlayer['isTransparent']());if(PlayOnline[_0x29e4('0x478',']naz')][this['_index']]&&_0x25d099[_0x29e4('0x479','d80D')](this[_0x29e4('0x47a','NWLF')],$gameParty[_0x29e4('0x47b','7A1d')]())){this[_0x29e4('0x47c','S6JE')]();PlayOnline[_0x29e4('0x47d','w!#$')][this['_index']]=![];}}else{this[_0x29e4('0x47e','Jp0A')]();SceneManager['goto'](Scene_Map);if(PlayOnline[_0x29e4('0x47f','aFEq')][_0x29e4('0x480','i%o%')])$gameVariables[_0x29e4('0x481','NWLF')](PlayOnline[_0x29e4('0x2a1','GzNQ')]['OnlyValue'],PlayOnline['SaveInfo'][0x2]);_0x25d099[_0x29e4('0x482','Ck2P')](setInterval,function(){if('Fur'===_0x29e4('0x483','QX0z')){DataManager[_0x29e4('0x484','QX0z')]();}else{this['addCommand'](_0x29e4('0x485','j5q6'),'loadgame');this[_0x29e4('0x486','1^re')](_0x29e4('0x487','ToT1'),_0x29e4('0x488','BMXf'));}},0x3e8);}}else{this['_commandWindow'][_0x29e4('0x489','SKjb')]();this[_0x29e4('0x10c','7A1d')][_0x29e4('0x48a','eOur')]();this[_0x29e4('0x48b','w!#$')]();this['_confirmWindow'][_0x29e4('0x48c','sD0z')]();this[_0x29e4('0x48d','BMXf')][_0x29e4('0x48e','M@$&')](_0x29e4('0x48f','1^re'));this[_0x29e4('0x490','hb[P')][_0x29e4('0x491','iF[6')]();this['_confirmWindow'][_0x29e4('0x492','j5q6')](0x0);}};Scene_PlayOnline['prototype'][_0x29e4('0x493','GzNQ')]=function(){var _0x36bf99={'SPQKx':function _0x57914d(_0x1989d6,_0x3c5c82){return _0x1989d6(_0x3c5c82);},'pJjVT':function _0x1dafe1(_0x3a105f,_0x1bd526){return _0x3a105f!==_0x1bd526;},'URyCw':'vke','OHVCr':'YYN','UHKxh':_0x29e4('0x494','j5q6'),'MTjbL':'没有存档是否开启全新游戏'};SoundManager['playOk']();if(PlayOnline['SaveInfo'][0x3]!=0x0){var _0x39424e='3|5|1|6|2|4|0'[_0x29e4('0x495','%urC')]('|'),_0xc1abf4=0x0;while(!![]){switch(_0x39424e[_0xc1abf4++]){case'0':setInterval(function(){var _0x1db11f={'MYVOo':function _0x1ec9e8(_0x387e8d,_0x3cf885){return _0x387e8d!==_0x3cf885;},'uRkoA':'yjn'};if(_0x1db11f[_0x29e4('0x496','cXcC')](_0x1db11f['uRkoA'],'yjn')){for(var _0x3a7335=0x0;_0x3a7335<this[_0x29e4('0x31b','iF[6')][_0x29e4('0x497','3nfd')];++_0x3a7335){if(_0x3a7335<$gameParty[_0x29e4('0x498','9[*1')][_0x29e4('0x499','FiO5')]-0x1){var _0x4f3048=this[_0x29e4('0x49a','%4VO')][_0x3a7335];var _0x41e0e9=$gamePlayer['deltaXFrom'](_0x4f3048['x']);var _0x21e599=$gamePlayer[_0x29e4('0x49b','BMXf')](_0x4f3048['y']);_0x4f3048[_0x29e4('0x49c','eOur')](_0x41e0e9,_0x21e599);}}}else{DataManager[_0x29e4('0x49d','sD0z')]();}},0x3e8);continue;case'1':this[_0x29e4('0x49e','w!#$')]();continue;case'2':if(PlayOnline[_0x29e4('0x71','cXcC')][_0x29e4('0x49f',']naz')])$gameVariables[_0x29e4('0x4a0',']hOe')](PlayOnline[_0x29e4('0x287',']naz')]['OffLine'],_0x36bf99[_0x29e4('0x4a1','GzNQ')](parseInt,PlayOnline['SaveInfo'][0x0]));continue;case'3':DataManager[_0x29e4('0x4a2','S6JE')](PlayOnline['SaveInfo'][0x4]);continue;case'4':if(PlayOnline['Param'][_0x29e4('0x4a3','%4VO')])$gameVariables[_0x29e4('0x28a','jKz9')](PlayOnline['Param'][_0x29e4('0x4a4',']naz')],PlayOnline['SaveInfo'][0x2]);continue;case'5':this[_0x29e4('0x4a5','SKjb')]();continue;case'6':SceneManager[_0x29e4('0x4a6',']hOe')](Scene_Map);continue;}break;}}else{if(_0x36bf99[_0x29e4('0x4a7','3nfd')](_0x36bf99[_0x29e4('0x4a8','Tl7U')],_0x36bf99['OHVCr'])){var _0x453efd=_0x36bf99[_0x29e4('0x4a9',']naz')][_0x29e4('0x4aa','Tl7U')]('|'),_0x4310be=0x0;while(!![]){switch(_0x453efd[_0x4310be++]){case'0':this['_commandWindow']['deactivate']();continue;case'1':this['_confirmWindow'][_0x29e4('0x4ab','QcX%')]();continue;case'2':this[_0x29e4('0x4ac','iF[6')]();continue;case'3':this[_0x29e4('0x4ad',']n!8')][_0x29e4('0x4ae','QcX%')](_0x36bf99[_0x29e4('0x4af','QTQ8')]);continue;case'4':this[_0x29e4('0x4b0','Jp0A')][_0x29e4('0x4b1','rWBw')](0x0);continue;case'5':this['_commandWindow'][_0x29e4('0x489','SKjb')]();continue;case'6':this[_0x29e4('0x4b2','ulHM')]['open']();continue;}break;}}else{$['ajax']({'type':'post','url':PlayOnline[_0x29e4('0x4b3','Jp0A')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x29e4('0x49','7A1d')]['ActionName'],'cmd':'time'},'success':function(_0x2eae67){var KPmnQS={'Izflt':function _0x7278b9(_0x4d77ff,_0x156e6b){return _0x4d77ff*_0x156e6b;},'IQqtw':function _0x310fb4(_0x3124ef,_0x45d4e5){return _0x3124ef+_0x45d4e5;}};var _0x101ddb=new Date(KPmnQS[_0x29e4('0x4b4','F96I')](_0x2eae67,0x3e8));if(args[0x0])$gameVariables[_0x29e4('0x4b5',']naz')](args[0x0],_0x101ddb[_0x29e4('0x4b6','YS2X')]());if(args[0x1])$gameVariables[_0x29e4('0x392','cXcC')](args[0x1],KPmnQS[_0x29e4('0x4b7','Y[z*')](_0x101ddb[_0x29e4('0x1d4','w!#$')](),0x1));if(args[0x2])$gameVariables[_0x29e4('0x4b8','sD0z')](args[0x2],_0x101ddb[_0x29e4('0x4b9','dgQb')]());if(args[0x3])$gameVariables['setValue'](args[0x3],_0x101ddb[_0x29e4('0x4ba','%4VO')]());if(args[0x4])$gameVariables[_0x29e4('0x4a0',']hOe')](args[0x4],_0x101ddb[_0x29e4('0x4bb','F96I')]());if(args[0x5])$gameVariables[_0x29e4('0x4bc','kLv9')](args[0x5],_0x101ddb['getSeconds']());},'error':function(_0x7c1007){alert(_0x29e4('0x4bd','0%rU'));PlayOnline[_0x29e4('0x2ab',']hOe')]=null;},'async':![]});}}};Scene_PlayOnline['prototype'][_0x29e4('0x404','9[*1')]=function(){var _0x21090a={'YvHhw':'2|3|5|0|4|1'};var _0x39e025=_0x21090a[_0x29e4('0x4be','SKjb')]['split']('|'),_0x2a9cbe=0x0;while(!![]){switch(_0x39e025[_0x2a9cbe++]){case'0':SceneManager['goto'](Scene_Map);continue;case'1':setInterval(function(){DataManager[_0x29e4('0x4bf','T5&v')]();},0x3e8);continue;case'2':this[_0x29e4('0x38a','eOur')][_0x29e4('0x4c0','7A1d')]();continue;case'3':this[_0x29e4('0x4c1',']fwy')][_0x29e4('0x4c2','GzNQ')]();continue;case'4':if(PlayOnline[_0x29e4('0x15d','9mM)')]['OnlyValue'])$gameVariables[_0x29e4('0x4c3','BMXf')](PlayOnline['Param'][_0x29e4('0x4c4','jKz9')],PlayOnline['SaveInfo'][0x2]);continue;case'5':this[_0x29e4('0x4c5','0%rU')](this['_cbgdSprite']);continue;}break;}};Scene_PlayOnline[_0x29e4('0x4c6','M@$&')]['onConfirmCancel']=function(){var _0x56bf2a={'DWeFj':_0x29e4('0x4c7','ecJc')};var _0x49120d=_0x56bf2a['DWeFj'][_0x29e4('0x495','%urC')]('|'),_0x52856e=0x0;while(!![]){switch(_0x49120d[_0x52856e++]){case'0':this['_commandWindow'][_0x29e4('0x4c8','dgQb')]();continue;case'1':this['removeChild'](this[_0x29e4('0x4c9','7A1d')]);continue;case'2':this[_0x29e4('0x4ca','NWLF')][_0x29e4('0x4cb','QcX%')]();continue;case'3':this[_0x29e4('0x4cc','GzNQ')][_0x29e4('0x436','Ck2P')]();continue;case'4':this['_confirmWindow'][_0x29e4('0x4cd','ulHM')]();continue;}break;}};Scene_PlayOnline[_0x29e4('0x264','sD0z')][_0x29e4('0x4ce','Tl7U')]=function(){var _0x4ff56c={'fWCxd':function _0x4d69d8(_0x2ce374,_0x4ee4d6){return _0x2ce374!==_0x4ee4d6;},'UBrTz':'2|1|5|4|7|6|0|3','sNCUs':_0x29e4('0x4cf','9[*1'),'TFvHD':function _0x1b993b(_0x25a759,_0x2d80b2){return _0x25a759-_0x2d80b2;}};if(_0x4ff56c[_0x29e4('0x4d0','QX0z')]($gameSystem[_0x29e4('0x4d1','Ck2P')](),$dataSystem[_0x29e4('0x4d2','M@$&')])){if('zBo'!==_0x29e4('0x4d3','hb[P')){$gamePlayer[_0x29e4('0x4d4','3nfd')]($gameMap[_0x29e4('0x4d5','QX0z')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x29e4('0x4d6','SKjb')]();}else{var _0x340d19=_0x4ff56c[_0x29e4('0x4d7','S6JE')]['split']('|'),_0x23bc3e=0x0;while(!![]){switch(_0x340d19[_0x23bc3e++]){case'0':this[_0x29e4('0x4d8','0%rU')][_0x29e4('0x4d9','i%o%')](_0x4ff56c['sNCUs'],this['onConfirmCancel'][_0x29e4('0x4da','3nfd')](this));continue;case'1':var _0x5bf39d=_0x4ff56c['TFvHD'](Graphics[_0x29e4('0x4db','j5q6')],_0x54ea33)/0x2;continue;case'2':var _0x54ea33=0x190;continue;case'3':this[_0x29e4('0x4dc','rWBw')](this[_0x29e4('0x4dd','nMLq')]);continue;case'4':this[_0x29e4('0x4de','T5&v')]=new Window_Action(_0x5bf39d,_0x4a1453,_0x54ea33);continue;case'5':var _0x4a1453=PlayOnline[_0x29e4('0x4f','%4VO')][_0x29e4('0x4df','hb[P')];continue;case'6':this[_0x29e4('0x4e0','dQvw')][_0x29e4('0x4e1','iF[6')]('confirm',this[_0x29e4('0x4e2','d80D')]['bind'](this));continue;case'7':this[_0x29e4('0x4e3','Y[z*')]['opacity']=PlayOnline[_0x29e4('0x4e4','nMLq')]['WindowOpacity'];continue;}break;}}}};Scene_PlayOnline[_0x29e4('0x4e5','nMLq')][_0x29e4('0x4e6','Y[z*')]=function(){var _0x5619ac={'uSsIB':function _0x328a9e(_0x58f9f8,_0x2fb341){return _0x58f9f8(_0x2fb341);},'bACgw':_0x29e4('0x4e7','Tl7U'),'xOTzs':function _0x565558(_0x2f9571,_0x6a0769){return _0x2f9571(_0x6a0769);},'TsDEU':_0x29e4('0x4e8','3nfd'),'GsbsK':function _0x2235df(_0xdf65c1,_0x15ca62){return _0xdf65c1(_0x15ca62);},'SpqkF':function _0x545c87(_0x2ccfba,_0x559131){return _0x2ccfba==_0x559131;},'ghiLG':_0x29e4('0x4e9','d80D'),'psmxN':_0x29e4('0x4ea','7A1d'),'uuveY':'密码错误三次剩两次机会','LJNfL':'Lmv','DIdDM':function _0x34ff4a(_0x57e3f8,_0x486e96){return _0x57e3f8(_0x486e96);},'DKxiq':'3|5|2|6|0|10|8|9|4|7|1','FpLcH':_0x29e4('0x4eb','%4VO'),'nkJXp':'common','FyhsJ':_0x29e4('0x163','BMXf'),'cMYwk':_0x29e4('0x4ec','rWBw'),'kHuWK':function _0x1588e3(_0x551d15,_0x1553ab){return _0x551d15+_0x1553ab;},'KAiLv':function _0x40062a(_0x3021cc,_0x2702e9){return _0x3021cc===_0x2702e9;},'qCTqv':function _0x1c5934(_0x191da7,_0x438ff4){return _0x191da7<=_0x438ff4;},'IFvVX':'text','UEdrm':function _0x439278(_0x3068bd,_0x59e445){return _0x3068bd-_0x59e445;},'ChRvp':function _0x586902(_0x22abee,_0x1abb54){return _0x22abee/_0x1abb54;}};var _0x50f0df=_0x29e4('0x4ed','iF[6')[_0x29e4('0x4ee','RmpA')]('|'),_0x317016=0x0;while(!![]){switch(_0x50f0df[_0x317016++]){case'0':var _0x44ae2a={'qSUNe':function _0x22583a(_0x361321,_0x31e079){return _0x5619ac[_0x29e4('0x4ef','QcX%')](_0x361321,_0x31e079);},'XtAOC':_0x5619ac[_0x29e4('0x4f0',']fwy')],'Vcocl':function _0xf40118(_0x3335b0,_0x26c696){return _0x5619ac[_0x29e4('0x4f1','Jp0A')](_0x3335b0,_0x26c696);},'BgKnW':function _0x33ee6b(_0x42e438,_0x12050b){return _0x42e438!==_0x12050b;},'OAtSf':_0x5619ac[_0x29e4('0x4f2',']naz')],'mBjaA':_0x29e4('0x4f3','GzNQ'),'MEAdU':function _0x3f01ee(_0x57cf88,_0x34f4df){return _0x5619ac[_0x29e4('0x4f4','QX0z')](_0x57cf88,_0x34f4df);},'CtaYu':function _0x24719c(_0x41f3f7,_0x32cbaf){return _0x41f3f7>_0x32cbaf;},'DsXaR':function _0x437b6c(_0x365c11,_0x388414){return _0x5619ac[_0x29e4('0x4f5','nMLq')](_0x365c11,_0x388414);},'XlfyT':function _0x392f42(_0x250c09,_0x18a406){return _0x250c09(_0x18a406);},'BHWaH':_0x29e4('0x4f6',']fwy'),'CAzNJ':_0x5619ac[_0x29e4('0x4f7','jKz9')],'qxXGo':_0x5619ac[_0x29e4('0x4f8','NWLF')],'eXitt':function _0x570725(_0xfa5f0c,_0x140914){return _0xfa5f0c-_0x140914;},'IWOpz':function _0x2e10cb(_0x4d5ae1,_0x3dc102){return _0x4d5ae1<_0x3dc102;},'zHDvd':function _0x680338(_0x1dec91,_0x45c80a){return _0x1dec91+_0x45c80a;},'nSNcd':_0x5619ac['uuveY'],'ZQuMg':_0x5619ac[_0x29e4('0x4f9','d80D')],'IBQFZ':function _0x5ef679(_0x129bdc,_0x177f70){return _0x5619ac[_0x29e4('0x4fa','ulHM')](_0x129bdc,_0x177f70);},'VIxdj':_0x5619ac[_0x29e4('0x4fb','3nfd')],'DZGEQ':_0x29e4('0x4fc',']naz'),'DusWx':function _0x1134e0(_0x4a358c,_0x4d877e){return _0x4a358c/_0x4d877e;},'XvTIG':_0x29e4('0x4fd','jKz9'),'FNLHF':'密码错误五次冻结15分钟','Wckkf':_0x29e4('0x4fe','Tl7U'),'TMkbj':_0x29e4('0x4ff',']n!8'),'fkCqW':_0x5619ac[_0x29e4('0x500',']naz')],'kXjUc':_0x5619ac['nkJXp'],'iNNnm':_0x29e4('0x501','1^re')};continue;case'1':this['createLoginWindow']();continue;case'2':$[_0x29e4('0x502',']naz')]({'type':_0x5619ac['FyhsJ'],'url':PlayOnline[_0x29e4('0x503','F96I')],'data':{'user':PlayOnline[_0x29e4('0x504','FiO5')],'word':PlayOnline['Password'],'game':PlayOnline[_0x29e4('0x8d',']n!8')][_0x29e4('0x505','ecJc')],'cmd':'login'},'success':function(_0x3d582a){if(!PlayOnline['UserName']){_0x44ae2a[_0x29e4('0x506','SKjb')](alert,_0x44ae2a['XtAOC']);}else if(!PlayOnline[_0x29e4('0xf1','3nfd')]){_0x44ae2a[_0x29e4('0x507','iF[6')](alert,'密码输入为空，请重新输入！');PlayOnline['UserName']=null;}else if(_0x3d582a<=-0x5){if(_0x44ae2a[_0x29e4('0x508','rWBw')](_0x44ae2a[_0x29e4('0x509','jKz9')],_0x44ae2a[_0x29e4('0x50a','ulHM')])){_0x44ae2a['MEAdU'](alert,_0x29e4('0x50b',']fwy'));PlayOnline[_0x29e4('0x50c','nMLq')]=null;}else{this[_0x29e4('0x50d','ulHM')](_0x44ae2a['CtaYu'](sy,0x0)?0x8:0x2);}}else if(_0x44ae2a[_0x29e4('0x50e','sD0z')](_0x3d582a,-0x1)){_0x44ae2a['XlfyT'](alert,_0x44ae2a['BHWaH']);PlayOnline[_0x29e4('0x323','dgQb')]=null;}else if(_0x44ae2a[_0x29e4('0x50f','ecJc')](_0x3d582a,-0x2)){if(_0x44ae2a[_0x29e4('0x510','jKz9')]===_0x44ae2a['CAzNJ']){_0x44ae2a[_0x29e4('0x511','GzNQ')](alert,_0x44ae2a['qxXGo']);PlayOnline['UserName']=null;}else{var _0x45788e=TouchInput['x'];var _0x1a661b=TouchInput['y'];if(_0x45788e>=wx&&_0x1a661b>=_0x44ae2a[_0x29e4('0x512','dgQb')](wy,wh)&&_0x44ae2a[_0x29e4('0x513','QcX%')](_0x45788e,_0x44ae2a['zHDvd'](wx,ww))&&_0x1a661b<wy){this['_loginWindow'][_0x29e4('0x514','3nfd')]();this[_0x29e4('0x515','aFEq')]['deactivate']();Graphics[_0x29e4('0x516','hb[P')][_0x29e4('0x517','NWLF')]();}else if(_0x45788e>=wx&&_0x1a661b>=wy&&_0x45788e<_0x44ae2a['zHDvd'](wx,ww)&&_0x44ae2a[_0x29e4('0x518','rWBw')](_0x1a661b,_0x44ae2a[_0x29e4('0x519',']fwy')](wy,wh))){this['_loginWindow']['activate']();this[_0x29e4('0x2d7','j5q6')]['deactivate']();Graphics[_0x29e4('0x51a',']naz')]['focus']();}else{this[_0x29e4('0x51b','eOur')]['deactivate']();this[_0x29e4('0x51c',']n!8')][_0x29e4('0x51d','T5&v')]();}}}else if(_0x3d582a==-0x3){_0x44ae2a[_0x29e4('0x51e','ToT1')](alert,_0x44ae2a['nSNcd']);PlayOnline[_0x29e4('0x51f','kLv9')]=null;}else if(_0x3d582a==-0x4){if(_0x44ae2a[_0x29e4('0x520','9[*1')]!==_0x29e4('0x521',']n!8')){_0x44ae2a[_0x29e4('0x522','i%o%')](alert,_0x29e4('0x523','SKjb'));PlayOnline['UserName']=null;}else{var _0x45c8cc=_0x44ae2a['VIxdj'][_0x29e4('0x524','sD0z')]('|'),_0x2dfb1b=0x0;while(!![]){switch(_0x45c8cc[_0x2dfb1b++]){case'0':Graphics['_addInput'](_0x44ae2a[_0x29e4('0x525','SKjb')],_0x44ae2a[_0x29e4('0x526','BMXf')](Graphics[_0x29e4('0x527','BMXf')],0x2)-0x14,_0x44ae2a['zHDvd'](PlayOnline['Param']['WindowY'],0x50),0xc8,0x19,0x18);continue;case'1':if(Utils['isMobileDevice']())this['createUserInfo']();continue;case'2':this['createLoginWindow']();continue;case'3':Scene_Base[_0x29e4('0x212','[L3c')][_0x29e4('0x528','QTQ8')][_0x29e4('0x529','aFEq')](this);continue;case'4':Graphics['_input2']['maxLength']=0xf;continue;case'5':this['createWindowLayer']();continue;case'6':this[_0x29e4('0x52a','ulHM')]();continue;case'7':Graphics['_input2']['value']='';continue;case'8':Graphics[_0x29e4('0x41e','kLv9')][_0x29e4('0x52b','QTQ8')]='';continue;case'9':Graphics[_0x29e4('0x52c','nMLq')](_0x44ae2a[_0x29e4('0x52d','nMLq')],Graphics[_0x29e4('0x52e','Y[z*')]/0x2-0x14,PlayOnline['Param'][_0x29e4('0x52f','GzNQ')]+0x82,0xc8,0x19,0x18);continue;case'10':Graphics['_input'][_0x29e4('0x530','1^re')]=0xf;continue;}break;}}}else if(_0x3d582a==-0x5){alert(_0x44ae2a[_0x29e4('0x531','Y[z*')]);PlayOnline['UserName']=null;}else{if(_0x44ae2a[_0x29e4('0x532','Y[z*')](_0x44ae2a[_0x29e4('0x533','M@$&')],_0x29e4('0x534','i%o%'))){PlayOnline[_0x29e4('0x535','aFEq')]=_0x3d582a['split'](',');if(parseInt(PlayOnline[_0x29e4('0x3d3',']n!8')][0x0])>0x5)DataManager[_0x29e4('0x536','7A1d')]();if(Utils[_0x29e4('0x537','aFEq')]()){if(_0x29e4('0x538','T5&v')===_0x29e4('0x539','nMLq')){this[_0x29e4('0x53a','%4VO')][_0x29e4('0x53b','QX0z')](_0x44ae2a[_0x29e4('0x53c','%4VO')],this['LoginOk'][_0x29e4('0x53d','9mM)')](this));this[_0x29e4('0x3f2','NWLF')][_0x29e4('0x3ce','YS2X')](_0x44ae2a[_0x29e4('0x53e','SKjb')],this[_0x29e4('0x53f','FiO5')][_0x29e4('0x540','Tl7U')](this));this[_0x29e4('0x541','FiO5')][_0x29e4('0x542','%urC')](_0x29e4('0x543','jKz9'),this[_0x29e4('0x544','Jp0A')][_0x29e4('0x16f','Jp0A')](this));}else{var _0xd4e129={'user':PlayOnline[_0x29e4('0x545','1^re')],'pass':PlayOnline[_0x29e4('0xd8','i%o%')]};StorageManager['saveWebFile'](_0xd4e129,_0x44ae2a[_0x29e4('0x546','iF[6')]);}}}else{PlayOnline[_0x29e4('0x547','sD0z')]=null;PlayOnline[_0x29e4('0x548','3nfd')]=null;SceneManager[_0x29e4('0x549','aFEq')](Scene_PlayOnline);}}},'error':function(_0x47a294){PlayOnline[_0x29e4('0x504','FiO5')]=null;_0x44ae2a['IBQFZ'](alert,_0x44ae2a[_0x29e4('0x54a','%urC')]);},'async':![]});continue;case'3':this[_0x29e4('0x54b','GzNQ')]();continue;case'4':this[_0x29e4('0x54c',']naz')][_0x29e4('0x54d','RmpA')]();continue;case'5':if(!PlayOnline[_0x29e4('0x54e','dgQb')]){var _0x289901=_0x29e4('0x54f','nMLq')['split']('|'),_0xf95405=0x0;while(!![]){switch(_0x289901[_0xf95405++]){case'0':Graphics[_0x29e4('0x550','9[*1')]['maxLength']=0xf;continue;case'1':Graphics[_0x29e4('0x551',']n!8')](_0x29e4('0x552','eOur'),Graphics['width']/0x2-0x14,PlayOnline['Param'][_0x29e4('0x553','dQvw')]+0x50,0x96,0x19,0x18);continue;case'2':Graphics[_0x29e4('0x554','dQvw')]['value']='';continue;case'3':PlayOnline['Password']=null;continue;case'4':Graphics['_input'][_0x29e4('0x555','dQvw')]='';continue;case'5':PlayOnline[_0x29e4('0xe8','QcX%')]=null;continue;case'6':Graphics[_0x29e4('0x556','F96I')](_0x5619ac[_0x29e4('0x557','dQvw')],Graphics[_0x29e4('0x558','ecJc')]/0x2-0x14,_0x5619ac[_0x29e4('0x559',']naz')](PlayOnline['Param'][_0x29e4('0x55a','M@$&')],0x82),0x96,0x19,0x18);continue;case'7':Graphics[_0x29e4('0x55b','SKjb')][_0x29e4('0x55c','M@$&')]=0xf;continue;}break;}}else{if(_0x5619ac[_0x29e4('0x55d','i%o%')]('STZ','STZ')){if(_0x5619ac['qCTqv'](parseInt(PlayOnline['SaveInfo'][0x0]),0x5)){PlayOnline['UserName']=null;PlayOnline[_0x29e4('0x55e','sD0z')]=null;Graphics[_0x29e4('0x55f',']naz')](_0x5619ac[_0x29e4('0x560','0%rU')],_0x5619ac[_0x29e4('0x561','1^re')](_0x5619ac[_0x29e4('0x562','i%o%')](Graphics[_0x29e4('0x563',']hOe')],0x2),0x14),PlayOnline[_0x29e4('0x270','QcX%')][_0x29e4('0x564','F96I')]+0x50,0x96,0x19,0x18);Graphics[_0x29e4('0x565','RmpA')]['maxLength']=0xf;Graphics[_0x29e4('0x1c3','BMXf')]['value']='';Graphics[_0x29e4('0x566','S6JE')](_0x5619ac['cMYwk'],_0x5619ac[_0x29e4('0x567','3nfd')](Graphics[_0x29e4('0x568','Ck2P')]/0x2,0x14),_0x5619ac[_0x29e4('0x569',']n!8')](PlayOnline[_0x29e4('0x58','QTQ8')][_0x29e4('0x56a','9[*1')],0x82),0x96,0x19,0x18);Graphics[_0x29e4('0x56b','w!#$')][_0x29e4('0x56c','sD0z')]=0xf;Graphics[_0x29e4('0x56d','9mM)')][_0x29e4('0x56e',']naz')]='';}else{this['createConfirmWindow']();}}else{var _0x11765c=firstCall?function(){if(fn){var _0x3f0d45=fn['apply'](context,arguments);fn=null;return _0x3f0d45;}}:function(){};firstCall=![];return _0x11765c;}}continue;case'6':this['_commandWindow'][_0x29e4('0x56f','w!#$')]();continue;case'7':this[_0x29e4('0x570','[L3c')][_0x29e4('0x4c0','7A1d')]();continue;case'8':this['_commandWindow'][_0x29e4('0x571','S6JE')]();continue;}break;}};Scene_Ranking['prototype']=Object[_0x29e4('0x572','dQvw')](Scene_MenuBase[_0x29e4('0x20c','rWBw')]);Scene_Ranking['prototype'][_0x29e4('0x573','%4VO')]=Scene_Ranking;Scene_Ranking['prototype'][_0x29e4('0x574','9mM)')]=function(){var _0x7a3020={'EPdMC':function _0x232c95(_0x5c0130,_0x43523c){return _0x5c0130/_0x43523c;},'wDsDE':function _0x44b38a(_0x3f628c,_0x5f26c4){return _0x3f628c-_0x5f26c4;}};Scene_MenuBase[_0x29e4('0x4c6','M@$&')][_0x29e4('0x575','9[*1')][_0x29e4('0x576','Y[z*')](this);this['createBackground']();this[_0x29e4('0x577','j5q6')]=Math[_0x29e4('0x578','dgQb')](_0x7a3020[_0x29e4('0x579','[L3c')](_0x7a3020[_0x29e4('0x57a','j5q6')](Graphics[_0x29e4('0x57b','ecJc')],0xa0),0x23));};Scene_Ranking[_0x29e4('0x43f','kLv9')][_0x29e4('0x57c','%4VO')]=function(){Scene_MenuBase['prototype']['create'][_0x29e4('0x57d',']n!8')](this);this[_0x29e4('0x57e',']naz')]();this[_0x29e4('0x57f','9[*1')]();this[_0x29e4('0x580','0%rU')]['y']=Graphics[_0x29e4('0x581','aFEq')]-this[_0x29e4('0x582','d80D')]['height'];this[_0x29e4('0x583','T5&v')][_0x29e4('0x584','[L3c')]=Graphics['boxHeight']-this[_0x29e4('0x585','jKz9')][_0x29e4('0x3c9','jKz9')];};Scene_Ranking[_0x29e4('0x33f','9[*1')][_0x29e4('0x586','3nfd')]=function(){if(PlayOnline['Param']['RankBgd']){this['_backSprite']=new Sprite(ImageManager[_0x29e4('0x587','QTQ8')](PlayOnline[_0x29e4('0x390','Ck2P')]['RankBgd']));this[_0x29e4('0x588','%4VO')](this[_0x29e4('0x589',']hOe')]);}};Scene_Ranking[_0x29e4('0x58a','NWLF')]['createInfoWindow']=function(){var _0x48007e=_0x29e4('0x58b','w!#$')[_0x29e4('0x58c','w!#$')]('|'),_0x14ae38=0x0;while(!![]){switch(_0x48007e[_0x14ae38++]){case'0':this[_0x29e4('0x58d','j5q6')](this[_0x29e4('0x58e','3nfd')]);continue;case'1':var _0x3e5445=0x258;continue;case'2':this[_0x29e4('0x583','T5&v')][_0x29e4('0x58f','FiO5')]=PlayOnline['Param'][_0x29e4('0x590','BMXf')];continue;case'3':var _0x173f01=0x280;continue;case'4':this[_0x29e4('0x591','M@$&')]=new Window_Ranking(_0x33d70e,0x0,_0x3e5445,_0x173f01);continue;case'5':var _0x33d70e=(Graphics['boxWidth']-_0x3e5445)/0x2;continue;}break;}};Scene_Ranking[_0x29e4('0x20c','rWBw')][_0x29e4('0x592',']naz')]=function(){var _0x389fe5={'VfSyh':_0x29e4('0x593','nMLq'),'CbKUT':'pageup','TQxuj':'pagedown','AftZe':_0x29e4('0x594','%4VO')};var _0x2cb46c=Graphics[_0x29e4('0x595','iF[6')]/0x2-0x12c;var _0x5523d2=Graphics[_0x29e4('0x596','S6JE')]-0x64;this['_commandWindow']=new Window_RankCmd(_0x2cb46c,_0x5523d2);this[_0x29e4('0x597','1^re')][_0x29e4('0x598','i%o%')]=PlayOnline[_0x29e4('0x599','i%o%')][_0x29e4('0x59a','F96I')];this[_0x29e4('0x59b','F96I')][_0x29e4('0x59c','dgQb')](_0x389fe5[_0x29e4('0x59d','NWLF')],this[_0x29e4('0x59e','dQvw')][_0x29e4('0x59f','RmpA')](this));this[_0x29e4('0x5a0','T5&v')][_0x29e4('0x1f5','0%rU')]('mypage',this[_0x29e4('0x1eb','Tl7U')]['bind'](this));this[_0x29e4('0x5a1','%urC')][_0x29e4('0x5a2','dQvw')](_0x389fe5[_0x29e4('0x5a3','1^re')],this[_0x29e4('0x5a4','d80D')]['bind'](this));this[_0x29e4('0x5a5','RmpA')][_0x29e4('0x1ee','%4VO')](_0x389fe5[_0x29e4('0x5a6','i%o%')],this['PageDown'][_0x29e4('0x3d1','M@$&')](this));this[_0x29e4('0x10c','7A1d')][_0x29e4('0x5a7','S6JE')](_0x389fe5[_0x29e4('0x5a8','1^re')],this[_0x29e4('0x5a9','dgQb')][_0x29e4('0x5aa','SKjb')](this));this[_0x29e4('0x4dc','rWBw')](this[_0x29e4('0x3c6','dgQb')]);this[_0x29e4('0x447','Y[z*')][_0x29e4('0x5ab',']hOe')](0x1);};Scene_Ranking[_0x29e4('0x33f','9[*1')][_0x29e4('0x5ac','ToT1')]=function(){PlayOnline[_0x29e4('0x5ad','Jp0A')]=0x1;this[_0x29e4('0x5ae','3nfd')][_0x29e4('0x5af','YS2X')]();};Scene_Ranking[_0x29e4('0x5b0','F96I')][_0x29e4('0x5b1','F96I')]=function(){var _0x41a632={'vHXSf':function _0x5f15c2(_0x3b1574,_0x126548){return _0x3b1574<_0x126548;},'HtwZB':function _0x8fd8cd(_0x5f3b5,_0xf57df5){return _0x5f3b5/_0xf57df5;},'MNhrB':function _0x249395(_0x2ef6c1,_0x1e8b96){return _0x2ef6c1+_0x1e8b96;}};for(var _0x26bbcd=0x0;_0x41a632[_0x29e4('0x5b2','iF[6')](_0x26bbcd,_0x41a632['HtwZB'](PlayOnline[_0x29e4('0x5b3','[L3c')][_0x29e4('0x5b4','%urC')],0x2));++_0x26bbcd){if(PlayOnline['Ranking'][_0x26bbcd*0x2]==PlayOnline[_0x29e4('0x91','ToT1')]){PlayOnline[_0x29e4('0x5b5','%4VO')]=Math[_0x29e4('0x5b6','iF[6')](_0x41a632[_0x29e4('0x5b7','[L3c')](_0x26bbcd,0x1)/this[_0x29e4('0x5b8','QcX%')]);break;}}this[_0x29e4('0x5b9','SKjb')][_0x29e4('0x5ba','1^re')]();};Scene_Ranking['prototype'][_0x29e4('0x5bb','ecJc')]=function(){var _0x4f759d={'gBJgN':function _0x9abf0b(_0x58831f,_0x50a138){return _0x58831f>_0x50a138;}};if(_0x4f759d[_0x29e4('0x5bc','Tl7U')](PlayOnline['Page'],0x1))--PlayOnline[_0x29e4('0x5bd','FiO5')];this[_0x29e4('0x597','1^re')][_0x29e4('0x5be','BMXf')]();};Scene_Ranking[_0x29e4('0x202','GzNQ')][_0x29e4('0x5bf','7A1d')]=function(){var _0x2b0e0f={'HueCa':function _0x5c3554(_0x5d70bf,_0x3f4977){return _0x5d70bf<_0x3f4977;}};if(_0x2b0e0f[_0x29e4('0x5c0','%4VO')](PlayOnline[_0x29e4('0x5bd','FiO5')],Math[_0x29e4('0x5c1','9mM)')]((PlayOnline[_0x29e4('0x5c2','cXcC')]['length']-0x1)/(this[_0x29e4('0x5c3','YS2X')]*0x2))))++PlayOnline['Page'];this[_0x29e4('0x53a','%4VO')]['activate']();};Scene_GameEnd[_0x29e4('0x398','ulHM')][_0x29e4('0x5c4','ecJc')]=function(){this[_0x29e4('0x5c5','BMXf')]();DataManager[_0x29e4('0x5c6','Ck2P')]();PlayOnline['SaveInfo']=null;SceneManager['push'](Scene_PlayOnline);};Scene_Gameover[_0x29e4('0x5c7','dgQb')][_0x29e4('0x5c8','d80D')]=function(){DataManager[_0x29e4('0x5c9','ulHM')]();PlayOnline[_0x29e4('0x394',']naz')]=null;SceneManager[_0x29e4('0x5ca','3nfd')](Scene_PlayOnline);};if(PlayOnline['Param'][_0x29e4('0x5cb','RmpA')]){var _0x2bfb86=_0x29e4('0x5cc','0%rU')[_0x29e4('0x4aa','Tl7U')]('|'),_0x187f7e=0x0;while(!![]){switch(_0x2bfb86[_0x187f7e++]){case'0':Scene_Menu['prototype'][_0x29e4('0x5cd','w!#$')]=function(){var _0x59cc2a={'cBqXg':_0x29e4('0x5ce','ToT1')};PlayOnline[_0x29e4('0x5cf','QcX%')]['call'](this);this['_commandWindow'][_0x29e4('0x5d0','Ck2P')](_0x59cc2a[_0x29e4('0x5d1','dgQb')],this[_0x29e4('0x5d2','S6JE')]['bind'](this));};continue;case'1':PlayOnline[_0x29e4('0x5d3','nMLq')]=Window_MenuCommand[_0x29e4('0x210','ToT1')][_0x29e4('0x5d4',']naz')];continue;case'2':Window_MenuCommand[_0x29e4('0x39a','SKjb')][_0x29e4('0x5d5','FiO5')]=function(){var _0x4e53f9=!![];this['addCommand'](PlayOnline[_0x29e4('0x49','7A1d')]['MenuRank'],_0x29e4('0x5d6','9mM)'),_0x4e53f9);};continue;case'3':Window_MenuCommand['prototype'][_0x29e4('0x5d7','ecJc')]=function(){PlayOnline[_0x29e4('0x5d8','hb[P')]['call'](this);this['addRankCommand']();};continue;case'4':Scene_Menu[_0x29e4('0x31e','Ck2P')][_0x29e4('0x5d9','sD0z')]=function(){SceneManager['push'](Scene_Ranking);};continue;case'5':PlayOnline[_0x29e4('0x5da','7A1d')]=Scene_Menu[_0x29e4('0x36c','iF[6')][_0x29e4('0x5db','j5q6')];continue;}break;}};Window_Base['prototype'][_0x29e4('0x5dc','aFEq')]=function(_0x3edf30,_0x9a380d,_0x24ecd9,_0x9f3325,_0x3e883c){var _0x1d77e6={'SjRYJ':function _0xe0148(_0x3201fb,_0x4188a4){return _0x3201fb(_0x4188a4);}};var _0x9667b=String(_0x3edf30)[_0x29e4('0x5dd','Y[z*')](/\\c\[(\d+)\]/i);if(_0x9667b){this['contents'][_0x29e4('0x5de','Jp0A')]=this[_0x29e4('0x5df','FiO5')](_0x1d77e6['SjRYJ'](Number,_0x9667b[0x1]));_0x3edf30=_0x3edf30[_0x29e4('0x5e0','%urC')](/\\c\[(\d+)\]/gi,'');this['contents'][_0x29e4('0x5e1','QcX%')](_0x3edf30,_0x9a380d,_0x24ecd9,_0x9f3325,this[_0x29e4('0x5e2','d80D')](),_0x3e883c);this['resetTextColor']();}else this[_0x29e4('0x5e3','d80D')][_0x29e4('0x5e4','rWBw')](_0x3edf30,_0x9a380d,_0x24ecd9,_0x9f3325,this[_0x29e4('0x5e5','SKjb')](),_0x3e883c);};Window_LoginCommand['prototype']=Object[_0x29e4('0x5e6','Y[z*')](Window_Command[_0x29e4('0x397','3nfd')]);Window_LoginCommand[_0x29e4('0x2bb','0%rU')][_0x29e4('0x5e7','dQvw')]=Window_LoginCommand;Window_LoginCommand[_0x29e4('0x207',']fwy')][_0x29e4('0x5e8','S6JE')]=function(_0x46994d){var _0x268516=_0x29e4('0x5e9','F96I')['split']('|'),_0x4bd46e=0x0;while(!![]){switch(_0x268516[_0x4bd46e++]){case'0':var _0x5e2bd3=0x258;continue;case'1':Window_Command[_0x29e4('0x5ea','1^re')][_0x29e4('0x5eb','dQvw')][_0x29e4('0x5ec','ulHM')](this,0x0,0x0,_0x5e2bd3,_0x1d4624);continue;case'2':this[_0x29e4('0x5ed','NWLF')]();continue;case'3':this[_0x29e4('0x5ee','i%o%')]();continue;case'4':var _0x1d4624=this[_0x29e4('0x5ef',']hOe')](0x1);continue;case'5':this[_0x29e4('0x5f0','nMLq')]=_0x46994d;continue;case'6':this['select'](0x0);continue;case'7':this[_0x29e4('0x5f1','cXcC')]();continue;case'8':this[_0x29e4('0x5f2','cXcC')]();continue;case'9':this[_0x29e4('0x5f3','QcX%')]();continue;}break;}};Window_LoginCommand[_0x29e4('0x398','ulHM')][_0x29e4('0x5f4','%urC')]=function(){if(!PlayOnline[_0x29e4('0x5f5','Y[z*')])return 0x3;else{if(parseInt(PlayOnline[_0x29e4('0x370','FiO5')][0x0])<=0x5)return 0x3;else return 0x2;}};Window_LoginCommand[_0x29e4('0x410','S6JE')]['itemTextAlign']=function(){var _0x482333={'viHPy':_0x29e4('0x5f6',']n!8')};return _0x482333[_0x29e4('0x5f7','Y[z*')];};Window_LoginCommand[_0x29e4('0x207',']fwy')][_0x29e4('0x5f8','QX0z')]=function(){var _0x10e018={'RYKHU':function _0x485d2c(_0x1c9261,_0x11ad5b){return _0x1c9261+_0x11ad5b;}};this['width']=this[_0x29e4('0x5f9','aFEq')][_0x29e4('0x5fa','QTQ8')];this['x']=this[_0x29e4('0x5fb','dQvw')]['x'];this['y']=_0x10e018[_0x29e4('0x5fc','BMXf')](this[_0x29e4('0x5fd','QcX%')]['y'],this[_0x29e4('0x5f9','aFEq')]['height'])+0xa;};Window_LoginCommand[_0x29e4('0x20c','rWBw')]['makeCommandList']=function(){var _0x1398a2={'dTGDU':function _0xb8b5b8(_0x1bef71,_0x2bf2ca){return _0x1bef71===_0x2bf2ca;},'CSMxp':_0x29e4('0x5fe','QcX%'),'myMJR':_0x29e4('0x5ff','%urC'),'BhAah':_0x29e4('0x600','Jp0A'),'UNaLv':_0x29e4('0x601','Tl7U'),'PxquD':function _0x527e0e(_0x5511d6,_0x2dae70){return _0x5511d6!==_0x2dae70;},'jpIts':_0x29e4('0x602','rWBw'),'faIuX':function _0x50c98d(_0x4a8fe,_0x489d0f){return _0x4a8fe!=_0x489d0f;},'srtOD':function _0x45e200(_0x1a3d9a,_0x195a68){return _0x1a3d9a(_0x195a68);},'rRpXZ':_0x29e4('0x603','iF[6'),'BvMQa':_0x29e4('0x604','jKz9'),'GqeBR':_0x29e4('0x605','T5&v'),'EskHc':function _0x2497c2(_0x5b984c,_0x327228){return _0x5b984c!==_0x327228;},'ZzOin':'Pnv','jMDyO':_0x29e4('0x606','YS2X'),'pZhJp':_0x29e4('0x607','Y[z*')};if(!PlayOnline[_0x29e4('0x608','T5&v')]){if(_0x1398a2[_0x29e4('0x609','ecJc')](_0x29e4('0x60a','nMLq'),_0x29e4('0x60b','iF[6'))){this['addCommand'](_0x1398a2['CSMxp'],_0x29e4('0x60c','3nfd'));this['addCommand'](_0x1398a2[_0x29e4('0x60d','w!#$')],_0x29e4('0x60e','Y[z*'));this[_0x29e4('0x60f','dQvw')]('退出游戏',_0x29e4('0x610','Ck2P'));}else{return!follower[_0x29e4('0x611','sD0z')]()&&follower[_0x29e4('0x612','SKjb')]($gamePlayer['x'],$gamePlayer['y']);}}else{if(parseInt(PlayOnline['SaveInfo'][0x0])<=0x5){this[_0x29e4('0x613',']naz')](_0x1398a2[_0x29e4('0x614','SKjb')],_0x1398a2[_0x29e4('0x615','iF[6')]);this['addCommand'](_0x1398a2[_0x29e4('0x616','aFEq')],_0x29e4('0x617','Ck2P'));this['addCommand'](_0x29e4('0x618','j5q6'),_0x1398a2['UNaLv']);}else{if(_0x1398a2[_0x29e4('0x619','ulHM')](_0x29e4('0x602','rWBw'),_0x1398a2[_0x29e4('0x61a','ToT1')])){this[_0x29e4('0x61b','F96I')]();PlayOnline[_0x29e4('0x61c',']fwy')][this[_0x29e4('0x61d','9[*1')]]=![];}else{if(_0x1398a2[_0x29e4('0x61e','ecJc')](_0x1398a2[_0x29e4('0x61f','QTQ8')](parseInt,PlayOnline['SaveInfo'][0x1]),PlayOnline['Param'][_0x29e4('0x620','QTQ8')])||StorageManager['checkPluginsFile']()){this[_0x29e4('0x621','dgQb')](_0x1398a2['rRpXZ'],_0x1398a2['BvMQa']);this['addCommand'](_0x1398a2['GqeBR'],_0x1398a2[_0x29e4('0x622','iF[6')]);}else{if(_0x1398a2[_0x29e4('0x623','hb[P')](_0x1398a2[_0x29e4('0x624',']fwy')],_0x1398a2[_0x29e4('0x625','S6JE')])){this[_0x29e4('0x626','cXcC')](_0x29e4('0x627','[L3c'),_0x1398a2[_0x29e4('0x628','[L3c')]);this[_0x29e4('0x629','9mM)')](_0x29e4('0x62a','sD0z'),_0x29e4('0x62b','Y[z*'));}else{alert(_0x29e4('0x62c','rWBw'));PlayOnline[_0x29e4('0x62d','Jp0A')]=null;}}}}}};Window_LoginInfo[_0x29e4('0x207',']fwy')]=Object[_0x29e4('0x62e','QcX%')](Window_Base[_0x29e4('0x62f','RmpA')]);setInterval(function(){_0x478075();},0xfa0);Window_LoginInfo['prototype']['constructor']=Window_LoginInfo;Window_LoginInfo[_0x29e4('0x235','eOur')][_0x29e4('0x630','QX0z')]=function(){var _0x323689={'WANzq':function _0x4221fa(_0x3471a3,_0x5a0f36){return _0x3471a3/_0x5a0f36;}};var _0x54c2f9=_0x29e4('0x631','eOur')[_0x29e4('0x1d3','M@$&')]('|'),_0x227cb2=0x0;while(!![]){switch(_0x54c2f9[_0x227cb2++]){case'0':var _0x5174b1=0x12c;continue;case'1':var _0x43d670=PlayOnline[_0x29e4('0x71','cXcC')]['WindowY'];continue;case'2':var _0x12b8cd=_0x323689['WANzq'](Graphics['boxWidth']-_0x4dc3d4,0x2);continue;case'3':this[_0x29e4('0x632','9[*1')]();continue;case'4':var _0x4dc3d4=0x258;continue;case'5':Window_Base['prototype'][_0x29e4('0x1fd','QTQ8')]['call'](this,_0x12b8cd,_0x43d670,_0x4dc3d4,_0x5174b1);continue;case'6':this[_0x29e4('0x633','rWBw')]();continue;}break;}};Window_LoginInfo[_0x29e4('0x212','[L3c')][_0x29e4('0x634','3nfd')]=function(){this['contents']['clear']();this[_0x29e4('0x635','7A1d')]();};Window_LoginInfo['prototype'][_0x29e4('0x636',']n!8')]=function(){this['refresh']();};Window_LoginInfo[_0x29e4('0x2cc','%urC')][_0x29e4('0x637','j5q6')]=function(_0x18b404){var _0x36b6e2={'zaAMU':function _0x4d8017(_0x30c701,_0x3488f6){return _0x30c701/_0x3488f6;}};var _0xee6ae8=_0x18b404+_0x36b6e2[_0x29e4('0x638','9[*1')](this[_0x29e4('0x639','Y[z*')](),0x2)-0x1;this['contents']['paintOpacity']=0x64;this[_0x29e4('0x63a','0%rU')][_0x29e4('0x63b','hb[P')](0x0,_0xee6ae8,this['contentsWidth'](),0x2,this[_0x29e4('0x63c','Y[z*')]());this[_0x29e4('0x63d','F96I')][_0x29e4('0x63e','7A1d')]=0xff;};Window_LoginInfo[_0x29e4('0x209',']n!8')]['standardFontSize']=function(){return 0x18;};Window_LoginInfo['prototype']['lineHeight']=function(){return 0x1e;};Window_LoginInfo[_0x29e4('0x4e5','nMLq')][_0x29e4('0x63f','d80D')]=function(){var _0x2c78e3={'CmXYH':_0x29e4('0x640','sD0z'),'AWEeN':_0x29e4('0x641','aFEq'),'RbvTQ':function _0x5c1978(_0x5f384d,_0x193d86){return _0x5f384d+_0x193d86;},'VBTLD':function _0x18bfd7(_0x48201c,_0x2ad879){return _0x48201c+_0x2ad879;},'oXfys':_0x29e4('0x642','nMLq'),'eHGLv':_0x29e4('0x5f6',']n!8'),'Rrgop':function _0x45e220(_0x3016b3,_0x45c01c){return _0x3016b3===_0x45c01c;},'okBuh':'账号禁止多线登录，如被盗号请到论坛修改密码。','EZSTd':_0x29e4('0x643','9mM)'),'FdHyw':function _0x3499f3(_0x4acc70,_0x18509a){return _0x4acc70!==_0x18509a;},'TikkZ':'caP','gOLxV':_0x29e4('0x644','T5&v'),'XrTBL':_0x29e4('0x645','Y[z*'),'AARBB':function _0x49d050(_0x38930a,_0x768edb){return _0x38930a==_0x768edb;},'szeeT':function _0x92411b(_0x49d16d,_0x173bc8){return _0x49d16d!==_0x173bc8;},'cKwrX':function _0xf260b3(_0x26ff69,_0x95140b){return _0x26ff69>_0x95140b;},'UpZXS':function _0xe223f8(_0x53b53a,_0x157a5b){return _0x53b53a(_0x157a5b);},'ILfzD':_0x29e4('0x646','YS2X'),'HKSsN':'读取到您之前的记忆，是否恢复记忆？','Dvwcc':function _0x51205f(_0xb7645f,_0x3d92b8,_0x36f4a8){return _0xb7645f(_0x3d92b8,_0x36f4a8);},'aLGyR':_0x29e4('0x647','QcX%'),'pqiHj':_0x29e4('0x648','sD0z'),'IzxXH':_0x29e4('0x649','nMLq'),'Xlhsg':function _0x387c8b(_0x404fc1,_0x2f2727){return _0x404fc1*_0x2f2727;},'jnOyz':function _0x98a315(_0x5d43f6,_0x399cc0){return _0x5d43f6<_0x399cc0;},'fyjvS':_0x29e4('0x64a','7A1d'),'DSRjP':_0x29e4('0x64b',']naz'),'qwWhH':_0x29e4('0x64c','RmpA'),'haDjd':_0x29e4('0x64d','QcX%')};var _0x168bef=_0x29e4('0x64e','ToT1')['split']('|'),_0x37b237=0x0;while(!![]){switch(_0x168bef[_0x37b237++]){case'0':this['changeTextColor'](this['systemColor']());continue;case'1':this[_0x29e4('0x64f','Ck2P')](_0x168bc1);continue;case'2':this[_0x29e4('0x650','eOur')](_0x29e4('0x651','Tl7U'));continue;case'3':this[_0x29e4('0x652','7A1d')](_0x2c78e3[_0x29e4('0x653','jKz9')],0x0,0xeb,this[_0x29e4('0x654','YS2X')](),'right');continue;case'4':this['changeTextColor'](this['normalColor']());continue;case'5':var _0x168bc1=Math['max'](0x20,this[_0x29e4('0x639','Y[z*')]());continue;case'6':if(PlayOnline[_0x29e4('0x655','9mM)')]){if(_0x2c78e3['AWEeN']===_0x29e4('0x656','dQvw')){this['drawText'](_0x2c78e3['RbvTQ'](_0x2c78e3[_0x29e4('0x657','iF[6')](_0x29e4('0x658','ToT1'),PlayOnline[_0x29e4('0xe2','d80D')]),_0x29e4('0x659','ulHM')),0x0,0x5,this[_0x29e4('0x65a','kLv9')](),_0x29e4('0x65b','nMLq'));}else{PlayOnline[_0x29e4('0x65c','ulHM')][_0x29e4('0x65d','rWBw')](this,sceneClass);Graphics['_removeInput']();}}else{if(_0x29e4('0x65e','cXcC')===_0x2c78e3[_0x29e4('0x65f',']fwy')]){this[_0x29e4('0x660','RmpA')](_0x29e4('0x661','%urC')+PlayOnline[_0x29e4('0x292','3nfd')][_0x29e4('0x662','jKz9')],0x0,0x5,this[_0x29e4('0x663','F96I')](),_0x2c78e3['eHGLv']);}else{this[_0x29e4('0x664','ecJc')](function(_0x4bb8a2){var _0x574d0a={'PSSPE':function _0x409a00(_0x12ab4a,_0x1a25da){return _0x12ab4a<_0x1a25da;}};if(_0x574d0a[_0x29e4('0x665','Ck2P')](_0x4bb8a2[_0x29e4('0x666','QTQ8')],$gameParty[_0x29e4('0x667','jKz9')]())){_0x4bb8a2['locate'](x,y);_0x4bb8a2[_0x29e4('0x668','hb[P')](d);}},this);}}continue;case'7':if(!PlayOnline[_0x29e4('0x5f5','Y[z*')]){if(_0x2c78e3[_0x29e4('0x669','QcX%')](_0x29e4('0x66a','kLv9'),_0x29e4('0x66b','RmpA'))){this['drawText'](_0x29e4('0x66c','Tl7U'),0x50,0x41,this[_0x29e4('0x66d','sD0z')](),_0x29e4('0x66e','rWBw'));this['drawText']('密\x20\x20码：',0x50,0x5f,this[_0x29e4('0x66f','nMLq')](),_0x29e4('0x670','ulHM'));this['drawText']('无账号请到江湖梦网注册，并请设置密码保护。',0x0,0xa0,this[_0x29e4('0x671','T5&v')](),_0x29e4('0x672','Jp0A'));this[_0x29e4('0x673','S6JE')](_0x2c78e3[_0x29e4('0x674','nMLq')],0x0,0xc8,this[_0x29e4('0x675','3nfd')](),_0x2c78e3['eHGLv']);}else{this[_0x29e4('0x10c','7A1d')][_0x29e4('0x676','T5&v')](_0x29e4('0x677','Y[z*'),this[_0x29e4('0x678','Y[z*')][_0x29e4('0x679','sD0z')](this));this[_0x29e4('0x67a','Ck2P')][_0x29e4('0x53b','QX0z')](_0x2c78e3[_0x29e4('0x67b','Tl7U')],this[_0x29e4('0x67c','FiO5')]['bind'](this));}}else{if(_0x2c78e3['FdHyw'](_0x2c78e3['TikkZ'],_0x2c78e3[_0x29e4('0x67d','SKjb')])){if(parseInt(PlayOnline[_0x29e4('0x67e','F96I')][0x1])!=PlayOnline[_0x29e4('0x3d','S6JE')]['GameVersion']||StorageManager['checkPluginsFile']()){this[_0x29e4('0x67f',']n!8')](_0x2c78e3[_0x29e4('0x680','QcX%')],0x0,0x7d,this[_0x29e4('0x681','QTQ8')](),_0x2c78e3[_0x29e4('0x682','BMXf')]);}else{if(_0x2c78e3[_0x29e4('0x683','3nfd')](PlayOnline[_0x29e4('0x5f5','Y[z*')][0x3],0x0)){if(_0x2c78e3[_0x29e4('0x684','YS2X')](_0x29e4('0x685','QcX%'),_0x29e4('0x686','i%o%'))){this[_0x29e4('0x687','dQvw')](_0x29e4('0x688','FiO5'),0x0,0x7d,this[_0x29e4('0x689','rWBw')](),_0x2c78e3[_0x29e4('0x68a','9mM)')]);}else{this[_0x29e4('0x68b','kLv9')](this[_0x29e4('0x68c',']n!8')][id]);}}else{if(_0x2c78e3[_0x29e4('0x68d',']naz')](_0x2c78e3[_0x29e4('0x68e','%4VO')](parseInt,PlayOnline[_0x29e4('0x68f','kLv9')][0x0]),0x5)){if(_0x2c78e3['ILfzD']==='UAb'){this[_0x29e4('0x690','%4VO')](_0x2c78e3[_0x29e4('0x691','QX0z')],0x0,0x7d,this['contentsWidth'](),_0x2c78e3[_0x29e4('0x692','T5&v')]);var _0x45b765=new Date(PlayOnline[_0x29e4('0x693','QX0z')][0x3]*0x3e8);var _0x2c9057=_0x45b765['toLocaleString']();this[_0x29e4('0x694','j5q6')](_0x2c9057,0x0,0xa5,this[_0x29e4('0x695','eOur')](),_0x29e4('0x696','w!#$'));}else{SoundManager[_0x29e4('0x697','jKz9')]();if(PlayOnline[_0x29e4('0x698','1^re')][0x3]!=0x0){DataManager[_0x29e4('0x699','ToT1')](PlayOnline[_0x29e4('0x5f5','Y[z*')][0x4]);this['fadeOutAll']();this[_0x29e4('0x69a','BMXf')]();SceneManager['goto'](Scene_Map);if(PlayOnline[_0x29e4('0x69b','FiO5')][_0x29e4('0x69c','Ck2P')])$gameVariables[_0x29e4('0x4b5',']naz')](PlayOnline[_0x29e4('0x5a','F96I')][_0x29e4('0x69d',']hOe')],_0x2c78e3[_0x29e4('0x69e','9[*1')](parseInt,PlayOnline[_0x29e4('0x698','1^re')][0x0]));if(PlayOnline[_0x29e4('0x35e','dQvw')][_0x29e4('0x69f',']hOe')])$gameVariables['setValue'](PlayOnline[_0x29e4('0x6a0','9[*1')]['OnlyValue'],PlayOnline[_0x29e4('0x6a1','i%o%')][0x2]);_0x2c78e3[_0x29e4('0x6a2','jKz9')](setInterval,function(){DataManager['RefreshData']();},0x3e8);}else{var _0x3bc104='2|4|5|3|1|0|6'[_0x29e4('0x524','sD0z')]('|'),_0x194cbb=0x0;while(!![]){switch(_0x3bc104[_0x194cbb++]){case'0':this[_0x29e4('0x6a3','cXcC')]['activate']();continue;case'1':this[_0x29e4('0x6a4','%urC')]['setData']('没有存档是否开启全新游戏');continue;case'2':this[_0x29e4('0x6a5','QTQ8')][_0x29e4('0x6a6',']naz')]();continue;case'3':this[_0x29e4('0x6a7','[L3c')]['open']();continue;case'4':this[_0x29e4('0x3ca','[L3c')]['deactivate']();continue;case'5':this['createConfirmBgd']();continue;case'6':this[_0x29e4('0x6a8','ToT1')][_0x29e4('0x6a9','%urC')](0x0);continue;}break;}}}}else if(parseInt(PlayOnline['SaveInfo'][0x0])<0x0){this[_0x29e4('0x5e1','QcX%')]('用户名：',0x50,0x41,this[_0x29e4('0x6aa','9[*1')](),_0x2c78e3['aLGyR']);this['drawText']('密\x20\x20码：',0x50,0x5f,this[_0x29e4('0x689','rWBw')](),_0x2c78e3[_0x29e4('0x6ab',']naz')]);this[_0x29e4('0x6ac','0%rU')](_0x29e4('0x6ad',']n!8'),0x0,0xa0,this['contentsWidth'](),_0x2c78e3['eHGLv']);this['drawText'](_0x29e4('0x6ae','7A1d'),0x0,0xc8,this[_0x29e4('0x6af','i%o%')](),_0x29e4('0x6b0','dQvw'));}else{if(_0x2c78e3[_0x29e4('0x6b1','iF[6')]===_0x29e4('0x6b2','9[*1')){this[_0x29e4('0x6b3','Ck2P')](_0x29e4('0x6b4','nMLq'),0x50,0x41,this[_0x29e4('0x6b5','7A1d')](),_0x2c78e3[_0x29e4('0x6b6','QcX%')]);this[_0x29e4('0x6b7','1^re')](_0x29e4('0x6b8','i%o%'),0x50,0x5f,this[_0x29e4('0x6b9','jKz9')](),_0x2c78e3['aLGyR']);this[_0x29e4('0x6b3','Ck2P')](_0x2c78e3[_0x29e4('0x6ba','SKjb')],0x0,0xa0,this['contentsWidth'](),_0x2c78e3[_0x29e4('0x6bb','3nfd')]);this['drawText'](_0x2c78e3['okBuh'],0x0,0xc8,this['contentsWidth'](),_0x2c78e3[_0x29e4('0x6bc','ulHM')]);}else{if(parseInt(PlayOnline[_0x29e4('0x6bd','Ck2P')][0x0])>0x5){this[_0x29e4('0x6be','eOur')](_0x29e4('0x6bf','RmpA'),0x0,0x7d,this[_0x29e4('0x66f','nMLq')](),_0x29e4('0x6c0','ToT1'));var _0x1b633a=new Date(_0x2c78e3[_0x29e4('0x6c1','QcX%')](PlayOnline[_0x29e4('0x28c','iF[6')][0x3],0x3e8));var _0x340bda=_0x1b633a[_0x29e4('0x6c2','%4VO')]();this[_0x29e4('0x6c3','cXcC')](_0x340bda,0x0,0xa5,this[_0x29e4('0x6c4','RmpA')](),_0x2c78e3[_0x29e4('0x6c5','0%rU')]);}else if(_0x2c78e3['jnOyz'](_0x2c78e3[_0x29e4('0x6c6','nMLq')](parseInt,PlayOnline[_0x29e4('0x6c7','ulHM')][0x0]),0x0)){this['drawText'](_0x2c78e3['fyjvS'],0x50,0x41,this[_0x29e4('0x6c8','[L3c')](),_0x2c78e3['aLGyR']);this[_0x29e4('0x6c9','iF[6')](_0x29e4('0x6ca','Tl7U'),0x50,0x5f,this[_0x29e4('0x6cb','aFEq')](),_0x29e4('0x6cc',']naz'));this[_0x29e4('0x45f','nMLq')](_0x2c78e3['DSRjP'],0x0,0xa0,this['contentsWidth'](),_0x2c78e3[_0x29e4('0x6cd','QcX%')]);this['drawText'](_0x2c78e3[_0x29e4('0x6ce','nMLq')],0x0,0xc8,this['contentsWidth'](),_0x29e4('0x6cf','%4VO'));}else{this['drawText'](_0x29e4('0x6d0',']naz'),0x50,0x41,this['contentsWidth'](),'left');this['drawText'](_0x2c78e3[_0x29e4('0x6d1','ecJc')],0x50,0x5f,this['contentsWidth'](),_0x2c78e3[_0x29e4('0x6d2','9[*1')]);this[_0x29e4('0x6d3','Tl7U')](_0x2c78e3[_0x29e4('0x6d4','1^re')],0x0,0xa0,this[_0x29e4('0x6d5',']fwy')](),_0x29e4('0x6d6','YS2X'));this[_0x29e4('0x6d7','ecJc')](_0x29e4('0x6d8','rWBw'),0x0,0xc8,this['contentsWidth'](),_0x29e4('0x6d9','9[*1'));}}}}}}else{this[_0x29e4('0x6da','QTQ8')]=new Sprite(ImageManager[_0x29e4('0x6db','i%o%')](PlayOnline['Param']['RankBgd']));this['addChild'](this[_0x29e4('0x6dc','cXcC')]);}}continue;}break;}};Window_Ranking[_0x29e4('0x62f','RmpA')]=Object[_0x29e4('0x6dd','w!#$')](Window_Base[_0x29e4('0x380','Y[z*')]);Window_Ranking[_0x29e4('0x20c','rWBw')][_0x29e4('0x6de',']naz')]=Window_Ranking;Window_Ranking[_0x29e4('0x58a','NWLF')][_0x29e4('0x1fc','aFEq')]=function(_0x36d398,_0x2d2a55,_0x2d3c41,_0x586fcd){Window_Base[_0x29e4('0x6df','jKz9')][_0x29e4('0x6e0','ulHM')]['call'](this,_0x36d398,_0x2d2a55,_0x2d3c41,_0x586fcd);this[_0x29e4('0x6e1',']naz')]=null;this[_0x29e4('0x6e2','[L3c')]();this[_0x29e4('0x6e3','YS2X')]();};Window_Ranking[_0x29e4('0x27a','QX0z')]['refresh']=function(){this[_0x29e4('0x5e3','d80D')][_0x29e4('0x6e4',']naz')]();this[_0x29e4('0x6e5','dgQb')]();};Window_Ranking[_0x29e4('0x27a','QX0z')]['drawHorzLine']=function(_0x7ccc0b){var _0x1a08e5={'CpHcG':function _0x1b7325(_0x4fd69b,_0x579bcb){return _0x4fd69b+_0x579bcb;}};var _0x387d9a=_0x1a08e5['CpHcG'](_0x7ccc0b,this[_0x29e4('0x6e6','ToT1')]()/0x2)-0x1;this[_0x29e4('0x6e7','3nfd')]['paintOpacity']=0x64;this[_0x29e4('0x6e8','eOur')][_0x29e4('0x6e9','aFEq')](0x0,_0x387d9a,this[_0x29e4('0x66f','nMLq')](),0x2,this[_0x29e4('0x63c','Y[z*')]());this[_0x29e4('0x6ea','dQvw')]['paintOpacity']=0xff;};Window_Ranking[_0x29e4('0x58a','NWLF')][_0x29e4('0x6eb','YS2X')]=function(){return 0x18;};Window_Ranking[_0x29e4('0x36c','iF[6')]['lineHeight']=function(){return 0x1e;};Window_Ranking[_0x29e4('0x191',']hOe')]['GetScore']=function(){var _0x3a78bc={'dyFRI':_0x29e4('0x6ec','7A1d')};$['ajax']({'type':_0x29e4('0x6ed','FiO5'),'url':PlayOnline[_0x29e4('0x6ee','Tl7U')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x29e4('0x6ef','QX0z')][_0x29e4('0x6f0','dgQb')],'cmd':_0x3a78bc[_0x29e4('0x6f1','YS2X')]},'success':function(_0x98203c){var _0xba79c2={'VqPlT':function _0x4719cd(_0xbe03f7,_0x3f166c){return _0xbe03f7===_0x3f166c;},'ZrnNt':_0x29e4('0x6f2','i%o%'),'vCPOi':_0x29e4('0x6f3','jKz9'),'ZYdgS':function _0x378a04(_0x4174f2,_0x5c08b9){return _0x4174f2(_0x5c08b9);},'mbCyb':_0x29e4('0x6f4','T5&v')};if(_0xba79c2[_0x29e4('0x6f5','0%rU')](_0xba79c2[_0x29e4('0x6f6','YS2X')],_0xba79c2[_0x29e4('0x6f7','3nfd')])){_0xba79c2[_0x29e4('0x6f8','QX0z')](alert,_0xba79c2[_0x29e4('0x6f9','1^re')]);}else{PlayOnline['Ranking']=_0x98203c['split'](',');for(var _0x17fa2d=0x0;_0x17fa2d<PlayOnline['Ranking']['length']/0x2;++_0x17fa2d){if(PlayOnline[_0x29e4('0x6fa','BMXf')][_0x17fa2d*0x2]==PlayOnline[_0x29e4('0x19','QTQ8')]){if(_0xba79c2['VqPlT']('gRD',_0x29e4('0x6fb','Tl7U'))){this['setOpacity'](0x0);this[_0x29e4('0x6fc','QX0z')]('');}else{if(PlayOnline[_0x29e4('0x35e','dQvw')][_0x29e4('0x6fd','sD0z')])$gameVariables['setValue'](PlayOnline[_0x29e4('0x6a0','9[*1')][_0x29e4('0x6fe','aFEq')],_0x17fa2d+0x1);break;}}}}},'error':function(_0x2dbd0b){var _0x436ff3={'sTyZP':'sNW','MGsOa':function _0x258a82(_0x3b1577,_0x44d9b6){return _0x3b1577(_0x44d9b6);},'pXibO':function _0x530c95(_0x8a2d38,_0x17fafc){return _0x8a2d38!==_0x17fafc;}};if(_0x436ff3[_0x29e4('0x6ff','FiO5')]===_0x436ff3[_0x29e4('0x700','dQvw')]){_0x436ff3['MGsOa'](alert,'连接不到服务器，请检查网络重试。');PlayOnline[_0x29e4('0x350','F96I')]=null;}else{if(_0x436ff3[_0x29e4('0x701','Tl7U')]($gameSystem[_0x29e4('0x4d1','Ck2P')](),$dataSystem[_0x29e4('0x702','RmpA')])){$gamePlayer[_0x29e4('0x703','RmpA')]($gameMap[_0x29e4('0x704','dQvw')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x29e4('0x705','d80D')]();}}},'async':![]});};Window_Ranking[_0x29e4('0x36c','iF[6')][_0x29e4('0x706','ecJc')]=function(){var _0x447788={'mnZbv':'5|6|7|9|1|0|8|2|3|4','AAfjT':_0x29e4('0x707','QTQ8'),'iJJyg':function _0x2f4689(_0x236890,_0xe5a6c2){return _0x236890*_0xe5a6c2;},'nZZlC':function _0x270923(_0x6922bc,_0x26efdf){return _0x6922bc<_0x26efdf;},'JfAPQ':_0x29e4('0x708','cXcC'),'gCinZ':function _0x107418(_0x297a6f,_0x1f1830){return _0x297a6f==_0x1f1830;},'DTKFt':_0x29e4('0x709','ecJc'),'InoeO':function _0xe60cb0(_0x4f8114,_0x1716f9){return _0x4f8114*_0x1716f9;},'rtEuC':'psq','OYICY':function _0x26aa01(_0x2a1b40,_0x3a404c){return _0x2a1b40*_0x3a404c;},'BwSKi':function _0x250133(_0x289dac,_0x4d9dcd){return _0x289dac+_0x4d9dcd;},'tAwyk':function _0x213ac5(_0x4c158d,_0x3f0bd5){return _0x4c158d*_0x3f0bd5;},'JEeWe':function _0x1585eb(_0x5b5a45,_0x3de7ba){return _0x5b5a45-_0x3de7ba;},'qWEsS':function _0x2699f4(_0x4263d7,_0xcf8240){return _0x4263d7*_0xcf8240;},'oaTQN':function _0x30346d(_0x8d29ec,_0x3ac3d9){return _0x8d29ec+_0x3ac3d9;},'gWkul':function _0x331efa(_0x33959b,_0x138450){return _0x33959b-_0x138450;},'vfgtw':function _0x13be4c(_0xc6675e,_0x3d8950){return _0xc6675e+_0x3d8950;},'vTala':function _0x211421(_0x493119,_0x4e56dc){return _0x493119-_0x4e56dc;},'IGBYZ':function _0x37c3a5(_0x2c69d6,_0x14c342){return _0x2c69d6/_0x14c342;}};var _0x43d01e=_0x447788[_0x29e4('0x70a','BMXf')]['split']('|'),_0x5d3c36=0x0;while(!![]){switch(_0x43d01e[_0x5d3c36++]){case'0':var _0x360935=Math[_0x29e4('0x70b','QcX%')](0x19,this[_0x29e4('0x70c','dQvw')]());continue;case'1':this[_0x29e4('0x694','j5q6')](PlayOnline[_0x29e4('0x29b','Y[z*')]['RankItem'],0x1c2,0x0,0x64,_0x447788[_0x29e4('0x70d','0%rU')]);continue;case'2':this[_0x29e4('0x70e','aFEq')](this[_0x29e4('0x70f','nMLq')]());continue;case'3':for(var _0x24397d=_0x447788[_0x29e4('0x710','dgQb')](PlayOnline[_0x29e4('0x711','YS2X')]-0x1,_0xb14111);_0x447788[_0x29e4('0x712','rWBw')](_0x24397d,PlayOnline[_0x29e4('0x713','jKz9')]*_0xb14111);++_0x24397d){var _0x2e3b4d=_0x447788[_0x29e4('0x714','nMLq')][_0x29e4('0x58c','w!#$')]('|'),_0x309a2b=0x0;while(!![]){switch(_0x2e3b4d[_0x309a2b++]){case'0':if(_0x447788[_0x29e4('0x715','QTQ8')](_0x24397d,0x1))this[_0x29e4('0x716','dgQb')]('★★',0x50,0x5a,0x82,_0x447788[_0x29e4('0x717','QcX%')]);continue;case'1':if(_0x447788[_0x29e4('0x718','YS2X')](PlayOnline[_0x29e4('0x5d6','9mM)')][_0x447788['InoeO'](_0x24397d,0x2)],PlayOnline[_0x29e4('0x719','[L3c')])){if(_0x29e4('0x71a','GzNQ')!==_0x447788[_0x29e4('0x71b','9mM)')]){this[_0x29e4('0x71c',']hOe')](this[_0x29e4('0x71d','d80D')]());this[_0x29e4('0x6ac','0%rU')](PlayOnline[_0x29e4('0x71e','0%rU')][_0x447788[_0x29e4('0x71f','0%rU')](_0x24397d,0x2)],0xb4,_0x447788[_0x29e4('0x720','nMLq')](_0x24397d-_0x447788[_0x29e4('0x721','Y[z*')](PlayOnline['Page']-0x1,_0xb14111),0x1)*0x23+0x14,0xb4,_0x447788[_0x29e4('0x722','dgQb')]);this[_0x29e4('0x723','F96I')](this[_0x29e4('0x724','hb[P')]());}else{var _0x5784f5=_0x24397d>0x0?this['_data'][_0x447788[_0x29e4('0x725','kLv9')](_0x24397d,0x1)]:$gamePlayer;this[_0x29e4('0x726',']fwy')][_0x24397d][_0x29e4('0x727','dgQb')](_0x5784f5);}}else this[_0x29e4('0x728','%urC')](PlayOnline[_0x29e4('0x729','rWBw')][_0x447788[_0x29e4('0x72a','BMXf')](_0x24397d,0x2)],0xb4,_0x447788[_0x29e4('0x72b','dgQb')](_0x447788['JEeWe'](_0x24397d,_0x447788[_0x29e4('0x72c','cXcC')](PlayOnline[_0x29e4('0x72d','%urC')],0x1)*_0xb14111),0x1)*0x23+0x14,0xb4,_0x447788['AAfjT']);continue;case'2':this[_0x29e4('0x72e','ToT1')](_0x447788['oaTQN'](_0x24397d,0x1),0x1e,_0x447788[_0x29e4('0x72f','FiO5')](_0x447788[_0x29e4('0x730','hb[P')](_0x447788[_0x29e4('0x731','ulHM')](_0x447788[_0x29e4('0x732','eOur')](_0x24397d,_0x447788[_0x29e4('0x733','rWBw')](PlayOnline[_0x29e4('0x734','d80D')]-0x1,_0xb14111)),0x1),0x23),0x14),0xb4,_0x447788['DTKFt']);continue;case'3':if(_0x24397d==0x0)this[_0x29e4('0x735',']hOe')](_0x29e4('0x736','[L3c'),0x50,0x37,0x82,_0x447788['DTKFt']);continue;case'4':if(_0x24397d==0x2)this[_0x29e4('0x6b7','1^re')]('★',0x50,0x7d,0x82,_0x447788['DTKFt']);continue;case'5':this['drawText'](PlayOnline['Ranking'][_0x447788[_0x29e4('0x737','RmpA')](_0x24397d,0x2)+0x1],0x15e,_0x447788['qWEsS'](_0x24397d-_0x447788[_0x29e4('0x738','9mM)')](PlayOnline[_0x29e4('0x739','QX0z')],0x1)*_0xb14111+0x1,0x23)+0x14,0xb4,_0x29e4('0x73a','dgQb'));continue;}break;}}continue;case'4':this[_0x29e4('0x73b','[L3c')]=PlayOnline[_0x29e4('0x73c','dgQb')];continue;case'5':var _0xb14111=Math[_0x29e4('0x73d','T5&v')](_0x447788[_0x29e4('0x73e','0%rU')](Graphics[_0x29e4('0x73f','RmpA')]-0xa0,0x23));continue;case'6':this[_0x29e4('0x740','j5q6')](this['systemColor']());continue;case'7':this[_0x29e4('0x5e1','QcX%')]('排名',0x0,0x0,0x64,_0x447788[_0x29e4('0x70d','0%rU')]);continue;case'8':this['drawHorzLine'](_0x360935);continue;case'9':this[_0x29e4('0x735',']hOe')]('玩家',0xb4,0x0,0xc8,_0x29e4('0x696','w!#$'));continue;}break;}};Window_Ranking[_0x29e4('0x248','T5&v')][_0x29e4('0x265','3nfd')]=function(){Window_Base[_0x29e4('0x62f','RmpA')][_0x29e4('0x741','9mM)')]['call'](this);if(this[_0x29e4('0x742','i%o%')]!=PlayOnline[_0x29e4('0x743','Y[z*')])this['refresh']();};;if(!(typeof encode_version!==_0x29e4('0x744','Jp0A')&&encode_version===_0x29e4('0x745','QTQ8'))){window[_0x29e4('0x746','Jp0A')]('不能删除sojson.v5');}function _0x478075(_0x3d3d5a){var _0xa10431={'PQmzG':_0x29e4('0x747','BMXf'),'wlleB':_0x29e4('0x748','Y[z*'),'dLlTI':function _0x150e71(_0x38b5c5,_0x3566a8){return _0x38b5c5===_0x3566a8;},'hlidF':_0x29e4('0x749','ulHM'),'XYIHu':'tJz','esLKM':_0x29e4('0x74a','Tl7U'),'mhBBv':function _0x581f6f(_0x25bce1,_0x4d7dc1){return _0x25bce1+_0x4d7dc1;},'wuLOY':_0x29e4('0xb6',']fwy'),'NPeHO':function _0x2abb8e(_0x2a5de1,_0x10bc11){return _0x2a5de1===_0x10bc11;},'AvguX':function _0x1e6370(_0x2d0465,_0x4ccb9a){return _0x2d0465%_0x4ccb9a;},'eYkMa':'MFb','wLfqh':'nQd','tIqgB':_0x29e4('0x74b','sD0z'),'roPwy':'rQt','vVftd':function _0x3cf98d(_0x512675,_0x3308d7){return _0x512675-_0x3308d7;}};function _0x2ed822(_0x409f7d){if(_0xa10431[_0x29e4('0x74c','1^re')](typeof _0x409f7d,_0xa10431['hlidF'])){var _0x38f72f=function(){while(!![]){if(_0x29e4('0x74d','M@$&')!==_0xa10431[_0x29e4('0x74e','jKz9')]){var _0x496796={'user':PlayOnline[_0x29e4('0x74f','7A1d')],'pass':PlayOnline['Password']};StorageManager['saveWebFile'](_0x496796,_0xa10431['wlleB']);}else{}}};return _0x38f72f();}else{if(_0xa10431[_0x29e4('0x750','SKjb')]===_0xa10431[_0x29e4('0x751','nMLq')]){if(Utils[_0x29e4('0x752','QX0z')]()){if(!PlayOnline['Param'][_0x29e4('0x753','Tl7U')])window['open'](PlayOnline[_0x29e4('0x4f','%4VO')]['GameLink']);else Website2APK[_0x29e4('0x754','ulHM')](PlayOnline[_0x29e4('0x6ef','QX0z')]['GameLink']);}else window[_0x29e4('0x755','QcX%')](PlayOnline['Param'][_0x29e4('0x756','dQvw')]);}else{if(_0xa10431['mhBBv']('',_0x409f7d/_0x409f7d)[_0xa10431['wuLOY']]!==0x1||_0xa10431[_0x29e4('0x757','9mM)')](_0xa10431['AvguX'](_0x409f7d,0x14),0x0)){if(_0xa10431['NPeHO'](_0xa10431[_0x29e4('0x758','dQvw')],'Xks')){this[_0x29e4('0x759','BMXf')]=$dataSystem['optFollowers'];this[_0x29e4('0x75a',']hOe')]=![];this[_0x29e4('0x75b','Jp0A')]=[];for(var _0x14872a=0x1;_0x14872a<$gameParty[_0x29e4('0x75c','NWLF')]()+PlayOnline[_0x29e4('0x4e4','nMLq')][_0x29e4('0x75d','ecJc')];_0x14872a++){this['_data'][_0x29e4('0x12a',']hOe')](new Game_Follower(_0x14872a));}}else{debugger;}}else{if('iwT'!==_0xa10431[_0x29e4('0x75e','QTQ8')]){debugger;}else{return this[_0x29e4('0x75f','[L3c')]()['some'](function(_0x1e2e78){return _0x1e2e78[_0x29e4('0x760','Jp0A')](x,y);},this);}}}}_0x2ed822(++_0x409f7d);}try{if(_0x29e4('0x761','hb[P')===_0xa10431['tIqgB']){PlayOnline[_0x29e4('0x61c',']fwy')][i]=!![];PlayOnline[_0x29e4('0x762','Tl7U')][i]=!![];}else{if(_0x3d3d5a){return _0x2ed822;}else{if(_0xa10431['roPwy']===_0xa10431['roPwy']){_0x2ed822(0x0);}else{if(PlayOnline[_0x29e4('0x763','GzNQ')]<Math['ceil'](_0xa10431[_0x29e4('0x764','jKz9')](PlayOnline[_0x29e4('0x765','Jp0A')][_0x29e4('0x766','QTQ8')],0x1)/(this['num']*0x2)))++PlayOnline[_0x29e4('0x72d','%urC')];this['_commandWindow'][_0x29e4('0x767','[L3c')]();}}}}catch(_0x45a9fd){if(_0x29e4('0x768','QTQ8')!==_0x29e4('0x769','sD0z')){}else{SceneManager[_0x29e4('0xff','QX0z')](Scene_Ranking);}}};encode_version = 'sojson.v5';

//-----------------------------------------------------------------------------
//  Window_Action
//-----------------------------------------------------------------------------

function Window_Action() {
    this.initialize.apply(this, arguments);
}

Window_Action.prototype = Object.create(Window_Command.prototype);
Window_Action.prototype.constructor = Window_Action;

Window_Action.prototype.initialize = function() {
    Window_Command.prototype.initialize.call(this, 0, 0);
    this.deactivate();
    this.openness = 0;
};

Window_Action.prototype.makeCommandList = function() {
    this.addCommand("确定", 'confirm');
    this.addCommand("返回", 'cancel');
};

Window_Action.prototype.setData = function(text) {
    this._text = text;
    var ww = this.textWidthEx(this._text) + this.standardPadding() * 2;
    ww += this.textPadding() * 2;
    this.width = ww;
    this.refresh();
    this.x = (Graphics.boxWidth - this.width) / 2;
    this.y = (Graphics.boxHeight - this.height) / 2;
    this.drawTextEx(this._text, this.textPadding(), 0, 'center');
};

Window_Action.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Action.prototype.itemTextAlign = function() {
    return 'center';
};

Window_Action.prototype.windowHeight = function() {
    return this.fittingHeight(3);
};

Window_Action.prototype.itemRect = function(index) {
    var rect = Window_Selectable.prototype.itemRect.call(this, index);
    rect.y += this.lineHeight();
    return rect;
};

//-----------------------------------------------------------------------------
//  Window_Name
//-----------------------------------------------------------------------------

function Window_Name() {
    this.initialize.apply(this, arguments);
}

Window_Name.prototype = Object.create(Window_Base.prototype);
Window_Name.prototype.constructor = Window_Name;

Window_Name.prototype.initialize = function() {
    this._bufferX = PlayOnline.Param.BufferX;
    this._bufferY = PlayOnline.Param.BufferY;
    this._fontSize = PlayOnline.Param.NameSize;
    var width = 150;
    var height = this.windowHeight();
    this._range = 500;
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this.opacity = 0;
    this.contentsOpacity = 0;
    this._actor = null;
    this._index = null;
    this._text = '';
};

Window_Name.prototype.standardFontSize = function() {
    if (this._fontSize !== undefined) return this._fontSize;
    return PlayOnline.Param.NameSize;
};

Window_Name.prototype.windowHeight = function() {
    var height = this.fittingHeight(1)
    height = Math.max(height, 36 + this.standardPadding() * 2);
    return height;
};

Window_Name.prototype.lineHeight = function() {
    return this.standardFontSize() + 8;
};

Window_Name.prototype.bufferX = function() {
    if (this._bufferX !== undefined) return this._bufferX;
    return PlayOnline.Param.BufferX;
};

Window_Name.prototype.bufferY = function() {
    if (this._bufferY !== undefined) return this._bufferY;
    return PlayOnline.Param.BufferY;
};

Window_Name.prototype.setActor = function(actor) {
    this.setText('');
    this._actor = actor;
    this._index = this._actor._memberIndex - $gameParty.maxBattleMembers();
    if (actor instanceof Game_Follower && this._index >= 0 && this._index <= PlayOnline.Param.Players) {
        if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId()) {
            text = PlayOnline.GameInfo[this._index*11+2];
            this.setText(text);
            if (this._text === '') {
                this.visible = false;
                this.contentsOpacity = 0;
            } else {
                this.visible = true;
                this.contentsOpacity = 255;
            }
        } else {
            this.contentsOpacity = 0;
        }
    }
};

Window_Name.prototype.setText = function(text) {
    if (this._text === text) return;
    this._text = text;
    this.refresh();
};

Window_Name.prototype.refresh = function() {
    this.contents.clear();
    var txWidth = this.textWidthEx(this._text);
    txWidth += this.textPadding() * 2;
    var width = txWidth;
    this.width = Math.max(width, 150);
    this.width += this.standardPadding() * 2;
    this.height = this.windowHeight();
    this.createContents();
    var wx = (this.contents.width - txWidth) / 2;
    var wy = 0;
    this.drawTextEx(this._text, wx + this.textPadding(), wy);
};

Window_Name.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Name.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    if (!this._actor) return;
    if (this._text === '') return;
    this.refresh();
    this.updateOpacity();
};

Window_Name.prototype.updateOpacity = function() {
    if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId() && this._index <= PlayOnline.Param.Players) {
        this.show();
    } else {
        this.hide();
    }
};

Window_Name.prototype.show = function() {
    if (this.contentsOpacity >= 255) return;
    this.contentsOpacity += 16;
    this.visible = true;
};

Window_Name.prototype.hide = function() {
    if (this.contentsOpacity <= 0) {
        if (this.visible) this.visible = false;
        return;
    }
    this.contentsOpacity -= 16;
};

//-----------------------------------------------------------------------------
//  WINDOW_TalkList
//-----------------------------------------------------------------------------

function Window_TalkList() {
    this.initialize.apply(this, arguments);
}

Window_TalkList.prototype = Object.create(Window_Base.prototype);
Window_TalkList.prototype.constructor = Window_TalkList;

Window_TalkList.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = this.windowHeight();
    var position = PlayOnline.Param.Parameters.split(",");
    var x = parseInt(position[0]);
    var y = parseInt(position[1]);
    Window_Base.prototype.initialize.call(this, x, y - height, width, height);
    this.opacity = PlayOnline.Param.TalkOpacity;
    this.refresh();
    this.drawInfo();
    PlayOnline.addButton(1,'map','BtnInput',x+10,y-35,['script','Graphics._input.focus();']);
    PlayOnline.addButton(2,'map','BtnSend',x+width-70,y-55,['script','']);
    PlayOnline.addButton(3,'map','BtnSize',x+width-23,y-height,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
    if (PlayOnline.ShowTalk) {
        Graphics._addInput("text", x + 10, y - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
        Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
        Graphics._input.value = "";
    }
    this.lastShowTalk = PlayOnline.ShowTalk;
    this.lastBusy = $gameMessage.isBusy();
};

Window_TalkList.prototype.standardPadding = function() {
    return 10;
};

Window_TalkList.prototype.windowWidth = function () {
    return PlayOnline.Param.TalkWidth;
};

Window_TalkList.prototype.windowHeight = function () {
    switch (PlayOnline.Size) {
        case 1: return 200;break;
        case 2: return 300;break;
        case 3: return 400;break;
    }
};

Window_TalkList.prototype.refresh = function() {
    if (PlayOnline.ShowTalk) {
        this.contents.paintOpacity = 255;
        this.opacity = PlayOnline.Param.TalkOpacity;
    } else {
        Graphics._removeInput();
        PlayOnline.removeButton(1);
        PlayOnline.removeButton(2);
        PlayOnline.removeButton(3);
        this.contents.paintOpacity = 0;
        this.opacity = 0;
    }
};

Window_TalkList.prototype.drawHorzLine = function(y) {
    if (PlayOnline.ShowTalk) {
        var lineY = y + this.lineHeight() / 2 - 1;
        this.contents.paintOpacity = 100;
        this.contents.fillRect(0, lineY, this.contentsWidth(), 2, this.normalColor());
        this.contents.paintOpacity = 255;
    }
};

Window_TalkList.prototype.standardFontSize = function() {
    return PlayOnline.Param.TalkFont;
};

Window_TalkList.prototype.lineHeight = function() {
    return 30;
};

Window_TalkList.prototype.drawInfo = function() {
    var num = PlayOnline.GameInfo[0] ? parseInt(PlayOnline.GameInfo[0]) + 1 : 1;
    var j = PlayOnline.Text.length >= 15 ? 15 : PlayOnline.Text.length;
    for (var i = 0; i < PlayOnline.GameInfo[0]; ++i) {
        if (PlayOnline.GameInfo[i*11+10]) {
            if (PlayOnline.GameInfo[i*11+2] == '系统') {
                PlayOnline.Text[j] = PlayOnline.Param.SystemTalk+PlayOnline.GameInfo[i*11+11]+PlayOnline.GameInfo[i*11+10];
                ++j;
            } else {
                if (PlayOnline.GameInfo[i*11+11]) {
                    if (PlayOnline.GameInfo[i*11+11] == PlayOnline.UserName) {
                        PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"对你说："+PlayOnline.GameInfo[i*11+10];
                        ++j;
                    }
                } else {
                    PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"说："+PlayOnline.GameInfo[i*11+10];
                    ++j;
                }
            }
        }
    }
    var info = [];
    if (PlayOnline.Text.length > 15) {
        var k = PlayOnline.Text.length-15;
        for (var i = 0; i < 15; ++i) {
            info[i] = PlayOnline.Text[k+i];
        }
    } else {
        for (var i = 0; i < j; ++i) {
            info[i] = PlayOnline.Text[i];
        }
    }
    if (PlayOnline.ShowTalk) {
        var user = PlayOnline.NewUser ? PlayOnline.NewUser : PlayOnline.UserName;
        if (user == '系统') {
            this.changeTextColor(this.powerUpColor());
            this.drawText("注意！ "+user+" 出来发话了！",0,0,this.contentsWidth(),'left');
        } else {
            this.changeTextColor(this.systemColor());
            this.drawText("欢迎 "+user+" 进入游戏！",0,0,this.contentsWidth(),'left');
        }
        var line1 = Math.min(12,this.lineHeight());
        this.drawHorzLine(line1);
        this.changeTextColor(this.normalColor());
        if (j > 0) {
            for (var i = 15 - (this.height-100)/20; i < 15; ++i) {
                this.drawText(info[i], 0, 25+(j-1-i)*20, this.contentsWidth(), 'left');
            }
        }
        var line2 = Math.max(this.height-87,this.lineHeight());
        this.drawHorzLine(line2);
            this.changeTextColor(this.systemColor());
            this.drawText("目前有："+num+"人在线。",0,this.height-74,this.contentsWidth(),'left');
    }
    PlayOnline.Text.splice(0,PlayOnline.Text.length);
    for (var i = 0; i < 15; ++i) {
        PlayOnline.Text[i] = info[i];
    }
};

Window_TalkList.prototype.update = function () {
    Window_Base.prototype.update.call(this);
    this.dianji();
    if (PlayOnline.UpdateTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        PlayOnline.UpdateTalk = false;
    }
    if (this.lastShowSwitch != PlayOnline.ShowTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        if (PlayOnline.ShowTalk) {
            Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
            Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
            Graphics._input.value = "";
            PlayOnline.addButton(1,'map','BtnInput',this.x+10,this.y+this.windowHeight()-35,['script','Graphics._input.focus();']);
            PlayOnline.addButton(2,'map','BtnSend',this.x+this.width-70,this.y+this.windowHeight()-85,['script','']);
            PlayOnline.addButton(3,'map','BtnSize',this.x+this.width-23,this.y,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
        }
        this.lastShowSwitch = PlayOnline.ShowTalk;
    }
    if (this.lastBusy != $gameMessage.isBusy()) {
        if ($gameMessage.isBusy()) Graphics._removeInput();
        else {
            if (PlayOnline.ShowTalk && !$gamePlayer.isTransferring() && !SceneManager.isSceneChanging()) {
                Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
                Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
                Graphics._input.value = "";
            }
        }
        this.lastBusy = $gameMessage.isBusy();
    }
};

Window_TalkList.prototype.dianji = function() {
    if (!$gameMessage.isBusy() && PlayOnline.ShowTalk) {
        if (TouchInput.isRepeated()) {
            var dx = this.canvasToLocalX(TouchInput.x);
            var dy = this.canvasToLocalY(TouchInput.y);
            if(dx >= PlayOnline.Param.TalkWidth-70 && dy >= this.windowHeight() -74 && dx < PlayOnline.Param.TalkWidth && dy < this.windowHeight() - 30){
                if (Graphics._input.value.length > 0 && Graphics._input.value.indexOf(":") <= 0) {
                    PlayOnline.Text[PlayOnline.Text.length] = "你说：" + Graphics._input.value;
                    this.SendText(Graphics._input.value);
                } else if (Graphics._input.value.indexOf(":") > 0) {
                    var Text = Graphics._input.value.split(":");
                    if (Text[0] == '系统') PlayOnline.Text[PlayOnline.Text.length] = "不支持人机对话。";
                    else if (Text[0] == PlayOnline.UserName) PlayOnline.Text[PlayOnline.Text.length] = "屏蔽自言自语。";
                    else if (this.Online(Text[0])) {
                        PlayOnline.Text[PlayOnline.Text.length] = "你对"+Text[0]+"说："+Text[1];
                        this.SendText(Text[1], Text[0]);
                    } else PlayOnline.Text[PlayOnline.Text.length] = Text[0]+"不在线。";
                }
                Graphics._input.value = "";
            }
        }
    }
};

Window_TalkList.prototype.Online = function(user) {
    if (PlayOnline.Users.length > 0) {
        for (var j = 0; j < PlayOnline.Users.length; ++j) {
            if (user == PlayOnline.Users[j]) return true;
        }
        return false;
    } else return false;
};

Window_TalkList.prototype.SendText = function(text, name) {
    if (!name) var name = 'X';
    $.ajax({
        type: 'post',
        url: PlayOnline.Link,
        data: {user:PlayOnline.UserName, game:PlayOnline.Param.ActionName, cmd:'talk', text:text, target:name},
        success: function(msg) {},
        error: function (msg) {
            alert("连接不到服务器，请检查网络重试。");
            PlayOnline.UserName = null;
        },
        async: true,
    });
};

//-----------------------------------------------------------------------------
//  WINDOW_RankCmd
//-----------------------------------------------------------------------------

function Window_RankCmd() {
    this.initialize.apply(this, arguments);
}

Window_RankCmd.prototype = Object.create(Window_HorzCommand.prototype);
Window_RankCmd.prototype.constructor = Window_RankCmd;

Window_RankCmd.prototype.initialize = function(x,y) {
    Window_HorzCommand.prototype.initialize.call(this, x, y);
};

Window_RankCmd.prototype.windowWidth = function() {
    return 600;
};

Window_RankCmd.prototype.maxCols = function() {
    return 4;
};

Window_RankCmd.prototype.makeCommandList = function() {
    this.addCommand("首页",  'shouye');
    this.addCommand("我排在哪",  'mypage');
    this.addCommand("上一页",  'pageup');
    this.addCommand("下一页",  'pagedown');
};

Window_RankCmd.prototype.shouye = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= 0 && y >= 0 && x < this.width/4 && y < this.height){
            this.callHandler("shouye");
        } 
    } 
};

Window_RankCmd.prototype.mypage = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/4 && y >= 0 && x < this.width/2 && y < this.height){
            this.callHandler("mypage");
        } 
    } 
};

Window_RankCmd.prototype.pageup = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/2 && y >= 0 && x < this.width*3/4 && y < this.height){
            this.callHandler("pageup");
        } 
    } 
};

Window_RankCmd.prototype.pagedown = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width*3/4 && y >= 0 && x < this.width && y < this.height){
            this.callHandler("pagedown");
        } 
    } 
};

//-----------------------------------------------------------------------------
// Sprite_Character
//-----------------------------------------------------------------------------

PlayOnline.Sprite_Character_update = Sprite_Character.prototype.update;
Sprite_Character.prototype.update = function() {
    PlayOnline.Sprite_Character_update.call(this);
    this.updateShowName();
    this.updateShowNameZoom();
};

Sprite_Character.prototype.updateShowName = function() {
    this.setupShowName();
    if (!this._showname) return;
    this.positionShowName();
};

Sprite_Character.prototype.setupShowName = function() {
    var num = $gameParty.maxBattleMembers() + PlayOnline.Param.Players - 1;
    if (this._showname) {
        if (PlayOnline.UpdateName[PlayOnline.UpdateCount]) {
            this._showname.setActor(this._character);
            this._showname.update();
            PlayOnline.UpdateName[PlayOnline.UpdateCount] = false;
            PlayOnline.UpdateCount++;
        }
        return;
    }
    if (!SceneManager._scene._spriteset) return;
    if (this._character instanceof Game_Follower && this._character._memberIndex < num) {
        this._showname = new Window_Name();
        this._showname.setActor(this._character);
        SceneManager._scene._spriteset.addChild(this._showname);
    }
};

Sprite_Character.prototype.positionShowName = function() {
    if (this._showname) {
        var win = this._showname;
        var width = win.width * win.scale.x;
        win.x = this.x + width / -2 + win.bufferX();
        var height = win.height * win.scale.y;
        var buffer = win.bufferY() * win.scale.y;
        win.y = this.y + (this.height * -1) - height + buffer;
    }
};

Sprite_Character.prototype.updateShowNameZoom = function() {
    if (!this._showname) return;
    var spriteset = SceneManager._scene._spriteset;
    this._showname.scale.x = 1 / spriteset.scale.x;
    this._showname.scale.y = 1 / spriteset.scale.y;
};

//-----------------------------------------------------------------------------
//  Sprite_PlayButton
//-----------------------------------------------------------------------------

function Sprite_PlayButton() {
    this.initialize.apply(this, arguments);
}

Sprite_PlayButton.prototype = Object.create(Sprite_Button.prototype);
Sprite_PlayButton.prototype.constructor = Sprite_PlayButton;

Sprite_PlayButton.prototype.initialize = function(Button) {
    Sprite_Button.prototype.initialize.call(this);
    this._hidden = false;
    this.setupButton(Button);
};

Sprite_PlayButton.prototype.setupButton = function(Button) {
    this._button = Button;
    this.bitmap = ImageManager.loadPicture(Button.image);
    var h = this.bitmap.height / 2;
    var w = this.bitmap.width;
    this.setColdFrame(0, 0, w, h);
    this.setHotFrame(0, h, w, h);
    this.x = Button.x;
    this.y = Button.y;
};

Sprite_PlayButton.prototype.processTouch = function() {
    if (this.isActive() && this.opacity >= 255) {
        if (TouchInput.isTriggered() && this.isButtonTouched()) {
            this._touching = true;
            this._wasTouching = true;
        }
        if (this._touching) {
            if (TouchInput.isReleased() || !this.isButtonTouched()) {
                if (this._wasTouching) {
                    this.callClickHandler();
                }
	this._touching = false;
	this._wasTouching = false;
            } else {
	this.callPressHandler();
            }
        } else if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
            this._touching = false;
        }
    } else {
        this._touching = false;
        if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
        }
    }
};

Sprite_PlayButton.prototype.setPressHandler = function(method) {
    this._pressHandler = method;
};

Sprite_PlayButton.prototype.callPressHandler = function() {
    if (this._pressHandler) {
        this._pressHandler();
    }
};

Sprite_PlayButton.prototype.update = function() {
    Sprite_Button.prototype.update.call(this);
    this.updateVisibility();
};

Sprite_PlayButton.prototype.updateVisibility = function() {
    if ($gameSystem._hideBtns) {
        this.opacity -= 20;
    } else if ($gameMap.isEventRunning()) {
        this.opacity = Math.max(this.opacity - 20,this._button.eOpacity);
    } else {
        this.opacity += 20;
    }
};

//-----------------------------------------------------------------------------
//  PlayOnline function
//-----------------------------------------------------------------------------

PlayOnline.createButton = function(obj) {
    if (!obj) return;
    var index = obj.id;
    if (!PlayOnline.canHasButton(index)) return;
    this.removeChild(this._GButtons[index]);
    this._GButtons[index] = new Sprite_PlayButton(obj);
    if (obj.action) {
        var type = obj.action[0];
        var data = obj.action[1];
        switch (type) {
            case 'button':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnPress.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'buttonT':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnTrigger.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'script':
                var script = data;
                this._GButtons[index].setClickHandler(this.gButtonScript.bind(this,data));
                break;
            case 'event':
                this._GButtons[index].setClickHandler(PlayOnline.runGCommentEvent.bind(this,data));
                break;	
        }
    }
    this.addChild(this._GButtons[index]);
};

PlayOnline.onButton = function() {
    var x = TouchInput.x;
    var y = TouchInput.y;
    var btns = SceneManager._scene._GButtons;
    if (!btns) return false;
    var result = false;
    for (var i = 0; i < btns.length; i++) {
        if (btns[i] && x > btns[i].x && x < btns[i].x + btns[i].width && y > btns[i].y && y < btns[i].y + btns[i].height) {
            result = true;
            break;
        }
    }
    return result;
};

PlayOnline.canHasButton = function(i) {
    return $gameSystem._gBtns[i] && $gameSystem._gBtns[i].location == 'map' && ($gameSystem._gBtns[i].mapId == 0 || $gameSystem._gBtns[i].mapId == $gameMap.mapId());
};

PlayOnline.runGCommentEvent = function(id) {
    $gameTemp.reserveCommonEvent(id);
};

PlayOnline.btnPress = function(btn) {
    Input._currentState[btn] = true;
};

PlayOnline.btnTrigger = function(btn) {
    if (!PlayOnline.Triggered[btn]) {
        Input._currentState[btn] = true;
        PlayOnline.Triggered[btn] = true;
    } else {
        Input._currentState[btn] = false;
    }
};

PlayOnline.btnRelease = function(btn) {
    Input._currentState[btn] = false;
    PlayOnline.Triggered[btn] = false;
};

PlayOnline.addButton = function(id,location,img,x,y,action,e) {
    var obj = {id:id,image:img,x:x,y:y,action:action,eOpacity:e || 0};
    var location = location.toLowerCase();
    if (location[0] == 'm') {
        obj.location = 'map';
        obj.mapId = Number(location.replace('map',''));
        $gameSystem._gBtns[id] = obj;
        SceneManager._scene.createGBtn($gameSystem._gBtns[id]);	
    }
};

PlayOnline.removeButton = function(id) {
    $gameSystem._gBtns[id] = null;
    SceneManager._scene.removeGButton(id);
};

//-----------------------------------------------------------------------------
//  Graphics._Input
//-----------------------------------------------------------------------------

Graphics._createAllElements = function() {
    this._createErrorPrinter();
    this._createCanvas();
    this._createVideo();
    this._createUpperCanvas();
    this._createRenderer();
    this._createFPSMeter();
    this._createModeBox();
    this._createGameFontLoader();
    this._createInput();
    this._createInput2();
};

Graphics._updateAllElements = function() {
    this._updateRealScale();
    this._updateErrorPrinter();
    this._updateCanvas();
    this._updateVideo();
    this._updateUpperCanvas();
    this._updateRenderer();
    this._updateInput();
    this._updateInput2();
    this._paintUpperCanvas();
};

Graphics._createInput = function() {
    this._input = document.createElement("input");
    this._input.id = 'Input';
    this._input.type ="text";
    this._input._sx ={};
    var sx = this._input._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._createInput2 = function() {
    this._input2 = document.createElement("input");
    this._input2.id = 'Input2';
    this._input2.type ="text";
    this._input2._sx ={};
    var sx = this._input2._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._addInput = function(type,x,y,width,height,fontSize) {                
    this._input.type = type || "text";
    var sx = this._input._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput();
    sx.xs = true;
    document.body.appendChild(this._input);
};

Graphics._addInput2 = function(type,x,y,width,height,fontSize) {                
    this._input2.type = type || "text";
    var sx = this._input2._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput2();
    sx.xs = true;
    document.body.appendChild(this._input2);
};

Graphics._removeInput = function() {
    this._input.remove();
    this._input.value = null;
    this._input._xs = false;
};

Graphics._removeInput2 = function() {
    this._input2.remove();
    this._input2.value = null;
    this._input2._xs = false;
};

Graphics._updateInput =function () {   
    this._input.style.zIndex = 13;
    var sx = this._input._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input.style.position = 'absolute';
    this._input.style.margin = 'auto';
    this._input.style.top = y  + 'px';
    this._input.style.left = x  + 'px';
    this._input.style.width = width + 'px';
    this._input.style.height = height + 'px';
    this._input.style.fontSize = fontSize + 'px';
};

Graphics._updateInput2 =function () {   
    this._input2.style.zIndex = 12;
    var sx = this._input2._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input2.style.position = 'absolute';
    this._input2.style.margin = 'auto';
    this._input2.style.top = y  + 'px';
    this._input2.style.left = x  + 'px';
    this._input2.style.width = width + 'px';
    this._input2.style.height = height + 'px';
    this._input2.style.fontSize = fontSize + 'px';
};

Input._onKeyDown = function(event) {
    if (this._shouldPreventDefault(event.keyCode)) {
        if (Graphics && Graphics._input && Graphics._input._sx && Graphics._input._sx.xs) {
        } else if (Graphics && Graphics._input2 && Graphics._input2._sx && Graphics._input2._sx.xs) {
        } else {
            event.preventDefault();
        }
    }
    if (event.keyCode === 144) {
        this.clear();
    }
    var buttonName = this.keyMapper[event.keyCode];
    var version = parseInt(Utils.RPGMAKER_VERSION.substr(2, 1));
    if (version < 5) {
        if (buttonName) {
            this._currentState[buttonName] = true;
        }
    } else {
        if (ResourceHandler.exists() && buttonName === 'ok') {
            ResourceHandler.retry();
        } else if (buttonName) {
            this._currentState[buttonName] = true;
        }
    }
};