﻿//-----------------------------------------------------------------------------
// PlayOnline.js
// Version: 2.3
//-----------------------------------------------------------------------------
/*:
 * @plugindesc 网络登录同屏聊天排行云存档2.3版
 * @author 江湖梦网 Fanzi
 *
 * @param  游戏名字
 * @desc 登陆后显示欢迎语
 * @default 
 *
 * @param  后台程序
 * @desc 游戏在江湖梦网申请到的后台程序名
 * @default game
 *
 * @param  游戏版本
 * @desc 输入当前游戏的版本号，用于检测游戏更新
 * @default 1
 *
 * @param  下载地址
 * @desc 游戏更新的网站入口，论坛发布指引帖子填写地址
 * @default https://bbs.jhmeng.cn/
 *
 * @param  窗口位置
 * @desc 登陆窗口显示的高度
 * @default 100
 *
 * @param  窗口透明度
 * @desc 调节登录窗口的透明度
 * @default 255
 *
 * @param  弹窗背景
 * @desc 弹出窗口背景图片 384 * 144 img/pictures 留空无背景
 * @default popup
 *
 * @param  存档通知
 * @desc 用于做定点云存档成功的返回信息，自动云存档留空不显示
 * @default 您已成功存档！
 *
 * @param  统一变量
 * @desc 指定一个变量号，开发者可在服务器修改该变量，登录赋值
 * @default 5
 *
 * @param  挂机变量
 * @desc 离线的服务器时间写入该号变量，单位为秒数
 * @default 3
 *
 * @param  排行变量
 * @desc 游戏数据转化为一个变量用于排行依据的变量号
 * @default 2
 *
 * @param  排行记录
 * @desc 用于查看排行后记录当前排名的临时变量号
 * @default 4
 *
 * @param  在线时长
 * @desc 玩家多少分钟内有数据更新认定在线
 * @default 3
 *
 * @param  显示上限
 * @desc 同个地图显示玩家人数限制
 * @default 10
 *
 * @param  角色数量
 * @desc 游戏中的角色数量，根据数据库角色数量填写
 * @default 4
 *
 * @param  移动速度
 * @desc 队友及其他玩家显示的移动速度放慢倍数，可以是小数
 * @default 2
 *
 * @param  队友跟随
 * @desc 选择队友跟随形态，true为随机跟随，false为贪食蛇式
 * @default true
 *
 * @param  队友漫步
 * @desc 队友原地踏步的机率，可以是小数
 * @default 3
 *
 * @param  队友传送
 * @desc 队友远离队长一定距离时，自动传送到队长身边
 * @default 6
 *
 * @param  显名性质
 * @desc 显示玩家的用户名或者角色名，true为用户名，false为第一角色名
 * @default true
 *
 * @param  人名X位
 * @desc 显示玩家名字距离角色头顶原点X位移
 * @default 0
 *
 * @param  人名Y位
 * @desc 显示玩家名字距离头顶原点Y位移
 * @default 30
 *
 * @param  人名字号
 * @desc 显示玩家名字的字体大小
 * @default 16
 *
 * @param  聊天窗XY
 * @desc 聊天窗口显示位置(左下角)
 * @default 600,600
 *
 * @param  聊天窗宽
 * @desc 聊天窗口宽度
 * @default 200
 *
 * @param  聊天明暗
 * @desc 聊天窗口、背景及信息明暗度
 * @default 255
 *
 * @param  聊天字号
 * @desc 聊天窗口显示字体大小
 * @default 20
 *
 * @param  显示聊天
 * @desc 聊天窗口是否默认显示
 * @default true
 *
 * @param  系统说话
 * @desc 用于系统说话开关，提示获得物品等
 * @default 恭喜
 *
 * @param  菜单排行
 * @desc 主菜单指令列表增加显示排行的文本，留空不建立
 * @default 排行
 *
 * @param  排行内容
 * @desc 排行界面显示数字序列的表头文本
 * @default 总战力
 *
 * @param  排行背景
 * @desc 排行界面背景图片 img/pictures 留空无背景
 * @default bgd
 *
 * @param  排行透明度
 * @desc 调节排行窗口的透明度，独立于聊天窗口
 * @default 255
 *
 * @param  息屏登出
 * @desc 手机进入息屏时登出游戏，true为启用，false为不启用
 * @default true
 *
 * @param  屏幕按钮
 * @desc 将按钮图片(仿发送按钮)放在 img/pictures 填入文件名
 * @default 
 *
 * @param  APK打包
 * @desc 用于兼容APK打包后的弹出窗口，Website2APK打包填true，其它填false
 * @default false
 *
 * @help
 * 本插件用于将游戏网络化，为游戏提供服务器对接，接入本插件的游
 * 戏将可发布为PC终端、WEB终端、APP终端皆同步通讯，服务器端提
 * 供账号检测、密码保护、单线登录、玩家同屏、云存档、在线双频道
 * 聊天、玩家排行等功能，在线商城有配套商城插件。
 * 插件参数可供开发者自定义设计游戏，包括透明度与背景图片用于界
 * 面美化；排行变量由开发者在游戏中计算存入指定变量，该变量将与
 * 服务器进行通讯，排行名称自定义；游戏时长为判定玩家在线的依据，
 * 即该时间内玩家有数据更新即为在线；显示上线控制同地图玩家显示
 * 数量；显名性质用以选择同屏玩家显示用户名或者角色名；玩家名字
 * 位置大小均自定义；聊天窗口大小、位置、透明度、字体均可自定义；
 * 挂机变量用以离线经验奖励调用，该数值为服务器离线时间单位秒数；
 * 统一变量用以指定变量由开发者在服务器修改，玩家登录即更新其值；
 * 本插件中玩家同屏功能与集合队友的事件指令有冲突，显示队友跟随
 * 即开启玩家同屏，此时不可集合队友，关闭显示队友即关闭玩家同屏。
 * 本插件内置屏幕按钮功能，如需模拟手机按键可仿发送按钮制作图片
 * 放入pictures文件夹，屏幕按钮参数多个图片用英文逗号隔开，在事件
 * 中执行脚本，具体见后面脚本详述；插件提供一个系统给所有在线玩
 * 家全体发信息的指令，可结合系统说话参数做系统提示，具体见后文。
 * 本插件由 江湖梦网 Fanzi 开发，入驻游戏开发者可免费安装使用，
 * 江湖梦网作为游戏服务平台，承载付费资金的代收与转发。
 *                     有意合作者请登录 江湖梦网 https://bbs.jhmeng.cn/
 *                                               联系QQ：12868880  Fanzi
 * 插件指令:
 * 游戏云存档：SaveGame
 * 需自动云存档则建立并行处理的，公共事件定期执行SaveGame指令，
 * 自动云存档建议600帧执行一次，定点存档直接运行SaveGame指令。
 * 显示聊天窗：ShowTalk
 * 隐藏聊天窗：HideTalk
 * 打开排行榜：SceneRanking
 * 服务器记数：AskValue n   参数 n 填写变量号则将记数值返回该号变量
 * 服务器清数：ResetValue   开发者可设定记数值达到多少则清零
 * 事件发信息：SendWord \C[2]信息内容\C[0]   （颜色代码同显示文本）
 * 例如系统说话参数为：恭喜，插件指令为：SendWord 捡到金子，当某
 * 个玩家触发该指令时则显示“恭喜###捡到金子”。
 * 读取服务器时间：AskData 6 7 8 9             (6、7、8、9为系统变量ID)
 * 注意空格隔开，执行此插件指令后 变量 6 = 年份，变量 7 = 月份
 * 变量 8 = 日期，变量 9 = 钟点（小时数），此功能可用于设计日常任务
 *
 * 脚本指令：
 * 返回游戏登录：DataManager.ResetData();
 * 屏幕按钮脚本：
 * 生成：PlayOnline.addButton(id,'type','img',x,y,['actionType','action'],e);
 * 消除：PlayOnline.removeButton(1);        括号里填按钮ID
 * 脚本参数说明：
 * id：              按键序号，插件已用了1、2、3，新增从4开始，不可重复
 * type：          map/mapX 填map则所有地图通用，填mapX则对应id的
 *                     地图有用，其中X为地图id号
 * img：           按钮图片文件名，注意大小写，不用后缀，img/pictures
 * x：               屏幕坐标X
 * y：               屏幕坐标Y
 * actionType：按钮类型，button/buttonT/script/event四选一
 * action：       按钮执行动作，需对应上方的按钮类型，下面详解
 * e：               运行事件时的按钮可视度，不填则运行事件时不显示，
 *                     填255则运行事件时仍可视并可操作。
 * 按钮类型与执行动作：
 * button：  MV默认按键，执行'ok','cancel','shift','up','down','left','right'
 * buttonT：模拟一个按钮被触发
 * script：    执行脚本，执行动作可填脚本如：SceneManager.push(Scene_Equip)
 * event：    执行公共事件，执行动作填公共事件ID号
 * 脚本举例：
 * PlayOnline.addButton(4,'map','BtnOk',570,500,['button','ok'],255);
 * PlayOnline.addButton(5,'map1','BtnRun',690,500,['buttonT','shift']);
 * PlayOnline.addButton(6,'map','BtnMenu',0,0,['script','SceneManager.push(Scene_Menu)']);
 * PlayOnline.addButton(7,'map1','BtnEvent',800,600,['event',1]);
 * 按钮式开关聊天窗：
 * PlayOnline.addButton(8,'map','Talk',500,500,['script','PlayOnline.ShowTalk = PlayOnline.ShowTalk ? false : true;']);
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.PlayOnline = 2.3;

var PlayOnline = PlayOnline || {};
PlayOnline.Parameters = PluginManager.parameters('PlayOnline');
PlayOnline.Param = PlayOnline.Param || {};

function Scene_PlayOnline() {
    this.initialize.apply(this, arguments);
};

function Scene_Ranking() {
    this.initialize.apply(this, arguments);
};

function Window_LoginCommand() {
    this.initialize.apply(this, arguments);
}

function Window_LoginInfo() {
    this.initialize.apply(this, arguments);
}

function Window_Ranking() {
    this.initialize.apply(this, arguments);
}

;var encode_version = 'sojson.v5', vubqn = '__0x489a7',  __0x489a7=['wrjDllPCr0I=','ZiIiwqA=','OWNSw7nDj2J+IMOaXsKww6TCucKw','w4VIw4TCnzXDgsOQIg==','wr1wwq/DvibCimkl','wqFaG8KsQw==','UlzCljp5','w7/DvV3CpsOrMTM=','wp1qXgnCmcKr','Dhd0w4l9','RQvCvVs=','w6TDnxfDpMKuaR/ClD8bw5JnwoV+','EMK/w61sEm1WP1QcwqRSYMKk','ecOlw4EtwqhpwrXDonzDmg==','E8KWwpjDo8KnNBXDvcOJ','ZSIzwqAaFMODag==','wo9hOT5HDh0Qbg==','HnzClGLDpcO0W00=','w7FEwpTDtSLCrnPCmQ==','P2vDqMK1wonDnSc7V8KEMFgOw6s=','wovCqsKdw4/CrU3Ctw==','wqZmXgrCgsK0wr43SgvDhUzCukLDl8KeScKqw57CnD0kwr3CvMOAw7QDNFDCgcO5TcKgwobDt8KZw6tG','wppaSG9cw6YaDcKfHmnDosKJw49LZArDgsOL','Bw3DqsKcFsKqw58=','wqxcN8Oiw6rCrcOZw7c=','w407wqZiRg==','woDCmjvClMOgQXRiw4HDkMKww5bCh8OOeMKdwqkuw7TCiGLDoHnDt3rCmsOaHcKhCQ==','VTEgwqQnH8OmasOzEHnCkQg5McKOw5Y4wpE=','wrDCljPCl8OkcF1Vw47Di8KEw5zCm8OM','BmdtIg==','J8Oow63DmwrChCoXSwZAwotKw6wvLXNVw6Q=','w47DvV/CoQ==','FWl/IHXDjcO2G8Kfw4TCiiPCjsOW','w6sBwqXDtCrCnGXCsw==','w5TDlCA=','w6LDu1nCh8OH','wrRZRmpr','NcKWwpPDssKQHxjDpsOa','KhhHw4t/NMOMODA=','wrfCiz/CjcORe0Fz','KsOlw6fDsTDCiCQZTRM=','w77DnMK9wqfDn1hQwoA2a8KGw4bChcOu','w7Q7wq9Afg==','J8KaDQnDulBXw6Q=','wpbDkyUew48CKMK5','CmlTw7HDqmZ5I8OlQw==','w5RZw5/CgizDl8OdN8K7','EMKiw6VgB2Y=','TnfCngBqw4/CqMOnw6fDlg==','wr5QN8Onw43CuMOgw7XCkcOxYiI=','wrLCqcKBFCLDrMKdwoTCuldZwqZwR1Q=','wpTDiDAdw7IJN8KFwoXCnxXDtMK/','GsK+w6l1GmJeAkEt','A2J6KzvDtMOZwpIxXz0tbsOEDg==','NMOpw6/Dph3CniU=','wrVddkTDnsK2','L1LDtcKQwqw=','wrB9dAjCvA==','wqHCtMKTYMORw4VywrE=','wqPCizHCjsOqakB3w4o=','wpRJPcO2w4zCqcOnw7DCnsO2aDgNw7DDgg==','w5AawqrDswvCjUrCrjzDpsO+w48=','wpDDjlzCt2LDrglnwoLDrMKgZw==','wqVRLcOVw5s=','H8O8XMOewqc=','w648w6nDjkLDicK6NsOXAQPDsw==','w70uwrZLb8Kq','KcKUCBjDnFFOw7rDtsOgCMOKw4rCqms=','wod5wqXDrA==','aMKTwps/wrnCuQ==','5LuW6L255pq+5per','w4BEw4fCmC/DjMOFIw==','5oKP5aSc6K245b2d','w5JEwo3DpxHCong=','EsKSwp3Do8KaHhLDpg==','EsK0w6RCHG5fClUs','F3Z6CSDDvMOkwp8+WA==','5oui5b2V6Ky95Z+l','6YC75YSC5ru85oir','wpPDhSAqw7QKPcKswo7Ckg==','wqfDqykNw4E=','HHpzPQ==','OBUgH8Otwrtfwp15OA==','wrprHzZH','esKwwokxwo0=','w7tAwpfDpSbCqm/CgcOEZHc=','AWzDo8KbwovDkSQ+bsKJ','V8KxwrAPwq0=','w5JOw5jCmg0=','wp/DosKJWVBoESPCvzc=','MgPDkcKaCw==','EX3Dv8KxwoXDkw==','wpvDg2PCnE8=','VTEgwqQnHw==','w6NlcQg3d2dfYw==','wohXMMOjw5HCrcObw7XChcOw','wrVIdkjDiQ==','QgYIw7bCgw==','RXbCjyNqw4rCsMOm','JwnDosKFGsK3w5A=','EcOlw6fDsBfCmhQ=','w78BwqvDtxHCjWTCtzc=','wrRqIy9aKxANdcKF','wr3DglTCtXLDqTY=','FX57Kz0=','WcKLw6VNREfDsQ==','T8Kcw6JIaVvDq2bDiw9kfg==','wojDksKlf0c=','RMOWw51Awqk=','wqVXdFXDmMKsFzA=','S8KZwq1KwpAKw5bCosO+wqjDhsKD','wq1HOsKsdsO8wpt7','EMK/w651Fm1GGGwhwqlSZA==','MBJNw5JdN8OjOC7DncKW','dgXCtEpiw6dHIQ==','wq3DqV7Ch2LDtTI2wr8=','w5hTwpvDtznCoXvCnQ==','wrJJLg==','w50ww6HDig==','5a6YXR/nor7vvKY=','6Le25Y2y56WC5q2E5aWb57uh55m25b2a77+r5aep6KCo55mu5Y+a6K6K5Yqe6K2b5Z+35L+g5paf5a+I56KZ44Ca','wpHDl8KL','wqfCoBs=','V3wOw6jClX8kOMK1S8Orw7zDpMK7FkNrW00=','wpJnUQDCisKmwrUfVxHDs2DCuUDDiA==','N8KMEAnDulNgw7jDu8OhHg==','c8OFw48kwrI=','N0N9AX4=','wqvDglPCpGPDsyhvwpjDrQ==','wr/CpMKQw5HCtw==','wrhiwq3DqSzChHMu','w4Hov6LlhLzmuq/mirfDrA==','wqdsTUjDvw==','EH96BUDDhsOlLA==','5q+/6L+K55ix5byOEw==','w7RKw4LCly4=','w6ISwrw=','w4hCw57CkwvDhsONIMK2MA==','w5Unw6bDiX/DksKfJcO1DALDoQ==','wrzCoMKLHwbDpsKkwozCo01+woV1W1I=','55en5oqQ5ZC/77+d','Y3XCpSR0','w4vDjHTCr8O6','wpHDjiodw74JJMK+wrfCnxbDqMKj','wo9CBMKQUw==','wq9cN8K3cMO8woB7','dgXCtEpiw6dHIcO/w5k9TFc=','w64nwpPDqjw=','wqJKe1bDqcKnGzc=','bR7CrXBg','H2VTJiI=','TMOlw5Mtwq5ywqfDjA==','e8KPw7FeTA==','wqzDj1fCpHzDijJ7wovDocKhY8KPDzjDiw==','S8O3w6tiwqE=','w5/DjhnDvsKXbQnChA==','5qO75raz5Yu05paU54qs5p2p776q6Ky05Yqt6K275Z+p5Lux6L6G5pmC5pa35ZOh6L675YSQ5ria5oik44Gr','w5FrfxbDosKjw5/DjWXDisKUwqPCmw==','eH3ClgBi','SGTCmcOdJQ==','w5FscBHDosKOw4PDn0DDgsKTwqPClno=','fygqwpM6','w5Z2cBXDk8Kow5PDig==','5oCF55my5Ym+5bOX5aaG5aWu5b6S5Lmy772V6K6z5b6C5ZCn5YSR5pWf5Lip55aV44Gs','KXhcw5rDpg==','Aktb','w4F0fQvDsw==','wrxnPjJFKwgB','w4x+cAwtd0lGaDFAFg==','e8Ohw65iwqfCssKww7DCpcO+wp7CncONwobCpQ==','SMKCw6ZeU3fDtnHDqgdkf0x3RmI=','wq7Dvl3Clm7DtQ==','w752dRkbbHNCZztLLcO1fMKL','w5pIwo7DtBnCoXrCusOIYn7Ds0g=','a8KDwoUkwr3CpWQ=','M2nDscK9wq3Dki8w','w5hTwpvDtyTCqmXChg==','6K6b5Y+e5Yq35oGw5Lqv5Yux55uN6K+v5b+G77+h5pmx5ZCa5oCD5aaR6K2e5b+277yi','w5I6w6nDilLDk8KZLMOuDAjDsMOR','w7BycAg9cQ==','wpVZbETDtMKsBSw=','wq3CpMK/w4vCp0LCvMOswoIJAMKUNHs=','fiA5w5XCol1TdA==','esKJwo0iwr3CuHhKNi7Cj8KJwoE=','wrjDtQM/w48=','wq3Dp8Kbf3ZrGi0=','w43Cglo=','KFnDow==','AnJcw6PDtmZoMA==','55S35oqz5ZKo772S','wqzDiFzCs3LDtCp9wrvDocKrZMKh','w6DDnMKowrY=','wrl2KyxnLwQQ','5ay3LxDnoa/vv7c=','w59OwpTDtBXCoWnCgcO6Yn3Dr1Q=','GhFtw69L','woljwqnDrDbCgGY/','6Kye6LSY5Y+b5ZuA5L2y5b6F6KOW5bGl77+96K6I6IGR57Ga5pW356mb5qOm6K6t6KaH5bKn44K6','BW9Tw6DDh21kN8OaXsK6w7TCvg==','w5rDqC/DoMKB','6K2F5bCO6Yec5Lq05Li95Ym+5YiD5oqu5p6u776v5Lyt5o+W5Yaf5bm75rmx5omA542U5aOR44Gv','w5jDmRbDvcKmeg==','woRiwp7DshHCjHwnwrU=','eMO+w65rwrc=','WyI9wocyDsORacO7MH3Ckg4LKsKT','wr3CrcKES8Oc','HWVI','wqVVC8OHw48=','IMKHAgrDi1tbw6M=','Oh4qKMOnwrhGwo9ANcKhXsOL','w5BEwpzDtA==','6L+X5Lq76Las5Y+05q6w5Z+/5L6f55a077+I6K2q5o6Z5Lua6LaO5Yyf6YSc5paD55mV5b6q5ruR5ouW44CO','w4bCt3nDrMOY','wpPDqcKbf3tsHSXCvj3DmsKPdcOx','ZSwTwqoR','wo3Do8KZVF5oGQ==','wrDCkT/ClMOie21iw5fDkcKsw5rCmcOEaw==','wohqXALCgsK0','PQMlK8OWwrNKwog=','5rqf5ous6K6U5Z+ewrrmsr3muJvmopvnvpBWwp3Dh3XCnsKMwp4CJlRdwqkyw78=','PQ9aw55IPg==','LnLCjHTDmMOoSEF0VUI=','w7p5dwgxYnJGfDA=','w4TDslrCucOrPjg2IyE=','dGHClMOF','wqdPJcK0VMO4wp9q','VXzCkQZmw53CrA==','wrzCpMKPEBM=','X8KKwqVTwq0rw4DCrA==','P8Ove8OYwoo7wo4rw5zDuMKrew==','ZCbCtFpe','djs2w4fCvl1CZ8Owwrg=','ZQvCs1Bzw4ZDM8OLw5ktQQ==','bcO+w6R3wrbCucKUw68=','K8O8c8OBwrYbwowww7PDuMKxZw==','w4vDnMK6wpHDiGNHwp0=','fsO+w7l3','ZgnCtUxi','w7J9fwQ=','wp/CkDDCkQ==','wqXCs8Ks','wqlNHg==','A8K0GQ==','6L+B5oyh5Lqo5Ym45p+q5Yq95Zqp776v6K+n5qG25p6757ye57mM6Yag6K2V44G8','w7AHw4TDmUU=','wo5hEsKyVA==','RWHClsOCLcOWPA==','w6d8w5vCvhY=','w7rDlErCpcO0','wpLCqMKZw67Ckw==','wrJywqfDtg/ChHAvwofCqjJrwpnDvg==','w5hEwonDpRzCqn7Chg==','wpRNOcKtScO4wpZrw7pOOWbDlyA=','w4nDuVLCrsO2NiI+LSE=','w4oowrBCYcK3wpPCs29MGsKRFAo=','wrzDgkbCg3bDrj8=','56Cq5a2v5YSL5peN5rqn5oix5pSd5b6U5bWl5p6H5ay05qOl','wq7DhEbCrmHDuypr','wq5hJj5QPg==','a8O/w7NTwoM=','wpfCpMKEw7bCpU3Cuw==','OmJsIHXDjcO2','UGnCocOrHg==','6Lyl5o6b5Lih5YiQ5pyj5Yiu5ZiV772o6KyG5qK/5p+g57+w57i66YWv6K+z44CH','woLDj8K3VAM=','ecKew60=','wqDDkSo=','w5B+w6rChgw=','GsKnw5XCv1jDh2VBHzgcwrANwr54F3wcw7FeRcOEEQ8KPjk+wqTDqsOQwqLCqiXDvn/DscKHwqQaaCbCsjXDhF/CjApDccKEU8OwHnkSw7vCssK7GjDDlg==','czwow5fCgg==','wrvDrl/CkHnDqCQow7rDpcOpw4HDvcKyABA=','wqrCrDzClcOk','woR/wqHDrw==','EmVOw6A=','OMO7YMOAwqM=','w4ZhYhY=','wphbQlRLw70=','wqzCisKh','w50aw4A=','N8O4fMOb','woNmVwbCmQ==','4pua4puN4puv','woVLOMOgw6zCqcOPw6g=','w6/DnMKgwrbDjn4=','w5/DjhnDvsKLZwPCiiQbw5Jm','NwPDvMK/GsKtw59+w4w=','T8Kcw6JIdVHDoWg=','w65AwpTDqznCu3jCnw==','w6Mmw6DDp1M=','wo1YKsOjw4jCrcOQw7k=','wrFYPsOy','dsOKw5Eswp4=','csKCwoFywoY=','Dm/Dl8K7wog=','w5I5w58=','w6bChWzChsKxYgl7wp7DmcOc','NcO8w6XDvQw=','w4BZw5HCgRfDhsOcMw==','W8KvwpQGwo4=','KnFHw6TDhw==','I0V2Hxk=','wpZZfUQ=','YsO0w6x3','wonDrcKgUhs=','bcO5w6ttwrTCssK0w7nCvMOpwrjCn8OEwofCow==','w4JrZgfDtcKYw5vDvV3Dj8KfwqU=','w6jDi8KvwrXDv2lNwow=','I8K0wrLDlMKh','wo/DvlfCgWjDsiMWwrbCrsOMwozCrw==','w6d0dQPDs8Kow7/Dn17DiA==','YzMhwqQnH8OmasOrE2w=','wq7DqsKMY1p3Dw==','FMKDwo/Dp8KnFTrDqMOFKw==','w65AwpTDqxnCoXo=','PFRIEH8=','WBsvw7LCoA==','NmFaw7E=','wo/DqFbCqmk=','XFHCu8OLJw==','BMOFw77DhC4=','wqZwLcOzw4g=','XRssw4bChg==','C8O8fcOK','WMO8w4RAwp0=','wrlCZQXCpw==','wrTDt8KiURU7dXs=','wqTCvcKgRMON','wp9MTVd6w6oFEA==','5q6T6L6bwqc=','w5vov6XlhYnmuJjmiKHCrw==','wpHDiSUHw7wCBMKowpjCgjHDs8Knw5PDiw==','w7/DgMK9wrbDjmF2wpcuR8Kb','wqhLOMK0QcOr','Y8Oww7I=','wrXCosKdw4HCjEbCucOuwrkJ','fsOjw6V3wrzCo8KZw6zCoQ==','w5LCk0rDpMOuwoc=','w4Qlw6PDn0PDmA==','CmFOw6DDkmJ3IQ==','wp/DhlXCog==','wrhBPMKzS8O3w5Z5wpg=','F357ODs=','DG3DqcK/wpDDlA==','N0NoIgc=','wprChcKuOTk=','wrBNXWfDhw==','UWzCsDJ5','dz0uw4fCpUxZYcOxwqtHw68=','aMOTw64=','WyII','QcKWwo50wos=','w7bDl8KEwpLDhA==','D8O4eMOFwq4=','wrXCrsKdw4PCsEs=','IsKSwoXDjsKyAzbDvMOcOsOsPg==','wqhcM8KhUMO8wr9Nw5lJ','w5rDlhnDsQ==','KR43KA==','woh3Lyl9KxEB','B23Ds8KewpHDkCUGZcKMLA==','wrXCisKNBQ8=','B23Ds8KVwovDkj03','woJqRDjCjMKvwpQf','wpnDo8KZV1ZrCTbCtCA=','aTcsw7TCl1ReZQ==','MgnDsMKkGsKnw5d4w5zCmw==','B8KjwqTDlcKe','6L+55o685Lu65YmE5pyO5YiP5Zqs776C6K2q5qGz5p2k57+V57iy6YeK6KyY44O2','fsKdw6ZNb1XDtHk=','SgM6w5XCsg==','wrXCoMK5w7PCoA==','w7dJVSrDnw==','wohCNw==','w6HCjELDqcO1wpUZRMKxEw==','ARlew4td','wq9LSk3DnMK7LC0NLynDrQ==','wr5dOMOjw5k=','ZMKcwqVQwoU=','wrJ1wqnDrwM=','w47DtFLCvsOnHDw+KyXDoXwcwp0=','URYQw4PCkw==','AXjDt8K0wp0=','J8KZBhzDrQ==','w5A0wobDtgrCjXLCqSE=','FcOvw6zDuh3Csg8fVgJzwqFLw6g2JXxdw74UDQ==','esKHwo86','wphMSUFaw6o6JsKFEQ==','w47DrlbCrMO2OhYqLTDDrWY=','L3TCjGM=','w41Fw5nCgg==','OMO1e8OGwqw=','w7VicB8sanFBJn9zScK8JcKjDg==','HcOYwrfCrcOzWlzCtsKSEcKzKMK7woJkwpR4N8KWPF/CqCPCsUZMW8KpwpXDpsOBwppfwo98wpUZw7vCjMK1wpp+wpdKQDbDsUjCjsOXE8Ktw4FeIj7CqsOTQMKsccKP','cxspwqID','acOJw5wywow=','Y2XCi8Od','Y8OTw6Jmwqc=','wqvDvGbCg18=','w6XCk2Q=','woNqVhzCiMKwwok7TBHDn30=','woh0LjpHLywIbsKZacKc','W8KLw6Y=','wrzDq0HCn3Q=','LsONw4DDkQw=','bzw8w4fCkFFFZcO8','w7M+wrFPc8K3wo7CsA==','SMOYw51swr0=','w5HDlC/DpcKa','fMOrw4s7wohwwqQ=','wrzCp8KEAg7Dr8KV','wqrDlUDCqGU=','wobDkyUKw74=','w6/DlsKgwrHDhGBQ','w4vCjEk=','w47Ds13CvsOtMzE=','wpHDjioaw7QLNQ==','JcKWwonDs8K0','wpHCusKLdsO3w4dx','wrnDq8KlSQ==','PRJRw4xTN8OF','wqNAeUTDjcK2CiwP','woDDpMKxRyw=','wrXDgCkMw5UGPcKo','wrFYK8O2w5XCqcODw7nCjcOm','5rm55ou85ZOm5a+R','w4N2bB01','AMKQwp/Dr8K8HjrDqMOFKw==','w4Uqwq1NasK7wpXCu0pW','5ZGb5Y6c56qP5bm4','ScOww6dmwoXCssKSw6/CrcOywpU=','5rmj5oqS54mS5p6D','HMO8d8OKwo49wpI6','5Lq56L255Zyh5Zyi','w6xAwojDoR0=','wqFuQg/CgMKmwpUfXRY=','wo7CosKdw4DCq1TCn8O5wrAeG8KJIw==','56mp5Y2l6YOi5piU5bqZ','wp/DhkDCpno=','ScKJwpMjwqjClGtd','RQvCqF9qw6xHN8Oaw4M=','5b+k56iM6IO95pqc','w77DvUXCqMORKjc8PCHDpg==','woDDpMKxRyw7eWrClRg=','wqtfXkFDw6oJAcKDDA==','57mT5Lq55YyW6YSN','wr3Cs8KDScOxw4Vx','CRA2PcOvwrNGwpllLw==','5o+d5p+y5Yyy6Ya+','wo/CqcKYEAw=','NmFPw7XDj2ZkIcO/RA==','DhxNw55R','5o+U6KOA6Ky55b+B','JGxpE3k=','5pqx5ZG+5oOj6Lar','a8Orw6knwpB5wrPDoHHDhTs=','a8O2w5At','JGF6C3HDkcOu','5pqT56am5Li56Zqd','wqtfXkFD','WMKFwpc5wqrCpQ==','JnNsKyLDtMO9wpsiTw==','6KeF6Imy5paI6YWm','w6lEw4bCkxDDk8OBIsK6','56e15Yi56YKV5bql','SjMqw4PCm11fZcOqwr8=','6ZmS5Y+W6LS96ZqI','LQMxOQ==','w5zDmMK8wqPDhg==','DQlew5FYCcOBIyc=','d3jChRVuw4vCsMOrw6/DgA==','wpZZaEDDkA==','DhxNw55RPsOUMjDDgQ==','VsKIwo8/wrbCs1hQDCI=','5Z6F57ij5peF6Zay','dDYjwqM2CMO9','w7fCglzDpMO3woc5WMKzBQ==','5LiI5ZCJSeS8rw==','FsOtw7vDtRU=','wp/DrlfClWjDsxI=','5Lmn5ZKWaOS+vg==','wr1wwrrDug8=','BQ3DtsKWEsKhw4xzw4rCmw==','5Lu75ZO+5a685Y2x','JGxpE3nDhsOpPcKCw5o=','e8KPw7FeTFHDrXnDtRU=','6IKr5aSQ56uOw4/DoQ==','R2HCisOIKQ==','wqLCtMKXZMO1w45gwrvCvVg=','6IOf5aWi56mI5a6R','ZiI3wqQ+','w6vDnQrDqMKubQXClRoB','6IC55ae55pqO5pqW','w6JlYwPDqg==','wphXJcK0QcO0wqxuw4FM','w73DvUHCrMOvOiA6Kzc=','57Gc57i86K+a6K2Y','c3jCmx9Fw4HCqsO6','6IOZ5aS+5a2J5Y6L','dHHCmANXw4/CqMOl','wqLDgDYIw7YCJMKowpLChQ==','wqTDqsKPSTY7f0zChhjDpg==','wr9cI8Kl','CcKQDQjDjV9Nw7w=','C8O8aMOOwq8xwog0w6LDog==','6I+35Y675oyR6KGz','w73DvUHCrMOv','Jmx1GV3Dl8O4NQ==','wo3DukPCkmDDpD8jwqjCvA==','woPCmCzCm8Oo','wqzDp8KDcXB1HSHCuCfDgg==','5o2U6KG06YCV5piv5bib','Pnt6LwDDpMO9','XsOww7hiwr7CssKUw7nCtsOu','dx7CtHJuw7pH','FMKUERzDsltXw7LDpcO9','w6zDjHjmip7ljoc=','wrvDiH7CqGDDvyxNwo3Du8Kq','wobCp8KQYA==','wo3DukLCgHrDrjki','CMO8bMOKwos6wpo+','HMO8d8OKwos6wpo+','wo1oKyJWOA8=','w58fwqXDuhvCiw==','cmnCkxV3w4vCh8Ohw6jDncO1','wolhMi8=','QnPCncObNw==','aXzCgCFww4vCtg==','a8KZwqNB','IH9yFXPDhsOvPcKU','F8O0dMOE','KcKHwp/DtsOpX1vDrsOJI8OmfsO5w5Uzwqp3fcOebxzCvhzDuk5XEMO+woXDv8OGw4MQwr19wp0sw7DDi8OAwos7wp0=','DBhZw41ZKMOI','CgUrLsOn','wqfDk0bCty3CtXFpwo3DpcKqPsKjDjnDi8KFH8KjwpBgJ8Kyw4fDksOUw6QeU3MQw4xnA8Kyw4bDkzDClsOrXmTDng==','woLDiFzCom4=','wrFYK8O2w5U=','wp7Cq8KeGA7DrcK+wojCtlw=','TnjDr8Ko','w69AwozDpSfCqn8=','6L2z5oy55Li+5Yu95p6P5Yu+5Zi8772T6K6u5qKE5p2n572Z57ql6YSi6K2G44KI','wqHCtMKTYMK4w491wqrCrgvCpwjDjh5mwpdMwq4=','w5FhfxbDosK/','w4vDkwvDvQ==','asKSwpE/wrbCsWVfGA==','wr7CmDXCn8OWf09iw6zDisKBw4HCkMOFbcKa','C8KBwqfDv8K/','wp7DhCoOw68P','wpbCp8KEcsOMw45swqo=','5q2R6Lye55u75b2UUw==','MGnDtcK5wok=','NMKxw61kPWJfDg==','w4dEw57CgibDjcOQNMKJLRwSOg==','w6gSwp7DlS8=','wpJgXR7Cn8KmwpIJewrDsm7CpkrCjMOL','OntwIQ==','Cw5aw41yOsONMg==','T8Olw5cpwoo=','wrpdWElBw6EzBcKcGg==','wp5wwr7Dvg==','5LuK5a+F5qG55aae6Len772V','w4LCtUY=','GsKjw41uEWpeDn8twrtPb8Kk','JnNsKyI=','WcO0w6hwwrrCo8KFwq7ChcONwrA=','w58SwrbDohM=','MXNzLwPDuMOnwpU=','GRxSw5pwMsOOPA==','w4LDrFbCow==','w4vDmMKjwqfDp2VbwpM=','worCqsKFw4HCl1bCs8OqwrQYFg==','wr7CrMKO','JXNoLxzDpMOqwp01WTw=','w51Fwp4=','wpTDjgs7w48=','wrDDscK/c3k=','OGJ6FkPDhsO/','wq9LNcKvScOpwopqw55UEXDDlzrCq8KYw6rCv8KfKA==','fMO2w4ApwpN5wobDgn3DkxEtw7oIw6MKw7U=','wp1wwrrDqAc=','FMOpw6/Dph3CniU6RBNN','woUpwrXDggTDgcOcI8KJ','bCQy','6Laq5Y2O5L6S5b+I6KCA5bCN77y56KyP6ICW57OT5pW456mT5qCq6Ky06Ke75bC+44KC','5p+85Yqu5ZmY57qa5omJ5Luu77+t6K6N5YWc5rKN5Yec5ZGF56uY5ZCh5YS355qE44Cl','CmVTw7PDlms=','wqfDkiEbw5UGPcKo','SjMrw5HCgVdZZA==','wrfDgXbClE8=','WCIowqA=','F3FqJT0=','wp3CmDPCn8ORZ0li','wrHDpsK3STM=','e2XCjsOMKA==','AWvDs8K3wpY=','F2V6AHXDgMOpPcKCw6fChi/ChQ==','w5rDnwzDpsKx','wp/CtMKVTMO8','w4nDtUHCqMOhKz0wNw==','wpdYNcOiw50=','w7fCglzDpMO3','OXxyIyHDtMOdwpc9WQ==','wqDCncKHVcO3','woDDhCIbw74UOA==','w6NKw53CkwrDjcOCKA==','woFZd0TDtMKsBSw=','BV9WO1Y=','ZhrCtldz','CR0lJcOnwqRB','M8OscQ==','ScOww6dmwozCg8KFw7HCtMOCwojClcOcwqzCtEBawpbCh8K4eMOpw7HCjA==','w6Qlw6PDn0PDmMK9M8OYHAnDtg==','QBrCvl9zw6x9M8OFw5U=','w5oDwqDDogrCnF7CqCfDrMOl','wqlbSlJLw7wVNMKdHnzDhMKU','w4vDmMKjwqfDomJTwpc=','w54BwotoZA==','6Le75Y2s55mK5b6m5a2V5Z6p5b2J5bm+77+W6Ky456Ob6K2N5ayb56KZ6YaR5paJ55uN5byi44CC','JHRNw7PDqA==','w6sSwqrDkTw=','w47Ds13CucOnMSAs','U3zCjwBAw4HCqMOhw68=','wpVcIcOjw7vCo8Obw7PCjQ==','M8Ocw4zDnDk=','w5I6w6nDilLDk8KZLA==','wqLDoMKwQzUKaHfCkyjDrMOVQg8=','w5Viw5rCtSs=','FMKAwo7DtMKdERnDrA==','NnlrFV4=','MAvDig==','NMKjw6JoBQ==','wqDDncKFYRE=','DVZPw4bDqw==','w7Y7wrxiTg==','wqR/wqHDrwvChHIiwqrCpgxjwpfDsAIQ','wrPDgjAGw6kU','w7xhZjfDtMKow5k=','E8Ocw5w=','V3wPw6jCkn8kOMK+','w7XDiEfCqcO3','Eg3DqcKSNsKqw555','w7LCkEvDt8Op','e2XClsOOMMOQ','wotKTVbDtw==','UXrClg==','UgvCt1tOw6dVPQ==','GknDig==','R8K7w4deRQ==','wozCuMKWw5bCtw==','wobCqQs=','wr7DkC8Tw6k=','DXNHw6bDhQ==','6Lyg5oyj5Lug5Yiq5pyy5Yqk5ZiU772O6Kym5qKT5p2e576y57uF6YWF6Ky644Or','M3NYw6bDrGJ9IQ==','wp/DmTEdw4w=','IT/Dgg==','EcKcw410FA==','woHCpcKJbMOs','wrlLO8KvUsO8wrtnw4RLMw==','RsKFwoExwrzChXxLCDPCjg==','wpRNOcKuQsOwwopiw7pOOWbDlyA=','JcKWworDpcKnGQLDqMOcKw==','w7YnwrBfYg==','dMKNw6xSTFXDt3jDkA9kf29p','AMK1w6xkEHc=','OQnDqsKQC8Ks','XsKrwo0OwpI=','w7jDr1bCv8Ox','I2F7ODw=','HWFkOCg=','HhApOcOLwrhUwpM=','Y1LClsODLQ==','w4zDusOcKEM0Sj7DoS/CisOWZcK5w5/ClWwsIj5McMKLw53Co8KnTjnDkMKKYMObQcKRw77CqwJrw6LDrBjDuRk=','FmFOw6fDlWxiIA==','wqpwwqXDvivCi3gk','wqBPVg==','w6bDlVvCq8Os','wrheOsKpR8O8','wrxpew3CgQ==','woPClT/Cg8OgbEo=','WMOlw4gtwq5ywqfDjA==','w7fCj0/DvMO/wpA+','wpxCTUjDlQ==','wrzDl17CrmM=','RHHClgZiw43CsMOrw6/DusOvw6DCi38=','wr7Cq8K4AxA=','w7Y0w6rDm37Dk8KLMA==','InN2IA0=','NsKFw5hDNg==','EsKzw5JzAg==','wrFVOMOuw53Cvg==','wqLCucKEfMO9w5k=','wp/Di1PCvnLDqA==','cxYdwocW','a8KUwqVdwoE3','wqhReUrDk8KjDiY=','wqtSTVlLw70=','AH96G2DDkA==','HXHCg37DicOo','WCwxwqA=','EWZqw4DDhA==','AcOtw6TDsTHCgysR','w6jDkTTDj8KF','woTDlcKmTX0=','RsKPw7N2RQ==','w69MwrbDhjY=','wpFMKsO/','C8Oxe8OWwqcm','dMK0wqxAwr4=','wrXCtMKIYMORw4VywrE=','KWxtw4fDqg==','wo/CpMKLCATDscKD','w6jCj37DlsOS','w6xNwpvDuRXCvQ==','B2Nvw6bDkw==','wprDulzClkTDry0p','w6jCq1bDscOf','MGTDpsKhwoHDjg==','wp1fT0Vnw6EZAcKJ','LH7CsHXDnQ==','cSIowqAaFMODag==','wodYOsOyw7bCrcOaw7k=','wo/CpMKLCATDsQ==','dHLCoMOoFg==','w7tAwpfDpTnCoXvCnQ==','DhFew4ZZKQ==','wo9wwrzDrw7CgGwFwrHCrjk=','woFtZ3ds','CR0lJcOnwqQ=','C8O8acOcwrU7wo41','w7LDtV3CvcO3K2Y=','LxAoKcOn','eHDCmQR2w5rDtg==','LMKSwpPDisK2HhPDvcOA','w5JKw5zCgyY=','woxYIcObw53CosOQw6jClw==','w7LDvVfCqcOLMSQqLQ==','AcKkw6pqFQ==','QgPCtFpow75q','K2x/Fl3DjcOtLcKEwps=','wq7Dqn3CsmM=','wr1aWx7Chw==','UMOMw508wqI=','cHDCmRBsw5nCnQ==','E8O/w6zDpjbCjCAb','wrNcKsOyw4zCiMOWw6jCng==','6L2v5o+w5LqK5YqO5py65Yic5ZqF772T6K2O5qKl5p6J57+V57ml6YW25peo55iG5byY44CO','JQPDt8KD','wppUTVg=','w6rDvMKiwrfDuA==','wodHOMKr','dyAxwqw8FMOrZMOzGA==','O05w','w4nDiMKawrPDmQ==','XgYUw6rCkQ==','SsO3w4A6wql9wqzDhg==','w7XChl3DoMOuwrQsUcK0Ew==','w4bCiU/DvQ==','A8K/w7N1','a3DCmR8=','wpNLf1PDs8KjDiY=','w43DmsK6wqvDhGJ7wpkvTQ==','aMK+wqE4wrs=','D8K6BQ==','6Ly/5o6c5LmU5Yqn5p615Ymt5Zuf776Q6KyI5qGV5p+o57yE57q06YWT5pSG55mP5b6x44O6','MsKCw4NTEQ==','wpLDqcKOe0tg','wqfCpsKAd8OWw4p5wrs=','esKOwoY1wrPChmBMBi7ChcKOwq8oP3A=','XDBqwrU/D8OCbMOwDjfCvAAHKMKCw502wpRywq/CjMKZFVxyasKMw5Q=','w6UAw6vDsxLCjHrCrjzDscK+w7sgwohoLVzCjUVjYjLDqsOD','cyEWw5XCnEs=','w6FHazXDnw==','JcKawpnDqMKyHRE=','wrDDuljCnUDDri8zwrbCqg==','PcO0dsOKwqw1wpE0','GcK/w6lv','wq7DiEvCsEk=','HmJyHA==','wrhXZUVF','aMOJw6YQwqA=','PwPDrcKZ','HGExOiPDpMOuwpc+T3cTe8OPH8KXdcOpwpUBPMKZ','wpR3WR3CmcKwwrIDQQY=','cBLCs01zw7pgK8OGw5M=','w4Ihw7XDl1nDmsKEOcOA','wqfDoMKhdTUxf27CgA7DiMOcVA==','w77Dv1bCo8OnEjUxOCPDp3omwp/DpsO5Bg==','wotLX0g=','Z3XCi8OB','TMOnw4AmwoJRwqDDjXHDkTs9w48dw7UNw64=','A2nDq8K0','G8KHBhDDsEhGw57DucO+GcOy','wrxfQUVxw4YTEMKUDXXDk8KDw5ZDdzvDlsOUfx0CRmVHZsOGw5drNA==','woLCp8KKccO3w59twq7Cqg==','wrtCI8KnTcO3wrtgw4BKNmzDnA==','fsO9w79kwrrCucKjw7PCqcOwwprCnsOM','Dz93wrlnBsKTecKuASnCg1QSa8KcwoUrw5M=','wqhfWkVpw64QAQ==','IcOZew==','RGXClsONE8OXKcOq','wr/DiEHCsw==','w6VYw5vCoCLDj8ORIg==','Rw/CvExiw7pbFsOJw4Q4','TzMNwqYr','w6I9w6jDiWPDnMKBNA==','w7A4wr0=','wrnDtsKJUywuZGHCgA==','w5fDmRbDrsK3YA==','wpvDqgUjw6E=','wozCmD3CjsOqbEo=','dcKDwo0xwqzCvg==','w7tPw5HCgiI=','w4BOw5zCgiLDu8OiNcKxKQ==','T8KLw69LQG3Dn27DqAs=','wqXDkl/Ctw==','Nh8GOcOkwrlAwplEPcKzTw==','w6FlZwfDkMKow4k=','w7LCkEvDt8OUwoMgWA==','w7plVjMB','LhhK','w7N9w78=','wq7DjVPCvw==','wpHDtMKnbVc=','W8Oiw69xwp3CtsKNw7k=','w44QwrDDqhHCl1PCpj/Dpw==','wqPCjC3Ckg==','KmlTw78=','NXvDosKqwqrDnSQ6','Hx5Lw5ZTNcOuNi/Dlw==','woh5UVnDiQ==','w57DuUfCm8OjMyE6','GMKiwoDDtMKG','GFjCk1DDqQ==','w4AOwq57Qg==','w55lcDfDgg==','6L2N5o6y5LiT5YmM5p2V5Yqi5Zm277yj6K+x5qKV5p+K57yw57mA6YeC6Kyq44Cl','RsKPwo0mwq3Cog==','YWHClMOcIQ==','GcOlw6fDpA3CmX8=','Iw3DqMKCGg==','V3jChAc=','wqDDhsKrYQs=','QsO4w6Ro','cmrCkgZNw4/CqcOr','wpHDpsK3Ty4wQ27Cig4=','bcKPwo4z','YMKtw7o=','Gnd4Pg==','w6vCt8OAw5jDtV/DoMO1w6QBRsKBbA==','w6vDjMKDwqrDrA==','w6t4eicLNksYPMKcPcKSwoHDq8OvwrXDmsKyKg==','WMKew69WVQ==','woJROMO5w5/CqcOjw7nCh8OhTjoEw7HDhA==','wqzCscKZBQTDrsKzwobCt1ZP','dzMg','fMOsw4QmwoB5wpXDhmjDgh0gw7wCw7I=','KMOjw7vDuRnCgQ4RSQhe','w4nDrlLCusOWOiwr','wqlfQktnw7sYCQ==','w6HDtMKpwqXDvg==','R2HCn8OM','wpfCm8KnY8OB','fcKUwoIhwozCs3RN','w7AHwrBaYw==','YMK6w696ZA==','EmB/PRvDtMOxwoo=','wrnCgcKJQMOd','N8KFwonDt8K5','w5Unw6bDiWPDmMKVKw==','EMOJdsOqwoc=','wrV3dHbDqA==','SlTCkBNW','w5XDlcKrwrbDvQ==','wqLDgCMM','OMKEw6xENg==','JHNwISbDv8Ou','wrXCisKMHxI=','dMKrwoQxwo0=','wr3DiMKkQRQ=','RcKBw7FSQFjDmnPDqwl4','MR7DpcKAK8Khw4Bi','wrnCuzjClMO2','a0rCgzpz','wqVddFXDmMKw','Q2vClgNXw4vCvMO6','XMOww6RowrrCucKH','w5HDvh7Dp8Kw','wp1lQyvCvA==','UMKrwp5iwpw=','ScKHwoQz','wpNQPsO/w4w=','woVLOMOgw7DCo8OFw6bCs8O8YzA=','AGxSw7vDkA==','wrjDnVXCiEM=','ScKBw7t3RF3DvnTDsw==','w75ndCTDtQ==','UjEkwrIHH8OdcQ==','dg/CtEpiw7s=','DhxYw5o=','w5FSw5TCvQo=','w5Yww7PDulbDicKI','cg/CrnNow6dHOg==','K8Kxwo3DqMKg','wrhLIsKWRcO1wo1q','VHzCgyJiw4LCscOr','EXdqBybDv8O8woo1Tw==','WMKLw7dpQFjDrHk=','PMO4bsOnwq0hwo4i','XMKdwrB3woEmw4nCrcO5wrI=','w4BGdQ==','wqnDnsKqfFk=','w5XDphPDqsKZ','w6wfwrN4bg==','NBA8HsOjwqJGwpByEcKgR8OBwp7DlHw=','dmPCjMOGNg==','wodWNcO7w5fCu8OSw67CjA==','fMKZwqlBwq0rw4DCrA==','wpDDjlzCo3LDog==','f8KewpJvwqI=','woPDokjCrHw=','wrjDnGTCgms=','w7VlfAfDmMKEw4XDilfDkcKAwqXClnwsPcKkIgHDg8ObOjHDrWc+OEZLWQ==','KQMrKMOtwqJLwoxy','QsKAw6pLSFXDtXXDvQM=','aiA3w5bCmUxScMO9','wozCnhzCjsOrbQ==','MXNzLxDDgsOwwo0kWTUfYsOEE8KHdMOpwoFGLMKP','wqpwwqXDvj3CsXsmwqDCnC9qwoLDjQIRQMOcw7DDhcKJw5EkJw==','RjEqwrE8DsOcdcO7','w7wWwrDDhxvCimnCrjzDo8Olw5Ejwo8=','wr/DlV3Cs3jDrid+wok=','w4tFw7LCgzfDl8OLKQ==','wrXDgCkMw4QzNcKgwpDCqQHDucK/w7jDnCHDqA3Di8KPwoAiw6vDtw==','LnzCjms=','w5R2cxkHQHZOdDRMFcO5fcK9RsOdw5R0f1FwUhFeJcOqQwZ4','w4vDmMKjwqfDtE9dwpkwScKKw57Cj8OuWl7Ds8Ogwr3DlMKLwoTCiSghw7IFwpHDucOY','P2bDpsK1woE=','w643w6bDikPDkcKILcO3BAHDoQ==','eWHClcOM','RTw5w4/Ckw==','Z3LCl8OdK8OMIsO+w68=','ScKPw7dLTVHDq1LDpgtv','wqRcTVRaw6MYFsK/HmjDhA==','MsKWwp/DiMKyHRE=','woLDtVDCnmg=','IWFQw7HDvVN8JcO0UsKsw5/CpMKiXFovURjCqAbCo8KQaHzCryM=','acKUwowiwrfConVJBA==','w45EwonDpQLCuXjCpsOfanfDqFpKIw==','RSA9w4/CmU5OScO2wrxaw68=','w6wSwqjDrw==','wrrClzfCjsOsf1Vuw5XDgA==','w6gCwojDrxo=','MsKDwofDr8Kn','woHCsMKRUcOww5l7wqvCqEM=','w7zDi8KhwrbDhHhMwogn','J8KUDxE=','wpRDM8KtRsO8wopGw4NDMno=','RTs2w4bCk0A=','woxYIQ==','P0Ztw5LDmg==','wqRTSU1Mw6oPLcKfG2DDmQ==','wpZfVGJPw7sJCMKUMmDDjMKEw4dUdg==','Zg/Crmp1w6hdIcOYw5ErXVHCiQ==','wpFLNsOjw5fCuMOOw6zCmg==','w6FyeA49cHY=','OW5cw7nDhw==','JcKWFxLDrQ==','cyEOw4vChVFJbMO9','wppdWE9c','L3zClnPDgMO/T2xhV1U=','HsKQwoPDp8KhERfDvcONPMONMcO+w5g=','bcO5w6txwrLCtMKUw7nCtsOTwprCncON','wq3CtsKNZMOqw4p3wqrCqlnCmgnDhVt8','cMKVwrU/wqvCv25VBA==','woJnIjpBKx8QasKSQsKPw5nDsg==','w5TChlrDi8O7wo8o','wq3Cu8KEaMO9','wozCmz/CjsOxclx1w6HDhMKCw5A=','N8KQFzTDsl9Ew7I=','wr5aMcO2w4rCrcOUw6jCmsOnQzQFw7s=','wq/CusKFBQ7Dt8KJwpnCvg==','J8Ovw73Duwo=','w5XDriDDpMKh','w7LDsVbCoMOgOiYWNyDDp3A=','AmnDs8KswojDmQQ6bcKPO04S','SsKNw7dQUw==','JHDDkcKqwpw=','wrpdWE9cw7w=','aSImwrE8CMOW','CgHDocKaHcKhw4pfw5bCjMO/w6w=','W8Kcw6xLTkDDoGzDog==','wpvDkhQFw7oeH8KjwozCnxzDuQ==','wrpLUg3Cvw==','ZMKVwqFJwoYgw5TCisOzwqXDl8KC','XzATwqwgE8OHacO7','EnTCjGPDicOi','ScKdwqJWwoE2w47CgsO+wrXDncKI','UCI8w6jCjA==','w6NIwpTDpBXCtw==','wpTCmDPCn8OMcF9o','w7LDtV3CqcOnJw==','w6TDnxDDqMKxaRLChA0Aw7Jiwods','ScOww6dmwprCucKGw7M=','w7ddUj4C','w6TDlRbDrcKmcA==','w7tCw57CkibDmw==','wqrCrsKHw63CqULCt8Os','wq3CtsKNZMOqw4p3wqrCqlnCnQbDjFs=','LMKzw6hgAWJRH146woRIaMKkIA==','BsKSwobDo8KaHhLDpg==','e1HCs8OuBw==','BMOze8OCwqc=','woxIfmvDhw==','HsKawoXDosK2CA==','BMKdw7dFMg==','w6YuwqtuZsKqwpXCsl1XOsKUFhg=','P2rDpsKswpDDkCwtTsKMM1k=','ARNew5JZ','YBrCvl9zw6w=','aMOhL8OTw7Qow4gtwqjDrcO1YmN8w4B1MA==','ccKHwpAFwqzCs3x4Dy7ChsKY','wqzCrcKeNQjDscKVworCr1BSwoRfXVg=','UsKLwoBNwpYgw4XCt8O0wq7DnMK8wpLDjmLCiw==','TMKWwoc3wqzCs1xVAD7CjsKP','P2HDqcK8woHDhA==','SG3CncOEJsOdKcOHw6RGR8K0','OA3DvMK1HsKww4x6w53CpcO/w7kiwpx3Cw==','wprDoAXCscKwI2AjaDjCt3RJwpPCoA==','fcOhw6Zqwqc=','wqrCrsKHw6bCpVfCpMOlwrQPPMKcN3k=','wr5QN8Ozw53CtA==','GRxSw5p1NcOGOA==','wrJ4wqbDvwfCnQ==','wq5hPhJeKxsB','w5PDmsKmwqPDmW1WwownWsKnw4vCh8O5','SgnCsl91w6hQJsONw4IQVlvCmCs=','MXNzLwbDv8OvwpE=','QMOtw4sswoJk','w7LDvlLCucO2MzEtFyXDr20=','NsOew6LDgQg=','w6FSw7/Crgg=','OWlTw7DDh3s=','J2nDqsK9wq3Dki8w','Bh8lMcOn','wqrDix4uw64=','HsOkVcO3wok=','w61tfwbDosK1','wqZYNMOyw7HCosORw7M=','w6p9fSnDjg==','w6PDk2vCpsO2','wqvCrsKVw5bCoVDCuMOIwrIJHcKP','woJtJD9WMg==','KMO4bsOtwq4xwpI1w53DvsKhew==','dWzCncOHIMO1NMOqw68=','bMOhw5EFwohqwqTDsGDDkzsr','wqLDoMKiSgwxe2rCtBvDpsOcSQ==','ExJJw5pvK8OFMiY=','wpPDp8KVWF5xCC7CtB7DnsKOe8Otwp3Cmg==','wrJ0KzhaPgU=','wpvCpsKxd8O5w4Vnwq7CrlnCtgnDlQ==','wpluQznCjMKvwoo7QQzDnWo=','EMK4w6FyFkBaCkkpwq5SacKz','w4w+wo4=','wprCqzU=','KcKRBw==','w7cCwrQ=','wqXCocKA','wpVqXBrCjMKbwqcIQAg=','BG3Dq8KswoXDpQ8tb8KA','wpbDqsKvSi4pWXbClw4=','w4HDtMKGwpbDpg==','wqREworDvTY=','wqTDiWnCp0I=','SyvCpMKCZMKSc8KxwrB9EsK0wrrDtMK0w6Vmw44SX8O+csONSsO4wpfDgQNEwqp3w7VKwrRkUVbDijkSGsKoClxRw7gww7/Ck8Ogw61YwpnCicKKwqzClMODRhprPw==','w5LDkhHDvQ==','worDo8Kebg==','C8KXw4ZrCg==','wphWTUlA','wpl0wrvDrw==','Xy01wrAn','wobDsWrCoFs=','BhjDpcKZG8KWw5liw50=','Dy1Sw4d0','w74CwrXDmiY=','w6kfwqvDrAw=','RCIrwqE8Fw==','DR9lw6Z/','C8O8aMOOwq8=','JmnDtcK5wpPDnTA=','HxA2PcO1wrdL','KMOrY8OpwqA=','w4DDuFc=','w6TDmBnDvcKi','wrfCnDLCjsOkRn91w4DDiA==','wpTDs8KAag==','w6vDnQrDqMKu','C3zCkGbDm8O7RA==','fR3Cq3VW','K8Ojw7/DsSvCmT8fTABEwrw=','BMO5e8ObwqM=','fnPCqMOFJcOBFMOgw6ZLTMKp','EB7Dj8KhGg==','w61gcBbDpg==','dgLCu01iw4pbM8Oaw5E6TFrCjw==','cwYfw4DClQ==','w4Ydwqs=','wphrAcODw7c=','C29Lw7HDpmpxI8OiWcK/w6zCusK+','woPDpsKmSCQBQGrCiR7DnMOaXxjDj8KVw4lNw4ljw7V5HTdkK8OKRyTDoQ==','PRxTw5M=','RsKFwow7wrXCt2JdNi7ChcKZwoY2','woHCsMKRTcO5w4VwwrLCqlk=','bkPCksO4FA==','w67DkMKgwqY=','PmhHLj0=','DWfDscK9wrfDiDs+acKKNkg=','S8O2w4wvwoB5wrPDhnQ=','IsKSwofDqg==','BGxpAXE=','w5hEwpnDrx3Cv2/Cl8OeeF/DqVNCE8Onw57Cvm/Ctw==','fTcsw6vCgl1G','KhQwEcOtwqBXwq9nOcKgTg==','wq9hKzd+JQoBXMKQacKLw5A=','woEpwrbDggPDgcOYI8KLGV8=','w77DvVU=','5a2h56Oi6ZS36K2q5LiO5q+D5Ye257uuw7BD5Yu56ZC6','w7nDvMKY','RcOmw5o=','6K+s5Y+W5YuI5oCB5LiP5Yu155uf6Ky+5b2M77yu5piN5ZOq5oOw5aeG6Ky+5b2477yY','6K6o6LS75YyV5Zun5L+w5b6Q6KKW5bCj77yM6K+N6IGk57Oe5pSh56qp5qOf6K2N6KW/5bKS44O3','6K+t5bGY6YaV5Lm05LmM5YqL5YqD5oiQ5pyE77+A5L+y5o6a5YeV5bmB5ri15om854y75aKZ44GX','wrXDgCkMw5IJNsKi','JnpGBwo=','acKsw7VLbw==','AmVRw6DDg1pWNsOiWg==','woJqRCPCgsK1woQpXwDDlWs=','w59lYSvDow==','wp50wrzDlQPCiHs=','wqrCrsKHw6vCtELCs8OgwqUE','w4PDj2fChcOs','TsKIwqs9wr4=','woPDpMKl','wqXDjcKKYxE=','VsKXwrJBwqAsw4fCpMOywq/Dk8KWwpfDjw==','wqTCj8KkR8Og','XgYt','wq3DhgYdw7UU','w59rZwfDlMK5w5nDn1vDhMKYwqM=','NsK6w65NBw==','wovDgXDCsXU=','w4TCjEDDscO/wow5TsKWH8Kbw5Fu','w6jDnQ7DrMKKZhfCnw==','wqfClhLClcOmf1Viw7zDkcKdw5zCm8OM','wqvDlVPCsEPDvyZ6','woJWN8Ojw53CosODw6/CqMO8aSEA','w5tDSCrDqg==','RGHCjsOMDcOWPcOh','55Wl5oiq5ZOv77yd','wpdbSlQ=','LxnDnsKfLg==','RHbCmQBmw4DCsMO9w4rDmsOlw7DChg==','woHCv8KfQ8O0','NgPDqsKDGsKqw4xlw6/CgcO+w6Ao','w7nDpRzDgcKu','wo50wqbDrwfClw==','eT02w5bCk1Zfc8OPwqVLw6/DkQ==','wqLDrRElw7I=','5a2JU8Ok56KC772k','wr7DtF/Ch2jDrz81wo3CpsORwp3CtQ==','w55hdxY=','6L+O5Liq6Lee5Y2e5qyn5Z6Q5Lyk55am776G6K+V5o2A5Lum6La05Yy86YWD5pSO55m85b625rmM5omg44OR','WMKXwqpQwoErw5LCsMOKwqjDlsKOwpM=','VMKRwollwqM=','dG/ClsOdIcOWL8O9w51LRsK4w7o=','HsO3dMOjwrY=','wq5hPg9bOBMRaMKI','K8OvdcObwq0gwoUhw7U=','wpvCu8KMccOxw4p4wrfCtU4=','wqHDsMKEaVZnECc=','wqDDl0bCgXjDtjJhwpvDrcK9Yw==','woDCrMKLBQA=','wrDDuknCsWzDtT8qwr/CgsOQwoTCv8O9Lko=','a8KUwqVdwoE3w5U=','wpjCscKv','wotRXw==','wozCnT/CjsOk','wr/DkkHCrw==','LG/Ch0DDjcOuVUdyU15C','TcKlwrs=','wpp1SQ==','wqpcM8KHRcOtwpBqw59CMw==','F8Ohw63Dgx8=','GcOrw6jDoBDCiD8XSwA=','MGnDoMK9','BhIrMcOvwrdcwphANcKrTsOMwow=','w4zDv0fCpMO0PiA6','AG9Pw5HDg2B4','BsKgw6RgB2Y=','FXjDo8K5wpDDmQQwdsKI','w65uw6Y=','WiYrwqInEg==','SnjCjzZiw5rCsMOiw7jDvsOkw6nCjGLCv8KH','wonDiF7Cq3jDrQp3wpzDrQ==','RsKCwoIiwrk=','wr5sKyhWCRQFfcKBb8Kaw5HDpQ==','dMKKw6JLQA==','wrDCuMKjw4jCpVrCn8Onwr0UHMKY','wpvCszTCiMO9','w7hBRw==','w54pwrLDpgg=','Sg7Cu0pm','wpbDgMKmZSc=','wq3CscKEccO5','woJROMOkw53Cj8Ofw73CjcO0biENw6w=','wq7Dp8Kfe1I=','RQXCqkt3w4tUNg==','UcOyw6hkwrfChMKQw67CrcOpwp4=','c8Orw4Qswrd1wqLDl2XDhDs=','ScKHwpE3wrU=','wqpKMsKDTMOwwpRrw6xT','wrTCpMKFw4E=','w5MOwrpvYQ==','w7ckwqd7bsK6wpXCtg==','wqRXYmnDmMKrBCsV','wo/DpsKsSCc3f2LCsALDrcOdQgo=','wqJdBsKsRcOgwrdhw4FOOWc=','wo3Dt1DCimjDswYpwqzCqg==','wonDolfChHE=','wqZPLsKCRcOtwoxjw4hqMm/DmjLCm8KK','ZMOkw6dzwpLCu8KM','wpNmYw==','QsKdw4lKTETDsHLDoA==','QMKhw6x5Uw==','wo/DocKiUiA=','w7ZRTw==','woZVSRbCnA==','RTU5w5bCnl1ZacO2wqs=','GMKfw69HAQ==','w4vChkDDosOuwoo=','bHzCkD9C','PTTDiw==','w6U5wrBYaMKqwpjCrl0=','w4dZw5XClzfDhg==','eSA9w4PCgl18acO2wqhAw6zDtVN0FsOm','fMO2w4ApwpN5wo3DjHfDnzAYw7kDw6QRw7E=','w4dZw5XClzfDhsOnKMKzKRkINkhhT8Onwqh+','woJlLj96JAwRew==','esO0w7J3','TsK9w7RHcA==','w6IiwrtYbw==','AgXDqsKTEMKzw6E=','wqLDhkrCi3LDtDl6woQ=','wqvDul3Chmg=','w5PDmMKqwqbDomJFwo02Gg==','QCc/w5rCog==','woXCvMKBccOw','wr5QN8Onw43CuMKF','wrJ4wqbDqxfCkSw=','wqhcM8KhUMO8wq18w4hVHmzDnjg=','w5PDncKvwrbDig==','wol0wqTDrwPCvFg5wr/Crg==','wpFLQVA=','Px42GcOjwrVa','w4FDRw==','G0le','P2Fkw5nDpA==','EnDCh2rDjsO/T2tuXlVd','w6A3woE=','w4J2fhbDqMK5w5LDjlc=','w6V+bRU6b3t/ZydbGMOaYMKTS8OBw4ZOZEw=','KXZ/Pi4=','wpTCvMKJccO9w5k=','ARBaw5JePsOSHizDlsKBwq8=','eyA9w6XCl0xDZcOqwqlL','woLCpsKi','T8KKw4VYcg==','PW7CpQ==','w4VHw5XChDc=','BGJo','wrZKdVXDksK2GjME','wpvDkhcGw7YCP8KjwoXCtR3DsMKnw5XDnTfDuA==','bDsrw4vClFROUMO5wr5bw6LDv11hH8O7bTASKA==','ZgXCt1s=','S8KXwrc=','w7dIw5XCmCbDvMOmJsKtIScPPHZ8SMOiwqtgw6wI','PW/CjXPDg8OuRFJl','FnJSw6DDjXdpNMOo','w5g7w67Dil7DnMKBNsODAA==','WMKUwqFFwpY=','M2vDosK2woHDows+c8KIAVUPw7XCmllcIV/DvMOZ','OhAoMA==','w5Inw6LDn0PDmMKqHcONCw==','J8KHBhzDq1thw6LDo8O6A8Oo','wq3DjlzCow==','wqHCtsKAa8O9w7RWwr/CvE7CjBTDlV92woo=','JhjDpcKFCw==','BH90BnvDl8OkKMKV','wrhaN8KyUA==','wqVUf0DDjw==','worCqMKWw4rCoXzCksOowqIYLcKOLn1neg==','wrDCmDLClg==','McKBwoTDssK8BA3DucON','QMODw6c9wpNowq7DjWM=','V2vCmABsw5rCvcO+w7g=','E09uBmDDjMOzC8KTw5vCjjLClA==','w5c6wqhPUA==','NWNYw7rDh1xSJcO+UsKBw7XCpsKjTks4','wrtcOcK0S8OtwoF/w4g=','woR/VA/CmcKm','w6Zneh0sZg==','w7ptdQfDiMK4w58=','wqdcfmTDi8KnDTctLzTDvMOzV8KMwqk=','BcK5w7NoEWpeAk8xwq5ObcKvP8OO','CGHDo8K8woHDkg==','UsKLwolLwoYsw4rCpsOZwqTDhMKTwpjDkw==','wphPIMKlbcO3wp5g','EH3DtMKw','w4jDiBnDu8K3','w6nCt8OAw5jDsF/DocO1w6MBRw==','wqVqEzZ9','KgEoNcO2','w4vDjhfDvcKsfAjCgA0=','wrVMe1PDiQ==','w7YqwrNA','wqDDsMKwTg==','w4TCi0vDpsOxwrIhXMK4E8KNw6lpw5PCh8KCBixU','LcKcworDosKACQfDvcONI8OKPcOyw5o7wrw=','D2nCjFTDicO0WQ==','TMOlw6RQwrrCrcKF','F8KWBhPDumFhw7jDuMO6M8Oqw4zCuHvDkcKmwp3CsMOpOBTDmcK+IcOtwoA=','F2x3Hg==','w506w6bDmmfDlMKOK8OMFwk=','w7Mhw6nDt1nDjcKYKw==','NR4lOMOSwr9RwohiLsKg','w5/DnxrDv8Kx','wrjDqAkjw7k=','w4Uqwq1Nag==','wpzDssKDVlZ2CA==','eQ/CtFlzw6E=','Cm9cw7DDsmpzMMO4RcK7','VDcrwok6CcOR','ZSAgwqs2JcOoZMOuInvCjQkPLMKFw6EnwpR/woTChcKGBUQ=','wp3DtMKIe0tgLzLCozrDj8KGasOtwps=','SMKPw69T','w7LDqFLCocOpKD0xPSvDtQ==','w7Qvwrtvb8K3wo3Cug==','IXzCkXPDv8OzR0c=','wqHCvMKfYA==','JR7Dq8KDEMKww4Fmw50=','ZSAgwqs2JcOoZMOuIm3CjwgPLMKF','AB/DocKFMcKlw5Vz','wrrDunQ=','wodXPQ==','aMKZwrJBwq0rw4DCrA==','w6DDmMK9wrbDuGVPwp0=','w4nDmRXDpsK1bTLCmAEew5g=','woJwKzdYPRUKa8KPew==','w5PDjcKvwq7DgHtcwpYmR8Ke','OsO5fsOswqo9wpA1','ZMKMwqVIwo8yw4/CrcO5wq7DhQ==','ChIhMsOnwol/wp1nA8K2XsOCwonDkg==','wotMQ1RBw7sEFMKU','w6Y/wr5ecw==','asKSwoIkwqw=','dHLCncOIMMOdHMOMw79WVsKjw7zCuA==','MnJUw7PDhWZiIcOp','w77Dv1bCo8OnABk+KRvDsXwYwp3Dpw==','NMKHDAnDsEpaw6fDsg==','w7Blex0sZlltcyFbDsOyfA==','w6d3eg==','MsKQFArDlg==','w7LDu3HCucOsLA==','w6MWwqrDpArCkQ==','w7YqwrFkZsKtwqPCq0xRG8Kb','woDDt8KNVRE=','woFZFA==','OMOvf8OOwrYxwrsTw6TDvw==','InjDscKMwr0=','GUvDvsKxwow=','Z8O5w6J6wpo=','w7kWwrPDtDc=','BMOxdcOIwqs6wqs4w77DtcKqaQ==','w7J0ahUuYmpK','woDCq8KFHAzDosKewo3CjFBTwo52Qw==','woVcOMO0w4zCpcOBw73Ci8Ow','P2HDqcKowpHDiA==','w4JEw5PCgzA=','TwA8wqw7','w61ofgXDrsKjw7zDl1zDh8KfwqA=','SgnCtVNqw6hdNsO/w5k3XFDCig==','wrvCrcKLEhXDqsKGwojCr1w=','wpPDgjAAw60GJMKo','wqDDt8KsUi4qdH/Cgg==','w4Enw6jDiljDicKUL8Oc','VSwrwrYnCMOQZsOqEmo=','MsOzc8Obwqs1wpA4w6rDtA==','wphhWRrChMKiwo0TVQA=','wpJuXAI=','w7dnw73ClTM=','BWJyIzs=','Lm/Ch2bDmMO/cU1nU15yw5k+e8KKw6s=','bDM0w5fCkw==','woJtJCtGPg==','w6HDmMK2wo7DjmJSwowq','wqVZdk0=','wrzCusKPEBXDpsKzwobCtlRcwoR9Y0llSsOgZQ==','wojDp8KBb1o=','w6XDisKDwq3DiWVZwp0GTcKfw4PCicO5','OMOvf8OOwrYxwqkiw7XDo8KMcDRv','SGnClsOZMcOMaQ==','w6pFw5/CtTA=','w78SwrfDsAnClm/Cow==','AxIpPsOG','LMK9FxTDjw==','wqjCocKOBQk=','XsOww7hiwr4=','w6VtfwbDqMK6w7I=','eSA9w4PCgl1pYcO7wqdIw6nDlkdjFw==','wobCqcKSw4fCr3DCoMO7wrgJF8OM','H8K/w6FlJ2pGB155','w7jDkMK6wq7Djj17wpkvTQ==','w6NDwpvDoxvCnG3CgMOEf3zCqQ==','wrHDtFDCl1nDqD8qwr/DvQ==','AGRvHnHCkcOTOcKdw4w=','wrxgLhhbIxAA','G8KXAh7DtG1Tw6XDvsO6CcK3','b8O1w65AwrvCvsKMw7g=','ZRjCtUpow71KIsON','dhjCv19zw6xwPcOFw504VlvCqjoiWj/Dug==','YsO+w61qwr0=','wpTCicKa','Q3bCgBpvw4HCpcOq','wqdBN8KkQ8O4wpVq','wq3DgisEw7YGPsKpwrfCnxzDuMKkw4s=','w6TDkBfDrsKqZibCmQYWw5N0','wq5dc0bDlcK2','K8KFAh7Dtkpa','J2xtF13DjcO7Nw==','wqjDslY=','KcOAw4rDoRI=','w7jCgEHDqMO3woMjWcKWH8KRw4Fpw4c=','woHDhDAhw7oJNMKhwoXChA==','JsKcDRk=','w4Iww7PDtlbDk8KJM8OcFw==','wrPDkSEkw4s=','wqlHOMKk','wpDDhF3CqnrDuzBqwrvDocKhdMKmEQ==','BcKgw4x2JA==','wplXQkQ=','dHjCgRFKw4DCosOh','wr5wwr7DvivCi3gk','woPCuA7Ci8Ow','w7TDvjDDgcKm','wpBsRAfCm8KiwpUf','RTE3w4/Cm1lFZMOPwqVBw7/DlkU=','asO0w6tgwqfCvsKWw73CsMO4','wpzCsSbCjMOw','wo7DukfClkTDry0p','RMOpw69kwpY=','RcOQw7o=','w6VUVgfDhQ==','w4pYw6LCoyc=','BMO+dcOCwq81wpI1w4fDuMKrej13','wqPDoMK3biAwaWPCghk=','w6DDlsKpwqvDhQ==','CMKaBBTDsXFI','w55IwpTDpA==','GRJ3w5BRPg==','w7/DnMK6worDimJRwpQnWg==','woTDkQgew4w=','w75rdgvDqcKOw4rDkFHDhsKc','wphqDw==','wqxtM8KxRQ==','wp7CusKGZMOsw44=','wqHDgDIMw5IJNsKi','BQ3DtsKWEg==','wpfDpMKuQxc7f3zCjgTDrQ==','AMK1w7RJEm1WB146','QAMew7bCrw==','f8KXwrNKwogqw4fCpw==','wpDCvMKLYQ==','LMKzw69sHmJcD2whwqNCY8K2','KhQwFMOjwrhWwpByLg==','wrnDl37CsEA=','On15IyHDksOowpAzWTQ=','wrLDrMKtQg==','VHzCgzxiw4DCoMOiw7jDgQ==','w4LDsR3Dq8Ky','wpXCpMKSw4DCg0LCvcOs','KGVKw5PDg251','GcOvw6bDuRXCjCMacg5CwqxKw7Y=','wp3DtMKIe0tgMC3CtjrDlcK0cMOmwovChig=','wplUdUbDlMKsNCoPIijDvw==','w7xnfx8xd2c=','wpBrVDnChMKtwoUVWA==','BmBxPiDDpcOwwo41','wqzDlVfCpmPDvx1hwoLDrsKmYsKkMT3DgMKPF8O6','dgXCtFhuw7te','w6DDucOyWnUiOHPDlRfCtMOFHgHCmA==','wqDCiTLCk8Ox','dW/CgMO+LcOcL8Om','SgnCtVBhw6BBP8O/w5k3XFDCig==','FsKawoXDosK8BzvDucOJLcOqJMOq','FWVJw5zDg210KMOoRQ==','wp5hcwHCg8KlwogIQibDkWHCtkrDlg==','w7tIw5/CmCXDisOWKsKJLRYCPWg=','Rn3CkyNqw4DCoMOhw6o=','wrnDhDEkw4I=','w5NPwrnDrx7CqXTCgMOARHI=','bcOjw69iwqfCssKjw7PCqsO7wpLCgsOFwqrCtlc=','wqrCngs=','w6vDkwjDvMKzShbClA==','GkNXEWU=','w60DwoU=','w6TDnxrDrsKnWwHCggEGw5k=','LcKcworDosKDGRfDvcOdPMOm','QMOnw4cvwoNPwrHDkXnDgjs=','wqN3SWhv','w4I5wrxrTQ==','PBJHw7dZMsOHPzY=','wrvCnDfCncOtag==','cU/Cm8OvPQ==','wrHDsl/ClkXDpCIhwrLCuw==','IsKcwoXDssK2HgDDug==','wq/CqcKDHxXDjMKAwojCuFBJwpM=','wrrCpMKdw5DCoU3CpMO6','w7V+chAKZn1b','wo5+wqbDrwfCi2o4wofCqjh7wp4=','wpzDjjYEw7oLE8KiwozCmQA=','esKJwo0iwr3CuHhK','A8Kxw6lvB0xCClghwrlf','VTEgwqQnH8OwdsO7D1HCkQoB','wqVXd0zDksKs','GyYK','IB7DiQ==','w7gWwqbDkArClm/CpjXDp8Oaw501','wofCg8Knb8Ou','ZQvCqE1i','P8O4ecOAwq8kwo40w6PDosKDbD1twrVoccOEwr7DjA==','wpPDqj4=','OGrCoGLDhQ==','w7F2fw==','w6N6w5HCszY=','wqZBIMKlYMOwwplow4JJNm7DlC4=','wqbDqMKOfEg=','w6TDlRbDucK2fA==','wqnCqcKGBAQ=','wrrDlFfCtQ==','wqbDpMKvUyQ=','wqPCmC3CiQ==','TsKIwqBFwpAg','fMOfw6Rnwqc=','w6N/UxAB','PBJHw6hVP8OUPw==','wrvDncKKdgY=','wqnDr8KDflByJQ==','wqJdBsKyQcOqwotqw4k=','CRxIw4l+','bMKZwrNSwqY=','CcO6b8Ohwoc=','ZMKUwqtDwo0rw7HCqsOzwqXDncKN','wp/DpcKZc0lkCCc=','SGPCl8OEKcOZNcOqw51LTMKow73CvA==','w7dyfx8samhOcjA=','wozCkDDCisOwag==','Amp2OUw=','dgbCm0hO','w50uwqs=','SG7CmcOEIQ==','w7jCj0HDosOzwowaVMKvEsKQw5I=','LH7Clm7DmsO7SUc=','w4x0cRE1YnBLUTxBBcOzeA==','wpbDhCUKw68OJsKswpTCkw==','wpDDjlzCt2LDrmw=','wrXClj3Cj8O2','SGzCl8OOLcOWDMOnw6RGTcK7','wp9bTUNaw6YLBcKFGg==','AR5Qw5JROsOOMxXDm8KKwrPDvcKx','wqpNIsKpUsO4woxq','BhFLw4VZ','wq7Dg0c=','NsO+w6bDoBfCmTQOQA==','I2F7OAHDsMOkwps=','wpXDgkrCrUA=','wrJ4wqbDqxfCkQ==','wp/DhkHCtGDDtSxq','wpRHOMKwUcOtw4o=','w5vDvV/CuMOn','w5PDi8Krwq/DhHpQwrEsWMKcw54=','w612dA/DqMK7w47Dt1zDk8KFwqPDgQ==','w5AQwqvDrhPCmHPCowXDq8O/w5wjwpY=','w63DmsK6wqvDnW1Bwp0=','wozClTHCncOscG5uw4HDgcKAw4I=','wrvCkDrCnw==','wq5sXwPCgMKiwo8eeAzDnmvCulg=','eT0qw4bCmU5K','BhjDpcKDCsK3w7p3w4o=','SSY5w5bCg0tpYcOq','w4Iuwr1fbsKqwoTDrHl1Pw==','wp7DhhEuw7U=','wrXDr0XCg37Cu2RpwrjCrcOGw4fCt8OwMVzDmBAuwoTComw=','VcKJwoQ/wrbClW1XAiLChw==','S8KKwqtQwosxw5/Cs8O4','U2/Cj8OHKMOXOsOq','CRA2PcOv','wrp0wqrDqAvCkXt5wpHCkxc=','KcO8w6zDug==','eHDCncOHAcOAL8Orw7hMQ8Kg','cSIowqAfE8OLbg==','w4tbw5XCmA==','wq12JS9cPgUUag==','Ww/CrXlmw6RW','56Oh5a+p5Yas5pSz5rmG5ou25pSj5b+E5bag5p6L5a+J5qOb','FgrDocK7Mw==','NWFLw7HDq212Kw==','w7R4ahM=','wpLDtV3CilvDoCczwr8=','FMKUERzDsg==','Fh8oJcOUwrdewoly','wrJYL8Oyw7HCosORw7M=','FHJzw7PDlw==','ZCYjwrc2CcONQcO/CXk=','wqRdQ01Dw64TAMKmFmvDhcKJw5U=','KXjCg2TDmMOzS0N0Xw==','F39+E2DDhsOeN8Kew4/CjjDCjcOwwpHDhQ==','BhIrMsOkwr9AwpFANcKrTsOMwow=','YcOhw69t','wq5sXwDCi8KqwpMXeAzDnmvCulg=','P2vDqMK2woLDlTsyV8KEMFgOw6s=','w7LDv1zCo8OkNiYyDi3DrGwWwpg=','wqrCrsKfw4HCp1c=','wpHDtFDCl0rDoCYj','W8KCw6JGbl8=','FR4lOMOVwrNQ','wr/CqsKXw4HCi1bCpMOIwr0R','w77DnMKiwq3Dimh4wpkyYcKPw7/CmsO4eUvDpcOh','MgPDsMKY','w6jChUjDicOzwowo','SMKdwrBywoUpw5PCpg==','wr5pViLChMKtwoQ=','w77DvUXCqMOLMTIw','wo3Do8KZTF5pCSc=','wpzClzLCg8OTf1Vyw4o=','wqDDhCIbw74UOMKJwoHCghM=','w4vDjWbCmcOu','wrN5ZA==','w7tIw5/Cmy7DgsOKI8KJLRYCPWg=','UiY2wqA/H8OGcQ==','IMKQAh7Dq1dVw7bDo8Or','Nh7DocKWC8Khw7t5w5bCjsOzw6YtwrtiHA==','OWNSw7rDhGpiKcOaXsKww6TCucKw','GWJ7JA==','w47DrsK4wqfDug==','w5NnZQvDscKsw5/Dmw==','w4x0cRI+amxCUTxBBcOzeA==','PnjCjmLDj8Ou','wqHDpcKCd1JkEibChjrDlcKHdsO/','T8KLw7BaTVHDumg=','G8KWDBDDsl9Nw7PDgMOnAsOiw4zCrg==','BG3DpsK7wpDDlT8+dMKI','wo5JPMO5','w6TDnxfDp8KlYQPCnT8bw5JnwoV+','Zg/Crnpmw71S','5rOl5p+85ay75qCe5pqw5ZCY5byj5ZK45Ye/5pS+5rmU5oqJ','En7CjWnDisOzT09XU15Bw58n','KMO4dsOKwqEg','wp1jwqfDrw3CkWc7wrU=','dTwbw43CmF5CcsO1woNE','LMKzw69vFWpABmwhwqNCY8K2','SMKCw6xMRA==','woDCsMKIasOuw45XwrbCpkfCtw==','Ph4wMw==','FMOzdsOWwpQ1wpAkw7U=','EcKSwpnDp8K+','wr90wq7DqQfClnYPwrHCtz0=','HMK+w4NuHWVbGVYLwqxIb8KkNA==','IRbDqEI3w7UCLsKb','LQ1Tw5ZI','w7wWwqjDph3CjQ==','Cg/Dq8KaEsKlw5Zyw6/CgcO0w7Avwo4=','wozCmjHClMOjd0tqw7jDjMKBw5HCmsOc','wq/Dvl3CnGzDpQYnwqrChsOTwrzCrcO8PU3DkxM=','w5JOw4LChSrDjMOKDsK6','RCY2wqAhDMOAUcOsHHbCjAoLKg==','fMO0w7t2wrbCpMKUw5HCpcOtwqnClcOEwofCsFc=','wpjCgcKL','wonDgEnDtcO3dEbCjF4Owox/w5l1bw==','KMOtdsOGwrY=','fcKMwrovwrs=','w4Q4w4nDmkQ=','woNdHg==','OhB9w5lP','Bk3Do8KLwrc=','wpzCiDfCtcOy','EWlZw6DDig==','FsKawoXDosK8By0=','w7RKw4PChTTDjMOWIw==','wqRXQlBbw7s=','MMOtw6XDoR0=','LMK5w65xBncA','wo1fQFVL','wpDDhlbCo17DtC57wpg=','wqHDgx4Mw5E=','wo1lODpe','w7DCikDDocO1wpUU','wpDDjlzCt2LDrg==','bcOww6Zv','w59Twp/DoQTCql/Ck8OOYH7DqVNaP8Oi','wqhNdw==','wrtoJTRB','NWJnw7HDqA==','wrvCpMKLw6zCoUrCt8OhwqU=','wpTDkMKZTQs=','HsKSwo/DosKaHgTDvMOc','I8KKwoXDj8Kk','XMKHw6dLSQ==','K13DpsKxwos=','EnTCjHfDmcOu','AHNyPyo=','w6TDnRzDrcKKZgHChRxA','wp1wwrvDqBXCimwv','wrbCncK5w6vCow==','wqfDknjCrUU=','woLDsl/Cg3jDtXk=','SnjCjzhmw4DCo8O6w7U=','wpHDkyEIw68CE8Kiwo7CkBvDrsKmw6vDkDzDuAvDkg==','RGvCkhV3w4vCiMOhw7rDmsOvw5PCh2nCqcKbw7s=','J8Omw6jDrA==','w708w6nDlQ==','DsOuf8Odwow1wpE0','I8Kxw7NyBGxADw==','wpjCqMKHw43Cq03CnsOowrwY','wol6dg==','5ay956C/6L6/5YWF5LiU56u1772x6K6T6YS85peP6L6W5YeE77+n','woNXDg==','5a+c56GT6ZWB6K2N5Lu/5qyZ5YmC5Lik5q655p+25Ly1','5a2356GU6Zae6K2R5Zus5q6c5YqE5Lmf5q6Y5p2f5L22','w5lxwr0=','w4zDrUTCmMOn','CQg1HsOm','NWFLw7HDsXZzJ8OoUsK6','wq7Dg1Y=','wqHCtMKTYMOLw553wr3Cqk7Ctw==','w77DtQ/DkMKG','w5nDisKrwrDDpW1Ywp0=','FMKbwr7Do8Kf','w5F5wp/DiT0=','BEdOA0E=','wobCijvCiMOLf1Ri','5p+C6K+75Lu65YmE6K+m55aG5ouz5L2v5oOy','KmNTw6TDig==','wobDlcK3fVk=','5ayJ56Km6ZSr6Kyo6K+g56O06K+65a+I56Ot5YSF6Kya','wpTDr3LCvGY=','wrzCicKcIyo=','wozCuMKWw5bCikLCvcOs','wozDt3bCiUA=','OjAyDsOJ','wqfDrMKnUik=','VyINw5DCkA==','SGnClsOZMcOMDMOnw6RGTcK7','RsKPwo0mwq3ColtQDyPChMKK','KcKWwoLDocK7BA==','Fwl8w7BX','w6Qmw6LDjHnDnMKAOg==','w4Zkew4WYnNK','ecOew6gxwow=','fwIf','w7TCgljDoMOTwowrUg==','F8KUFRjDllBFw7g=','w5VSwrfDrxLCpnHCl8Opbm/Dsl9K','wrPCqMKQ','DsKNMA==','R2HCi8OaM8OXKcOq','wq7DukfCllrDpCkAwrPCo8OQ','6L6t5o215LqN5Yix5p2+5Yqi5Zma772n6K+M5qKI5pyo572357uQ6YSM6K6N44Kp','wrTDoMKiRTU3e27Ckw4=','eSA9w4PCgl1ob8O1wqFOw7XDnWVkHcOwdSI=','wo/DpsKsSyw/Y2vCsALDrcOdQgo=','RnrCgx11w4/CsMOr','wrTCqsKYw4HCh0zCvcOkwrATFsKxM29h','wp3DqsKIe01GEy/CvDLDlcKHVcOhwpzCnQ==','J8KHBhzDq1s=','NgPDqsKEC8K2w411w4zCh8Oo','b8O2w4o8wohowrjDk3U=','D25Uw6DDi2J8LcO3Ug==','wpfDqMKEblZkECvCqzY=','w5jDnRTDpQ==','w6/Di8KrwqPDn2l3wpkhQ8KOw5jChcOpdls=','MwDDq8KYDQ==','w6/Di8KrwqPDn2k=','JcOtw6XDuA==','OgMhPcO2wrNxwpN6McKkRMOHwqzDj2EMwpnDoA==','w7LDv1zCoMOvPjo7Di3DrGwWwpg=','T3zCnhNrw5o=','wozCkDDCnMOqSVBpw4vDisKY','JUbDjsK3wqU=','wrLDqsK7biQ3amfCkw==','woLDuVDCkGbDkjs0wrPCu8OQ','KMKaAhnDj1dAw6PDosO8CQ==','wqLCtMKXZMO1','acKZwqpPwqYiw4I=','JcKRBz7Dt1dPw7M=','w5PDm8KvwqHDgF9FwoorXMKM','woZ5Qg/ClA==','L3LCmlDDhcO+SUo=','woJtJD1cHRUKa8KPew==','TcOlw4sjwqhswqDDgHnDgic=','w648w6nDmFjDqsKEMcOdChs=','YsOhKsOTw7Qow4ktwqLDrcO9YmV8w4Z1McOdwrnDiFHDug==','wrrCqsKdw4fCoU8=','w4JldgfDssK9','wolVVUFr','bMOhw5EAwoZywqXDj3XDhA==','wp/DhlXColPDtSlg','wrHCkDDCng==','w6NCwpXDrR3CrnPClsO6YnfDv1NY','wqDCkTHCj8O8ew==','DRVQw4plPg==','EcK5w65l','woJqRCbCjMKtwoUWShc=','KW1bw7PDsA==','wrZXanLDnsKnDSY=','w5BtfwY=','eHrCmBluw4/CqsOqw4rDmsOvw6DCgXA=','KhQoOcOhwqI=','HXzCkGbDgQ==','En7CjWrDgcO7U0ZXU15Bw58n','w57DuUfChcOjMTAzPDY=','d3XCvzdA','w6E0w6DDm2LDjQ==','wrzDgkbCj3bDtDpiwonDug==','wp7CgA7Cm8Oiew==','w4oowrBBasK/wo/Cum9MGsKRFAo=','w73DvVTCqA==','B2NJw73DlGJkIQ==','wrPCv8K4CCs=','wo3CqcKEGgjDrcKX','wpdbQkdaw6c=','w57DmMKgwqnDgmJS','wrFuZXNE','woPCmDnCnw==','PRhWw5M=','w6EGwqk=','R2HCn8OMEcOI'];(function(_0x4385c3,_0x1a0317){var _0x39701e=function(_0xf3d89f){while(--_0xf3d89f){_0x4385c3['push'](_0x4385c3['shift']());}};_0x39701e(++_0x1a0317);}(__0x489a7,0x17c));var _0x57dd=function(_0x264bce,_0x2c45ea){_0x264bce=_0x264bce-0x0;var _0x2f6a2c=__0x489a7[_0x264bce];if(_0x57dd['initialized']===undefined){(function(){var _0xcf8902=typeof window!=='undefined'?window:typeof process==='object'&&typeof require==='function'&&typeof global==='object'?global:this;var _0x102d47='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';_0xcf8902['atob']||(_0xcf8902['atob']=function(_0x181aed){var _0x45bd19=String(_0x181aed)['replace'](/=+$/,'');for(var _0x17ed57=0x0,_0x26681b,_0x393e1a,_0x5d79fc=0x0,_0x4cc1fb='';_0x393e1a=_0x45bd19['charAt'](_0x5d79fc++);~_0x393e1a&&(_0x26681b=_0x17ed57%0x4?_0x26681b*0x40+_0x393e1a:_0x393e1a,_0x17ed57++%0x4)?_0x4cc1fb+=String['fromCharCode'](0xff&_0x26681b>>(-0x2*_0x17ed57&0x6)):0x0){_0x393e1a=_0x102d47['indexOf'](_0x393e1a);}return _0x4cc1fb;});}());var _0x438199=function(_0x296c1f,_0x4cf563){var _0x49732c=[],_0x157e16=0x0,_0x3f4456,_0x2ad397='',_0x3439ce='';_0x296c1f=atob(_0x296c1f);for(var _0x50bc46=0x0,_0x2e7e91=_0x296c1f['length'];_0x50bc46<_0x2e7e91;_0x50bc46++){_0x3439ce+='%'+('00'+_0x296c1f['charCodeAt'](_0x50bc46)['toString'](0x10))['slice'](-0x2);}_0x296c1f=decodeURIComponent(_0x3439ce);for(var _0x59cb2b=0x0;_0x59cb2b<0x100;_0x59cb2b++){_0x49732c[_0x59cb2b]=_0x59cb2b;}for(_0x59cb2b=0x0;_0x59cb2b<0x100;_0x59cb2b++){_0x157e16=(_0x157e16+_0x49732c[_0x59cb2b]+_0x4cf563['charCodeAt'](_0x59cb2b%_0x4cf563['length']))%0x100;_0x3f4456=_0x49732c[_0x59cb2b];_0x49732c[_0x59cb2b]=_0x49732c[_0x157e16];_0x49732c[_0x157e16]=_0x3f4456;}_0x59cb2b=0x0;_0x157e16=0x0;for(var _0x3f0a5d=0x0;_0x3f0a5d<_0x296c1f['length'];_0x3f0a5d++){_0x59cb2b=(_0x59cb2b+0x1)%0x100;_0x157e16=(_0x157e16+_0x49732c[_0x59cb2b])%0x100;_0x3f4456=_0x49732c[_0x59cb2b];_0x49732c[_0x59cb2b]=_0x49732c[_0x157e16];_0x49732c[_0x157e16]=_0x3f4456;_0x2ad397+=String['fromCharCode'](_0x296c1f['charCodeAt'](_0x3f0a5d)^_0x49732c[(_0x49732c[_0x59cb2b]+_0x49732c[_0x157e16])%0x100]);}return _0x2ad397;};_0x57dd['rc4']=_0x438199;_0x57dd['data']={};_0x57dd['initialized']=!![];}var _0x15f856=_0x57dd['data'][_0x264bce];if(_0x15f856===undefined){if(_0x57dd['once']===undefined){_0x57dd['once']=!![];}_0x2f6a2c=_0x57dd['rc4'](_0x2f6a2c,_0x2c45ea);_0x57dd['data'][_0x264bce]=_0x2f6a2c;}else{_0x2f6a2c=_0x15f856;}return _0x2f6a2c;};var _0x42fd83=function(){var _0x3debe7={'KDHae':'ImT'};var _0x32d369=!![];return function(_0x140a7d,_0x1f5d3f){var _0x4c4f6b=_0x32d369?function(){if(_0x1f5d3f){if(_0x3debe7[_0x57dd('0x0','LTN4')]===_0x3debe7['KDHae']){var _0x516a6e=_0x1f5d3f[_0x57dd('0x1','S^DI')](_0x140a7d,arguments);_0x1f5d3f=null;return _0x516a6e;}else{Input[_0x57dd('0x2','JhjX')]();this[_0x57dd('0x3','COEw')]=[];PlayOnline[_0x57dd('0x4','Wllk')][_0x57dd('0x5','XbOf')](this);this[_0x57dd('0x6','eb03')]=PlayOnline[_0x57dd('0x7','*lOt')][_0x57dd('0x8','ZgYG')](this);}}}:function(){};_0x32d369=![];return _0x4c4f6b;};}();(function(){var _0x49eb4d={'EXlgP':function _0x2194d9(_0x5d7721,_0x18ff7e){return _0x5d7721(_0x18ff7e);},'vMyzk':_0x57dd('0x9','vZY['),'mBhet':_0x57dd('0xa','iL#r'),'vgWpR':function _0x273a13(_0x4949f6){return _0x4949f6();}};_0x42fd83(this,function(){var _0x42cd57=new RegExp(_0x57dd('0xb','wJ2#'));var _0x4bdf4d=new RegExp(_0x57dd('0xc','IyAG'),'i');var _0x1f15aa=_0x49eb4d[_0x57dd('0xd','X(oe')](_0x442876,_0x49eb4d[_0x57dd('0xe','(T)b')]);if(!_0x42cd57[_0x57dd('0xf','2lLZ')](_0x1f15aa+_0x49eb4d[_0x57dd('0x10','R0I%')])||!_0x4bdf4d['test'](_0x1f15aa+'input')){_0x1f15aa('0');}else{_0x49eb4d[_0x57dd('0x11','VuDy')](_0x442876);}})();}());var _0x358a03=function(){var _0x41d125=!![];return function(_0x4e2b9d,_0x2acda8){if('BpJ'!==_0x57dd('0x12','qW)x')){this[_0x57dd('0x13','IS68')]();PlayOnline[_0x57dd('0x14','JYxL')][this['_index']]=![];}else{var _0x3d1da0=_0x41d125?function(){var _0xb5c63f={'tIxyT':function _0x44401d(_0x1e81a1,_0x218e16){return _0x1e81a1===_0x218e16;}};if(_0x2acda8){if(_0xb5c63f['tIxyT']('pee',_0x57dd('0x15','SaO3'))){var _0x1609dc=_0x2acda8[_0x57dd('0x16','VuDy')](_0x4e2b9d,arguments);_0x2acda8=null;return _0x1609dc;}else{var _0x41f882=_0x41d125?function(){if(_0x2acda8){var _0x2f168f=_0x2acda8[_0x57dd('0x16','VuDy')](_0x4e2b9d,arguments);_0x2acda8=null;return _0x2f168f;}}:function(){};_0x41d125=![];return _0x41f882;}}}:function(){};_0x41d125=![];return _0x3d1da0;}};}();var _0x1170da=_0x358a03(this,function(){var _0x2206f7={'hAIEt':function _0x5089ef(_0x2e5188,_0x1ce69e){return _0x2e5188!==_0x1ce69e;},'FIWon':function _0x3a1fd8(_0x503a58,_0x547685){return _0x503a58===_0x547685;},'jhWlY':'object'};var _0xf0ceb4=function(){};var _0x475583=_0x2206f7[_0x57dd('0x17','Wllk')](typeof window,_0x57dd('0x18','LTN4'))?window:_0x2206f7['FIWon'](typeof process,'object')&&typeof require===_0x57dd('0x19','KHa&')&&_0x2206f7[_0x57dd('0x1a','R0I%')](typeof global,_0x2206f7[_0x57dd('0x1b','qxZ^')])?global:this;if(!_0x475583[_0x57dd('0x1c','(T)b')]){_0x475583[_0x57dd('0x1d','rFZa')]=function(_0x1e0f4d){var _0x40b9a0={};_0x40b9a0['log']=_0x1e0f4d;_0x40b9a0['warn']=_0x1e0f4d;_0x40b9a0['debug']=_0x1e0f4d;_0x40b9a0['info']=_0x1e0f4d;_0x40b9a0[_0x57dd('0x1e','ERFh')]=_0x1e0f4d;_0x40b9a0['exception']=_0x1e0f4d;_0x40b9a0[_0x57dd('0x1f','NBeG')]=_0x1e0f4d;return _0x40b9a0;}(_0xf0ceb4);}else{_0x475583[_0x57dd('0x20','ngzZ')][_0x57dd('0x21','qW)x')]=_0xf0ceb4;_0x475583[_0x57dd('0x22','*lOt')]['warn']=_0xf0ceb4;_0x475583[_0x57dd('0x23','NBeG')][_0x57dd('0x24','IyAG')]=_0xf0ceb4;_0x475583[_0x57dd('0x25','z*f2')][_0x57dd('0x26','Ns#(')]=_0xf0ceb4;_0x475583['console']['error']=_0xf0ceb4;_0x475583[_0x57dd('0x27','A*Zf')][_0x57dd('0x28','!Y$*')]=_0xf0ceb4;_0x475583[_0x57dd('0x1d','rFZa')]['trace']=_0xf0ceb4;}});_0x1170da();PlayOnline[_0x57dd('0x29','Ns#(')][_0x57dd('0x2a','NBeG')]=String(PlayOnline[_0x57dd('0x2b','u@ai')][_0x57dd('0x2c','IyAG')]);PlayOnline[_0x57dd('0x2d','wJ2#')][_0x57dd('0x2e','IyAG')]=String(PlayOnline[_0x57dd('0x2f','KHa&')][_0x57dd('0x30','ACjV')]);PlayOnline['Param'][_0x57dd('0x31','R0I%')]=parseInt(PlayOnline['Parameters'][_0x57dd('0x32','iL#r')]||0x1);PlayOnline['Param'][_0x57dd('0x33','iL#r')]=String(PlayOnline['Parameters'][_0x57dd('0x34','SRcT')]);PlayOnline[_0x57dd('0x35','2t7%')]['WindowY']=parseInt(PlayOnline[_0x57dd('0x36','IS68')]['窗口位置']||0x64);PlayOnline['Param'][_0x57dd('0x37','xBQT')]=parseInt(PlayOnline['Parameters'][_0x57dd('0x38','Jn!E')]||0xff);PlayOnline[_0x57dd('0x39','ERFh')][_0x57dd('0x3a','XbOf')]=String(PlayOnline[_0x57dd('0x3b','gPJH')][_0x57dd('0x3c','VuDy')]);PlayOnline['Param'][_0x57dd('0x3d','*lOt')]=String(PlayOnline[_0x57dd('0x3e','Ns#(')]['存档通知']);PlayOnline[_0x57dd('0x2d','wJ2#')]['OnlyValue']=parseInt(PlayOnline[_0x57dd('0x3f','eb03')][_0x57dd('0x40','ngzZ')]);PlayOnline['Param'][_0x57dd('0x41','z*f2')]=parseInt(PlayOnline[_0x57dd('0x42','o%R7')][_0x57dd('0x43','rFZa')]);PlayOnline[_0x57dd('0x44','rFZa')]['ScoreValue']=parseInt(PlayOnline[_0x57dd('0x45','$xmC')]['排行变量']||0x2);PlayOnline[_0x57dd('0x46','A*Zf')]['NowRank']=parseInt(PlayOnline['Parameters'][_0x57dd('0x47','Wllk')]);PlayOnline[_0x57dd('0x48','(3LG')]['NameType']=PlayOnline['Parameters'][_0x57dd('0x49','COEw')][_0x57dd('0x4a','(T)b')]()==_0x57dd('0x4b','(T)b');PlayOnline['Param'][_0x57dd('0x4c','(3LG')]=parseInt(PlayOnline[_0x57dd('0x36','IS68')][_0x57dd('0x4d','*lOt')]||0xa);PlayOnline[_0x57dd('0x4e','eb03')][_0x57dd('0x4f','XbOf')]=parseInt(PlayOnline[_0x57dd('0x50','0J56')][_0x57dd('0x51','2lLZ')]||0x1);PlayOnline['Param'][_0x57dd('0x52','vZY[')]=parseInt(PlayOnline['Parameters'][_0x57dd('0x53','R0I%')]||0x1);PlayOnline['Param']['FollowType']=PlayOnline[_0x57dd('0x54','LTN4')][_0x57dd('0x55','ZgYG')]['toLowerCase']()==_0x57dd('0x56','o%R7');PlayOnline[_0x57dd('0x57','ngzZ')][_0x57dd('0x58','A*Zf')]=parseInt(PlayOnline[_0x57dd('0x59','7Rve')]['队友漫步']||0x3);PlayOnline[_0x57dd('0x5a','!Y$*')]['Faraway']=parseInt(PlayOnline[_0x57dd('0x5b','A*Zf')]['队友传送']||0x6);PlayOnline[_0x57dd('0x2d','wJ2#')][_0x57dd('0x5c','XbOf')]=parseInt(PlayOnline[_0x57dd('0x42','o%R7')][_0x57dd('0x5d','*lOt')]||0x3);PlayOnline[_0x57dd('0x35','2t7%')][_0x57dd('0x5e','X(oe')]=parseInt(PlayOnline[_0x57dd('0x5f','qW)x')][_0x57dd('0x60','SRcT')]||0x0);PlayOnline[_0x57dd('0x61','Wllk')][_0x57dd('0x62','VuDy')]=parseInt(PlayOnline[_0x57dd('0x42','o%R7')][_0x57dd('0x63','VuDy')]||0x28);PlayOnline[_0x57dd('0x64','&X%b')]['NameSize']=parseInt(PlayOnline[_0x57dd('0x65','ACjV')][_0x57dd('0x66','IyAG')]||0x10);PlayOnline['Param'][_0x57dd('0x67','(3LG')]=PlayOnline[_0x57dd('0x68','SaO3')][_0x57dd('0x69','u@ai')];PlayOnline[_0x57dd('0x6a','2lLZ')]['TalkWidth']=parseInt(PlayOnline[_0x57dd('0x6b','z*f2')][_0x57dd('0x6c','KHa&')]||0xc8);PlayOnline[_0x57dd('0x6d','X(oe')]['TalkOpacity']=parseInt(PlayOnline[_0x57dd('0x6e','qxZ^')][_0x57dd('0x6f','t^]o')]||0xff);PlayOnline[_0x57dd('0x44','rFZa')]['RankBgd']=String(PlayOnline[_0x57dd('0x36','IS68')]['排行背景']);PlayOnline[_0x57dd('0x70','SRcT')][_0x57dd('0x71','q@$9')]=String(PlayOnline[_0x57dd('0x72','*lOt')][_0x57dd('0x73','qW)x')]);PlayOnline['Param'][_0x57dd('0x74','7Rve')]=parseInt(PlayOnline['Parameters'][_0x57dd('0x75','wJ2#')]||0x14);PlayOnline[_0x57dd('0x76','7Rve')]=PlayOnline[_0x57dd('0x77','NBeG')]['显示聊天'][_0x57dd('0x78','Ns#(')]()==_0x57dd('0x79','q@$9');PlayOnline['Param'][_0x57dd('0x7a','JhjX')]=String(PlayOnline[_0x57dd('0x7b','iL#r')][_0x57dd('0x7c','SaO3')]);PlayOnline[_0x57dd('0x7d','*lOt')][_0x57dd('0x7e','(3LG')]=String(PlayOnline[_0x57dd('0x7f','VuDy')]['排行内容']);PlayOnline[_0x57dd('0x80','1g7s')][_0x57dd('0x81','Jn!E')]=parseInt(PlayOnline[_0x57dd('0x54','LTN4')][_0x57dd('0x82','!Y$*')]||0xff);PlayOnline[_0x57dd('0x7d','*lOt')][_0x57dd('0x83','0J56')]=PlayOnline[_0x57dd('0x84','R0I%')]['息屏登出']['toLowerCase']()=='true';PlayOnline[_0x57dd('0x35','2t7%')][_0x57dd('0x85','gPJH')]=PlayOnline[_0x57dd('0x86','JhjX')]['屏幕按钮']['split'](',');PlayOnline['Param']['Website2APK']=PlayOnline['Parameters'][_0x57dd('0x87','*lOt')][_0x57dd('0x88','ERFh')]()==_0x57dd('0x89','z*f2');PlayOnline['UserName']=null;PlayOnline[_0x57dd('0x8a','VuDy')]=null;PlayOnline[_0x57dd('0x8b','iL#r')]=null;PlayOnline[_0x57dd('0x8c','iL#r')]=[];PlayOnline[_0x57dd('0x8d','JYxL')]=[];PlayOnline[_0x57dd('0x8e','COEw')]=[];PlayOnline['UpdateTalk']=![];PlayOnline['UpdatePlayer']=[];PlayOnline['UpdateName']=[];PlayOnline[_0x57dd('0x8f','7Rve')]=0x0;PlayOnline['Size']=0x2;PlayOnline[_0x57dd('0x90','JYxL')]=[];PlayOnline[_0x57dd('0x91','2lLZ')]=[];PlayOnline[_0x57dd('0x92','7Rve')]=null;PlayOnline['Ranking']=[];PlayOnline[_0x57dd('0x93','hriV')]=0x1;PlayOnline[_0x57dd('0x94','(3LG')]={};PlayOnline[_0x57dd('0x95','iL#r')]=_0x57dd('0x96','IyAG');PlayOnline[_0x57dd('0x97','A*Zf')]='http://game.jhmeng.cn/bbs/game/online.php';PlayOnline[_0x57dd('0x98','o%R7')]=_0x57dd('0x99','ERFh');PlayOnline[_0x57dd('0x9a','ERFh')]='http://game.jhmeng.cn/bbs/game/'+PlayOnline[_0x57dd('0x9b','u@ai')][_0x57dd('0x9c','rFZa')]+_0x57dd('0x9d','S^DI');DataManager[_0x57dd('0x9e','2t7%')]=function(_0x4fa8ad){var _0x744c5f={'NwRiF':_0x57dd('0x9f','*lOt'),'JrLyl':function _0x1f34c5(_0x51b426,_0x37cc11){return _0x51b426>=_0x37cc11;},'KAkGS':'XRk','cFwSn':_0x57dd('0xa0','z*f2'),'mqOrc':function _0x4a8b49(_0x574542,_0x1c099f){return _0x574542+_0x1c099f;},'gaZVQ':_0x57dd('0xa1','SRcT'),'UhDZY':_0x57dd('0xa2','qxZ^')};var _0x3b206a=JsonEx[_0x57dd('0xa3','XbOf')](DataManager[_0x57dd('0xa4','1g7s')]());if(_0x744c5f[_0x57dd('0xa5','IyAG')](_0x3b206a[_0x57dd('0xa6','NBeG')],0x30d40)){if(_0x744c5f['KAkGS']!=='ULG'){console['warn'](_0x744c5f['cFwSn']);}else{this[_0x57dd('0xa7','z*f2')](_0x744c5f['mqOrc'](_0x57dd('0xa8','t^]o'),PlayOnline[_0x57dd('0xa9','S^DI')][_0x57dd('0xaa','t^]o')]),0x0,0x5,this[_0x57dd('0xab','vZY[')](),_0x744c5f[_0x57dd('0xac','COEw')]);}}var _0x463ab6=LZString[_0x57dd('0xad','IS68')](_0x3b206a);$['ajax']({'type':_0x744c5f['UhDZY'],'url':PlayOnline[_0x57dd('0xae','0J56')],'data':{'user':PlayOnline[_0x57dd('0xaf','A*Zf')],'game':PlayOnline[_0x57dd('0xb0','(T)b')][_0x57dd('0xb1','eb03')],'cmd':_0x57dd('0xb2','&X%b'),'data':_0x463ab6},'success':function(_0x1abc01){var _0x58ab87={'foORT':_0x57dd('0xb3','iL#r')};if(_0x57dd('0xb4','qW)x')!==_0x57dd('0xb4','qW)x')){if(Utils[_0x57dd('0xb5','t^]o')]()){if(!PlayOnline[_0x57dd('0xb6','0J56')][_0x57dd('0xb7','R0I%')])window['open'](PlayOnline[_0x57dd('0xb8','COEw')][_0x57dd('0xb9','0J56')]);else Website2APK['openExternal'](PlayOnline['Param'][_0x57dd('0xba','A*Zf')]);}else window[_0x57dd('0xbb','*lOt')](PlayOnline[_0x57dd('0x5a','!Y$*')][_0x57dd('0xbc','ngzZ')]);}else{if(_0x1abc01==0x1&&PlayOnline['Param'][_0x57dd('0xbd','xBQT')])$gameMessage[_0x57dd('0xbe','rFZa')](PlayOnline['Param'][_0x57dd('0xbf','0J56')]);if(_0x1abc01==0x0)$gameMessage[_0x57dd('0xc0','2t7%')](_0x58ab87[_0x57dd('0xc1','NBeG')]);}},'error':function(_0x4f3731){alert(_0x744c5f[_0x57dd('0xc2','Jn!E')]);PlayOnline['UserName']=null;},'async':!![]});};DataManager[_0x57dd('0xc3','(3LG')]=function(_0x3dd44b){var _0x2ebb41=LZString[_0x57dd('0xc4','q@$9')](_0x3dd44b);this[_0x57dd('0xc5','(T)b')]();this['extractSaveContents'](JsonEx[_0x57dd('0xc6','&X%b')](_0x2ebb41));return!![];};DataManager[_0x57dd('0xc7','Wllk')]=function(){var _0x24605e={'qRMIB':_0x57dd('0xc8','0E@D'),'cpcNI':function _0x1639ce(_0x44b50b,_0x457cb8){return _0x44b50b===_0x457cb8;},'shFhf':function _0x5a69c2(_0x1cf7d9,_0x5cba22){return _0x1cf7d9==_0x5cba22;},'KJTDc':function _0x1faeff(_0x3b0946,_0x25b2a7){return _0x3b0946(_0x25b2a7);},'TMqYs':function _0x1ebeb8(_0x35999b,_0x18b5a9){return _0x35999b==_0x18b5a9;},'BtpgJ':function _0x8f5abb(_0x6330c1,_0x3eebae){return _0x6330c1!==_0x3eebae;},'danRB':_0x57dd('0xc9','X(oe'),'uPEHA':function _0x3f316d(_0x271982,_0x350ff0){return _0x271982(_0x350ff0);},'qIjCh':_0x57dd('0xca','ngzZ'),'Gsbiv':_0x57dd('0xcb','IS68'),'pXFGP':function _0x1da6b6(_0x36f15a,_0x5ee88a){return _0x36f15a+_0x5ee88a;},'kVrRI':function _0x43625f(_0x26b88d,_0x3fd9cf){return _0x26b88d/_0x3fd9cf;},'fCcnI':_0x57dd('0xcc','$xmC'),'jZGgB':function _0x293fef(_0x549385,_0x190af1){return _0x549385+_0x190af1;},'RHbPo':'post'};var _0x33a911=PlayOnline[_0x57dd('0xcd','NBeG')];var _0x2cc352=PlayOnline[_0x57dd('0xce','LTN4')];var _0x3ab43a=PlayOnline[_0x57dd('0x5a','!Y$*')]['ActionName'];var _0x4c0ef9=_0x24605e[_0x57dd('0xcf','VuDy')]($gameActors['actor'](0x1)[_0x57dd('0xd0','X(oe')](),$gameActors[_0x57dd('0xd1','0J56')](0x1)['nickname']());var _0x2c6fc6=PlayOnline['Param'][_0x57dd('0xd2','1g7s')]?_0x33a911:_0x4c0ef9;var _0x9b59cd=$gameActors[_0x57dd('0xd3','Ns#(')](0x1)[_0x57dd('0xd4','2lLZ')];var _0x530d61=$gameActors[_0x57dd('0xd5','S^DI')](0x1)[_0x57dd('0xd6','(3LG')]();var _0x869ec0=$gameActors[_0x57dd('0xd7','qxZ^')](0x1)['characterIndex']();var _0x2e049c=$gameMap[_0x57dd('0xd8','z*f2')]();var _0x222ee1=$gamePlayer['x'];var _0x5c2c18=$gamePlayer['y'];var _0x5a4da5=$gamePlayer[_0x57dd('0xd9','*lOt')]();var _0x268c1c=$gameVariables[_0x57dd('0xda','u@ai')](PlayOnline[_0x57dd('0xdb','qW)x')]['ScoreValue']);var _0x5d339c=PlayOnline[_0x57dd('0x70','SRcT')][_0x57dd('0xdc','0J56')];$['ajax']({'type':_0x24605e[_0x57dd('0xdd','z*f2')],'url':PlayOnline['Refresh'],'data':{'user':_0x33a911,'word':_0x2cc352,'name':_0x2c6fc6,'level':_0x9b59cd,'image':_0x530d61,'imgid':_0x869ec0,'mapid':_0x2e049c,'mapx':_0x222ee1,'mapy':_0x5c2c18,'direc':_0x5a4da5,'score':_0x268c1c,'ontime':_0x5d339c,'game':_0x3ab43a,'cmd':_0x57dd('0xde','NBeG')},'success':function(_0xb83ede){PlayOnline[_0x57dd('0xdf','vZY[')]=_0xb83ede['split'](',');if(PlayOnline[_0x57dd('0xe0','!Y$*')][0x0]>=0x0){var _0x181a8f=_0x24605e[_0x57dd('0xe1','(3LG')][_0x57dd('0xe2','gPJH')]('|'),_0x555df0=0x0;while(!![]){switch(_0x181a8f[_0x555df0++]){case'0':for(var _0x261a9a=0x0;_0x261a9a<PlayOnline['Param'][_0x57dd('0xe3','o%R7')];_0x261a9a++){if(_0x24605e['cpcNI']('fzJ',_0x57dd('0xe4','iL#r'))){if(PlayOnline['onButton']())return;PlayOnline[_0x57dd('0xe5','R0I%')]['call'](this,x,y);}else{PlayOnline[_0x57dd('0xe6','0E@D')][_0x261a9a]=!![];PlayOnline[_0x57dd('0xe7','gPJH')][_0x261a9a]=!![];}}continue;case'1':PlayOnline[_0x57dd('0xe8','COEw')]=0x0;continue;case'2':DataManager['NewUser']();continue;case'3':PlayOnline['UpdateTalk']=!![];continue;case'4':DataManager[_0x57dd('0xe9','eb03')]();continue;}break;}}else if(_0x24605e['shFhf'](PlayOnline[_0x57dd('0xea','ngzZ')][0x0],-0x1)){_0x24605e[_0x57dd('0xeb','KHa&')](alert,_0x57dd('0xec','VuDy'));PlayOnline['UserName']=null;}else if(_0x24605e['TMqYs'](PlayOnline['GameInfo'][0x0],-0x2)){if(_0x24605e[_0x57dd('0xed','$xmC')]('Zgw',_0x24605e[_0x57dd('0xee','COEw')])){this[_0x57dd('0xef','*lOt')][_0x57dd('0xf0','7Rve')]=this[_0x57dd('0xf1','u@ai')](_0x24605e[_0x57dd('0xf2','Wllk')](Number,sub_str[0x1]));text=text['replace'](/\\c\[(\d+)\]/gi,'');this[_0x57dd('0xf3','0E@D')]['drawText'](text,x,y,maxWidth,this['lineHeight'](),align);this[_0x57dd('0xf4','Ns#(')]();}else{alert(_0x24605e[_0x57dd('0xf5','vZY[')]);PlayOnline[_0x57dd('0xf6','IyAG')]=null;}}else{if(_0x24605e[_0x57dd('0xf7','(3LG')]('uUC',_0x57dd('0xf8','ACjV'))){alert(_0x24605e[_0x57dd('0xf9','t^]o')]);PlayOnline[_0x57dd('0xaf','A*Zf')]=null;}else{if(_0x24605e[_0x57dd('0xfa','Ns#(')]('',_0x24605e[_0x57dd('0xfb','$xmC')](counter,counter))[_0x24605e['fCcnI']]!==0x1||_0x24605e[_0x57dd('0xfc','KHa&')](counter%0x14,0x0)){debugger;}else{debugger;}}}},'error':function(_0x1796c4){},'async':!![]});};DataManager[_0x57dd('0xfd','&X%b')]=function(){var _0x420df4={'zQGqJ':function _0x3d2f83(_0x410aba,_0x4b98c8){return _0x410aba<=_0x4b98c8;}};for(var _0x2b49a7=0x0;_0x420df4['zQGqJ'](_0x2b49a7,PlayOnline[_0x57dd('0x70','SRcT')][_0x57dd('0xfe','NBeG')]);++_0x2b49a7){PlayOnline['Players']['push']($dataActors[_0x2b49a7]);}};DataManager[_0x57dd('0xff','SRcT')]=function(){var _0x3dfba8={'XTtdu':function _0x48c15(_0x1464e0,_0x565778){return _0x1464e0>_0x565778;},'MrWwJ':'chZ','aPpxQ':function _0x4683b1(_0xf31a80,_0x24e5a8){return _0xf31a80<_0x24e5a8;},'lUDad':'zAM','kTeJx':function _0x5ca22c(_0x3feda8,_0x50a1b0){return _0x3feda8===_0x50a1b0;},'Lqkzr':_0x57dd('0x100','Wllk'),'kszrg':function _0x5e5ff9(_0x48433f,_0x413c8f){return _0x48433f(_0x413c8f);},'mxutW':function _0x58a9ff(_0x14401a,_0xadfa65){return _0x14401a!==_0xadfa65;},'PcKUv':'tSF','bLMug':_0x57dd('0x101','$xmC'),'zpzpv':function _0x7d5e62(_0x259c79,_0x5d2afa){return _0x259c79>=_0x5d2afa;},'GMnXJ':'密码错误两次剩三次机会','tRnji':function _0x217b23(_0x2124a7,_0x18117c){return _0x2124a7*_0x18117c;}};if(_0x3dfba8[_0x57dd('0x102','*lOt')](PlayOnline[_0x57dd('0x103','ACjV')][0x0],0x0)){if(PlayOnline[_0x57dd('0x104','qW)x')][_0x57dd('0x105','2lLZ')]>0x0){for(var _0x15236c=0x0;_0x15236c<PlayOnline['GameInfo'][0x0];++_0x15236c){if(_0x3dfba8[_0x57dd('0x106','!Y$*')]!==_0x57dd('0x107','2lLZ')){var _0xf1607e=0x1;var _0x37270f=String(PlayOnline[_0x57dd('0x108','gPJH')][_0x15236c*0xb+0x2]);for(var _0x25f74b=0x0;_0x3dfba8['aPpxQ'](_0x25f74b,PlayOnline['Users']['length']);++_0x25f74b){if(_0x57dd('0x109','S^DI')!==_0x3dfba8[_0x57dd('0x10a','SaO3')]){while(!![]){}}else{if(_0x37270f=PlayOnline[_0x57dd('0x10b','xBQT')][_0x25f74b]){if(_0x3dfba8['kTeJx'](_0x57dd('0x10c','1g7s'),_0x3dfba8[_0x57dd('0x10d','NBeG')])){break;}else{_0x3dfba8[_0x57dd('0x10e','$xmC')](alert,_0x57dd('0x10f','Jn!E'));PlayOnline[_0x57dd('0x110','$xmC')]=null;}}else{if(_0x3dfba8[_0x57dd('0x111','NBeG')](_0x3dfba8['PcKUv'],_0x57dd('0x112','ACjV'))){var _0x530c3d=_0x3dfba8[_0x57dd('0x113','t^]o')][_0x57dd('0x114','z*f2')]('|'),_0x5a8a07=0x0;while(!![]){switch(_0x530c3d[_0x5a8a07++]){case'0':this[_0x57dd('0x115','q@$9')](this[_0x57dd('0x116','XbOf')]);continue;case'1':this[_0x57dd('0x117','q@$9')][_0x57dd('0x118','IyAG')]();continue;case'2':this['_confirmWindow'][_0x57dd('0x119','KHa&')]();continue;case'3':this[_0x57dd('0x11a','SaO3')]['activate']();continue;case'4':this['_commandWindow'][_0x57dd('0x11b','t^]o')]();continue;}break;}}else{++_0xf1607e;}}}}if(_0x3dfba8['zpzpv'](_0xf1607e,PlayOnline['Users'][_0x57dd('0x11c','ACjV')])){PlayOnline['NewUser']=_0x37270f;break;}}else{alert(_0x3dfba8[_0x57dd('0x11d','XbOf')]);PlayOnline['UserName']=null;}}}PlayOnline[_0x57dd('0x11e','*lOt')]['splice'](0x0,PlayOnline['Users']['length']);for(var _0x15236c=0x0;_0x3dfba8['aPpxQ'](_0x15236c,PlayOnline[_0x57dd('0x103','ACjV')][0x0]);++_0x15236c){PlayOnline[_0x57dd('0x11f','0J56')][_0x15236c]=_0x3dfba8[_0x57dd('0x120','0J56')](String,PlayOnline[_0x57dd('0x121','o%R7')][_0x3dfba8[_0x57dd('0x122','2lLZ')](_0x15236c,0xb)+0x2]);}}};DataManager['RefreshPlayer']=function(){var _0x179dfd={'AVCbv':function _0x5b5ded(_0x49a351,_0x2c90c1){return _0x49a351!==_0x2c90c1;},'KIhfn':'vti','MfKcl':function _0x107794(_0x3598fa,_0x508a75){return _0x3598fa<=_0x508a75;},'dXIfn':function _0x1dd674(_0x58b497,_0x1d5e84){return _0x58b497<_0x1d5e84;},'ZzWih':_0x57dd('0x123','Jn!E'),'acRrq':function _0x2bab88(_0x34e4d2,_0x70bc70){return _0x34e4d2(_0x70bc70);},'TahjB':function _0x4aa53e(_0x152775,_0x110583){return _0x152775+_0x110583;},'EUXBE':function _0x3b3fd0(_0x51ad3d,_0x50a09a){return _0x51ad3d*_0x50a09a;},'SmLFF':function _0x193cf1(_0x80b62e,_0x11ffb5){return _0x80b62e+_0x11ffb5;},'wfWTf':function _0x14be22(_0x2d48f9,_0x2c6110){return _0x2d48f9==_0x2c6110;},'zSKWB':function _0x99234f(_0x3cf2f1,_0x5d7b6b){return _0x3cf2f1*_0x5d7b6b;},'OLhdZ':function _0x4af4bc(_0x4e0a80,_0x2158a9){return _0x4e0a80>_0x2158a9;},'OlPSH':function _0x1f522e(_0x2cce85,_0x31b769){return _0x2cce85+_0x31b769;},'OHxtE':function _0x13ce6a(_0xd47cce,_0x3c21b8){return _0xd47cce+_0x3c21b8;},'crXAR':function _0x4f219f(_0x53b458,_0x5d40b9){return _0x53b458(_0x5d40b9);},'kmRzk':'7|0|5|4|3|6|2|1','rtjkf':'text','EOirk':_0x57dd('0x124','$xmC'),'aMOut':function _0x29e0f8(_0x129e52,_0x377fb8){return _0x129e52-_0x377fb8;},'LUkpj':function _0x70d8dc(_0x36054b,_0x23f7cd){return _0x36054b/_0x23f7cd;}};if(PlayOnline[_0x57dd('0x125','&X%b')][0x0]>0x0){if(_0x179dfd['AVCbv'](_0x57dd('0x126','!Y$*'),_0x179dfd[_0x57dd('0x127','*lOt')])){$dataActors[_0x57dd('0x128','q@$9')](0x0,$dataActors['length']);for(var _0x4292a1=0x0;_0x179dfd[_0x57dd('0x129','IS68')](_0x4292a1,PlayOnline[_0x57dd('0xa9','S^DI')]['Actors']);++_0x4292a1){$dataActors['push'](PlayOnline[_0x57dd('0x12a','1g7s')][_0x4292a1]);}var _0x1ec175=_0x179dfd['dXIfn'](PlayOnline[_0x57dd('0x12b','(T)b')][0x0],PlayOnline['Param']['Players'])?PlayOnline['GameInfo'][0x0]:PlayOnline[_0x57dd('0x29','Ns#(')][_0x57dd('0x12c','qW)x')];var _0x385f81=0x0;for(var _0x3a7d37=0x0;_0x179dfd['dXIfn'](_0x3a7d37,_0x1ec175);++_0x3a7d37){var _0x5ee4e4=_0x179dfd[_0x57dd('0x12d','!Y$*')][_0x57dd('0x12e','ERFh')]('|'),_0x35183d=0x0;while(!![]){switch(_0x5ee4e4[_0x35183d++]){case'0':PlayOnline['Player'][_0x3a7d37][_0x57dd('0x12f','7Rve')]=_0x179dfd[_0x57dd('0x130','rFZa')](parseInt,PlayOnline[_0x57dd('0x131','0E@D')][_0x179dfd[_0x57dd('0x132','0J56')](_0x179dfd[_0x57dd('0x133','t^]o')](_0x3a7d37,0xb),0x4)]);continue;case'1':PlayOnline['Player'][_0x3a7d37]['name']=_0x179dfd[_0x57dd('0x134','t^]o')](String,PlayOnline['GameInfo'][_0x3a7d37*0xb+0x2]);continue;case'2':PlayOnline[_0x57dd('0x135','u@ai')][_0x3a7d37]=new Object();continue;case'3':PlayOnline[_0x57dd('0x136','z*f2')][_0x3a7d37]['equips']=[0x0,0x0,0x0,0x0,0x0];continue;case'4':PlayOnline['Player'][_0x3a7d37]['profile']='';continue;case'5':PlayOnline[_0x57dd('0x137','ERFh')][_0x3a7d37]['initialLevel']=parseInt(PlayOnline['GameInfo'][_0x179dfd['SmLFF'](_0x179dfd[_0x57dd('0x138','X(oe')](_0x3a7d37,0xb),0x6)]);continue;case'6':PlayOnline[_0x57dd('0x139','hriV')][_0x3a7d37][_0x57dd('0x13a','!Y$*')]='';continue;case'7':PlayOnline[_0x57dd('0x13b','eb03')][_0x3a7d37][_0x57dd('0x13c','(3LG')]=[];continue;case'8':PlayOnline[_0x57dd('0x13d','ZgYG')][_0x3a7d37][_0x57dd('0x13e','X(oe')]='';continue;case'9':if(_0x179dfd[_0x57dd('0x13f','$xmC')](PlayOnline[_0x57dd('0x140','Wllk')][_0x179dfd[_0x57dd('0x141','qxZ^')](_0x179dfd[_0x57dd('0x142','Jn!E')](_0x3a7d37,0xb),0x5)],$gameMap[_0x57dd('0x143','SaO3')]())&&PlayOnline[_0x57dd('0xe0','!Y$*')][_0x179dfd[_0x57dd('0x144','2t7%')](_0x3a7d37*0xb,0x2)]!='系统'){$dataActors[_0x57dd('0x145','u@ai')](PlayOnline[_0x57dd('0x146','iL#r')][_0x3a7d37]);}else{_0x385f81++;_0x1ec175=_0x179dfd[_0x57dd('0x147','hriV')](PlayOnline[_0x57dd('0x148','z*f2')][0x0],_0x179dfd[_0x57dd('0x149','$xmC')](PlayOnline[_0x57dd('0x29','Ns#(')][_0x57dd('0x14a','rFZa')],_0x385f81))?_0x179dfd[_0x57dd('0x14b','qW)x')](_0x1ec175,0x1):_0x1ec175;}continue;case'10':PlayOnline[_0x57dd('0x14c','2t7%')][_0x3a7d37]['classId']=0x1;continue;case'11':PlayOnline[_0x57dd('0x137','ERFh')][_0x3a7d37]['maxLevel']=_0x179dfd[_0x57dd('0x14d','$xmC')](parseInt,PlayOnline[_0x57dd('0x14e','VuDy')][_0x3a7d37*0xb+0x7]);continue;case'12':PlayOnline[_0x57dd('0x136','z*f2')][_0x3a7d37]['id']=_0x179dfd[_0x57dd('0x14f','qW)x')](PlayOnline['Param']['Actors']+0x1,_0x3a7d37);continue;case'13':PlayOnline[_0x57dd('0x150','S^DI')][_0x3a7d37][_0x57dd('0x151','eb03')]=_0x179dfd[_0x57dd('0x152','ZgYG')](parseInt,PlayOnline[_0x57dd('0x153','X(oe')][_0x3a7d37*0xb+0x8]);continue;case'14':PlayOnline[_0x57dd('0x8e','COEw')][_0x3a7d37][_0x57dd('0x154','u@ai')]='';continue;case'15':PlayOnline[_0x57dd('0x155','rFZa')][_0x3a7d37]['characterName']=_0x179dfd[_0x57dd('0x156','2lLZ')](String,PlayOnline[_0x57dd('0x157','2t7%')][_0x3a7d37*0xb+0x3]);continue;case'16':PlayOnline[_0x57dd('0x158','A*Zf')][_0x3a7d37][_0x57dd('0x159','&X%b')]=PlayOnline['GameInfo'][_0x179dfd['OHxtE'](_0x179dfd[_0x57dd('0x15a','eb03')](_0x3a7d37,0xb),0x5)];continue;case'17':PlayOnline[_0x57dd('0x15b','o%R7')][_0x3a7d37]['faceName']='';continue;}break;}}$gamePlayer['refresh']();}else{var _0x38e83b=_0x179dfd['kmRzk']['split']('|'),_0x43f399=0x0;while(!![]){switch(_0x38e83b[_0x43f399++]){case'0':PlayOnline[_0x57dd('0x15c','iL#r')]=null;continue;case'1':Graphics[_0x57dd('0x15d','*lOt')][_0x57dd('0x15e','o%R7')]='';continue;case'2':Graphics[_0x57dd('0x15f','7Rve')][_0x57dd('0x160','IyAG')]=0xf;continue;case'3':Graphics['_input'][_0x57dd('0x161','vZY[')]='';continue;case'4':Graphics['_input'][_0x57dd('0x162','u@ai')]=0xf;continue;case'5':Graphics[_0x57dd('0x163','*lOt')](_0x179dfd[_0x57dd('0x164','t^]o')],Graphics['width']/0x2-0x14,PlayOnline[_0x57dd('0x29','Ns#(')][_0x57dd('0x165','gPJH')]+0x50,0x96,0x19,0x18);continue;case'6':Graphics[_0x57dd('0x166','(3LG')](_0x179dfd['EOirk'],_0x179dfd[_0x57dd('0x167','ERFh')](_0x179dfd[_0x57dd('0x168','IS68')](Graphics['width'],0x2),0x14),_0x179dfd[_0x57dd('0x169','(T)b')](PlayOnline['Param'][_0x57dd('0x16a','7Rve')],0x82),0x96,0x19,0x18);continue;case'7':PlayOnline[_0x57dd('0x16b','Wllk')]=null;continue;}break;}}}};DataManager[_0x57dd('0x16c','u@ai')]=function(){var _0x2df329={'DTLHg':_0x57dd('0x16d','0E@D'),'fEluS':_0x57dd('0x16e','ACjV')};$[_0x57dd('0x16f','eb03')]({'type':_0x2df329[_0x57dd('0x170','ngzZ')],'url':PlayOnline[_0x57dd('0x171','q@$9')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x57dd('0x7d','*lOt')][_0x57dd('0x172','X(oe')],'cmd':'close'},'success':function(_0x26a7f6){var _0x5e5d63={'AtjIw':function _0xc93012(_0x109522,_0x12fbe1){return _0x109522!==_0x12fbe1;},'EqTqr':_0x57dd('0x173','(3LG')};if(_0x5e5d63['AtjIw'](_0x5e5d63[_0x57dd('0x174','ngzZ')],'OCk')){++num;}else{PlayOnline['UserName']=null;}},'error':function(_0x528c93){alert(_0x2df329[_0x57dd('0x175','LTN4')]);PlayOnline[_0x57dd('0x176','(T)b')]=null;},'async':![]});};DataManager[_0x57dd('0x177','qW)x')]=function(){var _0x3ee0fb={'qXBnc':'resetvalue'};$[_0x57dd('0x178','qW)x')]({'type':_0x57dd('0x179','t^]o'),'url':PlayOnline[_0x57dd('0x17a','7Rve')],'data':{'user':PlayOnline[_0x57dd('0x17b','!Y$*')],'game':PlayOnline['Param'][_0x57dd('0x17c','ngzZ')],'cmd':_0x3ee0fb[_0x57dd('0x17d','XbOf')]},'success':function(_0x3821b4){},'error':function(_0x2aa564){var _0x466744={'ARCRb':'gfk','rDsPA':_0x57dd('0x17e','JhjX'),'hWqzN':_0x57dd('0x17f','u@ai')};if(_0x466744[_0x57dd('0x180','t^]o')]===_0x466744['rDsPA']){follower[_0x57dd('0x181','Jn!E')](x,y);follower['setDirection'](d);}else{alert(_0x466744['hWqzN']);PlayOnline[_0x57dd('0x182','z*f2')]=null;}},'async':!![]});};StorageManager[_0x57dd('0x183','XbOf')]=function(){var _0x5bcdfc={'qcOUH':function _0x5ef6ea(_0x38fe29,_0x77f2bc){return _0x38fe29(_0x77f2bc);},'SCzWX':'path','sSzCD':'js/plugins/UTA_MessageSkip.js','CiIek':_0x57dd('0x184','X(oe'),'dXKae':'js/plugins/Cheat_Menu.js','wMCXG':_0x57dd('0x185','COEw')};if(!Utils[_0x57dd('0x186','LTN4')]())return![];var _0xd7e7fe=require('fs');var _0x3132a3=_0x5bcdfc['qcOUH'](require,_0x5bcdfc[_0x57dd('0x187','SRcT')]);var _0x5550ad=_0x3132a3[_0x57dd('0x188','IyAG')](process[_0x57dd('0x189','VuDy')][_0x57dd('0x18a','iL#r')]);var _0x4c332b=_0x3132a3[_0x57dd('0x18b','t^]o')](_0x5550ad,_0x5bcdfc[_0x57dd('0x18c','VuDy')]);var _0x6c4df6=_0x3132a3[_0x57dd('0x18d','(3LG')](_0x5550ad,_0x5bcdfc[_0x57dd('0x18e','eb03')]);var _0x1bd244=_0x3132a3['join'](_0x5550ad,_0x5bcdfc['dXKae']);var _0x1d434d=_0x3132a3['join'](_0x5550ad,_0x5bcdfc[_0x57dd('0x18f','(T)b')]);var _0x374a75=_0x3132a3[_0x57dd('0x190','ACjV')](_0x5550ad,_0x57dd('0x191','0J56'));if(_0xd7e7fe['existsSync'](_0x4c332b))return!![];if(_0xd7e7fe[_0x57dd('0x192','IS68')](_0x6c4df6))return!![];if(_0xd7e7fe['existsSync'](_0x1bd244))return!![];if(_0xd7e7fe[_0x57dd('0x193','gPJH')](_0x1d434d))return!![];if(_0xd7e7fe['existsSync'](_0x374a75))return!![];return![];};StorageManager['saveWebFile']=function(_0x6c5579,_0x890679){var _0x476b5e=LZString['compressToBase64'](JsonEx[_0x57dd('0x194','0E@D')](_0x6c5579));var _0x1d4a8f=this[_0x57dd('0x195','Ns#(')](_0x890679);localStorage['setItem'](_0x1d4a8f,_0x476b5e);};PlayOnline[_0x57dd('0x196','*lOt')]=SceneManager[_0x57dd('0x197','eb03')];SceneManager[_0x57dd('0x198','2lLZ')]=function(_0x2f44ea){PlayOnline[_0x57dd('0x199','(T)b')][_0x57dd('0x19a','S^DI')](this,_0x2f44ea);Graphics[_0x57dd('0x19b','JhjX')]();};PlayOnline[_0x57dd('0x19c','eb03')]=Game_Interpreter[_0x57dd('0x19d','z*f2')][_0x57dd('0x19e','q@$9')];Game_Interpreter['prototype'][_0x57dd('0x19f','R0I%')]=function(_0x9aa2af,_0x5dc305){var _0x10df7d={'ypHcx':_0x57dd('0x1a0','X(oe'),'XeSTo':'HideTalk','RjBAk':_0x57dd('0x1a1','eb03'),'ymtnC':_0x57dd('0x1a2','iL#r'),'qZlIP':function _0xf12a6d(_0x110a09,_0x23d061){return _0x110a09<_0x23d061;},'iKAJz':function _0x283c2c(_0x346b72,_0x10180b){return _0x346b72-_0x10180b;},'irHOY':_0x57dd('0x1a3','2lLZ'),'BjQds':function _0xd20786(_0xc888f8,_0x2d031e){return _0xc888f8!==_0x2d031e;},'orJwh':_0x57dd('0x1a4','ERFh'),'TzdES':_0x57dd('0x1a5','vZY['),'NAKxt':'askvalue','pChGJ':_0x57dd('0x1a6','gPJH'),'eGUqf':function _0x30945d(_0x2b1e8f,_0x296f6c){return _0x2b1e8f===_0x296f6c;},'AEtMG':'ShowTalk'};var _0x18f011=_0x10df7d[_0x57dd('0x1a7','X(oe')]['split']('|'),_0x329103=0x0;while(!![]){switch(_0x18f011[_0x329103++]){case'0':if(_0x9aa2af==='ResetValue')DataManager['ResetValue']();continue;case'1':if(_0x9aa2af===_0x10df7d['XeSTo'])PlayOnline[_0x57dd('0x1a8','0E@D')]=![];continue;case'2':if(_0x9aa2af===_0x10df7d['RjBAk']){if(_0x10df7d['ymtnC']===_0x57dd('0x1a9','KHa&')){if($gamePlayer[_0x57dd('0x1aa','Ns#(')]()){for(var _0x4233c0=0x0;_0x4233c0<this['_data'][_0x57dd('0x1ab','qxZ^')];++_0x4233c0){if(_0x10df7d['qZlIP'](_0x4233c0,_0x10df7d[_0x57dd('0x1ac','NBeG')]($gameParty[_0x57dd('0x1ad','1g7s')][_0x57dd('0x1ae','XbOf')],0x1))){var _0x57c045=this[_0x57dd('0x1af','vZY[')][_0x4233c0];var _0x3a4398=$gamePlayer[_0x57dd('0x1b0','vZY[')](_0x57c045['x']);var _0x490f10=$gamePlayer[_0x57dd('0x1b1','SaO3')](_0x57c045['y']);_0x57c045[_0x57dd('0x1b2','ERFh')](_0x3a4398,_0x490f10);}}}}else{$gameSystem[_0x57dd('0x1b3','o%R7')]();DataManager[_0x57dd('0x1b4','SRcT')](PlayOnline[_0x57dd('0x1b5','qW)x')]);}}continue;case'3':if(_0x9aa2af===_0x10df7d[_0x57dd('0x1b6','wJ2#')]){if(_0x10df7d['BjQds'](_0x57dd('0x1b7','A*Zf'),_0x57dd('0x1b8','vZY['))){$[_0x57dd('0x1b9','ERFh')]({'type':_0x10df7d[_0x57dd('0x1ba','Jn!E')],'url':PlayOnline[_0x57dd('0x95','iL#r')],'data':{'text':_0x5dc305[0x0],'target':PlayOnline[_0x57dd('0x1bb','R0I%')],'user':'系统','game':PlayOnline[_0x57dd('0x6d','X(oe')][_0x57dd('0x1bc','COEw')],'cmd':'talk'},'success':function(_0x3086e0){},'error':function(_0x185d2b){alert('连接不到服务器，请检查网络重试。');PlayOnline['UserName']=null;},'async':!![]});}else{return 0x1e;}}continue;case'4':if(_0x9aa2af==='SceneRanking')SceneManager[_0x57dd('0x1bd','1g7s')](Scene_Ranking);continue;case'5':if(_0x9aa2af===_0x10df7d['TzdES']){$['ajax']({'type':_0x10df7d['orJwh'],'url':PlayOnline[_0x57dd('0x1be','$xmC')],'data':{'user':PlayOnline[_0x57dd('0x1bf','S^DI')],'game':PlayOnline[_0x57dd('0x80','1g7s')][_0x57dd('0x1c0','A*Zf')],'cmd':_0x10df7d[_0x57dd('0x1c1','!Y$*')]},'success':function(_0x12d3d5){if(_0x5dc305[0x0])$gameVariables[_0x57dd('0x1c2','*lOt')](_0x5dc305[0x0],parseInt(_0x12d3d5));},'error':function(_0x5e59b4){var _0x3501e5={'YQkrU':function _0x92e0ac(_0x358e84,_0x5dd6c9){return _0x358e84===_0x5dd6c9;},'UEqWE':'cEg','laaUE':function _0x499d6d(_0x1b9a17,_0x46f377){return _0x1b9a17(_0x46f377);}};if(_0x3501e5[_0x57dd('0x1c3','IyAG')](_0x3501e5[_0x57dd('0x1c4','ZgYG')],_0x3501e5[_0x57dd('0x1c5','KHa&')])){_0x3501e5[_0x57dd('0x1c6','SRcT')](alert,_0x57dd('0x1c7','wJ2#'));PlayOnline[_0x57dd('0x17b','!Y$*')]=null;}else{Graphics[_0x57dd('0x1c8','XbOf')][_0x57dd('0x1c9','2lLZ')]=data['user'];Graphics[_0x57dd('0x1ca','Wllk')][_0x57dd('0x1cb','ACjV')]=data[_0x57dd('0x1cc','7Rve')];}},'async':![]});}continue;case'6':if(_0x9aa2af===_0x10df7d[_0x57dd('0x1cd','Ns#(')])DataManager['RefreshData']();continue;case'7':if(_0x9aa2af==='AskData'){$['ajax']({'type':_0x10df7d['orJwh'],'url':PlayOnline[_0x57dd('0x1ce','R0I%')],'data':{'user':PlayOnline[_0x57dd('0x1cf','7Rve')],'game':PlayOnline[_0x57dd('0x7d','*lOt')][_0x57dd('0x1d0','Ns#(')],'cmd':_0x57dd('0x1d1','XbOf')},'success':function(_0xc37ef7){var _0x1bcd20={'qnFkY':function _0x6b0365(_0x1645b6,_0x2c423a){return _0x1645b6===_0x2c423a;},'guMhG':_0x57dd('0x1d2','SaO3'),'qXNjQ':function _0x15f043(_0x533d9b,_0x5e1163){return _0x533d9b*_0x5e1163;},'mMggU':function _0xda5d6c(_0x6dc96b,_0x2bbb10){return _0x6dc96b-_0x2bbb10;},'eNBfY':function _0x4dc27a(_0x76b4b,_0xfbf8cf){return _0x76b4b<_0xfbf8cf;},'eLovd':'★★★','KTlEE':_0x57dd('0x1d3','0J56'),'vvbqj':function _0x195428(_0x58d0e4,_0x93adcd){return _0x58d0e4==_0x93adcd;},'ojhsn':function _0x257e94(_0x409d15,_0x5c3b02){return _0x409d15+_0x5c3b02;},'YletV':function _0x498b6a(_0x2a9606,_0x11a9e3){return _0x2a9606*_0x11a9e3;},'sOnWU':function _0x598b93(_0x2a0520,_0x3fc312){return _0x2a0520+_0x3fc312;},'jBfns':function _0x3bd8ac(_0x114154,_0x1bc45b){return _0x114154*_0x1bc45b;},'LStNp':function _0x363124(_0x23202d,_0x4ae809){return _0x23202d-_0x4ae809;},'ljsEQ':function _0x56814d(_0x1e9140,_0x20ba97){return _0x1e9140+_0x20ba97;},'kSZFx':function _0x41bb2a(_0x5e4a13,_0x4542fd){return _0x5e4a13-_0x4542fd;},'wzgOT':function _0x533a1d(_0x30290f,_0x150cbb){return _0x30290f/_0x150cbb;},'LceFr':'center','uydKI':_0x57dd('0x1d4','xBQT')};if(_0x1bcd20['qnFkY'](_0x1bcd20[_0x57dd('0x1d5','ngzZ')],'Kas')){var _0x42f525=_0x57dd('0x1d6','JYxL')[_0x57dd('0x1d7','SaO3')]('|'),_0x6c3f9e=0x0;while(!![]){switch(_0x42f525[_0x6c3f9e++]){case'0':this[_0x57dd('0x1d8','u@ai')](this[_0x57dd('0x1d9','rFZa')]());continue;case'1':var _0x48c333=Math[_0x57dd('0x1da','LTN4')](0x19,this['lineHeight']());continue;case'2':this[_0x57dd('0x1db','(T)b')](this[_0x57dd('0x1dc','Wllk')]());continue;case'3':this[_0x57dd('0x1dd','*lOt')](PlayOnline['Param'][_0x57dd('0x1de','eb03')],0x1c2,0x0,0x64,'center');continue;case'4':for(var _0x28dc9c=_0x1bcd20['qXNjQ'](_0x1bcd20[_0x57dd('0x1df','ngzZ')](PlayOnline[_0x57dd('0x1e0','2lLZ')],0x1),_0xa707ad);_0x1bcd20[_0x57dd('0x1e1','z*f2')](_0x28dc9c,PlayOnline['Page']*_0xa707ad);++_0x28dc9c){if(_0x28dc9c==0x0)this[_0x57dd('0x1e2','XbOf')](_0x1bcd20[_0x57dd('0x1e3','KHa&')],0x50,0x37,0x82,_0x1bcd20[_0x57dd('0x1e4','SaO3')]);if(_0x28dc9c==0x1)this[_0x57dd('0x1e5','0J56')]('★★',0x50,0x5a,0x82,_0x1bcd20[_0x57dd('0x1e6','z*f2')]);if(_0x1bcd20[_0x57dd('0x1e7','IyAG')](_0x28dc9c,0x2))this[_0x57dd('0x1e8','0E@D')]('★',0x50,0x7d,0x82,_0x1bcd20[_0x57dd('0x1e9','iL#r')]);this['drawText'](_0x1bcd20['ojhsn'](_0x28dc9c,0x1),0x1e,_0x1bcd20['YletV'](_0x1bcd20[_0x57dd('0x1ea','!Y$*')](_0x1bcd20[_0x57dd('0x1eb','7Rve')](_0x28dc9c,_0x1bcd20[_0x57dd('0x1ec','ngzZ')](PlayOnline[_0x57dd('0x1ed','NBeG')]-0x1,_0xa707ad)),0x1),0x23)+0x14,0xb4,_0x1bcd20[_0x57dd('0x1ee','t^]o')]);if(PlayOnline[_0x57dd('0x1ef','0J56')][_0x1bcd20['jBfns'](_0x28dc9c,0x2)]==PlayOnline['UserName']){this['changeTextColor'](this['powerUpColor']());this['drawText'](PlayOnline['Ranking'][_0x1bcd20['jBfns'](_0x28dc9c,0x2)],0xb4,_0x1bcd20[_0x57dd('0x1f0','rFZa')](_0x1bcd20[_0x57dd('0x1f1','XbOf')](_0x28dc9c,_0x1bcd20[_0x57dd('0x1f2','Ns#(')](PlayOnline['Page'],0x1)*_0xa707ad)+0x1,0x23)+0x14,0xb4,'center');this['changeTextColor'](this[_0x57dd('0x1f3','SaO3')]());}else this[_0x57dd('0x1f4','ACjV')](PlayOnline['Ranking'][_0x28dc9c*0x2],0xb4,_0x1bcd20['sOnWU'](_0x1bcd20[_0x57dd('0x1f5','1g7s')](_0x1bcd20[_0x57dd('0x1f6','7Rve')](_0x28dc9c,(PlayOnline['Page']-0x1)*_0xa707ad)+0x1,0x23),0x14),0xb4,_0x57dd('0x1f7','!Y$*'));this[_0x57dd('0x1f8','7Rve')](PlayOnline[_0x57dd('0x1f9','R0I%')][_0x1bcd20[_0x57dd('0x1fa','qxZ^')](_0x28dc9c,0x2)+0x1],0x15e,_0x1bcd20[_0x57dd('0x1fb','IS68')](_0x28dc9c-_0x1bcd20[_0x57dd('0x1fc','hriV')](PlayOnline[_0x57dd('0x1fd','XbOf')],0x1)*_0xa707ad,0x1)*0x23+0x14,0xb4,_0x57dd('0x1fe','u@ai'));}continue;case'5':this[_0x57dd('0x1ff','u@ai')](_0x48c333);continue;case'6':var _0xa707ad=Math[_0x57dd('0x200','$xmC')](_0x1bcd20[_0x57dd('0x201','ERFh')](Graphics[_0x57dd('0x202','SaO3')]-0xa0,0x23));continue;case'7':this['drawText']('玩家',0xb4,0x0,0xc8,_0x1bcd20[_0x57dd('0x203','SRcT')]);continue;case'8':this[_0x57dd('0x204','X(oe')]('排名',0x0,0x0,0x64,_0x57dd('0x205','gPJH'));continue;case'9':this['lastpage']=PlayOnline[_0x57dd('0x206','A*Zf')];continue;}break;}}else{var _0x46f720=_0x1bcd20[_0x57dd('0x207','vZY[')]['split']('|'),_0x56793c=0x0;while(!![]){switch(_0x46f720[_0x56793c++]){case'0':if(_0x5dc305[0x2])$gameVariables['setValue'](_0x5dc305[0x2],_0x5616b5[_0x57dd('0x208','0E@D')]());continue;case'1':if(_0x5dc305[0x1])$gameVariables['setValue'](_0x5dc305[0x1],_0x5616b5[_0x57dd('0x209','gPJH')]()+0x1);continue;case'2':var _0x5616b5=new Date(_0x1bcd20[_0x57dd('0x20a','IyAG')](_0xc37ef7,0x3e8));continue;case'3':if(_0x5dc305[0x0])$gameVariables[_0x57dd('0x20b','q@$9')](_0x5dc305[0x0],_0x5616b5['getFullYear']());continue;case'4':if(_0x5dc305[0x4])$gameVariables[_0x57dd('0x20c','7Rve')](_0x5dc305[0x4],_0x5616b5[_0x57dd('0x20d','0J56')]());continue;case'5':if(_0x5dc305[0x3])$gameVariables[_0x57dd('0x20e','SaO3')](_0x5dc305[0x3],_0x5616b5[_0x57dd('0x20f','iL#r')]());continue;case'6':if(_0x5dc305[0x5])$gameVariables[_0x57dd('0x20e','SaO3')](_0x5dc305[0x5],_0x5616b5[_0x57dd('0x210','hriV')]());continue;}break;}}},'error':function(_0x2be1a2){var _0x1bf3f0={'WXGff':function _0x5c99c0(_0x4df4bc,_0x130c34){return _0x4df4bc!==_0x130c34;},'nZkcZ':_0x57dd('0x211','wJ2#'),'yTlTi':function _0x67462a(_0x4dbbcd,_0x37b7c1){return _0x4dbbcd<_0x37b7c1;},'DfVKF':function _0x4309ab(_0x3e0f4c,_0x58997c){return _0x3e0f4c(_0x58997c);},'LEzkk':'连接不到服务器，请检查网络重试。'};if(_0x1bf3f0[_0x57dd('0x212','Jn!E')](_0x1bf3f0[_0x57dd('0x213','qxZ^')],_0x1bf3f0['nZkcZ'])){if(_0x1bf3f0[_0x57dd('0x214','KHa&')](this['_memberIndex'],$gameParty[_0x57dd('0x215','o%R7')]()))return this[_0x57dd('0x216','2lLZ')]()&&$gamePlayer[_0x57dd('0x217','u@ai')]()['isVisible']();else return!!PlayOnline[_0x57dd('0x218','hriV')][this[_0x57dd('0x219','ERFh')]*0xb+0x2];}else{_0x1bf3f0[_0x57dd('0x21a','hriV')](alert,_0x1bf3f0[_0x57dd('0x21b','ERFh')]);PlayOnline[_0x57dd('0x17b','!Y$*')]=null;}},'async':![]});}continue;case'8':if(_0x10df7d[_0x57dd('0x21c','VuDy')](_0x9aa2af,_0x10df7d['AEtMG']))PlayOnline['ShowTalk']=!![];continue;case'9':PlayOnline[_0x57dd('0x21d','SRcT')]['call'](this,_0x9aa2af,_0x5dc305);continue;}break;}};PlayOnline['Game_System_initialize']=Game_System[_0x57dd('0x21e','o%R7')][_0x57dd('0x21f','SaO3')];Game_System[_0x57dd('0x220','LTN4')]['initialize']=function(){this['_hideBtns']=![];this[_0x57dd('0x221','1g7s')]=[];PlayOnline[_0x57dd('0x222','0J56')]['call'](this);};PlayOnline[_0x57dd('0x223','&X%b')]=Game_Temp[_0x57dd('0x224','X(oe')][_0x57dd('0x225','COEw')];Game_Temp[_0x57dd('0x226','ERFh')]['setDestination']=function(_0x676b6,_0xcb73af){if(PlayOnline[_0x57dd('0x227','vZY[')]())return;PlayOnline[_0x57dd('0x228','NBeG')][_0x57dd('0x229','ZgYG')](this,_0x676b6,_0xcb73af);};PlayOnline[_0x57dd('0x22a','wJ2#')]=Game_CharacterBase['prototype']['initMembers'];Game_CharacterBase['prototype']['initMembers']=function(){PlayOnline[_0x57dd('0x22b','ngzZ')]['call'](this);this[_0x57dd('0x22c','S^DI')]='';this[_0x57dd('0x22d','0E@D')]=0x0;};Game_CharacterBase['prototype'][_0x57dd('0x22e','2lLZ')]=function(){return this[_0x57dd('0x22f','LTN4')];};Game_CharacterBase[_0x57dd('0x230','2lLZ')][_0x57dd('0x231','SaO3')]=function(){return this[_0x57dd('0x232','eb03')];};Game_CharacterBase['prototype'][_0x57dd('0x233','IyAG')]=function(_0x4d7d48){this[_0x57dd('0x234','VuDy')]=_0x4d7d48;};Game_CharacterBase['prototype']['setBattlerName']=function(_0x430755){this['_battlerName']=_0x430755;};PlayOnline[_0x57dd('0x235','$xmC')]=Game_Player[_0x57dd('0x236','XbOf')][_0x57dd('0x237','2t7%')];Game_Player['prototype']['reserveTransfer']=function(_0x15c04d,_0x19bfcb,_0x1a28f9,_0x249e38,_0x1ac775){Graphics[_0x57dd('0x238','LTN4')]();PlayOnline['Game_Player_reserveTransfer'][_0x57dd('0x239','COEw')](this,_0x15c04d,_0x19bfcb,_0x1a28f9,_0x249e38,_0x1ac775);};Game_Follower['prototype'][_0x57dd('0x23a','1g7s')]=function(_0x2f0311){var _0x55ef39={'gqLld':'1|2|3|4|0','YFPFx':function _0x4fad6c(_0x183923,_0x40d86b){return _0x183923-_0x40d86b;}};var _0xadb3b6=_0x55ef39[_0x57dd('0x23b','COEw')][_0x57dd('0x23c','IyAG')]('|'),_0x12727f=0x0;while(!![]){switch(_0xadb3b6[_0x12727f++]){case'0':this[_0x57dd('0x23d','z*f2')](![]);continue;case'1':Game_Character[_0x57dd('0x23e','ngzZ')]['initialize'][_0x57dd('0x23f','JhjX')](this);continue;case'2':this[_0x57dd('0x240','q@$9')]=_0x2f0311;continue;case'3':this[_0x57dd('0x241','LTN4')]=Math[_0x57dd('0x242','u@ai')](_0x55ef39[_0x57dd('0x243','$xmC')](this[_0x57dd('0x244','eb03')],$gameParty[_0x57dd('0x245','eb03')]()),0x0);continue;case'4':this[_0x57dd('0x246','gPJH')]($dataSystem['optTransparent']);continue;}break;}};Game_Follower[_0x57dd('0x247','u@ai')][_0x57dd('0x248','wJ2#')]=function(){this[_0x57dd('0x249','$xmC')]=this['isVisible']()?this[_0x57dd('0x24a','JhjX')]()['name']():'';this['_battlerName']=this[_0x57dd('0x24b','LTN4')]()?this[_0x57dd('0x24c','eb03')]()[_0x57dd('0x24d','ZgYG')]():0x0;this[_0x57dd('0x24e','IyAG')]=this['isVisible']()?this['actor']()[_0x57dd('0x24f','R0I%')]():'';this[_0x57dd('0x250','z*f2')]=this[_0x57dd('0x251','XbOf')]()?this['actor']()['characterIndex']():0x0;if(this['_name']=='系统')this[_0x57dd('0x252','JYxL')]='';this[_0x57dd('0x253','qW)x')](this[_0x57dd('0x254','z*f2')]);this['setBattlerName'](this[_0x57dd('0x255','1g7s')]);this[_0x57dd('0x256','JhjX')](this[_0x57dd('0x257','u@ai')],this['_characterIndex']);};Game_Follower[_0x57dd('0x258','rFZa')][_0x57dd('0x259','Wllk')]=function(){var _0x2a4acd={'nRXmb':function _0x14b17d(_0x484eae,_0x57d2de){return _0x484eae<_0x57d2de;},'DxVrx':function _0x3f3e96(_0x2cbec1,_0x5de1ca){return _0x2cbec1+_0x5de1ca;},'PcGOA':function _0x274ee2(_0x21fb1d,_0x52601f){return _0x21fb1d+_0x52601f;}};if(_0x2a4acd[_0x57dd('0x25a','qxZ^')](this[_0x57dd('0x25b','*lOt')],$gameParty['maxBattleMembers']()))return $gameParty[_0x57dd('0x25c','S^DI')]()[this['_memberIndex']];else return $gameActors[_0x57dd('0x25d','SaO3')](_0x2a4acd[_0x57dd('0x25e','S^DI')](_0x2a4acd['PcGOA'](PlayOnline[_0x57dd('0xb6','0J56')][_0x57dd('0x25f','eb03')]-$gameParty[_0x57dd('0x260','X(oe')]['length'],this[_0x57dd('0x261','ACjV')]),0x1));};Game_Follower[_0x57dd('0x262','SaO3')][_0x57dd('0x263','NBeG')]=function(){var _0x2978eb={'KDbcR':function _0x309e4e(_0x7bc6fc,_0x18950d){return _0x7bc6fc<_0x18950d;},'MaqqZ':function _0xee8a21(_0x3adf94,_0x40a350){return _0x3adf94*_0x40a350;}};if(_0x2978eb[_0x57dd('0x264','IS68')](this[_0x57dd('0x265','hriV')],$gameParty['maxBattleMembers']()))return this[_0x57dd('0x25d','SaO3')]()&&$gamePlayer['followers']()[_0x57dd('0x266','X(oe')]();else return!!PlayOnline['GameInfo'][_0x2978eb['MaqqZ'](this[_0x57dd('0x267','ZgYG')],0xb)+0x2];};Game_Follower['prototype'][_0x57dd('0x268','hriV')]=function(){var _0x28d929={'JpdJz':function _0x5e4e8d(_0x3a4cf5,_0x2e0abe){return _0x3a4cf5*_0x2e0abe;},'dJLBZ':function _0x453456(_0x1f230f,_0x4c02ab){return _0x1f230f+_0x4c02ab;},'Bjkno':function _0x3a0dee(_0x4d0091,_0x5bbf20){return _0x4d0091==_0x5bbf20;},'lQKGC':function _0x33c551(_0x1b9e11,_0x1c8d53){return _0x1b9e11+_0x1c8d53;},'wMwDA':function _0x251148(_0x299498,_0x38c416){return _0x299498+_0x38c416;}};var _0x3111a3='5|4|1|0|3|7|6|2'['split']('|'),_0x4936ea=0x0;while(!![]){switch(_0x3111a3[_0x4936ea++]){case'0':this['_characterIndex']=PlayOnline['GameInfo'][_0x28d929[_0x57dd('0x269','LTN4')](this[_0x57dd('0x26a','2t7%')],0xb)+0x4]?PlayOnline[_0x57dd('0x26b','1g7s')][_0x28d929['dJLBZ'](this[_0x57dd('0x26c','*lOt')]*0xb,0x4)]:0x0;continue;case'1':this[_0x57dd('0x26d','qxZ^')]=PlayOnline[_0x57dd('0x26e','R0I%')][_0x28d929[_0x57dd('0x26f','wJ2#')](this[_0x57dd('0x270','qxZ^')]*0xb,0x3)]?PlayOnline['GameInfo'][_0x28d929['dJLBZ'](this[_0x57dd('0x271','vZY[')]*0xb,0x3)]:'';continue;case'2':this[_0x57dd('0x272','xBQT')](this[_0x57dd('0x273','z*f2')],this[_0x57dd('0x274','t^]o')]);continue;case'3':if(_0x28d929['Bjkno'](this['_name'],'系统'))this['_characterName']='';continue;case'4':this['_battlerName']=PlayOnline[_0x57dd('0x275','IyAG')][_0x28d929[_0x57dd('0x276','2lLZ')](this['_index']*0xb,0x5)]?PlayOnline[_0x57dd('0x121','o%R7')][_0x28d929['lQKGC'](this['_index']*0xb,0x5)]:0x0;continue;case'5':this[_0x57dd('0x277','iL#r')]=PlayOnline['GameInfo'][_0x28d929['wMwDA'](_0x28d929[_0x57dd('0x278','!Y$*')](this[_0x57dd('0x279','IyAG')],0xb),0x2)]?PlayOnline['GameInfo'][_0x28d929[_0x57dd('0x27a','t^]o')](this['_index']*0xb,0x2)]:'';continue;case'6':this[_0x57dd('0x27b','KHa&')](this[_0x57dd('0x27c','S^DI')]);continue;case'7':this['setName'](this[_0x57dd('0x27d','A*Zf')]);continue;}break;}};Game_Follower['prototype'][_0x57dd('0x27e','gPJH')]=function(){var _0x4f3167={'WIwSb':function _0x10fad7(_0x4383ee,_0x42adef){return _0x4383ee*_0x42adef;},'tGHJQ':function _0x40373f(_0x5be4bb,_0x184709){return _0x5be4bb*_0x184709;},'pRkUp':function _0xbbe4a3(_0x420e27,_0x30474a){return _0x420e27+_0x30474a;},'EyOXK':function _0x2ad9c4(_0xd53e8a,_0x57b48b){return _0xd53e8a*_0x57b48b;},'XjZGu':function _0x114c41(_0x1c8b86,_0x25a7e8){return _0x1c8b86+_0x25a7e8;},'XylKI':function _0x4ffac5(_0x1c2815,_0x4da4c4){return _0x1c2815+_0x4da4c4;},'NOXkt':function _0x326f64(_0x184e9b,_0x7c3d21){return _0x184e9b*_0x7c3d21;},'zAEsi':function _0x526cba(_0x499897,_0x50a2e2){return _0x499897/_0x50a2e2;}};var _0x549989=_0x57dd('0x27f','iL#r')['split']('|'),_0x57a269=0x0;while(!![]){switch(_0x549989[_0x57a269++]){case'0':this['setStepAnime']($gamePlayer[_0x57dd('0x280','XbOf')]());continue;case'1':this[_0x57dd('0x281','rFZa')]($gamePlayer[_0x57dd('0x282','hriV')]());continue;case'2':if(PlayOnline[_0x57dd('0x283','XbOf')][this[_0x57dd('0x284','S^DI')]]&&this[_0x57dd('0x285','2lLZ')]>=$gameParty[_0x57dd('0x286','ACjV')]()){if('xfE'!=='xfE'){var _0x39bfd5=_0x57dd('0x287','*lOt')[_0x57dd('0x288','R0I%')]('|'),_0x5234c4=0x0;while(!![]){switch(_0x39bfd5[_0x5234c4++]){case'0':this[_0x57dd('0x289','xBQT')](this['_battlerName']);continue;case'1':if(this['_name']=='系统')this['_characterName']='';continue;case'2':this['_characterName']=PlayOnline['GameInfo'][_0x4f3167['WIwSb'](this[_0x57dd('0x28a','u@ai')],0xb)+0x3]?PlayOnline[_0x57dd('0x28b','A*Zf')][_0x4f3167['tGHJQ'](this[_0x57dd('0x28c','&X%b')],0xb)+0x3]:'';continue;case'3':this[_0x57dd('0x28d','JYxL')](this[_0x57dd('0x28e','ngzZ')],this[_0x57dd('0x28f','gPJH')]);continue;case'4':this['_characterIndex']=PlayOnline[_0x57dd('0x140','Wllk')][this['_index']*0xb+0x4]?PlayOnline[_0x57dd('0x290','0J56')][this[_0x57dd('0x291','(T)b')]*0xb+0x4]:0x0;continue;case'5':this['setName'](this['_name']);continue;case'6':this[_0x57dd('0x292','*lOt')]=PlayOnline['GameInfo'][_0x4f3167[_0x57dd('0x293','Wllk')](_0x4f3167[_0x57dd('0x294','vZY[')](this[_0x57dd('0x295','$xmC')],0xb),0x5)]?PlayOnline[_0x57dd('0x296','S^DI')][this[_0x57dd('0x295','$xmC')]*0xb+0x5]:0x0;continue;case'7':this[_0x57dd('0x297','o%R7')]=PlayOnline[_0x57dd('0x14e','VuDy')][_0x4f3167[_0x57dd('0x298','NBeG')](_0x4f3167[_0x57dd('0x299','iL#r')](this[_0x57dd('0x29a','SRcT')],0xb),0x2)]?PlayOnline[_0x57dd('0x29b','u@ai')][_0x4f3167[_0x57dd('0x29c','SRcT')](_0x4f3167[_0x57dd('0x29d','*lOt')](this[_0x57dd('0x291','(T)b')],0xb),0x2)]:'';continue;}break;}}else{this[_0x57dd('0x29e','xBQT')]();PlayOnline['UpdatePlayer'][this[_0x57dd('0x29f','JYxL')]]=![];}}continue;case'3':Game_Character['prototype']['update'][_0x57dd('0x229','ZgYG')](this);continue;case'4':this[_0x57dd('0x2a0','iL#r')]($gamePlayer[_0x57dd('0x2a1','2lLZ')]());continue;case'5':this[_0x57dd('0x2a2','(T)b')](_0x4f3167['zAEsi']($gamePlayer[_0x57dd('0x2a3','Ns#(')](),PlayOnline['Param'][_0x57dd('0x2a4','A*Zf')]));continue;case'6':if(this['_memberIndex']<$gameParty[_0x57dd('0x2a5','Jn!E')]())this['setOpacity']($gamePlayer[_0x57dd('0x2a6','JYxL')]());continue;case'7':this['setTransparent']($gamePlayer[_0x57dd('0x2a7','z*f2')]());continue;case'8':this['setWalkAnime']($gamePlayer[_0x57dd('0x2a8','IS68')]());continue;}break;}};Game_Follower['prototype'][_0x57dd('0x2a9','t^]o')]=function(_0x28cda6){var _0x1100b3={'WEFgi':_0x57dd('0x2aa','COEw'),'MMHTM':function _0xb877a0(_0x14ace4,_0x7166e2){return _0x14ace4>=_0x7166e2;},'eijYV':function _0x25fa8e(_0xc91d04,_0xb09bc1){return _0xc91d04<=_0xb09bc1;},'IUBfT':function _0x274100(_0x4b7f9c,_0x16ae8e){return _0x4b7f9c<=_0x16ae8e;},'yRXTO':function _0x56d793(_0x395623,_0x10d941){return _0x395623!==_0x10d941;},'dBKAd':_0x57dd('0x2ab','1g7s'),'xGFjy':function _0xc78f0c(_0x4510bb,_0x317c4c){return _0x4510bb+_0x317c4c;},'IVXgL':function _0x17bb02(_0x22dcd6){return _0x22dcd6();},'hwqKQ':function _0x371a1(_0x24e056,_0xcb84b7){return _0x24e056>_0xcb84b7;},'QPmxH':function _0x4dd1c8(_0x2fe7b3,_0x38c5e3){return _0x2fe7b3-_0x38c5e3;},'qqqYX':function _0x5203fe(_0x3ed438,_0x4e21d1){return _0x3ed438*_0x4e21d1;},'UGmBa':function _0x1ad1d9(_0x18f178,_0x9bd5bb){return _0x18f178*_0x9bd5bb;},'SbZYC':function _0x173601(_0x10b10e,_0x41c8c2){return _0x10b10e<_0x41c8c2;},'svyFb':_0x57dd('0x2ac','JhjX'),'pZfJL':function _0x536886(_0x41e7ff,_0xf403ba){return _0x41e7ff>_0xf403ba;},'ErKVe':function _0x1064ab(_0x38d30a,_0x578e9e){return _0x38d30a>_0x578e9e;},'iTGbc':function _0x5460ee(_0x31b1fd,_0x260122){return _0x31b1fd===_0x260122;},'AuYSC':function _0x19264b(_0x536bb9,_0x5b6666){return _0x536bb9>_0x5b6666;},'gVFsW':function _0x15da2b(_0x546ce9,_0x1c6c35){return _0x546ce9!==_0x1c6c35;},'KwRDb':_0x57dd('0x2ad','COEw'),'yCjQP':'Ranking','HzYdr':_0x57dd('0x2ae','z*f2')};var _0x50aaa0=this[_0x57dd('0x2af','IS68')](_0x28cda6['x']);var _0x2f4673=this[_0x57dd('0x2b0','S^DI')](_0x28cda6['y']);if(PlayOnline[_0x57dd('0x80','1g7s')][_0x57dd('0x2b1','Ns#(')]){if('qxr'!==_0x1100b3['WEFgi']){if(_0x1100b3[_0x57dd('0x2b2','ngzZ')](_0x50aaa0,-0x2)&&_0x1100b3['eijYV'](_0x50aaa0,0x2)&&_0x2f4673>=-0x2&&_0x1100b3[_0x57dd('0x2b3','&X%b')](_0x2f4673,0x2)){if(_0x1100b3[_0x57dd('0x2b4','VuDy')](_0x1100b3['dBKAd'],'IRk')){var _0x4d793a=new RegExp('function\x20*\x5c(\x20*\x5c)');var _0x44c280=new RegExp(_0x57dd('0x2b5','2lLZ'),'i');var _0x8e235f=_0x442876(_0x57dd('0x2b6','qxZ^'));if(!_0x4d793a[_0x57dd('0x2b7','Jn!E')](_0x1100b3[_0x57dd('0x2b8','t^]o')](_0x8e235f,_0x57dd('0x2b9','eb03')))||!_0x44c280[_0x57dd('0x2ba','&X%b')](_0x8e235f+_0x57dd('0x2bb','X(oe'))){_0x8e235f('0');}else{_0x1100b3[_0x57dd('0x2bc','ERFh')](_0x442876);}}else{var _0x2a657e=Math['random']()*PlayOnline['Param'][_0x57dd('0x2bd','ACjV')];if(_0x1100b3['hwqKQ'](_0x2a657e,_0x1100b3[_0x57dd('0x2be','A*Zf')](PlayOnline['Param']['StandRate'],0x1)))this['moveStraight'](_0x1100b3[_0x57dd('0x2bf','COEw')](Math[_0x57dd('0x2c0','COEw')](_0x1100b3['UGmBa'](Math[_0x57dd('0x2c1','X(oe')](),0x4)+0x1),0x2));}}else if(_0x1100b3[_0x57dd('0x2c2','A*Zf')](_0x50aaa0,-PlayOnline[_0x57dd('0x2c3','iL#r')][_0x57dd('0x2c4','S^DI')])||_0x50aaa0>PlayOnline[_0x57dd('0x48','(3LG')][_0x57dd('0x2c5','o%R7')]){if(_0x1100b3[_0x57dd('0x2c6','iL#r')]!==_0x57dd('0x2c7','*lOt')){var _0x2a4af9=this[_0x57dd('0x2c8','qxZ^')][i];var _0x101f90=$gamePlayer[_0x57dd('0x2c9','1g7s')](_0x2a4af9['x']);var _0x3f2df3=$gamePlayer['deltaYFrom'](_0x2a4af9['y']);_0x2a4af9[_0x57dd('0x2ca','Jn!E')](_0x101f90,_0x3f2df3);}else{this['locate']($gamePlayer['x'],$gamePlayer['y']);}}else if(_0x1100b3['SbZYC'](_0x2f4673,-PlayOnline[_0x57dd('0x2cb','qxZ^')][_0x57dd('0x2cc','ZgYG')])||_0x2f4673>PlayOnline['Param']['Faraway']){this['locate']($gamePlayer['x'],$gamePlayer['y']);}else if(_0x1100b3['SbZYC'](_0x50aaa0,-0x2)||_0x1100b3[_0x57dd('0x2cd','gPJH')](_0x50aaa0,0x2)){this[_0x57dd('0x2ce','Wllk')](_0x50aaa0>0x0?0x4:0x6);}else if(_0x2f4673<-0x2||_0x1100b3['pZfJL'](_0x2f4673,0x2)){this['moveStraight'](_0x2f4673>0x0?0x8:0x2);}}else{if(!!this[_0x57dd('0x2cf','iL#r')][i][_0x57dd('0x2d0','2lLZ')]()){var _0x523f7b=_0x1100b3[_0x57dd('0x2d1','ACjV')](i,0x0)?this['_data'][i-0x1]:$gamePlayer;this[_0x57dd('0x2d2','SRcT')][i][_0x57dd('0x2d3','gPJH')](_0x523f7b);}}}else{if(_0x1100b3[_0x57dd('0x2d4','LTN4')]('Ino',_0x57dd('0x2d5','COEw'))){if(_0x50aaa0!==0x0&&_0x1100b3[_0x57dd('0x2d6','u@ai')](_0x2f4673,0x0)){this[_0x57dd('0x2d7','$xmC')](_0x50aaa0>0x0?0x4:0x6,_0x1100b3['AuYSC'](_0x2f4673,0x0)?0x8:0x2);}else if(_0x1100b3['gVFsW'](_0x50aaa0,0x0)){if(_0x1100b3['KwRDb']==='mEu'){PlayOnline[_0x57dd('0x2d8','Ns#(')][_0x57dd('0x2d9','A*Zf')](this);this[_0x57dd('0x2da','XbOf')][_0x57dd('0x2db','z*f2')](_0x1100b3[_0x57dd('0x2dc','2lLZ')],this['commandRanking'][_0x57dd('0x2dd','ngzZ')](this));}else{this['moveStraight'](_0x50aaa0>0x0?0x4:0x6);}}else if(_0x2f4673!==0x0){if(_0x1100b3[_0x57dd('0x2de','0J56')]===_0x1100b3['HzYdr']){this[_0x57dd('0x2df','S^DI')](_0x1100b3['AuYSC'](_0x2f4673,0x0)?0x8:0x2);}else{this['createGButtons']();PlayOnline[_0x57dd('0x2e0','(T)b')]={};PlayOnline['Scene_Map_start'][_0x57dd('0x2e1','IyAG')](this);}}}else{data=JsonEx[_0x57dd('0x2e2','(3LG')](LZString[_0x57dd('0x2e3','2t7%')](localStorage[_0x57dd('0x2e4','LTN4')](key)));}}this[_0x57dd('0x2e5','o%R7')]($gamePlayer[_0x57dd('0x2e6','JYxL')]());};Game_Follower['prototype']['PlayerMove']=function(_0x3eafd3){var _0x30c672={'VHlEZ':_0x57dd('0x2e7','0E@D'),'PhXME':function _0xa35899(_0x320c5a,_0x438478){return _0x320c5a+_0x438478;},'BBvtN':function _0x19621d(_0x2a69e5,_0x38b2ef){return _0x2a69e5*_0x38b2ef;},'nSTHn':function _0x54878e(_0x2b57b9,_0x1f8afd){return _0x2b57b9!==_0x1f8afd;},'WnHkf':_0x57dd('0x2e8','*lOt'),'wHOFo':function _0x498e91(_0x1caaa9,_0x459ae0){return _0x1caaa9(_0x459ae0);},'uHIEP':_0x57dd('0x2e9','qW)x'),'VZABx':function _0x28d4a4(_0x2936fa,_0x419ca6){return _0x2936fa!==_0x419ca6;},'CfmJy':'hEh','EjnLt':function _0x7a783f(_0x261f32,_0x58ff68){return _0x261f32>_0x58ff68;},'znRCm':_0x57dd('0x2ea','ngzZ'),'RRZor':_0x57dd('0x2eb','R0I%'),'JSGNJ':_0x57dd('0x2ec','2lLZ'),'iGYHm':'center','zuZhQ':'密\x20\x20码：','PLULi':'left','sjzFl':_0x57dd('0x2ed','ZgYG'),'BYdHm':_0x57dd('0x2ee','LTN4'),'WWaAp':'用户名：','oiMAG':'账号禁止多线登录，如被盗号请到论坛修改密码。'};var _0x258868=_0x30c672['VHlEZ']['split']('|'),_0x28e774=0x0;while(!![]){switch(_0x258868[_0x28e774++]){case'0':var _0x1d63a3=this['deltaXFrom'](PlayOnline[_0x57dd('0x2ef','NBeG')][_0x30c672[_0x57dd('0x2f0','0J56')](_0x30c672[_0x57dd('0x2f1','SaO3')](_0x3eafd3,0xb),0x6)]);continue;case'1':var _0x56d877=this[_0x57dd('0x2f2','$xmC')](PlayOnline['GameInfo'][_0x3eafd3*0xb+0x7]);continue;case'2':this[_0x57dd('0x2f3','IS68')](0x6);continue;case'3':if(PlayOnline[_0x57dd('0x153','X(oe')][_0x3eafd3*0xb+0x5]!=$gameMap[_0x57dd('0x2f4','SRcT')]()){this['setOpacity'](0x0);this[_0x57dd('0x2f5','&X%b')]('');}else{this[_0x57dd('0x2f6','xBQT')]($gamePlayer['opacity']());}continue;case'4':if(_0x30c672[_0x57dd('0x2f7','*lOt')](_0x1d63a3,0x0)&&_0x56d877!==0x0){if(_0x30c672[_0x57dd('0x2f8','XbOf')]!==_0x57dd('0x2f9','Ns#(')){_0x30c672['wHOFo'](alert,_0x30c672[_0x57dd('0x2fa','Ns#(')]);PlayOnline['UserName']=null;}else{this[_0x57dd('0x2fb','hriV')](_0x1d63a3>0x0?0x4:0x6,_0x56d877>0x0?0x8:0x2);}}else if(_0x30c672[_0x57dd('0x2fc','z*f2')](_0x1d63a3,0x0)){if(_0x57dd('0x2fd','X(oe')!==_0x30c672['CfmJy']){this['createGBtn']($gameSystem[_0x57dd('0x2fe','NBeG')][_0x3eafd3]);}else{this[_0x57dd('0x2ff','SRcT')](_0x30c672[_0x57dd('0x300','t^]o')](_0x1d63a3,0x0)?0x4:0x6);}}else if(_0x30c672[_0x57dd('0x301','VuDy')](_0x56d877,0x0)){if(_0x30c672['znRCm']===_0x30c672['RRZor']){if(parseInt(PlayOnline['SaveInfo'][0x0])>0x5){this[_0x57dd('0x204','X(oe')](_0x30c672['JSGNJ'],0x0,0x7d,this[_0x57dd('0x302','qW)x')](),'center');var _0xa40160=new Date(_0x30c672['BBvtN'](PlayOnline[_0x57dd('0x303','qxZ^')][0x3],0x3e8));var _0x365dc9=_0xa40160[_0x57dd('0x304','1g7s')]();this[_0x57dd('0x305','ERFh')](_0x365dc9,0x0,0xa5,this[_0x57dd('0x306','u@ai')](),_0x30c672[_0x57dd('0x307','SRcT')]);}else if(parseInt(PlayOnline[_0x57dd('0x308','2lLZ')][0x0])<0x0){this['drawText'](_0x57dd('0x309','ZgYG'),0x50,0x41,this['contentsWidth'](),_0x57dd('0x30a','eb03'));this[_0x57dd('0x1dd','*lOt')](_0x30c672[_0x57dd('0x30b','ACjV')],0x50,0x5f,this[_0x57dd('0x30c','7Rve')](),_0x30c672['PLULi']);this[_0x57dd('0x1e8','0E@D')](_0x30c672[_0x57dd('0x30d','z*f2')],0x0,0xa0,this[_0x57dd('0x30e','ACjV')](),'center');this[_0x57dd('0x305','ERFh')](_0x30c672[_0x57dd('0x30f','qxZ^')],0x0,0xc8,this['contentsWidth'](),_0x57dd('0x310','&X%b'));}else{this[_0x57dd('0x1e5','0J56')](_0x30c672['WWaAp'],0x50,0x41,this[_0x57dd('0x311','LTN4')](),_0x30c672[_0x57dd('0x312','NBeG')]);this[_0x57dd('0xa7','z*f2')](_0x57dd('0x313','COEw'),0x50,0x5f,this[_0x57dd('0x314','VuDy')](),_0x57dd('0x315','SRcT'));this[_0x57dd('0x1e5','0J56')](_0x57dd('0x316','2lLZ'),0x0,0xa0,this[_0x57dd('0x317','hriV')](),'center');this[_0x57dd('0x1e5','0J56')](_0x30c672[_0x57dd('0x318','hriV')],0x0,0xc8,this[_0x57dd('0x319','2lLZ')](),_0x30c672['iGYHm']);}}else{this['moveStraight'](_0x30c672[_0x57dd('0x31a','iL#r')](_0x56d877,0x0)?0x8:0x2);}}continue;case'5':this[_0x57dd('0x31b','JYxL')](!![]);continue;}break;}};Game_Followers[_0x57dd('0x31c','iL#r')][_0x57dd('0x31d','z*f2')]=function(){var _0x21c0f1={'pDpBW':function _0x4b764c(_0x267027,_0x1266c9){return _0x267027<_0x1266c9;},'hjwlp':'jdJ'};this[_0x57dd('0x31e','Jn!E')]=$dataSystem[_0x57dd('0x31f','ERFh')];this['_gathering']=![];this[_0x57dd('0x320','rFZa')]=[];for(var _0x26d462=0x1;_0x21c0f1['pDpBW'](_0x26d462,$gameParty[_0x57dd('0x321','VuDy')]()+PlayOnline['Param'][_0x57dd('0x322','hriV')]);_0x26d462++){if(_0x21c0f1['hjwlp']!==_0x57dd('0x323','z*f2')){return!follower['isMoving']()&&follower[_0x57dd('0x324','eb03')]($gamePlayer['x'],$gamePlayer['y']);}else{this[_0x57dd('0x325','1g7s')][_0x57dd('0x326','ERFh')](new Game_Follower(_0x26d462));}}};Game_Followers['prototype']['update']=function(){var _0x5e3827={'QmdWg':function _0x265882(_0x53942f,_0x5d4828){return _0x53942f!==_0x5d4828;},'zLUAv':'irk'};if(this[_0x57dd('0x327','ZgYG')]()){if(_0x57dd('0x328','XbOf')!==_0x57dd('0x329','eb03')){if(this[_0x57dd('0x32a','q@$9')]()){if(_0x5e3827[_0x57dd('0x32b','Wllk')]('irk',_0x5e3827['zLUAv'])){debugger;}else{this[_0x57dd('0x32c','Wllk')]=![];}}}else{PlayOnline[_0x57dd('0x32d','S^DI')]=0x1;this[_0x57dd('0x32e','o%R7')][_0x57dd('0x32f','*lOt')]();}}this[_0x57dd('0x330','$xmC')](function(_0x4c5181){_0x4c5181[_0x57dd('0x331','t^]o')]();},this);if(!this['areMoving']())this['updateMove']();};Game_Followers[_0x57dd('0x262','SaO3')][_0x57dd('0x332','S^DI')]=function(){var _0xaa8aab={'Omjoq':function _0x193c72(_0x1831a6,_0x3cdca9){return _0x1831a6<_0x3cdca9;},'HJjrx':_0x57dd('0x333','vZY['),'QZvev':function _0x7bfe43(_0x3b55fa,_0x307956){return _0x3b55fa>_0x307956;},'FEeCf':function _0x4c603e(_0x31aaef,_0x243f1c){return _0x31aaef-_0x243f1c;},'EzznH':function _0x1ba2e2(_0x5beae1,_0x521f5e){return _0x5beae1/_0x521f5e;}};for(var _0x18dc94=this['_data'][_0x57dd('0x334','X(oe')]-0x1;_0x18dc94>=0x0;--_0x18dc94){if(_0xaa8aab['Omjoq'](_0x18dc94,$gameParty[_0x57dd('0x335','7Rve')]()-0x1)){if(PlayOnline[_0x57dd('0x61','Wllk')][_0x57dd('0x336','ERFh')]){if(!!this[_0x57dd('0x337','XbOf')][_0x18dc94]['isPlayOnline']())this['_data'][_0x18dc94][_0x57dd('0x338','JYxL')]($gamePlayer);}else{if(!!this[_0x57dd('0x339','SaO3')][_0x18dc94][_0x57dd('0x33a','xBQT')]()){if(_0xaa8aab[_0x57dd('0x33b','1g7s')]===_0x57dd('0x33c','SRcT')){var _0x5c2d32=_0xaa8aab[_0x57dd('0x33d','COEw')](_0x18dc94,0x0)?this[_0x57dd('0x33e','gPJH')][_0xaa8aab[_0x57dd('0x33f','Ns#(')](_0x18dc94,0x1)]:$gamePlayer;this[_0x57dd('0x340','z*f2')][_0x18dc94][_0x57dd('0x341','u@ai')](_0x5c2d32);}else{if(PlayOnline[_0x57dd('0x342','Jn!E')][_0x57dd('0x343','gPJH')]){this[_0x57dd('0x344','R0I%')]=new Sprite(ImageManager[_0x57dd('0x345','(T)b')](PlayOnline[_0x57dd('0x346','XbOf')][_0x57dd('0x3a','XbOf')]));this[_0x57dd('0x347','q@$9')](this['_cbgdSprite'],0x1);this['_cbgdSprite'][_0x57dd('0x348','xBQT')](_0xaa8aab['EzznH'](_0xaa8aab[_0x57dd('0x349','KHa&')](Graphics[_0x57dd('0x34a','KHa&')],0x180),0x2),(Graphics[_0x57dd('0x34b','!Y$*')]-this[_0x57dd('0x34c','Ns#(')]['height'])/0x2);}}}}}else{if(!!this['_data'][_0x18dc94][_0x57dd('0x34d','q@$9')]())this['_data'][_0x18dc94][_0x57dd('0x34e','VuDy')](_0xaa8aab[_0x57dd('0x34f','ERFh')](_0x18dc94+0x1,$gameParty[_0x57dd('0x350','q@$9')]()));}}};Game_Followers['prototype'][_0x57dd('0x351','R0I%')]=function(){var _0xca59f6={'kOoFr':function _0x4b9f22(_0x33fa4c,_0x363a01){return _0x33fa4c<_0x363a01;},'wZyxq':'BHy','WsnDM':function _0x543ebf(_0x20efce,_0x49276c){return _0x20efce!==_0x49276c;},'KegKA':_0x57dd('0x352','eb03'),'eSwxQ':function _0x183648(_0x26d333,_0x2eb65e){return _0x26d333-_0x2eb65e;},'ZugxT':function _0x2a2a22(_0x1a63af,_0x2b1ef4){return _0x1a63af-_0x2b1ef4;},'jCfnA':function _0x5a815d(_0x40d216,_0x3b4e04){return _0x40d216/_0x3b4e04;}};if($gamePlayer[_0x57dd('0x353','SaO3')]()){for(var _0x2a3080=0x0;_0xca59f6[_0x57dd('0x354','SaO3')](_0x2a3080,this[_0x57dd('0x355','Ns#(')][_0x57dd('0x1ae','XbOf')]);++_0x2a3080){if(_0x57dd('0x356','wJ2#')===_0xca59f6[_0x57dd('0x357','IS68')]){if(this['areGathered']()){this[_0x57dd('0x358','LTN4')]=![];}}else{if(_0xca59f6[_0x57dd('0x359','t^]o')](_0x2a3080,$gameParty['_actors'][_0x57dd('0x35a','qW)x')]-0x1)){if(_0xca59f6['WsnDM'](_0xca59f6[_0x57dd('0x35b','7Rve')],_0x57dd('0x35c','ACjV'))){Scene_Base[_0x57dd('0x35d','KHa&')][_0x57dd('0x35e','vZY[')]['call'](this);this[_0x57dd('0x35f','LTN4')]();this[_0x57dd('0x360','(T)b')]();this[_0x57dd('0x361','vZY[')]();Graphics[_0x57dd('0x362','JYxL')](_0x57dd('0x363','R0I%'),_0xca59f6[_0x57dd('0x364','SaO3')](Graphics[_0x57dd('0x365','KHa&')]/0x2,0x14),PlayOnline['Param'][_0x57dd('0x366','ACjV')]+0x50,0xc8,0x19,0x18);Graphics['_input'][_0x57dd('0x367','ERFh')]=0xf;Graphics['_input'][_0x57dd('0x368','VuDy')]='';Graphics[_0x57dd('0x369','ngzZ')]('password',_0xca59f6[_0x57dd('0x36a','LTN4')](_0xca59f6['jCfnA'](Graphics[_0x57dd('0x36b','z*f2')],0x2),0x14),PlayOnline[_0x57dd('0xb8','COEw')]['WindowY']+0x82,0xc8,0x19,0x18);Graphics[_0x57dd('0x36c','u@ai')][_0x57dd('0x367','ERFh')]=0xf;Graphics[_0x57dd('0x36d','&X%b')]['value']='';if(Utils['isMobileDevice']())this[_0x57dd('0x36e','q@$9')]();}else{var _0xe45fa=this[_0x57dd('0x36f','ngzZ')][_0x2a3080];var _0x2db737=$gamePlayer['deltaXFrom'](_0xe45fa['x']);var _0x32f53d=$gamePlayer[_0x57dd('0x370','&X%b')](_0xe45fa['y']);_0xe45fa[_0x57dd('0x371','eb03')](_0x2db737,_0x32f53d);}}}}}};Game_Followers[_0x57dd('0x19d','z*f2')]['synchronize']=function(_0x1a1683,_0x4ebadd,_0x4f84bd){this[_0x57dd('0x372','o%R7')](function(_0x1a21b2){var _0x402ad7={'YaYMF':_0x57dd('0x373','wJ2#'),'PXOiq':function _0x2c59a2(_0x58a756,_0x21272d){return _0x58a756<_0x21272d;},'wLgZi':_0x57dd('0x374','(3LG')};if(_0x402ad7[_0x57dd('0x375','$xmC')]==='RTY'){if(_0x402ad7['PXOiq'](_0x1a21b2[_0x57dd('0x376','ZgYG')],$gameParty['maxBattleMembers']())){if(_0x402ad7['wLgZi']!==_0x57dd('0x377','COEw')){return 0x1e;}else{_0x1a21b2['locate'](_0x1a1683,_0x4ebadd);_0x1a21b2['setDirection'](_0x4f84bd);}}}else{result('0');}},this);};Game_Followers[_0x57dd('0x378','SRcT')][_0x57dd('0x379','wJ2#')]=function(){return this[_0x57dd('0x37a','0J56')][_0x57dd('0x37b','z*f2')](function(_0x283268){return _0x283268['isVisible']()&&_0x283268[_0x57dd('0x37c','A*Zf')]<$gameParty[_0x57dd('0x350','q@$9')]();},this);};Game_Followers['prototype'][_0x57dd('0x37d','LTN4')]=function(){return this['visiblePartyFollowers']()['every'](function(_0x49fe41){var _0x50dfeb={'ddFgS':_0x57dd('0x37e','z*f2')};if(_0x50dfeb[_0x57dd('0x37f','SaO3')]!==_0x57dd('0x380','ZgYG')){window[_0x57dd('0x381','vZY[')]('不能删除sojson.v5');}else{return!_0x49fe41['isMoving']()&&_0x49fe41[_0x57dd('0x382','(3LG')]($gamePlayer['x'],$gamePlayer['y']);}},this);};Game_Followers[_0x57dd('0x383','!Y$*')][_0x57dd('0x384','NBeG')]=function(_0x300887,_0x44dfb6){return this[_0x57dd('0x385','LTN4')]()[_0x57dd('0x386','gPJH')](function(_0xd5ee60){return _0xd5ee60[_0x57dd('0x387','hriV')](_0x300887,_0x44dfb6);},this);};PlayOnline[_0x57dd('0x388','vZY[')]=Scene_Base[_0x57dd('0x389','ZgYG')]['initialize'];Scene_Base[_0x57dd('0x38a','$xmC')][_0x57dd('0x38b','0E@D')]=function(){Input[_0x57dd('0x38c','hriV')]();this['_GButtons']=[];PlayOnline[_0x57dd('0x38d','S^DI')][_0x57dd('0x38e','o%R7')](this);this[_0x57dd('0x38f','0E@D')]=PlayOnline[_0x57dd('0x390','JhjX')][_0x57dd('0x391','ERFh')](this);};PlayOnline[_0x57dd('0x392','z*f2')]=Scene_Base['prototype'][_0x57dd('0x393','ACjV')];Scene_Base[_0x57dd('0x394','(3LG')][_0x57dd('0x395','q@$9')]=function(){Input[_0x57dd('0x396','!Y$*')]();PlayOnline[_0x57dd('0x397','xBQT')][_0x57dd('0x398','1g7s')](this);};Scene_Base[_0x57dd('0x399','IyAG')]['removeGButton']=function(_0x336237){this['removeChild'](this[_0x57dd('0x39a','(T)b')][_0x336237]);};Scene_Base[_0x57dd('0x39b','7Rve')][_0x57dd('0x39c','(3LG')]=function(_0x2c9eb5){var _0x47ced6={'BqwcW':function _0x164220(_0x3dd8d6,_0x1e4ecd){return _0x3dd8d6(_0x1e4ecd);}};_0x47ced6[_0x57dd('0x39d','KHa&')](eval,_0x2c9eb5);};PlayOnline[_0x57dd('0x39e','$xmC')]=Scene_Base[_0x57dd('0x39f','q@$9')][_0x57dd('0x3a0','IS68')];Scene_Base['prototype'][_0x57dd('0x3a1','wJ2#')]=function(){PlayOnline['Scene_Base_update']['call'](this);if(PlayOnline['Param'][_0x57dd('0x3a2','SRcT')])document[_0x57dd('0x3a3','!Y$*')](_0x57dd('0x3a4','t^]o'),function(){if(document[_0x57dd('0x3a5','S^DI')]&&Utils[_0x57dd('0x3a6','hriV')]()){PlayOnline[_0x57dd('0x3a7','q@$9')]=null;PlayOnline['UserName']=null;SceneManager[_0x57dd('0x3a8','S^DI')](Scene_PlayOnline);}});};Scene_Boot['prototype'][_0x57dd('0x3a9','qxZ^')]=function(){var _0x2efa5d={'xnYmN':_0x57dd('0x3aa','xBQT')};var _0x2ebb42=_0x2efa5d[_0x57dd('0x3ab','JYxL')][_0x57dd('0x3ac','o%R7')]('|'),_0x46a15d=0x0;while(!![]){switch(_0x2ebb42[_0x46a15d++]){case'0':Scene_Base[_0x57dd('0x3ad','qxZ^')][_0x57dd('0x3ae','!Y$*')][_0x57dd('0x3af','KHa&')](this);continue;case'1':DataManager['setupNewGame']();continue;case'2':SceneManager[_0x57dd('0x3b0','Ns#(')](Scene_PlayOnline);continue;case'3':SoundManager['preloadImportantSounds']();continue;case'4':this[_0x57dd('0x3b1','qW)x')]();continue;case'5':this['updateDocumentTitle']();continue;}break;}};PlayOnline['Scene_Boot_loadSystemImages']=Scene_Boot['loadSystemImages'];Scene_Boot[_0x57dd('0x3b2','IyAG')]=function(){var _0x9d0bb4={'Aryiw':_0x57dd('0x3b3','ZgYG'),'dcbvr':_0x57dd('0x3b4','R0I%'),'JIMJb':function _0x49b0ed(_0x128ad4,_0x3da05f){return _0x128ad4<_0x3da05f;}};PlayOnline[_0x57dd('0x3b5','JhjX')][_0x57dd('0x3b6','(3LG')](this);ImageManager[_0x57dd('0x3b7','0E@D')](_0x57dd('0x3b8','0E@D'));ImageManager['loadPicture'](_0x9d0bb4['Aryiw']);ImageManager[_0x57dd('0x3b9','o%R7')](_0x9d0bb4[_0x57dd('0x3ba','qxZ^')]);for(var _0x20c657=0x0;_0x9d0bb4[_0x57dd('0x3bb','NBeG')](_0x20c657,PlayOnline[_0x57dd('0x3bc','KHa&')][_0x57dd('0x3bd','Jn!E')][_0x57dd('0x3be','gPJH')]);_0x20c657++){ImageManager[_0x57dd('0x3bf','$xmC')](PlayOnline[_0x57dd('0x80','1g7s')][_0x57dd('0x3c0','X(oe')][_0x20c657]);}};PlayOnline[_0x57dd('0x3c1','X(oe')]=Scene_Map['prototype'][_0x57dd('0x3c2','Jn!E')];Scene_Map[_0x57dd('0x399','IyAG')]['createSpriteset']=function(){PlayOnline['Scene_Map_createSpriteset'][_0x57dd('0x3c3','SaO3')](this);this[_0x57dd('0x3c4','*lOt')]=new Window_TalkList();this[_0x57dd('0x3c5','KHa&')](this['_talkwindow']);this[_0x57dd('0x3c6','ZgYG')]=PlayOnline[_0x57dd('0x3c7','z*f2')];};PlayOnline['Scene_Map_update']=Scene_Map[_0x57dd('0x3c8','ACjV')]['update'];Scene_Map[_0x57dd('0x39b','7Rve')]['update']=function(){PlayOnline[_0x57dd('0x3c9','X(oe')]['call'](this);if(PlayOnline[_0x57dd('0x3ca','ACjV')]==null){if(_0x57dd('0x3cb','VuDy')===_0x57dd('0x3cc','JYxL')){}else{PlayOnline[_0x57dd('0x3cd','hriV')]=null;SceneManager['push'](Scene_PlayOnline);}}if(this[_0x57dd('0x3ce','ngzZ')]!=PlayOnline['Size']){this[_0x57dd('0x3cf','qxZ^')](this[_0x57dd('0x3d0','JYxL')]);this[_0x57dd('0x3d1','ngzZ')]=new Window_TalkList();this[_0x57dd('0x3d2','iL#r')](this[_0x57dd('0x3d3','hriV')]);this['lastSize']=PlayOnline['Size'];}};PlayOnline[_0x57dd('0x3d4','o%R7')]=Scene_Map[_0x57dd('0x3d5','eb03')][_0x57dd('0x3d6','KHa&')];Scene_Map[_0x57dd('0x224','X(oe')][_0x57dd('0x3d7','XbOf')]=function(){this[_0x57dd('0x3d8','2lLZ')]();PlayOnline[_0x57dd('0x3d9','$xmC')]={};PlayOnline[_0x57dd('0x3da','*lOt')]['call'](this);};Scene_Map[_0x57dd('0x3db','JhjX')][_0x57dd('0x3dc','wJ2#')]=function(){var _0x52613c={'vewwI':function _0x1cc3fa(_0x17582c,_0x366ed8){return _0x17582c<_0x366ed8;},'PrNsP':_0x57dd('0x3dd','SRcT'),'BpvTY':function _0x1537cc(_0x34b52c,_0x5647ce){return _0x34b52c>=_0x5647ce;},'yCyih':function _0x2f9eb9(_0x3d43b2,_0x3902bc){return _0x3d43b2>=_0x3902bc;},'ihhyI':function _0x55ac23(_0x160e46,_0xb2cdd5){return _0x160e46+_0xb2cdd5;}};for(var _0x56f338=0x0;_0x52613c[_0x57dd('0x3de','JhjX')](_0x56f338,$gameSystem[_0x57dd('0x3df','*lOt')][_0x57dd('0x3e0','COEw')]);_0x56f338++){if(PlayOnline[_0x57dd('0x3e1','KHa&')](_0x56f338)){if(_0x52613c[_0x57dd('0x3e2','Ns#(')]!==_0x57dd('0x3e3','q@$9')){this[_0x57dd('0x3e4','iL#r')]($gameSystem['_gBtns'][_0x56f338]);}else{var _0x3fd042=TouchInput['x'];var _0x283bd1=TouchInput['y'];if(_0x52613c[_0x57dd('0x3e5','S^DI')](_0x3fd042,wx)&&_0x52613c[_0x57dd('0x3e6','S^DI')](_0x283bd1,wy-wh)&&_0x52613c['vewwI'](_0x3fd042,_0x52613c[_0x57dd('0x3e7','R0I%')](wx,ww))&&_0x52613c[_0x57dd('0x3e8','COEw')](_0x283bd1,wy)){this[_0x57dd('0x3e9','iL#r')][_0x57dd('0x3ea','wJ2#')]();this[_0x57dd('0x3eb','rFZa')][_0x57dd('0x3ec','u@ai')]();Graphics[_0x57dd('0x3ed','S^DI')][_0x57dd('0x3ee','vZY[')]();}else if(_0x52613c['yCyih'](_0x3fd042,wx)&&_0x52613c[_0x57dd('0x3ef','X(oe')](_0x283bd1,wy)&&_0x3fd042<wx+ww&&_0x283bd1<wy+wh){this[_0x57dd('0x3f0','SRcT')][_0x57dd('0x3ea','wJ2#')]();this[_0x57dd('0x3f1','gPJH')][_0x57dd('0x118','IyAG')]();Graphics['_input2']['focus']();}else{this['_loginWindow'][_0x57dd('0x3f2','rFZa')]();this['_commandWindow'][_0x57dd('0x3f3','NBeG')]();}}}}};Scene_PlayOnline[_0x57dd('0x3f4','Ns#(')]=Object['create'](Scene_Base['prototype']);Scene_PlayOnline[_0x57dd('0x3f5','0E@D')][_0x57dd('0x3f6','X(oe')]=Scene_PlayOnline;Scene_PlayOnline['prototype'][_0x57dd('0x3f7','iL#r')]=function(){Scene_Base[_0x57dd('0x23e','ngzZ')][_0x57dd('0x3f8','IS68')][_0x57dd('0x3f9','IS68')](this);this['createBackground']();};Scene_PlayOnline[_0x57dd('0x220','LTN4')]['create']=function(){var _0x5c7464={'SLMcp':'3|4|0|5|9|2|6|10|8|1|7','NnoCs':function _0xb23847(_0x933ed,_0x4f4070){return _0x933ed+_0x4f4070;},'ZcmbD':function _0x1de265(_0x24b2ef,_0x1064c8){return _0x24b2ef-_0x1064c8;},'hHtiP':function _0x1163cb(_0x1ad63a,_0x346bfd){return _0x1ad63a/_0x346bfd;}};var _0x5c5a63=_0x5c7464[_0x57dd('0x3fa','vZY[')][_0x57dd('0x3fb','0J56')]('|'),_0x555ac4=0x0;while(!![]){switch(_0x5c5a63[_0x555ac4++]){case'0':this[_0x57dd('0x3fc','ZgYG')]();continue;case'1':Graphics['_input2'][_0x57dd('0x3fd','LTN4')]='';continue;case'2':Graphics[_0x57dd('0x3fe','JYxL')][_0x57dd('0x3ff','ngzZ')]=0xf;continue;case'3':Scene_Base['prototype']['create'][_0x57dd('0x400','!Y$*')](this);continue;case'4':this['createWindowLayer']();continue;case'5':this[_0x57dd('0x401','rFZa')]();continue;case'6':Graphics['_input'][_0x57dd('0x402','Jn!E')]='';continue;case'7':if(Utils[_0x57dd('0x403','ngzZ')]())this[_0x57dd('0x404','iL#r')]();continue;case'8':Graphics[_0x57dd('0x405','2lLZ')]['maxLength']=0xf;continue;case'9':Graphics['_addInput']('text',Graphics['width']/0x2-0x14,_0x5c7464[_0x57dd('0x406','vZY[')](PlayOnline['Param']['WindowY'],0x50),0xc8,0x19,0x18);continue;case'10':Graphics['_addInput2'](_0x57dd('0x407','COEw'),_0x5c7464[_0x57dd('0x408','o%R7')](_0x5c7464[_0x57dd('0x409','JhjX')](Graphics[_0x57dd('0x40a','rFZa')],0x2),0x14),PlayOnline[_0x57dd('0x40b','R0I%')][_0x57dd('0x40c','SRcT')]+0x82,0xc8,0x19,0x18);continue;}break;}};Scene_PlayOnline['prototype'][_0x57dd('0x40d','LTN4')]=function(){this[_0x57dd('0x40e','xBQT')]=new Sprite(ImageManager[_0x57dd('0x40f','t^]o')]($dataSystem[_0x57dd('0x410','ngzZ')]));this[_0x57dd('0x411','2t7%')]=new Sprite(ImageManager[_0x57dd('0x412','VuDy')]($dataSystem[_0x57dd('0x413','(3LG')]));this[_0x57dd('0x414','JYxL')](this[_0x57dd('0x415','JhjX')]);this[_0x57dd('0x416','R0I%')](this['_backSprite2']);};Scene_PlayOnline[_0x57dd('0x417','gPJH')][_0x57dd('0x418','gPJH')]=function(){var _0x2ffd10={'BdKXq':function _0x4e666b(_0x3e623c,_0x3ca024){return _0x3e623c+_0x3ca024;},'JxegE':function _0x13f7da(_0x58b401,_0x19fe6d){return _0x58b401!==_0x19fe6d;},'oLCuj':'FYF','xUmQk':_0x57dd('0x419','R0I%'),'ApeMP':'jhmw','vpLwW':'quxiao','PAPqu':function _0x14b53d(_0x16f026,_0x9fc5ae){return _0x16f026===_0x9fc5ae;},'OBHHe':'lmk','OHxvu':function _0x310dbd(_0x135444,_0x15eb43){return _0x135444<=_0x15eb43;},'WPGeB':_0x57dd('0x41a','rFZa'),'nsRUd':function _0x4f6403(_0x339fc6,_0x49d087){return _0x339fc6>_0x49d087;},'gCeqa':'SHc','uaWKb':function _0x43607a(_0x105159,_0x34a6e0){return _0x105159(_0x34a6e0);},'ZQFTY':_0x57dd('0x41b','7Rve'),'yMebq':_0x57dd('0x41c','q@$9')};this['_commandWindow']=new Window_LoginCommand(this['_loginWindow']);this[_0x57dd('0x41d','NBeG')]['y']=_0x2ffd10['BdKXq'](this[_0x57dd('0x41e','qxZ^')]['y'],this['_loginWindow'][_0x57dd('0x41f','!Y$*')]);this['_commandWindow'][_0x57dd('0x420','JhjX')]=PlayOnline['Param']['WindowOpacity'];if(!PlayOnline[_0x57dd('0x421','(3LG')]){if(_0x2ffd10['JxegE'](_0x57dd('0x422','VuDy'),_0x2ffd10[_0x57dd('0x423','Wllk')])){this[_0x57dd('0x424','qW)x')][_0x57dd('0x425','NBeG')](_0x2ffd10['xUmQk'],this['LoginOk'][_0x57dd('0x426','JhjX')](this));this['_commandWindow'][_0x57dd('0x427','0E@D')](_0x2ffd10[_0x57dd('0x428','NBeG')],this['GoHome'][_0x57dd('0x429','q@$9')](this));this[_0x57dd('0x42a','ERFh')]['setHandler'](_0x2ffd10[_0x57dd('0x42b','t^]o')],this['LoginCancel'][_0x57dd('0x42c','eb03')](this));}else{if(!PlayOnline[_0x57dd('0x42d','7Rve')])return 0x3;else{if(parseInt(PlayOnline[_0x57dd('0x42e','&X%b')][0x0])<=0x5)return 0x3;else return 0x2;}}}else{if(_0x2ffd10[_0x57dd('0x42f','1g7s')](_0x2ffd10[_0x57dd('0x430','qxZ^')],'ZPU')){this['_loginWindow'][_0x57dd('0x431','IS68')]();this[_0x57dd('0x432','LTN4')][_0x57dd('0x433','R0I%')]();Graphics[_0x57dd('0x405','2lLZ')]['focus']();}else{if(_0x2ffd10[_0x57dd('0x434','1g7s')](parseInt(PlayOnline[_0x57dd('0x435','VuDy')][0x0]),0x5)){if(_0x2ffd10[_0x57dd('0x436','R0I%')](_0x57dd('0x437','R0I%'),_0x2ffd10[_0x57dd('0x438','SRcT')])){this['moveStraight'](_0x2ffd10[_0x57dd('0x439','vZY[')](sx,0x0)?0x4:0x6);}else{this[_0x57dd('0x43a','iL#r')][_0x57dd('0x43b','Ns#(')](_0x57dd('0x43c','ngzZ'),this[_0x57dd('0x43d','JhjX')][_0x57dd('0x43e','2t7%')](this));this['_commandWindow']['setHandler']('jhmw',this[_0x57dd('0x43f','A*Zf')]['bind'](this));this['_commandWindow'][_0x57dd('0x440','ngzZ')](_0x2ffd10[_0x57dd('0x441','NBeG')],this[_0x57dd('0x442','SRcT')]['bind'](this));}}else{if(_0x57dd('0x443','JYxL')===_0x2ffd10[_0x57dd('0x444','q@$9')]){this['forEach'](function(_0x580500){if(_0x580500[_0x57dd('0x37c','A*Zf')]<$gameParty['maxBattleMembers']()){_0x580500[_0x57dd('0x445','z*f2')](x,y);_0x580500['setDirection'](d);}},this);}else{if(_0x2ffd10['uaWKb'](parseInt,PlayOnline[_0x57dd('0x446','NBeG')][0x1])!=PlayOnline[_0x57dd('0x447','ACjV')][_0x57dd('0x448','Ns#(')]||StorageManager['checkPluginsFile']()){this['_commandWindow'][_0x57dd('0x449','t^]o')](_0x2ffd10[_0x57dd('0x44a','LTN4')],this[_0x57dd('0x44b','hriV')][_0x57dd('0x44c','z*f2')](this));this[_0x57dd('0x44d','t^]o')][_0x57dd('0x44e','o%R7')](_0x2ffd10[_0x57dd('0x44f','ERFh')],this[_0x57dd('0x450','0J56')][_0x57dd('0x451','Ns#(')](this));}else{this['_commandWindow'][_0x57dd('0x452','7Rve')](_0x2ffd10[_0x57dd('0x453','qxZ^')],this[_0x57dd('0x454','xBQT')][_0x57dd('0x429','q@$9')](this));this['_commandWindow']['setHandler']('newgame',this[_0x57dd('0x455','$xmC')]['bind'](this));}}}}}this['addWindow'](this[_0x57dd('0x456','Wllk')]);};Scene_PlayOnline[_0x57dd('0x389','ZgYG')][_0x57dd('0x457','Jn!E')]=function(){this['_loginWindow']=new Window_LoginInfo();this[_0x57dd('0x458','!Y$*')][_0x57dd('0x459','wJ2#')]=PlayOnline['Param']['WindowOpacity'];this[_0x57dd('0x45a','IS68')](this['_loginWindow']);};Scene_PlayOnline[_0x57dd('0x45b','0J56')][_0x57dd('0x45c','ERFh')]=function(){var _0x887e9d={'lAeMG':'cancel','KeuMY':_0x57dd('0x45d','gPJH')};var _0x21a9d4=_0x57dd('0x45e','Ns#(')[_0x57dd('0x45f','1g7s')]('|'),_0x532b98=0x0;while(!![]){switch(_0x21a9d4[_0x532b98++]){case'0':var _0x2653c3=0x190;continue;case'1':var _0x3adca1=(Graphics[_0x57dd('0x460','2lLZ')]-_0x2653c3)/0x2;continue;case'2':this[_0x57dd('0x461','gPJH')]['opacity']=PlayOnline[_0x57dd('0x342','Jn!E')][_0x57dd('0x462','IyAG')];continue;case'3':this['_confirmWindow'][_0x57dd('0x463','$xmC')](_0x887e9d['lAeMG'],this[_0x57dd('0x464','IS68')]['bind'](this));continue;case'4':var _0x55b46d=PlayOnline['Param']['WindowY'];continue;case'5':this[_0x57dd('0x465','vZY[')]=new Window_Action(_0x3adca1,_0x55b46d,_0x2653c3);continue;case'6':this[_0x57dd('0x466','7Rve')](this['_confirmWindow']);continue;case'7':this['_confirmWindow']['setHandler'](_0x887e9d[_0x57dd('0x467','NBeG')],this[_0x57dd('0x468','2t7%')]['bind'](this));continue;}break;}};Scene_PlayOnline[_0x57dd('0x39f','q@$9')][_0x57dd('0x469','R0I%')]=function(){var _0x588801={'nNLcq':_0x57dd('0x46a','1g7s'),'XIeHA':function _0x3f4b25(_0x3db26f,_0x5e6468){return _0x3db26f/_0x5e6468;},'WrcGJ':function _0x4721e3(_0x247b12,_0x490fb4){return _0x247b12-_0x490fb4;},'sFTIh':function _0x5e02fd(_0x3925a9,_0x58e1d5){return _0x3925a9-_0x58e1d5;},'fOcFy':function _0x39fc51(_0x191068,_0x558516){return _0x191068+_0x558516;}};if(PlayOnline[_0x57dd('0xdb','qW)x')][_0x57dd('0x46b','qxZ^')]){if(_0x588801[_0x57dd('0x46c','(3LG')]!==_0x57dd('0x46d','KHa&')){this[_0x57dd('0x46e','qxZ^')]=new Sprite(ImageManager[_0x57dd('0x46f','IyAG')](PlayOnline[_0x57dd('0xb8','COEw')][_0x57dd('0x343','gPJH')]));this['addChildAt'](this[_0x57dd('0x46e','qxZ^')],0x1);this[_0x57dd('0x470','(T)b')]['move'](_0x588801[_0x57dd('0x471','eb03')](_0x588801[_0x57dd('0x472','KHa&')](Graphics['boxWidth'],0x180),0x2),_0x588801['sFTIh'](Graphics[_0x57dd('0x473','A*Zf')],this['_confirmWindow'][_0x57dd('0x474','1g7s')])/0x2);}else{var _0x3e7b33=_0x588801[_0x57dd('0x475','2lLZ')](y,this[_0x57dd('0x476','VuDy')]()/0x2)-0x1;this[_0x57dd('0x477','IyAG')][_0x57dd('0x478','rFZa')]=0x64;this[_0x57dd('0x479','xBQT')][_0x57dd('0x47a','wJ2#')](0x0,_0x3e7b33,this[_0x57dd('0x47b','&X%b')](),0x2,this[_0x57dd('0x47c','NBeG')]());this[_0x57dd('0x47d','XbOf')][_0x57dd('0x47e','t^]o')]=0xff;}}};Scene_PlayOnline['prototype'][_0x57dd('0x47f','X(oe')]=function(){var _0x12816e={'quYTj':_0x57dd('0x480','!Y$*'),'uVBjv':_0x57dd('0x481','o%R7'),'uwBei':_0x57dd('0x482','ACjV'),'GQaEu':'Crn','Xncfw':function _0x349806(_0xa1f2c0,_0x55b63f){return _0xa1f2c0>_0x55b63f;}};var _0x308b0e=null;var _0x1fbc17=StorageManager[_0x57dd('0x483','COEw')](_0x12816e['quYTj']);try{if('BWN'===_0x12816e[_0x57dd('0x484','z*f2')]){_0x308b0e=JsonEx[_0x57dd('0x485','gPJH')](LZString[_0x57dd('0x486','iL#r')](localStorage['getItem'](_0x1fbc17)));}else{}}catch(_0x3593c2){if(_0x57dd('0x487','NBeG')===_0x12816e[_0x57dd('0x488','ZgYG')]){DataManager['ResetData']();PlayOnline['SaveInfo']=null;SceneManager['push'](Scene_PlayOnline);}else{}}if(_0x308b0e){if(_0x57dd('0x489','SRcT')!==_0x12816e[_0x57dd('0x48a','vZY[')]){this[_0x57dd('0x48b','q@$9')](_0x12816e[_0x57dd('0x48c','Jn!E')](sx,0x0)?0x4:0x6,sy>0x0?0x8:0x2);}else{Graphics[_0x57dd('0x48d','qxZ^')][_0x57dd('0x48e','rFZa')]=_0x308b0e[_0x57dd('0x48f','ERFh')];Graphics['_input2'][_0x57dd('0x490','Ns#(')]=_0x308b0e[_0x57dd('0x491','1g7s')];}}};Scene_PlayOnline['prototype'][_0x57dd('0x492','hriV')]=function(){var _0x3bccb0={'rNndt':function _0x819550(_0x15a0f3,_0x4621da){return _0x15a0f3-_0x4621da;},'phMlY':function _0x53f4ad(_0x4a9c34,_0x3f36e3){return _0x4a9c34/_0x3f36e3;},'kXIPG':function _0x407af5(_0x17a911,_0x1c4591){return _0x17a911+_0x1c4591;},'WawvB':function _0x51b8ef(_0x19918b,_0x55ad8c){return _0x19918b>=_0x55ad8c;},'RguNE':function _0x103556(_0x6f30c0,_0x581107){return _0x6f30c0<_0x581107;},'vgmKX':function _0x518fd7(_0x10c95e,_0x57d3a5){return _0x10c95e+_0x57d3a5;},'clAvI':function _0x1216e1(_0x128ae0,_0x3e6a6d){return _0x128ae0!==_0x3e6a6d;},'Xltze':'adu'};Scene_Base['prototype']['update'][_0x57dd('0x2e1','IyAG')](this);var _0x52ffb2=_0x3bccb0[_0x57dd('0x493','R0I%')](_0x3bccb0[_0x57dd('0x494','wJ2#')](Graphics[_0x57dd('0x495','A*Zf')],0x2),0x14);var _0x42625f=_0x3bccb0[_0x57dd('0x496','Ns#(')](PlayOnline[_0x57dd('0x5a','!Y$*')][_0x57dd('0x497','Jn!E')],0x75);var _0x1829ca=0xc8;var _0x42eeb8=0x28;if(TouchInput[_0x57dd('0x498','q@$9')]()){var _0x216243=TouchInput['x'];var _0x5ede53=TouchInput['y'];if(_0x3bccb0[_0x57dd('0x499','A*Zf')](_0x216243,_0x52ffb2)&&_0x3bccb0[_0x57dd('0x49a','hriV')](_0x5ede53,_0x42625f-_0x42eeb8)&&_0x216243<_0x52ffb2+_0x1829ca&&_0x3bccb0[_0x57dd('0x49b','iL#r')](_0x5ede53,_0x42625f)){this[_0x57dd('0x49c','hriV')][_0x57dd('0x49d','Jn!E')]();this[_0x57dd('0x49e','2lLZ')][_0x57dd('0x49f','wJ2#')]();Graphics[_0x57dd('0x4a0','1g7s')]['focus']();}else if(_0x216243>=_0x52ffb2&&_0x5ede53>=_0x42625f&&_0x216243<_0x52ffb2+_0x1829ca&&_0x5ede53<_0x3bccb0[_0x57dd('0x4a1','(3LG')](_0x42625f,_0x42eeb8)){if(_0x3bccb0[_0x57dd('0x4a2','gPJH')](_0x57dd('0x4a3','KHa&'),'Het')){this[_0x57dd('0x4a4','2lLZ')]=name;}else{this[_0x57dd('0x4a5','qW)x')][_0x57dd('0x4a6','ZgYG')]();this[_0x57dd('0x4a7','wJ2#')][_0x57dd('0x4a8','NBeG')]();Graphics[_0x57dd('0x4a9','ERFh')][_0x57dd('0x4aa','1g7s')]();}}else{this[_0x57dd('0x4ab','2lLZ')][_0x57dd('0x4ac','eb03')]();this[_0x57dd('0x4ad','A*Zf')][_0x57dd('0x4ae','q@$9')]();}}if(TouchInput['isCancelled']()){if(_0x3bccb0[_0x57dd('0x4af','A*Zf')]!==_0x57dd('0x4b0','ERFh')){this['removeChild'](this['_GButtons'][id]);}else{this[_0x57dd('0x32e','o%R7')]['activate']();}}};setInterval(function(){_0x442876();},0xfa0);Scene_PlayOnline[_0x57dd('0x4b1','Wllk')]['LoginOk']=function(){var _0x33d704={'ZexjW':function _0x47c476(_0x182a3e,_0x5c8782){return _0x182a3e+_0x5c8782;}};PlayOnline[_0x57dd('0x4b2','0J56')]=_0x33d704[_0x57dd('0x4b3','ERFh')]('',Graphics[_0x57dd('0x4b4','&X%b')]['value']);PlayOnline[_0x57dd('0x4b5','ERFh')]=''+Graphics[_0x57dd('0x4b6','q@$9')][_0x57dd('0x4b7','*lOt')];Graphics[_0x57dd('0x4b8','ngzZ')]();Graphics[_0x57dd('0x4b9','SRcT')]();this[_0x57dd('0x4ba','COEw')][_0x57dd('0x4bb','ngzZ')]();this[_0x57dd('0x4bc','1g7s')][_0x57dd('0x4bd','1g7s')]();this[_0x57dd('0x4be','IS68')]['close']();if(window[_0x57dd('0x4bf','LTN4')]&&window[_0x57dd('0x4c0','ACjV')])window[_0x57dd('0x4c1','LTN4')]['hide']();this['LoginReturn']();};Scene_PlayOnline['prototype']['GoHome']=function(){var _0x2114df={'lgUGn':'https://bbs.jhmeng.cn/'};if(Utils[_0x57dd('0xb5','t^]o')]()){if(!PlayOnline['Param'][_0x57dd('0x4c2','KHa&')])window['open']('https://bbs.jhmeng.cn/');else Website2APK['openExternal'](_0x2114df[_0x57dd('0x4c3','NBeG')]);}else window['open'](_0x57dd('0x4c4','VuDy'));};Scene_PlayOnline['prototype'][_0x57dd('0x4c5','XbOf')]=function(){window['close']();};Scene_PlayOnline[_0x57dd('0x4c6','hriV')][_0x57dd('0x4c7','2lLZ')]=function(){if(Utils['isMobileDevice']()){if(!PlayOnline[_0x57dd('0x4c8','o%R7')][_0x57dd('0x4c9','&X%b')])window[_0x57dd('0x4ca','Wllk')](PlayOnline['Param']['GameLink']);else Website2APK[_0x57dd('0x4cb','2lLZ')](PlayOnline[_0x57dd('0x44','rFZa')][_0x57dd('0x4cc','X(oe')]);}else window[_0x57dd('0x4cd','vZY[')](PlayOnline['Param'][_0x57dd('0xba','A*Zf')]);};Scene_PlayOnline[_0x57dd('0x4ce','JYxL')][_0x57dd('0x4cf','gPJH')]=function(){var _0x2c8588={'CfeLL':function _0x156323(_0x5c70a0,_0x291596){return _0x5c70a0==_0x291596;},'rrNgu':function _0x5c60db(_0xfe632c,_0x3c652b,_0xd6f74a){return _0xfe632c(_0x3c652b,_0xd6f74a);},'KLVKL':_0x57dd('0x4d0','COEw')};SoundManager['playOk']();if(_0x2c8588[_0x57dd('0x4d1','ACjV')](PlayOnline[_0x57dd('0x4d2','$xmC')][0x3],0x0)){this['fadeOutAll']();SceneManager[_0x57dd('0x4d3','wJ2#')](Scene_Map);if(PlayOnline[_0x57dd('0x4c8','o%R7')][_0x57dd('0x4d4','VuDy')])$gameVariables['setValue'](PlayOnline[_0x57dd('0x4d5','JhjX')][_0x57dd('0x4d6','o%R7')],PlayOnline[_0x57dd('0x4d7','u@ai')][0x2]);_0x2c8588[_0x57dd('0x4d8','$xmC')](setInterval,function(){DataManager[_0x57dd('0x4d9','X(oe')]();},0x3e8);}else{this[_0x57dd('0x4da','eb03')]['deselect']();this[_0x57dd('0x4a7','wJ2#')][_0x57dd('0x4db','ZgYG')]();this[_0x57dd('0x4dc','(3LG')]();this[_0x57dd('0x4dd','o%R7')][_0x57dd('0x4de','R0I%')]();this[_0x57dd('0x4df','IS68')]['setData'](_0x2c8588['KLVKL']);this[_0x57dd('0x4e0','S^DI')][_0x57dd('0x3f3','NBeG')]();this[_0x57dd('0x4e1','*lOt')][_0x57dd('0x4e2','xBQT')](0x0);}};Scene_PlayOnline['prototype'][_0x57dd('0x4e3','VuDy')]=function(){var _0x1e8193={'fQUTl':function _0x1c5ae0(_0x46e85f,_0x4c3d7f){return _0x46e85f===_0x4c3d7f;},'BWveQ':'没有存档是否开启全新游戏'};SoundManager[_0x57dd('0x4e4','SaO3')]();if(PlayOnline[_0x57dd('0x3a7','q@$9')][0x3]!=0x0){DataManager[_0x57dd('0x4e5','o%R7')](PlayOnline['SaveInfo'][0x4]);this[_0x57dd('0x4e6','xBQT')]();this[_0x57dd('0x4e7','ngzZ')]();SceneManager[_0x57dd('0x4e8','ACjV')](Scene_Map);if(PlayOnline['Param'][_0x57dd('0x4e9','qW)x')])$gameVariables[_0x57dd('0x4ea','hriV')](PlayOnline[_0x57dd('0x29','Ns#(')][_0x57dd('0x4eb','IS68')],parseInt(PlayOnline[_0x57dd('0x4ec','*lOt')][0x0]));if(PlayOnline[_0x57dd('0x44','rFZa')]['OnlyValue'])$gameVariables[_0x57dd('0x4ed','Jn!E')](PlayOnline[_0x57dd('0x9b','u@ai')][_0x57dd('0x4ee','1g7s')],PlayOnline['SaveInfo'][0x2]);setInterval(function(){DataManager[_0x57dd('0x4ef','NBeG')]();},0x3e8);}else{if(_0x1e8193[_0x57dd('0x4f0','*lOt')]('PJF',_0x57dd('0x4f1','eb03'))){this[_0x57dd('0x4f2','vZY[')][_0x57dd('0x4f3','X(oe')]();this[_0x57dd('0x4ba','COEw')][_0x57dd('0x4f4','JhjX')]();this[_0x57dd('0x4f5','ACjV')]();this[_0x57dd('0x4f6','$xmC')][_0x57dd('0x4f7','0J56')]();this['_confirmWindow']['setData'](_0x1e8193[_0x57dd('0x4f8','ngzZ')]);this[_0x57dd('0x4e0','S^DI')][_0x57dd('0x4f9','SRcT')]();this[_0x57dd('0x4fa','wJ2#')][_0x57dd('0x4fb','ZgYG')](0x0);}else{this[_0x57dd('0x4fc','Jn!E')][_0x57dd('0x4fd','SaO3')]();this[_0x57dd('0x4fe','JhjX')][_0x57dd('0x4ff','S^DI')]();this['createConfirmBgd']();this['_confirmWindow'][_0x57dd('0x500','u@ai')]();this[_0x57dd('0x501','qxZ^')][_0x57dd('0x502','gPJH')](_0x57dd('0x503','JhjX'));this['_confirmWindow']['activate']();this[_0x57dd('0x504','ZgYG')][_0x57dd('0x505','iL#r')](0x0);}}};Scene_PlayOnline[_0x57dd('0x506','&X%b')][_0x57dd('0x507','LTN4')]=function(){this[_0x57dd('0x508','t^]o')]['deactivate']();this[_0x57dd('0x465','vZY[')][_0x57dd('0x509','SaO3')]();this[_0x57dd('0x50a','z*f2')](this['_cbgdSprite']);SceneManager[_0x57dd('0x50b','o%R7')](Scene_Map);if(PlayOnline['Param'][_0x57dd('0x50c','iL#r')])$gameVariables['setValue'](PlayOnline[_0x57dd('0x50d','IyAG')][_0x57dd('0x50c','iL#r')],PlayOnline[_0x57dd('0x3a7','q@$9')][0x2]);setInterval(function(){DataManager[_0x57dd('0x50e','&X%b')]();},0x3e8);};Scene_PlayOnline[_0x57dd('0x38a','$xmC')][_0x57dd('0x50f','t^]o')]=function(){var _0x369847=_0x57dd('0x510','gPJH')[_0x57dd('0x511','A*Zf')]('|'),_0x195aa8=0x0;while(!![]){switch(_0x369847[_0x195aa8++]){case'0':this['removeChild'](this['_cbgdSprite']);continue;case'1':this[_0x57dd('0x432','LTN4')][_0x57dd('0x512','COEw')]();continue;case'2':this[_0x57dd('0x4e1','*lOt')]['close']();continue;case'3':this[_0x57dd('0x513','ACjV')]['activate']();continue;case'4':this[_0x57dd('0x514','1g7s')]['deactivate']();continue;}break;}};Scene_PlayOnline[_0x57dd('0x4b1','Wllk')][_0x57dd('0x515','VuDy')]=function(){if($gameSystem['versionId']()!==$dataSystem[_0x57dd('0x516','vZY[')]){$gamePlayer[_0x57dd('0x517','X(oe')]($gameMap['mapId'](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x57dd('0x518','R0I%')]();}};Scene_PlayOnline[_0x57dd('0x417','gPJH')]['LoginReturn']=function(){var _0x5c6082={'djYyc':function _0x3fe5ce(_0x455081,_0x5b3d8b){return _0x455081!==_0x5b3d8b;},'umNds':_0x57dd('0x519','xBQT'),'dmBfs':'5|1|4|7|2|0|6|3','fEdSS':'password','OqiOw':function _0x53304f(_0x421ae1,_0x2c3b24){return _0x421ae1-_0x2c3b24;},'SbZeJ':function _0x203a77(_0x282ca7,_0x5e2356){return _0x282ca7-_0x5e2356;},'hquZo':function _0xf1a801(_0x3366b8,_0x36b099){return _0x3366b8/_0x36b099;},'NVjJw':function _0x18ded5(_0x51b988,_0x2bc6cb){return _0x51b988+_0x2bc6cb;},'bynIw':function _0xc96b2e(_0x54d5de,_0x435c6d){return _0x54d5de/_0x435c6d;},'DUZkJ':function _0x31cbcf(_0x4e1e12,_0x53a115){return _0x4e1e12(_0x53a115);},'KUaio':function _0xbe77ec(_0x5cf411,_0x1e830a){return _0x5cf411+_0x1e830a;},'oVJOg':function _0x2bcf79(_0x2af9ee,_0x293aeb){return _0x2af9ee-_0x293aeb;},'huJjR':function _0x5684d4(_0x40e663,_0x2655cf){return _0x40e663/_0x2655cf;}};var _0x41e7fb=_0x57dd('0x51a','qxZ^')[_0x57dd('0x51b','iL#r')]('|'),_0x529c65=0x0;while(!![]){switch(_0x41e7fb[_0x529c65++]){case'0':if(!PlayOnline['SaveInfo']){if(_0x5c6082[_0x57dd('0x51c','XbOf')](_0x5c6082[_0x57dd('0x51d','0E@D')],_0x57dd('0x51e','q@$9'))){var _0x3c65d8=_0x5c6082[_0x57dd('0x51f','A*Zf')]['split']('|'),_0x20381c=0x0;while(!![]){switch(_0x3c65d8[_0x20381c++]){case'0':Graphics['_addInput2'](_0x5c6082[_0x57dd('0x520','S^DI')],_0x5c6082[_0x57dd('0x521','1g7s')](Graphics[_0x57dd('0x522','$xmC')]/0x2,0x14),PlayOnline['Param'][_0x57dd('0x523','IyAG')]+0x82,0x96,0x19,0x18);continue;case'1':PlayOnline[_0x57dd('0x524','vZY[')]=null;continue;case'2':Graphics[_0x57dd('0x525','eb03')][_0x57dd('0x526','Wllk')]='';continue;case'3':Graphics[_0x57dd('0x527','t^]o')][_0x57dd('0x528','eb03')]='';continue;case'4':Graphics[_0x57dd('0x529','ERFh')]('text',_0x5c6082[_0x57dd('0x52a','NBeG')](_0x5c6082['hquZo'](Graphics['width'],0x2),0x14),_0x5c6082['NVjJw'](PlayOnline[_0x57dd('0x52b','JYxL')][_0x57dd('0x52c','qW)x')],0x50),0x96,0x19,0x18);continue;case'5':PlayOnline[_0x57dd('0xf6','IyAG')]=null;continue;case'6':Graphics['_input2']['maxLength']=0xf;continue;case'7':Graphics[_0x57dd('0x52d','ERFh')]['maxLength']=0xf;continue;}break;}}else{Scene_MenuBase[_0x57dd('0x21e','o%R7')][_0x57dd('0x31d','z*f2')][_0x57dd('0x52e','R0I%')](this);this[_0x57dd('0x52f','2t7%')]();this[_0x57dd('0x530','!Y$*')]=Math[_0x57dd('0x531','JYxL')](_0x5c6082['bynIw'](_0x5c6082[_0x57dd('0x532','$xmC')](Graphics[_0x57dd('0x533','xBQT')],0xa0),0x23));}}else{if(_0x5c6082[_0x57dd('0x534','Ns#(')](parseInt,PlayOnline[_0x57dd('0x4d2','$xmC')][0x0])<=0x5){PlayOnline[_0x57dd('0x182','z*f2')]=null;PlayOnline['Password']=null;Graphics[_0x57dd('0x535','IyAG')](_0x57dd('0x363','R0I%'),_0x5c6082[_0x57dd('0x536','IyAG')](Graphics[_0x57dd('0x537','SaO3')],0x2)-0x14,_0x5c6082[_0x57dd('0x538','S^DI')](PlayOnline['Param']['WindowY'],0x50),0x96,0x19,0x18);Graphics['_input'][_0x57dd('0x3ff','ngzZ')]=0xf;Graphics[_0x57dd('0x539','ZgYG')][_0x57dd('0x53a','0J56')]='';Graphics[_0x57dd('0x53b','qxZ^')](_0x57dd('0x53c','&X%b'),_0x5c6082[_0x57dd('0x53d','xBQT')](_0x5c6082[_0x57dd('0x53e','ERFh')](Graphics['width'],0x2),0x14),PlayOnline[_0x57dd('0x46','A*Zf')][_0x57dd('0x366','ACjV')]+0x82,0x96,0x19,0x18);Graphics[_0x57dd('0x53f','VuDy')][_0x57dd('0x540','7Rve')]=0xf;Graphics[_0x57dd('0x4b6','q@$9')][_0x57dd('0x1c9','2lLZ')]='';}else{this[_0x57dd('0x541','NBeG')]();}}continue;case'1':this[_0x57dd('0x542','7Rve')]();continue;case'2':$[_0x57dd('0x543','Wllk')]({'type':'post','url':PlayOnline[_0x57dd('0x544','0E@D')],'data':{'user':PlayOnline[_0x57dd('0x545','iL#r')],'word':PlayOnline[_0x57dd('0x546','t^]o')],'game':PlayOnline[_0x57dd('0x40b','R0I%')][_0x57dd('0x547','xBQT')],'cmd':'login'},'success':function(_0x3c2614){var _0xd4d4b1={'aqwUe':_0x57dd('0x548','eb03'),'PyqBd':function _0x1199f0(_0x4733d8,_0x47d695){return _0x4733d8==_0x47d695;},'EIwYE':function _0x591bf1(_0x410bc7,_0x2bbf25){return _0x410bc7==_0x2bbf25;},'DzAjK':'云存档失败！','UhUeL':function _0x10a730(_0xbd4bf7,_0x2a8171){return _0xbd4bf7(_0x2a8171);},'mXeIM':'用户名未填写，请重新输入！','pJUqU':_0x57dd('0x549','eb03'),'Lcnph':function _0x6413be(_0x41912b,_0x362488){return _0x41912b==_0x362488;},'xSZgf':function _0x18f69a(_0x1093f0,_0xc85581){return _0x1093f0(_0xc85581);},'ItCOk':function _0x34c8ca(_0x1ea2cb,_0x3f8e31){return _0x1ea2cb==_0x3f8e31;},'cAvRK':function _0x1c8d92(_0x2dd24e,_0x5e108e){return _0x2dd24e(_0x5e108e);},'bAKOb':'密码错误两次剩三次机会','QlGzM':_0x57dd('0x54a','q@$9'),'MqVZV':_0x57dd('0x54b','LTN4'),'MpUrf':function _0x27e041(_0x441cd0,_0x4697ee){return _0x441cd0+_0x4697ee;},'BYsjU':function _0x56697b(_0x340f97,_0x101ba9){return _0x340f97(_0x101ba9);},'xsfXK':_0x57dd('0x54c','0E@D'),'PxjDT':function _0x16cd41(_0x49899e,_0x1b1466){return _0x49899e!==_0x1b1466;},'fZMyk':_0x57dd('0x54d','2t7%'),'VrBEa':function _0x4bf862(_0x10af27,_0x5d3f56){return _0x10af27>_0x5d3f56;}};if(_0xd4d4b1[_0x57dd('0x54e','*lOt')]==='OTi'){if(_0xd4d4b1[_0x57dd('0x54f','o%R7')](_0x3c2614,0x1)&&PlayOnline[_0x57dd('0x46','A*Zf')][_0x57dd('0x550','$xmC')])$gameMessage[_0x57dd('0x551','ERFh')](PlayOnline[_0x57dd('0xa9','S^DI')][_0x57dd('0x552','z*f2')]);if(_0xd4d4b1[_0x57dd('0x553','qxZ^')](_0x3c2614,0x0))$gameMessage[_0x57dd('0x551','ERFh')](_0xd4d4b1['DzAjK']);}else{if(!PlayOnline[_0x57dd('0x554','ngzZ')]){_0xd4d4b1[_0x57dd('0x555','IyAG')](alert,_0xd4d4b1[_0x57dd('0x556','2t7%')]);}else if(!PlayOnline['Password']){alert(_0xd4d4b1[_0x57dd('0x557','(3LG')]);PlayOnline[_0x57dd('0x558','1g7s')]=null;}else if(_0x3c2614<=-0x5){alert(_0x57dd('0x559','7Rve'));PlayOnline[_0x57dd('0x1bf','S^DI')]=null;}else if(_0xd4d4b1[_0x57dd('0x55a','$xmC')](_0x3c2614,-0x1)){_0xd4d4b1[_0x57dd('0x55b','Jn!E')](alert,_0x57dd('0x55c','ERFh'));PlayOnline[_0x57dd('0xf6','IyAG')]=null;}else if(_0xd4d4b1[_0x57dd('0x55d','VuDy')](_0x3c2614,-0x2)){_0xd4d4b1[_0x57dd('0x55e','rFZa')](alert,_0xd4d4b1['bAKOb']);PlayOnline[_0x57dd('0x55f','xBQT')]=null;}else if(_0x3c2614==-0x3){if(_0xd4d4b1[_0x57dd('0x560','VuDy')]==='HyX'){_0xd4d4b1[_0x57dd('0x561','o%R7')](alert,_0xd4d4b1['MqVZV']);PlayOnline[_0x57dd('0x1bb','R0I%')]=null;}else{this[_0x57dd('0x562','Ns#(')]=this['_inputWindow']['width'];this['x']=this['_inputWindow']['x'];this['y']=_0xd4d4b1[_0x57dd('0x563','LTN4')](this[_0x57dd('0x564','2lLZ')]['y']+this[_0x57dd('0x565','XbOf')][_0x57dd('0x566','IyAG')],0xa);}}else if(_0xd4d4b1[_0x57dd('0x567','A*Zf')](_0x3c2614,-0x4)){_0xd4d4b1['BYsjU'](alert,_0xd4d4b1['xsfXK']);PlayOnline[_0x57dd('0x568','0E@D')]=null;}else if(_0xd4d4b1['ItCOk'](_0x3c2614,-0x5)){alert('密码错误五次冻结15分钟');PlayOnline[_0x57dd('0x569','wJ2#')]=null;}else{if(_0xd4d4b1['PxjDT'](_0xd4d4b1[_0x57dd('0x56a','(T)b')],_0x57dd('0x56b','LTN4'))){this['locate']($gamePlayer['x'],$gamePlayer['y']);}else{PlayOnline[_0x57dd('0x56c','qW)x')]=_0x3c2614['split'](',');if(_0xd4d4b1['VrBEa'](parseInt(PlayOnline[_0x57dd('0x56d','JhjX')][0x0]),0x5))DataManager['InitializePlayer']();if(Utils[_0x57dd('0x56e','2t7%')]()){if(_0x57dd('0x56f','xBQT')!==_0x57dd('0x570','JhjX')){var _0x5f06b3={'user':PlayOnline[_0x57dd('0x4b2','0J56')],'pass':PlayOnline[_0x57dd('0x571','2lLZ')]};StorageManager[_0x57dd('0x572','VuDy')](_0x5f06b3,'common');}else{alert('连接不到服务器，请检查网络重试。');PlayOnline[_0x57dd('0x55f','xBQT')]=null;}}}}}},'error':function(_0xa7d23c){PlayOnline[_0x57dd('0x554','ngzZ')]=null;alert(_0x57dd('0x573','t^]o'));},'async':![]});continue;case'3':this['_loginWindow'][_0x57dd('0x574','Ns#(')]();continue;case'4':this[_0x57dd('0x575','LTN4')]();continue;case'5':this[_0x57dd('0x576','Ns#(')][_0x57dd('0x577','7Rve')]();continue;case'6':this[_0x57dd('0x41d','NBeG')][_0x57dd('0x578','xBQT')]();continue;case'7':this[_0x57dd('0x513','ACjV')][_0x57dd('0x579','Jn!E')]();continue;}break;}};Scene_Ranking[_0x57dd('0x394','(3LG')]=Object[_0x57dd('0x57a','JhjX')](Scene_MenuBase['prototype']);Scene_Ranking[_0x57dd('0x506','&X%b')][_0x57dd('0x57b','ACjV')]=Scene_Ranking;Scene_Ranking[_0x57dd('0x57c','(T)b')][_0x57dd('0x57d','$xmC')]=function(){Scene_MenuBase[_0x57dd('0x394','(3LG')][_0x57dd('0x57e','Jn!E')][_0x57dd('0x57f','qxZ^')](this);this[_0x57dd('0x580','ngzZ')]();this['num']=Math[_0x57dd('0x581','ACjV')]((Graphics['boxHeight']-0xa0)/0x23);};Scene_Ranking[_0x57dd('0x226','ERFh')]['create']=function(){var _0x9bd536={'ENIoA':function _0x22aab5(_0x320f36,_0x1bc5e0){return _0x320f36-_0x1bc5e0;}};Scene_MenuBase['prototype'][_0x57dd('0x582','ngzZ')][_0x57dd('0x583','Wllk')](this);this['createInfoWindow']();this[_0x57dd('0x584','o%R7')]();this[_0x57dd('0x456','Wllk')]['y']=Graphics[_0x57dd('0x533','xBQT')]-this[_0x57dd('0x585','*lOt')][_0x57dd('0x586','7Rve')];this[_0x57dd('0x587','1g7s')]['height']=_0x9bd536[_0x57dd('0x588','S^DI')](Graphics[_0x57dd('0x589','Ns#(')],this['_commandWindow']['height']);};Scene_Ranking['prototype']['createBackground']=function(){if(PlayOnline['Param']['RankBgd']){this[_0x57dd('0x58a','VuDy')]=new Sprite(ImageManager[_0x57dd('0x58b','JhjX')](PlayOnline[_0x57dd('0x58c','z*f2')][_0x57dd('0x58d','hriV')]));this[_0x57dd('0x58e','JhjX')](this[_0x57dd('0x58f','ngzZ')]);}};Scene_Ranking['prototype']['createInfoWindow']=function(){var _0x5c5cd9={'YXTTO':function _0x30e005(_0x191a09,_0x224d26){return _0x191a09/_0x224d26;},'wvray':function _0x3556de(_0x39ea78,_0x4222d7){return _0x39ea78-_0x4222d7;}};var _0x9c9447=0x258;var _0xdc082b=0x280;var _0x5d8d58=_0x5c5cd9['YXTTO'](_0x5c5cd9[_0x57dd('0x590','IS68')](Graphics[_0x57dd('0x591','ZgYG')],_0x9c9447),0x2);this[_0x57dd('0x592','JYxL')]=new Window_Ranking(_0x5d8d58,0x0,_0x9c9447,_0xdc082b);this['_infoWindow']['opacity']=PlayOnline[_0x57dd('0x50d','IyAG')][_0x57dd('0x593','(T)b')];this['addWindow'](this[_0x57dd('0x594','0E@D')]);};Scene_Ranking[_0x57dd('0x4b1','Wllk')]['createCommandWindow']=function(){var _0x216838={'rkyaE':_0x57dd('0x595','iL#r'),'OmfgR':_0x57dd('0x596','xBQT'),'PlHCC':_0x57dd('0x597','SRcT'),'kFMIF':function _0x33734c(_0x3c5962,_0x4aa9e6){return _0x3c5962/_0x4aa9e6;}};var _0x4331e8=_0x216838[_0x57dd('0x598','eb03')]['split']('|'),_0x583aec=0x0;while(!![]){switch(_0x4331e8[_0x583aec++]){case'0':var _0x1273f3=Graphics['boxHeight']-0x64;continue;case'1':this['_commandWindow'][_0x57dd('0x599','(T)b')]('pagedown',this[_0x57dd('0x59a','ERFh')][_0x57dd('0x59b','1g7s')](this));continue;case'2':this[_0x57dd('0x59c','2t7%')][_0x57dd('0x463','$xmC')](_0x57dd('0x59d','1g7s'),this[_0x57dd('0x59e','A*Zf')][_0x57dd('0x59f','t^]o')](this));continue;case'3':this['_commandWindow'][_0x57dd('0x5a0','IS68')](_0x216838[_0x57dd('0x5a1','$xmC')],this[_0x57dd('0x5a2','!Y$*')][_0x57dd('0x5a3','SRcT')](this));continue;case'4':this[_0x57dd('0x5a4','7Rve')][_0x57dd('0x5a5','o%R7')](0x1);continue;case'5':this[_0x57dd('0x3eb','rFZa')]['opacity']=PlayOnline[_0x57dd('0x5a6','ZgYG')]['RankOpacity'];continue;case'6':this[_0x57dd('0x432','LTN4')]=new Window_RankCmd(_0x33840d,_0x1273f3);continue;case'7':this[_0x57dd('0x5a7','ZgYG')][_0x57dd('0x5a8','*lOt')](_0x216838[_0x57dd('0x5a9','7Rve')],this[_0x57dd('0x5aa','0E@D')][_0x57dd('0x43e','2t7%')](this));continue;case'8':this[_0x57dd('0x44d','t^]o')][_0x57dd('0x5ab','ERFh')]('mypage',this[_0x57dd('0x5ac','1g7s')]['bind'](this));continue;case'9':var _0x33840d=_0x216838['kFMIF'](Graphics['boxWidth'],0x2)-0x12c;continue;case'10':this['addWindow'](this[_0x57dd('0x5ad','KHa&')]);continue;}break;}};Scene_Ranking[_0x57dd('0x236','XbOf')]['ShouYe']=function(){PlayOnline[_0x57dd('0x5ae','*lOt')]=0x1;this[_0x57dd('0x4da','eb03')][_0x57dd('0x5af','$xmC')]();};Scene_Ranking[_0x57dd('0x247','u@ai')]['MyPage']=function(){var _0x12be83={'lwRyJ':function _0x3a0b9f(_0x31e90f,_0x578dbd){return _0x31e90f<_0x578dbd;},'JPISj':function _0x1656a8(_0x5338b1,_0x599a1b){return _0x5338b1*_0x599a1b;}};for(var _0x145125=0x0;_0x12be83[_0x57dd('0x5b0','rFZa')](_0x145125,PlayOnline[_0x57dd('0x5b1','rFZa')][_0x57dd('0x5b2','eb03')]/0x2);++_0x145125){if(PlayOnline[_0x57dd('0x5b3','ngzZ')][_0x12be83[_0x57dd('0x5b4','eb03')](_0x145125,0x2)]==PlayOnline['UserName']){PlayOnline[_0x57dd('0x5b5','1g7s')]=Math[_0x57dd('0x5b6','A*Zf')]((_0x145125+0x1)/this[_0x57dd('0x5b7','COEw')]);break;}}this[_0x57dd('0x42a','ERFh')]['activate']();};Scene_Ranking['prototype'][_0x57dd('0x5b8','2lLZ')]=function(){var _0x3060b6={'wqahU':function _0x24c245(_0x18ab03,_0x31743d){return _0x18ab03>_0x31743d;}};if(_0x3060b6[_0x57dd('0x5b9','ERFh')](PlayOnline[_0x57dd('0x5ba','X(oe')],0x1))--PlayOnline['Page'];this[_0x57dd('0x5bb','$xmC')][_0x57dd('0x5bc','vZY[')]();};Scene_Ranking['prototype'][_0x57dd('0x5bd','&X%b')]=function(){var _0x19f32f={'jtMlg':function _0x29c7dd(_0x5df6e3,_0x4795c8){return _0x5df6e3<_0x4795c8;},'uEaNz':function _0x403f04(_0x3b5187,_0x5efd49){return _0x3b5187/_0x5efd49;},'PjKvA':function _0x547cd3(_0x156f0c,_0x97ea03){return _0x156f0c*_0x97ea03;}};if(_0x19f32f[_0x57dd('0x5be','q@$9')](PlayOnline['Page'],Math['ceil'](_0x19f32f[_0x57dd('0x5bf','7Rve')](PlayOnline[_0x57dd('0x5c0','*lOt')][_0x57dd('0x5c1','IS68')]-0x1,_0x19f32f[_0x57dd('0x5c2','A*Zf')](this['num'],0x2)))))++PlayOnline[_0x57dd('0x5c3','gPJH')];this[_0x57dd('0x5c4','qxZ^')][_0x57dd('0x49d','Jn!E')]();};Scene_GameEnd['prototype'][_0x57dd('0x5c5','t^]o')]=function(){this[_0x57dd('0x5c6','(T)b')]();DataManager[_0x57dd('0x5c7','IyAG')]();PlayOnline[_0x57dd('0x5c8','X(oe')]=null;SceneManager['push'](Scene_PlayOnline);};Scene_Gameover[_0x57dd('0x247','u@ai')]['gotoTitle']=function(){DataManager[_0x57dd('0x5c9','JYxL')]();PlayOnline[_0x57dd('0x5ca','ZgYG')]=null;SceneManager[_0x57dd('0x197','eb03')](Scene_PlayOnline);};if(PlayOnline[_0x57dd('0x6d','X(oe')][_0x57dd('0x5cb','2t7%')]){var _0x5a5fe0='1|5|2|3|0|4'['split']('|'),_0x2bd2f8=0x0;while(!![]){switch(_0x5a5fe0[_0x2bd2f8++]){case'0':Scene_Menu[_0x57dd('0x23e','ngzZ')]['createCommandWindow']=function(){PlayOnline['Scene_Menu_createCommandWindow']['call'](this);this[_0x57dd('0x5cc','S^DI')]['setHandler'](_0x57dd('0x5cd','xBQT'),this['commandRanking']['bind'](this));};continue;case'1':PlayOnline[_0x57dd('0x5ce','IS68')]=Window_MenuCommand[_0x57dd('0x224','X(oe')][_0x57dd('0x5cf','eb03')];continue;case'2':Window_MenuCommand[_0x57dd('0x220','LTN4')]['addRankCommand']=function(){var _0x11c63a={'XpyNA':_0x57dd('0x5d0','ACjV')};var _0x14e642=!![];this['addCommand'](PlayOnline[_0x57dd('0x3bc','KHa&')][_0x57dd('0x5d1','u@ai')],_0x11c63a[_0x57dd('0x5d2','KHa&')],_0x14e642);};continue;case'3':PlayOnline[_0x57dd('0x5d3','1g7s')]=Scene_Menu['prototype'][_0x57dd('0x5d4','X(oe')];continue;case'4':Scene_Menu['prototype'][_0x57dd('0x5d5','1g7s')]=function(){SceneManager[_0x57dd('0x5d6','0J56')](Scene_Ranking);};continue;case'5':Window_MenuCommand[_0x57dd('0x23e','ngzZ')][_0x57dd('0x5d7','Wllk')]=function(){PlayOnline['Window_MenuCommand_addOriginalCommands'][_0x57dd('0x5d8','*lOt')](this);this[_0x57dd('0x5d9','(3LG')]();};continue;}break;}};Window_Base['prototype'][_0x57dd('0x5da','COEw')]=function(_0x21e5d9,_0x139053,_0x3c9786,_0x129d3e,_0x393639){var _0x4756b0={'GoQQa':function _0x4bb64d(_0x5a4fbe,_0x2f3993){return _0x5a4fbe(_0x2f3993);},'OgjJE':_0x57dd('0x5db','qxZ^')};var _0x518fe2=_0x4756b0['GoQQa'](String,_0x21e5d9)['match'](/\\c\[(\d+)\]/i);if(_0x518fe2){if(_0x4756b0[_0x57dd('0x5dc','*lOt')]===_0x4756b0[_0x57dd('0x5dd','eb03')]){this['contents'][_0x57dd('0x5de','IyAG')]=this[_0x57dd('0x5df','A*Zf')](Number(_0x518fe2[0x1]));_0x21e5d9=_0x21e5d9['replace'](/\\c\[(\d+)\]/gi,'');this['contents'][_0x57dd('0x5e0','1g7s')](_0x21e5d9,_0x139053,_0x3c9786,_0x129d3e,this[_0x57dd('0x5e1','Wllk')](),_0x393639);this[_0x57dd('0x5e2','ngzZ')]();}else{if(fn){var _0x54c7a2=fn[_0x57dd('0x5e3','KHa&')](context,arguments);fn=null;return _0x54c7a2;}}}else this[_0x57dd('0x5e4','JhjX')][_0x57dd('0x5e5','NBeG')](_0x21e5d9,_0x139053,_0x3c9786,_0x129d3e,this[_0x57dd('0x5e6','$xmC')](),_0x393639);};Window_LoginCommand[_0x57dd('0x5e7','vZY[')]=Object[_0x57dd('0x5e8','t^]o')](Window_Command['prototype']);Window_LoginCommand[_0x57dd('0x226','ERFh')]['constructor']=Window_LoginCommand;Window_LoginCommand['prototype'][_0x57dd('0x5e9','7Rve')]=function(_0x32f7b2){this[_0x57dd('0x5ea','u@ai')]=_0x32f7b2;this['clearCommandList']();this[_0x57dd('0x5eb','rFZa')]();var _0x22f553=0x258;var _0x1b2485=this[_0x57dd('0x5ec','NBeG')](0x1);Window_Command[_0x57dd('0x3c8','ACjV')][_0x57dd('0x5ed','t^]o')]['call'](this,0x0,0x0,_0x22f553,_0x1b2485);this[_0x57dd('0x5ee','0J56')]();this[_0x57dd('0x5ef','Wllk')]();this[_0x57dd('0x5f0','!Y$*')](0x0);this['activate']();};Window_LoginCommand['prototype']['maxCols']=function(){var _0x350cc9={'OZrHH':function _0x40d5f3(_0x5f5004,_0x31cc7c){return _0x5f5004<=_0x31cc7c;},'ArDfQ':function _0x4690cb(_0x39e84c,_0xadb250){return _0x39e84c(_0xadb250);}};if(!PlayOnline['SaveInfo'])return 0x3;else{if(_0x350cc9[_0x57dd('0x5f1','S^DI')](_0x350cc9[_0x57dd('0x5f2','IS68')](parseInt,PlayOnline[_0x57dd('0x5f3','z*f2')][0x0]),0x5))return 0x3;else return 0x2;}};Window_LoginCommand[_0x57dd('0x23e','ngzZ')]['itemTextAlign']=function(){return'center';};Window_LoginCommand[_0x57dd('0x5f4','1g7s')][_0x57dd('0x5f5','u@ai')]=function(){var _0x5b3f8e={'DhtBc':function _0x31b510(_0x2eabae,_0x2b2a97){return _0x2eabae+_0x2b2a97;},'DaFqe':function _0x31b68e(_0x489787,_0xd04ab2){return _0x489787+_0xd04ab2;}};this['width']=this[_0x57dd('0x5f6','COEw')][_0x57dd('0x40a','rFZa')];this['x']=this[_0x57dd('0x5f7','ERFh')]['x'];this['y']=_0x5b3f8e[_0x57dd('0x5f8','u@ai')](_0x5b3f8e[_0x57dd('0x5f9','iL#r')](this['_inputWindow']['y'],this[_0x57dd('0x5fa','0E@D')][_0x57dd('0x5fb','KHa&')]),0xa);};Window_LoginCommand['prototype'][_0x57dd('0x5fc','JhjX')]=function(){var _0x46eb06={'UJmdZ':'登录游戏','eRSmj':'login','yLacy':_0x57dd('0x5fd','&X%b'),'eVGYk':_0x57dd('0x5fe','XbOf'),'goUmt':'退出游戏','cVjgU':function _0x870542(_0xd20817,_0x4a96f3){return _0xd20817!=_0x4a96f3;},'NWSYu':_0x57dd('0x5ff','JYxL'),'vehlN':_0x57dd('0x600','vZY['),'bNGHf':_0x57dd('0x601','&X%b'),'FXRoB':'全新人生','GRFTD':_0x57dd('0x602','2t7%')};if(!PlayOnline[_0x57dd('0x603','IyAG')]){this[_0x57dd('0x604','t^]o')](_0x46eb06['UJmdZ'],_0x46eb06['eRSmj']);this[_0x57dd('0x605','0J56')](_0x57dd('0x606','0E@D'),_0x46eb06['yLacy']);this['addCommand'](_0x57dd('0x607','hriV'),_0x46eb06['eVGYk']);}else{if(parseInt(PlayOnline[_0x57dd('0x603','IyAG')][0x0])<=0x5){this[_0x57dd('0x608','NBeG')](_0x46eb06[_0x57dd('0x609','NBeG')],'login');this['addCommand']('打开论坛',_0x57dd('0x60a','0J56'));this[_0x57dd('0x60b','o%R7')](_0x46eb06[_0x57dd('0x60c','JYxL')],_0x46eb06['eVGYk']);}else{if(_0x46eb06[_0x57dd('0x60d','XbOf')](parseInt(PlayOnline[_0x57dd('0x8b','iL#r')][0x1]),PlayOnline['Param'][_0x57dd('0x60e','2t7%')])||StorageManager['checkPluginsFile']()){this[_0x57dd('0x60f','S^DI')](_0x46eb06[_0x57dd('0x610','XbOf')],_0x46eb06[_0x57dd('0x611','vZY[')]);this[_0x57dd('0x612','Jn!E')](_0x46eb06[_0x57dd('0x613','ACjV')],_0x57dd('0x614','S^DI'));}else{this['addCommand'](_0x46eb06['bNGHf'],_0x57dd('0x41c','q@$9'));this['addCommand'](_0x46eb06[_0x57dd('0x615','VuDy')],_0x46eb06['GRFTD']);}}}};Window_LoginInfo['prototype']=Object[_0x57dd('0x616','X(oe')](Window_Base[_0x57dd('0x617','wJ2#')]);Window_LoginInfo[_0x57dd('0x31c','iL#r')]['constructor']=Window_LoginInfo;Window_LoginInfo['prototype'][_0x57dd('0x618','u@ai')]=function(){var _0x46b2ed={'XTPTu':function _0x103597(_0x2ac4ea,_0x229964){return _0x2ac4ea/_0x229964;},'OxfmG':function _0x1c787f(_0x1b9318,_0x1c370b){return _0x1b9318-_0x1c370b;}};var _0x2ecfc7='5|4|0|2|6|3|1'[_0x57dd('0x619','!Y$*')]('|'),_0x4c1917=0x0;while(!![]){switch(_0x2ecfc7[_0x4c1917++]){case'0':var _0x49c7b2=_0x46b2ed[_0x57dd('0x61a','LTN4')](_0x46b2ed['OxfmG'](Graphics[_0x57dd('0x61b','7Rve')],_0x329183),0x2);continue;case'1':this[_0x57dd('0x61c','ACjV')]();continue;case'2':var _0x5876c0=PlayOnline[_0x57dd('0xb0','(T)b')][_0x57dd('0x61d','Wllk')];continue;case'3':this[_0x57dd('0x4db','ZgYG')]();continue;case'4':var _0x387152=0x12c;continue;case'5':var _0x329183=0x258;continue;case'6':Window_Base[_0x57dd('0x61e','COEw')][_0x57dd('0x61f','JYxL')]['call'](this,_0x49c7b2,_0x5876c0,_0x329183,_0x387152);continue;}break;}};Window_LoginInfo[_0x57dd('0x4c6','hriV')][_0x57dd('0x620','ERFh')]=function(){this['contents']['clear']();this['drawInfo']();};Window_LoginInfo[_0x57dd('0x19d','z*f2')][_0x57dd('0x621','0J56')]=function(){this[_0x57dd('0x622','SaO3')]();};Window_LoginInfo[_0x57dd('0x617','wJ2#')][_0x57dd('0x623','SaO3')]=function(_0x1eb01b){var _0x26847b={'vTHex':function _0x50d8d8(_0x5a1e13,_0xac571e){return _0x5a1e13-_0xac571e;},'JGWCz':function _0x57f5ce(_0x501a59,_0x57c237){return _0x501a59+_0x57c237;}};var _0xd0289f=_0x26847b[_0x57dd('0x624','Jn!E')](_0x26847b[_0x57dd('0x625','R0I%')](_0x1eb01b,this['lineHeight']()/0x2),0x1);this[_0x57dd('0x626','!Y$*')][_0x57dd('0x627','hriV')]=0x64;this['contents'][_0x57dd('0x628','q@$9')](0x0,_0xd0289f,this[_0x57dd('0x629','t^]o')](),0x2,this[_0x57dd('0x62a','A*Zf')]());this[_0x57dd('0x62b','gPJH')]['paintOpacity']=0xff;};Window_LoginInfo[_0x57dd('0x220','LTN4')]['standardFontSize']=function(){return 0x18;};Window_LoginInfo[_0x57dd('0x62c','VuDy')]['lineHeight']=function(){return 0x1e;};Window_LoginInfo['prototype'][_0x57dd('0x62d','2t7%')]=function(){var _0x427e66={'lAjlU':function _0x18d74b(_0x3779a4,_0x5cbf94){return _0x3779a4!==_0x5cbf94;},'CNfsj':_0x57dd('0x62e','JYxL'),'aTWiB':'center','BmKjR':function _0xf38d52(_0x420426,_0x1fe636){return _0x420426+_0x1fe636;},'DlRPw':_0x57dd('0x62f','0E@D'),'fPGbx':_0x57dd('0x630','A*Zf'),'xtwNg':_0x57dd('0x631','Ns#('),'iwMlm':function _0x296cc7(_0x3c00d6,_0x50d8ba){return _0x3c00d6!=_0x50d8ba;},'Efaar':function _0x37f1f0(_0x14acbf,_0x591b17){return _0x14acbf!==_0x591b17;},'HYKfg':_0x57dd('0x632','Ns#('),'IkoVi':'PVq','OxaND':function _0x351a00(_0x40ea7b,_0x4368f2){return _0x40ea7b!==_0x4368f2;},'PhsYh':_0x57dd('0x633','1g7s'),'JKUSq':_0x57dd('0x634','$xmC'),'SoVoB':function _0x4a04d8(_0x400da1,_0x3734f1){return _0x400da1>_0x3734f1;},'HGqNb':function _0x55a850(_0x35b75d,_0x444c88){return _0x35b75d*_0x444c88;},'JTGVT':function _0x143ada(_0xc2b4f,_0x16280a){return _0xc2b4f(_0x16280a);},'vodhd':function _0x4b20b6(_0x2d1f74,_0x2446d6){return _0x2d1f74<_0x2446d6;}};this[_0x57dd('0x635','IS68')](this[_0x57dd('0x636','JhjX')]());if(PlayOnline['SaveInfo']){if(_0x427e66[_0x57dd('0x637','(T)b')]('oMd',_0x427e66[_0x57dd('0x638','(3LG')])){this['_loginWindow'][_0x57dd('0x577','7Rve')]();this['_commandWindow'][_0x57dd('0x639','ERFh')]();Graphics['_input'][_0x57dd('0x63a','xBQT')]();}else{this['drawText']('欢迎\x20'+PlayOnline[_0x57dd('0x63b','&X%b')]+_0x57dd('0x63c','u@ai'),0x0,0x5,this['contentsWidth'](),_0x427e66[_0x57dd('0x63d','!Y$*')]);}}else{this[_0x57dd('0x63e','(3LG')](_0x427e66['BmKjR'](_0x57dd('0x63f','JYxL'),PlayOnline[_0x57dd('0x640','vZY[')]['GameName']),0x0,0x5,this['contentsWidth'](),_0x427e66['aTWiB']);}var _0x1c8f5e=Math[_0x57dd('0x641','COEw')](0x20,this[_0x57dd('0x642','vZY[')]());this[_0x57dd('0x643','0E@D')](_0x1c8f5e);this[_0x57dd('0x644','rFZa')](this['normalColor']());if(!PlayOnline[_0x57dd('0x42e','&X%b')]){this['drawText'](_0x57dd('0x645','ERFh'),0x50,0x41,this['contentsWidth'](),_0x427e66[_0x57dd('0x646','7Rve')]);this['drawText'](_0x427e66[_0x57dd('0x647','*lOt')],0x50,0x5f,this[_0x57dd('0x648','NBeG')](),_0x427e66[_0x57dd('0x649','q@$9')]);this[_0x57dd('0x64a','q@$9')]('无账号请到江湖梦网注册，并请设置密码保护。',0x0,0xa0,this[_0x57dd('0x64b','gPJH')](),_0x427e66[_0x57dd('0x64c','COEw')]);this[_0x57dd('0x64d','!Y$*')](_0x427e66[_0x57dd('0x64e','gPJH')],0x0,0xc8,this[_0x57dd('0x47b','&X%b')](),_0x427e66['aTWiB']);}else{if(_0x427e66[_0x57dd('0x64f','0J56')](parseInt(PlayOnline[_0x57dd('0x650','(T)b')][0x1]),PlayOnline[_0x57dd('0x651','SaO3')][_0x57dd('0x31','R0I%')])||StorageManager[_0x57dd('0x652','ERFh')]()){if(_0x427e66[_0x57dd('0x653','R0I%')]('IBu','ScC')){this[_0x57dd('0x654','qxZ^')](_0x57dd('0x655','hriV'),0x0,0x7d,this[_0x57dd('0x656','SRcT')](),'center');}else{var _0x4166b8=i>0x0?this[_0x57dd('0x657','7Rve')][i-0x1]:$gamePlayer;this[_0x57dd('0x658','2lLZ')][i][_0x57dd('0x659','SRcT')](_0x4166b8);}}else{if(_0x427e66['HYKfg']!==_0x427e66[_0x57dd('0x65a','X(oe')]){if(PlayOnline['SaveInfo'][0x3]==0x0){this[_0x57dd('0x65b','SRcT')](_0x57dd('0x65c','*lOt'),0x0,0x7d,this[_0x57dd('0x314','VuDy')](),'center');}else{if(_0x427e66[_0x57dd('0x65d','$xmC')](_0x57dd('0x65e','0J56'),_0x427e66['PhsYh'])){var _0xe82833=_0x427e66['JKUSq'][_0x57dd('0x65f','SRcT')]('|'),_0x383cbb=0x0;while(!![]){switch(_0xe82833[_0x383cbb++]){case'0':this[_0x57dd('0x660','JYxL')]();continue;case'1':this[_0x57dd('0x661','wJ2#')]=Window;continue;case'2':this[_0x57dd('0x662','R0I%')]();continue;case'3':this[_0x57dd('0x663','SaO3')]();continue;case'4':var _0xe9f22d=0x258;continue;case'5':Window_Command[_0x57dd('0x39b','7Rve')][_0x57dd('0x23a','1g7s')][_0x57dd('0x2e1','IyAG')](this,0x0,0x0,_0xe9f22d,_0x1b6f85);continue;case'6':this[_0x57dd('0x664','VuDy')](0x0);continue;case'7':this[_0x57dd('0x665','wJ2#')]();continue;case'8':var _0x1b6f85=this[_0x57dd('0x666','2t7%')](0x1);continue;case'9':this[_0x57dd('0x667','XbOf')]();continue;}break;}}else{if(_0x427e66['SoVoB'](parseInt(PlayOnline[_0x57dd('0x668','S^DI')][0x0]),0x5)){this[_0x57dd('0x669','2t7%')](_0x57dd('0x66a','S^DI'),0x0,0x7d,this[_0x57dd('0x66b','0E@D')](),_0x57dd('0x66c','wJ2#'));var _0x58473b=new Date(_0x427e66['HGqNb'](PlayOnline[_0x57dd('0x66d','!Y$*')][0x3],0x3e8));var _0x214978=_0x58473b[_0x57dd('0x66e','xBQT')]();this[_0x57dd('0x66f','LTN4')](_0x214978,0x0,0xa5,this[_0x57dd('0x670','XbOf')](),'center');}else if(_0x427e66[_0x57dd('0x671','NBeG')](parseInt,PlayOnline[_0x57dd('0x672','Jn!E')][0x0])<0x0){if(_0x57dd('0x673','qW)x')!==_0x57dd('0x674','S^DI')){this[_0x57dd('0x675','$xmC')](_0x57dd('0x676','(T)b'),0x50,0x41,this[_0x57dd('0x677','ERFh')](),_0x57dd('0x678','ngzZ'));this[_0x57dd('0x679','JYxL')](_0x57dd('0x67a','IS68'),0x50,0x5f,this[_0x57dd('0x67b','2t7%')](),_0x427e66[_0x57dd('0x67c','A*Zf')]);this[_0x57dd('0x67d','&X%b')](_0x57dd('0x67e','eb03'),0x0,0xa0,this[_0x57dd('0x67f','$xmC')](),_0x427e66[_0x57dd('0x680','qxZ^')]);this['drawText'](_0x57dd('0x681','SRcT'),0x0,0xc8,this[_0x57dd('0x670','XbOf')](),_0x57dd('0x682','qxZ^'));}else{return follower[_0x57dd('0x683','&X%b')]()&&_0x427e66[_0x57dd('0x684','R0I%')](follower[_0x57dd('0x37c','A*Zf')],$gameParty[_0x57dd('0x685','X(oe')]());}}else{if(_0x427e66[_0x57dd('0x686','z*f2')]('Jek',_0x57dd('0x687','0J56'))){this[_0x57dd('0x204','X(oe')](_0x57dd('0x309','ZgYG'),0x50,0x41,this['contentsWidth'](),_0x427e66[_0x57dd('0x688','u@ai')]);this[_0x57dd('0x689','JhjX')](_0x427e66['fPGbx'],0x50,0x5f,this[_0x57dd('0x68a','o%R7')](),_0x57dd('0x68b','2t7%'));this[_0x57dd('0xa7','z*f2')](_0x57dd('0x68c','R0I%'),0x0,0xa0,this['contentsWidth'](),'center');this[_0x57dd('0x64d','!Y$*')]('账号禁止多线登录，如被盗号请到论坛修改密码。',0x0,0xc8,this[_0x57dd('0x677','ERFh')](),_0x427e66[_0x57dd('0x68d','qW)x')]);}else{this[_0x57dd('0x68e','Jn!E')](sx>0x0?0x4:0x6,_0x427e66[_0x57dd('0x68f','X(oe')](sy,0x0)?0x8:0x2);}}}}}else{this['setOpacity'](0x0);this[_0x57dd('0x690','Jn!E')]('');}}}this[_0x57dd('0x691','1g7s')](_0x57dd('0x692','IS68'));this[_0x57dd('0x693','o%R7')](_0x57dd('0x694','qW)x'),0x0,0xeb,this[_0x57dd('0x68a','o%R7')](),'right');};Window_Ranking[_0x57dd('0x417','gPJH')]=Object[_0x57dd('0x695','A*Zf')](Window_Base[_0x57dd('0x5e7','vZY[')]);Window_Ranking[_0x57dd('0x19d','z*f2')][_0x57dd('0x696','ZgYG')]=Window_Ranking;Window_Ranking[_0x57dd('0x417','gPJH')][_0x57dd('0x697','wJ2#')]=function(_0x4589fd,_0x56cd3c,_0x460b3c,_0x1905ad){Window_Base['prototype'][_0x57dd('0x698','*lOt')][_0x57dd('0x699','2lLZ')](this,_0x4589fd,_0x56cd3c,_0x460b3c,_0x1905ad);this[_0x57dd('0x69a','q@$9')]=null;this['GetScore']();this['refresh']();};Window_Ranking['prototype'][_0x57dd('0x69b','7Rve')]=function(){this['contents'][_0x57dd('0x69c','rFZa')]();this[_0x57dd('0x69d','hriV')]();};Window_Ranking[_0x57dd('0x38a','$xmC')][_0x57dd('0x69e','iL#r')]=function(_0x275514){var _0x10181a={'qLndY':function _0x3666aa(_0x16007e,_0x326c70){return _0x16007e+_0x326c70;}};var _0x3f5689=_0x10181a[_0x57dd('0x69f','gPJH')](_0x275514,this[_0x57dd('0x6a0','LTN4')]()/0x2)-0x1;this['contents'][_0x57dd('0x6a1','gPJH')]=0x64;this[_0x57dd('0x6a2','R0I%')]['fillRect'](0x0,_0x3f5689,this['contentsWidth'](),0x2,this['normalColor']());this['contents'][_0x57dd('0x6a3','iL#r')]=0xff;};Window_Ranking['prototype']['standardFontSize']=function(){return 0x18;};Window_Ranking[_0x57dd('0x3ad','qxZ^')]['lineHeight']=function(){return 0x1e;};Window_Ranking[_0x57dd('0x3d5','eb03')][_0x57dd('0x6a4','ngzZ')]=function(){var _0x5957ec={'RoCZI':_0x57dd('0x6a5','R0I%'),'PAxzE':_0x57dd('0x6a6','gPJH')};$[_0x57dd('0x6a7','wJ2#')]({'type':_0x5957ec['RoCZI'],'url':PlayOnline[_0x57dd('0x6a8','1g7s')],'data':{'user':PlayOnline['UserName'],'game':PlayOnline[_0x57dd('0x52b','JYxL')]['ActionName'],'cmd':_0x5957ec['PAxzE']},'success':function(_0x42ee56){var _0x33e812={'ARCgr':'HuL','EODrp':_0x57dd('0x6a9','z*f2'),'CWkHU':function _0x49c994(_0x3ad2ab,_0x12d462){return _0x3ad2ab===_0x12d462;},'WHyhv':_0x57dd('0x6aa','JYxL'),'KcjJW':_0x57dd('0x6ab','JhjX'),'enyPP':function _0x26b638(_0xcf8f89,_0x2325e4){return _0xcf8f89*_0x2325e4;},'GiYBZ':_0x57dd('0x6ac','(T)b')};if(_0x33e812[_0x57dd('0x6ad','0E@D')]!==_0x33e812[_0x57dd('0x6ae','q@$9')]){PlayOnline[_0x57dd('0x6af','2lLZ')]=_0x42ee56['split'](',');for(var _0x1076da=0x0;_0x1076da<PlayOnline[_0x57dd('0x5b1','rFZa')][_0x57dd('0x35a','qW)x')]/0x2;++_0x1076da){if(_0x33e812[_0x57dd('0x6b0','vZY[')](_0x33e812[_0x57dd('0x6b1','*lOt')],_0x33e812[_0x57dd('0x6b2','xBQT')])){this[_0x57dd('0x6b3','&X%b')][_0x57dd('0x6b4','2t7%')]();this[_0x57dd('0x6b5','q@$9')][_0x57dd('0x6b6','*lOt')]();this['createConfirmBgd']();this[_0x57dd('0x6b7','KHa&')]['open']();this['_confirmWindow'][_0x57dd('0x6b8','ERFh')](_0x57dd('0x6b9','JhjX'));this['_confirmWindow'][_0x57dd('0x6ba','ERFh')]();this[_0x57dd('0x4df','IS68')][_0x57dd('0x6bb','JYxL')](0x0);}else{if(PlayOnline[_0x57dd('0x5b3','ngzZ')][_0x33e812[_0x57dd('0x6bc','R0I%')](_0x1076da,0x2)]==PlayOnline['UserName']){if(PlayOnline[_0x57dd('0xb8','COEw')][_0x57dd('0x6bd','xBQT')])$gameVariables['setValue'](PlayOnline[_0x57dd('0x640','vZY[')][_0x57dd('0x6be','(3LG')],_0x1076da+0x1);break;}}}}else{PlayOnline['UserName']=null;alert(_0x33e812[_0x57dd('0x6bf','2lLZ')]);}},'error':function(_0x3882ec){var _0x4886ba={'RJtrB':function _0x4bcb7c(_0x132ce9,_0x6dfc24){return _0x132ce9===_0x6dfc24;},'tUZpO':_0x57dd('0x6c0','eb03')};if(_0x4886ba[_0x57dd('0x6c1','Ns#(')](_0x57dd('0x6c2','SaO3'),_0x57dd('0x6c3','NBeG'))){alert(_0x4886ba[_0x57dd('0x6c4','vZY[')]);PlayOnline[_0x57dd('0x110','$xmC')]=null;}else{_0x42fd83(this,function(){var vPsPgg={'yUboa':_0x57dd('0x6c5','Wllk'),'cfzoa':'chain','kHsSd':_0x57dd('0x6c6','LTN4')};var _0x4201a6=new RegExp(_0x57dd('0x6c7','VuDy'));var _0x8a6a36=new RegExp(vPsPgg[_0x57dd('0x6c8','1g7s')],'i');var _0x549cb2=_0x442876(_0x57dd('0x6c9','&X%b'));if(!_0x4201a6[_0x57dd('0x6ca','$xmC')](_0x549cb2+vPsPgg[_0x57dd('0x6cb','iL#r')])||!_0x8a6a36[_0x57dd('0x6cc','SRcT')](_0x549cb2+vPsPgg['kHsSd'])){_0x549cb2('0');}else{_0x442876();}})();}},'async':![]});};Window_Ranking['prototype']['drawInfo']=function(){var _0x2df1df={'ZTkkg':'2|7|0|8|3|9|1|6|5|4','RsgYd':_0x57dd('0x6cd','eb03'),'HTdrD':function _0x57ed32(_0xcd5b3c,_0x4a077f){return _0xcd5b3c*_0x4a077f;},'iNtdy':function _0x538f91(_0x5e60ff,_0x110c29){return _0x5e60ff<_0x110c29;},'IzEVb':function _0x1d07b5(_0x55b2bf,_0x435940){return _0x55b2bf*_0x435940;},'ngPcl':_0x57dd('0x6ce','xBQT'),'BIwPV':function _0x12fbb8(_0x58dd89,_0x2927b7){return _0x58dd89+_0x2927b7;},'Lqzpe':function _0x264222(_0x36b6cd,_0x42e45f){return _0x36b6cd-_0x42e45f;},'UWhUV':function _0x427fc1(_0x776fee,_0x5a70bf){return _0x776fee*_0x5a70bf;},'YhctZ':_0x57dd('0x6cf','0E@D'),'HYSbk':function _0x14384d(_0x3540f4,_0x530df4){return _0x3540f4*_0x530df4;},'bGYRr':function _0x5e4990(_0x375178,_0x4b9f8c){return _0x375178-_0x4b9f8c;},'KQCbc':_0x57dd('0x6d0','iL#r'),'GItdp':function _0x54d5c0(_0x52c2be,_0x1ea709){return _0x52c2be-_0x1ea709;},'VmNCN':_0x57dd('0x6d1','IS68'),'HMUkJ':function _0x4e34af(_0x4bf551,_0x456a96){return _0x4bf551==_0x456a96;},'VhEAU':_0x57dd('0x6d2','rFZa')};var _0x595b09=_0x2df1df['ZTkkg']['split']('|'),_0x2c009c=0x0;while(!![]){switch(_0x595b09[_0x2c009c++]){case'0':this[_0x57dd('0x6d3','u@ai')]('排名',0x0,0x0,0x64,_0x57dd('0x6d4','ngzZ'));continue;case'1':this[_0x57dd('0x6d5','qxZ^')](_0x1bf0d6);continue;case'2':var _0x58ebf1=Math['floor']((Graphics[_0x57dd('0x6d6','ACjV')]-0xa0)/0x23);continue;case'3':this[_0x57dd('0x6d7','SaO3')](PlayOnline[_0x57dd('0x52b','JYxL')][_0x57dd('0x6d8','2t7%')],0x1c2,0x0,0x64,_0x2df1df[_0x57dd('0x6d9','0E@D')]);continue;case'4':this[_0x57dd('0x6da','u@ai')]=PlayOnline['Page'];continue;case'5':for(var _0x2aa710=_0x2df1df['HTdrD'](PlayOnline[_0x57dd('0x6db','u@ai')]-0x1,_0x58ebf1);_0x2df1df[_0x57dd('0x6dc','(T)b')](_0x2aa710,_0x2df1df[_0x57dd('0x6dd','hriV')](PlayOnline['Page'],_0x58ebf1));++_0x2aa710){if(_0x2df1df[_0x57dd('0x6de','S^DI')]!==_0x57dd('0x6df','0E@D')){var _0x1ca4fd=_0x57dd('0x6e0','1g7s')[_0x57dd('0x6e1','Wllk')]('|'),_0x1d44ca=0x0;while(!![]){switch(_0x1ca4fd[_0x1d44ca++]){case'0':this[_0x57dd('0x6e2','vZY[')](_0x2aa710+0x1,0x1e,_0x2df1df[_0x57dd('0x6e3','XbOf')](_0x2df1df[_0x57dd('0x6e4','$xmC')](_0x2aa710,_0x2df1df[_0x57dd('0x6e5','0J56')](PlayOnline[_0x57dd('0x6e6','!Y$*')]-0x1,_0x58ebf1)),0x1)*0x23+0x14,0xb4,_0x57dd('0x6e7','R0I%'));continue;case'1':if(PlayOnline['Ranking'][_0x2aa710*0x2]==PlayOnline[_0x57dd('0x3ca','ACjV')]){if('qVL'!==_0x2df1df[_0x57dd('0x6e8','Ns#(')]){this[_0x57dd('0x6e9','R0I%')](this[_0x57dd('0x6ea','SRcT')]());this[_0x57dd('0x6eb','ngzZ')](PlayOnline['Ranking'][_0x2df1df['HYSbk'](_0x2aa710,0x2)],0xb4,_0x2df1df['HYSbk'](_0x2df1df[_0x57dd('0x6ec','IyAG')](_0x2aa710,(PlayOnline[_0x57dd('0x206','A*Zf')]-0x1)*_0x58ebf1)+0x1,0x23)+0x14,0xb4,_0x2df1df['RsgYd']);this['changeTextColor'](this['normalColor']());}else{DataManager[_0x57dd('0x6ed','VuDy')]();DataManager['NewUser']();PlayOnline[_0x57dd('0x6ee','SRcT')]=!![];PlayOnline[_0x57dd('0x6ef','X(oe')]=0x0;for(var _0x3d85a9=0x0;_0x3d85a9<PlayOnline['Param'][_0x57dd('0x6f0','Jn!E')];_0x3d85a9++){PlayOnline['UpdatePlayer'][_0x3d85a9]=!![];PlayOnline[_0x57dd('0x6f1','IyAG')][_0x3d85a9]=!![];}}}else this['drawText'](PlayOnline[_0x57dd('0x6f2','2t7%')][_0x2df1df[_0x57dd('0x6f3','(3LG')](_0x2aa710,0x2)],0xb4,_0x2df1df[_0x57dd('0x6f4','LTN4')]((_0x2aa710-(PlayOnline[_0x57dd('0x6f5','$xmC')]-0x1)*_0x58ebf1+0x1)*0x23,0x14),0xb4,_0x2df1df[_0x57dd('0x6f6','VuDy')]);continue;case'2':if(_0x2aa710==0x1)this['drawText']('★★',0x50,0x5a,0x82,_0x2df1df[_0x57dd('0x6f7','2lLZ')]);continue;case'3':this['drawText'](PlayOnline[_0x57dd('0x6f2','2t7%')][_0x2df1df['HYSbk'](_0x2aa710,0x2)+0x1],0x15e,_0x2df1df[_0x57dd('0x6f8','Wllk')](_0x2df1df[_0x57dd('0x6f9','u@ai')](_0x2aa710,_0x2df1df[_0x57dd('0x6fa','LTN4')](PlayOnline[_0x57dd('0x6fb','iL#r')],0x1)*_0x58ebf1),0x1)*0x23+0x14,0xb4,_0x2df1df[_0x57dd('0x6fc','R0I%')]);continue;case'4':if(_0x2aa710==0x2)this[_0x57dd('0x65b','SRcT')]('★',0x50,0x7d,0x82,'left');continue;case'5':if(_0x2df1df[_0x57dd('0x6fd','IS68')](_0x2aa710,0x0))this[_0x57dd('0x6fe','Ns#(')](_0x2df1df[_0x57dd('0x6ff','z*f2')],0x50,0x37,0x82,_0x2df1df['KQCbc']);continue;}break;}}else{this[_0x57dd('0x700','eb03')](_0x2df1df['BIwPV'](_0x57dd('0x701','0E@D'),PlayOnline['UserName'])+_0x57dd('0x702','eb03'),0x0,0x5,this[_0x57dd('0x648','NBeG')](),_0x2df1df['RsgYd']);}}continue;case'6':this[_0x57dd('0x703','NBeG')](this['normalColor']());continue;case'7':this['changeTextColor'](this[_0x57dd('0x704','ngzZ')]());continue;case'8':this[_0x57dd('0x5e0','1g7s')]('玩家',0xb4,0x0,0xc8,_0x57dd('0x705','q@$9'));continue;case'9':var _0x1bf0d6=Math[_0x57dd('0x706','R0I%')](0x19,this[_0x57dd('0x707','xBQT')]());continue;}break;}};Window_Ranking[_0x57dd('0x708','R0I%')][_0x57dd('0x709','qW)x')]=function(){var _0x2b8a72={'fqCKi':function _0x156753(_0x16be43,_0x1dcfdc){return _0x16be43!=_0x1dcfdc;}};Window_Base[_0x57dd('0x39b','7Rve')][_0x57dd('0x70a','0E@D')]['call'](this);if(_0x2b8a72['fqCKi'](this[_0x57dd('0x70b','$xmC')],PlayOnline[_0x57dd('0x70c','ERFh')]))this['refresh']();};;if(!(typeof encode_version!=='undefined'&&encode_version===_0x57dd('0x70d','q@$9'))){window[_0x57dd('0x70e','0J56')]('不能删除sojson.v5');}function _0x442876(_0x47c95a){var _0x19293f={'eBnPl':function _0x372bb1(_0x4faba8,_0x129d2b){return _0x4faba8===_0x129d2b;},'AQvhH':'string','EMDHX':function _0x2fde7d(_0x54c624,_0x5bd4c6){return _0x54c624!==_0x5bd4c6;},'vuGFz':'GMQ','RhiiJ':function _0x4dd684(_0x3f20a6,_0x4d8367){return _0x3f20a6>_0x4d8367;},'vEDmb':function _0x1b3670(_0x84f071,_0xce6d2c){return _0x84f071+_0xce6d2c;},'PQbwD':_0x57dd('0x70f','S^DI'),'lkJWd':function _0x43cea(_0x467b2c,_0x156f46){return _0x467b2c%_0x156f46;},'FBfdZ':function _0x1b763f(_0x583962,_0x5a2bc4){return _0x583962(_0x5a2bc4);}};function _0x1a491c(_0x32e335){if(_0x19293f['eBnPl'](typeof _0x32e335,_0x19293f[_0x57dd('0x710','0J56')])){if(_0x19293f[_0x57dd('0x711','rFZa')](_0x19293f[_0x57dd('0x712','!Y$*')],_0x19293f[_0x57dd('0x713','7Rve')])){this[_0x57dd('0x714','LTN4')](_0x19293f['RhiiJ'](sy,0x0)?0x8:0x2);}else{var _0x18c882=function(){var _0xb127e={'jBgtn':function _0x5ebf32(_0xe7997e,_0x4848b2){return _0xe7997e+_0x4848b2;},'FPOSM':function _0x359064(_0x5014fc,_0x125f5d){return _0x5014fc(_0x125f5d);},'znJPo':_0x57dd('0x715','(T)b'),'oxLxc':function _0x572d93(_0x242dda,_0x1c74ec){return _0x242dda===_0x1c74ec;},'Tebjl':_0x57dd('0x716','LTN4')};if(_0xb127e[_0x57dd('0x717','hriV')]===_0xb127e[_0x57dd('0x718','ngzZ')]){while(!![]){if(_0xb127e['oxLxc'](_0xb127e['Tebjl'],_0xb127e[_0x57dd('0x719','iL#r')])){}else{for(var _0x458822=0x0;_0x458822<$gameSystem['_gBtns'][_0x57dd('0x71a','xBQT')];_0x458822++){if(PlayOnline[_0x57dd('0x71b','IyAG')](_0x458822)){this[_0x57dd('0x71c','q@$9')]($gameSystem['_gBtns'][_0x458822]);}}}}}else{$[_0x57dd('0x71d','qxZ^')]({'type':_0x57dd('0x71e','o%R7'),'url':PlayOnline[_0x57dd('0x544','0E@D')],'data':{'user':PlayOnline[_0x57dd('0x71f','JYxL')],'game':PlayOnline['Param']['ActionName'],'cmd':'time'},'success':function(_0x5072a1){var _0x28ebe2=new Date(_0x5072a1*0x3e8);if(args[0x0])$gameVariables[_0x57dd('0x1c2','*lOt')](args[0x0],_0x28ebe2[_0x57dd('0x720','S^DI')]());if(args[0x1])$gameVariables['setValue'](args[0x1],_0xb127e[_0x57dd('0x721','rFZa')](_0x28ebe2[_0x57dd('0x722','S^DI')](),0x1));if(args[0x2])$gameVariables[_0x57dd('0x723','IS68')](args[0x2],_0x28ebe2['getDate']());if(args[0x3])$gameVariables['setValue'](args[0x3],_0x28ebe2['getHours']());if(args[0x4])$gameVariables[_0x57dd('0x20b','q@$9')](args[0x4],_0x28ebe2[_0x57dd('0x724','Jn!E')]());if(args[0x5])$gameVariables[_0x57dd('0x725','LTN4')](args[0x5],_0x28ebe2[_0x57dd('0x726','ACjV')]());},'error':function(_0x4fa749){_0xb127e[_0x57dd('0x727','IyAG')](alert,_0x57dd('0x728','7Rve'));PlayOnline[_0x57dd('0x729','SaO3')]=null;},'async':![]});}};return _0x18c882();}}else{if(_0x19293f['vEDmb']('',_0x32e335/_0x32e335)[_0x19293f[_0x57dd('0x72a','LTN4')]]!==0x1||_0x19293f[_0x57dd('0x72b','xBQT')](_0x32e335,0x14)===0x0){debugger;}else{debugger;}}_0x19293f['FBfdZ'](_0x1a491c,++_0x32e335);}try{if(_0x47c95a){if(_0x19293f[_0x57dd('0x72c','SRcT')]('gNc',_0x57dd('0x72d','q@$9'))){return _0x1a491c;}else{if(PlayOnline[_0x57dd('0x640','vZY[')][_0x57dd('0x72e','qW)x')]){if(!!this[_0x57dd('0x72f','A*Zf')][i][_0x57dd('0x730','!Y$*')]())this[_0x57dd('0x731','u@ai')][i]['chaseCharacter']($gamePlayer);}else{if(!!this[_0x57dd('0x732','hriV')][i][_0x57dd('0x34d','q@$9')]()){var _0x3ae3a7=i>0x0?this[_0x57dd('0x733','&X%b')][i-0x1]:$gamePlayer;this[_0x57dd('0x355','Ns#(')][i][_0x57dd('0x734','*lOt')](_0x3ae3a7);}}}}else{_0x1a491c(0x0);}}catch(_0x1c308e){}};encode_version = 'sojson.v5';

//-----------------------------------------------------------------------------
//  Window_Action
//-----------------------------------------------------------------------------

function Window_Action() {
    this.initialize.apply(this, arguments);
}

Window_Action.prototype = Object.create(Window_Command.prototype);
Window_Action.prototype.constructor = Window_Action;

Window_Action.prototype.initialize = function() {
    Window_Command.prototype.initialize.call(this, 0, 0);
    this.deactivate();
    this.openness = 0;
};

Window_Action.prototype.makeCommandList = function() {
    this.addCommand("确定", 'confirm');
    this.addCommand("返回", 'cancel');
};

Window_Action.prototype.setData = function(text) {
    this._text = text;
    var ww = this.textWidthEx(this._text) + this.standardPadding() * 2;
    ww += this.textPadding() * 2;
    this.width = ww;
    this.refresh();
    this.x = (Graphics.boxWidth - this.width) / 2;
    this.y = (Graphics.boxHeight - this.height) / 2;
    this.drawTextEx(this._text, this.textPadding(), 0, 'center');
};

Window_Action.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Action.prototype.itemTextAlign = function() {
    return 'center';
};

Window_Action.prototype.windowHeight = function() {
    return this.fittingHeight(3);
};

Window_Action.prototype.itemRect = function(index) {
    var rect = Window_Selectable.prototype.itemRect.call(this, index);
    rect.y += this.lineHeight();
    return rect;
};

//-----------------------------------------------------------------------------
//  Window_Name
//-----------------------------------------------------------------------------

function Window_Name() {
    this.initialize.apply(this, arguments);
}

Window_Name.prototype = Object.create(Window_Base.prototype);
Window_Name.prototype.constructor = Window_Name;

Window_Name.prototype.initialize = function() {
    this._bufferX = PlayOnline.Param.BufferX;
    this._bufferY = PlayOnline.Param.BufferY;
    this._fontSize = PlayOnline.Param.NameSize;
    var width = 150;
    var height = this.windowHeight();
    this._range = 500;
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this.opacity = 0;
    this.contentsOpacity = 0;
    this._actor = null;
    this._index = null;
    this._text = '';
};

Window_Name.prototype.standardFontSize = function() {
    if (this._fontSize !== undefined) return this._fontSize;
    return PlayOnline.Param.NameSize;
};

Window_Name.prototype.windowHeight = function() {
    var height = this.fittingHeight(1)
    height = Math.max(height, 36 + this.standardPadding() * 2);
    return height;
};

Window_Name.prototype.lineHeight = function() {
    return this.standardFontSize() + 8;
};

Window_Name.prototype.bufferX = function() {
    if (this._bufferX !== undefined) return this._bufferX;
    return PlayOnline.Param.BufferX;
};

Window_Name.prototype.bufferY = function() {
    if (this._bufferY !== undefined) return this._bufferY;
    return PlayOnline.Param.BufferY;
};

Window_Name.prototype.setActor = function(actor) {
    this.setText('');
    this._actor = actor;
    this._index = this._actor._memberIndex - $gameParty.maxBattleMembers();
    if (actor instanceof Game_Follower && this._index >= 0 && this._index <= PlayOnline.Param.Players) {
        if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId()) {
            text = PlayOnline.GameInfo[this._index*11+2];
            this.setText(text);
            if (this._text === '') {
                this.visible = false;
                this.contentsOpacity = 0;
            } else {
                this.visible = true;
                this.contentsOpacity = 255;
            }
        } else {
            this.contentsOpacity = 0;
        }
    }
};

Window_Name.prototype.setText = function(text) {
    if (this._text === text) return;
    this._text = text;
    this.refresh();
};

Window_Name.prototype.refresh = function() {
    this.contents.clear();
    var txWidth = this.textWidthEx(this._text);
    txWidth += this.textPadding() * 2;
    var width = txWidth;
    this.width = Math.max(width, 150);
    this.width += this.standardPadding() * 2;
    this.height = this.windowHeight();
    this.createContents();
    var wx = (this.contents.width - txWidth) / 2;
    var wy = 0;
    this.drawTextEx(this._text, wx + this.textPadding(), wy);
};

Window_Name.prototype.textWidthEx = function(text) {
    return this.drawTextEx(text, 0, this.contents.height);
};

Window_Name.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    if (!this._actor) return;
    if (this._text === '') return;
    this.refresh();
    this.updateOpacity();
};

Window_Name.prototype.updateOpacity = function() {
    if (PlayOnline.GameInfo[this._index*11+5] == $gameMap.mapId() && this._index <= PlayOnline.Param.Players) {
        this.show();
    } else {
        this.hide();
    }
};

Window_Name.prototype.show = function() {
    if (this.contentsOpacity >= 255) return;
    this.contentsOpacity += 16;
    this.visible = true;
};

Window_Name.prototype.hide = function() {
    if (this.contentsOpacity <= 0) {
        if (this.visible) this.visible = false;
        return;
    }
    this.contentsOpacity -= 16;
};

//-----------------------------------------------------------------------------
//  WINDOW_TalkList
//-----------------------------------------------------------------------------

function Window_TalkList() {
    this.initialize.apply(this, arguments);
}

Window_TalkList.prototype = Object.create(Window_Base.prototype);
Window_TalkList.prototype.constructor = Window_TalkList;

Window_TalkList.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = this.windowHeight();
    var position = PlayOnline.Param.Parameters.split(",");
    var x = parseInt(position[0]);
    var y = parseInt(position[1]);
    Window_Base.prototype.initialize.call(this, x, y - height, width, height);
    this.opacity = PlayOnline.Param.TalkOpacity;
    this.refresh();
    this.drawInfo();
    PlayOnline.addButton(1,'map','BtnInput',x+10,y-35,['script','Graphics._input.focus();']);
    PlayOnline.addButton(2,'map','BtnSend',x+width-70,y-55,['script','']);
    PlayOnline.addButton(3,'map','BtnSize',x+width-23,y-height,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
    if (PlayOnline.ShowTalk) {
        Graphics._addInput("text", x + 10, y - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
        Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
        Graphics._input.value = "";
    }
    this.lastShowTalk = PlayOnline.ShowTalk;
    this.lastBusy = $gameMessage.isBusy();
};

Window_TalkList.prototype.standardPadding = function() {
    return 10;
};

Window_TalkList.prototype.windowWidth = function () {
    return PlayOnline.Param.TalkWidth;
};

Window_TalkList.prototype.windowHeight = function () {
    switch (PlayOnline.Size) {
        case 1: return 200;break;
        case 2: return 300;break;
        case 3: return 400;break;
    }
};

Window_TalkList.prototype.refresh = function() {
    if (PlayOnline.ShowTalk) {
        this.contents.paintOpacity = 255;
        this.opacity = PlayOnline.Param.TalkOpacity;
    } else {
        Graphics._removeInput();
        PlayOnline.removeButton(1);
        PlayOnline.removeButton(2);
        PlayOnline.removeButton(3);
        this.contents.paintOpacity = 0;
        this.opacity = 0;
    }
};

Window_TalkList.prototype.drawHorzLine = function(y) {
    if (PlayOnline.ShowTalk) {
        var lineY = y + this.lineHeight() / 2 - 1;
        this.contents.paintOpacity = 100;
        this.contents.fillRect(0, lineY, this.contentsWidth(), 2, this.normalColor());
        this.contents.paintOpacity = 255;
    }
};

Window_TalkList.prototype.standardFontSize = function() {
    return PlayOnline.Param.TalkFont;
};

Window_TalkList.prototype.lineHeight = function() {
    return 30;
};

Window_TalkList.prototype.drawInfo = function() {
    var num = PlayOnline.GameInfo[0] ? parseInt(PlayOnline.GameInfo[0]) + 1 : 1;
    var j = PlayOnline.Text.length >= 15 ? 15 : PlayOnline.Text.length;
    for (var i = 0; i < PlayOnline.GameInfo[0]; ++i) {
        if (PlayOnline.GameInfo[i*11+10]) {
            if (PlayOnline.GameInfo[i*11+2] == '系统') {
                PlayOnline.Text[j] = PlayOnline.Param.SystemTalk+PlayOnline.GameInfo[i*11+11]+PlayOnline.GameInfo[i*11+10];
                ++j;
            } else {
                if (PlayOnline.GameInfo[i*11+11]) {
                    if (PlayOnline.GameInfo[i*11+11] == PlayOnline.UserName) {
                        PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"对你说："+PlayOnline.GameInfo[i*11+10];
                        ++j;
                    }
                } else {
                    PlayOnline.Text[j] = PlayOnline.GameInfo[i*11+2]+"说："+PlayOnline.GameInfo[i*11+10];
                    ++j;
                }
            }
        }
    }
    var info = [];
    if (PlayOnline.Text.length > 15) {
        var k = PlayOnline.Text.length-15;
        for (var i = 0; i < 15; ++i) {
            info[i] = PlayOnline.Text[k+i];
        }
    } else {
        for (var i = 0; i < j; ++i) {
            info[i] = PlayOnline.Text[i];
        }
    }
    if (PlayOnline.ShowTalk) {
        var user = PlayOnline.NewUser ? PlayOnline.NewUser : PlayOnline.UserName;
        if (user == '系统') {
            this.changeTextColor(this.powerUpColor());
            this.drawText("注意！ "+user+" 出来发话了！",0,0,this.contentsWidth(),'left');
        } else {
            this.changeTextColor(this.systemColor());
            this.drawText("欢迎 "+user+" 进入游戏！",0,0,this.contentsWidth(),'left');
        }
        var line1 = Math.min(12,this.lineHeight());
        this.drawHorzLine(line1);
        this.changeTextColor(this.normalColor());
        if (j > 0) {
            for (var i = 15 - (this.height-100)/20; i < 15; ++i) {
                this.drawText(info[i], 0, 25+(j-1-i)*20, this.contentsWidth(), 'left');
            }
        }
        var line2 = Math.max(this.height-87,this.lineHeight());
        this.drawHorzLine(line2);
            this.changeTextColor(this.systemColor());
            this.drawText("目前有："+num+"人在线。",0,this.height-74,this.contentsWidth(),'left');
    }
    PlayOnline.Text.splice(0,PlayOnline.Text.length);
    for (var i = 0; i < 15; ++i) {
        PlayOnline.Text[i] = info[i];
    }
};

Window_TalkList.prototype.update = function () {
    Window_Base.prototype.update.call(this);
    this.dianji();
    if (PlayOnline.UpdateTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        PlayOnline.UpdateTalk = false;
    }
    if (this.lastShowSwitch != PlayOnline.ShowTalk) {
        this.contents.clear();
        this.refresh();
        this.drawInfo();
        if (PlayOnline.ShowTalk) {
            Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
            Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
            Graphics._input.value = "";
            PlayOnline.addButton(1,'map','BtnInput',this.x+10,this.y+this.windowHeight()-35,['script','Graphics._input.focus();']);
            PlayOnline.addButton(2,'map','BtnSend',this.x+this.width-70,this.y+this.windowHeight()-85,['script','']);
            PlayOnline.addButton(3,'map','BtnSize',this.x+this.width-23,this.y,['script','PlayOnline.Size+=PlayOnline.Size==3?-2:1;']);
        }
        this.lastShowSwitch = PlayOnline.ShowTalk;
    }
    if (this.lastBusy != $gameMessage.isBusy()) {
        if ($gameMessage.isBusy()) Graphics._removeInput();
        else {
            if (PlayOnline.ShowTalk && !$gamePlayer.isTransferring() && !SceneManager.isSceneChanging()) {
                Graphics._addInput("text", this.x + 10, this.y + this.windowHeight() - 35, PlayOnline.Param.TalkWidth - 25, 18, 16);
                Graphics._input.maxLength = Math.round((PlayOnline.Param.TalkWidth - PlayOnline.Param.TalkFont * 2 - 20) / 10);
                Graphics._input.value = "";
            }
        }
        this.lastBusy = $gameMessage.isBusy();
    }
};

Window_TalkList.prototype.dianji = function() {
    if (!$gameMessage.isBusy() && PlayOnline.ShowTalk) {
        if (TouchInput.isRepeated()) {
            var dx = this.canvasToLocalX(TouchInput.x);
            var dy = this.canvasToLocalY(TouchInput.y);
            if(dx >= PlayOnline.Param.TalkWidth-70 && dy >= this.windowHeight() -74 && dx < PlayOnline.Param.TalkWidth && dy < this.windowHeight() - 30){
                if (Graphics._input.value.length > 0 && Graphics._input.value.indexOf(":") <= 0) {
                    PlayOnline.Text[PlayOnline.Text.length] = "你说：" + Graphics._input.value;
                    this.SendText(Graphics._input.value);
                } else if (Graphics._input.value.indexOf(":") > 0) {
                    var Text = Graphics._input.value.split(":");
                    if (Text[0] == '系统') PlayOnline.Text[PlayOnline.Text.length] = "不支持人机对话。";
                    else if (Text[0] == PlayOnline.UserName) PlayOnline.Text[PlayOnline.Text.length] = "屏蔽自言自语。";
                    else if (this.Online(Text[0])) {
                        PlayOnline.Text[PlayOnline.Text.length] = "你对"+Text[0]+"说："+Text[1];
                        this.SendText(Text[1], Text[0]);
                    } else PlayOnline.Text[PlayOnline.Text.length] = Text[0]+"不在线。";
                }
                Graphics._input.value = "";
            }
        }
    }
};

Window_TalkList.prototype.Online = function(user) {
    if (PlayOnline.Users.length > 0) {
        for (var j = 0; j < PlayOnline.Users.length; ++j) {
            if (user == PlayOnline.Users[j]) return true;
        }
        return false;
    } else return false;
};

Window_TalkList.prototype.SendText = function(text, name) {
    if (!name) var name = 'X';
    $.ajax({
        type: 'post',
        url: PlayOnline.Link,
        data: {user:PlayOnline.UserName, game:PlayOnline.Param.ActionName, cmd:'talk', text:text, target:name},
        success: function(msg) {},
        error: function (msg) {
            alert("连接不到服务器，请检查网络重试。");
            PlayOnline.UserName = null;
        },
        async: true,
    });
};

//-----------------------------------------------------------------------------
//  WINDOW_RankCmd
//-----------------------------------------------------------------------------

function Window_RankCmd() {
    this.initialize.apply(this, arguments);
}

Window_RankCmd.prototype = Object.create(Window_HorzCommand.prototype);
Window_RankCmd.prototype.constructor = Window_RankCmd;

Window_RankCmd.prototype.initialize = function(x,y) {
    Window_HorzCommand.prototype.initialize.call(this, x, y);
};

Window_RankCmd.prototype.windowWidth = function() {
    return 600;
};

Window_RankCmd.prototype.maxCols = function() {
    return 4;
};

Window_RankCmd.prototype.makeCommandList = function() {
    this.addCommand("首页",  'shouye');
    this.addCommand("我排在哪",  'mypage');
    this.addCommand("上一页",  'pageup');
    this.addCommand("下一页",  'pagedown');
};

Window_RankCmd.prototype.shouye = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= 0 && y >= 0 && x < this.width/4 && y < this.height){
            this.callHandler("shouye");
        } 
    } 
};

Window_RankCmd.prototype.mypage = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/4 && y >= 0 && x < this.width/2 && y < this.height){
            this.callHandler("mypage");
        } 
    } 
};

Window_RankCmd.prototype.pageup = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width/2 && y >= 0 && x < this.width*3/4 && y < this.height){
            this.callHandler("pageup");
        } 
    } 
};

Window_RankCmd.prototype.pagedown = function() {
    if (TouchInput.isTriggered()) {
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        if(x >= this.width*3/4 && y >= 0 && x < this.width && y < this.height){
            this.callHandler("pagedown");
        } 
    } 
};

//-----------------------------------------------------------------------------
// Sprite_Character
//-----------------------------------------------------------------------------

PlayOnline.Sprite_Character_update = Sprite_Character.prototype.update;
Sprite_Character.prototype.update = function() {
    PlayOnline.Sprite_Character_update.call(this);
    this.updateShowName();
    this.updateShowNameZoom();
};

Sprite_Character.prototype.updateShowName = function() {
    this.setupShowName();
    if (!this._showname) return;
    this.positionShowName();
};

Sprite_Character.prototype.setupShowName = function() {
    var num = $gameParty.maxBattleMembers() + PlayOnline.Param.Players - 1;
    if (this._showname) {
        if (PlayOnline.UpdateName[PlayOnline.UpdateCount]) {
            this._showname.setActor(this._character);
            this._showname.update();
            PlayOnline.UpdateName[PlayOnline.UpdateCount] = false;
            PlayOnline.UpdateCount++;
        }
        return;
    }
    if (!SceneManager._scene._spriteset) return;
    if (this._character instanceof Game_Follower && this._character._memberIndex < num) {
        this._showname = new Window_Name();
        this._showname.setActor(this._character);
        SceneManager._scene._spriteset.addChild(this._showname);
    }
};

Sprite_Character.prototype.positionShowName = function() {
    if (this._showname) {
        var win = this._showname;
        var width = win.width * win.scale.x;
        win.x = this.x + width / -2 + win.bufferX();
        var height = win.height * win.scale.y;
        var buffer = win.bufferY() * win.scale.y;
        win.y = this.y + (this.height * -1) - height + buffer;
    }
};

Sprite_Character.prototype.updateShowNameZoom = function() {
    if (!this._showname) return;
    var spriteset = SceneManager._scene._spriteset;
    this._showname.scale.x = 1 / spriteset.scale.x;
    this._showname.scale.y = 1 / spriteset.scale.y;
};

//-----------------------------------------------------------------------------
//  Sprite_PlayButton
//-----------------------------------------------------------------------------

function Sprite_PlayButton() {
    this.initialize.apply(this, arguments);
}

Sprite_PlayButton.prototype = Object.create(Sprite_Button.prototype);
Sprite_PlayButton.prototype.constructor = Sprite_PlayButton;

Sprite_PlayButton.prototype.initialize = function(Button) {
    Sprite_Button.prototype.initialize.call(this);
    this._hidden = false;
    this.setupButton(Button);
};

Sprite_PlayButton.prototype.setupButton = function(Button) {
    this._button = Button;
    this.bitmap = ImageManager.loadPicture(Button.image);
    var h = this.bitmap.height / 2;
    var w = this.bitmap.width;
    this.setColdFrame(0, 0, w, h);
    this.setHotFrame(0, h, w, h);
    this.x = Button.x;
    this.y = Button.y;
};

Sprite_PlayButton.prototype.processTouch = function() {
    if (this.isActive() && this.opacity >= 255) {
        if (TouchInput.isTriggered() && this.isButtonTouched()) {
            this._touching = true;
            this._wasTouching = true;
        }
        if (this._touching) {
            if (TouchInput.isReleased() || !this.isButtonTouched()) {
                if (this._wasTouching) {
                    this.callClickHandler();
                }
	this._touching = false;
	this._wasTouching = false;
            } else {
	this.callPressHandler();
            }
        } else if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
            this._touching = false;
        }
    } else {
        this._touching = false;
        if (this._wasTouching) {
            this.callClickHandler();
            this._wasTouching = false;
        }
    }
};

Sprite_PlayButton.prototype.setPressHandler = function(method) {
    this._pressHandler = method;
};

Sprite_PlayButton.prototype.callPressHandler = function() {
    if (this._pressHandler) {
        this._pressHandler();
    }
};

Sprite_PlayButton.prototype.update = function() {
    Sprite_Button.prototype.update.call(this);
    this.updateVisibility();
};

Sprite_PlayButton.prototype.updateVisibility = function() {
    if ($gameSystem._hideBtns) {
        this.opacity -= 20;
    } else if ($gameMap.isEventRunning()) {
        this.opacity = Math.max(this.opacity - 20,this._button.eOpacity);
    } else {
        this.opacity += 20;
    }
};

//-----------------------------------------------------------------------------
//  PlayOnline function
//-----------------------------------------------------------------------------

PlayOnline.createButton = function(obj) {
    if (!obj) return;
    var index = obj.id;
    if (!PlayOnline.canHasButton(index)) return;
    this.removeChild(this._GButtons[index]);
    this._GButtons[index] = new Sprite_PlayButton(obj);
    if (obj.action) {
        var type = obj.action[0];
        var data = obj.action[1];
        switch (type) {
            case 'button':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnPress.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'buttonT':
                var button = data;
                this._GButtons[index].setPressHandler(PlayOnline.btnTrigger.bind(this,data));
                this._GButtons[index].setClickHandler(PlayOnline.btnRelease.bind(this,data));
                break;
            case 'script':
                var script = data;
                this._GButtons[index].setClickHandler(this.gButtonScript.bind(this,data));
                break;
            case 'event':
                this._GButtons[index].setClickHandler(PlayOnline.runGCommentEvent.bind(this,data));
                break;	
        }
    }
    this.addChild(this._GButtons[index]);
};

PlayOnline.onButton = function() {
    var x = TouchInput.x;
    var y = TouchInput.y;
    var btns = SceneManager._scene._GButtons;
    if (!btns) return false;
    var result = false;
    for (var i = 0; i < btns.length; i++) {
        if (btns[i] && x > btns[i].x && x < btns[i].x + btns[i].width && y > btns[i].y && y < btns[i].y + btns[i].height) {
            result = true;
            break;
        }
    }
    return result;
};

PlayOnline.canHasButton = function(i) {
    return $gameSystem._gBtns[i] && $gameSystem._gBtns[i].location == 'map' && ($gameSystem._gBtns[i].mapId == 0 || $gameSystem._gBtns[i].mapId == $gameMap.mapId());
};

PlayOnline.runGCommentEvent = function(id) {
    $gameTemp.reserveCommonEvent(id);
};

PlayOnline.btnPress = function(btn) {
    Input._currentState[btn] = true;
};

PlayOnline.btnTrigger = function(btn) {
    if (!PlayOnline.Triggered[btn]) {
        Input._currentState[btn] = true;
        PlayOnline.Triggered[btn] = true;
    } else {
        Input._currentState[btn] = false;
    }
};

PlayOnline.btnRelease = function(btn) {
    Input._currentState[btn] = false;
    PlayOnline.Triggered[btn] = false;
};

PlayOnline.addButton = function(id,location,img,x,y,action,e) {
    var obj = {id:id,image:img,x:x,y:y,action:action,eOpacity:e || 0};
    var location = location.toLowerCase();
    if (location[0] == 'm') {
        obj.location = 'map';
        obj.mapId = Number(location.replace('map',''));
        $gameSystem._gBtns[id] = obj;
        SceneManager._scene.createGBtn($gameSystem._gBtns[id]);	
    }
};

PlayOnline.removeButton = function(id) {
    $gameSystem._gBtns[id] = null;
    SceneManager._scene.removeGButton(id);
};

//-----------------------------------------------------------------------------
//  Graphics._Input
//-----------------------------------------------------------------------------

Graphics._createAllElements = function() {
    this._createErrorPrinter();
    this._createCanvas();
    this._createVideo();
    this._createUpperCanvas();
    this._createRenderer();
    this._createFPSMeter();
    this._createModeBox();
    this._createGameFontLoader();
    this._createInput();
    this._createInput2();
};

Graphics._updateAllElements = function() {
    this._updateRealScale();
    this._updateErrorPrinter();
    this._updateCanvas();
    this._updateVideo();
    this._updateUpperCanvas();
    this._updateRenderer();
    this._updateInput();
    this._updateInput2();
    this._paintUpperCanvas();
};

Graphics._createInput = function() {
    this._input = document.createElement("input");
    this._input.id = 'Input';
    this._input.type ="text";
    this._input._sx ={};
    var sx = this._input._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._createInput2 = function() {
    this._input2 = document.createElement("input");
    this._input2.id = 'Input2';
    this._input2.type ="text";
    this._input2._sx ={};
    var sx = this._input2._sx;
    sx.xs = false;
    sx.x= 0;
    sx.y=0;
    sx.width =100;
    sx.height= 20;
    sx.fontSize = 18;
};

Graphics._addInput = function(type,x,y,width,height,fontSize) {                
    this._input.type = type || "text";
    var sx = this._input._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput();
    sx.xs = true;
    document.body.appendChild(this._input);
};

Graphics._addInput2 = function(type,x,y,width,height,fontSize) {                
    this._input2.type = type || "text";
    var sx = this._input2._sx;
    sx.x= x;
    sx.y= y;
    sx.width = width|| 100;
    sx.height= height|| 20;
    sx.fontSize = fontSize || 18;
    this._updateInput2();
    sx.xs = true;
    document.body.appendChild(this._input2);
};

Graphics._removeInput = function() {
    this._input.remove();
    this._input.value = null;
    this._input._xs = false;
};

Graphics._removeInput2 = function() {
    this._input2.remove();
    this._input2.value = null;
    this._input2._xs = false;
};

Graphics._updateInput =function () {   
    this._input.style.zIndex = 13;
    var sx = this._input._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input.style.position = 'absolute';
    this._input.style.margin = 'auto';
    this._input.style.top = y  + 'px';
    this._input.style.left = x  + 'px';
    this._input.style.width = width + 'px';
    this._input.style.height = height + 'px';
    this._input.style.fontSize = fontSize + 'px';
};

Graphics._updateInput2 =function () {   
    this._input2.style.zIndex = 12;
    var sx = this._input2._sx;
    var x = sx.x  * this._realScale + (window.innerWidth - this._width * this._realScale) / 2;
    var y = sx.y  * this._realScale + (window.innerHeight - this._height * this._realScale) / 2;
    var width = sx.width * this._realScale;
    var height = sx.height * this._realScale;
    var fontSize = sx.fontSize * this._realScale;
    this._input2.style.position = 'absolute';
    this._input2.style.margin = 'auto';
    this._input2.style.top = y  + 'px';
    this._input2.style.left = x  + 'px';
    this._input2.style.width = width + 'px';
    this._input2.style.height = height + 'px';
    this._input2.style.fontSize = fontSize + 'px';
};

Input._onKeyDown = function(event) {
    if (this._shouldPreventDefault(event.keyCode)) {
        if (Graphics && Graphics._input && Graphics._input._sx && Graphics._input._sx.xs) {
        } else if (Graphics && Graphics._input2 && Graphics._input2._sx && Graphics._input2._sx.xs) {
        } else {
            event.preventDefault();
        }
    }
    if (event.keyCode === 144) {
        this.clear();
    }
    var buttonName = this.keyMapper[event.keyCode];
    var version = parseInt(Utils.RPGMAKER_VERSION.substr(2, 1));
    if (version < 5) {
        if (buttonName) {
            this._currentState[buttonName] = true;
        }
    } else {
        if (ResourceHandler.exists() && buttonName === 'ok') {
            ResourceHandler.retry();
        } else if (buttonName) {
            this._currentState[buttonName] = true;
        }
    }
};